@@ -10,7 +10,7 @@
 #include "util/arg_auto_set.h"
 #include "Rev.h"
 
-#define version "1.2.7"
+#define version "1.2.8"
 
 #include "plugin_3state.h"
 
@@ -50,6 +50,7 @@ static void help1(void)
 	printf("options are:\n");
 	printf(" --prefix=path              change installation prefix from /usr to path\n");
 	printf(" --debug                    build full debug version (-g -O0, extra asserts)\n");
+	printf(" --profile                  build prifiling version if availabel (-pg)\n");
 	printf(" --symbols                  include symbols (add -g, but no -O0 or extra asserts)\n");
 	printf(" --coord=32|64              set coordinate integer type's width in bits\n");
 	printf(" --dot_pcb_pcb=path         .pcb-rnd config path under $HOME/\n");
@@ -97,6 +98,10 @@ int hook_custom_arg(const char *key, const char *value)
 		pup_set_debug(strue);
 		return 1;
 	}
+	if (strcmp(key, "profile") == 0) {
+		put("/local/pcb/profile", strue);
+		return 1;
+	}
 	if (strcmp(key, "symbols") == 0) {
 		put("/local/pcb/symbols", strue);
 		return 1;
@@ -244,6 +249,7 @@ int hook_postinit()
 
 	put("/local/pcb/want_bison", sfalse);
 	put("/local/pcb/debug", sfalse);
+	put("/local/pcb/profile", sfalse);
 	put("/local/pcb/symbols", sfalse);
 	put("/local/pcb/coord_bits", "32");
 	want_coord_bits = 32;
@@ -322,15 +328,14 @@ int safe_atoi(const char *s)
 int hook_detect_target()
 {
 	int need_gtklibs = 0, want_glib = 0, want_gtk, want_gtk2, want_gtk3, want_gd, want_stroke, need_inl = 0, want_cairo, want_xml2, has_gtk2 = 0, has_gtk3 = 0, want_gl;
-	const char *host_ansi, *host_ped, *target_ansi, *target_ped;
+	const char *host_ansi, *host_ped, *target_ansi, *target_ped, *target_pg, *target_no_pie;
 
-	want_gl     = plug_is_enabled("hid_gtk2_gl");
 	want_gtk2   = plug_is_enabled("hid_gtk2_gdk") || plug_is_enabled("hid_gtk2_gl");
-	want_gtk3   = plug_is_enabled("hid_gtk3_cairo");
+	want_gtk3   = plug_is_enabled("hid_gtk3_cairo") || plug_is_enabled("hid_gtk3_gl");
 	want_gtk    = want_gtk2 | want_gtk3;
 	want_gd     = plug_is_enabled("export_png") ||  plug_is_enabled("export_nelma") ||  plug_is_enabled("export_gcode");
 	want_stroke = plug_is_enabled("stroke");
-	want_cairo  = plug_is_enabled("export_bboard") | want_gtk3;
+	want_cairo  = plug_is_enabled("export_bboard") | plug_is_enabled("hid_gtk3_cairo");
 	want_xml2   = plug_is_enabled("io_eagle");
 
 	require("cc/fpic",  0, 0);
@@ -338,6 +343,8 @@ int hook_detect_target()
 	host_ped = get("/host/cc/argstd/pedantic");
 	target_ansi = get("/target/cc/argstd/ansi");
 	target_ped = get("/target/cc/argstd/pedantic");
+	target_pg = get("/target/cc/argstd/pg");
+	target_no_pie = get("/target/cc/argstd/no-pie");
 
 	{ /* need to set debug flags here to make sure libs are detected with the modified cflags; -ansi matters in what #defines we need for some #includes */
 		const char *tmp, *fpic, *debug;
@@ -355,7 +362,12 @@ int hook_detect_target()
 			append("/target/cc/cflags", target_ansi);
 			append("/target/cc/cflags", " ");
 			append("/target/cc/cflags", target_ped);
-			printf("DEBUG='%s'\n", get("/target/cc/cflags"));
+		}
+		if (istrue(get("/local/pcb/profile"))) {
+			append("/target/cc/cflags", " ");
+			append("/target/cc/cflags", target_pg);
+			append("/target/cc/cflags", " ");
+			append("/target/cc/cflags", target_no_pie);
 		}
 	}
 
@@ -438,7 +450,6 @@ int hook_detect_target()
 			if (!istrue(get("libs/gui/gtk2gl/presents"))) {
 				report_repeat("WARNING: Since there's no gl support for gtk found, disabling the gl rendering...\n");
 				hook_custom_arg("Disable-hid_gtk2_gl", NULL);
-				want_gl = 0;
 			}
 			need_gtklibs = 1;
 			has_gtk2 = 1;
@@ -450,16 +461,6 @@ int hook_detect_target()
 		}
 	}
 
-	if (want_gl) {
-		require("libs/gui/glu/presents", 0, 0);
-		if (!istrue(get("libs/gui/glu/presents"))) {
-			report_repeat("WARNING: Since there's no GLU found, disabling the hid_gtk2_gl plugin...\n");
-			hook_custom_arg("Disable-hid_gtk2_gl", NULL);
-		}
-		else
-			put("/local/pcb/has_glu", strue);
-	}
-
 	if (want_gtk3) {
 		if (istrue(get("libs/gui/cairo/presents"))) {
 			require("libs/gui/gtk3/presents", 0, 0);
@@ -478,6 +479,23 @@ int hook_detect_target()
 		}
 	}
 
+	want_gl = plug_is_enabled("hid_gtk2_gl") || plug_is_enabled("hid_gtk3_gl");
+	if (want_gl) {
+		require("libs/gui/glu/presents", 0, 0);
+		if (!istrue(get("libs/gui/glu/presents"))) {
+			report_repeat("WARNING: Since there's no GLU found, disabling the hid_gtk2_gl plugin...\n");
+			goto disable_gl;
+		}
+		else
+			put("/local/pcb/has_glu", strue);
+	}
+	else {
+		disable_gl:;
+		hook_custom_arg("Disable-lib_hid_gl", NULL);
+		hook_custom_arg("Disable-hid_gtk2_gl", NULL);
+		hook_custom_arg("Disable-hid_gtk3_gl", NULL);
+	}
+
 	/* gtk2 vs. gtk3 xor logic */
 	if (has_gtk2 && has_gtk3) {
 		report_repeat("Selected both gtk2 and gtk3 HIDs; gtk2 and gtk3 are incompatible, disabling gtk2 in favor of gtk3.\n");
@@ -507,6 +525,7 @@ int hook_detect_target()
 		require("libs/sul/libxml2/presents", 0, 0);
 		if (!istrue(get("libs/sul/libxml2/presents"))) {
 			report("libxml2 is not available, disabling io_eagle...\n");
+			report_repeat("WARNING: Since there's no libxml2 found, disabling the Eagle IO plugin...\n");
 			hook_custom_arg("Disable-io_eagle", NULL);
 		}
 		put("/local/pcb/want_libxml2", strue);
@@ -713,6 +732,13 @@ int hook_detect_target()
 		}
 	}
 
+	if (istrue(get("/local/pcb/profile"))) {
+		append("/local/pcb/cflags_profile", " ");
+		append("/local/pcb/cflags_profile", target_pg);
+		append("/local/pcb/cflags_profile", " ");
+		append("/local/pcb/cflags_profile", target_no_pie);
+	}
+
 	if (!istrue(get("cc/inline")))
 		need_inl = 1;
 
@@ -834,7 +860,7 @@ int hook_generate()
 
 	printf("Generating compat_inc.h (%d)\n", generr |= tmpasm("../src", "compat_inc.h.in", "compat_inc.h"));
 
-	printf("Generating opengl.h (%d)\n", generr |= tmpasm("../src_plugins/hid_gtk2_gl", "opengl.h.in", "opengl.h"));
+	printf("Generating opengl.h (%d)\n", generr |= tmpasm("../src_plugins/lib_hid_gl", "opengl.h.in", "opengl.h"));
 	if (plug_is_enabled("gpmi"))
 		gpmi_config();
 
