@@ -82,7 +82,7 @@ static int trycc(int logdepth, const char *cc, const char *test_c)
 	return ret;
 }
 
-int find_cc(int logdepth, int fatal)
+int find_cc(const char *name, int logdepth, int fatal)
 {
 	char *test_c = "#include <stdio.h>\nint main() { printf(\"OK\\n\");\nreturn 0;}\n";
 	char *out = NULL, *targetcc;
@@ -153,7 +153,7 @@ int find_cc(int logdepth, int fatal)
 	/* ldflags */
 	ldflags = get("/arg/cc/ldflags");
 	if (ldflags != NULL) {
-		logprintf(logdepth+1, "using user supplied ldflags '%s' (will test later)\n", cflags);
+		logprintf(logdepth+1, "using user supplied ldflags '%s' (will test later)\n", ldflags);
 		put("cc/ldflags", ldflags);
 	}
 	if (get("cc/ldflags") == NULL)
@@ -176,11 +176,11 @@ int find_cc(int logdepth, int fatal)
 	return 0;
 }
 
-int find_cc_argstd(int logdepth, int fatal)
+int find_cc_argstd(const char *name, int logdepth, int fatal)
 {
 	char *test_c = "#include <stdio.h>\nint main() { printf(\"OK\\n\");\nreturn 0;}\n";
 	char *out = NULL;
-	char **flg, *flags[] = {"-ansi", "-pedantic", "-Wall", "-std=c99", "-Werror", NULL};
+	char **flg, *flags[] = {"-ansi", "-pedantic", "-Wall", "-std=c99", "-Werror", "-Wextra", "-W", "-pg", "-no-pie", NULL};
 
 	require("cc/cc", logdepth, fatal);
 
@@ -205,11 +205,57 @@ int find_cc_argstd(int logdepth, int fatal)
 		put(name, found);
 	}
 
+	put("cc/argstd/presents", strue); /* to avoid re-detection*/
+
 	report("\n");
 	return 0;
 }
 
-int find_inline(int logdepth, int fatal)
+int find_cc_argmachine(const char *name, int logdepth, int fatal)
+{
+#define ARGM(flag) "-m" #flag , "-mno-" #flag
+	const char *test_c = "#include <stdio.h>\nint main() { printf(\"OK\\n\");\nreturn 0;}\n";
+	char *out = NULL;
+	const char **flg, *flags[] = { ARGM(mmx), ARGM(sse), ARGM(sse2), ARGM(sse3), ARGM(ssse3), ARGM(sse4), ARGM(sse4.1), ARGM(sse4.2), ARGM(avx), ARGM(avx2), NULL};
+
+	require("cc/cc", logdepth, fatal);
+
+	logprintf(logdepth, "find_cc: Detecting CC machine args\n");
+	report("Checking for cc args for machine... ");
+
+	for(flg = flags; *flg != NULL; flg++) {
+		char name[128], *end;
+		const char *found = "";
+		{
+			const char* ptr = (*flg) + 1;
+			strcpy(name, "cc/argmachine/");
+			end = name + strlen(name);
+			while(*ptr) {
+				if('.'!=*ptr && '-'!=*ptr) *end++ = *ptr;
+				++ptr;
+			}
+			*end = '\0';
+		}
+		end = strchr(name, '=');
+		if (end != NULL)
+			*end = '_';
+		if (compile_run(logdepth+1, test_c, NULL, *flg, "", &out) == 0) {
+			if (target_emu_fail(out) || (strncmp(out, "OK", 2) == 0)) {
+				found = *flg;
+				report(" ");
+				report(found);
+			}
+			free(out);
+		}
+		put(name, found);
+	}
+
+	report("\n");
+	return 0;
+#undef ARGM
+}
+
+int find_inline(const char *name, int logdepth, int fatal)
 {
 	const char *test_c =
 		NL "#include <stdio.h>"
@@ -237,7 +283,7 @@ int find_inline(int logdepth, int fatal)
 	return 1;
 }
 
-int find_varargmacro(int logdepth, int fatal)
+int find_varargmacro(const char *name, int logdepth, int fatal)
 {
 	const char *test_c =
 		NL "#include <stdio.h>"
@@ -263,7 +309,7 @@ int find_varargmacro(int logdepth, int fatal)
 	return 1;
 }
 
-int find_funcmacro(int logdepth, int fatal)
+int find_funcmacro(const char *name, int logdepth, int fatal)
 {
 	const char *test_c =
 		NL "#include <stdio.h>"
@@ -287,7 +333,7 @@ int find_funcmacro(int logdepth, int fatal)
 	return 1;
 }
 
-int find_constructor(int logdepth, int fatal)
+int find_constructor(const char *name, int logdepth, int fatal)
 {
 	const char *test_c =
 		NL "#include <stdio.h>"
@@ -316,7 +362,7 @@ int find_constructor(int logdepth, int fatal)
 	return 1;
 }
 
-int find_destructor(int logdepth, int fatal)
+int find_destructor(const char *name, int logdepth, int fatal)
 {
 	const char *test_c =
 		NL "#include <stdio.h>"
@@ -345,7 +391,7 @@ int find_destructor(int logdepth, int fatal)
 	return 1;
 }
 
-static int test_fattr(int logdepth, int fatal, const char *fattr)
+static int test_fattr(const char *name, int logdepth, int fatal, const char *fattr)
 {
 	char path[64];
 	char test_c[256];
@@ -380,9 +426,107 @@ static int test_fattr(int logdepth, int fatal, const char *fattr)
 	return 1;
 }
 
-int find_fattr_unused(int logdepth, int fatal)
+int find_fattr_unused(const char *name, int logdepth, int fatal)
 {
-	return test_fattr(logdepth, fatal, "unused");
+	return test_fattr(name, logdepth, fatal, "unused");
+}
+
+static int test_declspec(const char *name, int logdepth, int fatal, const char *dspec)
+{
+	char path[64];
+	char test_c[256];
+	const char *test_c_tmp =
+		NL "#include <stdio.h>"
+		NL "void  __declspec (%s) test1();"
+		NL "void test1()"
+		NL "{"
+		NL "	puts(\"OK\");"
+		NL "}"
+		NL "int main() {"
+		NL "	test1();"
+		NL "	return 0;"
+		NL "}"
+		NL ;
+
+	require("cc/cc", logdepth, fatal);
+
+	sprintf(test_c, test_c_tmp, dspec);
+	sprintf(path, "cc/declspec/%s/presents", dspec);
+
+	report("Checking for declspec %s... ", dspec);
+	logprintf(logdepth, "test_declspec: trying to find %s...\n", dspec);
+	logdepth++;
+	if (try(logdepth, NULL, test_c, "OK")) {
+		put(path, strue);
+		report("Found.\n");
+		return 0;
+	}
+	put(path, sfalse);
+	report("Not found.\n");
+	return 1;
+}
+
+int find_declspec_dllimport(const char *name, int logdepth, int fatal)
+{
+	return test_declspec(name, logdepth, fatal, "dllimport");
+}
+
+int find_declspec_dllexport(const char *name, int logdepth, int fatal)
+{
+	return test_declspec(name, logdepth, fatal, "dllexport");
+}
+
+static int test_dll_auxfile(const char *name, int logdepth, int fatal, const char *path, const char *ldflag, const char *filename)
+{
+	char ldflags[64];
+	char test_c[256];
+	const char *test_c_template =
+		NL "#include <stdio.h>"
+		NL "void %s test1();"
+		NL "void test1()"
+		NL "{"
+		NL "	puts(\"OK\");"
+		NL "}"
+		NL "int main() {"
+		NL "	test1();"
+		NL "	return 0;"
+		NL "}"
+		NL ;
+	const char *dspec;
+
+	require("cc/cc", logdepth, fatal);
+	require("cc/declspec/dllexport/*", logdepth, 0);
+
+	if (istrue("cc/declspec/dllexport/presents"))
+		dspec = " __declspec(dllexport) ";
+	else
+		dspec = "";
+
+	sprintf(test_c, test_c_template, dspec);
+
+	report("Checking for DLL flag %s... ", ldflag);
+	logprintf(logdepth, "test_dll_auxfile: trying to find %s...\n", ldflag);
+	logdepth++;
+	sprintf(ldflags, "%s,%s", ldflag, filename);
+	if (try_flags(logdepth, NULL, test_c, NULL, ldflags, "OK")) {
+		unlink(filename);
+		put(path, ldflag);
+		report("Found.\n");
+		return 0;
+	}
+	unlink(filename);
+	report("Not found.\n");
+	return 1;
+}
+
+int find_cc_wloutimplib(const char *name, int logdepth, int fatal)
+{
+	return test_dll_auxfile(name, logdepth, fatal, "cc/wloutimplib", "-Wl,--out-implib", "libscconfig_0.a");
+}
+
+int find_cc_wloutputdef(const char *name, int logdepth, int fatal)
+{
+	return test_dll_auxfile(name, logdepth, fatal, "cc/wloutputdef", "-Wl,--output-def", "libscconfig_0.def");
 }
 
 /* Hello world program to test compiler flags */
@@ -404,7 +548,7 @@ static int try_hello(int logdepth, const char *cflags, const char *ldflags, cons
 	return 0;
 }
 
-int find_rdynamic(int logdepth, int fatal)
+int find_rdynamic(const char *name, int logdepth, int fatal)
 {
 
 	require("cc/cc", logdepth, fatal);
@@ -413,14 +557,46 @@ int find_rdynamic(int logdepth, int fatal)
 	logprintf(logdepth, "find_rdynamic: trying to find rdynamic...\n");
 	logdepth++;
 
-	if (try_hello(logdepth, NULL, "+-rdynamic", "cc/rdynamic", "-rdynamic")) return 0;
-	if (try_hello(logdepth, NULL, NULL,         "cc/rdynamic", ""))          return 0;
+	if (try_hello(logdepth, NULL, "-rdynamic", "cc/rdynamic", "-rdynamic")) return 0;
+	if (try_hello(logdepth, NULL, NULL,        "cc/rdynamic", ""))          return 0;
 
 	report("Not found.\n");
 	return 1;
 }
 
-int find_soname(int logdepth, int fatal)
+int find_cc_fnopie(const char *name, int logdepth, int fatal)
+{
+	const char *test_c = test_hello_world;
+
+	require("cc/cc", logdepth, fatal);
+
+	report("Checking for -fno-pie... ");
+	logprintf(logdepth, "find_cc_fnopie: trying to find -fno-pie...\n");
+	logdepth++;
+
+	if (try_icl(logdepth, "cc/fnopie", test_c, NULL, "-fno-pie", NULL)) return 0;
+	if (try_icl(logdepth, "cc/fnopie", test_c, NULL, "-fno-pie", "-static")) return 0;
+	if (try_icl(logdepth, "cc/fnopie", test_c, NULL, NULL, NULL)) return 0;
+	return try_fail(logdepth, "cc/fnopie");
+}
+
+int find_cc_fnopic(const char *name, int logdepth, int fatal)
+{
+	const char *test_c = test_hello_world;
+
+	require("cc/cc", logdepth, fatal);
+
+	report("Checking for -fno-pic... ");
+	logprintf(logdepth, "find_cc_fnopic: trying to find -fno-pic...\n");
+	logdepth++;
+
+	if (try_icl(logdepth, "cc/fnopic", test_c, NULL, "-fno-pic", NULL)) return 0;
+	if (try_icl(logdepth, "cc/fnopic", test_c, NULL, "-fno-pic", "-static")) return 0;
+	if (try_icl(logdepth, "cc/fnopic", test_c, NULL, NULL, NULL)) return 0;
+	return try_fail(logdepth, "cc/fnopic");
+}
+
+int find_soname(const char *name, int logdepth, int fatal)
 {
 
 	require("cc/cc", logdepth, fatal);
@@ -429,30 +605,30 @@ int find_soname(int logdepth, int fatal)
 	logprintf(logdepth, "find_soname: trying to find soname...\n");
 	logdepth++;
 
-	if (try_hello(logdepth, NULL, "+-Wl,-soname,libscconfig.0", "cc/soname", "-Wl,-soname,")) return 0;
-	if (try_hello(logdepth, NULL, NULL,                         "cc/soname", ""))             return 0;
+	if (try_hello(logdepth, NULL, "-Wl,-soname,libscconfig.0", "cc/soname", "-Wl,-soname,")) return 0;
+	if (try_hello(logdepth, NULL, NULL,                        "cc/soname", ""))             return 0;
 
 	report("Not found.\n");
 	return 1;
 }
 
 
-int find_wlrpath(int logdepth, int fatal)
+int find_wlrpath(const char *name, int logdepth, int fatal)
 {
 
 	require("cc/cc", logdepth, fatal);
 
-	report("Checking for soname... ");
-	logprintf(logdepth, "find_soname: trying to find soname...\n");
+	report("Checking for rpath... ");
+	logprintf(logdepth, "find_wlrpath: trying to find rpath...\n");
 	logdepth++;
 
-	if (try_hello(logdepth, NULL, "+-Wl,-rpath=.",  "cc/wlrpath", "-Wl,-rpath=")) return 0;
+	if (try_hello(logdepth, NULL, "-Wl,-rpath=.",  "cc/wlrpath", "-Wl,-rpath=")) return 0;
 
 	report("Not found.\n");
 	return 1;
 }
 
-int find_fpic(int logdepth, int fatal)
+int find_fpic(const char *name, int logdepth, int fatal)
 {
 
 	require("cc/cc", logdepth, fatal);
@@ -461,9 +637,9 @@ int find_fpic(int logdepth, int fatal)
 	logprintf(logdepth, "find_fpic: trying to find -fpic...\n");
 	logdepth++;
 
-	if (try_hello(logdepth, NULL, "+-fPIC", "cc/fpic", "-fPIC")) return 0;
-	if (try_hello(logdepth, NULL, "+-fpic", "cc/fpic", "-fpic")) return 0;
-	if (try_hello(logdepth, NULL, NULL,     "cc/fpic", ""))      return 0;
+	if (try_hello(logdepth, NULL, "-fPIC", "cc/fpic", "-fPIC")) return 0;
+	if (try_hello(logdepth, NULL, "-fpic", "cc/fpic", "-fpic")) return 0;
+	if (try_hello(logdepth, NULL, NULL,    "cc/fpic", ""))      return 0;
 
 	report("Not found.\n");
 	return 1;
@@ -509,11 +685,12 @@ static int try_dynlib(int logdepth, const char *cflags, char *concated_ldflags,
 	const char *fpic;
 	const char *ld_include;
 	const char *dlc;
-	char *libname;
+	char *libname, *libname_dyn;
 	char *cflags_c;
 	char *oname = ".o";
 	int ret = 0;
 
+
 	dlc = get("libs/dl-compat");
 	if ((dlc != NULL) && (strcmp(dlc, strue) == 0))
 		ld_include = "<dl-compat.h>";
@@ -521,32 +698,32 @@ static int try_dynlib(int logdepth, const char *cflags, char *concated_ldflags,
 		ld_include = "<dlfcn.h>";
 
 	fpic = get("cc/fpic");
+	if (fpic == NULL) fpic = "";
 
 	if (cflags == NULL)
 		cflags="";
-	else if (*cflags == '+')
-		cflags++;
 
-	cflags_c = malloc(strlen(cflags) + 8 + sizeof(fpic));
-	sprintf(cflags_c, "+%s -c %s", cflags, fpic);
+	cflags_c = malloc(strlen(cflags) + 8 + strlen(fpic));
+	sprintf(cflags_c, "%s -c %s", cflags, fpic);
 
 
-	libname = (char *)get("sys/ext_dynlib");
+	libname_dyn = libname = (char *)get("sys/ext_dynlib");
 	if ((compile_code(logdepth, test_lib, &oname, NULL, cflags_c, NULL) != 0) ||
-			(compile_file(logdepth, oname, &libname, NULL, NULL, concated_ldflags) != 0)) {
+			(compile_file(logdepth, oname, &libname_dyn, NULL, NULL, concated_ldflags) != 0)) {
 			report("FAILED (compiling dynlib)\n");
 	}
 	else {
-		sprintf(test_host_app, test_host, ld_include, libname);
+		sprintf(test_host_app, test_host, ld_include, libname_dyn);
 		if (try_flags(logdepth, NULL, test_host_app, host_app_cflags, host_app_ldflags, "OK")) {
 			put(name, value);
 			report("OK (%s)\n", value);
 			ret = 1;
 		}
 	}
-	unlink(libname);
+	unlink(libname_dyn);
 	unlink(oname);
-	free(libname);
+	if (libname != libname_dyn)
+		free(libname_dyn);
 	free(oname);
 	free(concated_ldflags);
 	free(cflags_c);
@@ -554,7 +731,7 @@ static int try_dynlib(int logdepth, const char *cflags, char *concated_ldflags,
 }
 
 
-int find_ldflags_dynlib(int logdepth, int fatal)
+int find_ldflags_dynlib(const char *name, int logdepth, int fatal)
 {
 
 	require("cc/cc",       logdepth, fatal);
@@ -566,8 +743,105 @@ int find_ldflags_dynlib(int logdepth, int fatal)
 	logprintf(logdepth, "find_ldflags_dynlib: trying to find dynamic library ldflags...\n");
 	logdepth++;
 
-	if (try_dynlib(logdepth, NULL, concat_nodes("+-dynamic -shared", "cc/rdynamic", "libs/ldl", NULL), "cc/ldflags_dynlib", "-dynamic -shared", NULL, get("libs/ldl"))) return 0;
-	if (try_dynlib(logdepth, NULL, concat_nodes("+-shared",          "cc/rdynamic", "libs/ldl", NULL), "cc/ldflags_dynlib", "-shared",          NULL, get("libs/ldl"))) return 0;
+	if (try_dynlib(logdepth, NULL, concat_nodes("-dynamic -shared", "cc/rdynamic", "libs/ldl", NULL), "cc/ldflags_dynlib", "-dynamic -shared", NULL, get("libs/ldl"))) return 0;
+	if (try_dynlib(logdepth, NULL, concat_nodes("-shared",          "cc/rdynamic", "libs/ldl", NULL), "cc/ldflags_dynlib", "-shared",          NULL, get("libs/ldl"))) return 0;
+	report("Not found.\n");
+	return 1;
+}
+
+static int try_dll(int logdepth, const char *cflags, const char *lib_ldflags, const char *name, const char *value, const char *host_app_cflags, const char *host_app_ldflags)
+{
+	static const char *test_lib_template =
+		NL "#include <stdio.h>"
+		NL "%s void hello();"
+		NL "void hello() {"
+		NL "	puts(\"OK\");"
+		NL "}"
+		NL ;
+	static const char *test_host_template =
+		NL "%s void hello();"
+		NL "int main() {"
+		NL "	hello();"
+		NL "	return 0;"
+		NL "}"
+		NL ;
+	char test_lib[1024];
+	char test_host[1024];
+	const char *fpic;
+	const char *dspec_dllimport;
+	const char *dspec_dllexport;
+	char *libname, *libname_dyn;
+	char *cflags_c;
+	char *oname = ".o";
+	int ret = 0;
+
+	fpic = get("cc/fpic");
+	if (fpic == NULL) fpic = "";
+
+	if (istrue(get("cc/declspec/dllimport/presents")))
+		dspec_dllimport = " __declspec(dllimport) ";
+	else
+		dspec_dllimport = "";
+
+	if (istrue(get("cc/declspec/dllexport/presents")))
+		dspec_dllexport = " __declspec(dllexport) ";
+	else
+		dspec_dllexport = "";
+
+	if (cflags == NULL)
+		cflags="";
+
+	cflags_c = malloc(strlen(cflags) + 8 + strlen(fpic));
+	sprintf(cflags_c, "%s -c %s", cflags, fpic);
+
+	libname_dyn = libname = (char *)get("sys/ext_dynlib_native");
+	sprintf(test_lib, test_lib_template, dspec_dllexport);
+	if ((compile_code(logdepth, test_lib, &oname, NULL, cflags_c, NULL) != 0) ||
+			(compile_file(logdepth, oname, &libname_dyn, NULL, NULL, lib_ldflags) != 0)) {
+			report("FAILED (compiling DLL)\n");
+	}
+	else {
+		sprintf(test_host, test_host_template, dspec_dllimport);
+		if (try_flags(logdepth, NULL, test_host, host_app_cflags, host_app_ldflags, "OK")) {
+			put(name, value);
+			report("OK (%s)\n", value);
+			ret = 1;
+		}
+	}
+	unlink(libname_dyn);
+	unlink(oname);
+	if (libname != libname_dyn)
+		free(libname_dyn);
+	free(oname);
+	free(cflags_c);
+	return ret;
+}
+
+int find_ldflags_dll(const char *name, int logdepth, int fatal)
+{
+	char dll_ldflags[128];
+	static const char *dll_implib_name = "libscconfigdll_0.a";
+
+	require("cc/cc",                   logdepth, fatal);
+	require("cc/fpic",                 logdepth, 0);
+	require("cc/declspec/dllimport/*", logdepth, 0);
+	require("cc/declspec/dllexport/*", logdepth, 0);
+	require("cc/wloutimplib",          logdepth, 0);
+
+	report("Checking for DLL ldflags... ");
+	logprintf(logdepth, "find_ldflags_dll: trying to find DLL ldflags...\n");
+
+	if (try_dll(logdepth+1, NULL, "-shared", "cc/ldflags_dll", "-shared", NULL, NULL)) return 0;
+
+	if (get("cc/wloutimplib")) {
+		sprintf(dll_ldflags, "-shared %s,%s", get("cc/wloutimplib"), dll_implib_name);
+		if (try_dll(logdepth+1, NULL, dll_ldflags, "cc/ldflags_dll", "-shared", NULL, "-L. -lscconfigdll_0")) {
+			unlink(dll_implib_name);
+			return 0;
+		}
+		unlink(dll_implib_name);
+	}
+
 	report("Not found.\n");
 	return 1;
 }
@@ -594,7 +868,7 @@ static int try_alloca(int logdepth, const char *cflags, const char *ldflags, con
 	return 0;
 }
 
-int find_alloca(int logdepth, int fatal)
+int find_alloca(const char *name, int logdepth, int fatal)
 {
 	require("cc/cc", logdepth, fatal);
 
@@ -610,7 +884,7 @@ int find_alloca(int logdepth, int fatal)
 }
 
 
-int find__exit(int logdepth, int fatal)
+int find__exit(const char *name, int logdepth, int fatal)
 {
 	const char *test_c =
 		NL "#include <stdio.h>"
@@ -627,7 +901,7 @@ int find__exit(int logdepth, int fatal)
 	logprintf(logdepth, "find__exit: trying to find _exit()...\n");
 	logdepth++;
 
-	if (try_flags_inv(logdepth, NULL, test_c, "+", "+", "BAD")) {
+	if (try_flags_inv(logdepth, NULL, test_c, NULL, NULL, "BAD")) {
 		put("cc/_exit/presents", strue);
 		report("found\n");
 		return 0;
