@@ -1,6 +1,7 @@
 /*
     scconfig - detection of standard library features: file system specific calls
     Copyright (C) 2010  Tibor Palinkas
+    Copyright (C) 2018  Aron Barath
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -29,19 +30,19 @@
 #include "db.h"
 #include "dep.h"
 
-int find_fs_realpath(int logdepth, int fatal)
+int find_fs_realpath(const char *name, int logdepth, int fatal)
 {
 	char *test_c =
 		NL "#include <limits.h>"
 		NL "#include <stdlib.h>"
 		NL "#include <stdio.h>"
-		NL "int main() {"
 		NL "#ifdef PATH_MAX"
-		NL "	char out[PATH_MAX];"
+		NL "char out_buf[PATH_MAX];"
 		NL "#else"
-		NL "	char out[32768];"
+		NL "char out_buf[32768];"
 		NL "#endif"
-		NL "	if (realpath(\".\", out) == out)"
+		NL "int main() {"
+		NL "	if (realpath(\".\", out_buf) == out_buf)"
 		NL "		puts(\"OK\");"
 		NL "	return 0;"
 		NL "}"
@@ -60,7 +61,33 @@ int find_fs_realpath(int logdepth, int fatal)
 }
 
 
-int find_fs_readdir(int logdepth, int fatal)
+int find_fs__fullpath(const char *name, int logdepth, int fatal)
+{
+	char *test_c =
+		NL "#include <stdio.h>"
+		NL "#include <conio.h>"
+		NL "#include <stdlib.h>"
+		NL "#include <direct.h>"
+		NL "int main() {"
+		NL "	char full[_MAX_PATH];"
+		NL "	if (_fullpath(full, \".\", _MAX_PATH) != NULL)"
+		NL "		puts(\"OK\");"
+		NL "	return 0;"
+		NL "}"
+		NL;
+
+	require("cc/cc", logdepth, fatal);
+
+	report("Checking for _fullpath()... ");
+	logprintf(logdepth, "find_fs__fullpath: trying to find _fullpath...\n");
+	logdepth++;
+
+	if (try_icl(logdepth, "libs/fs/_fullpath", test_c, NULL, NULL, NULL)) return 0;
+	return try_fail(logdepth, "libs/fs/_fullpath");
+}
+
+
+int find_fs_readdir(const char *name, int logdepth, int fatal)
 {
 	char *test_c =
 		NL "#include <stdlib.h>"
@@ -99,7 +126,7 @@ int find_fs_readdir(int logdepth, int fatal)
 }
 
 
-int find_fs_findnextfile(int logdepth, int fatal)
+int find_fs_findnextfile(const char *name, int logdepth, int fatal)
 {
 	char *test_c =
 		NL "#include <stdlib.h>"
@@ -130,15 +157,101 @@ int find_fs_findnextfile(int logdepth, int fatal)
 	return try_fail(logdepth, "libs/fs/findnextfile");
 }
 
+int find_fs_access(const char *name, int logdepth, int fatal)
+{
+	char *test_c =
+		NL "int my_test() { return access(\".\", 0); }"
+		NL "#include <stdio.h>"
+		NL "int main() {"
+		NL "	if (my_test() == 0)"
+		NL "		puts(\"OK\");"
+		NL "	return 0;"
+		NL "}"
+		NL;
+	const char* includes[] = { "#include <unistd.h>", "#include <stdlib.h>\n#include <direct.h>", "#include <stdlib.h>", NULL };
+	const char** inc;
 
-int find_fs_stat_macros(int logdepth, int fatal)
+	require("cc/cc", logdepth, fatal);
+
+	report("Checking for access()... ");
+	logprintf(logdepth, "find_fs_access: trying to find access...\n");
+	logdepth++;
+
+	for (inc=includes; *inc; ++inc)
+		if (try_icl(logdepth, "libs/fs/access", test_c, *inc, NULL, NULL)) return 0;
+
+	return try_fail(logdepth, "libs/fs/access");
+}
+
+int find_fs_access_macros(const char *rname, int logdepth, int fatal)
+{
+	char *test_c_templ =
+		NL "%s"
+		NL "void my_test() { int a = %s; }"
+		NL "#include <stdio.h>"
+		NL "int main() {"
+		NL "	my_test();"
+		NL "	puts(\"OK\");"
+		NL "	return 0;"
+		NL "}"
+		NL;
+	char test_c[256];
+
+	char *names[][3] = {
+		{"F_OK",  "F_OK",  NULL},
+		{"R_OK",  "R_OK",  NULL},
+		{"W_OK",  "W_OK",  NULL},
+		{"X_OK",  "X_OK",  NULL},
+		{NULL,    NULL,    NULL}
+		};
+	char **n;
+	const char* access_includes;
+	int name, pr;
+	char nodename[128];
+
+	require("cc/cc", logdepth, fatal);
+	if (require("libs/fs/access/*", logdepth, fatal)!=0 ||
+		!istrue(get("libs/fs/access/presents"))) {
+		put("libs/fs/access/macros/presents", sfalse);
+		return 1;
+	}
+	access_includes = get("libs/fs/access/includes");
+
+	report("Checking for access macros:\n");
+	logprintf(logdepth, "find_fs_access_macros: trying to find access macros...\n");
+	logdepth++;
+
+	pr = 0;
+	for(name = 0; *names[name] != NULL; name++) {
+		report(" %s...\t", names[name][0]);
+		for(n = &names[name][0]; *n != NULL; n++) {
+			sprintf(test_c, test_c_templ, access_includes, *n);
+			if (try_icl(logdepth, NULL, test_c, NULL, NULL, NULL)) {
+				sprintf(nodename, "libs/fs/access/macros/%s", names[name][0]);
+				put(nodename, *n);
+				report("found as %s\n", *n);
+				pr++;
+				goto found;
+			}
+		}
+		report("not found\n");
+		found:;
+	}
+
+	put("libs/fs/access/macros/presents", ((pr > 0) ? (strue) : (sfalse)));
+	return (pr == 0);
+}
+
+int find_fs_stat_macros(const char *rname, int logdepth, int fatal)
 {
 	char *test_c_templ =
 		NL "#include <sys/stat.h>"
 		NL "#include <sys/types.h>"
+		NL "void my_test() { int a = %s(0); }"
+		NL "#include <stdio.h>"
 		NL "int main() {"
-		NL "	int a = %s(0);"
-		NL "		puts(\"OK\");"
+		NL "	my_test();"
+		NL "	puts(\"OK\");"
 		NL "	return 0;"
 		NL "}"
 		NL;
@@ -186,11 +299,12 @@ int find_fs_stat_macros(int logdepth, int fatal)
 	return (pr == 0);
 }
 
-int find_fs_stat_fields(int logdepth, int fatal)
+int find_fs_stat_fields(const char *rname, int logdepth, int fatal)
 {
 	char *test_c_templ =
 		NL "#include <sys/stat.h>"
 		NL "#include <sys/types.h>"
+		NL "#include <stdio.h>"
 		NL "int main() {"
 		NL "	struct stat st;"
 		NL "	(void)st.%s;"
@@ -200,7 +314,7 @@ int find_fs_stat_fields(int logdepth, int fatal)
 		NL;
 	char test_c[256];
 
-	char *names[] = {"st_blksize", "st_blocks", "st_rdev", NULL };
+	char *names[] = {"st_blksize", "st_blocks", "st_rdev", "st_mtim", "st_mtime", "st_birthtim", "st_birthtime", NULL };
 	int name, pr;
 	char nodename[128];
 
@@ -228,7 +342,56 @@ int find_fs_stat_fields(int logdepth, int fatal)
 	return (pr == 0);
 }
 
-int find_fs_getcwd(int logdepth, int fatal)
+static int find_fs_any_lstat(const char *name, int logdepth, int fatal, char *fn)
+{
+	/* make sure <stdio.h> does not affect our lstat() detection */
+	const char *test_c_in =
+		NL "void my_puts(const char *s);"
+		NL "int main() {"
+		NL "	struct stat buf;"
+		NL "	if (%s(\".\", &buf) == 0)"
+		NL "		my_puts(\"OK\");"
+		NL "	return 0;"
+		NL "}"
+		NL "#include <stdio.h>"
+		NL "void my_puts(const char *s)"
+		NL "{"
+		NL "	puts(s);"
+		NL "}"
+		NL;
+	char test_c[384], node[64];
+	const char *incs[] = {"#include <sys/stat.h>", "#include <unistd.h>", "#include <sys/types.h>\n#include <sys/stat.h>\n#include <unistd.h>", NULL};
+	const char **inc;
+
+	require("cc/cc", logdepth, fatal);
+
+	sprintf(node, "libs/fs/%s", fn);
+	sprintf(test_c, test_c_in, fn);
+
+	report("Checking for %s... ", fn);
+	logprintf(logdepth, "find_fs_%s: trying to find lstat()...\n", fn);
+	logdepth++;
+
+	for (inc = incs; *inc; ++inc) {
+		if (try_icl(logdepth, node, test_c, *inc, NULL, NULL))
+			return 0;
+	}
+
+	return try_fail(logdepth, node);
+}
+
+int find_fs_lstat(const char *name, int logdepth, int fatal)
+{
+	return find_fs_any_lstat(name, logdepth, fatal, "lstat");
+}
+
+int find_fs_statlstat(const char *name, int logdepth, int fatal)
+{
+	return find_fs_any_lstat(name, logdepth, fatal, "statlstat");
+}
+
+
+int find_fs_getcwd(const char *name, int logdepth, int fatal)
 {
 	char *test_c =
 		NL "#include <unistd.h>"
@@ -250,7 +413,7 @@ int find_fs_getcwd(int logdepth, int fatal)
 	return try_fail(logdepth, "libs/fs/getcwd");
 }
 
-int find_fs__getcwd(int logdepth, int fatal)
+int find_fs__getcwd(const char *name, int logdepth, int fatal)
 {
 	char *test_c =
 		NL "#include <stdlib.h>"
@@ -273,7 +436,7 @@ int find_fs__getcwd(int logdepth, int fatal)
 }
 
 
-int find_fs_getwd(int logdepth, int fatal)
+int find_fs_getwd(const char *name, int logdepth, int fatal)
 {
 	char *test_c =
 		NL "#include <unistd.h>"
@@ -295,13 +458,14 @@ int find_fs_getwd(int logdepth, int fatal)
 	return try_fail(logdepth, "libs/fs/getwd");
 }
 
-int find_fs_mkdir(int logdepth, int fatal)
+int find_fs_mkdir(const char *name, int logdepth, int fatal)
 {
 	char *dir;
 	char test_c[1024];
 	char *test_c_in =
 		NL "#include <stdio.h>"
 		NL "int main() {"
+		NL no_implicit(int, "mkdir", "mkdir")
 		NL "	if (mkdir(\"%s\"%s) == 0)"
 		NL "		puts(\"OK\");"
 		NL "	return 0;"
@@ -338,7 +502,18 @@ int find_fs_mkdir(int logdepth, int fatal)
 		return 0;
 	}
 
+	/* win32, 1 argument, with <direct.h> */
 	oops2:;
+	sprintf(test_c, test_c_in, dir, "");
+	if (try_icl(logdepth, "libs/fs/mkdir", test_c, "#include <direct.h>\n", NULL, NULL)) {
+		if (!is_dir(dir))
+			goto oops3;
+		put("libs/fs/mkdir/num_args", "1");
+		rmdir(dir);
+		return 0;
+	}
+
+	oops3:;
 	put("libs/fs/mkdir/includes", "");
 	put("libs/fs/mkdir/ldflags", "");
 	put("libs/fs/mkdir/cdflags", "");
@@ -347,7 +522,7 @@ int find_fs_mkdir(int logdepth, int fatal)
 	return try_fail(logdepth, "libs/fs/mkdir");
 }
 
-int find_fs__mkdir(int logdepth, int fatal)
+int find_fs__mkdir(const char *name, int logdepth, int fatal)
 {
 	char *dir;
 	char test_c[1024];
@@ -399,7 +574,7 @@ int find_fs__mkdir(int logdepth, int fatal)
 	return try_fail(logdepth, "libs/fs/_mkdir");
 }
 
-int find_fs_mkdtemp(int logdepth, int fatal)
+int find_fs_mkdtemp(const char *name, int logdepth, int fatal)
 {
 	char *test_c =
 		NL "#include <stdio.h>"
@@ -428,7 +603,7 @@ int find_fs_mkdtemp(int logdepth, int fatal)
 	return try_fail(logdepth, "libs/fs/mkdtemp");
 }
 
-int find_fs_mmap(int logdepth, int fatal)
+int find_fs_mmap(const char *name, int logdepth, int fatal)
 {
 	char test_c[1024];
 	char *tmp;
@@ -484,3 +659,76 @@ int find_fs_mmap(int logdepth, int fatal)
 	free(tmp);
 	return try_fail(logdepth, "libs/fs/mmap");
 }
+
+/* Haiku/BeOS next_dev */
+int find_fsmount_next_dev(const char *name, int logdepth, int fatal)
+{
+	char *test_c =
+		NL "#include <stdio.h>"
+		NL "int main()"
+		NL "{"
+		NL "	int32 pos = 0;"
+		NL "	dev_t res = next_dev(&pos);"
+		NL "	if (res >= 0)"
+		NL "		puts(\"OK\");"
+		NL "	return 0;"
+		NL "}"
+		NL;
+
+	require("cc/cc", logdepth, fatal);
+
+	report("Checking for next_dev... ");
+	logprintf(logdepth, "find_fsmount_next_dev: trying to find next_dev()...\n");
+	logdepth++;
+
+	if (try_icl(logdepth, "libs/fsmount/next_dev", test_c, "#include <fs_info.h>\n", NULL, NULL)) return 0;
+	return try_fail(logdepth, "libs/fsmount/next_dev");
+}
+
+int find_fsmount_fsstat_fields(const char *name, int logdepth, int fatal)
+{
+	const char *fields[] = {"f_fstypename", NULL};
+	return try_icl_sfields(logdepth, "libs/fsmount/struct_fsstat", "struct fsstat", fields, "#include <sys/fsstat.h>", NULL, NULL, 0);
+}
+
+int find_fsmount_statfs_fields(const char *name, int logdepth, int fatal)
+{
+	const char *fields[] = {"f_fstypename", "f_type", NULL};
+	return try_icl_sfields(logdepth, "libs/fsmount/struct_statfs", "struct statfs", fields, "#include <sys/statfs.h>", NULL, NULL, 0);
+}
+
+int find_fsmount_statvfs_fields(const char *name, int logdepth, int fatal)
+{
+	const char *fields[] = {"f_fstypename", "f_type", "f_basetype", NULL};
+	return try_icl_sfields(logdepth, "libs/fsmount/struct_statvfs", "struct statvfs", fields, "#include <sys/statvfs.h>", NULL, NULL, 0);
+}
+
+int find_fs_ustat(const char *name, int logdepth, int fatal)
+{
+	const char *key = "libs/fs/ustat";
+	const char *test_c =
+		NL "#include <stdio.h>"
+		NL "#include <sys/stat.h>"
+		NL "int main()"
+		NL "{"
+		NL "	struct stat stat_buf;"
+		NL "	struct ustat ustat_buf;"
+		NL "	if (stat(\".\", &stat_buf) == 0 &&"
+		NL "		ustat(stat_buf.st_dev, &ustat_buf) == 0)"
+		NL "		puts(\"OK\");"
+		NL "	return 0;"
+		NL "}"
+		NL;
+
+	require("cc/cc", logdepth, fatal);
+
+	report("Checking for ustat... ");
+	logprintf(logdepth, "find_fs_ustat: trying to find ustat()...\n");
+	logdepth++;
+
+	if (try_icl(logdepth, key, test_c, "#include <ustat.h>", NULL, NULL)) return 0;
+	if (try_icl(logdepth, key, test_c, "#include <unistd.h>", NULL, NULL)) return 0;
+	if (try_icl(logdepth, key, test_c, "#include <sys/types.h>\n#include <unistd.h>", NULL, NULL)) return 0;
+	if (try_icl(logdepth, key, test_c, "#include <sys/types.h>\n#include <unistd.h>\n#include <ustat.h>", NULL, NULL)) return 0;
+	return try_fail(logdepth, key);
+}
