@@ -29,7 +29,7 @@
 #include "db.h"
 #include "dep.h"
 
-int find_io_pipe(int logdepth, int fatal)
+int find_io_pipe(const char *name, int logdepth, int fatal)
 {
 	char *test_c =
 		NL "#include <unistd.h>"
@@ -52,7 +52,7 @@ int find_io_pipe(int logdepth, int fatal)
 	return try_fail(logdepth, "libs/io/pipe");
 }
 
-int find_io_dup2(int logdepth, int fatal)
+int find_io_dup2(const char *name, int logdepth, int fatal)
 {
 	char *test_c =
 		NL "#include <unistd.h>"
@@ -75,13 +75,14 @@ int find_io_dup2(int logdepth, int fatal)
 }
 
 
-int find_io_fileno(int logdepth, int fatal)
+int find_io_fileno(const char *name, int logdepth, int fatal)
 {
 	char test_c[256];
 	char *test_c_ =
 		NL "#include <stdio.h>"
 		NL "int main() {"
-		NL "	if (%s(stdout) > 0)"
+		NL no_implicit(int, "%s", "%s")
+		NL "	if (%s(stdout) >= 0)"
 		NL "		puts(\"OK\"); "
 		NL "	return 0;"
 		NL "}"
@@ -89,19 +90,25 @@ int find_io_fileno(int logdepth, int fatal)
 
 	require("cc/cc", logdepth, fatal);
 
-	report("Checking for fileno(2)... ");
-	logprintf(logdepth, "find_io_fileno: trying to find fileno(2)...\n");
+	report("Checking for fileno(3)... ");
+	logprintf(logdepth, "find_io_fileno: trying to find fileno(3)...\n");
 	logdepth++;
 
 	/* UNIX */
-	sprintf(test_c, test_c_, "fileno");
+	sprintf(test_c, test_c_, "fileno", "fileno", "fileno");
 	if (try_icl(logdepth, "libs/io/fileno", test_c, "#include <unistd.h>\n", NULL, NULL)) {
 		put("libs/io/fileno/call", "fileno");
 		return 0;
 	}
 
+	sprintf(test_c, test_c_, "fileno", "fileno", "fileno");
+	if (try_icl(logdepth, "libs/io/fileno", test_c, "#define _XOPEN_SOURCE\n#include <unistd.h>\n", NULL, NULL)) {
+		put("libs/io/fileno/call", "fileno");
+		return 0;
+	}
+
 	/* windows */
-	sprintf(test_c, test_c_, "_fileno");
+	sprintf(test_c, test_c_, "_fileno", "_fileno", "_fileno");
 	if (try_icl(logdepth, "libs/io/fileno", test_c, "#include <stdio.h>\n", NULL, NULL)) {
 		put("libs/io/fileno/call", "_fileno");
 		return 0;
