@@ -51,7 +51,7 @@ static int trydlc(int logdepth, const char *test_c_dlc, const char *cflagsf, con
 	return 0;
 }
 
-int find_lib_ldl(int logdepth, int fatal)
+int find_lib_ldl(const char *name, int logdepth, int fatal)
 {
 	const char *ldl, *dlc;
 	char *s;
@@ -97,7 +97,7 @@ int find_lib_ldl(int logdepth, int fatal)
 				report("OK ()\n");
 				return 0;
 			}
-			if (try_icl(logdepth, NULL, test_c, NULL, NULL, "+-ldl")) {
+			if (try_icl(logdepth, NULL, test_c, NULL, NULL, "-ldl")) {
 				put("libs/ldl", "-ldl");
 				put("libs/ldl/includes", "#include <dlfcn.h>\\n");
 				report("OK (-ldl)\n");
@@ -110,20 +110,20 @@ int find_lib_ldl(int logdepth, int fatal)
 			   - first assume the linker will find it
 			   - next assume gcc and pass rpath to the linker
 			   - finally try static linking */
-			if (trydlc(logdepth, test_c_dlc, "+-I%s/include", "-L%s/lib -ldl-compat\000%s", dlc)) {
+			if (trydlc(logdepth, test_c_dlc, "-I%s/include", "-L%s/lib -ldl-compat\000%s", dlc)) {
 				put("libs/ldl/includes", "#include <dl-compat.h>\\n");
 				return 0;
 			}
-			if (trydlc(logdepth, test_c_dlc, "+-I%s/include", "-L%s/lib -Wl,-rpath=%s/lib -ldl-compat", dlc)) {
+			if (trydlc(logdepth, test_c_dlc, "-I%s/include", "-L%s/lib -Wl,-rpath=%s/lib -ldl-compat", dlc)) {
 				put("libs/ldl/includes", "#include <dl-compat.h>\\n");
 				return 0;
 			}
-			if (trydlc(logdepth, test_c_dlc, "+-I%s/include", "%s/lib/libdl-compat.a\000%s", dlc)) {
+			if (trydlc(logdepth, test_c_dlc, "-I%s/include", "%s/lib/libdl-compat.a\000%s", dlc)) {
 				put("libs/ldl/includes", "#include <dl-compat.h>\\n");
 				return 0;
 			}
 		}
-		else if (try_icl(logdepth, NULL, test_c_dlc, NULL, NULL, "+-ldl-compat")) {
+		else if (try_icl(logdepth, NULL, test_c_dlc, NULL, NULL, "-ldl-compat")) {
 			/* check at normal system installation */
 			put("libs/ldl", "-ldl-compat");
 			put("libs/ldl/includes", "#include <dl-compat.h>\\n");
@@ -133,9 +133,7 @@ int find_lib_ldl(int logdepth, int fatal)
 	}
 	else {
 		report("User provided... ");
-		s = malloc(strlen(ldl)+1);
-		*s = '+';
-		strcpy(s+1, ldl);
+		s = strclone(ldl);
 		if (try_icl(logdepth, NULL, test_c, NULL, NULL, s)) {
 			put("libs/ldl", ldl);
 			put("libs/ldl/includes", "#include <dlfcn.h>\\n");
@@ -149,7 +147,7 @@ int find_lib_ldl(int logdepth, int fatal)
 	return 1;
 }
 
-int find_lib_LoadLibrary(int logdepth, int fatal)
+int find_lib_LoadLibrary(const char *name, int logdepth, int fatal)
 {
 	/*char *s;*/
 
@@ -174,7 +172,7 @@ int find_lib_LoadLibrary(int logdepth, int fatal)
 	return try_fail(logdepth, "libs/LoadLibrary");
 }
 
-int find_lib_errno(int logdepth, int fatal)
+int find_lib_errno(const char *name, int logdepth, int fatal)
 {
 	char *test_c =
 		NL "#include <errno.h>"
