@@ -30,7 +30,7 @@
 #include "db.h"
 #include "dep.h"
 
-int find_sys_ptrwidth(int logdepth, int fatal)
+int find_sys_ptrwidth(const char *name, int logdepth, int fatal)
 {
 	char *end, W[32];
 	char *out = NULL;
@@ -66,7 +66,7 @@ int find_sys_ptrwidth(int logdepth, int fatal)
 	return 0;
 }
 
-int find_sys_byte_order(int logdepth, int fatal)
+int find_sys_byte_order(const char *name, int logdepth, int fatal)
 {
 	char *end, *W;
 	char *out = NULL;
@@ -243,7 +243,7 @@ static int try_tmp_all(int logdepth)
 	return 0;
 }
 
-int find_tmp(int logdepth, int fatal)
+int find_tmp(const char *name, int logdepth, int fatal)
 {
 	const char *usertmp;
 
@@ -356,7 +356,7 @@ int test_shell(const char *shell, int logdepth, int quote)
 	return 0;
 }
 
-static int find_shell_escape(int logdepth, int fatal, const char *shell)
+static int find_shell_escape(const char *name, int logdepth, int fatal, const char *shell)
 {
 	char cmdline[256];
 	char **t;
@@ -404,7 +404,7 @@ static int find_shell_escape(int logdepth, int fatal, const char *shell)
 	return 1;
 }
 
-int find_shell(int logdepth, int fatal)
+int find_shell(const char *name, int logdepth, int fatal)
 {
 	const char *shells[] = {
 		"/bin/sh -c",
@@ -423,7 +423,7 @@ int find_shell(int logdepth, int fatal)
 	if (cross_blind) {
 		const char *shell = get("/arg/sys/target-shell");
 		if (shell == NULL) {
-			report("Need sys/shell in blind cross compiling mode\n");
+			report("Need to specify sys/target-shell in blind cross compiling mode, because the shell cannot be detected (note: scconfig will not attempt to run the target shell)\n");
 			exit(1);
 		}
 
@@ -440,7 +440,7 @@ int find_shell(int logdepth, int fatal)
 		if ((test_shell(*s, logdepth+1, 0)) || (test_shell(*s, logdepth+1, 1))) {
 			report("%s\n", *s);
 			logprintf(logdepth, "found a shell '%s', need quote: %s\n", *s, get("sys/shell_needs_quote"));
-			return find_shell_escape(logdepth, fatal, *s);
+			return find_shell_escape(name, logdepth, fatal, *s);
 		}
 	}
 
