@@ -1,7 +1,7 @@
 /*
     scconfig - detection of types and type sizes
     Copyright (C) 2012  Tibor Palinkas
-    Copyright (C) 2017  Aron Barath
+    Copyright (C) 2017-2018  Aron Barath
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -31,8 +31,10 @@
 
 /* assume there is no integer that is at least this wide, in bytes */
 #define MAX_INT_WIDTH 9
+/* ('long double' can be 16 bytes; see https://en.wikipedia.org/wiki/Long_double) */
+#define MAX_FLT_WIDTH 17
 
-static int try_size(int logdepth, char *cflags, char *ldflags, const char *type, int use_stdint, const char *path, const char **sizearr, unsigned char *inc_stdint)
+static int try_size(int logdepth, char *cflags, char *ldflags, const char *type, int use_stdint, const char *path, const char **sizearr, unsigned char *inc_stdint, const int max_width)
 {
 	char *out = NULL;
 	const char *test_c_template =
@@ -66,7 +68,7 @@ static int try_size(int logdepth, char *cflags, char *ldflags, const char *type,
 
 		if (strncmp(out, "OK", 2) == 0) {
 			size = atoi(out+3);
-			if ((size > 0) && (size < MAX_INT_WIDTH)) {
+			if ((size > 0) && (size < max_width)) {
 				sprintf(test_c, "%d", size);
 				put(path, test_c);
 				sizearr[size] = type;
@@ -87,7 +89,7 @@ static int try_size(int logdepth, char *cflags, char *ldflags, const char *type,
 	return -1;
 }
 
-int find_types_stdint(int logdepth, int fatal)
+int find_types_stdint(const char *name, int logdepth, int fatal)
 {
 	char *test_c =
 		NL "#include <stdint.h>"
@@ -109,16 +111,18 @@ int find_types_stdint(int logdepth, int fatal)
 	return try_fail(logdepth, "libs/types/stdint");
 }
 
-int find_types_sizes(int logdepth, int fatal)
+int find_types_sizes(const char *name, int logdepth, int fatal)
 {
 	const char *stdint;
 	const char *sizearr_u[MAX_INT_WIDTH];
 	const char *sizearr_s[MAX_INT_WIDTH];
+	const char *sizearr_f[MAX_FLT_WIDTH];
 	unsigned char inc_stdint_u[MAX_INT_WIDTH];
 	unsigned char inc_stdint_s[MAX_INT_WIDTH];
 	int n;
 	const char *includes = "";
 	const char *path_template = "sys/types/size/%d_%c_int";
+	const char *path_template_f = "sys/types/size/%d_float";
 	char path[64];
 
 	require("cc/cc", logdepth, fatal);
@@ -131,30 +135,36 @@ int find_types_sizes(int logdepth, int fatal)
 		inc_stdint_u[n] = 0;
 		inc_stdint_s[n] = 0;
 	}
+	for(n = 0; n < MAX_FLT_WIDTH; n++)
+		sizearr_f[n] = NULL;
 
-	try_size(logdepth+1, NULL, NULL, "unsigned long long int", 0, "sys/types/size/unsigned_long_long_int", sizearr_u, NULL);
-	try_size(logdepth+1, NULL, NULL, "unsigned char", 0, "sys/types/size/unsigned_char", sizearr_u, NULL);
-	try_size(logdepth+1, NULL, NULL, "unsigned short int", 0, "sys/types/size/unsigned_short_int", sizearr_u, NULL);
-	try_size(logdepth+1, NULL, NULL, "unsigned int", 0, "sys/types/size/unsigned_int", sizearr_u, NULL);
-	try_size(logdepth+1, NULL, NULL, "unsigned long int", 0, "sys/types/size/unsigned_long_int", sizearr_u, NULL);
+	try_size(logdepth+1, NULL, NULL, "unsigned long long int", 0, "sys/types/size/unsigned_long_long_int", sizearr_u, NULL, MAX_INT_WIDTH);
+	try_size(logdepth+1, NULL, NULL, "unsigned char", 0, "sys/types/size/unsigned_char", sizearr_u, NULL, MAX_INT_WIDTH);
+	try_size(logdepth+1, NULL, NULL, "unsigned short int", 0, "sys/types/size/unsigned_short_int", sizearr_u, NULL, MAX_INT_WIDTH);
+	try_size(logdepth+1, NULL, NULL, "unsigned int", 0, "sys/types/size/unsigned_int", sizearr_u, NULL, MAX_INT_WIDTH);
+	try_size(logdepth+1, NULL, NULL, "unsigned long int", 0, "sys/types/size/unsigned_long_int", sizearr_u, NULL, MAX_INT_WIDTH);
 
-	try_size(logdepth+1, NULL, NULL, "signed long long int", 0, "sys/types/size/signed_long_long_int", sizearr_s, NULL);
-	try_size(logdepth+1, NULL, NULL, "signed char", 0, "sys/types/size/signed_char", sizearr_s, NULL);
-	try_size(logdepth+1, NULL, NULL, "signed short int", 0, "sys/types/size/signed_short_int", sizearr_s, NULL);
-	try_size(logdepth+1, NULL, NULL, "signed int", 0, "sys/types/size/signed_int", sizearr_s, NULL);
-	try_size(logdepth+1, NULL, NULL, "signed long int", 0, "sys/types/size/signed_long_int", sizearr_s, NULL);
+	try_size(logdepth+1, NULL, NULL, "signed long long int", 0, "sys/types/size/signed_long_long_int", sizearr_s, NULL, MAX_INT_WIDTH);
+	try_size(logdepth+1, NULL, NULL, "signed char", 0, "sys/types/size/signed_char", sizearr_s, NULL, MAX_INT_WIDTH);
+	try_size(logdepth+1, NULL, NULL, "signed short int", 0, "sys/types/size/signed_short_int", sizearr_s, NULL, MAX_INT_WIDTH);
+	try_size(logdepth+1, NULL, NULL, "signed int", 0, "sys/types/size/signed_int", sizearr_s, NULL, MAX_INT_WIDTH);
+	try_size(logdepth+1, NULL, NULL, "signed long int", 0, "sys/types/size/signed_long_int", sizearr_s, NULL, MAX_INT_WIDTH);
 
 	if ((stdint != NULL) && (istrue(stdint))) {
-		try_size(logdepth+1, NULL, NULL, "uint8_t", 1, "sys/types/size/uint8_t", sizearr_u, inc_stdint_u);
-		try_size(logdepth+1, NULL, NULL, "uint16_t", 1, "sys/types/size/uint16_t", sizearr_u, inc_stdint_u);
-		try_size(logdepth+1, NULL, NULL, "uint32_t", 1, "sys/types/size/uint32_t", sizearr_u, inc_stdint_u);
-		try_size(logdepth+1, NULL, NULL, "uint64_t", 1, "sys/types/size/uint64_t", sizearr_u, inc_stdint_u);
-		try_size(logdepth+1, NULL, NULL, "int8_t", 1, "sys/types/size/int8_t", sizearr_s, inc_stdint_s);
-		try_size(logdepth+1, NULL, NULL, "int16_t", 1, "sys/types/size/int16_t", sizearr_s, inc_stdint_s);
-		try_size(logdepth+1, NULL, NULL, "int32_t", 1, "sys/types/size/int32_t", sizearr_s, inc_stdint_s);
-		try_size(logdepth+1, NULL, NULL, "int64_t", 1, "sys/types/size/int64_t", sizearr_s, inc_stdint_s);
+		try_size(logdepth+1, NULL, NULL, "uint8_t", 1, "sys/types/size/uint8_t", sizearr_u, inc_stdint_u, MAX_INT_WIDTH);
+		try_size(logdepth+1, NULL, NULL, "uint16_t", 1, "sys/types/size/uint16_t", sizearr_u, inc_stdint_u, MAX_INT_WIDTH);
+		try_size(logdepth+1, NULL, NULL, "uint32_t", 1, "sys/types/size/uint32_t", sizearr_u, inc_stdint_u, MAX_INT_WIDTH);
+		try_size(logdepth+1, NULL, NULL, "uint64_t", 1, "sys/types/size/uint64_t", sizearr_u, inc_stdint_u, MAX_INT_WIDTH);
+		try_size(logdepth+1, NULL, NULL, "int8_t", 1, "sys/types/size/int8_t", sizearr_s, inc_stdint_s, MAX_INT_WIDTH);
+		try_size(logdepth+1, NULL, NULL, "int16_t", 1, "sys/types/size/int16_t", sizearr_s, inc_stdint_s, MAX_INT_WIDTH);
+		try_size(logdepth+1, NULL, NULL, "int32_t", 1, "sys/types/size/int32_t", sizearr_s, inc_stdint_s, MAX_INT_WIDTH);
+		try_size(logdepth+1, NULL, NULL, "int64_t", 1, "sys/types/size/int64_t", sizearr_s, inc_stdint_s, MAX_INT_WIDTH);
 	}
 
+	try_size(logdepth+1, NULL, NULL, "float", 0, "sys/types/size/float", sizearr_f, NULL, MAX_FLT_WIDTH);
+	try_size(logdepth+1, NULL, NULL, "double", 0, "sys/types/size/double", sizearr_f, NULL, MAX_FLT_WIDTH);
+	try_size(logdepth+1, NULL, NULL, "long double", 0, "sys/types/size/long_double", sizearr_f, NULL, MAX_FLT_WIDTH);
+
 	for(n = 0; n < MAX_INT_WIDTH; n++) {
 		if (sizearr_u[n] != NULL) {
 			report("Found best fit %d bytes wide uint: %s\n", n, sizearr_u[n]);
@@ -171,6 +181,13 @@ int find_types_sizes(int logdepth, int fatal)
 				includes = "#include <stdint.h>";
 		}
 	}
+	for(n = 0; n < MAX_FLT_WIDTH; n++) {
+		if (sizearr_f[n] != NULL) {
+			report("Found best fit %d bytes wide float: %s\n", n, sizearr_f[n]);
+			sprintf(path, path_template_f, n);
+			put(path, sizearr_f[n]);
+		}
+	}
 
 	put("sys/types/size/presents", strue); /* to avoid redetection */
 	put("sys/types/size/includes", includes);
@@ -179,7 +196,7 @@ int find_types_sizes(int logdepth, int fatal)
 }
 
 
-int find_types_something_t(int logdepth, int fatal, const char *prefix, const char *typ, const char *define, const char *try_include)
+int find_types_something_t(const char *name, int logdepth, int fatal, const char *prefix, const char *typ, const char *define, const char *try_include)
 {
 	char *out = NULL;
 	int res;
@@ -323,53 +340,63 @@ int find_types_something_t(int logdepth, int fatal, const char *prefix, const ch
 }
 
 
-int find_types_size_t(int logdepth, int fatal)
+int find_types_size_t(const char *name, int logdepth, int fatal)
 {
-	return find_types_something_t(logdepth, fatal, "sys/types", "size_t", NULL, NULL);
+	return find_types_something_t(name, logdepth, fatal, "sys/types", "size_t", NULL, NULL);
 }
 
-int find_types_off_t(int logdepth, int fatal)
+int find_types_off_t(const char *name, int logdepth, int fatal)
 {
-	return find_types_something_t(logdepth, fatal, "sys/types", "off_t", NULL, NULL);
+	return find_types_something_t(name, logdepth, fatal, "sys/types", "off_t", NULL, NULL);
 }
 
-int find_types_off64_t(int logdepth, int fatal)
+int find_types_off64_t(const char *name, int logdepth, int fatal)
 {
-	return find_types_something_t(logdepth, fatal, "sys/types", "off64_t", NULL, NULL) &&
-	       find_types_something_t(logdepth, fatal, "sys/types", "off64_t", "#define _LARGEFILE64_SOURCE", NULL);
+	return find_types_something_t(name, logdepth, fatal, "sys/types", "off64_t", NULL, NULL) &&
+	       find_types_something_t(name, logdepth, fatal, "sys/types", "off64_t", "#define _LARGEFILE64_SOURCE", NULL);
 }
 
-int find_types_gid_t(int logdepth, int fatal)
+int find_types_gid_t(const char *name, int logdepth, int fatal)
 {
-	return find_types_something_t(logdepth, fatal, "sys/types", "gid_t", NULL, NULL);
+	return find_types_something_t(name, logdepth, fatal, "sys/types", "gid_t", NULL, NULL);
 }
 
-int find_types_uid_t(int logdepth, int fatal)
+int find_types_uid_t(const char *name, int logdepth, int fatal)
 {
-	return find_types_something_t(logdepth, fatal, "sys/types", "uid_t", NULL, NULL);
+	return find_types_something_t(name, logdepth, fatal, "sys/types", "uid_t", NULL, NULL);
 }
 
-int find_types_pid_t(int logdepth, int fatal)
+int find_types_pid_t(const char *name, int logdepth, int fatal)
 {
-	return find_types_something_t(logdepth, fatal, "sys/types", "pid_t", NULL, NULL);
+	return find_types_something_t(name, logdepth, fatal, "sys/types", "pid_t", NULL, NULL);
 }
 
-int find_types_mode_t(int logdepth, int fatal)
+int find_types_mode_t(const char *name, int logdepth, int fatal)
 {
-	return find_types_something_t(logdepth, fatal, "sys/types", "mode_t", NULL, NULL);
+	return find_types_something_t(name, logdepth, fatal, "sys/types", "mode_t", NULL, NULL);
 }
 
-int find_types_nlink_t(int logdepth, int fatal)
+int find_types_nlink_t(const char *name, int logdepth, int fatal)
 {
-	return find_types_something_t(logdepth, fatal, "sys/types", "nlink_t", NULL, NULL);
+	return find_types_something_t(name, logdepth, fatal, "sys/types", "nlink_t", NULL, NULL);
 }
 
-int find_types_ptrdiff_t(int logdepth, int fatal)
+int find_types_ptrdiff_t(const char *name, int logdepth, int fatal)
 {
-	return find_types_something_t(logdepth, fatal, "sys/types", "ptrdiff_t", NULL, NULL);
+	return find_types_something_t(name, logdepth, fatal, "sys/types", "ptrdiff_t", NULL, NULL);
 }
 
-int find_types_void_ptr(int logdepth, int fatal)
+int find_types_dev_t(const char *name, int logdepth, int fatal)
 {
-	return find_types_something_t(logdepth, fatal, "sys/types", "void *", NULL, NULL);
+	return find_types_something_t(name, logdepth, fatal, "sys/types", "dev_t", NULL, NULL);
+}
+
+int find_types_ino_t(const char *name, int logdepth, int fatal)
+{
+	return find_types_something_t(name, logdepth, fatal, "sys/types", "ino_t", NULL, NULL);
+}
+
+int find_types_void_ptr(const char *name, int logdepth, int fatal)
+{
+	return find_types_something_t(name, logdepth, fatal, "sys/types", "void *", NULL, NULL);
 }
