@@ -10,6 +10,8 @@ void deps_sul_init()
 	dep_add("libs/sul/libxml2/*",  find_sul_libxml2);
 	dep_add("libs/sul/pcre/*",     find_sul_pcre);
 	dep_add("libs/sul/aspell/*",   find_sul_aspell);
+	dep_add("libs/sul/regex/*",    find_sul_regex);
+	dep_add("libs/sul/libext2fs/*",find_sul_libext2fs);
 
 	dep_add("libs/sul/genht/*",    find_sul_genht);
 	dep_add("libs/sul/genhvbox/*", find_sul_genhvbox);
