@@ -7,7 +7,7 @@
 
 #define is_space(c)  (((c) == ' ') || ((c) == '\t'))
 #define is_sep(c)    (((c) == '\n') || ((c) == '\r') || ((c) == ';'))
-#define is_addr(c)   ( (((c) >= '0') && ((c) <= '9')) || (((c) >= 'a') && ((c) <= 'z')) || (((c) >= 'A') && ((c) <= 'Z')) || ((c) == '_') || ((c) == '?') || ((c) == '.') || ((c) == ',')  || ((c) == ',') || ((c) == '-') || ((c) == '/') )
+#define is_addr(c)   ( (((c) >= '0') && ((c) <= '9')) || (((c) >= 'a') && ((c) <= 'z')) || (((c) >= 'A') && ((c) <= 'Z')) || ((c) == '_') || ((c) == '?') || ((c) == '.') || ((c) == ',')  || ((c) == ',') || ((c) == '-') || ((c) == '/') || ((c) == '&') )
 
 /* this local copy is to make tmpasm compile independently */
 static char *strclone(const char *str)
@@ -312,7 +312,7 @@ static void got_kw(tmpasm_t *ctx, tmpasm_kw_t kw, int terminated)
 
 
 			if (TOP->kw == KW_SWITCH)
-				TOP->kw = KW_none;
+				TOP->kw = TOP->old_kw;
 			else {
 				pop(ctx);
 
@@ -384,6 +384,7 @@ static void got_kw(tmpasm_t *ctx, tmpasm_kw_t kw, int terminated)
 			}
 			TOP->last_code = code_append(ctx, KW_SWITCH);
 			TOP->state = ST_PREDATA;
+			TOP->old_kw = TOP->kw;
 			TOP->kw = KW_SWITCH;
 			loc_update();
 			break;
