@@ -77,7 +77,14 @@ static char *scc_path(scc_t *ud, const char *fn)
 static const char *scc_get(tmpasm_t *ctx, const char *addr)
 {
 	(void) ctx;  /* not used */
-	if (*addr == '?') {
+
+	if (*addr == '&') { /* return whether exists */
+		if (get(addr+1) != NULL)
+			return strue;
+		return sfalse;
+	}
+
+	if (*addr == '?') { /* safe get: return "" instead of NULL to avoid runtime error */
 		const char *res = get(addr+1);
 		if (res == NULL)
 			return "";
@@ -220,11 +227,14 @@ static void instr_report(tmpasm_t *ctx, char *iname, int argc, tmpasm_arg_t *arg
 
 static void instr_abort(tmpasm_t *ctx, char *iname, int argc, tmpasm_arg_t *argv[])
 {
-	(void) ctx;  /* not used */
+	scc_t *ud = (scc_t *)ctx->user_data;
 	(void) iname;  /* not used */
 	(void) argc;  /* not used */
 	(void) argv;  /* not used */
 	report("Abort requested by template.\n");
+	if (ud->fout) fflush(ud->fout);
+	fflush(stdout);
+	fflush(stderr);
 	abort();
 }
 
