@@ -7,6 +7,8 @@
 #include "find_slang.h"
 #include "find_gpm.h"
 #include "find_pty.h"
+#include "find_ioctl.h"
+#include "find_term.h"
 
 void deps_tty_init()
 {
@@ -22,4 +24,9 @@ void deps_tty_init()
 	dep_add("libs/tty/ncurses/resizeterm/*",find_ncurses_resizeterm);
 	dep_add("libs/tty/ncurses/*",          find_ncurses);
 	dep_add("libs/tty/ncursesw/*",         find_ncursesw);
+
+	dep_add("libs/tty/ioctl/*",            find_tty_ioctl);
+
+	dep_add("libs/tty/cfmakeraw/*",        find_tty_cfmakeraw);
+	dep_add("libs/tty/cfsetspeed/*",       find_tty_cfsetspeed);
 }
