@@ -1,7 +1,8 @@
 /*
  *                            COPYRIGHT
  *
- *  PCB, interactive printed circuit board design
+ *  pcb-rnd, interactive printed circuit board design
+ *  (this file is based on PCB, interactive printed circuit board design)
  *  Copyright (C) 1994,1995,1996,2006 Thomas Nau
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -18,9 +19,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- *  Contact addresses for paper mail and Email:
- *  Thomas Nau, Schlehenweg 15, 88471 Baustetten, Germany
- *  Thomas.Nau@rz.uni-ulm.de
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  *
  */
 
@@ -46,7 +48,6 @@ struct pcb_attribute_list_s {
 /* Returns NULL if the name isn't found, else the value for that named
    attribute.  */
 char *pcb_attribute_get(pcb_attribute_list_t * list, const char *name);
-pcb_attribute_t *pcb_attribute_get_attr(pcb_attribute_list_t * list, const char *name);
 
 /* Adds an attribute to the list.  If the attribute already exists, the value
    is replaced. Returns non-zero if an existing attribute was replaced.  */
