@@ -1,7 +1,8 @@
 /*
  *                            COPYRIGHT
  *
- *  PCB, interactive printed circuit board design
+ *  pcb-rnd, interactive printed circuit board design
+ *  (this file is based on PCB, interactive printed circuit board design)
  *  Copyright (C) 1994,1995,1996 Thomas Nau
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -18,9 +19,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- *  Contact addresses for paper mail and Email:
- *  Thomas Nau, Schlehenweg 15, 88471 Baustetten, Germany
- *  Thomas.Nau@rz.uni-ulm.de
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  *
  */
 #include "config.h"
@@ -109,7 +111,7 @@ pcb_board_t *pcb_board_new_(pcb_bool SetDefaultNames)
 
 	pcb_font_create_default(ptr);
 
-	return (ptr);
+	return ptr;
 }
 
 #include "defpcb_internal.c"
@@ -242,7 +244,7 @@ pcb_bool pcb_board_change_name(char *Name)
 	free(PCB->Name);
 	PCB->Name = Name;
 	pcb_board_changed(0);
-	return (pcb_true);
+	return pcb_true;
 }
 
 void pcb_board_resize(pcb_coord_t Width, pcb_coord_t Height)
@@ -332,7 +334,12 @@ void pcb_board_set_text_scale(int Scale)
 /* sets or resets changed flag and redraws status */
 void pcb_board_set_changed_flag(pcb_bool New)
 {
+	pcb_bool old = PCB->Changed;
+
 	PCB->Changed = New;
+
+	if (old != New)
+		pcb_event(PCB_EVENT_BOARD_META_CHANGED, NULL);
 }
 
 
