@@ -1,7 +1,8 @@
 /*
  *                            COPYRIGHT
  *
- *  PCB, interactive printed circuit board design
+ *  pcb-rnd, interactive printed circuit board design
+ *  (this file is based on PCB, interactive printed circuit board design)
  *  Copyright (C) 1994,1995,1996, 2004 Thomas Nau
  *  15 Oct 2008 Ineiev: add different crosshair shapes
  *
@@ -19,9 +20,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
- *  Contact addresses for paper mail and Email:
- *  Thomas Nau, Schlehenweg 15, 88471 Baustetten, Germany
- *  Thomas.Nau@rz.uni-ulm.de
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  *
  */
 
@@ -72,7 +74,7 @@ struct pcb_board_s {
 	  ViaOn,											/* visibility flags */
 	  RatOn, InvisibleObjectsOn, PinOn,
 	  RatDraw,										/* we're drawing rats */
-	  SubcOn, SubcPartsOn, padstack_mark_on;
+	  SubcOn, SubcPartsOn, padstack_mark_on, hole_on;
 	pcb_bool loose_subc;          /* when set, subc parts are not locked into the subc */
 	pcb_coord_t CursorX,									/* cursor position as saved with layout */
 	  CursorY;
@@ -178,7 +180,7 @@ void pcb_board_set_changed_flag(pcb_bool New);
 void pcb_board_changed(int reverted);
 
 
-/* Obsolete pcb_board_t field accessors - do not use */
+/* pcb_board_t field accessors - do not use; required for path.[ch] to not depend on board.h */
 const char *pcb_board_get_filename(void);
 const char *pcb_board_get_name(void);
 
