@@ -21,6 +21,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  */
 
 #include "config.h"
@@ -45,8 +49,6 @@ typedef struct {
 } desc_t;
 
 static desc_t desc[] = {
-	{PCB_BRAVE_PSTK_VIA, "pstk_via", "use padstack for vias",
-		"placing new vias will place padstacks instsad of old via objects", 0 },
 	{0, NULL, NULL, NULL}
 };
 
