@@ -1,7 +1,8 @@
 /*
  *                            COPYRIGHT
  *
- *  PCB, interactive printed circuit board design
+ *  pcb-rnd, interactive printed circuit board design
+ *  (this file is based on PCB, interactive printed circuit board design)
  *  Copyright (C) 1994,1995,1996, 2005 Thomas Nau
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -18,9 +19,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- *  Contact addresses for paper mail and Email:
- *  Thomas Nau, Schlehenweg 15, 88471 Baustetten, Germany
- *  Thomas.Nau@rz.uni-ulm.de
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  *
  */
 
@@ -47,6 +49,7 @@
 #include "obj_all_op.h"
 #include "obj_subc_op.h"
 #include "obj_pstk_op.h"
+#include "obj_pstk.h"
 #include "layer_grp.h"
 #include "event.h"
 #include "safe_fs.h"
@@ -71,21 +74,21 @@ static pcb_opfunc_t AddBufferFunctions = {
 
 #warning TODO: rename this; move_to_buffer is also used to move from buffer to pcb
 static pcb_opfunc_t MoveBufferFunctions = {
-	pcb_lineop_move_to_buffer,
-	pcb_textop_move_to_buffer,
-	pcb_polyop_move_to_buffer,
-	pcb_viaop_move_to_buffer,
-	pcb_elemop_move_to_buffer,
+	pcb_lineop_move_buffer,
+	pcb_textop_move_buffer,
+	pcb_polyop_move_buffer,
+	pcb_viaop_move_buffer,
+	pcb_elemop_move_buffer,
 	NULL,
 	NULL,
 	NULL,
 	NULL,
 	NULL,
-	pcb_arcop_move_to_buffer,
-	pcb_ratop_move_to_buffer,
+	pcb_arcop_move_buffer,
+	pcb_ratop_move_buffer,
 	NULL,
-	pcb_subcop_move_to_buffer,
-	pcb_pstkop_move_to_buffer,
+	pcb_subcop_move_buffer,
+	pcb_pstkop_move_buffer,
 };
 
 int pcb_set_buffer_bbox(pcb_buffer_t *Buffer)
@@ -248,7 +251,7 @@ pcb_bool pcb_buffer_load_layout(pcb_board_t *pcb, pcb_buffer_t *Buffer, const ch
 		pcb_board_remove(newPCB);
 		Buffer->from_outside = 1;
 		pcb_event(PCB_EVENT_LAYERS_CHANGED, NULL); /* undo the events generated on load */
-		return (pcb_true);
+		return pcb_true;
 	}
 
 	/* release unused memory */
@@ -257,10 +260,10 @@ pcb_bool pcb_buffer_load_layout(pcb_board_t *pcb, pcb_buffer_t *Buffer, const ch
 		PCB_CLEAR_PARENT(Buffer->Data);
 	pcb_event(PCB_EVENT_LAYERS_CHANGED, NULL); /* undo the events generated on load */
 	Buffer->from_outside = 0;
-	return (pcb_false);
+	return pcb_false;
 }
 
-void pcb_buffer_rotate(pcb_buffer_t *Buffer, pcb_uint8_t Number)
+void pcb_buffer_rotate90(pcb_buffer_t *Buffer, pcb_uint8_t Number)
 {
 	/* rotate vias */
 	PCB_VIA_LOOP(Buffer->Data);
@@ -268,7 +271,13 @@ void pcb_buffer_rotate(pcb_buffer_t *Buffer, pcb_uint8_t Number)
 		pcb_r_delete_entry(Buffer->Data->via_tree, (pcb_box_t *) via);
 		PCB_VIA_ROTATE90(via, Buffer->X, Buffer->Y, Number);
 		pcb_pin_bbox(via);
-		pcb_r_insert_entry(Buffer->Data->via_tree, (pcb_box_t *) via, 0);
+		pcb_r_insert_entry(Buffer->Data->via_tree, (pcb_box_t *) via);
+	}
+	PCB_END_LOOP;
+
+	PCB_PADSTACK_LOOP(Buffer->Data);
+	{
+		pcb_pstk_rotate90(padstack, Buffer->X, Buffer->Y, Number);
 	}
 	PCB_END_LOOP;
 
@@ -290,28 +299,28 @@ void pcb_buffer_rotate(pcb_buffer_t *Buffer, pcb_uint8_t Number)
 	{
 		pcb_r_delete_entry(layer->line_tree, (pcb_box_t *) line);
 		pcb_line_rotate90(line, Buffer->X, Buffer->Y, Number);
-		pcb_r_insert_entry(layer->line_tree, (pcb_box_t *) line, 0);
+		pcb_r_insert_entry(layer->line_tree, (pcb_box_t *) line);
 	}
 	PCB_ENDALL_LOOP;
 	PCB_ARC_ALL_LOOP(Buffer->Data);
 	{
 		pcb_r_delete_entry(layer->arc_tree, (pcb_box_t *) arc);
 		pcb_arc_rotate90(arc, Buffer->X, Buffer->Y, Number);
-		pcb_r_insert_entry(layer->arc_tree, (pcb_box_t *) arc, 0);
+		pcb_r_insert_entry(layer->arc_tree, (pcb_box_t *) arc);
 	}
 	PCB_ENDALL_LOOP;
 	PCB_TEXT_ALL_LOOP(Buffer->Data);
 	{
 		pcb_r_delete_entry(layer->text_tree, (pcb_box_t *) text);
 		pcb_text_rotate90(text, Buffer->X, Buffer->Y, Number);
-		pcb_r_insert_entry(layer->text_tree, (pcb_box_t *) text, 0);
+		pcb_r_insert_entry(layer->text_tree, (pcb_box_t *) text);
 	}
 	PCB_ENDALL_LOOP;
 	PCB_POLY_ALL_LOOP(Buffer->Data);
 	{
 		pcb_r_delete_entry(layer->polygon_tree, (pcb_box_t *) polygon);
 		pcb_poly_rotate90(polygon, Buffer->X, Buffer->Y, Number);
-		pcb_r_insert_entry(layer->polygon_tree, (pcb_box_t *) polygon, 0);
+		pcb_r_insert_entry(layer->polygon_tree, (pcb_box_t *) polygon);
 	}
 	PCB_ENDALL_LOOP;
 
@@ -320,7 +329,7 @@ void pcb_buffer_rotate(pcb_buffer_t *Buffer, pcb_uint8_t Number)
 	pcb_box_rotate90(&Buffer->BoundingBox, Buffer->X, Buffer->Y, Number);
 }
 
-void pcb_buffer_free_rotate(pcb_buffer_t *Buffer, pcb_angle_t angle)
+void pcb_buffer_rotate(pcb_buffer_t *Buffer, pcb_angle_t angle)
 {
 	double cosa, sina;
 
@@ -334,6 +343,12 @@ void pcb_buffer_free_rotate(pcb_buffer_t *Buffer, pcb_angle_t angle)
 	}
 	PCB_END_LOOP;
 
+	PCB_PADSTACK_LOOP(Buffer->Data);
+	{
+		pcb_pstk_rotate(padstack, Buffer->X, Buffer->Y, cosa, sina, angle);
+	}
+	PCB_END_LOOP;
+
 	/* elements */
 	PCB_ELEMENT_LOOP(Buffer->Data);
 	{
@@ -376,7 +391,7 @@ pcb_data_t *pcb_buffer_new(pcb_board_t *pcb)
 
 /* -------------------------------------------------------------------------- */
 
-static const char pcb_acts_FreeRotateBuffer[] = "pcb_buffer_free_rotate([Angle])";
+static const char pcb_acts_FreeRotateBuffer[] = "FreeRotateBuffer([Angle])";
 
 static const char pcb_acth_FreeRotateBuffer[] =
 	"Rotates the current paste buffer contents by the specified angle.  The\n"
@@ -402,7 +417,7 @@ int pcb_act_FreeRotateBuffer(int argc, const char **argv, pcb_coord_t x, pcb_coo
 		return 0;
 
 	pcb_notify_crosshair_change(pcb_false);
-	pcb_buffer_free_rotate(PCB_PASTEBUFFER, strtod(angle_s, 0));
+	pcb_buffer_rotate(PCB_PASTEBUFFER, strtod(angle_s, 0));
 	pcb_notify_crosshair_change(pcb_true);
 	return 0;
 }
@@ -590,6 +605,8 @@ pcb_bool pcb_buffer_copy_to_layout(pcb_board_t *pcb, pcb_coord_t X, pcb_coord_t
 	printf("Entering CopyPastebufferToLayout.....\n");
 #endif
 
+	pcb_data_clip_inhibit_inc(pcb->Data);
+
 	/* set movement vector */
 	ctx.copy.pcb = pcb;
 	ctx.copy.DeltaX = X - PCB_PASTEBUFFER->X;
@@ -661,6 +678,10 @@ pcb_bool pcb_buffer_copy_to_layout(pcb_board_t *pcb, pcb_coord_t X, pcb_coord_t
 	/* paste subcircuits */
 	PCB_SUBC_LOOP(PCB_PASTEBUFFER->Data);
 	{
+		if (pcb->is_footprint) {
+			pcb_message(PCB_MSG_WARNING, "Can not paste subcircuit in the footprint edit mode\n");
+			break;
+		}
 		pcb_subcop_copy(&ctx, subc);
 		changed = pcb_true;
 	}
@@ -708,7 +729,9 @@ pcb_bool pcb_buffer_copy_to_layout(pcb_board_t *pcb, pcb_coord_t X, pcb_coord_t
 	printf("  .... Leaving CopyPastebufferToLayout.\n");
 #endif
 
-	return (changed);
+	pcb_data_clip_inhibit_dec(pcb->Data, pcb_true);
+
+	return changed;
 }
 
 void pcb_buffer_set_number(int Number)
@@ -729,6 +752,7 @@ static const char pcb_acts_PasteBuffer[] =
 	"PasteBuffer(Rotate, 1..3)\n"
 	"PasteBuffer(Convert|Restore|Mirror)\n"
 	"PasteBuffer(ToLayout, X, Y, units)\n"
+	"PasteBuffer(ToLayout, crosshair)\n"
 	"PasteBuffer(Save, Filename, [format], [force])";
 
 static const char pcb_acth_PasteBuffer[] = "Various operations on the paste buffer.";
@@ -776,7 +800,8 @@ coordinates in the layout.  The @code{X} and @code{Y} are treated like
 @code{+} or @code{-}, then that amount is relative to the last
 location.  Otherwise, it's absolute.  Units can be
 @code{mil} or @code{mm}; if unspecified, units are PCB's internal
-units, currently 1/100 mil.
+units, currently 1/100 mil. If "crosshair" is used instead of coordinates,
+the paste happens at the current crosshair coords.
 
 
 @item 1..PCB_MAX_BUFFER
@@ -837,7 +862,7 @@ static int pcb_act_PasteBuffer(int argc, const char **argv, pcb_coord_t x, pcb_c
 
 		case F_Rotate:
 			if (sbufnum) {
-				pcb_buffer_rotate(PCB_PASTEBUFFER, (pcb_uint8_t) atoi(sbufnum));
+				pcb_buffer_rotate90(PCB_PASTEBUFFER, (pcb_uint8_t) atoi(sbufnum));
 				pcb_crosshair_range_to_buffer();
 			}
 			break;
@@ -883,10 +908,13 @@ static int pcb_act_PasteBuffer(int argc, const char **argv, pcb_coord_t x, pcb_c
 				static pcb_coord_t oldx = 0, oldy = 0;
 				pcb_coord_t x, y;
 				pcb_bool absolute;
-
 				if (argc == 1) {
 					x = y = 0;
 				}
+				else if (strcmp(argv[1], "crosshair") == 0) {
+					x = pcb_crosshair.X;
+					y = pcb_crosshair.Y;
+				}
 				else if (argc == 3 || argc == 4) {
 					x = pcb_get_value(PCB_ACTION_ARG(1), PCB_ACTION_ARG(3), &absolute, NULL);
 					if (!absolute)
