@@ -1,7 +1,8 @@
 /*
  *                            COPYRIGHT
  *
- *  PCB, interactive printed circuit board design
+ *  pcb-rnd, interactive printed circuit board design
+ *  (this file is based on PCB, interactive printed circuit board design)
  *  Copyright (C) 1994,1995,1996 Thomas Nau
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -18,9 +19,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- *  Contact addresses for paper mail and Email:
- *  Thomas Nau, Schlehenweg 15, 88471 Baustetten, Germany
- *  Thomas.Nau@rz.uni-ulm.de
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  *
  */
 
@@ -55,8 +57,8 @@ void pcb_buffer_move_selected(pcb_board_t *pcb, pcb_buffer_t *Buffer, pcb_coord_
 pcb_bool pcb_buffer_load_layout(pcb_board_t *pcb, pcb_buffer_t *Buffer, const char *Filename, const char *fmt);
 
 /* rotates the contents of the pastebuffer by Number * 90 degrees or free angle*/
-void pcb_buffer_rotate(pcb_buffer_t *Buffer, pcb_uint8_t Number);
-void pcb_buffer_free_rotate(pcb_buffer_t *Buffer, pcb_angle_t angle);
+void pcb_buffer_rotate90(pcb_buffer_t *Buffer, pcb_uint8_t Number);
+void pcb_buffer_rotate(pcb_buffer_t *Buffer, pcb_angle_t angle);
 
 /* flip elements/subcircuits/tracks from one side to the other; also swap
    the layer stackup of padstacks */
