@@ -1,9 +1,10 @@
 /*
  *                            COPYRIGHT
  *
- *  PCB, interactive printed circuit board design
+ *  pcb-rnd, interactive printed circuit board design
+ *  (this file is based on PCB, interactive printed circuit board design)
  *  Copyright (C) 1994,1995,1996,2006 Thomas Nau
- *  pcb-rnd Copyright (C) 2017 Alain Vigne
+ *  pcb-rnd Copyright (C) 2017,2018 Alain Vigne
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -19,14 +20,16 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- *  Contact addresses for paper mail and Email:
- *  Thomas Nau, Schlehenweg 15, 88471 Baustetten, Germany
- *  Thomas.Nau@rz.uni-ulm.de
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  *
  */
 
 #include "config.h"
 #include <signal.h>
+#include <genht/hash.h>
 #include <genvector/gds_char.h>
 #include "conf_core.h"
 #include "board.h"
@@ -89,7 +92,7 @@ char *pcb_get_info_copyright(void)
 		gds_init(&info);
 
 		gds_append_str(&info, "Recent pcb-rnd copyright:\n");
-		gds_append_str(&info, "Copyright (C) Tibor Palinkas 2013..2017 (pcb-rnd)\n");
+		gds_append_str(&info, "Copyright (C) Tibor Palinkas 2013..2018 (pcb-rnd)\n");
 		gds_append_str(&info, "For a more complete list of pcb-rnd authors and\ncontributors, see http://repo.hu/cgi-bin/pcb-rnd-people.cgi\n\n");
 
 		gds_append_str(&info, "Historical copyright:\n");
@@ -212,6 +215,25 @@ char *pcb_get_info_compile_options(void)
 	return info.array;
 }
 
+/** pcb_get_info_license:
+ *  Returns a string about how the program is licensed.
+ */
+char *pcb_get_info_license(void)
+{
+	static gds_t info;
+	static int first_time = 1;
+
+	if (first_time) {
+		first_time = 0;
+		gds_init(&info);
+
+		gds_append_str(&info, "pcb-rnd is licensed under the terms of the GNU\n");
+		gds_append_str(&info, "General Public License version 2\n");
+		gds_append_str(&info, "See the COPYING file for more information\n\n");
+	}
+	return info.array;
+}
+
 /** pcb_get_infostr:
  *  Returns a string that has a bunch of information about this copy of pcb.
  *  Can be used for things like "about" dialog boxes.
@@ -231,9 +253,8 @@ char *pcb_get_infostr(void)
 		tmp = pcb_get_info_copyright();
 		gds_append_str(&info, tmp);
 		gds_append_str(&info, "\n\n");
-		gds_append_str(&info, "pcb-rnd is licensed under the terms of the GNU\n");
-		gds_append_str(&info, "General Public License version 2\n");
-		gds_append_str(&info, "See the LICENSE file for more information\n\n");
+		tmp = pcb_get_info_license();
+		gds_append_str(&info, tmp);
 		tmp = pcb_get_info_websites(NULL);
 		gds_append_str(&info, tmp);
 
@@ -288,3 +309,130 @@ void pcb_catch_signal(int Signal)
 	pcb_message(PCB_MSG_ERROR, "aborted by %s signal\n", s);
 	exit(1);
 }
+
+
+/*** files loaded ***/
+
+htsp_t pcb_file_loaded;
+
+
+pcb_file_loaded_t *pcb_file_loaded_category(const char *name, int alloc)
+{
+	pcb_file_loaded_t *cat = htsp_get(&pcb_file_loaded, name);
+
+	if ((cat == NULL) && (alloc)) {
+		cat = calloc(sizeof(pcb_file_loaded_t), 1);
+		cat->type = PCB_FLT_CATEGORY;
+		cat->name = pcb_strdup(name);
+		htsp_init(&cat->data.category.children, strhash, strkeyeq);
+		htsp_set(&pcb_file_loaded, cat->name, cat);
+	}
+	return cat;
+}
+
+int pcb_file_loaded_file_free(pcb_file_loaded_t *file)
+{
+	free(file->data.file.path);
+	free(file->data.file.desc);
+	free(file->name);
+	free(file);
+	return 0;
+}
+
+int pcb_file_loaded_clear(pcb_file_loaded_t *cat)
+{
+	htsp_entry_t *e;
+
+	assert(cat->type == PCB_FLT_CATEGORY);
+
+	for (e = htsp_first(&cat->data.category.children); e; e = htsp_next(&cat->data.category.children, e)) {
+		pcb_file_loaded_file_free(e->value);
+		htsp_delentry(&cat->data.category.children, e);
+	}
+	return 0;
+}
+
+int pcb_file_loaded_clear_at(const char *catname)
+{
+	pcb_file_loaded_t *cat = pcb_file_loaded_category(catname, 0);
+	if (cat != NULL)
+		return pcb_file_loaded_clear(cat);
+	return 0;
+}
+
+/* clear the subtree of a category, keeping the category; return 0 on success */
+int pcb_file_loaded_set(pcb_file_loaded_t *cat, const char *name, const char *path, const char *desc)
+{
+	pcb_file_loaded_t *file;
+
+	assert(cat->type == PCB_FLT_CATEGORY);
+	file = htsp_get(&cat->data.category.children, name);
+	if (file != NULL) {
+		free(file->data.file.path);
+		free(file->data.file.desc);
+	}
+	else {
+		file = malloc(sizeof(pcb_file_loaded_t));
+		file->type = PCB_FLT_FILE;
+		file->name = pcb_strdup(name);
+		htsp_set(&cat->data.category.children, file->name, file);
+	}
+	if (path != NULL)
+		file->data.file.path = pcb_strdup(path);
+	else
+		file->data.file.path = NULL;
+
+	if (desc != NULL)
+		file->data.file.desc = pcb_strdup(desc);
+	else
+		file->data.file.desc = NULL;
+	return 0;
+}
+
+int pcb_file_loaded_set_at(const char *catnam, const char *name, const char *path, const char *desc)
+{
+	pcb_file_loaded_t *cat = pcb_file_loaded_category(catnam, 1);
+	return pcb_file_loaded_set(cat, name, path, desc);
+}
+
+int pcb_file_loaded_del(pcb_file_loaded_t *cat, const char *name)
+{
+	pcb_file_loaded_t *file;
+	assert(cat->type == PCB_FLT_CATEGORY);
+	file = htsp_pop(&cat->data.category.children, name);
+	if (file != NULL) {
+		if (file->type != PCB_FLT_FILE)
+			return -1;
+		pcb_file_loaded_file_free(file);
+	}
+	return 0;
+}
+
+int pcb_file_loaded_del_at(const char *catname, const char *name)
+{
+	pcb_file_loaded_t *cat = pcb_file_loaded_category(catname, 1);
+	return pcb_file_loaded_del(cat, name);
+}
+
+void pcb_file_loaded_init(void)
+{
+	htsp_init(&pcb_file_loaded, strhash, strkeyeq);
+}
+
+void pcb_file_loaded_uninit(void)
+{
+	htsp_entry_t *e;
+
+	for (e = htsp_first(&pcb_file_loaded); e; e = htsp_next(&pcb_file_loaded, e)) {
+		pcb_file_loaded_t *cat = e->value;
+		pcb_file_loaded_clear(cat);
+		free(cat->name);
+		htsp_uninit(&cat->data.category.children);
+		free(cat);
+		htsp_delentry(&pcb_file_loaded, e);
+	}
+
+	htsp_uninit(&pcb_file_loaded);
+}
+
+
