@@ -1,7 +1,8 @@
 /*
  *                            COPYRIGHT
  *
- *  PCB, interactive printed circuit board design
+ *  pcb-rnd, interactive printed circuit board design
+ *  (this file is based on PCB, interactive printed circuit board design)
  *  Copyright (C) 1994,1995,1996, 2005 Thomas Nau
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -18,9 +19,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- *  Contact addresses for paper mail and Email:
- *  Thomas Nau, Schlehenweg 15, 88471 Baustetten, Germany
- *  Thomas.Nau@rz.uni-ulm.de
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  *
  */
 
@@ -463,7 +465,7 @@ pcb_bool pcb_chg_selected_thermals(int types, int therm_style, unsigned long lid
 		pcb_draw();
 		pcb_undo_inc_serial();
 	}
-	return (change);
+	return change;
 }
 
 /* ----------------------------------------------------------------------
@@ -485,7 +487,7 @@ pcb_bool pcb_chg_selected_size(int types, pcb_coord_t Difference, pcb_bool fixIt
 		pcb_draw();
 		pcb_undo_inc_serial();
 	}
-	return (change);
+	return change;
 }
 
 /* ----------------------------------------------------------------------
@@ -510,7 +512,7 @@ pcb_bool pcb_chg_selected_clear_size(int types, pcb_coord_t Difference, pcb_bool
 		pcb_draw();
 		pcb_undo_inc_serial();
 	}
-	return (change);
+	return change;
 }
 
 /* --------------------------------------------------------------------------
@@ -532,7 +534,7 @@ pcb_bool pcb_chg_selected_2nd_size(int types, pcb_coord_t Difference, pcb_bool f
 		pcb_draw();
 		pcb_undo_inc_serial();
 	}
-	return (change);
+	return change;
 }
 
 /* ----------------------------------------------------------------------
@@ -551,7 +553,7 @@ pcb_bool pcb_chg_selected_join(int types)
 		pcb_draw();
 		pcb_undo_inc_serial();
 	}
-	return (change);
+	return change;
 }
 
 /* ----------------------------------------------------------------------
@@ -570,7 +572,7 @@ pcb_bool pcb_set_selected_join(int types)
 		pcb_draw();
 		pcb_undo_inc_serial();
 	}
-	return (change);
+	return change;
 }
 
 /* ----------------------------------------------------------------------
@@ -589,7 +591,7 @@ pcb_bool pcb_clr_selected_join(int types)
 		pcb_draw();
 		pcb_undo_inc_serial();
 	}
-	return (change);
+	return change;
 }
 
 /* ----------------------------------------------------------------------
@@ -608,7 +610,7 @@ pcb_bool pcb_chg_selected_nonetlist(int types)
 		pcb_draw();
 		pcb_undo_inc_serial();
 	}
-	return (change);
+	return change;
 }
 
 #if 0
@@ -628,7 +630,7 @@ pcb_bool SetSelectedNonetlist(int types)
 		pcb_draw();
 		pcb_undo_inc_serial();
 	}
-	return (change);
+	return change;
 }
 
 /* ----------------------------------------------------------------------
@@ -647,7 +649,7 @@ pcb_bool ClrSelectedNonetlist(int types)
 		pcb_draw();
 		pcb_undo_inc_serial();
 	}
-	return (change);
+	return change;
 }
 #endif
 
@@ -667,7 +669,7 @@ pcb_bool pcb_chg_selected_square(int types)
 		pcb_draw();
 		pcb_undo_inc_serial();
 	}
-	return (change);
+	return change;
 }
 
 /* ----------------------------------------------------------------------
@@ -689,7 +691,7 @@ pcb_bool pcb_chg_selected_angle(int types, int is_start, pcb_angle_t Difference,
 		pcb_draw();
 		pcb_undo_inc_serial();
 	}
-	return (change);
+	return change;
 }
 
 /* ----------------------------------------------------------------------
@@ -711,7 +713,7 @@ pcb_bool pcb_chg_selected_radius(int types, int is_start, pcb_angle_t Difference
 		pcb_draw();
 		pcb_undo_inc_serial();
 	}
-	return (change);
+	return change;
 }
 
 
@@ -731,7 +733,7 @@ pcb_bool pcb_set_selected_square(int types)
 		pcb_draw();
 		pcb_undo_inc_serial();
 	}
-	return (change);
+	return change;
 }
 
 /* ----------------------------------------------------------------------
@@ -750,7 +752,7 @@ pcb_bool pcb_clr_selected_square(int types)
 		pcb_draw();
 		pcb_undo_inc_serial();
 	}
-	return (change);
+	return change;
 }
 
 /* ----------------------------------------------------------------------
@@ -769,7 +771,7 @@ pcb_bool pcb_chg_selected_octagon(int types)
 		pcb_draw();
 		pcb_undo_inc_serial();
 	}
-	return (change);
+	return change;
 }
 
 /* ----------------------------------------------------------------------
@@ -788,7 +790,7 @@ pcb_bool pcb_set_selected_octagon(int types)
 		pcb_draw();
 		pcb_undo_inc_serial();
 	}
-	return (change);
+	return change;
 }
 
 /* ----------------------------------------------------------------------
@@ -807,7 +809,7 @@ pcb_bool pcb_clr_selected_octagon(int types)
 		pcb_draw();
 		pcb_undo_inc_serial();
 	}
-	return (change);
+	return change;
 }
 
 /* ----------------------------------------------------------------------
@@ -829,7 +831,7 @@ pcb_bool pcb_chg_selected_hole(void)
 		pcb_draw();
 		pcb_undo_inc_serial();
 	}
-	return (change);
+	return change;
 }
 
 /* ----------------------------------------------------------------------
@@ -850,7 +852,7 @@ pcb_bool pcb_chg_selected_paste(void)
 		pcb_draw();
 		pcb_undo_inc_serial();
 	}
-	return (change);
+	return change;
 }
 
 
@@ -873,7 +875,7 @@ pcb_bool pcb_chg_obj_size(int Type, void *Ptr1, void *Ptr2, void *Ptr3, pcb_coor
 		pcb_draw();
 		pcb_undo_inc_serial();
 	}
-	return (change);
+	return change;
 }
 
 /* ---------------------------------------------------------------------------
@@ -895,7 +897,7 @@ pcb_bool pcb_chg_obj_1st_size(int Type, void *Ptr1, void *Ptr2, void *Ptr3, pcb_
 		pcb_draw();
 		pcb_undo_inc_serial();
 	}
-	return (change);
+	return change;
 }
 
 /* ---------------------------------------------------------------------------
@@ -917,7 +919,7 @@ pcb_bool pcb_chg_obj_radius(int Type, void *Ptr1, void *Ptr2, void *Ptr3, int is
 		pcb_draw();
 		pcb_undo_inc_serial();
 	}
-	return (change);
+	return change;
 }
 
 /* ---------------------------------------------------------------------------
@@ -939,7 +941,7 @@ pcb_bool pcb_chg_obj_angle(int Type, void *Ptr1, void *Ptr2, void *Ptr3, int is_
 		pcb_draw();
 		pcb_undo_inc_serial();
 	}
-	return (change);
+	return change;
 }
 
 
@@ -965,7 +967,7 @@ pcb_bool pcb_chg_obj_clear_size(int Type, void *Ptr1, void *Ptr2, void *Ptr3, pc
 		pcb_draw();
 		pcb_undo_inc_serial();
 	}
-	return (change);
+	return change;
 }
 
 /* ---------------------------------------------------------------------------
@@ -987,7 +989,7 @@ pcb_bool pcb_chg_obj_thermal(int Type, void *Ptr1, void *Ptr2, void *Ptr3, int t
 		pcb_draw();
 		pcb_undo_inc_serial();
 	}
-	return (change);
+	return change;
 }
 
 /* ---------------------------------------------------------------------------
@@ -1010,7 +1012,7 @@ pcb_bool pcb_chg_obj_2nd_size(int Type, void *Ptr1, void *Ptr2, void *Ptr3, pcb_
 		if (incundo)
 			pcb_undo_inc_serial();
 	}
-	return (change);
+	return change;
 }
 
 /* ---------------------------------------------------------------------------
@@ -1032,7 +1034,7 @@ pcb_bool pcb_chg_obj_mask_size(int Type, void *Ptr1, void *Ptr2, void *Ptr3, pcb
 		pcb_draw();
 		pcb_undo_inc_serial();
 	}
-	return (change);
+	return change;
 }
 
 /* ---------------------------------------------------------------------------
@@ -1052,7 +1054,7 @@ void *pcb_chg_obj_name(int Type, void *Ptr1, void *Ptr2, void *Ptr3, char *Name)
 
 	result = pcb_object_operation(&ChangeNameFunctions, &ctx, Type, Ptr1, Ptr2, Ptr3);
 	pcb_draw();
-	return (result);
+	return result;
 }
 
 /* ---------------------------------------------------------------------------
@@ -1072,7 +1074,7 @@ void *pcb_chg_obj_pinnum(int Type, void *Ptr1, void *Ptr2, void *Ptr3, char *Nam
 
 	result = pcb_object_operation(&ChangePinnumFunctions, &ctx, Type, Ptr1, Ptr2, Ptr3);
 	pcb_draw();
-	return (result);
+	return result;
 }
 
 /* ---------------------------------------------------------------------------
@@ -1088,9 +1090,9 @@ pcb_bool pcb_chg_obj_join(int Type, void *Ptr1, void *Ptr2, void *Ptr3)
 	if (pcb_object_operation(&ChangeJoinFunctions, &ctx, Type, Ptr1, Ptr2, Ptr3) != NULL) {
 		pcb_draw();
 		pcb_undo_inc_serial();
-		return (pcb_true);
+		return pcb_true;
 	}
-	return (pcb_false);
+	return pcb_false;
 }
 
 /* ---------------------------------------------------------------------------
@@ -1106,9 +1108,9 @@ pcb_bool pcb_set_obj_join(int Type, void *Ptr1, void *Ptr2, void *Ptr3)
 	if (pcb_object_operation(&SetJoinFunctions, &ctx, Type, Ptr1, Ptr2, Ptr3) != NULL) {
 		pcb_draw();
 		pcb_undo_inc_serial();
-		return (pcb_true);
+		return pcb_true;
 	}
-	return (pcb_false);
+	return pcb_false;
 }
 
 /* ---------------------------------------------------------------------------
@@ -1124,9 +1126,9 @@ pcb_bool pcb_clr_obj_join(int Type, void *Ptr1, void *Ptr2, void *Ptr3)
 	if (pcb_object_operation(&ClrJoinFunctions, &ctx, Type, Ptr1, Ptr2, Ptr3) != NULL) {
 		pcb_draw();
 		pcb_undo_inc_serial();
-		return (pcb_true);
+		return pcb_true;
 	}
-	return (pcb_false);
+	return pcb_false;
 }
 
 /* ---------------------------------------------------------------------------
@@ -1142,9 +1144,9 @@ pcb_bool pcb_chg_obj_nonetlist(int Type, void *Ptr1, void *Ptr2, void *Ptr3)
 	if (pcb_object_operation(&ChangeNonetlistFunctions, &ctx, Type, Ptr1, Ptr2, Ptr3) != NULL) {
 		pcb_draw();
 		pcb_undo_inc_serial();
-		return (pcb_true);
+		return pcb_true;
 	}
-	return (pcb_false);
+	return pcb_false;
 }
 
 /* ---------------------------------------------------------------------------
@@ -1161,9 +1163,9 @@ pcb_bool pcb_chg_obj_square(int Type, void *Ptr1, void *Ptr2, void *Ptr3, int st
 	if (pcb_object_operation(&ChangeSquareFunctions, &ctx, Type, Ptr1, Ptr2, Ptr3) != NULL) {
 		pcb_draw();
 		pcb_undo_inc_serial();
-		return (pcb_true);
+		return pcb_true;
 	}
-	return (pcb_false);
+	return pcb_false;
 }
 
 /* ---------------------------------------------------------------------------
@@ -1179,9 +1181,9 @@ pcb_bool pcb_set_obj_square(int Type, void *Ptr1, void *Ptr2, void *Ptr3)
 	if (pcb_object_operation(&SetSquareFunctions, &ctx, Type, Ptr1, Ptr2, Ptr3) != NULL) {
 		pcb_draw();
 		pcb_undo_inc_serial();
-		return (pcb_true);
+		return pcb_true;
 	}
-	return (pcb_false);
+	return pcb_false;
 }
 
 /* ---------------------------------------------------------------------------
@@ -1197,9 +1199,9 @@ pcb_bool pcb_clr_obj_square(int Type, void *Ptr1, void *Ptr2, void *Ptr3)
 	if (pcb_object_operation(&ClrSquareFunctions, &ctx, Type, Ptr1, Ptr2, Ptr3) != NULL) {
 		pcb_draw();
 		pcb_undo_inc_serial();
-		return (pcb_true);
+		return pcb_true;
 	}
-	return (pcb_false);
+	return pcb_false;
 }
 
 /* ---------------------------------------------------------------------------
@@ -1215,9 +1217,9 @@ pcb_bool pcb_chg_obj_octagon(int Type, void *Ptr1, void *Ptr2, void *Ptr3)
 	if (pcb_object_operation(&ChangeOctagonFunctions, &ctx, Type, Ptr1, Ptr2, Ptr3) != NULL) {
 		pcb_draw();
 		pcb_undo_inc_serial();
-		return (pcb_true);
+		return pcb_true;
 	}
-	return (pcb_false);
+	return pcb_false;
 }
 
 /* ---------------------------------------------------------------------------
@@ -1233,9 +1235,9 @@ pcb_bool pcb_set_obj_octagon(int Type, void *Ptr1, void *Ptr2, void *Ptr3)
 	if (pcb_object_operation(&SetOctagonFunctions, &ctx, Type, Ptr1, Ptr2, Ptr3) != NULL) {
 		pcb_draw();
 		pcb_undo_inc_serial();
-		return (pcb_true);
+		return pcb_true;
 	}
-	return (pcb_false);
+	return pcb_false;
 }
 
 /* ---------------------------------------------------------------------------
@@ -1251,9 +1253,9 @@ pcb_bool pcb_clr_obj_octagon(int Type, void *Ptr1, void *Ptr2, void *Ptr3)
 	if (pcb_object_operation(&ClrOctagonFunctions, &ctx, Type, Ptr1, Ptr2, Ptr3) != NULL) {
 		pcb_draw();
 		pcb_undo_inc_serial();
-		return (pcb_true);
+		return pcb_true;
 	}
-	return (pcb_false);
+	return pcb_false;
 }
 
 /* ---------------------------------------------------------------------------
@@ -1339,9 +1341,9 @@ void *pcb_chg_obj_name_query(int Type, void *Ptr1, void *Ptr2, void *Ptr3, int p
 			pcb_undo_inc_serial();
 		}
 		pcb_draw();
-		return (Ptr3);
+		return Ptr3;
 	}
-	return (NULL);
+	return NULL;
 }
 
 void pcb_flag_change(pcb_board_t *pcb, pcb_change_flag_t how, pcb_flag_values_t flag, int Type, void *Ptr1, void *Ptr2, void *Ptr3)
