@@ -1,7 +1,8 @@
 /*
  *                            COPYRIGHT
  *
- *  PCB, interactive printed circuit board design
+ *  pcb-rnd, interactive printed circuit board design
+ *  (this file is based on PCB, interactive printed circuit board design)
  *  Copyright (C) 1994,1995,1996 Thomas Nau
  *  Copyright (C) 1997, 1998, 1999, 2000, 2001 Harry Eaton
  *
@@ -19,7 +20,13 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
- *  Contact addresses for paper mail and Email:
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
+ *
+ *
+ *  Old contact info:
  *  Harry Eaton, 6697 Buttonhole Ct, Columbia, MD 21044, USA
  *  haceaton@aplcomm.jhuapl.edu
  *
@@ -534,6 +541,10 @@ static int pcb_act_Change2ndSize(int argc, const char **argv, pcb_coord_t x, pcb
 			if (pcb_chg_selected_2nd_size(PCB_TYPE_PIN, value, absolute))
 				pcb_board_set_changed_flag(pcb_true);
 			break;
+		case F_SelectedPadstacks:
+			if (pcb_chg_selected_2nd_size(PCB_TYPE_PSTK, value, absolute))
+				pcb_board_set_changed_flag(pcb_true);
+			break;
 		case F_Selected:
 		case F_SelectedObjects:
 			if (pcb_chg_selected_2nd_size(PCB_TYPEMASK_PIN, value, absolute))
