@@ -18,6 +18,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  */
 
 #include <assert.h>
@@ -37,10 +41,13 @@
 #include "compat_fs.h"
 #include "compat_misc.h"
 #include "safe_fs.h"
+#include "build_run.h"
 
 /* conf list node's name */
 const char *conf_list_name = "pcb-rnd-conf-v1";
 static const char *conf_user_fn = "~/" DOT_PCB_RND "/pcb-conf.lht";
+static const char *flcat = "conf";
+
 
 lht_doc_t *conf_root[CFR_max_alloc];
 int conf_root_lock[CFR_max_alloc];
@@ -112,16 +119,25 @@ int conf_insert_tree_as(conf_role_t role, lht_node_t *root)
 int conf_load_as(conf_role_t role, const char *fn, int fn_is_text)
 {
 	lht_doc_t *d;
+	const char *ifn, *role_name = conf_role_name(role);
+
 	if (conf_root_lock[role])
 		return -1;
 	if (conf_root[role] != NULL) {
 		lht_dom_uninit(conf_root[role]);
 		conf_root[role] = NULL;
+		if (role_name != NULL)
+			pcb_file_loaded_del_at(flcat, role_name);
 	}
-	if (fn_is_text)
+	if (fn_is_text) {
 		d = pcb_hid_cfg_load_str(fn);
-	else
+		ifn = "<string>";
+	}
+	else {
 		d = pcb_hid_cfg_load_lht(fn);
+		ifn = fn;
+	}
+
 	if (d == NULL) {
 		FILE *f;
 		char *efn;
@@ -129,7 +145,6 @@ int conf_load_as(conf_role_t role, const char *fn, int fn_is_text)
 		f = pcb_fopen_fn(fn, "r", &efn);
 		if (f != NULL) { /* warn only if the file is there - missing file is normal */
 			pcb_message(PCB_MSG_ERROR, "error: failed to load lht config: %s (%s)\n", fn, efn);
-			free(efn);
 			fclose(f);
 		}
 		free(efn);
@@ -145,17 +160,25 @@ int conf_load_as(conf_role_t role, const char *fn, int fn_is_text)
 		confroot->doc = d;
 		d->root = prjroot;
 		conf_root[role] = d;
+		if (role_name != NULL)
+			pcb_file_loaded_set_at(flcat, role_name, ifn, "project/conf");
 		return 0;
 	}
 
 	if ((d->root->type == LHT_LIST) && (strcmp(d->root->name, "pcb-rnd-conf-v1") == 0)) {
 		conf_root[role] = d;
+		if (role_name != NULL)
+			pcb_file_loaded_set_at(flcat, role_name, ifn, "conf");
 		return 0;
 	}
 
 	if ((d->root->type == LHT_HASH) && (strcmp(d->root->name, "geda-project-v1") == 0)) {
 		lht_node_t *confroot;
 		confroot = lht_tree_path_(d, d->root, "pcb-rnd-conf-v1", 1, 0, NULL);
+
+		if (role_name != NULL)
+			pcb_file_loaded_set_at(flcat, role_name, ifn, "project/conf");
+
 		if ((confroot != NULL)  && (confroot->type == LHT_LIST) && (strcmp(confroot->name, "li:pcb-rnd-conf-v1") == 0)) {
 			conf_root[role] = d;
 			return 0;
@@ -867,7 +890,9 @@ void conf_update(const char *path, int arr_idx)
 	conf_rev++;
 }
 
-static lht_node_t *conf_lht_get_first_(lht_node_t *cwd, int create)
+#define POL_ANY 63
+
+static lht_node_t *conf_lht_get_first_(lht_node_t *cwd, conf_policy_t pol, int create)
 {
 	lht_node_t *ov; /* normally the "overwrite" node */
 
@@ -875,12 +900,22 @@ static lht_node_t *conf_lht_get_first_(lht_node_t *cwd, int create)
 	cwd = conf_lht_get_confroot(cwd);
 	if (cwd == NULL)
 		return NULL;
+
+
 	ov = cwd->data.list.first;
+	if (pol != POL_ANY) {
+		for(; ov != NULL; ov = ov->next)
+			if (conf_policy_parse(ov->name) == pol)
+				break;
+	}
+	else
+		pol = POL_OVERWRITE; /* this is what is created by default */
+
 	if (ov == NULL) {
 		lht_node_t *new_ov;
 		if (!create)
 			return NULL;
-		new_ov = lht_dom_node_alloc(LHT_HASH, "overwrite");
+		new_ov = lht_dom_node_alloc(LHT_HASH, conf_policy_name(pol));
 		lht_dom_list_append(cwd, new_ov);
 		ov = new_ov;
 	}
@@ -897,7 +932,17 @@ lht_node_t *conf_lht_get_first(conf_role_t target, int create)
 	assert(target < CFR_max_alloc);
 	if (conf_root[target] == NULL)
 		return NULL;
-	return conf_lht_get_first_(conf_root[target]->root, create);
+	return conf_lht_get_first_(conf_root[target]->root, POL_ANY, create);
+}
+
+lht_node_t *conf_lht_get_first_pol(conf_role_t target, conf_policy_t pol, int create)
+{
+	assert(target != CFR_invalid);
+	assert(target >= 0);
+	assert(target < CFR_max_alloc);
+	if (conf_root[target] == NULL)
+		return NULL;
+	return conf_lht_get_first_(conf_root[target]->root, pol, create);
 }
 
 static lht_node_t *conf_lht_get_at_(conf_role_t target, const char *conf_path, const char *lht_path, int create)
@@ -1119,7 +1164,7 @@ int conf_set_dry(conf_role_t target, const char *path_, int arr_idx, const char
 		return -1;
 	}
 
-	cwd = conf_lht_get_first(target, 0);
+	cwd = conf_lht_get_first_pol(target, pol, 0);
 	if (cwd == NULL) {
 		free(path);
 		return -1;
@@ -1380,6 +1425,7 @@ int conf_set_from_cli(const char *prefix, const char *arg_, const char *val, con
 	}
 
 	/* now that we have a clean path (arg) and a value, try to set the config */
+	conf_lht_get_first_pol(CFR_CLI, pol, 1); /* make sure the root for the given policy is created */
 	ret = conf_set(CFR_CLI, arg, -1, val, pol);
 	if (ret != 0)
 		*why = "invalid config path";
