@@ -18,6 +18,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  */
 
 #ifndef PCB_CONF_H
@@ -372,4 +376,8 @@ const char *conf_get_project_conf_name(const char *project_fn, const char *pcb_f
 int pcb_conf_cmd_is_safe_(const char *path, const char *value, const char **val_out, int msg);
 #define pcb_conf_cmd_is_safe(path, val_out, msg) pcb_conf_cmd_is_safe_(#path, conf_core.path, val_out, msg)
 
+/* Get the first subtree that matches pol within target; allocate new
+   subtree if needed */
+lht_node_t *conf_lht_get_first_pol(conf_role_t target, conf_policy_t pol, int create);
+
 #endif
