@@ -29,7 +29,7 @@ typedef struct {
 		CFT_BOOLEAN unique_names;          /* force unique names */
 		CFT_BOOLEAN snap_pin;              /* snap to pins and pads */
 		CFT_BOOLEAN snap_offgrid_line;     /* Snap to certain off-grid points along a line. */
-		CFT_BOOLEAN marker_snaps;          /* marker snaps to grid or snp points, as any other click */
+		CFT_BOOLEAN marker_snaps;          /* marker snaps to grid or snap points, as any other click */
 		CFT_BOOLEAN highlight_on_point;    /* Highlight if crosshair is on endpoints. */
 		CFT_BOOLEAN show_solder_side;      /* mirror output */
 		CFT_BOOLEAN save_last_command;     /* the command entry editline always starts with the last command entered by user in the current session */
@@ -54,10 +54,12 @@ typedef struct {
 		CFT_BOOLEAN local_ref;             /* use local reference for moves, by setting the mark at the beginning of each move. */
 		CFT_BOOLEAN check_planes;          /* when set, only polygons and their clearances are drawn, to see if polygons have isolated regions. */
 		CFT_BOOLEAN hide_names;            /* when set, element names are not drawn. */
-		CFT_BOOLEAN description;           /* display element description as element name, instead of value */
-		CFT_BOOLEAN name_on_pcb;           /* display Reference Designator as element name, instead of value */
+		CFT_BOOLEAN description;           /* obsolete - DO NOT USE - kept for compatibility */
+		CFT_BOOLEAN name_on_pcb;           /* obsolete - DO NOT USE - kept for compatibility */
+		CFT_STRING  subc_id;               /* subcircuit ID template for diplaying the subcircuit label on the subcircuit layer; default to displaying the refes, if empty; syntax if the same as for DYNTEXT */
 		CFT_BOOLEAN fullscreen;            /* hide widgets to make more room for the drawing */
 		CFT_BOOLEAN move_linepoint_uses_route;	/* Moving a line point calculates a new line route. This allows 45/90 line modes when editing lines. */
+		CFT_BOOLEAN auto_via;              /* when drawing traces and switching layers or when moving an object from one layer to another, try to keep connections by automatically inserting vias. */
 		CFT_REAL route_radius;             /* temporary: route draw helper's arc radius at corners (factor of the trace thickness) */
 
 		CFT_INTEGER click_time;            /* default time for click expiration, in ms */
@@ -214,6 +216,10 @@ typedef struct {
 			CFT_COORD text_offset_x;      /* X offset of text from pin center */
 			CFT_COORD text_offset_y;      /* Y offset of text from pin center */
 		} pinout;
+		struct padstack {
+			CFT_INTEGER cross_thick;      /* cross thickness in pixels - 0 means disable crosses */
+			CFT_COORD cross_size;         /* cross size in word coords - size of one arm of the cross (minus the hole radius) */
+		} padstack;
 		struct messages {
 			CFT_INTEGER char_per_line;   /* width of an output line in characters (used by separator drawing in find.c) */		
 		} messages;
