@@ -1,7 +1,8 @@
 /*
  *                            COPYRIGHT
  *
- *  PCB, interactive printed circuit board design
+ *  pcb-rnd, interactive printed circuit board design
+ *  (this file is based on PCB, interactive printed circuit board design)
  *  Copyright (C) 1994,1995,1996, 2003, 2004 Thomas Nau
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -18,9 +19,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- *  Contact addresses for paper mail and Email:
- *  Thomas Nau, Schlehenweg 15, 88471 Baustetten, Germany
- *  Thomas.Nau@rz.uni-ulm.de
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  *
  */
 
@@ -50,6 +52,7 @@
 #include "obj_rat_draw.h"
 #include "obj_poly_draw.h"
 #include "obj_text_draw.h"
+#include "obj_subc_parent.h"
 
 #undef NDEBUG
 #include <assert.h>
@@ -442,6 +445,10 @@ static void pcb_draw_pstk_marks(const pcb_box_t *drawn_area)
 static void pcb_draw_pstk_holes(pcb_layergrp_id_t group, const pcb_box_t *drawn_area, pcb_pstk_draw_hole_t holetype)
 {
 	pcb_pstk_draw_t ctx;
+
+	if (!PCB->hole_on)
+		return;
+
 	ctx.pcb = PCB;
 	ctx.gid = group;
 	ctx.holetype = holetype;
@@ -552,36 +559,26 @@ void pcb_draw_layer(pcb_layer_t *Layer, const pcb_box_t * screen)
 	else
 		pcb_draw_out.active_padGC = pcb_draw_out.backpadGC;
 
-	if (lflg & PCB_LYT_COPPER) {
 		/* print the non-clearing polys */
+	if (lflg & PCB_LYT_COPPER) {
 		pcb_r_search(Layer->polygon_tree, screen, NULL, pcb_poly_draw_term_callback, &info, NULL);
 	}
 	else {
-		/* print the non-clearing polys */
 		pcb_r_search(Layer->polygon_tree, screen, NULL, pcb_poly_draw_callback, &info, NULL);
 	}
 
 	if (conf_core.editor.check_planes)
 		goto out;
 
+	/* draw all visible layer objects (with terminal gfx on copper) */
 	if (lflg & PCB_LYT_COPPER) {
-		/* draw all visible lines this layer - with terminal gfx */
 		pcb_r_search(Layer->line_tree, screen, NULL, pcb_line_draw_term_callback, Layer, NULL);
-
-		/* draw the layer arcs on screen */
 		pcb_r_search(Layer->arc_tree, screen, NULL, pcb_arc_draw_term_callback, Layer, NULL);
-
-		/* draw the layer text on screen */
 		pcb_r_search(Layer->text_tree, screen, NULL, pcb_text_draw_term_callback, Layer, NULL);
 	}
 	else {
-		/* draw all visible lines this layer */
 		pcb_r_search(Layer->line_tree, screen, NULL, pcb_line_draw_callback, Layer, NULL);
-
-		/* draw the layer arcs on screen */
 		pcb_r_search(Layer->arc_tree, screen, NULL, pcb_arc_draw_callback, Layer, NULL);
-
-		/* draw the layer text on screen */
 		pcb_r_search(Layer->text_tree, screen, NULL, pcb_text_draw_callback, Layer, NULL);
 	}
 
@@ -599,6 +596,85 @@ void pcb_draw_layer(pcb_layer_t *Layer, const pcb_box_t * screen)
 		pcb_draw_out.active_padGC = NULL;
 }
 
+/* This version is about 1% slower and used rarely, thus it's all dupped
+   from pcb_draw_layer() to keep the original speed there */
+void pcb_draw_layer_under(pcb_layer_t *Layer, const pcb_box_t *screen, pcb_data_t *data)
+{
+	pcb_draw_info_t info;
+	pcb_box_t scr2;
+	unsigned int lflg = 0;
+	pcb_rtree_it_t it;
+	pcb_any_obj_t *o;
+
+	if ((screen->X2 <= screen->X1) || (screen->Y2 <= screen->Y1)) {
+		scr2 = *screen;
+		screen = &scr2;
+		if (scr2.X2 <= scr2.X1)
+			scr2.X2 = scr2.X1+1;
+		if (scr2.Y2 <= scr2.Y1)
+			scr2.Y2 = scr2.Y1+1;
+	}
+
+	info.drawn_area = screen;
+	info.layer = Layer;
+
+	lflg = pcb_layer_flags_(Layer);
+	if (PCB_LAYERFLG_ON_VISIBLE_SIDE(lflg))
+		pcb_draw_out.active_padGC = pcb_draw_out.padGC;
+	else
+		pcb_draw_out.active_padGC = pcb_draw_out.backpadGC;
+
+		/* print the non-clearing polys */
+	if (Layer->polygon_tree != NULL) {
+		if (lflg & PCB_LYT_COPPER) {
+			for(o = pcb_rtree_first(&it, Layer->polygon_tree, (pcb_rtree_box_t *)screen); o != NULL; o = pcb_rtree_next(&it))
+				if (pcb_obj_is_under(o, data))
+					pcb_poly_draw_term_callback((pcb_box_t *)o, &info);
+		}
+		else {
+			for(o = pcb_rtree_first(&it, Layer->polygon_tree, (pcb_rtree_box_t *)screen); o != NULL; o = pcb_rtree_next(&it))
+				if (pcb_obj_is_under(o, data))
+					pcb_poly_draw_callback((pcb_box_t *)o, &info);
+		}
+	}
+
+	if (conf_core.editor.check_planes)
+		goto out;
+
+	/* draw all visible layer objects (with terminal gfx on copper) */
+	if (lflg & PCB_LYT_COPPER) {
+		if (Layer->line_tree != NULL)
+			for(o = pcb_rtree_first(&it, Layer->line_tree, (pcb_rtree_box_t *)screen); o != NULL; o = pcb_rtree_next(&it))
+				if (pcb_obj_is_under(o, data))
+					pcb_line_draw_term_callback((pcb_box_t *)o, Layer);
+		if (Layer->arc_tree != NULL)
+			for(o = pcb_rtree_first(&it, Layer->arc_tree, (pcb_rtree_box_t *)screen); o != NULL; o = pcb_rtree_next(&it))
+				if (pcb_obj_is_under(o, data))
+					pcb_arc_draw_term_callback((pcb_box_t *)o, Layer);
+		if (Layer->text_tree != NULL)
+			for(o = pcb_rtree_first(&it, Layer->text_tree, (pcb_rtree_box_t *)screen); o != NULL; o = pcb_rtree_next(&it))
+				if (pcb_obj_is_under(o, data))
+					pcb_text_draw_term_callback((pcb_box_t *)o, Layer);
+	}
+	else {
+		if (Layer->line_tree != NULL)
+			for(o = pcb_rtree_first(&it, Layer->line_tree, (pcb_rtree_box_t *)screen); o != NULL; o = pcb_rtree_next(&it))
+				if (pcb_obj_is_under(o, data))
+					pcb_line_draw_callback((pcb_box_t *)o, Layer);
+		if (Layer->arc_tree != NULL)
+			for(o = pcb_rtree_first(&it, Layer->arc_tree, (pcb_rtree_box_t *)screen); o != NULL; o = pcb_rtree_next(&it))
+				if (pcb_obj_is_under(o, data))
+					pcb_arc_draw_callback((pcb_box_t *)o, Layer);
+		if (Layer->text_tree != NULL)
+			for(o = pcb_rtree_first(&it, Layer->text_tree, (pcb_rtree_box_t *)screen); o != NULL; o = pcb_rtree_next(&it))
+				if (pcb_obj_is_under(o, data))
+					pcb_text_draw_callback((pcb_box_t *)o, Layer);
+	}
+
+	out:;
+		pcb_draw_out.active_padGC = NULL;
+}
+
 /* ---------------------------------------------------------------------------
  * draws one layer group.  If the exporter is not a GUI,
  * also draws the pins / pads / vias in this layer group.
@@ -639,7 +715,14 @@ static void DrawLayerGroup(int group, const pcb_box_t *drawn_area, int is_curren
 
 void pcb_erase_obj(int type, void *lptr, void *ptr)
 {
+	if (pcb_hidden_floater((pcb_any_obj_t *)ptr))
+		return;
+
 	switch (type) {
+	case PCB_TYPE_PSTK:
+		pcb_pstk_invalidate_erase((pcb_pstk_t *) ptr);
+		break;
+
 	case PCB_TYPE_VIA:
 	case PCB_TYPE_PIN:
 		pcb_pin_invalidate_erase((pcb_pin_t *) ptr);
@@ -677,6 +760,9 @@ void pcb_erase_obj(int type, void *lptr, void *ptr)
 
 void pcb_draw_obj(pcb_any_obj_t *obj)
 {
+	if (pcb_hidden_floater(obj))
+		return;
+
 	switch (obj->type) {
 	case PCB_OBJ_VIA:
 		if (PCB->ViaOn)
@@ -735,6 +821,9 @@ void pcb_draw_obj(pcb_any_obj_t *obj)
 
 static void pcb_draw_obj_label(pcb_layergrp_id_t gid, pcb_any_obj_t *obj)
 {
+	if (pcb_hidden_floater(obj))
+		return;
+
 	/* do not show layer-object labels of the other side on non-pinout views */
 	if ((!pcb_draw_doing_pinout) && (obj->parent_type == PCB_PARENT_LAYER)) {
 		pcb_layer_t *ly = pcb_layer_get_real(obj->parent.layer);
@@ -794,6 +883,8 @@ static void expose_end(pcb_hid_t *old_gui)
 	pcb_gui->destroy_gc(pcb_draw_out.pmGC);
 	pcb_gui = old_gui;
 
+	pcb_draw_out.fgGC = NULL;
+
 	delayed_labels_enabled = pcb_false;
 	vtp0_truncate(&delayed_labels, 0);
 }
