@@ -1,7 +1,8 @@
 /*
  *                            COPYRIGHT
  *
- *  PCB, interactive printed circuit board design
+ *  pcb-rnd, interactive printed circuit board design
+ *  (this file is based on PCB, interactive printed circuit board design)
  *  Copyright (C) 1994,1995,1996, 2004 Thomas Nau
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -18,9 +19,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- *  Contact addresses for paper mail and Email:
- *  Thomas Nau, Schlehenweg 15, 88471 Baustetten, Germany
- *  Thomas.Nau@rz.uni-ulm.de
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  *
  */
 
@@ -78,6 +80,12 @@ void pcb_draw(void);
 void pcb_redraw(void);
 void pcb_draw_obj(pcb_any_obj_t *obj);
 void pcb_draw_layer(pcb_layer_t *, const pcb_box_t *);
+
+/* Same as pcb_draw_layer(), but never draws an implicit outline and ignores
+   objects that are not in the subtree of data - useful for drawing a subtree,
+   e.g. a subc only */
+void pcb_draw_layer_under(pcb_layer_t *Layer, const pcb_box_t *screen, pcb_data_t *data);
+
 void pcb_erase_obj(int, void *, void *);
 void pcb_draw_ppv(pcb_layergrp_id_t group, const pcb_box_t * drawn_area);
 void pcb_draw_ppv_names(pcb_layergrp_id_t group, const pcb_box_t * drawn_area);
