@@ -18,6 +18,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  */
 
 #ifndef PCB_EVENT_H
@@ -33,7 +37,8 @@ typedef enum {
 	PCB_EVENT_LOAD_PRE,               /* called before loading a new design */
 	PCB_EVENT_LOAD_POST,              /* called after loading a new design, whether it was successful or not */
 
-	PCB_EVENT_BOARD_CHANGED,          /* called after the board being edited got replaced (used to be the PCBChanged action) */
+	PCB_EVENT_BOARD_CHANGED,          /* called after the board being edited got _replaced_ (used to be the PCBChanged action) */
+	PCB_EVENT_BOARD_META_CHANGED,     /* called if the metadata of the board has changed */
 	PCB_EVENT_ROUTE_STYLES_CHANGED,   /* called after any route style change (used to be the RouteStylesChanged action) */
 	PCB_EVENT_NETLIST_CHANGED,        /* called after any netlist change (used to be the NetlistChanged action) */
 	PCB_EVENT_LAYERS_CHANGED,         /* called after layers or layer groups change (used to be the LayersChanged action) */
