@@ -1,7 +1,8 @@
 /*
  *                            COPYRIGHT
  *
- *  PCB, interactive printed circuit board design
+ *  pcb-rnd, interactive printed circuit board design
+ *  (this file is based on PCB, interactive printed circuit board design)
  *  Copyright (C) 1994,1995,1996 Thomas Nau
  *  Copyright (C) 1997, 1998, 1999, 2000, 2001 Harry Eaton
  *
@@ -19,7 +20,13 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
- *  Contact addresses for paper mail and Email:
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
+ *
+ *
+ *  Old contact info:
  *  Harry Eaton, 6697 Buttonhole Ct, Columbia, MD 21044, USA
  *  haceaton@aplcomm.jhuapl.edu
  *
@@ -121,7 +128,7 @@ static int pcb_act_LoadFrom(int argc, const char **argv, pcb_coord_t x, pcb_coor
 
 	if (pcb_strcasecmp(function, "ElementToBuffer") == 0) {
 		pcb_notify_crosshair_change(pcb_false);
-		if (pcb_element_load_to_buffer(PCB_PASTEBUFFER, name))
+		if (pcb_element_load_to_buffer(PCB_PASTEBUFFER, name, format))
 			pcb_crosshair_set_mode(PCB_MODE_PASTE_BUFFER);
 		pcb_notify_crosshair_change(pcb_true);
 	}
@@ -221,8 +228,8 @@ static int pcb_act_normalize(int argc, const char **argv, pcb_coord_t x, pcb_coo
 /* --------------------------------------------------------------------------- */
 
 static const char pcb_acts_SaveTo[] =
-	"SaveTo(Layout|LayoutAs,filename)\n"
-	"SaveTo(AllConnections|AllUnusedPins|ElementConnections,filename)\n" "SaveTo(PasteBuffer,filename)";
+	"SaveTo(Layout|LayoutAs,filename,[fmt])\n"
+	"SaveTo(AllConnections|AllUnusedPins|ElementConnections,filename)\n" "SaveTo(PasteBuffer,filename,[fmt])";
 
 static const char pcb_acth_SaveTo[] = "Saves data to a file.";
 
