@@ -1,7 +1,8 @@
 /*
  *                            COPYRIGHT
  *
- *  PCB, interactive printed circuit board design
+ *  pcb-rnd, interactive printed circuit board design
+ *  (this file is based on PCB, interactive printed circuit board design)
  *  Copyright (C) 1994,1995,1996 Thomas Nau
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -18,9 +19,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- *  Contact addresses for paper mail and Email:
- *  Thomas Nau, Schlehenweg 15, 88471 Baustetten, Germany
- *  Thomas.Nau@rz.uni-ulm.de
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  *
  */
 
@@ -52,7 +54,7 @@ typedef void (*pcb_find_callback_t)(int current_type, void *current_ptr, int fro
 extern pcb_find_callback_t pcb_find_callback;
 
 #define PCB_LOOKUP_FIRST	\
-	(PCB_TYPE_PIN | PCB_TYPE_PAD | PCB_TYPE_PSTK)
+	(PCB_TYPE_PIN | PCB_TYPE_PAD | PCB_TYPE_PSTK | PCB_TYPE_SUBC_PART)
 #define PCB_LOOKUP_MORE	\
 	(PCB_TYPE_VIA | PCB_TYPE_LINE | PCB_TYPE_RATLINE | PCB_TYPE_POLY | PCB_TYPE_ARC | PCB_TYPE_SUBC_PART)
 #define PCB_SILK_TYPE	\
@@ -85,6 +87,7 @@ int pcb_drc_all(void);
 pcb_bool pcb_is_line_in_poly(pcb_line_t *, pcb_poly_t *);
 pcb_bool pcb_is_arc_in_poly(pcb_arc_t *, pcb_poly_t *);
 pcb_bool pcb_is_pad_in_poly(pcb_pad_t *, pcb_poly_t *);
+pcb_bool pcb_intersect_obj_obj(pcb_any_obj_t *a, pcb_any_obj_t *b);
 
 pcb_cardinal_t pcb_lookup_conn_by_obj(void *ctx, pcb_any_obj_t *obj, pcb_bool AndDraw, pcb_cardinal_t (*cb)(void *ctx, pcb_any_obj_t *obj));
 
