@@ -2,7 +2,8 @@
  *
  *                            COPYRIGHT
  *
- *  PCB, interactive printed circuit board design
+ *  pcb-rnd, interactive printed circuit board design
+ *  (this file is based on PCB, interactive printed circuit board design)
  *  Copyright (C) 1994,1995,1996, 2005 Thomas Nau
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -19,14 +20,16 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
- *  Contact addresses for paper mail and Email:
- *  Thomas Nau, Schlehenweg 15, 88471 Baustetten, Germany
- *  Thomas.Nau@rz.uni-ulm.de
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  *
  */
 
 #include "drc.h"
 #include "compat_nls.h"
+#include "data_it.h"
 
 #include "obj_arc_draw.h"
 #include "obj_pad_draw.h"
@@ -332,6 +335,7 @@ doIsBad:
 }
 
 
+unsigned long pcb_obj_type2oldtype(pcb_objtype_t type);
 
 /*-----------------------------------------------------------------------------
  * Check for DRC violations
@@ -397,6 +401,25 @@ int pcb_drc_all(void)
 	}
 	PCB_END_LOOP;
 
+	/* search net (bloat) from subcircuit terminals */
+	PCB_SUBC_LOOP(PCB->Data);
+	{
+		pcb_any_obj_t *o;
+		pcb_data_it_t it;
+
+		for(o = pcb_data_first(&it, subc->data, PCB_OBJ_CLASS_REAL); o != NULL; o = pcb_data_next(&it)) {
+			if (o->term == NULL) /* only terminals can be starting point of DRC net checks */
+				continue;
+			if (o->parent_type == PCB_PARENT_LAYER) { /* for layer objects, care about the ones on copper layers only */
+				pcb_layer_type_t lyt = pcb_layer_flags_(o->parent.layer);
+				if (!(lyt & PCB_LYT_COPPER))
+					continue;
+			}
+			DRCFind(pcb_obj_type2oldtype(o->type), (void *)subc, (void *)o, (void *)o);
+		}
+	}
+	PCB_END_LOOP;
+
 	if (!IsBad)
 	PCB_VIA_LOOP(PCB->Data);
 	{
@@ -894,7 +917,7 @@ static pcb_bool DRCFind(int What, void *ptr1, void *ptr2, void *ptr3)
 			free(object_type_list);
 
 			if (!throw_drc_dialog())
-				return (pcb_true);
+				return pcb_true;
 			pcb_undo_inc_serial();
 			pcb_undo(pcb_true);
 		}
@@ -939,7 +962,7 @@ static pcb_bool DRCFind(int What, void *ptr1, void *ptr2, void *ptr3)
 		User = pcb_false;
 		drc = pcb_false;
 		if (!throw_drc_dialog())
-			return (pcb_true);
+			return pcb_true;
 		pcb_undo_inc_serial();
 		pcb_undo(pcb_true);
 		/* highlight the rest of the encroaching net so it's not reported again */
@@ -954,9 +977,9 @@ static pcb_bool DRCFind(int What, void *ptr1, void *ptr2, void *ptr3)
 	}
 	drc = pcb_false;
 	DumpList();
-	TheFlag = PCB_FLAG_FOUND | PCB_FLAG_SELECTED;
+	TheFlag = PCB_FLAG_FOUND | PCB_FLAG_SELECTED | PCB_FLAG_DRC;
 	pcb_reset_conns(pcb_false);
-	return (pcb_false);
+	return pcb_false;
 }
 
 /*----------------------------------------------------------------------------
