@@ -2,7 +2,8 @@
  *
  *                            COPYRIGHT
  *
- *  PCB, interactive printed circuit board design
+ *  pcb-rnd, interactive printed circuit board design
+ *  (this file is based on PCB, interactive printed circuit board design)
  *  Copyright (C) 1994,1995,1996, 2005 Thomas Nau
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -19,9 +20,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
- *  Contact addresses for paper mail and Email:
- *  Thomas Nau, Schlehenweg 15, 88471 Baustetten, Germany
- *  Thomas.Nau@rz.uni-ulm.de
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  *
  */
 
@@ -240,14 +242,14 @@ static pcb_bool IsRatPointOnLineSpec(pcb_point_t *Point, pcb_line_t *Line)
 	/* either end */
 	if ((Point->X == Line->Point1.X && Point->Y == Line->Point1.Y)
 			|| (Point->X == Line->Point2.X && Point->Y == Line->Point2.Y))
-		return (pcb_true);
+		return pcb_true;
 
 	/* middle point */
 	pcb_obj_center((pcb_any_obj_t *)Line, &cx, &cy);
 	if ((Point->X == cx) && (Point->Y == cy))
 		return pcb_true;
 
-	return (pcb_false);
+	return pcb_false;
 }
 
 /* ---------------------------------------------------------------------------
@@ -271,7 +273,7 @@ static pcb_bool IsRatPointOnArcSpec(pcb_point_t *Point, pcb_arc_t *arc)
 	if ((Point->X == cx) && (Point->Y == cy))
 		return pcb_true;
 
-	return (pcb_false);
+	return pcb_false;
 }
 
 /* ---------------------------------------------------------------------------
@@ -491,25 +493,25 @@ pcb_bool pcb_intersect_line_arc(pcb_line_t *Line, pcb_arc_t *Arc)
 	r2 = Radius * l - d;
 	/* projection doesn't even intersect circle when r2 < 0 */
 	if (r2 < 0)
-		return (pcb_false);
+		return pcb_false;
 	/* check the ends of the line in case the projected point */
 	/* of intersection is beyond the line end */
 	if (pcb_is_point_on_arc(Line->Point1.X, Line->Point1.Y, MAX(0.5 * Line->Thickness + Bloat, 0.0), Arc))
-		return (pcb_true);
+		return pcb_true;
 	if (pcb_is_point_on_arc(Line->Point2.X, Line->Point2.Y, MAX(0.5 * Line->Thickness + Bloat, 0.0), Arc))
-		return (pcb_true);
+		return pcb_true;
 	if (l == 0.0)
-		return (pcb_false);
+		return pcb_false;
 	r2 = sqrt(r2);
 	Radius = -(dx * dx1 + dy * dy1);
 	r = (Radius + r2) / l;
 	if (r >= 0 && r <= 1
 			&& pcb_is_point_on_arc(Line->Point1.X + r * dx, Line->Point1.Y + r * dy, MAX(0.5 * Line->Thickness + Bloat, 0.0) + 1, Arc))
-		return (pcb_true);
+		return pcb_true;
 	r = (Radius - r2) / l;
 	if (r >= 0 && r <= 1
 			&& pcb_is_point_on_arc(Line->Point1.X + r * dx, Line->Point1.Y + r * dy, MAX(0.5 * Line->Thickness + Bloat, 0.0) + 1, Arc))
-		return (pcb_true);
+		return pcb_true;
 
 	/* check arc end points */
 	pcb_arc_get_end(Arc, 0, &ex, &ey);
@@ -626,11 +628,15 @@ pcb_bool pcb_is_pad_in_poly(pcb_pad_t *pad, pcb_poly_t *polygon)
  */
 pcb_bool pcb_is_poly_in_poly(pcb_poly_t *P1, pcb_poly_t *P2)
 {
+	int pcp_cnt = 0;
+	pcb_coord_t pcp_gap;
+
 	if (!P1->Clipped || !P2->Clipped)
 		return pcb_false;
 	assert(P1->Clipped->contours);
 	assert(P2->Clipped->contours);
 
+
 	/* first check if both bounding boxes intersect. If not, return quickly */
 	if (P1->Clipped->contours->xmin - Bloat > P2->Clipped->contours->xmax ||
 			P1->Clipped->contours->xmax + Bloat < P2->Clipped->contours->xmin ||
@@ -638,6 +644,22 @@ pcb_bool pcb_is_poly_in_poly(pcb_poly_t *P1, pcb_poly_t *P2)
 			P1->Clipped->contours->ymax + Bloat < P2->Clipped->contours->ymin)
 		return pcb_false;
 
+	/* cheat: poly-clear-poly means we did generate the clearance; this
+	   shall happen only if there's exactly one poly that is clearing the other */
+	if (PCB_FLAG_TEST(PCB_FLAG_CLEARPOLYPOLY, P1)) {
+		pcp_cnt++;
+		pcp_gap = P1->Clearance / 2.0;
+	}
+	if (PCB_FLAG_TEST(PCB_FLAG_CLEARPOLYPOLY, P2)) {
+		pcp_cnt++;
+		pcp_gap = P2->Clearance / 2.0;
+	}
+	if (pcp_cnt == 1) {
+		if (pcp_gap >= Bloat)
+			return pcb_false;
+		return pcb_true;
+	}
+
 	/* first check un-bloated case */
 	if (pcb_poly_isects_poly(P1->Clipped, P2, pcb_false))
 		return pcb_true;
@@ -662,7 +684,7 @@ pcb_bool pcb_is_poly_in_poly(pcb_poly_t *P1, pcb_poly_t *P2)
 					line.Point2.Y = v->point[1];
 					pcb_line_bbox(&line);
 					if (pcb_is_line_in_poly(&line, P2))
-						return (pcb_true);
+						return pcb_true;
 					line.Point1.X = line.Point2.X;
 					line.Point1.Y = line.Point2.Y;
 				}
@@ -670,7 +692,7 @@ pcb_bool pcb_is_poly_in_poly(pcb_poly_t *P1, pcb_poly_t *P2)
 		}
 	}
 
-	return (pcb_false);
+	return pcb_false;
 }
 
 /* ---------------------------------------------------------------------------
@@ -778,8 +800,11 @@ pcb_bool pcb_intersect_line_pin(pcb_pin_t *PV, pcb_line_t *Line)
    boxes do touch */
 PCB_INLINE pcb_bool_t pcb_intersect_line_polyline(pcb_pline_t *pl, pcb_coord_t x1, pcb_coord_t y1, pcb_coord_t x2, pcb_coord_t y2, pcb_coord_t thick)
 {
-	pcb_coord_t ox, oy;
-	double dx, dy, h;
+	pcb_coord_t ox, oy, vx, vy;
+	double dx, dy, h, l;
+
+	thick += Bloat*2;
+	if (thick < 0) thick = 0;
 
 	/* single-point line - check only one circle*/
 	if ((x1 == x2) && (y1 == y2))
@@ -791,13 +816,16 @@ PCB_INLINE pcb_bool_t pcb_intersect_line_polyline(pcb_pline_t *pl, pcb_coord_t x
 
 	dx = x2 - x1;
 	dy = y2 - y1;
-	h = 0.5 * thick / sqrt(PCB_SQUARE(dx) + PCB_SQUARE(dy));
+	l = sqrt(PCB_SQUARE(dx) + PCB_SQUARE(dy));
+	h = 0.5 * thick / l;
 	ox = dy * h + 0.5 * SGN(dy);
 	oy = -(dx * h + 0.5 * SGN(dx));
+	vx = (double)dx / -l * ((double)Bloat/2.0);
+	vy = (double)dy / -l * ((double)Bloat/2.0);
 
 	/* long line - consider edge intersection */
-	if (pcb_pline_isect_line(pl, x1 + ox, y1 + oy, x2 + ox, y2 + oy)) return pcb_true;
-	if (pcb_pline_isect_line(pl, x1 - ox, y1 - oy, x2 - ox, y2 - oy)) return pcb_true;
+	if (pcb_pline_isect_line(pl, x1 + ox + vx, y1 + oy + vy, x2 + ox - vx, y2 + oy - vy, NULL, NULL)) return pcb_true;
+	if (pcb_pline_isect_line(pl, x1 - ox + vx, y1 - oy + vy, x2 - ox - vx, y2 - oy - vy, NULL, NULL)) return pcb_true;
 
 	/* A corner case is when the polyline is fully within the line. By now we
 	   are sure there's no contour intersection, so if any of the polyline points
@@ -814,7 +842,7 @@ PCB_INLINE pcb_bool_t pcb_intersect_line_polyline(pcb_pline_t *pl, pcb_coord_t x
 	}
 }
 
-#define shape_line_to_pcb_line(shape_line, pcb_line) \
+#define shape_line_to_pcb_line(ps, shape_line, pcb_line) \
 	do { \
 		pcb_line.Point1.X = shape_line.x1 + ps->x; \
 		pcb_line.Point1.Y = shape_line.y1 + ps->y; \
@@ -837,7 +865,7 @@ PCB_INLINE pcb_bool_t pcb_pstk_intersect_line(pcb_pstk_t *ps, pcb_line_t *line)
 		case PCB_PSSH_LINE:
 		{
 			pcb_line_t tmp;
-			shape_line_to_pcb_line(shape->data.line, tmp);
+			shape_line_to_pcb_line(ps, shape->data.line, tmp);
 			return pcb_intersect_line_line(line, &tmp);
 		}
 		case PCB_PSSH_CIRC:
@@ -883,7 +911,7 @@ PCB_INLINE pcb_bool_t pcb_pstk_intersect_arc(pcb_pstk_t *ps, pcb_arc_t *arc)
 		case PCB_PSSH_LINE:
 		{
 			pcb_line_t tmp;
-			shape_line_to_pcb_line(shape->data.line, tmp);
+			shape_line_to_pcb_line(ps, shape->data.line, tmp);
 			return pcb_intersect_line_arc(&tmp, arc);
 		}
 		case PCB_PSSH_CIRC:
@@ -893,6 +921,31 @@ PCB_INLINE pcb_bool_t pcb_pstk_intersect_arc(pcb_pstk_t *ps, pcb_arc_t *arc)
 	return pcb_false;
 }
 
+PCB_INLINE pcb_polyarea_t *pcb_pstk_shp_poly2area(pcb_pstk_t *ps, pcb_pstk_shape_t *shape)
+{
+	int n;
+	pcb_pline_t *pl;
+	pcb_vector_t v;
+	pcb_polyarea_t *shp = pcb_polyarea_create();
+
+	v[0] = shape->data.poly.x[0] + ps->x; v[1] = shape->data.poly.y[0] + ps->y;
+	pl = pcb_poly_contour_new(v);
+	for(n = 1; n < shape->data.poly.len; n++) {
+		v[0] = shape->data.poly.x[n] + ps->x; v[1] = shape->data.poly.y[n] + ps->y;
+		pcb_poly_vertex_include(pl->head.prev, pcb_poly_node_create(v));
+	}
+	pcb_poly_contour_pre(pl, 1);
+	pcb_polyarea_contour_include(shp, pl);
+
+	if (!pcb_poly_valid(shp)) {
+/*		pcb_polyarea_free(&shp); shp = pcb_polyarea_create();*/
+		pcb_poly_contour_inv(pl);
+		pcb_polyarea_contour_include(shp, pl);
+	}
+
+	return shp;
+}
+
 PCB_INLINE pcb_bool_t pcb_pstk_intersect_poly(pcb_pstk_t *ps, pcb_poly_t *poly)
 {
 	pcb_pstk_shape_t *shape = pcb_pstk_shape_at(PCB, ps, poly->parent.layer);
@@ -902,20 +955,8 @@ PCB_INLINE pcb_bool_t pcb_pstk_intersect_poly(pcb_pstk_t *ps, pcb_poly_t *poly)
 		case PCB_PSSH_POLY:
 		{
 			/* convert the shape poly to a new poly so that it can be intersected */
-			pcb_polyarea_t *shp = pcb_polyarea_create();
-			pcb_pline_t *pl;
-			pcb_vector_t v;
 			pcb_bool res;
-			int n;
-
-			v[0] = shape->data.poly.x[0] + ps->x; v[1] = shape->data.poly.y[0] + ps->y;
-			pl = pcb_poly_contour_new(v);
-			for(n = 1; n < shape->data.poly.len; n++) {
-				v[0] = shape->data.poly.x[n] + ps->x; v[1] = shape->data.poly.y[n] + ps->y;
-				pcb_poly_vertex_include(pl->head.prev, pcb_poly_node_create(v));
-			}
-			pcb_poly_contour_pre(pl, 1);
-			pcb_polyarea_contour_include(shp, pl);
+			pcb_polyarea_t *shp = pcb_pstk_shp_poly2area(ps, shape);
 			res = pcb_polyarea_touching(shp, poly->Clipped);
 			pcb_polyarea_free(&shp);
 			return res;
@@ -923,7 +964,7 @@ PCB_INLINE pcb_bool_t pcb_pstk_intersect_poly(pcb_pstk_t *ps, pcb_poly_t *poly)
 		case PCB_PSSH_LINE:
 		{
 			pcb_line_t tmp;
-			shape_line_to_pcb_line(shape->data.line, tmp);
+			shape_line_to_pcb_line(ps, shape->data.line, tmp);
 			pcb_line_bbox(&tmp);
 			return pcb_is_line_in_poly(&tmp, poly);
 		}
@@ -940,9 +981,113 @@ PCB_INLINE pcb_bool_t pcb_pstk_intersect_poly(pcb_pstk_t *ps, pcb_poly_t *poly)
 		}
 	}
 	return pcb_false;
-
 }
 
+PCB_INLINE pcb_bool_t pstk_shape_int_circ_poly(pcb_pstk_t *p, pcb_pstk_shape_t *sp, pcb_pstk_t *c, pcb_pstk_shape_t *sc)
+{
+	pcb_coord_t px = sc->data.circ.x + c->x - p->x;
+	pcb_coord_t py = sc->data.circ.y + c->y - p->y;
+	return pcb_intersect_line_polyline(sp->data.poly.pa->contours,  px, py, px, py, sc->data.circ.dia);
+}
+
+PCB_INLINE pcb_bool_t pstk_shape_int_circ_line(pcb_pstk_t *l, pcb_pstk_shape_t *sl, pcb_pstk_t *c, pcb_pstk_shape_t *sc)
+{
+	pcb_pad_t tmp;
+	tmp.Point1.X = sl->data.line.x1 + l->x;
+	tmp.Point1.Y = sl->data.line.y1 + l->y;
+	tmp.Point2.X = sl->data.line.x2 + l->x;
+	tmp.Point2.Y = sl->data.line.y2 + l->y;
+	tmp.Thickness = sl->data.line.thickness;
+	tmp.Flags = pcb_no_flags();
+	return pcb_is_point_in_pad(sc->data.circ.x + c->x, sc->data.circ.y + c->y, sc->data.circ.dia/2, &tmp);
+}
+
+PCB_INLINE pcb_bool_t pcb_pstk_shape_intersect(pcb_pstk_t *ps1, pcb_pstk_shape_t *shape1, pcb_pstk_t *ps2, pcb_pstk_shape_t *shape2)
+{
+	if ((shape1->shape == PCB_PSSH_POLY) && (shape1->data.poly.pa == NULL))
+		pcb_pstk_shape_update_pa(&shape1->data.poly);
+	if ((shape2->shape == PCB_PSSH_POLY) && (shape2->data.poly.pa == NULL))
+		pcb_pstk_shape_update_pa(&shape2->data.poly);
+
+	switch(shape1->shape) {
+		case PCB_PSSH_POLY:
+			switch(shape2->shape) {
+				case PCB_PSSH_POLY:
+				{
+					pcb_bool res;
+					pcb_polyarea_t *shp1 = pcb_pstk_shp_poly2area(ps1, shape1);
+					pcb_polyarea_t *shp2 = pcb_pstk_shp_poly2area(ps2, shape2);
+					res = pcb_polyarea_touching(shp1, shp2);
+					pcb_polyarea_free(&shp1);
+					pcb_polyarea_free(&shp2);
+					return res;
+				}
+				case PCB_PSSH_LINE:
+					return pcb_intersect_line_polyline(shape1->data.poly.pa->contours, shape2->data.line.x1 + ps2->x - ps1->x, shape2->data.line.y1 + ps2->y - ps1->y, shape2->data.line.x2 + ps2->x - ps1->x, shape2->data.line.y2 + ps2->y - ps1->y, shape2->data.line.thickness);
+				case PCB_PSSH_CIRC:
+					return pstk_shape_int_circ_poly(ps1, shape1, ps2, shape2);
+			}
+			break;
+
+		case PCB_PSSH_LINE:
+			switch(shape2->shape) {
+				case PCB_PSSH_POLY:
+					return pcb_intersect_line_polyline(shape2->data.poly.pa->contours, shape1->data.line.x1 + ps1->x - ps2->x, shape1->data.line.y1 + ps1->y - ps2->y, shape1->data.line.x2 + ps1->x - ps2->x, shape1->data.line.y2 + ps1->y - ps2->y, shape1->data.line.thickness);
+				case PCB_PSSH_LINE:
+				{
+					pcb_line_t tmp1, tmp2;
+					shape_line_to_pcb_line(ps1, shape1->data.line, tmp1);
+					shape_line_to_pcb_line(ps2, shape2->data.line, tmp2);
+					return pcb_intersect_line_line(&tmp1, &tmp2);
+				}
+				case PCB_PSSH_CIRC:
+					return pstk_shape_int_circ_line(ps1, shape1, ps2, shape2);
+			}
+			break;
+
+		case PCB_PSSH_CIRC:
+			switch(shape2->shape) {
+				case PCB_PSSH_POLY:
+					return pstk_shape_int_circ_poly(ps2, shape2, ps1, shape1);
+				case PCB_PSSH_LINE:
+					return pstk_shape_int_circ_line(ps2, shape2, ps1, shape1);
+				case PCB_PSSH_CIRC:
+					{
+						double cdist2 = pcb_distance2(ps1->x + shape1->data.circ.x, ps1->y + shape1->data.circ.y, ps2->x + shape2->data.circ.x, ps2->y + shape2->data.circ.y);
+						double dia = ((double)shape1->data.circ.dia + (double)shape2->data.circ.dia)/2.0;
+						return cdist2 <= dia*dia;
+					}
+			}
+			break;
+	}
+	return pcb_false;
+}
+
+PCB_INLINE pcb_bool_t pcb_pstk_intersect_pstk(pcb_pstk_t *ps1, pcb_pstk_t *ps2)
+{
+	pcb_layer_t *ly;
+	pcb_data_t *data = ps1->parent.data;
+	int n;
+
+	for(n = 0, ly = PCB->Data->Layer; n < PCB->Data->LayerN; n++,ly++) {
+		pcb_pstk_shape_t *shape1;
+		pcb_pstk_shape_t *shape2;
+		pcb_layer_type_t lyt = pcb_layer_flags_(ly);
+
+		if (!(lyt & PCB_LYT_COPPER)) /* consider only copper for connections */
+			continue;
+
+		shape1 = pcb_pstk_shape_at(PCB, ps1, ly);
+		if (shape1 == NULL) continue;
+		shape2 = pcb_pstk_shape_at(PCB, ps2, ly);
+		if (shape2 == NULL) continue;
+
+		if (pcb_pstk_shape_intersect(ps1, shape1, ps2, shape2)) return pcb_true;
+	}
+	return pcb_false;
+}
+
+
 PCB_INLINE pcb_bool_t pcb_pstk_intersect_rat(pcb_pstk_t *ps, pcb_rat_t *rat)
 {
 	return ((rat->Point1.X == ps->x) && (rat->Point1.Y == ps->y)) || ((rat->Point2.X == ps->x) && (rat->Point2.Y == ps->y));
