@@ -2,7 +2,8 @@
  *
  *                            COPYRIGHT
  *
- *  PCB, interactive printed circuit board design
+ *  pcb-rnd, interactive printed circuit board design
+ *  (this file is based on PCB, interactive printed circuit board design)
  *  Copyright (C) 1994,1995,1996, 2005 Thomas Nau
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -19,9 +20,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
- *  Contact addresses for paper mail and Email:
- *  Thomas Nau, Schlehenweg 15, 88471 Baustetten, Germany
- *  Thomas.Nau@rz.uni-ulm.de
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  *
  */
 
@@ -43,7 +45,7 @@ static pcb_bool ListsEmpty(pcb_bool AndRats)
 	for (i = 0; i < pcb_max_layer && empty; i++)
 		empty = empty && LineList[i].Location >= LineList[i].Number
 			&& ArcList[i].Location >= ArcList[i].Number && PolygonList[i].Location >= PolygonList[i].Number;
-	return (empty);
+	return empty;
 }
 
 static void reassign_no_drc_flags(void)
@@ -79,7 +81,7 @@ static pcb_bool DoIt(pcb_bool AndRats, pcb_bool AndDraw)
 	while (!newone && !ListsEmpty(AndRats));
 	if (AndDraw)
 		pcb_draw();
-	return (newone);
+	return newone;
 }
 
 /* ---------------------------------------------------------------------------
@@ -228,7 +230,7 @@ static pcb_bool ListStart(pcb_any_obj_t *obj)
 	default:
 		assert(!"unhandled object type: can't start a find list from this\n");
 	}
-	return (pcb_false);
+	return pcb_false;
 }
 
 
@@ -371,6 +373,8 @@ void pcb_rat_find_hook(pcb_any_obj_t *obj, pcb_bool undo, pcb_bool AndRats)
 	User = pcb_false;
 }
 
+static pcb_bool pcb_reset_found_subc(pcb_subc_t *subc, pcb_bool AndDraw);
+
 /* ---------------------------------------------------------------------------
  * resets all used flags of pins and vias
  */
@@ -433,8 +437,7 @@ static pcb_bool pcb_reset_found_pins_vias_pads_(pcb_data_t *data, pcb_bool AndDr
 
 	PCB_SUBC_LOOP(data);
 	{
-		pcb_flag_change(PCB, PCB_CHGFLG_CLEAR, PCB_FLAG_FOUND, PCB_TYPE_SUBC, subc, subc, subc);
-		if (pcb_reset_found_pins_vias_pads_(subc->data, AndDraw))
+		if (pcb_reset_found_subc(subc, AndDraw))
 			change = pcb_true;
 	}
 	PCB_END_LOOP;
@@ -511,8 +514,7 @@ static pcb_bool pcb_reset_found_lines_polys_(pcb_data_t *data, pcb_bool AndDraw)
 
 	PCB_SUBC_LOOP(data);
 	{
-		pcb_flag_change(PCB, PCB_CHGFLG_CLEAR, PCB_FLAG_FOUND, PCB_TYPE_SUBC, subc, subc, subc);
-		if (pcb_reset_found_lines_polys_(subc->data, AndDraw))
+		if (pcb_reset_found_subc(subc, AndDraw))
 			change = pcb_true;
 	}
 	PCB_END_LOOP;
@@ -520,6 +522,19 @@ static pcb_bool pcb_reset_found_lines_polys_(pcb_data_t *data, pcb_bool AndDraw)
 	return change;
 }
 
+static pcb_bool pcb_reset_found_subc(pcb_subc_t *subc, pcb_bool AndDraw)
+{
+	pcb_bool res = pcb_false;
+
+	if (pcb_reset_found_lines_polys_(subc->data, AndDraw))
+		res = pcb_true;
+
+	if (pcb_reset_found_pins_vias_pads_(subc->data, AndDraw))
+		res = pcb_true;
+
+	return res;
+}
+
 pcb_bool pcb_reset_found_lines_polys(pcb_bool AndDraw)
 {
 	pcb_bool change = pcb_reset_found_lines_polys_(PCB->Data, AndDraw);
