@@ -2,7 +2,8 @@
  *
  *                            COPYRIGHT
  *
- *  PCB, interactive printed circuit board design
+ *  pcb-rnd, interactive printed circuit board design
+ *  (this file is based on PCB, interactive printed circuit board design)
  *  Copyright (C) 1994,1995,1996, 2005 Thomas Nau
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -19,9 +20,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
- *  Contact addresses for paper mail and Email:
- *  Thomas Nau, Schlehenweg 15, 88471 Baustetten, Germany
- *  Thomas.Nau@rz.uni-ulm.de
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  *
  */
 
@@ -74,7 +76,7 @@ static pcb_bool PrintAndSelectUnusedPinsAndPadsOfElement(pcb_element_t *Element,
 
 				/* reset found objects for the next pin */
 				if (PrepareNextLoop(FP))
-					return (pcb_true);
+					return pcb_true;
 			}
 		}
 	}
@@ -116,7 +118,7 @@ static pcb_bool PrintAndSelectUnusedPinsAndPadsOfElement(pcb_element_t *Element,
 
 			/* reset found objects for the next pin */
 			if (PrepareNextLoop(FP))
-				return (pcb_true);
+				return pcb_true;
 		}
 	}
 	PCB_END_LOOP;
@@ -126,7 +128,7 @@ static pcb_bool PrintAndSelectUnusedPinsAndPadsOfElement(pcb_element_t *Element,
 		fputs("}\n\n", FP);
 		SEPARATE(FP);
 	}
-	return (pcb_false);
+	return pcb_false;
 }
 
 /* ---------------------------------------------------------------------------
@@ -154,7 +156,7 @@ static pcb_bool PrepareNextLoop(FILE * FP)
 	/* reset Padstacks */
 	PadstackList.Number = PadstackList.Location = 0;
 
-	return (pcb_false);
+	return pcb_false;
 }
 
 /* ---------------------------------------------------------------------------
@@ -184,7 +186,7 @@ static pcb_bool PrintElementConnections(pcb_element_t *Element, FILE * FP, pcb_b
 		PrintPadConnections(PCB_SOLDER_SIDE, FP, pcb_false);
 		fputs("\t}\n", FP);
 		if (PrepareNextLoop(FP))
-			return (pcb_true);
+			return pcb_true;
 	}
 	PCB_END_LOOP;
 
@@ -208,11 +210,11 @@ static pcb_bool PrintElementConnections(pcb_element_t *Element, FILE * FP, pcb_b
 		PrintPinConnections(FP, pcb_false);
 		fputs("\t}\n", FP);
 		if (PrepareNextLoop(FP))
-			return (pcb_true);
+			return pcb_true;
 	}
 	PCB_END_LOOP;
 	fputs("}\n\n", FP);
-	return (pcb_false);
+	return pcb_false;
 }
 
 /* ---------------------------------------------------------------------------
