@@ -1,7 +1,8 @@
 /*
  *                            COPYRIGHT
  *
- *  PCB, interactive printed circuit board design
+ *  pcb-rnd, interactive printed circuit board design
+ *  (this file is based on PCB, interactive printed circuit board design)
  *  Copyright (C) 1994,1995,1996, 2005 Thomas Nau
  *  pcb-rnd Copyright (C) 2017 Tibor 'Igor2' Palinkas
  *
@@ -19,9 +20,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- *  Contact addresses for paper mail and Email:
- *  Thomas Nau, Schlehenweg 15, 88471 Baustetten, Germany
- *  Thomas.Nau@rz.uni-ulm.de
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  *
  */
 
@@ -41,6 +43,7 @@
 #include "compat_nls.h"
 #include "compat_misc.h"
 #include "event.h"
+#include "build_run.h"
 
 #define STEP_SYMBOLLINE 10
 
@@ -83,6 +86,14 @@ static void pcb_font_load_internal(pcb_font_t *font)
 	pcb_font_set_info(font);
 }
 
+static int pcb_parse_font_default(pcb_font_t *ptr, const char *filename)
+{
+	int res = pcb_parse_font(ptr, filename);
+	if (res == 0)
+		pcb_file_loaded_set_at("font", "default", filename, "original default font");
+	return res;
+}
+
 /* parses a file with font information and installs it into the provided PCB
  * checks directories given as colon separated list by resource fontPath
  * if the fonts filename doesn't contain a directory component */
@@ -90,7 +101,7 @@ void pcb_font_create_default(pcb_board_t *pcb)
 {
 	int res = -1;
 	pcb_io_err_inhibit_inc();
-	conf_list_foreach_path_first(res, &conf_core.rc.default_font_file, pcb_parse_font(&pcb->fontkit.dflt, __path__));
+	conf_list_foreach_path_first(res, &conf_core.rc.default_font_file, pcb_parse_font_default(&pcb->fontkit.dflt, __path__));
 	pcb_io_err_inhibit_dec();
 
 	if (res != 0) {
@@ -99,6 +110,7 @@ void pcb_font_create_default(pcb_board_t *pcb)
 		s = conf_concat_strlist(&conf_core.rc.default_font_file, &buff, NULL, ':');
 		pcb_message(PCB_MSG_WARNING, _("Can't find font-symbol-file. Searched: '%s'; falling back to the embedded default font\n"), s);
 		pcb_font_load_internal(&pcb->fontkit.dflt);
+		pcb_file_loaded_set_at("font", "default", "<internal>", "original default font");
 		gds_uninit(&buff);
 	}
 }
@@ -218,7 +230,7 @@ pcb_line_t *pcb_font_new_line_in_sym(pcb_symbol_t *Symbol, pcb_coord_t X1, pcb_c
 	line->Point2.X = X2;
 	line->Point2.Y = Y2;
 	line->Thickness = Thickness;
-	return (line);
+	return line;
 }
 
 pcb_poly_t *pcb_font_new_poly_in_sym(pcb_symbol_t *Symbol, int num_points)
@@ -248,8 +260,17 @@ pcb_arc_t *pcb_font_new_arc_in_sym(pcb_symbol_t *Symbol, pcb_coord_t cx, pcb_coo
 
 static pcb_font_t *pcb_font_(pcb_board_t *pcb, pcb_font_id_t id, int fallback, int unlink)
 {
-	if (id <= 0)
-		return &pcb->fontkit.dflt;
+	if (id <= 0) {
+		do_default:;
+		if (unlink) {
+			pcb_font_t *f = malloc(sizeof(pcb_font_t));
+			memcpy(f, &pcb->fontkit.dflt, sizeof(pcb_font_t));
+			memset(&pcb->fontkit.dflt, 0, sizeof(pcb_font_t));
+			return f;
+		}
+		else
+			return &pcb->fontkit.dflt;
+	}
 
 	if (pcb->fontkit.hash_inited) {
 		pcb_font_t *f = htip_get(&pcb->fontkit.fonts, id);
@@ -261,7 +282,7 @@ static pcb_font_t *pcb_font_(pcb_board_t *pcb, pcb_font_id_t id, int fallback, i
 	}
 
 	if (fallback)
-		return &pcb->fontkit.dflt;
+		goto do_default;
 
 	return NULL;
 }
