@@ -11,7 +11,6 @@ action_entry(AllUnusedPins)
 action_entry(Arc)
 action_entry(Arrow)
 action_entry(Block)
-action_entry(Description)
 action_entry(Cancel)
 action_entry(Center)
 action_entry(Clear)
@@ -29,14 +28,15 @@ action_entry(DeleteRats)
 action_entry(Drag)
 action_entry(DrillReport)
 action_entry(Element)
-action_entry(ElementByName)
 action_entry(ElementConnections)
 action_entry(ElementToBuffer)
 action_entry(Escape)
+action_entry(Eq)
 action_entry(Find)
 action_entry(FlipElement)
 action_entry(FoundPins)
 action_entry(Grid)
+action_entry(Hash)
 action_entry(InsertPoint)
 action_entry(Layer)
 action_entry(Layout)
@@ -48,16 +48,11 @@ action_entry(Lock)
 action_entry(Mirror)
 action_entry(Move)
 action_entry(MoveSelected)
-action_entry(NameOnPCB)
 action_entry(Netlist)
-action_entry(NetByName)
 action_entry(None)
 action_entry(Notify)
 action_entry(Object)
-action_entry(ObjectByName)
 action_entry(PasteBuffer)
-action_entry(PadByName)
-action_entry(PinByName)
 action_entry(PinOrPadName)
 action_entry(Pinout)
 action_entry(Polygon)
@@ -85,13 +80,14 @@ action_entry(SelectedNames)
 action_entry(SelectedObjects)
 action_entry(SelectedPads)
 action_entry(SelectedPins)
+action_entry(SelectedPadstacks)
 action_entry(SelectedTexts)
 action_entry(SelectedVias)
 action_entry(SelectedRats)
 action_entry(Stroke)
 action_entry(Subc)
+action_entry(SubcID)
 action_entry(Text)
-action_entry(TextByName)
 action_entry(TextScale)
 action_entry(Thermal)
 action_entry(ToLayout)
@@ -123,8 +119,6 @@ action_entry(ToggleCheckPlanes)
 action_entry(ToggleUniqueNames)
 action_entry(ToggleStroke)
 action_entry(Via)
-action_entry(ViaByName)
-action_entry(Value)
 action_entry(ViaDrillingHole)
 action_entry(ViaSize)
 action_entry(Zoom)
