@@ -1,7 +1,8 @@
 /*
  *                            COPYRIGHT
  *
- *  PCB, interactive printed circuit board design
+ *  pcb-rnd, interactive printed circuit board design
+ *  (this file is based on PCB, interactive printed circuit board design)
  *  Copyright (C) 1994,1995,1996 Thomas Nau
  *  Copyright (C) 1997, 1998, 1999, 2000, 2001 Harry Eaton
  *
@@ -19,7 +20,13 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
- *  Contact addresses for paper mail and Email:
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
+ *
+ *
+ *  Old contact info:
  *  Harry Eaton, 6697 Buttonhole Ct, Columbia, MD 21044, USA
  *  haceaton@aplcomm.jhuapl.edu
  *
@@ -62,7 +69,8 @@
 /* --------------------------------------------------------------------------- */
 /* Toggle actions are kept for compatibility; new code should use the conf system instead */
 static const char pcb_acts_Display[] =
-	"Display(NameOnPCB|Description|Value|Grid|Redraw|Pinout|PinOrPadName)\n"
+	"Display(SubcID, template)\n"
+	"Display(Grid|Redraw|Pinout|PinOrPadName)\n"
 	"Display(CycleClip|CycleCrosshair|ToggleAllDirections|ToggleStartDirection)\n"
 	"Display(ToggleGrid|ToggleRubberBandMode|ToggleUniqueNames)\n"
 	"Display(ToggleName|ToggleClearLine|ToggleFullPoly|ToggleSnapPin)\n"
@@ -76,10 +84,10 @@ static const char pcb_acth_Display[] = "Several display-related actions.";
 
 @table @code
 
-@item NameOnPCB
+@item SubcID
 @item Description
 @item Value
-Specify whether all elements show their name, description, or value.
+Specify the subcircuit ID template to be printed on the subcircuit layer
 
 @item Redraw
 Redraw the whole board.
@@ -227,8 +235,20 @@ static int pcb_act_Display(int argc, const char **argv, pcb_coord_t childX, pcb_
 	function = PCB_ACTION_ARG(0);
 	str_dir = PCB_ACTION_ARG(1);
 
+	id = pcb_funchash_get(function, NULL);
+	if (id == F_SubcID) { /* change the displayed name of elements */
+		if (argc > 0)
+			conf_set(CFR_DESIGN, "editor/subc_id", -1, str_dir, POL_OVERWRITE);
+		else
+			conf_set(CFR_DESIGN, "editor/subc_id", -1, "", POL_OVERWRITE);
+
+		pcb_gui->invalidate_all(); /* doesn't change too often, isn't worth anything more complicated */
+		pcb_draw();
+		return 0;
+	}
+
 	if (function && (!str_dir || !*str_dir)) {
-		switch (id = pcb_funchash_get(function, NULL)) {
+		switch (id) {
 
 			/* redraw layout */
 		case F_ClearAndRedraw:
@@ -236,49 +256,6 @@ static int pcb_act_Display(int argc, const char **argv, pcb_coord_t childX, pcb_
 			pcb_redraw();
 			break;
 
-			/* change the displayed name of elements */
-		case F_Value:
-		case F_NameOnPCB:
-		case F_Description:
-			PCB_ELEMENT_LOOP(PCB->Data);
-			{
-				pcb_elem_name_invalidate_erase(element);
-			}
-			PCB_END_LOOP;
-			switch (id) {
-			case F_Value:
-				if (conf_core.editor.description || conf_core.editor.name_on_pcb) {
-					conf_set_editor(description, 0);
-					conf_set_editor(name_on_pcb, 0);
-				}
-				else
-					conf_set_editor(name_on_pcb, 0); /* need to write once so the event is triggered */
-				break;
-			case F_NameOnPCB:
-				if (conf_core.editor.description || !conf_core.editor.name_on_pcb) {
-					conf_set_editor(description, 0);
-					conf_set_editor(name_on_pcb, 1);
-				}
-				else
-					conf_set_editor(name_on_pcb, 1); /* need to write once so the event is triggered */
-				break;
-			case F_Description:
-				if (!conf_core.editor.description || conf_core.editor.name_on_pcb) {
-					conf_set_editor(description, 1);
-					conf_set_editor(name_on_pcb, 0);
-				}
-				else
-					conf_set_editor(name_on_pcb, 0); /* need to write once so the event is triggered */
-				break;
-			}
-			PCB_ELEMENT_LOOP(PCB->Data);
-			{
-				pcb_elem_name_invalidate_draw(element);
-			}
-			PCB_END_LOOP;
-			pcb_draw();
-			break;
-
 			/* toggle line-adjust flag */
 		case F_ToggleAllDirections:
 			conf_toggle_editor(all_direction_lines);
@@ -438,14 +415,14 @@ static int pcb_act_Display(int argc, const char **argv, pcb_coord_t childX, pcb_
 			/* display the pinout of an element */
 		case F_Pinout:
 			{
-				pcb_element_t *element;
+				pcb_subc_t *subc;
 				void *ptrtmp;
 				pcb_coord_t x, y;
 
 				pcb_gui->get_coords(_("Click on an element"), &x, &y);
-				if ((pcb_search_screen(x, y, PCB_TYPE_ELEMENT, &ptrtmp, &ptrtmp, &ptrtmp)) != PCB_TYPE_NONE) {
-					element = (pcb_element_t *) ptrtmp;
-					pcb_gui->show_item(element);
+				if ((pcb_search_screen(x, y, PCB_TYPE_SUBC, &ptrtmp, &ptrtmp, &ptrtmp)) != PCB_TYPE_NONE) {
+					subc = (pcb_subc_t *) ptrtmp;
+					pcb_gui->show_item(subc);
 				}
 				break;
 			}
@@ -1562,7 +1539,7 @@ static int pcb_act_EditLayer(int argc, const char **argv, pcb_coord_t x, pcb_coo
 		attr[1].default_val.int_value = ly->comb & PCB_LYC_SUB;
 		attr[2].default_val.int_value = ly->comb & PCB_LYC_AUTO;
 
-		ar = pcb_gui->attribute_dialog(attr,sizeof(attr)/sizeof(attr[0]), rv, "edit layer properies", "Edit the properties of a logical layer", NULL);
+		ar = pcb_gui->attribute_dialog(attr,sizeof(attr)/sizeof(attr[0]), rv, "Edit layer properties", "Edit the properties of a logical layer", NULL);
 
 		if (ar == 0) {
 			pcb_layer_combining_t comb = 0;
@@ -1679,7 +1656,7 @@ static int pcb_act_EditGroup(int argc, const char **argv, pcb_coord_t x, pcb_coo
 
 		attr[0].default_val.str_value = pcb_strdup(g->name);
 
-		ar = pcb_gui->attribute_dialog(attr,sizeof(attr)/sizeof(attr[0]), rv, "edit layer properies", "Edit the properties of a logical layer", NULL);
+		ar = pcb_gui->attribute_dialog(attr,sizeof(attr)/sizeof(attr[0]), rv, "Edit layer properties", "Edit the properties of a logical layer", NULL);
 
 		if (ar == 0) {
 			if (strcmp(g->name, attr[0].default_val.str_value) != 0) {
