@@ -3,6 +3,19 @@
 
 #include "hid.h"
 
+typedef struct pcb_hid_attr_trnd_s pcb_hid_attr_trnd_t;
+
+/* tree node */
+struct pcb_hid_attr_trnd_s {
+	const char *text;               /* displayed on the UI */
+	long uid;                       /* tree-uniq; for referencing/addressing; valid values are >= 0, -1 is the terminator */
+	void *user_data;                /* arbitrary, used by the caller */
+	int expand;                     /* node expanded (open) */
+	pcb_hid_attr_trnd_t *next;      /* next sibling; each level is a singly linked list terminated by NULL */
+	pcb_hid_attr_trnd_t *parent;
+	pcb_hid_attr_trnd_t *children;  /* fisrt children on the next level, or NULL */
+};
+
 /* Used for HID attributes (exporting and printing, mostly).
    HA_boolean uses int_value, HA_enum sets int_value to the index and
    str_value to the enumeration string.  PCB_HATT_LABEL just shows the
@@ -13,12 +26,14 @@ struct pcb_hid_attr_val_s {
 	double real_value;
 	pcb_coord_t coord_value;
 	void (*func)();
+	pcb_hid_attr_trnd_t *tree; /* the current val_s is the root; this points to the first child node on the top level */
 };
 
 typedef enum pcb_hatt_compflags_e {
-	PCB_HATF_FRAME   = 1,  /* box/table has a visible frame around it */
-	PCB_HATF_LABEL   = 2,  /* direct children print label */
-	PCB_HATF_SCROLL  = 4   /* box/table is scrollable */
+	PCB_HATF_FRAME       = 1,  /* box/table has a visible frame around it */
+	PCB_HATF_LABEL       = 2,  /* direct children print label */
+	PCB_HATF_SCROLL      = 4,  /* box/table is scrollable */
+	PCB_HATF_HIDE_TABLAB = 8   /* hide tab labes of a TABBED - the tab mechanism works, but tab names are not displayed and are not clickable */
 } pcb_hatt_compflags_t;
 
 typedef enum pcb_hids_e {
@@ -33,11 +48,13 @@ typedef enum pcb_hids_e {
 	PCB_HATT_UNIT,
 	PCB_HATT_COORD,
 	PCB_HATT_BUTTON,              /* push button; default value is the label */
+	PCB_HATT_TREE,                /* tree/list view */
 
 	/* groups (e.g. boxes) */
 	PCB_HATT_BEGIN_HBOX,          /* NOTE: PCB_HATT_IS_COMPOSITE() depends on it */
 	PCB_HATT_BEGIN_VBOX,
 	PCB_HATT_BEGIN_TABLE,         /* min_val is the number of columns */
+	PCB_HATT_BEGIN_TABBED,        /* tabbed view (e.g. notebook); ->enumerations stores the tab names and a NULL; default_val's integer value is the index of the current tab */
 	PCB_HATT_END          /* close one level of PCB_HATT_* */
 } pcb_hids_t;
 
@@ -65,7 +82,7 @@ struct pcb_hid_attribute_s {
 	   the default_val.  */
 	void *value;
 
-	/* Advanced API: may not be available in all HIDs; supported by hid_gtk */
+	/* dynamic API */
 	int changed; /* 0 for initial values, 1 on user change */
 	void (*change_cb)(void *hid_ctx, void *caller_data, pcb_hid_attribute_t *attr); /* called upon value change by the user */
 	void *user_data; /* ignored; the caller is free to use it */
