@@ -18,6 +18,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  */
 
 #include <stdlib.h>
@@ -33,6 +37,7 @@
 #include "paths.h"
 #include "safe_fs.h"
 #include "compat_misc.h"
+#include "build_run.h"
 
 char hid_cfg_error_shared[1024];
 
@@ -288,8 +293,10 @@ pcb_hid_cfg_t *pcb_hid_cfg_load(const char *fn, int exact_fn, const char *embedd
 				char *end = *p + strlen(*p);
 				sprintf(end, "pcb-menu-%s.lht", fn);
 				doc = pcb_hid_cfg_load_lht(*p);
-				if (doc != NULL)
+				if (doc != NULL) {
 					pcb_message(PCB_MSG_INFO, "Loaded menu file '%s'\n", *p);
+					pcb_file_loaded_set_at("menu", "HID main", *p, "main menu system");
+				}
 			}
 			free(*p);
 		}
@@ -298,8 +305,10 @@ pcb_hid_cfg_t *pcb_hid_cfg_load(const char *fn, int exact_fn, const char *embedd
 	else
 		doc = pcb_hid_cfg_load_lht(fn);
 
-	if (doc == NULL)
+	if (doc == NULL) {
 		doc = pcb_hid_cfg_load_str(embedded_fallback);
+		pcb_file_loaded_set_at("menu", "HID main", "<internal>", "main menu system");
+	}
 	if (doc == NULL)
 		return NULL;
 
