@@ -1,7 +1,8 @@
 /*
  *                            COPYRIGHT
  *
- *  PCB, interactive printed circuit board design
+ *  pcb-rnd, interactive printed circuit board design
+ *  (this file is based on PCB, interactive printed circuit board design)
  *  Copyright (C) 1994,1995,1996 Thomas Nau
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -18,9 +19,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- *  Contact addresses for paper mail and Email:
- *  Thomas Nau, Schlehenweg 15, 88471 Baustetten, Germany
- *  Thomas.Nau@rz.uni-ulm.de
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  *
  */
 
@@ -75,7 +77,7 @@ void *pcb_insert_point_in_object(int Type, void *Ptr1, void *Ptr2, pcb_cardinal_
 	ptr = pcb_object_operation(&InsertFunctions, &ctx, Type, Ptr1, Ptr2, Ptr3);
 	if (ptr != NULL)
 		pcb_undo_inc_serial();
-	return (ptr);
+	return ptr;
 }
 
 pcb_point_t *pcb_adjust_insert_point(void)
@@ -109,7 +111,7 @@ pcb_point_t *pcb_adjust_insert_point(void)
 	if (pcb_crosshair.X == line->Point1.X)
 		m1 = 2;											/* 2 signals infinite slope */
 	else {
-		m = (double) (pcb_crosshair.X - line->Point1.X) / (pcb_crosshair.Y - line->Point1.Y);
+		m = (double) (pcb_crosshair.Y - line->Point1.Y) / (pcb_crosshair.X - line->Point1.X);
 		m1 = 0;
 		if (m > PCB_TAN_30_DEGREE)
 			m1 = (m > PCB_TAN_60_DEGREE) ? 2 : 1;
@@ -119,7 +121,7 @@ pcb_point_t *pcb_adjust_insert_point(void)
 	if (pcb_crosshair.X == line->Point2.X)
 		m2 = 2;											/* 2 signals infinite slope */
 	else {
-		m = (double) (pcb_crosshair.X - line->Point1.X) / (pcb_crosshair.Y - line->Point1.Y);
+		m = (double) (pcb_crosshair.Y - line->Point2.Y) / (pcb_crosshair.X - line->Point2.X);
 		m2 = 0;
 		if (m > PCB_TAN_30_DEGREE)
 			m2 = (m > PCB_TAN_60_DEGREE) ? 2 : 1;
