@@ -1,7 +1,8 @@
 /*
  *                            COPYRIGHT
  *
- *  PCB, interactive printed circuit board design
+ *  pcb-rnd, interactive printed circuit board design
+ *  (this file is based on PCB, interactive printed circuit board design)
  *  Copyright (C) 1994,1995,1996,2004,2006 Thomas Nau
  *  Copyright (C) 2016, 2017 Tibor 'Igor2' Palinkas (pcb-rnd extensions)
  *
@@ -19,9 +20,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- *  Contact addresses for paper mail and Email:
- *  Thomas Nau, Schlehenweg 15, 88471 Baustetten, Germany
- *  Thomas.Nau@rz.uni-ulm.de
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  *
  */
 
@@ -37,6 +39,7 @@
 #include "layer_vis.h"
 #include "rtree.h"
 #include "obj_pstk_inlines.h"
+#include "list_common.h"
 
 pcb_virt_layer_t pcb_virt_layers[] = {
 	{"invisible",      PCB_LYT_VIRTUAL + 1, PCB_LYT_VIRTUAL | PCB_LYT_INVIS },
@@ -105,9 +108,54 @@ do { \
 	} \
 } while(0)
 
+void pcb_layer_free(pcb_layer_t *layer)
+{
+	if (!layer->is_bound)
+		pcb_attribute_free(&layer->meta.real.Attributes);
+	PCB_TEXT_LOOP(layer);
+	{
+		free(text->TextString);
+	}
+	PCB_END_LOOP;
+	PCB_LINE_LOOP(layer);
+	{
+		if (line->Number)
+			free(line->Number);
+	}
+	PCB_END_LOOP;
+
+	list_map0(&layer->Line, pcb_line_t, pcb_line_free);
+	list_map0(&layer->Arc,  pcb_arc_t,  pcb_arc_free);
+	list_map0(&layer->Text, pcb_text_t, pcb_text_free);
+	PCB_POLY_LOOP(layer);
+	{
+		pcb_poly_free_fields(polygon);
+	}
+	PCB_END_LOOP;
+	list_map0(&layer->Polygon, pcb_poly_t, pcb_poly_free);
+	if (!layer->is_bound) {
+		if (layer->line_tree)
+			pcb_r_destroy_tree(&layer->line_tree);
+		if (layer->arc_tree)
+			pcb_r_destroy_tree(&layer->arc_tree);
+		if (layer->text_tree)
+			pcb_r_destroy_tree(&layer->text_tree);
+		if (layer->polygon_tree)
+			pcb_r_destroy_tree(&layer->polygon_tree);
+	}
+	memset(layer, 0, sizeof(pcb_layer_t));
+}
+
 pcb_bool pcb_layer_is_pure_empty(pcb_layer_t *layer)
 {
-	/* normal case: a layer is empty if all lists are empty */
+	/* if any local list is non-empty, the layer is non-empty */
+	if (layer->Line.lst.length > 0) return pcb_false;
+	if (layer->Arc.lst.length > 0) return pcb_false;
+	if (layer->Polygon.lst.length > 0) return pcb_false;
+	if (layer->Text.lst.length > 0) return pcb_false;
+
+	/* if the layer is locally empty, it might be a board layer that has
+	   objects from subcircuits so also check the rtrees */
 	return
 		PCB_RTREE_EMPTY(layer->line_tree) && 
 		PCB_RTREE_EMPTY(layer->arc_tree) && 
@@ -115,6 +163,38 @@ pcb_bool pcb_layer_is_pure_empty(pcb_layer_t *layer)
 		PCB_RTREE_EMPTY(layer->text_tree);
 }
 
+static pcb_bool pcb_layer_is_empty_glob(pcb_board_t *pcb, pcb_data_t *data, pcb_layer_t *layer)
+{
+	/* if any padstack has a shape on this layer, it is not empty */
+	PCB_PADSTACK_LOOP(data);
+	{
+		if (pcb_pstk_shape_at(pcb, padstack, layer) != NULL)
+			return 0;
+	}
+	PCB_END_LOOP;
+
+	/* need to recurse to subc */
+	PCB_SUBC_LOOP(data);
+	{
+		pcb_layer_id_t n;
+		pcb_layer_t *sl;
+
+		for(sl = subc->data->Layer, n = 0; n < subc->data->LayerN; sl++,n++) {
+			if (sl->meta.bound.real == layer) {
+				if (!pcb_layer_is_empty_(pcb, sl))
+					return 0;
+				/* can't break here: multiple bound layers may point to the same real layer! */
+			}
+		}
+
+		if (!pcb_layer_is_empty_glob(pcb, subc->data, layer))
+			return 0;
+	}
+	PCB_END_LOOP;
+
+	return 1;
+}
+
 pcb_bool pcb_layer_is_empty_(pcb_board_t *pcb, pcb_layer_t *layer)
 {
 	pcb_layer_type_t flags;
@@ -146,13 +226,8 @@ pcb_bool pcb_layer_is_empty_(pcb_board_t *pcb, pcb_layer_t *layer)
 		PCB_ENDALL_LOOP;
 	}
 
-	/* if any padstack has a shape on this layer, it is not empty */
-	PCB_PADSTACK_LOOP(pcb->Data);
-	{
-		if (pcb_pstk_shape_at(pcb, padstack, layer) != NULL)
-			return 0;
-	}
-	PCB_END_LOOP;
+	if (!pcb_layer_is_empty_glob(pcb, pcb->Data, layer))
+		return 0;
 
 #warning TODO: check top silk and bottom silk for elements
 
@@ -445,7 +520,7 @@ static void layer_init(pcb_board_t *pcb, pcb_layer_t *lp, pcb_layer_id_t idx, pc
 
 int pcb_layer_move(pcb_board_t *pcb, pcb_layer_id_t old_index, pcb_layer_id_t new_index, pcb_layergrp_id_t new_in_grp)
 {
-	pcb_layer_id_t l;
+	pcb_layer_id_t l, at = -1;
 
 	/* sanity checks */
 	if (old_index < -1 || old_index >= pcb->Data->LayerN) {
@@ -468,8 +543,6 @@ int pcb_layer_move(pcb_board_t *pcb, pcb_layer_id_t old_index, pcb_layer_id_t ne
 		return 1;
 	}
 
-	pcb_undo_add_layer_move(old_index, new_index);
-	pcb_undo_inc_serial();
 
 	if (old_index == -1) { /* append new layer at the end of the logical layer list, put it in the current group */
 		pcb_layergrp_t *g;
@@ -504,6 +577,7 @@ int pcb_layer_move(pcb_board_t *pcb, pcb_layer_id_t old_index, pcb_layer_id_t ne
 		pcb_event(PCB_EVENT_LAYERS_CHANGED, NULL);
 		pcb_layervis_change_group_vis(new_lid, 1, 1);
 		pcb_event(PCB_EVENT_LAYERVIS_CHANGED, NULL);
+		at = new_lid;
 	}
 	else if (new_index == -1) { /* Delete the layer at old_index */
 		pcb_layergrp_id_t gid;
@@ -560,6 +634,13 @@ int pcb_layer_move(pcb_board_t *pcb, pcb_layer_id_t old_index, pcb_layer_id_t ne
 		   layer list is insignificant, thus we shouldn't try to change it. */
 	}
 
+	if (at >= 0) {
+		pcb_undo_add_layer_move(old_index, new_index, at);
+		pcb_undo_inc_serial();
+	}
+	else
+		pcb_message(PCB_MSG_WARNING, "this operation is not undoable.\n");
+
 	return 0;
 }
 
@@ -589,10 +670,10 @@ pcb_layer_t *pcb_get_layer(pcb_data_t *data, pcb_layer_id_t id)
 void pcb_layer_link_trees(pcb_layer_t *dst, pcb_layer_t *src)
 {
 	/* we can't link non-existing trees - make sure src does have the trees initialized */
-	if (src->line_tree == NULL) src->line_tree = pcb_r_create_tree(NULL, 0, 0);
-	if (src->arc_tree == NULL) src->arc_tree = pcb_r_create_tree(NULL, 0, 0);
-	if (src->text_tree == NULL) src->text_tree = pcb_r_create_tree(NULL, 0, 0);
-	if (src->polygon_tree == NULL) src->polygon_tree = pcb_r_create_tree(NULL, 0, 0);
+	if (src->line_tree == NULL) src->line_tree = pcb_r_create_tree();
+	if (src->arc_tree == NULL) src->arc_tree = pcb_r_create_tree();
+	if (src->text_tree == NULL) src->text_tree = pcb_r_create_tree();
+	if (src->polygon_tree == NULL) src->polygon_tree = pcb_r_create_tree();
 
 	dst->line_tree = src->line_tree;
 	dst->arc_tree = src->arc_tree;
@@ -715,6 +796,43 @@ pcb_layer_t *pcb_layer_new_bound(pcb_data_t *data, pcb_layer_type_t type, const
 	return lay;
 }
 
+unsigned int pcb_layer_hash_bound(pcb_layer_t *ly, pcb_bool smirror)
+{
+	unsigned int hash;
+	pcb_layer_type_t lyt;
+	int offs;
+
+	assert(ly->is_bound);
+
+	lyt = ly->meta.bound.type;
+	offs = ly->meta.bound.stack_offs;
+
+	if (smirror) {
+		lyt = pcb_layer_mirror_type(lyt);
+		offs = -offs;
+	}
+
+	hash  = (unsigned long)arclist_length(&ly->Arc);
+	hash ^= (unsigned long)linelist_length(&ly->Line);
+	hash ^= (unsigned long)textlist_length(&ly->Text);
+	hash ^= (unsigned long)polylist_length(&ly->Polygon);
+	hash ^= (unsigned long)ly->comb ^ (unsigned long)lyt;
+
+	if (ly->meta.bound.type & PCB_LYT_INTERN)
+		hash ^= (unsigned int)offs;
+
+	return hash;
+}
+
+pcb_layer_type_t pcb_layer_mirror_type(pcb_layer_type_t lyt)
+{
+	if (lyt & PCB_LYT_TOP)
+		return (lyt & ~PCB_LYT_TOP) | PCB_LYT_BOTTOM;
+	else if (lyt & PCB_LYT_BOTTOM)
+		return (lyt & ~PCB_LYT_BOTTOM) | PCB_LYT_TOP;
+	return lyt;
+}
+
 int pcb_layer_type_map(pcb_layer_type_t type, void *ctx, void (*cb)(void *ctx, pcb_layer_type_t bit, const char *name, int class, const char *class_name))
 {
 	const pcb_layer_type_name_t *n;
