@@ -1,7 +1,8 @@
 /*
  *                            COPYRIGHT
  *
- *  PCB, interactive printed circuit board design
+ *  pcb-rnd, interactive printed circuit board design
+ *  (this file is based on PCB, interactive printed circuit board design)
  *  Copyright (C) 1994,1995,1996,2006 Thomas Nau
  *  Copyright (C) 2016 Tibor 'Igor2' Palinkas (pcb-rnd extensions)
  *
@@ -19,9 +20,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- *  Contact addresses for paper mail and Email:
- *  Thomas Nau, Schlehenweg 15, 88471 Baustetten, Germany
- *  Thomas.Nau@rz.uni-ulm.de
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  *
  */
 
@@ -85,7 +87,7 @@ typedef enum {
 
 typedef enum { /* bitfield */
 	PCB_LYC_SUB = 1,        /* bit 0 is 0 for add or 1 for sub */
-	PCB_LYC_AUTO = 2        /* bit 1 is 0 for manual drawn layer or 1 for auto-generated (from pins, pads, elements) */
+	PCB_LYC_AUTO = 2        /* bit 1 is 0 for manual drawn layer or 1 for side-effect objects from padstacks; this is how the user controls on which composite mask/paste layer padstack side effects should appear */
 } pcb_layer_combining_t;
 
 
@@ -176,6 +178,9 @@ pcb_layer_id_t pcb_layer_get_top_silk();
 
 /************ NEW API - new code should use these **************/
 
+/* Free all metadata and objects of a layer; does not free(layer) */
+void pcb_layer_free(pcb_layer_t *layer);
+
 /* Return the layer pointer (or NULL on invalid or virtual layers) for an id */
 pcb_layer_t *pcb_get_layer(pcb_data_t *data, pcb_layer_id_t id);
 
@@ -261,6 +266,13 @@ pcb_layer_t *pcb_layer_resolve_binding(pcb_board_t *pcb, pcb_layer_t *src);
 /* Allocate a new bound layer within data, set it up, but do not do the binding */
 pcb_layer_t *pcb_layer_new_bound(pcb_data_t *data, pcb_layer_type_t type, const char *name);
 
+/* Calculate a hash of a bound layer (ingoring its name) */
+unsigned int pcb_layer_hash_bound(pcb_layer_t *ly, pcb_bool smirror);
+
+/* Calculate mirrored version of some (bound) layer properties */
+pcb_layer_type_t pcb_layer_mirror_type(pcb_layer_type_t lyt);
+
+
 /* Modify tree pointers in dst to point to src's; allocates trees for src if they are not yet allocated */
 void pcb_layer_link_trees(pcb_layer_t *dst, pcb_layer_t *src);
 
