@@ -1,7 +1,8 @@
 /*
  *                            COPYRIGHT
  *
- *  PCB, interactive printed circuit board design
+ *  pcb-rnd, interactive printed circuit board design
+ *  (this file is based on PCB, interactive printed circuit board design)
  *  Copyright (C) 1994,1995,1996,2004,2006 Thomas Nau
  *  Copyright (C) 2016 Tibor 'Igor2' Palinkas (pcb-rnd extensions)
  *
@@ -19,9 +20,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- *  Contact addresses for paper mail and Email:
- *  Thomas Nau, Schlehenweg 15, 88471 Baustetten, Germany
- *  Thomas.Nau@rz.uni-ulm.de
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  *
  */
 
@@ -40,7 +42,7 @@ do { \
 	pcb->LayerGroups.cache.copper_valid = 0; \
 	if (!inhibit_notify) { \
 		pcb_event(PCB_EVENT_LAYERS_CHANGED, NULL); \
-		if (pcb_gui != NULL) \
+		if ((pcb_gui != NULL) && (pcb_exporter == NULL)) \
 			pcb_gui->invalidate_all(); \
 	} \
 } while(0)
@@ -162,13 +164,6 @@ pcb_bool pcb_layergrp_is_empty(pcb_board_t *pcb, pcb_layergrp_id_t num)
 	if (g->type & PCB_LYT_MASK)
 		return pcb_false;
 
-	if (g->type & PCB_LYT_PASTE) {
-		if (g->type & PCB_LYT_TOP)
-			return pcb_layer_is_paste_auto_empty(pcb, PCB_COMPONENT_SIDE);
-		if (g->type & PCB_LYT_BOTTOM)
-			return pcb_layer_is_paste_auto_empty(pcb, PCB_SOLDER_SIDE);
-	}
-
 	/* copper layers are always non-empty if there are vias or pins because of the rings */
 	if (g->type & PCB_LYT_COPPER) {
 		if ((pcb->Data->via_tree != NULL) && (pcb->Data->via_tree->size > 0))
@@ -489,7 +484,7 @@ void pcb_layergrp_fix_turn_to_outline(pcb_layergrp_t *g)
 	g->type |= PCB_LYT_OUTLINE;
 	g->type &= ~PCB_LYT_COPPER;
 	free(g->name);
-	g->name = pcb_strdup("global outline");
+	g->name = pcb_strdup("global_outline");
 }
 
 
@@ -749,6 +744,21 @@ pcb_layergrp_id_t pcb_layergrp_step(pcb_board_t *pcb, pcb_layergrp_id_t gid, int
 	return -1;
 }
 
+void pcb_layergrp_create_missing_substrate(pcb_board_t *pcb)
+{
+	pcb_layergrp_id_t g;
+	for(g = 0; g < pcb->LayerGroups.len-2; g++) {
+		pcb_layergrp_t *g0 = &pcb->LayerGroups.grp[g], *g1 = &pcb->LayerGroups.grp[g+1];
+		if ((g < pcb->LayerGroups.len-3) && (g1->type & PCB_LYT_OUTLINE)) g1++;
+		if ((g0->type & PCB_LYT_COPPER) && (g1->type & PCB_LYT_COPPER)) {
+			pcb_layergrp_t *ng = pcb_layergrp_insert_after(pcb, g);
+			ng->type = PCB_LYT_INTERN | PCB_LYT_SUBSTRATE;
+			ng->name = pcb_strdup("implicit_subst");
+			ng->valid = 1;
+		}
+	}
+}
+
 int pcb_layer_create_all_for_recipe(pcb_board_t *pcb, pcb_layer_t *layer, int num_layer)
 {
 	int n, existing_intern = 0, want_intern = 0;
