@@ -1,7 +1,8 @@
 /*
  *                            COPYRIGHT
  *
- *  PCB, interactive printed circuit board design
+ *  pcb-rnd, interactive printed circuit board design
+ *  (this file is based on PCB, interactive printed circuit board design)
  *  Copyright (C) 1994,1995,1996,2004,2006 Thomas Nau
  *  Copyright (C) 2016 Tibor 'Igor2' Palinkas (pcb-rnd extensions)
  *
@@ -19,9 +20,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- *  Contact addresses for paper mail and Email:
- *  Thomas Nau, Schlehenweg 15, 88471 Baustetten, Germany
- *  Thomas.Nau@rz.uni-ulm.de
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  *
  */
 
@@ -125,7 +127,7 @@ int pcb_layervis_change_group_vis(pcb_layer_id_t Layer, int On, pcb_bool ChangeS
 	done:;
 	/* update control panel and exit */
 	pcb_event(PCB_EVENT_LAYERVIS_CHANGED, NULL);
-	return (changed);
+	return changed;
 }
 
 /* ---------------------------------------------------------------------------
@@ -149,6 +151,7 @@ void pcb_layervis_reset_stack(void)
 	PCB->SubcPartsOn = pcb_true;
 	PCB->RatOn = pcb_true;
 	PCB->padstack_mark_on = pcb_true;
+	PCB->hole_on = pcb_true;
 
 	/* Bring the top copper group to the front and make it active.  */
 	if (pcb_layer_list(PCB, PCB_LYT_TOP | PCB_LYT_COPPER, &comp, 1) > 0)
