@@ -1,7 +1,8 @@
 /*
  *                            COPYRIGHT
  *
- *  PCB, interactive printed circuit board design
+ *  pcb-rnd, interactive printed circuit board design
+ *  (this file is based on PCB, interactive printed circuit board design)
  *  Copyright (C) 1994,1995,1996, 2004 Thomas Nau
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -18,9 +19,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- *  Contact addresses for paper mail and Email:
- *  Thomas Nau, Schlehenweg 15, 88471 Baustetten, Germany
- *  Thomas.Nau@rz.uni-ulm.de
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  *
  */
 
@@ -271,6 +273,7 @@ void pcb_main_uninit(void)
 	conf_uninit();
 	pcb_io_uninit();
 	pcb_plugin_uninit();
+	pcb_file_loaded_uninit();
 }
 
 static int arg_match(const char *in, const char *shrt, const char *lng)
@@ -278,17 +281,19 @@ static int arg_match(const char *in, const char *shrt, const char *lng)
 	return ((shrt != NULL) && (strcmp(in, shrt) == 0)) || ((lng != NULL) && (strcmp(in, lng) == 0));
 }
 
+/* action table number of columns for a single action */
+const int PCB_ACTION_ARGS_WIDTH = 5;
 const char *pcb_action_args[] = {
-/*short, -long, action, help */
-	NULL, "-show-actions", "PrintActions()",     "Print all available actions (human readable) and exit",
-	NULL, "-dump-actions", "DumpActions()",      "Print all available actions (script readable) and exit",
-	NULL, "-dump-plugins", "DumpPlugins()",      "Print all available plugins (script readable) and exit",
-	NULL, "-dump-plugindirs", "DumpPluginDirs()","Print directories plugins might be loaded from and exit",
-	NULL, "-show-paths",   "PrintPaths()",       "Print all configured paths and exit",
-	NULL, "-dump-config",  "dumpconf(native,1)", "Print the config tree and exit",
-	"V",  "-version",      "PrintVersion()",     "Print version info and exit",
-	NULL, "-copyright",    "PrintCopyright()",   "Print copyright and exit",
-	NULL, NULL, NULL /* terminator */
+/*short, -long, action, help, hint-on-error */
+	NULL, "-show-actions",    "PrintActions()",     "Print all available actions (human readable) and exit",   NULL,
+	NULL, "-dump-actions",    "DumpActions()",      "Print all available actions (script readable) and exit",  NULL,
+	NULL, "-dump-plugins",    "DumpPlugins()",      "Print all available plugins (script readable) and exit",  NULL,
+	NULL, "-dump-plugindirs", "DumpPluginDirs()",   "Print directories plugins might be loaded from and exit", NULL,
+	NULL, "-show-paths",      "PrintPaths()",       "Print all configured paths and exit",                     NULL,
+	NULL, "-dump-config",     "dumpconf(native,1)", "Print the config tree and exit",                          "Config dump not available - make sure you have configured pcb-rnd with --buildin-diag",
+	"V",  "-version",         "PrintVersion()",     "Print version info and exit",                             NULL,
+	NULL, "-copyright",       "PrintCopyright()",   "Print copyright and exit",                                NULL,
+	NULL, NULL, NULL, NULL, NULL /* terminator */
 };
 
 void print_pup_err(pup_err_stack_t *entry, char *string)
@@ -308,7 +313,7 @@ int main(int argc, char *argv[])
 	int n, hid_argc = 0;
 	char *cmd, *arg, **hid_argv, **sp;
 	const char **cs;
-	const char *main_action = NULL;
+	const char *main_action = NULL, *main_action_hint = NULL;
 	char *command_line_pcb = NULL;
 	vtp0_t plugin_cli_conf;
 
@@ -332,6 +337,7 @@ int main(int argc, char *argv[])
 
 
 	/* Minimal conf setup before we do anything else */
+	pcb_file_loaded_init();
 	pcb_minuid_init();
 	conf_init();
 	conf_core_init();
@@ -374,10 +380,12 @@ int main(int argc, char *argv[])
 				goto next_arg;
 			}
 
-			for(cs = pcb_action_args; cs[2] != NULL; cs += 4) {
+			for(cs = pcb_action_args; cs[2] != NULL; cs += PCB_ACTION_ARGS_WIDTH) {
 				if (arg_match(cmd, cs[0], cs[1])) {
-					if (main_action == NULL)
+					if (main_action == NULL) {
 						main_action = cs[2];
+						main_action_hint = cs[4];
+					}
 					else
 						fprintf(stderr, "Warning: can't execute multiple command line actions, ignoring %s\n", argv[n]);
 					goto next_arg;
@@ -510,8 +518,10 @@ int main(int argc, char *argv[])
 	conf_update(NULL, -1);
 
 	if (main_action != NULL) {
-		pcb_hid_parse_command(main_action);
-		exit(0);
+		int res = pcb_hid_parse_command(main_action);
+		if ((res != 0) && (main_action_hint != NULL))
+			fprintf(stderr, "\nHint: %s\n", main_action_hint);
+		exit(res);
 	}
 
 	pcb_gui->parse_arguments(&hid_argc, &hid_argv);
@@ -602,5 +612,5 @@ int main(int argc, char *argv[])
 
 	pcb_main_uninit();
 
-	return (0);
+	return 0;
 }
