@@ -1,7 +1,8 @@
 /*
  *                            COPYRIGHT
  *
- *  PCB, interactive printed circuit board design
+ *  pcb-rnd, interactive printed circuit board design
+ *  (this file is based on PCB, interactive printed circuit board design)
  *  Copyright (C) 1994,1995,1996 Thomas Nau
  *  Copyright (C) 1997, 1998, 1999, 2000, 2001 Harry Eaton
  *
@@ -19,7 +20,13 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
- *  Contact addresses for paper mail and Email:
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
+ *
+ *
+ *  Old contact info:
  *  Harry Eaton, 6697 Buttonhole Ct, Columbia, MD 21044, USA
  *  haceaton@aplcomm.jhuapl.edu
  *
@@ -33,6 +40,8 @@
 #include "hid_init.h"
 #include "conf_core.h"
 #include "plugins.h"
+#include "build_run.h"
+#include "safe_fs.h"
 
 
 /* --------------------------------------------------------------------------- */
@@ -92,9 +101,10 @@ static int help0(void)
 }
 
 extern const char *pcb_action_args[];
+extern const int PCB_ACTION_ARGS_WIDTH;
 static int help_main(void) {
 	const char **cs;
-	for(cs = pcb_action_args; cs[2] != NULL; cs += 4) {
+	for(cs = pcb_action_args; cs[2] != NULL; cs += PCB_ACTION_ARGS_WIDTH) {
 		fprintf(stderr, "%s [", Progname);
 		if (cs[0] != NULL)
 			fprintf(stderr, "-%s", cs[0]);
@@ -177,7 +187,7 @@ static const char pcb_acth_PrintVersion[] = "Print version.";
 
 int pcb_act_PrintVersion(int argc, const char **argv, pcb_coord_t x, pcb_coord_t y)
 {
-	printf("pcb-rnd version %s\n", PCB_VERSION);
+	printf("pcb-rnd version %s rev %s\n", PCB_VERSION, PCB_REVISION);
 	return 0;
 }
 
@@ -194,8 +204,8 @@ int pcb_act_PrintCopyright(int argc, const char **argv, pcb_coord_t x, pcb_coord
 				 "    Copyright (C) 1994,1995,1996,1997 Thomas Nau\n"
 				 "    Copyright (C) 1998, 1999, 2000 Harry Eaton\n\n");
 	printf("                COPYRIGHT for %s (pcb-rnd) version %s:\n"
-				 "    pcb-rnd, a fork of PCB with random improvements\n"
-				 "    Copyright (C) 2013, 2014, 2015, 2016, 2017 Tibor 'Igor2' Palinkas\n\n", Progname, PCB_VERSION);
+				 "    pcb-rnd, originally fork of PCB \n"
+				 "    Copyright (C) 2013,2014,2015,2016,2017,2018 Tibor 'Igor2' Palinkas\n\n", Progname, PCB_VERSION);
 	printf("    This program is free software; you can redistribute it and/or modify\n"
 				 "    it under the terms of the GNU General Public License as published by\n"
 				 "    the Free Software Foundation; either version 2 of the License, or\n"
@@ -242,6 +252,29 @@ int pcb_act_PrintPaths(int argc, const char **argv, pcb_coord_t x, pcb_coord_t y
 	return 0;
 }
 
+/* --------------------------------------------------------------------------- */
+static const char pcb_acts_PrintFiles[] = "PrintFiles()";
+static const char pcb_acth_PrintFiles[] = "Print files currently loaded.";
+
+static void print_cat(pcb_file_loaded_t *cat)
+{
+	htsp_entry_t *e;
+	printf("%s\n", cat->name);
+	for (e = htsp_first(&cat->data.category.children); e; e = htsp_next(&cat->data.category.children, e)) {
+		pcb_file_loaded_t *file = e->value;
+		printf(" %s\t%s\t%s\n", file->name, file->data.file.path, file->data.file.desc);
+	}
+}
+
+int pcb_act_PrintFiles(int argc, const char **argv, pcb_coord_t x, pcb_coord_t y)
+{
+	htsp_entry_t *e;
+	printf("# Data files loaded\n");
+	for (e = htsp_first(&pcb_file_loaded); e; e = htsp_next(&pcb_file_loaded, e))
+		print_cat(e->value);
+	return 0;
+}
+
 /* --------------------------------------------------------------------------- */
 static const char pcb_acts_DumpPlugins[] = "DumpPlugins()";
 
@@ -282,6 +315,15 @@ int pcb_act_DumpPluginDirs(int argc, const char **argv, pcb_coord_t x, pcb_coord
 	return 0;
 }
 
+static const char pcb_acts_System[] = "System(shell_cmd)";
+static const char pcb_acth_System[] = "Run shell command";
+int pcb_act_System(int argc, const char **argv, pcb_coord_t x, pcb_coord_t y)
+{
+	if (argc < 1)
+		return 1;
+	return pcb_system(argv[0]);
+}
+
 /* --------------------------------------------------------------------------- */
 
 pcb_hid_action_t main_action_list[] = {
@@ -303,11 +345,17 @@ pcb_hid_action_t main_action_list[] = {
 	{"PrintPaths", 0, pcb_act_PrintPaths,
 	 pcb_acth_PrintPaths, pcb_acts_PrintPaths}
 	,
+	{"PrintFiles", 0, pcb_act_PrintFiles,
+	 pcb_acth_PrintFiles, pcb_acts_PrintFiles}
+	,
 	{"DumpPlugins", 0, pcb_act_DumpPlugins,
 	 pcb_acth_DumpPlugins, pcb_acts_DumpPlugins}
 	,
 	{"DumpPluginDirs", 0, pcb_act_DumpPluginDirs,
 	 pcb_acth_DumpPluginDirs, pcb_acts_DumpPluginDirs}
+	,
+	{"System", 0, pcb_act_System,
+	 pcb_acth_System, pcb_acts_System}
 };
 
 PCB_REGISTER_ACTIONS(main_action_list, NULL)
