@@ -1,7 +1,8 @@
 /*
  *                            COPYRIGHT
  *
- *  PCB, interactive printed circuit board design
+ *  pcb-rnd, interactive printed circuit board design
+ *  (this file is based on PCB, interactive printed circuit board design)
  *  Copyright (C) 1994,1995,1996, 2005 Thomas Nau
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -18,9 +19,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- *  Contact addresses for paper mail and Email:
- *  Thomas Nau, Schlehenweg 15, 88471 Baustetten, Germany
- *  Thomas.Nau@rz.uni-ulm.de
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  *
  */
 
@@ -41,6 +43,7 @@
 #include "compat_misc.h"
 #include "netlist.h"
 #include "event.h"
+#include "data.h"
 
 #define STEP_POINT 100
 
@@ -233,6 +236,38 @@ pcb_lib_menu_t *pcb_netlist_find_net4pad(pcb_board_t *pcb, const pcb_pad_t *pad)
 	return pcb_netlist_find_net4pin_any(pcb, e->Name[PCB_ELEMNAME_IDX_REFDES].TextString, pad->Number);
 }
 
+pcb_lib_menu_t *pcb_netlist_find_net4term(pcb_board_t *pcb, const pcb_any_obj_t *term)
+{
+	pcb_data_t *data;
+	pcb_subc_t *sc;
+	char pinname[256];
+	int len;
+
+	if (term->term == NULL)
+		return NULL;
+
+	if (term->parent_type == PCB_PARENT_LAYER)
+		data = term->parent.layer->parent;
+	else if (term->parent_type == PCB_PARENT_DATA)
+		data = term->parent.data;
+	else
+		return NULL;
+
+	if (data->parent_type != PCB_PARENT_SUBC)
+		return NULL;
+
+	sc = data->parent.subc;
+	if (sc->refdes == NULL)
+		return NULL;
+
+	len = pcb_snprintf(pinname, sizeof(pinname), "%s-%s", sc->refdes, term->term);
+	if (len >= sizeof(pinname))
+		return NULL;
+
+	return pcb_netlist_find_net4pinname(pcb, pinname);
+}
+
+
 pcb_cardinal_t pcb_netlist_net_idx(pcb_board_t *pcb, pcb_lib_menu_t *net)
 {
 	pcb_lib_menu_t *first = &pcb->NetlistLib[PCB_NETLIST_EDITED].Menu[0];
