@@ -18,6 +18,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  */
 
 /* object model - type+union that can represent any object */
@@ -25,6 +29,7 @@
 #ifndef PCB_OBJ_H
 #define PCB_OBJ_H
 
+#include <genlist/gendlist.h>
 #include "obj_common.h"
 #include "global_typedefs.h"
 
