@@ -1,7 +1,8 @@
 /*
  *                            COPYRIGHT
  *
- *  PCB, interactive printed circuit board design
+ *  pcb-rnd, interactive printed circuit board design
+ *  (this file is based on PCB, interactive printed circuit board design)
  *  Copyright (C) 1994,1995,1996 Thomas Nau
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -18,9 +19,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- *  Contact addresses for paper mail and Email:
- *  Thomas Nau, Schlehenweg 15, 88471 Baustetten, Germany
- *  Thomas.Nau@rz.uni-ulm.de
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  *
  */
 
@@ -43,6 +45,7 @@
 #include "obj_arc_op.h"
 
 #include "obj_subc_parent.h"
+#include "obj_hash.h"
 
 /* TODO: could be removed if draw.c could be split up */
 #include "draw.h"
@@ -179,7 +182,7 @@ void pcb_arc_set_angles(pcb_layer_t *Layer, pcb_arc_t *a, pcb_angle_t new_sa, pc
 	a->StartAngle = new_sa;
 	a->Delta = new_da;
 	pcb_arc_bbox(a);
-	pcb_r_insert_entry(Layer->arc_tree, (pcb_box_t *) a, 0);
+	pcb_r_insert_entry(Layer->arc_tree, (pcb_box_t *) a);
 	pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_ARC, Layer, a);
 }
 
@@ -192,7 +195,7 @@ void pcb_arc_set_radii(pcb_layer_t *Layer, pcb_arc_t *a, pcb_coord_t new_width,
 	a->Width = new_width;
 	a->Height = new_height;
 	pcb_arc_bbox(a);
-	pcb_r_insert_entry(Layer->arc_tree, (pcb_box_t *) a, 0);
+	pcb_r_insert_entry(Layer->arc_tree, (pcb_box_t *) a);
 	pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_ARC, Layer, a);
 }
 
@@ -206,12 +209,12 @@ pcb_arc_t *pcb_arc_new(pcb_layer_t *Layer, pcb_coord_t X1, pcb_coord_t Y1, pcb_c
 	{
 		if (arc->X == X1 && arc->Y == Y1 && arc->Width == width &&
 				pcb_normalize_angle(arc->StartAngle) == pcb_normalize_angle(sa) && arc->Delta == dir)
-			return (NULL);						/* prevent stacked arcs */
+			return NULL;						/* prevent stacked arcs */
 	}
 	PCB_END_LOOP;
 	Arc = pcb_arc_alloc(Layer);
 	if (!Arc)
-		return (Arc);
+		return Arc;
 
 	Arc->ID = pcb_create_ID_get();
 	Arc->Flags = Flags;
@@ -224,7 +227,7 @@ pcb_arc_t *pcb_arc_new(pcb_layer_t *Layer, pcb_coord_t X1, pcb_coord_t Y1, pcb_c
 	Arc->StartAngle = sa;
 	Arc->Delta = dir;
 	pcb_add_arc_on_layer(Layer, Arc);
-	return (Arc);
+	return Arc;
 }
 
 static pcb_arc_t *pcb_arc_copy_meta(pcb_arc_t *dst, pcb_arc_t *src)
@@ -255,8 +258,8 @@ void pcb_add_arc_on_layer(pcb_layer_t *Layer, pcb_arc_t *Arc)
 {
 	pcb_arc_bbox(Arc);
 	if (!Layer->arc_tree)
-		Layer->arc_tree = pcb_r_create_tree(NULL, 0, 0);
-	pcb_r_insert_entry(Layer->arc_tree, (pcb_box_t *) Arc, 0);
+		Layer->arc_tree = pcb_r_create_tree();
+	pcb_r_insert_entry(Layer->arc_tree, (pcb_box_t *) Arc);
 	Arc->type = PCB_OBJ_ARC;
 	PCB_SET_PARENT(Arc, layer, Layer);
 }
@@ -270,23 +273,38 @@ void pcb_arc_free(pcb_arc_t * data)
 }
 
 
-int pcb_arc_eq(const pcb_element_t *e1, const pcb_arc_t *a1, const pcb_element_t *e2, const pcb_arc_t *a2)
+int pcb_arc_eq(const pcb_host_trans_t *tr1, const pcb_arc_t *a1, const pcb_host_trans_t *tr2, const pcb_arc_t *a2)
 {
+	double sgn1 = tr1->on_bottom ? -1 : +1;
+	double sgn2 = tr2->on_bottom ? -1 : +1;
+
 	if (pcb_field_neq(a1, a2, Thickness) || pcb_field_neq(a1, a2, Clearance)) return 0;
 	if (pcb_field_neq(a1, a2, Width) || pcb_field_neq(a1, a2, Height)) return 0;
-	if (pcb_element_neq_offsx(e1, a1, e2, a2, X) || pcb_element_neq_offsy(e1, a1, e2, a2, Y)) return 0;
-	if (pcb_field_neq(a1, a2, StartAngle) || pcb_field_neq(a1, a2, Delta)) return 0;
+
+	if (!PCB_FLAG_TEST(PCB_FLAG_FLOATER, a1) && !PCB_FLAG_TEST(PCB_FLAG_FLOATER, a2)) {
+		if (pcb_neq_tr_coords(tr1, a1->X, a1->Y, tr2, a2->X, a2->Y)) return 0;
+		if (pcb_normalize_angle(pcb_round(a1->StartAngle + tr1->rot)) != pcb_normalize_angle(pcb_round(a2->StartAngle + tr2->rot))) return 0;
+		if (pcb_round(a1->Delta * sgn1) != pcb_round(a2->Delta * sgn2)) return 0;
+	}
 
 	return 1;
 }
 
-unsigned int pcb_arc_hash(const pcb_element_t *e, const pcb_arc_t *a)
+unsigned int pcb_arc_hash(const pcb_host_trans_t *tr, const pcb_arc_t *a)
 {
-	return 
+	unsigned int crd = 0;
+	double sgn = tr->on_bottom ? -1 : +1;
+
+	if (!PCB_FLAG_TEST(PCB_FLAG_FLOATER, a)) {
+		pcb_coord_t x, y;
+		pcb_hash_tr_coords(tr, &x, &y, a->X, a->Y);
+		crd = pcb_hash_coord(x) ^ pcb_hash_coord(y) ^
+			pcb_hash_coord(pcb_normalize_angle(pcb_round(a->StartAngle + tr->rot))) ^ pcb_hash_coord(pcb_round(a->Delta * sgn));
+	}
+
+	return
 		pcb_hash_coord(a->Thickness) ^ pcb_hash_coord(a->Clearance) ^
-		pcb_hash_coord(a->Width) ^ pcb_hash_coord(a->Height) ^
-		pcb_hash_element_ox(e, a->X) ^ pcb_hash_element_oy(e, a->Y) ^
-		pcb_hash_coord(a->StartAngle) ^ pcb_hash_coord(a->Delta);
+		pcb_hash_coord(a->Width) ^ pcb_hash_coord(a->Height) ^ crd;
 }
 
 pcb_coord_t pcb_arc_length(const pcb_arc_t *arc)
@@ -326,27 +344,31 @@ void *pcb_arcop_add_to_buffer(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_arc_t *A
 	return a;
 }
 
-/* moves an arc to buffer */
-void *pcb_arcop_move_to_buffer(pcb_opctx_t *ctx, pcb_layer_t * layer, pcb_arc_t * arc)
+/* moves an arc between board and buffer */
+void *pcb_arcop_move_buffer(pcb_opctx_t *ctx, pcb_layer_t *dstly, pcb_arc_t *arc)
 {
-	pcb_layer_t *lay = &ctx->buffer.dst->Layer[pcb_layer_id(ctx->buffer.src, layer)];
+	pcb_layer_t *srcly = arc->parent.layer;
 
-	pcb_poly_restore_to_poly(ctx->buffer.src, PCB_TYPE_ARC, layer, arc);
-	pcb_r_delete_entry(layer->arc_tree, (pcb_box_t *) arc);
+	assert(arc->parent_type == PCB_PARENT_LAYER);
+	if ((dstly == NULL) || (dstly == srcly)) /* auto layer in dst */
+		dstly = &ctx->buffer.dst->Layer[pcb_layer_id(ctx->buffer.src, srcly)];
+
+	pcb_poly_restore_to_poly(ctx->buffer.src, PCB_TYPE_ARC, srcly, arc);
+	pcb_r_delete_entry(srcly->arc_tree, (pcb_box_t *) arc);
 
 	arclist_remove(arc);
-	arclist_append(&lay->Arc, arc);
+	arclist_append(&dstly->Arc, arc);
 
 	PCB_FLAG_CLEAR(PCB_FLAG_FOUND, arc);
 
-	if (!lay->arc_tree)
-		lay->arc_tree = pcb_r_create_tree(NULL, 0, 0);
-	pcb_r_insert_entry(lay->arc_tree, (pcb_box_t *) arc, 0);
-	pcb_poly_clear_from_poly(ctx->buffer.dst, PCB_TYPE_ARC, lay, arc);
+	if (!dstly->arc_tree)
+		dstly->arc_tree = pcb_r_create_tree();
+	pcb_r_insert_entry(dstly->arc_tree, (pcb_box_t *) arc);
+	pcb_poly_clear_from_poly(ctx->buffer.dst, PCB_TYPE_ARC, dstly, arc);
 
-	PCB_SET_PARENT(arc, layer, lay);
+	PCB_SET_PARENT(arc, layer, dstly);
 
-	return (arc);
+	return arc;
 }
 
 /* changes the size of an arc */
@@ -355,7 +377,7 @@ void *pcb_arcop_change_size(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_arc_t *Arc
 	pcb_coord_t value = (ctx->chgsize.is_absolute) ? ctx->chgsize.value : Arc->Thickness + ctx->chgsize.value;
 
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, Arc))
-		return (NULL);
+		return NULL;
 	if (value <= PCB_MAX_LINESIZE && value >= PCB_MIN_LINESIZE && value != Arc->Thickness) {
 		pcb_undo_add_obj_to_size(PCB_TYPE_ARC, Layer, Arc, Arc);
 		pcb_arc_invalidate_erase(Arc);
@@ -363,12 +385,12 @@ void *pcb_arcop_change_size(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_arc_t *Arc
 		pcb_poly_restore_to_poly(PCB->Data, PCB_TYPE_ARC, Layer, Arc);
 		Arc->Thickness = value;
 		pcb_arc_bbox(Arc);
-		pcb_r_insert_entry(Layer->arc_tree, (pcb_box_t *) Arc, 0);
+		pcb_r_insert_entry(Layer->arc_tree, (pcb_box_t *) Arc);
 		pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_ARC, Layer, Arc);
 		pcb_arc_invalidate_draw(Layer, Arc);
-		return (Arc);
+		return Arc;
 	}
-	return (NULL);
+	return NULL;
 }
 
 /* changes the clearance size of an arc */
@@ -377,7 +399,7 @@ void *pcb_arcop_change_clear_size(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_arc_
 	pcb_coord_t value = (ctx->chgsize.is_absolute) ? ctx->chgsize.value : Arc->Clearance + ctx->chgsize.value;
 
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, Arc) || !PCB_FLAG_TEST(PCB_FLAG_CLEARLINE, Arc))
-		return (NULL);
+		return NULL;
 	if (value < 0)
 		value = 0;
 	value = MIN(PCB_MAX_LINESIZE, value);
@@ -392,12 +414,12 @@ void *pcb_arcop_change_clear_size(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_arc_
 		pcb_poly_restore_to_poly(PCB->Data, PCB_TYPE_ARC, Layer, Arc);
 		Arc->Clearance = value;
 		pcb_arc_bbox(Arc);
-		pcb_r_insert_entry(Layer->arc_tree, (pcb_box_t *) Arc, 0);
+		pcb_r_insert_entry(Layer->arc_tree, (pcb_box_t *) Arc);
 		pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_ARC, Layer, Arc);
 		pcb_arc_invalidate_draw(Layer, Arc);
-		return (Arc);
+		return Arc;
 	}
-	return (NULL);
+	return NULL;
 }
 
 /* changes the radius of an arc (is_primary 0=width or 1=height or 2=both) */
@@ -407,7 +429,7 @@ void *pcb_arcop_change_radius(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_arc_t *A
 	void *a0, *a1;
 
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, Arc))
-		return (NULL);
+		return NULL;
 
 	switch(ctx->chgsize.is_primary) {
 		case 0: dst = &Arc->Width; break;
@@ -429,12 +451,12 @@ void *pcb_arcop_change_radius(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_arc_t *A
 		pcb_poly_restore_to_poly(PCB->Data, PCB_TYPE_ARC, Layer, Arc);
 		*dst = value;
 		pcb_arc_bbox(Arc);
-		pcb_r_insert_entry(Layer->arc_tree, (pcb_box_t *) Arc, 0);
+		pcb_r_insert_entry(Layer->arc_tree, (pcb_box_t *) Arc);
 		pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_ARC, Layer, Arc);
 		pcb_arc_invalidate_draw(Layer, Arc);
-		return (Arc);
+		return Arc;
 	}
-	return (NULL);
+	return NULL;
 }
 
 /* changes the angle of an arc (is_primary 0=start or 1=end) */
@@ -444,7 +466,7 @@ void *pcb_arcop_change_angle(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_arc_t *Ar
 	void *a0, *a1;
 
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, Arc))
-		return (NULL);
+		return NULL;
 
 	switch(ctx->chgangle.is_primary) {
 		case 0: dst = &Arc->StartAngle; break;
@@ -458,7 +480,8 @@ void *pcb_arcop_change_angle(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_arc_t *Ar
 	}
 
 	value = (ctx->chgangle.is_absolute) ? ctx->chgangle.value : (*dst) + ctx->chgangle.value;
-	value = fmod(value, 360.0);
+	if ((value > 360.0) || (value < -360.0))
+		value = fmod(value, 360.0);
 
 	if (value != *dst) {
 		pcb_undo_add_obj_to_change_angles(PCB_TYPE_ARC, Layer, Arc, Arc);
@@ -467,19 +490,19 @@ void *pcb_arcop_change_angle(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_arc_t *Ar
 		pcb_poly_restore_to_poly(PCB->Data, PCB_TYPE_ARC, Layer, Arc);
 		*dst = value;
 		pcb_arc_bbox(Arc);
-		pcb_r_insert_entry(Layer->arc_tree, (pcb_box_t *) Arc, 0);
+		pcb_r_insert_entry(Layer->arc_tree, (pcb_box_t *) Arc);
 		pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_ARC, Layer, Arc);
 		pcb_arc_invalidate_draw(Layer, Arc);
-		return (Arc);
+		return Arc;
 	}
-	return (NULL);
+	return NULL;
 }
 
 /* changes the clearance flag of an arc */
 void *pcb_arcop_change_join(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_arc_t *Arc)
 {
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, Arc))
-		return (NULL);
+		return NULL;
 	pcb_arc_invalidate_erase(Arc);
 	if (PCB_FLAG_TEST(PCB_FLAG_CLEARLINE, Arc)) {
 		pcb_poly_restore_to_poly(PCB->Data, PCB_TYPE_ARC, Layer, Arc);
@@ -492,14 +515,14 @@ void *pcb_arcop_change_join(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_arc_t *Arc
 		pcb_undo_add_obj_to_clear_poly(PCB_TYPE_ARC, Layer, Arc, Arc, pcb_true);
 	}
 	pcb_arc_invalidate_draw(Layer, Arc);
-	return (Arc);
+	return Arc;
 }
 
 /* sets the clearance flag of an arc */
 void *pcb_arcop_set_join(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_arc_t *Arc)
 {
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, Arc) || PCB_FLAG_TEST(PCB_FLAG_CLEARLINE, Arc))
-		return (NULL);
+		return NULL;
 	return pcb_arcop_change_join(ctx, Layer, Arc);
 }
 
@@ -507,7 +530,7 @@ void *pcb_arcop_set_join(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_arc_t *Arc)
 void *pcb_arcop_clear_join(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_arc_t *Arc)
 {
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, Arc) || !PCB_FLAG_TEST(PCB_FLAG_CLEARLINE, Arc))
-		return (NULL);
+		return NULL;
 	return pcb_arcop_change_join(ctx, Layer, Arc);
 }
 
@@ -520,11 +543,11 @@ void *pcb_arcop_copy(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_arc_t *Arc)
 														Arc->Y + ctx->copy.DeltaY, Arc->Width, Arc->Height, Arc->StartAngle,
 														Arc->Delta, Arc->Thickness, Arc->Clearance, pcb_flag_mask(Arc->Flags, PCB_FLAG_FOUND));
 	if (!arc)
-		return (arc);
+		return arc;
 	pcb_arc_copy_meta(arc, Arc);
 	pcb_arc_invalidate_draw(Layer, arc);
 	pcb_undo_add_obj_to_create(PCB_TYPE_ARC, Layer, arc, arc);
-	return (arc);
+	return arc;
 }
 
 /* moves an arc */
@@ -534,7 +557,6 @@ void *pcb_arcop_move_noclip(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_arc_t *Arc
 		pcb_arc_invalidate_erase(Arc);
 		pcb_arc_move(Arc, ctx->move.dx, ctx->move.dy);
 		pcb_arc_invalidate_draw(Layer, Arc);
-		pcb_draw();
 	}
 	else {
 		pcb_arc_move(Arc, ctx->move.dx, ctx->move.dy);
@@ -547,7 +569,7 @@ void *pcb_arcop_move(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_arc_t *Arc)
 	pcb_r_delete_entry(Layer->arc_tree, (pcb_box_t *) Arc);
 	pcb_poly_restore_to_poly(PCB->Data, PCB_TYPE_ARC, Layer, Arc);
 	pcb_arcop_move_noclip(ctx, Layer, Arc);
-	pcb_r_insert_entry(Layer->arc_tree, (pcb_box_t *) Arc, 0);
+	pcb_r_insert_entry(Layer->arc_tree, (pcb_box_t *) Arc);
 	pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_ARC, Layer, Arc);
 	return Arc;
 }
@@ -559,7 +581,7 @@ void *pcb_arcop_clip(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_arc_t *Arc)
 		pcb_poly_restore_to_poly(PCB->Data, PCB_TYPE_ARC, Layer, Arc);
 	}
 	if (ctx->clip.clear) {
-		pcb_r_insert_entry(Layer->arc_tree, (pcb_box_t *) Arc, 0);
+		pcb_r_insert_entry(Layer->arc_tree, (pcb_box_t *) Arc);
 		pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_ARC, Layer, Arc);
 	}
 	return Arc;
@@ -574,8 +596,8 @@ void *pcb_arcop_move_to_layer_low(pcb_opctx_t *ctx, pcb_layer_t * Source, pcb_ar
 	arclist_append(&Destination->Arc, arc);
 
 	if (!Destination->arc_tree)
-		Destination->arc_tree = pcb_r_create_tree(NULL, 0, 0);
-	pcb_r_insert_entry(Destination->arc_tree, (pcb_box_t *) arc, 0);
+		Destination->arc_tree = pcb_r_create_tree();
+	pcb_r_insert_entry(Destination->arc_tree, (pcb_box_t *) arc);
 
 	PCB_SET_PARENT(arc, layer, Destination);
 
@@ -592,12 +614,10 @@ void *pcb_arcop_move_to_layer(pcb_opctx_t *ctx, pcb_layer_t * Layer, pcb_arc_t *
 		pcb_message(PCB_MSG_WARNING, _("Sorry, the object is locked\n"));
 		return NULL;
 	}
-	if (ctx->move.dst_layer == Layer && Layer->meta.real.vis) {
+	if (ctx->move.dst_layer == Layer && Layer->meta.real.vis)
 		pcb_arc_invalidate_draw(Layer, Arc);
-		pcb_draw();
-	}
 	if (ctx->move.dst_layer == Layer)
-		return (Arc);
+		return Arc;
 	pcb_undo_add_obj_to_move_to_layer(PCB_TYPE_ARC, Layer, Arc, Arc);
 	pcb_poly_restore_to_poly(PCB->Data, PCB_TYPE_ARC, Layer, Arc);
 	if (Layer->meta.real.vis)
@@ -606,8 +626,7 @@ void *pcb_arcop_move_to_layer(pcb_opctx_t *ctx, pcb_layer_t * Layer, pcb_arc_t *
 	pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_ARC, ctx->move.dst_layer, Arc);
 	if (ctx->move.dst_layer->meta.real.vis)
 		pcb_arc_invalidate_draw(ctx->move.dst_layer, newone);
-	pcb_draw();
-	return (newone);
+	return newone;
 }
 
 /* destroys an arc from a layer */
@@ -625,11 +644,8 @@ void *pcb_arcop_destroy(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_arc_t *Arc)
 void *pcb_arcop_remve(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_arc_t *Arc)
 {
 	/* erase from screen */
-	if (Layer->meta.real.vis) {
+	if (Layer->meta.real.vis)
 		pcb_arc_invalidate_erase(Arc);
-		if (!ctx->remove.bulk)
-			pcb_draw();
-	}
 	pcb_undo_move_obj_to_remove(PCB_TYPE_ARC, Layer, Arc, Arc);
 	return NULL;
 }
@@ -641,13 +657,15 @@ void *pcb_arcop_remove_point(pcb_opctx_t *ctx, pcb_layer_t *l, pcb_arc_t *a, int
 
 void *pcb_arc_destroy(pcb_layer_t *Layer, pcb_arc_t *Arc)
 {
+	void *res;
 	pcb_opctx_t ctx;
 
 	ctx.remove.pcb = PCB;
-	ctx.remove.bulk = pcb_false;
 	ctx.remove.destroy_target = NULL;
 
-	return pcb_arcop_remve(&ctx, Layer, Arc);
+	res = pcb_arcop_remve(&ctx, Layer, Arc);
+	pcb_draw();
+	return res;
 }
 
 /* rotates an arc */
@@ -675,7 +693,7 @@ void pcb_arc_rotate(pcb_layer_t *layer, pcb_arc_t *arc, pcb_coord_t X, pcb_coord
 	pcb_r_delete_entry(layer->arc_tree, (pcb_box_t *) arc);
 	pcb_rotate(&arc->X, &arc->Y, X, Y, cosa, sina);
 	arc->StartAngle = pcb_normalize_angle(arc->StartAngle + angle);
-	pcb_r_insert_entry(layer->arc_tree, (pcb_box_t *) arc, 0);
+	pcb_r_insert_entry(layer->arc_tree, (pcb_box_t *) arc);
 }
 
 void pcb_arc_mirror(pcb_layer_t *layer, pcb_arc_t *arc, pcb_coord_t y_offs)
@@ -688,7 +706,7 @@ void pcb_arc_mirror(pcb_layer_t *layer, pcb_arc_t *arc, pcb_coord_t y_offs)
 	arc->Delta = PCB_SWAP_DELTA(arc->Delta);
 	pcb_arc_bbox(arc);
 	if (layer->arc_tree != NULL)
-		pcb_r_insert_entry(layer->arc_tree, (pcb_box_t *) arc, 0);
+		pcb_r_insert_entry(layer->arc_tree, (pcb_box_t *) arc);
 }
 
 void pcb_arc_flip_side(pcb_layer_t *layer, pcb_arc_t *arc)
@@ -699,7 +717,7 @@ void pcb_arc_flip_side(pcb_layer_t *layer, pcb_arc_t *arc)
 	arc->StartAngle = PCB_SWAP_ANGLE(arc->StartAngle);
 	arc->Delta = PCB_SWAP_DELTA(arc->Delta);
 	pcb_arc_bbox(arc);
-	pcb_r_insert_entry(layer->arc_tree, (pcb_box_t *) arc, 0);
+	pcb_r_insert_entry(layer->arc_tree, (pcb_box_t *) arc);
 }
 
 /* rotates an arc */
@@ -709,11 +727,10 @@ void *pcb_arcop_rotate90(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_arc_t *Arc)
 	pcb_poly_restore_to_poly(PCB->Data, PCB_TYPE_ARC, Layer, Arc);
 	pcb_r_delete_entry(Layer->arc_tree, (pcb_box_t *) Arc);
 	pcb_arc_rotate90(Arc, ctx->rotate.center_x, ctx->rotate.center_y, ctx->rotate.number);
-	pcb_r_insert_entry(Layer->arc_tree, (pcb_box_t *) Arc, 0);
+	pcb_r_insert_entry(Layer->arc_tree, (pcb_box_t *) Arc);
 	pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_ARC, Layer, Arc);
 	pcb_arc_invalidate_draw(Layer, Arc);
-	pcb_draw();
-	return (Arc);
+	return Arc;
 }
 
 void *pcb_arcop_rotate(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_arc_t *Arc)
@@ -877,8 +894,12 @@ static void pcb_arc_draw(pcb_layer_t * layer, pcb_arc_t * arc, int allow_term_gf
 	if (PCB_FLAG_TEST(PCB_FLAG_WARN, arc))
 		color = conf_core.appearance.color.warn;
 	else if (PCB_FLAG_TEST(PCB_FLAG_SELECTED | PCB_FLAG_FOUND, arc)) {
-		if (PCB_FLAG_TEST(PCB_FLAG_SELECTED, arc))
-			color = layer->meta.real.selected_color;
+		if (PCB_FLAG_TEST(PCB_FLAG_SELECTED, arc)) {
+			if (layer->is_bound)
+				PCB_OBJ_COLOR_ON_BOUND_LAYER(color, layer, 1);
+			else
+				color = layer->meta.real.selected_color;
+		}
 		else
 			color = conf_core.appearance.color.connected;
 	}
@@ -886,7 +907,7 @@ static void pcb_arc_draw(pcb_layer_t * layer, pcb_arc_t * arc, int allow_term_gf
 		color = (arc->override_color);
 	}
 	else if (layer->is_bound)
-		PCB_OBJ_COLOR_ON_BOUND_LAYER(color, layer);
+		PCB_OBJ_COLOR_ON_BOUND_LAYER(color, layer, 0);
 	else
 		color = layer->meta.real.color;
 
@@ -903,6 +924,9 @@ pcb_r_dir_t pcb_arc_draw_callback(const pcb_box_t * b, void *cl)
 {
 	pcb_arc_t *arc = (pcb_arc_t *)b;
 
+	if (pcb_hidden_floater((pcb_any_obj_t*)b))
+		return PCB_R_DIR_FOUND_CONTINUE;
+
 	if (!PCB->SubcPartsOn && pcb_lobj_parent_subc(arc->parent_type, &arc->parent))
 		return PCB_R_DIR_NOT_FOUND;
 
