@@ -1,7 +1,8 @@
 /*
  *                            COPYRIGHT
  *
- *  PCB, interactive printed circuit board design
+ *  pcb-rnd, interactive printed circuit board design
+ *  (this file is based on PCB, interactive printed circuit board design)
  *  Copyright (C) 1994,1995,1996 Thomas Nau
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -18,9 +19,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- *  Contact addresses for paper mail and Email:
- *  Thomas Nau, Schlehenweg 15, 88471 Baustetten, Germany
- *  Thomas.Nau@rz.uni-ulm.de
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  *
  */
 
@@ -29,6 +31,7 @@
 #ifndef PCB_OBJ_ARC_H
 #define PCB_OBJ_ARC_H
 
+#include <genlist/gendlist.h>
 #include "obj_common.h"
 
 struct pcb_arc_s {       /* holds information about arcs */
@@ -63,9 +66,9 @@ void pcb_arc_flip_side(pcb_layer_t *layer, pcb_arc_t *arc);
 pcb_box_t pcb_arc_mini_bbox(const pcb_arc_t *arc);
 
 
-/*** hash ***/
-int pcb_arc_eq(const pcb_element_t *e1, const pcb_arc_t *a1, const pcb_element_t *e2, const pcb_arc_t *a2);
-unsigned int pcb_arc_hash(const pcb_element_t *e, const pcb_arc_t *a);
+/*** hash and eq ***/
+int pcb_arc_eq(const pcb_host_trans_t *tr1, const pcb_arc_t *a1, const pcb_host_trans_t *tr2, const pcb_arc_t *a2);
+unsigned int pcb_arc_hash(const pcb_host_trans_t *tr, const pcb_arc_t *a);
 
 
 /* Return the x;y coordinate of the endpoint of an arc; if which is 0, return
