@@ -1,7 +1,8 @@
 /*
  *                            COPYRIGHT
  *
- *  PCB, interactive printed circuit board design
+ *  pcb-rnd, interactive printed circuit board design
+ *  (this file is based on PCB, interactive printed circuit board design)
  *  Copyright (C) 1994,1995,1996 Thomas Nau
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -18,17 +19,17 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- *  Contact addresses for paper mail and Email:
- *  Thomas Nau, Schlehenweg 15, 88471 Baustetten, Germany
- *  Thomas.Nau@rz.uni-ulm.de
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  *
  */
 
 #ifndef PCB_OBJ_COMMON_H
 #define PCB_OBJ_COMMON_H
 
-#include <genht/hash.h>
-#include <genlist/gendlist.h>
+#include <math.h>
 #include <string.h>
 #include "flag.h"
 #include "attrib.h"
@@ -89,7 +90,16 @@ const char *pcb_obj_type_name(pcb_objtype_t type);
 pcb_flag_values_t pcb_obj_valid_flags(unsigned long int objtype);
 
 
-int GetObjectBoundingBox(int Type, void *Ptr1, void *Ptr2, void *Ptr3, pcb_box_t *res);
+int pcb_obj_get_bbox(int Type, void *Ptr1, void *Ptr2, void *Ptr3, pcb_box_t *res);
+
+/* Host transformations: typically the transformations an object of a subc
+   inherits from the subc */
+typedef struct pcb_host_trans_s {
+	pcb_coord_t ox, oy;
+	int on_bottom;
+	double rot;
+	double cosa, sina; /* rot angle cache */
+} pcb_host_trans_t;
 
 /* memset object to 0, but keep the link field */
 #define reset_obj_mem(type, obj) \
@@ -131,7 +141,7 @@ void pcb_obj_add_attribs(void *obj, const pcb_attribute_list_t *src);
 	const char           *term; \
 	void                 *ratconn; \
 	unsigned char        thermal; \
-	unsigned char        intconn; \
+	unsigned char        intconn, intnoconn; \
 	pcb_attribute_list_t Attributes; \
 	char                 override_color[sizeof("#XXXXXX")]
 
@@ -152,42 +162,39 @@ struct pcb_any_line_s {
 	PCB_ANYLINEFIELDS;
 };
 
-/*** Functions and macros used for hashing ***/
-
-/* compare two strings and return 0 if they are equal. NULL == NULL means equal. */
-PCB_INLINE int pcb_neqs(const char *s1, const char *s2)
-{
-	if ((s1 == NULL) && (s2 == NULL)) return 0;
-	if ((s1 == NULL) || (s2 == NULL)) return 1;
-	return strcmp(s1, s2) != 0;
-}
-
-PCB_INLINE unsigned pcb_hash_coord(pcb_coord_t c)
-{
-	return murmurhash(&(c), sizeof(pcb_coord_t));
-}
-
 /* Return the geometric center of an object, as shown (center of bbox usually,
    but not for an arc) */
 void pcb_obj_center(const pcb_any_obj_t *obj, pcb_coord_t *x, pcb_coord_t *y);
 
+/* Return the clearance value of object on the specified layer (in
+   case of padstack - in case of other objects layer is ignored) */
+pcb_coord_t pcb_obj_clearance_at(pcb_board_t *pcb, const pcb_any_obj_t *o, pcb_layer_t *at);
+
+
 /* Update cached attributes (->term) */
 void pcb_obj_attrib_post_change(pcb_attribute_list_t *list, const char *name, const char *value);
 
 /* Returns the first invalid character of an ID (terminal, refdes) or NULL */
 const char *pcb_obj_id_invalid(const char *id);
 
+#define pcb_hidden_floater(obj) (conf_core.editor.hide_names && PCB_FLAG_TEST(PCB_FLAG_FLOATER, (obj)))
+
+
 /* set const char *dst to a color, depending on the bound layer type:
    top silk and copper get the color of the first crresponding layer
-   from current PCB, the rest get the far-side color */
-#define PCB_OBJ_COLOR_ON_BOUND_LAYER(dst, layer) \
+   from current PCB, the rest get the far-side color;
+   set the selected color if sel is true */
+#define PCB_OBJ_COLOR_ON_BOUND_LAYER(dst, layer, sel) \
 do { \
 	if (layer->meta.bound.type & PCB_LYT_TOP) { \
 		pcb_layer_id_t lid = -1; \
 		pcb_layergrp_t *g; \
 		pcb_layergrp_id_t grp = -1; \
 		if (layer->meta.bound.type & PCB_LYT_SILK) { \
-			dst = conf_core.appearance.color.element; \
+			if (sel) \
+				dst = conf_core.appearance.color.element_selected; \
+			else \
+				dst = conf_core.appearance.color.element; \
 			break; \
 		} \
 		else if (layer->meta.bound.type & PCB_LYT_COPPER) \
@@ -196,27 +203,19 @@ do { \
 		if ((g != NULL) && (g->len > 0)) \
 			lid = g->lid[0]; \
 		if ((lid >= 0) && (lid <= PCB_MAX_LAYER)) { \
-			dst = conf_core.appearance.color.layer[lid]; \
+			if (sel) \
+				dst = conf_core.appearance.color.layer_selected[lid]; \
+			else \
+				dst = conf_core.appearance.color.layer[lid]; \
 			break; \
 		} \
 	} \
-	dst = conf_core.appearance.color.invisible_objects; \
+	if (sel) \
+		dst = conf_core.appearance.color.element_selected; \
+	else \
+		dst = conf_core.appearance.color.invisible_objects; \
 } while(0)
 
-/* compare two fields and return 0 if they are equal */
-#define pcb_field_neq(s1, s2, f) ((s1)->f != (s2)->f)
-
-/* hash relative x and y within an element */
-#define pcb_hash_element_ox(e, c) ((e) == NULL ? pcb_hash_coord(c) : pcb_hash_coord(c - e->MarkX))
-#define pcb_hash_element_oy(e, c) ((e) == NULL ? pcb_hash_coord(c) : pcb_hash_coord(c - e->MarkY))
-
-#define pcb_hash_str(s) ((s) == NULL ? 0 : strhash(s))
-
-#define pcb_element_offs(e,ef, s,sf) ((e == NULL) ? (s)->sf : ((s)->sf) - ((e)->ef))
-#define pcb_element_neq_offsx(e1, x1, e2, x2, f) (pcb_element_offs(e1, MarkX, x1, f) != pcb_element_offs(e2, MarkX, x2, f))
-#define pcb_element_neq_offsy(e1, y1, e2, y2, f) (pcb_element_offs(e1, MarkY, y1, f) != pcb_element_offs(e2, MarkY, y2, f))
-
-
 /* check if an object has clearance to polygon */
 #define PCB_POLY_HAS_CLEARANCE(ply) \
 	(PCB_FLAG_TEST(PCB_FLAG_CLEARPOLYPOLY, (ply)) && ((ply)->Clearance != 0))
