@@ -1,7 +1,8 @@
 /*
  *                            COPYRIGHT
  *
- *  PCB, interactive printed circuit board design
+ *  pcb-rnd, interactive printed circuit board design
+ *  (this file is based on PCB, interactive printed circuit board design)
  *  Copyright (C) 1994,1995,1996 Thomas Nau
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -18,9 +19,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- *  Contact addresses for paper mail and Email:
- *  Thomas Nau, Schlehenweg 15, 88471 Baustetten, Germany
- *  Thomas.Nau@rz.uni-ulm.de
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  *
  */
 
@@ -136,10 +138,10 @@ pcb_line_t *pcb_element_line_alloc(pcb_element_t *Element)
  * returns pcb_false on error
  * if successful, update some other stuff and reposition the pastebuffer
  */
-pcb_bool pcb_element_load_to_buffer(pcb_buffer_t *Buffer, const char *Name)
+pcb_bool pcb_element_load_to_buffer(pcb_buffer_t *Buffer, const char *Name, const char *fmt)
 {
 	pcb_buffer_clear(PCB, Buffer);
-	if (!pcb_parse_element(Buffer->Data, Name)) {
+	if (!pcb_parse_element(Buffer->Data, Name, fmt)) {
 		if (conf_core.editor.show_solder_side)
 			pcb_buffer_flip_side(PCB, Buffer);
 		pcb_set_buffer_bbox(Buffer);
@@ -157,12 +159,12 @@ pcb_bool pcb_element_load_to_buffer(pcb_buffer_t *Buffer, const char *Name)
 			pcb_subc_t *subc = pcb_subclist_first(&Buffer->Data->subc);
 			pcb_subc_get_origin(subc, &Buffer->X, &Buffer->Y);
 		}
-		return (pcb_true);
+		return pcb_true;
 	}
 
 	/* release memory which might have been acquired */
 	pcb_buffer_clear(PCB, Buffer);
-	return (pcb_false);
+	return pcb_false;
 }
 
 
@@ -170,7 +172,7 @@ pcb_bool pcb_element_load_to_buffer(pcb_buffer_t *Buffer, const char *Name)
    into the buffer. Returns zero on success, non-zero on error.  */
 int pcb_element_load_footprint_by_name(pcb_buffer_t *Buffer, const char *Footprint)
 {
-	return !pcb_element_load_to_buffer(Buffer, Footprint);
+	return !pcb_element_load_to_buffer(Buffer, Footprint, NULL);
 }
 
 
@@ -183,7 +185,7 @@ pcb_bool pcb_element_smash_buffer(pcb_buffer_t *Buffer)
 	char tmp[128];
 
 	if (elementlist_length(&Buffer->Data->Element) != 1)
-		return (pcb_false);
+		return pcb_false;
 
 	/*
 	 * At this point the buffer should contain just a single element.
@@ -273,7 +275,7 @@ pcb_bool pcb_element_smash_buffer(pcb_buffer_t *Buffer)
 	PCB_END_LOOP;
 	pcb_element_destroy(element);
 	pcb_element_free(element);
-	return (pcb_true);
+	return pcb_true;
 }
 
 /* see if a polygon is a rectangle.  If so, canonicalize it. */
@@ -323,7 +325,7 @@ pcb_bool pcb_element_convert_from_buffer(pcb_buffer_t *Buffer)
 														 NULL, NULL, NULL, PCB_PASTEBUFFER->X,
 														 PCB_PASTEBUFFER->Y, 0, 100, pcb_flag_make(PCB_SWAP_IDENT ? PCB_FLAG_ONSOLDER : PCB_FLAG_NO), pcb_false);
 	if (!Element)
-		return (pcb_false);
+		return pcb_false;
 	PCB_VIA_LOOP(Buffer->Data);
 	{
 		char num[8];
@@ -423,7 +425,7 @@ pcb_bool pcb_element_convert_from_buffer(pcb_buffer_t *Buffer)
 	if (!hasParts) {
 		pcb_destroy_object(PCB->Data, PCB_TYPE_ELEMENT, Element, Element, Element);
 		pcb_message(PCB_MSG_ERROR, _("There was nothing to convert!\n" "Elements must have some silk, pads or pins.\n"));
-		return (pcb_false);
+		return pcb_false;
 	}
 	if (crooked)
 		pcb_message(PCB_MSG_ERROR, _("There were polygons that can't be made into pins!\n" "So they were not included in the element\n"));
@@ -435,7 +437,7 @@ pcb_bool pcb_element_convert_from_buffer(pcb_buffer_t *Buffer)
 	pcb_buffer_clear(PCB, Buffer);
 	pcb_move_obj_to_buffer(PCB, Buffer->Data, PCB->Data, PCB_TYPE_ELEMENT, Element, Element, Element);
 	pcb_set_buffer_bbox(Buffer);
-	return (pcb_true);
+	return pcb_true;
 }
 
 void pcb_element_rotate(pcb_data_t *Data, pcb_element_t *Element, pcb_coord_t X, pcb_coord_t Y, double cosa, double sina, pcb_angle_t angle)
@@ -489,12 +491,12 @@ void pcb_element_rotate(pcb_data_t *Data, pcb_element_t *Element, pcb_coord_t X,
 pcb_bool pcb_element_change_side(pcb_element_t *Element, pcb_coord_t yoff)
 {
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, Element))
-		return (pcb_false);
+		return pcb_false;
 	pcb_elem_invalidate_erase(Element);
 	pcb_undo_add_obj_to_mirror(PCB_TYPE_ELEMENT, Element, Element, Element, yoff);
 	pcb_element_mirror(PCB->Data, Element, yoff);
 	pcb_elem_invalidate_draw(Element);
-	return (pcb_true);
+	return pcb_true;
 }
 
 /* changes the side of all selected and visible elements;
@@ -511,7 +513,7 @@ pcb_bool pcb_selected_element_change_side(void)
 		}
 	}
 	PCB_END_LOOP;
-	return (change);
+	return change;
 }
 
 /* changes the layout-name of an element */
@@ -533,7 +535,7 @@ char *pcb_element_text_change(pcb_board_t * pcb, pcb_data_t * data, pcb_element_
 	Element->Name[which].TextString = new_name;
 	pcb_text_bbox(pcb_font(PCB, 0, 1), &Element->Name[which]);
 
-	pcb_r_insert_entry(data->name_tree[which], &Element->Name[which].BoundingBox, 0);
+	pcb_r_insert_entry(data->name_tree[which], &Element->Name[which].BoundingBox);
 
 	if (pcb && which == PCB_ELEMNAME_IDX_VISIBLE())
 		pcb_elem_name_invalidate_draw(Element);
@@ -548,7 +550,7 @@ void pcb_element_text_update(pcb_board_t *pcb, pcb_data_t *data, pcb_element_t *
 
 	pcb_r_delete_entry(data->name_tree[which], &Element->Name[which].BoundingBox);
 	pcb_text_bbox(pcb_font(PCB, 0, 1), &Element->Name[which]);
-	pcb_r_insert_entry(data->name_tree[which], &Element->Name[which].BoundingBox, 0);
+	pcb_r_insert_entry(data->name_tree[which], &Element->Name[which].BoundingBox);
 
 	if (pcb && which == PCB_ELEMNAME_IDX_VISIBLE())
 		pcb_elem_name_invalidate_draw(Element);
@@ -562,7 +564,7 @@ void pcb_element_text_set_font(pcb_board_t *pcb, pcb_data_t *data, pcb_element_t
 	pcb_r_delete_entry(data->name_tree[which], &Element->Name[which].BoundingBox);
 	Element->Name[which].fid = fid;
 	pcb_text_bbox(pcb_font(PCB, 0, 1), &Element->Name[which]);
-	pcb_r_insert_entry(data->name_tree[which], &Element->Name[which].BoundingBox, 0);
+	pcb_r_insert_entry(data->name_tree[which], &Element->Name[which].BoundingBox);
 
 	if (pcb && which == PCB_ELEMNAME_IDX_VISIBLE())
 		pcb_elem_name_invalidate_draw(Element);
@@ -587,7 +589,7 @@ pcb_element_t *pcb_element_copy(pcb_data_t *Data, pcb_element_t *Dest, pcb_eleme
 
 	/* abort on error */
 	if (!Dest)
-		return (Dest);
+		return Dest;
 
 	PCB_ELEMENT_PCB_LINE_LOOP(Src);
 	{
@@ -621,7 +623,7 @@ pcb_element_t *pcb_element_copy(pcb_data_t *Data, pcb_element_t *Dest, pcb_eleme
 	Dest->MarkY = Src->MarkY + dy;
 
 	pcb_element_bbox(Data, Dest, pcb_font(PCB, 0, 1));
-	return (Dest);
+	return Dest;
 }
 
 /* creates an new element; memory is allocated if needed */
@@ -654,7 +656,7 @@ pcb_element_t *pcb_element_new(pcb_data_t *Data, pcb_element_t *Element,
 	printf("  .... Leaving CreateNewElement.\n");
 #endif
 
-	return (Element);
+	return Element;
 }
 
 /* creates a new arc in an element */
@@ -808,9 +810,9 @@ void pcb_element_bbox(pcb_data_t *Data, pcb_element_t *Element, pcb_font_t *Font
 			pcb_r_delete_entry(Data->name_tree[n], (pcb_box_t *) text);
 		pcb_text_bbox(Font, text);
 		if (Data && !Data->name_tree[n])
-			Data->name_tree[n] = pcb_r_create_tree(NULL, 0, 0);
+			Data->name_tree[n] = pcb_r_create_tree();
 		if (Data)
-			pcb_r_insert_entry(Data->name_tree[n], (pcb_box_t *) text, 0);
+			pcb_r_insert_entry(Data->name_tree[n], (pcb_box_t *) text);
 	}
 	PCB_END_LOOP;
 
@@ -851,8 +853,8 @@ void pcb_element_bbox(pcb_data_t *Data, pcb_element_t *Element, pcb_font_t *Font
 		pcb_pin_bbox(pin);
 		if (Data) {
 			if (!Data->pin_tree)
-				Data->pin_tree = pcb_r_create_tree(NULL, 0, 0);
-			pcb_r_insert_entry(Data->pin_tree, (pcb_box_t *) pin, 0);
+				Data->pin_tree = pcb_r_create_tree();
+			pcb_r_insert_entry(Data->pin_tree, (pcb_box_t *) pin);
 		}
 		PCB_MAKE_MIN(box->X1, pin->BoundingBox.X1);
 		PCB_MAKE_MIN(box->Y1, pin->BoundingBox.Y1);
@@ -871,8 +873,8 @@ void pcb_element_bbox(pcb_data_t *Data, pcb_element_t *Element, pcb_font_t *Font
 		pcb_pad_bbox(pad);
 		if (Data) {
 			if (!Data->pad_tree)
-				Data->pad_tree = pcb_r_create_tree(NULL, 0, 0);
-			pcb_r_insert_entry(Data->pad_tree, (pcb_box_t *) pad, 0);
+				Data->pad_tree = pcb_r_create_tree();
+			pcb_r_insert_entry(Data->pad_tree, (pcb_box_t *) pad);
 		}
 		PCB_MAKE_MIN(box->X1, pad->BoundingBox.X1);
 		PCB_MAKE_MIN(box->Y1, pad->BoundingBox.Y1);
@@ -924,9 +926,9 @@ void pcb_element_bbox(pcb_data_t *Data, pcb_element_t *Element, pcb_font_t *Font
 	pcb_close_box(box);
 	pcb_close_box(vbox);
 	if (Data && !Data->element_tree)
-		Data->element_tree = pcb_r_create_tree(NULL, 0, 0);
+		Data->element_tree = pcb_r_create_tree();
 	if (Data)
-		pcb_r_insert_entry(Data->element_tree, box, 0);
+		pcb_r_insert_entry(Data->element_tree, box);
 }
 
 static char *BumpName(char *Name)
@@ -952,7 +954,7 @@ static char *BumpName(char *Name)
 	/* if this is not our string, put back the blown character */
 	if (start != temp)
 		*Name = c;
-	return (temp);
+	return temp;
 }
 
 
@@ -963,7 +965,7 @@ char *pcb_element_uniq_name(pcb_data_t *Data, char *Name)
 	pcb_bool unique = pcb_true;
 	/* null strings are ok */
 	if (!Name || !*Name)
-		return (Name);
+		return Name;
 
 	for (;;) {
 		PCB_ELEMENT_LOOP(Data);
@@ -976,7 +978,7 @@ char *pcb_element_uniq_name(pcb_data_t *Data, char *Name)
 		}
 		PCB_END_LOOP;
 		if (unique)
-			return (Name);
+			return Name;
 		unique = pcb_true;
 	}
 }
@@ -1087,7 +1089,7 @@ void pcb_element_move(pcb_data_t *Data, pcb_element_t *Element, pcb_coord_t DX,
 		}
 		pcb_pin_move(pin, DX, DY);
 		if (Data) {
-			pcb_r_insert_entry(Data->pin_tree, (pcb_box_t *) pin, 0);
+			pcb_r_insert_entry(Data->pin_tree, (pcb_box_t *) pin);
 			pcb_poly_clear_from_poly(Data, PCB_TYPE_PIN, Element, pin);
 		}
 	}
@@ -1100,7 +1102,7 @@ void pcb_element_move(pcb_data_t *Data, pcb_element_t *Element, pcb_coord_t DX,
 		}
 		pcb_pad_move(pad, DX, DY);
 		if (Data) {
-			pcb_r_insert_entry(Data->pad_tree, (pcb_box_t *) pad, 0);
+			pcb_r_insert_entry(Data->pad_tree, (pcb_box_t *) pad);
 			pcb_poly_clear_from_poly(Data, PCB_TYPE_PAD, Element, pad);
 		}
 	}
@@ -1116,26 +1118,28 @@ void pcb_element_move(pcb_data_t *Data, pcb_element_t *Element, pcb_coord_t DX,
 			pcb_r_delete_entry(PCB->Data->name_tree[n], (pcb_box_t *) text);
 		pcb_text_move(text, DX, DY);
 		if (Data && Data->name_tree[n])
-			pcb_r_insert_entry(PCB->Data->name_tree[n], (pcb_box_t *) text, 0);
+			pcb_r_insert_entry(PCB->Data->name_tree[n], (pcb_box_t *) text);
 	}
 	PCB_END_LOOP;
 	PCB_BOX_MOVE_LOWLEVEL(&Element->BoundingBox, DX, DY);
 	PCB_BOX_MOVE_LOWLEVEL(&Element->VBox, DX, DY);
 	PCB_MOVE(Element->MarkX, Element->MarkY, DX, DY);
 	if (Data)
-		pcb_r_insert_entry(Data->element_tree, (pcb_box_t *) Element, 0);
+		pcb_r_insert_entry(Data->element_tree, (pcb_box_t *) Element);
 }
 
 void *pcb_element_remove(pcb_element_t *Element)
 {
+	void *res;
 	pcb_opctx_t ctx;
 
 	ctx.remove.pcb = PCB;
-	ctx.remove.bulk = pcb_false;
 	ctx.remove.destroy_target = NULL;
 	PCB_CLEAR_PARENT(Element);
 
-	return pcb_elemop_remove(&ctx, Element);
+	res = pcb_elemop_remove(&ctx, Element);
+	pcb_draw();
+	return res;
 }
 
 /* rotate an element in 90 degree steps */
@@ -1187,6 +1191,7 @@ void pcb_element_rotate90(pcb_data_t *Data, pcb_element_t *Element, pcb_coord_t
 	pcb_poly_clear_from_poly(Data, PCB_TYPE_ELEMENT, Element, Element);
 }
 
+#if 0
 unsigned int pcb_element_hash(const pcb_element_t *e)
 {
 	unsigned int val = 0;
@@ -1285,7 +1290,7 @@ int pcb_element_eq(const pcb_element_t *e1, const pcb_element_t *e2)
 
 	return 1;
 }
-
+#endif
 
 
 /*** ops ***/
@@ -1314,11 +1319,11 @@ void *pcb_elemop_add_to_buffer(pcb_opctx_t *ctx, pcb_element_t *Element)
 		}
 		PCB_END_LOOP;
 	}
-	return (element);
+	return element;
 }
 
 /* moves a element to buffer without allocating memory for pins/names */
-void *pcb_elemop_move_to_buffer(pcb_opctx_t *ctx, pcb_element_t * element)
+void *pcb_elemop_move_buffer(pcb_opctx_t *ctx, pcb_element_t * element)
 {
 	/*
 	 * Delete the element from the source (remove it from trees,
@@ -1369,7 +1374,7 @@ void *pcb_elemop_change_2nd_size(pcb_opctx_t *ctx, pcb_element_t *Element)
 	pcb_coord_t value;
 
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, Element))
-		return (NULL);
+		return NULL;
 	PCB_PIN_LOOP(Element);
 	{
 		value = (ctx->chgsize.is_absolute) ? ctx->chgsize.value : pin->DrillingHole + ctx->chgsize.value;
@@ -1391,9 +1396,9 @@ void *pcb_elemop_change_2nd_size(pcb_opctx_t *ctx, pcb_element_t *Element)
 	}
 	PCB_END_LOOP;
 	if (changed)
-		return (Element);
+		return Element;
 	else
-		return (NULL);
+		return NULL;
 }
 
 /* changes ring dia of all pins of an element; returns pcb_true if changed */
@@ -1403,7 +1408,7 @@ void *pcb_elemop_change_1st_size(pcb_opctx_t *ctx, pcb_element_t *Element)
 	pcb_coord_t value;
 
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, Element))
-		return (NULL);
+		return NULL;
 	PCB_PIN_LOOP(Element);
 	{
 		value = (ctx->chgsize.is_absolute) ? ctx->chgsize.value : pin->DrillingHole + ctx->chgsize.value;
@@ -1423,9 +1428,9 @@ void *pcb_elemop_change_1st_size(pcb_opctx_t *ctx, pcb_element_t *Element)
 	}
 	PCB_END_LOOP;
 	if (changed)
-		return (Element);
+		return Element;
 	else
-		return (NULL);
+		return NULL;
 }
 
 /* changes the clearance of all pins of an element; returns pcb_true if changed */
@@ -1435,7 +1440,7 @@ void *pcb_elemop_change_clear_size(pcb_opctx_t *ctx, pcb_element_t *Element)
 	pcb_coord_t value;
 
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, Element))
-		return (NULL);
+		return NULL;
 	PCB_PIN_LOOP(Element);
 	{
 		value = (ctx->chgsize.is_absolute) ? ctx->chgsize.value : pin->Clearance + ctx->chgsize.value;
@@ -1481,9 +1486,9 @@ void *pcb_elemop_change_clear_size(pcb_opctx_t *ctx, pcb_element_t *Element)
 	PCB_END_LOOP;
 
 	if (changed)
-		return (Element);
+		return Element;
 	else
-		return (NULL);
+		return NULL;
 }
 
 /* changes the scaling factor of an element's outline; returns pcb_true if changed */
@@ -1493,7 +1498,7 @@ void *pcb_elemop_change_size(pcb_opctx_t *ctx, pcb_element_t *Element)
 	pcb_bool changed = pcb_false;
 
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, Element))
-		return (NULL);
+		return NULL;
 	if (pcb_silk_on(PCB))
 		pcb_elem_invalidate_erase(Element);
 	PCB_ELEMENT_PCB_LINE_LOOP(Element);
@@ -1520,8 +1525,8 @@ void *pcb_elemop_change_size(pcb_opctx_t *ctx, pcb_element_t *Element)
 		pcb_elem_invalidate_draw(Element);
 	}
 	if (changed)
-		return (Element);
-	return (NULL);
+		return Element;
+	return NULL;
 }
 
 /* changes the scaling factor of a elementname object; returns pcb_true if changed */
@@ -1531,7 +1536,7 @@ void *pcb_elemop_change_name_size(pcb_opctx_t *ctx, pcb_element_t *Element)
 		: PCB_ELEM_TEXT_DESCRIPTION(Element).Scale + PCB_COORD_TO_MIL(ctx->chgsize.value);
 
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, &Element->Name[0]))
-		return (NULL);
+		return NULL;
 	if (value <= PCB_MAX_TEXTSCALE && value >= PCB_MIN_TEXTSCALE) {
 		pcb_elem_name_invalidate_erase(Element);
 		PCB_ELEMENT_PCB_TEXT_LOOP(Element);
@@ -1540,20 +1545,20 @@ void *pcb_elemop_change_name_size(pcb_opctx_t *ctx, pcb_element_t *Element)
 			pcb_r_delete_entry(PCB->Data->name_tree[n], (pcb_box_t *) text);
 			text->Scale = value;
 			pcb_text_bbox(pcb_font(PCB, 0, 1), text);
-			pcb_r_insert_entry(PCB->Data->name_tree[n], (pcb_box_t *) text, 0);
+			pcb_r_insert_entry(PCB->Data->name_tree[n], (pcb_box_t *) text);
 		}
 		PCB_END_LOOP;
 		pcb_elem_name_invalidate_draw(Element);
-		return (Element);
+		return Element;
 	}
-	return (NULL);
+	return NULL;
 }
 
 
 void *pcb_elemop_change_name(pcb_opctx_t *ctx, pcb_element_t *Element)
 {
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, &Element->Name[0]))
-		return (NULL);
+		return NULL;
 	if (PCB_ELEMNAME_IDX_VISIBLE() == PCB_ELEMNAME_IDX_REFDES) {
 		if (conf_core.editor.unique_names && pcb_element_uniq_name(PCB->Data, ctx->chgname.new_name) != ctx->chgname.new_name) {
 			pcb_message(PCB_MSG_ERROR, _("Error: The name \"%s\" is not unique!\n"), ctx->chgname.new_name);
@@ -1567,7 +1572,7 @@ void *pcb_elemop_change_name(pcb_opctx_t *ctx, pcb_element_t *Element)
 void *pcb_elemop_change_nonetlist(pcb_opctx_t *ctx, pcb_element_t *Element)
 {
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, Element))
-		return (NULL);
+		return NULL;
 	PCB_FLAG_TOGGLE(PCB_FLAG_NONETLIST, Element);
 	return Element;
 }
@@ -1579,7 +1584,7 @@ void *pcb_elemop_change_square(pcb_opctx_t *ctx, pcb_element_t *Element)
 	void *ans = NULL;
 
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, Element))
-		return (NULL);
+		return NULL;
 	PCB_PIN_LOOP(Element);
 	{
 		ans = pcb_pinop_change_square(ctx, Element, pin);
@@ -1590,7 +1595,7 @@ void *pcb_elemop_change_square(pcb_opctx_t *ctx, pcb_element_t *Element)
 		ans = pcb_padop_change_square(ctx, Element, pad);
 	}
 	PCB_END_LOOP;
-	return (ans);
+	return ans;
 }
 
 /* sets the square flag of all pins on an element */
@@ -1599,7 +1604,7 @@ void *pcb_elemop_set_square(pcb_opctx_t *ctx, pcb_element_t *Element)
 	void *ans = NULL;
 
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, Element))
-		return (NULL);
+		return NULL;
 	PCB_PIN_LOOP(Element);
 	{
 		ans = pcb_pinop_set_square(ctx, Element, pin);
@@ -1610,7 +1615,7 @@ void *pcb_elemop_set_square(pcb_opctx_t *ctx, pcb_element_t *Element)
 		ans = pcb_padop_set_square(ctx, Element, pad);
 	}
 	PCB_END_LOOP;
-	return (ans);
+	return ans;
 }
 
 /* clears the square flag of all pins on an element */
@@ -1619,7 +1624,7 @@ void *pcb_elemop_clear_square(pcb_opctx_t *ctx, pcb_element_t *Element)
 	void *ans = NULL;
 
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, Element))
-		return (NULL);
+		return NULL;
 	PCB_PIN_LOOP(Element);
 	{
 		ans = pcb_pinop_clear_square(ctx, Element, pin);
@@ -1630,7 +1635,7 @@ void *pcb_elemop_clear_square(pcb_opctx_t *ctx, pcb_element_t *Element)
 		ans = pcb_padop_clear_square(ctx, Element, pad);
 	}
 	PCB_END_LOOP;
-	return (ans);
+	return ans;
 }
 
 /* changes the octagon flags of all pins of an element */
@@ -1639,14 +1644,14 @@ void *pcb_elemop_change_octagon(pcb_opctx_t *ctx, pcb_element_t *Element)
 	void *result = NULL;
 
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, Element))
-		return (NULL);
+		return NULL;
 	PCB_PIN_LOOP(Element);
 	{
 		pcb_pinop_change_octagon(ctx, Element, pin);
 		result = Element;
 	}
 	PCB_END_LOOP;
-	return (result);
+	return result;
 }
 
 /* sets the octagon flags of all pins of an element */
@@ -1655,14 +1660,14 @@ void *pcb_elemop_set_octagon(pcb_opctx_t *ctx, pcb_element_t *Element)
 	void *result = NULL;
 
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, Element))
-		return (NULL);
+		return NULL;
 	PCB_PIN_LOOP(Element);
 	{
 		pcb_pinop_set_octagon(ctx, Element, pin);
 		result = Element;
 	}
 	PCB_END_LOOP;
-	return (result);
+	return result;
 }
 
 /* clears the octagon flags of all pins of an element */
@@ -1671,14 +1676,14 @@ void *pcb_elemop_clear_octagon(pcb_opctx_t *ctx, pcb_element_t *Element)
 	void *result = NULL;
 
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, Element))
-		return (NULL);
+		return NULL;
 	PCB_PIN_LOOP(Element);
 	{
 		pcb_pinop_clear_octagon(ctx, Element, pin);
 		result = Element;
 	}
 	PCB_END_LOOP;
-	return (result);
+	return result;
 }
 
 /* copies an element onto the PCB.  Then does a draw. */
@@ -1706,7 +1711,7 @@ void *pcb_elemop_copy(pcb_opctx_t *ctx, pcb_element_t *Element)
 #ifdef DEBUG
 	printf(" ... Leaving pcb_elemop_copy.\n");
 #endif
-	return (element);
+	return element;
 }
 
 /* moves all names of an element to a new position */
@@ -1720,11 +1725,10 @@ void *pcb_elemop_move_name(pcb_opctx_t *ctx, pcb_element_t *Element)
 				pcb_r_delete_entry(PCB->Data->name_tree[n], (pcb_box_t *) text);
 			pcb_text_move(text, ctx->move.dx, ctx->move.dy);
 			if (PCB->Data->name_tree[n])
-				pcb_r_insert_entry(PCB->Data->name_tree[n], (pcb_box_t *) text, 0);
+				pcb_r_insert_entry(PCB->Data->name_tree[n], (pcb_box_t *) text);
 		}
 		PCB_END_LOOP;
 		pcb_elem_name_invalidate_draw(Element);
-		pcb_draw();
 	}
 	else {
 		PCB_ELEMENT_PCB_TEXT_LOOP(Element);
@@ -1733,24 +1737,21 @@ void *pcb_elemop_move_name(pcb_opctx_t *ctx, pcb_element_t *Element)
 				pcb_r_delete_entry(PCB->Data->name_tree[n], (pcb_box_t *) text);
 			pcb_text_move(text, ctx->move.dx, ctx->move.dy);
 			if (PCB->Data->name_tree[n])
-				pcb_r_insert_entry(PCB->Data->name_tree[n], (pcb_box_t *) text, 0);
+				pcb_r_insert_entry(PCB->Data->name_tree[n], (pcb_box_t *) text);
 		}
 		PCB_END_LOOP;
 	}
-	return (Element);
+	return Element;
 }
 
 /* moves an element */
 void *pcb_elemop_move(pcb_opctx_t *ctx, pcb_element_t *Element)
 {
-	pcb_bool didDraw = pcb_false;
-
 	if (pcb_silk_on(PCB) && (PCB_FRONT(Element) || PCB->InvisibleObjectsOn)) {
 		pcb_elem_invalidate_erase(Element);
 		pcb_element_move(PCB->Data, Element, ctx->move.dx, ctx->move.dy);
 		pcb_elem_name_invalidate_draw(Element);
 		pcb_elem_package_invalidate_draw(Element);
-		didDraw = pcb_true;
 	}
 	else {
 		if (PCB->PinOn)
@@ -1759,11 +1760,8 @@ void *pcb_elemop_move(pcb_opctx_t *ctx, pcb_element_t *Element)
 	}
 	if (PCB->PinOn) {
 		pcb_elem_pp_invalidate_draw(Element);
-		didDraw = pcb_true;
 	}
-	if (didDraw)
-		pcb_draw();
-	return (Element);
+	return Element;
 }
 
 /* destroys a element */
@@ -1802,11 +1800,8 @@ void *pcb_elemop_destroy(pcb_opctx_t *ctx, pcb_element_t *Element)
 void *pcb_elemop_remove(pcb_opctx_t *ctx, pcb_element_t *Element)
 {
 	/* erase from screen */
-	if ((pcb_silk_on(PCB) || PCB->PinOn) && (PCB_FRONT(Element) || PCB->InvisibleObjectsOn)) {
+	if ((pcb_silk_on(PCB) || PCB->PinOn) && (PCB_FRONT(Element) || PCB->InvisibleObjectsOn))
 		pcb_elem_invalidate_erase(Element);
-		if (!ctx->remove.bulk)
-			pcb_draw();
-	}
 	pcb_undo_move_obj_to_remove(PCB_TYPE_ELEMENT, Element, Element, Element);
 	return NULL;
 }
@@ -1817,8 +1812,7 @@ void *pcb_elemop_rotate90(pcb_opctx_t *ctx, pcb_element_t *Element)
 	pcb_elem_invalidate_erase(Element);
 	pcb_element_rotate90(PCB->Data, Element, ctx->rotate.center_x, ctx->rotate.center_y, ctx->rotate.number);
 	pcb_elem_invalidate_draw(Element);
-	pcb_draw();
-	return (Element);
+	return Element;
 }
 
 /* ----------------------------------------------------------------------
@@ -1831,12 +1825,11 @@ void *pcb_elemop_rotate90_name(pcb_opctx_t *ctx, pcb_element_t *Element)
 	{
 		pcb_r_delete_entry(PCB->Data->name_tree[n], (pcb_box_t *) text);
 		pcb_text_rotate90(text, ctx->rotate.center_x, ctx->rotate.center_y, ctx->rotate.number);
-		pcb_r_insert_entry(PCB->Data->name_tree[n], (pcb_box_t *) text, 0);
+		pcb_r_insert_entry(PCB->Data->name_tree[n], (pcb_box_t *) text);
 	}
 	PCB_END_LOOP;
 	pcb_elem_name_invalidate_draw(Element);
-	pcb_draw();
-	return (Element);
+	return Element;
 }
 
 /*** draw ***/
