@@ -1,7 +1,8 @@
 /*
  *                            COPYRIGHT
  *
- *  PCB, interactive printed circuit board design
+ *  pcb-rnd, interactive printed circuit board design
+ *  (this file is based on PCB, interactive printed circuit board design)
  *  Copyright (C) 1994,1995,1996 Thomas Nau
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -18,9 +19,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- *  Contact addresses for paper mail and Email:
- *  Thomas Nau, Schlehenweg 15, 88471 Baustetten, Germany
- *  Thomas.Nau@rz.uni-ulm.de
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  *
  */
 
@@ -43,6 +45,7 @@
 #include "obj_line_op.h"
 
 #include "obj_subc_parent.h"
+#include "obj_hash.h"
 
 /* TODO: maybe remove this and move lines from draw here? */
 #include "draw.h"
@@ -113,7 +116,7 @@ static pcb_r_dir_t line_callback(const pcb_box_t * b, void *cl)
 			i->test.Point1.Y = line->Point2.Y;
 			i->test.Point2.X = i->X2;
 			i->test.Point2.Y = i->Y2;
-			if (pcb_is_point_on_line(i->X1, i->Y1, 0.0, &i->test)) {
+			if (pcb_is_point_on_line(i->X1, i->Y1, 1, &i->test)) {
 				i->ans = line;
 				longjmp(i->env, 1);
 			}
@@ -123,7 +126,7 @@ static pcb_r_dir_t line_callback(const pcb_box_t * b, void *cl)
 			i->test.Point1.Y = line->Point1.Y;
 			i->test.Point2.X = i->X2;
 			i->test.Point2.Y = i->Y2;
-			if (pcb_is_point_on_line(i->X1, i->Y1, 0.0, &i->test)) {
+			if (pcb_is_point_on_line(i->X1, i->Y1, 1, &i->test)) {
 				i->ans = line;
 				longjmp(i->env, 1);
 			}
@@ -133,7 +136,7 @@ static pcb_r_dir_t line_callback(const pcb_box_t * b, void *cl)
 			i->test.Point1.Y = line->Point2.Y;
 			i->test.Point2.X = i->X1;
 			i->test.Point2.Y = i->Y1;
-			if (pcb_is_point_on_line(i->X2, i->Y2, 0.0, &i->test)) {
+			if (pcb_is_point_on_line(i->X2, i->Y2, 1, &i->test)) {
 				i->ans = line;
 				longjmp(i->env, 1);
 			}
@@ -143,7 +146,7 @@ static pcb_r_dir_t line_callback(const pcb_box_t * b, void *cl)
 			i->test.Point1.Y = line->Point1.Y;
 			i->test.Point2.X = i->X1;
 			i->test.Point2.Y = i->Y1;
-			if (pcb_is_point_on_line(i->X2, i->Y2, 0.0, &i->test)) {
+			if (pcb_is_point_on_line(i->X2, i->Y2, 1, &i->test)) {
 				i->ans = line;
 				longjmp(i->env, 1);
 			}
@@ -206,7 +209,7 @@ pcb_line_t *pcb_line_new(pcb_layer_t *Layer, pcb_coord_t X1, pcb_coord_t Y1, pcb
 
 	Line = pcb_line_alloc(Layer);
 	if (!Line)
-		return (Line);
+		return Line;
 	Line->ID = pcb_create_ID_get();
 	Line->Flags = Flags;
 	PCB_FLAG_CLEAR(PCB_FLAG_RAT, Line);
@@ -219,7 +222,7 @@ pcb_line_t *pcb_line_new(pcb_layer_t *Layer, pcb_coord_t X1, pcb_coord_t Y1, pcb
 	Line->Point2.Y = Y2;
 	Line->Point2.ID = pcb_create_ID_get();
 	pcb_add_line_on_layer(Layer, Line);
-	return (Line);
+	return Line;
 }
 
 static pcb_line_t *pcb_line_copy_meta(pcb_line_t *dst, pcb_line_t *src)
@@ -248,8 +251,8 @@ void pcb_add_line_on_layer(pcb_layer_t *Layer, pcb_line_t *Line)
 {
 	pcb_line_bbox(Line);
 	if (!Layer->line_tree)
-		Layer->line_tree = pcb_r_create_tree(NULL, 0, 0);
-	pcb_r_insert_entry(Layer->line_tree, (pcb_box_t *) Line, 0);
+		Layer->line_tree = pcb_r_create_tree();
+	pcb_r_insert_entry(Layer->line_tree, (pcb_box_t *) Line);
 }
 
 /* sets the bounding box of a line */
@@ -269,23 +272,36 @@ void pcb_line_bbox(pcb_line_t *Line)
 	pcb_set_point_bounding_box(&Line->Point2);
 }
 
-int pcb_line_eq(const pcb_element_t *e1, const pcb_line_t *l1, const pcb_element_t *e2, const pcb_line_t *l2)
+int pcb_line_eq(const pcb_host_trans_t *tr1, const pcb_line_t *l1, const pcb_host_trans_t *tr2, const pcb_line_t *l2)
 {
 	if (pcb_field_neq(l1, l2, Thickness) || pcb_field_neq(l1, l2, Clearance)) return 0;
-	if (pcb_element_neq_offsx(e1, l1, e2, l2, Point1.X) || pcb_element_neq_offsy(e1, l1, e2, l2, Point1.Y)) return 0;
-	if (pcb_element_neq_offsx(e1, l1, e2, l2, Point2.X) || pcb_element_neq_offsy(e1, l1, e2, l2, Point2.Y)) return 0;
-	if (pcb_neqs(l1->Number, l2->Number)) return 0;
+	if (pcb_neqs(l1->term, l2->term)) return 0;
+	if (!PCB_FLAG_TEST(PCB_FLAG_FLOATER, l1) && !PCB_FLAG_TEST(PCB_FLAG_FLOATER, l2)) {
+		if (pcb_neq_tr_coords(tr1, l1->Point1.X, l1->Point1.Y, tr2, l2->Point1.X, l2->Point1.Y)) goto swap;
+		if (pcb_neq_tr_coords(tr1, l1->Point2.X, l1->Point2.Y, tr2, l2->Point2.X, l2->Point2.Y)) return 0;
+	}
+	return 1;
+
+	swap:; /* check the line with swapped endpoints */
+	if (pcb_neq_tr_coords(tr1, l1->Point2.X, l1->Point2.Y, tr2, l2->Point1.X, l2->Point1.Y)) return 0;
+	if (pcb_neq_tr_coords(tr1, l1->Point1.X, l1->Point1.Y, tr2, l2->Point2.X, l2->Point2.Y)) return 0;
 	return 1;
 }
 
 
-unsigned int pcb_line_hash(const pcb_element_t *e, const pcb_line_t *l)
+unsigned int pcb_line_hash(const pcb_host_trans_t *tr, const pcb_line_t *l)
 {
-	return
-		pcb_hash_coord(l->Thickness) ^ pcb_hash_coord(l->Clearance) ^
-		pcb_hash_element_ox(e, l->Point1.X) ^ pcb_hash_element_oy(e, l->Point1.Y) ^
-		pcb_hash_element_ox(e, l->Point2.X) ^ pcb_hash_element_oy(e, l->Point2.Y) ^
-		pcb_hash_str(l->Number);
+	unsigned int crd = 0;
+
+	if (!PCB_FLAG_TEST(PCB_FLAG_FLOATER, l)) {
+		pcb_coord_t x1, y1, x2, y2;
+		pcb_hash_tr_coords(tr, &x1, &y1, l->Point1.X, l->Point1.Y);
+		pcb_hash_tr_coords(tr, &x2, &y2, l->Point2.X, l->Point2.Y);
+		crd = pcb_hash_coord(x1) ^ pcb_hash_coord(y1) ^ pcb_hash_coord(x2) ^ pcb_hash_coord(y2);
+	}
+
+	return pcb_hash_coord(l->Thickness) ^ pcb_hash_coord(l->Clearance) ^
+		pcb_hash_str(l->term) ^ crd;
 }
 
 
@@ -293,6 +309,7 @@ pcb_coord_t pcb_line_length(const pcb_line_t *line)
 {
 	pcb_coord_t dx = line->Point1.X - line->Point2.X;
 	pcb_coord_t dy = line->Point1.Y - line->Point2.Y;
+#warning TODO: use the distance func instead
 	return pcb_round(sqrt((double)dx*(double)dx + (double)dy*(double)dy));
 }
 
@@ -350,27 +367,31 @@ void *pcb_lineop_add_to_buffer(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_line_t
 	return pcb_line_copy_meta(line, Line);
 }
 
-/* moves a line to buffer */
-void *pcb_lineop_move_to_buffer(pcb_opctx_t *ctx, pcb_layer_t * layer, pcb_line_t * line)
+/* moves a line between board and buffer */
+void *pcb_lineop_move_buffer(pcb_opctx_t *ctx, pcb_layer_t *dstly, pcb_line_t *line)
 {
-	pcb_layer_t *lay = &ctx->buffer.dst->Layer[pcb_layer_id(ctx->buffer.src, layer)];
+	pcb_layer_t *srcly = line->parent.layer;
 
-	pcb_poly_restore_to_poly(ctx->buffer.src, PCB_TYPE_LINE, layer, line);
-	pcb_r_delete_entry(layer->line_tree, (pcb_box_t *) line);
+	assert(line->parent_type == PCB_PARENT_LAYER);
+	if ((dstly == NULL) || (dstly == srcly)) /* auto layer in dst */
+		dstly = &ctx->buffer.dst->Layer[pcb_layer_id(ctx->buffer.src, srcly)];
+
+	pcb_poly_restore_to_poly(ctx->buffer.src, PCB_TYPE_LINE, srcly, line);
+	pcb_r_delete_entry(srcly->line_tree, (pcb_box_t *)line);
 
 	linelist_remove(line);
-	linelist_append(&(lay->Line), line);
+	linelist_append(&(dstly->Line), line);
 
 	PCB_FLAG_CLEAR(PCB_FLAG_FOUND, line);
 
-	if (!lay->line_tree)
-		lay->line_tree = pcb_r_create_tree(NULL, 0, 0);
-	pcb_r_insert_entry(lay->line_tree, (pcb_box_t *) line, 0);
-	pcb_poly_clear_from_poly(ctx->buffer.dst, PCB_TYPE_LINE, lay, line);
+	if (!dstly->line_tree)
+		dstly->line_tree = pcb_r_create_tree();
+	pcb_r_insert_entry(dstly->line_tree, (pcb_box_t *)line);
+	pcb_poly_clear_from_poly(ctx->buffer.dst, PCB_TYPE_LINE, dstly, line);
 
-	PCB_SET_PARENT(line, layer, lay);
+	PCB_SET_PARENT(line, layer, dstly);
 
-	return (line);
+	return line;
 }
 
 /* changes the size of a line */
@@ -379,7 +400,7 @@ void *pcb_lineop_change_size(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_line_t *L
 	pcb_coord_t value = (ctx->chgsize.is_absolute) ? ctx->chgsize.value : Line->Thickness + ctx->chgsize.value;
 
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, Line))
-		return (NULL);
+		return NULL;
 	if (value <= PCB_MAX_LINESIZE && value >= PCB_MIN_LINESIZE && value != Line->Thickness) {
 		pcb_undo_add_obj_to_size(PCB_TYPE_LINE, Layer, Line, Line);
 		pcb_line_invalidate_erase(Line);
@@ -387,12 +408,12 @@ void *pcb_lineop_change_size(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_line_t *L
 		pcb_poly_restore_to_poly(ctx->chgsize.pcb->Data, PCB_TYPE_LINE, Layer, Line);
 		Line->Thickness = value;
 		pcb_line_bbox(Line);
-		pcb_r_insert_entry(Layer->line_tree, (pcb_box_t *) Line, 0);
+		pcb_r_insert_entry(Layer->line_tree, (pcb_box_t *) Line);
 		pcb_poly_clear_from_poly(ctx->chgsize.pcb->Data, PCB_TYPE_LINE, Layer, Line);
 		pcb_line_invalidate_draw(Layer, Line);
-		return (Line);
+		return Line;
 	}
-	return (NULL);
+	return NULL;
 }
 
 /*changes the clearance size of a line */
@@ -401,7 +422,7 @@ void *pcb_lineop_change_clear_size(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_lin
 	pcb_coord_t value = (ctx->chgsize.is_absolute) ? ctx->chgsize.value : Line->Clearance + ctx->chgsize.value;
 
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, Line) || !PCB_FLAG_TEST(PCB_FLAG_CLEARLINE, Line))
-		return (NULL);
+		return NULL;
 	if (value < 0)
 		value = 0;
 	value = MIN(PCB_MAX_LINESIZE, value);
@@ -416,12 +437,12 @@ void *pcb_lineop_change_clear_size(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_lin
 		pcb_r_delete_entry(Layer->line_tree, (pcb_box_t *) Line);
 		Line->Clearance = value;
 		pcb_line_bbox(Line);
-		pcb_r_insert_entry(Layer->line_tree, (pcb_box_t *) Line, 0);
+		pcb_r_insert_entry(Layer->line_tree, (pcb_box_t *) Line);
 		pcb_poly_clear_from_poly(ctx->chgsize.pcb->Data, PCB_TYPE_LINE, Layer, Line);
 		pcb_line_invalidate_draw(Layer, Line);
-		return (Line);
+		return Line;
 	}
-	return (NULL);
+	return NULL;
 }
 
 /* changes the name of a line */
@@ -431,14 +452,14 @@ void *pcb_lineop_change_name(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_line_t *L
 
 	Layer = Layer;
 	Line->Number = ctx->chgname.new_name;
-	return (old);
+	return old;
 }
 
 /* changes the clearance flag of a line */
 void *pcb_lineop_change_join(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_line_t *Line)
 {
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, Line))
-		return (NULL);
+		return NULL;
 	pcb_line_invalidate_erase(Line);
 	if (PCB_FLAG_TEST(PCB_FLAG_CLEARLINE, Line)) {
 		pcb_undo_add_obj_to_clear_poly(PCB_TYPE_LINE, Layer, Line, Line, pcb_false);
@@ -451,14 +472,14 @@ void *pcb_lineop_change_join(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_line_t *L
 		pcb_poly_clear_from_poly(ctx->chgsize.pcb->Data, PCB_TYPE_LINE, Layer, Line);
 	}
 	pcb_line_invalidate_draw(Layer, Line);
-	return (Line);
+	return Line;
 }
 
 /* sets the clearance flag of a line */
 void *pcb_lineop_set_join(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_line_t *Line)
 {
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, Line) || PCB_FLAG_TEST(PCB_FLAG_CLEARLINE, Line))
-		return (NULL);
+		return NULL;
 	return pcb_lineop_change_join(ctx, Layer, Line);
 }
 
@@ -466,7 +487,7 @@ void *pcb_lineop_set_join(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_line_t *Line
 void *pcb_lineop_clear_join(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_line_t *Line)
 {
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, Line) || !PCB_FLAG_TEST(PCB_FLAG_CLEARLINE, Line))
-		return (NULL);
+		return NULL;
 	return pcb_lineop_change_join(ctx, Layer, Line);
 }
 
@@ -480,11 +501,11 @@ void *pcb_lineop_copy(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_line_t *Line)
 																Line->Point2.X + ctx->copy.DeltaX,
 																Line->Point2.Y + ctx->copy.DeltaY, Line->Thickness, Line->Clearance, pcb_flag_mask(Line->Flags, PCB_FLAG_FOUND));
 	if (!line)
-		return (line);
+		return line;
 	pcb_line_copy_meta(line, Line);
 	pcb_line_invalidate_draw(Layer, line);
 	pcb_undo_add_obj_to_create(PCB_TYPE_LINE, Layer, line, line);
-	return (line);
+	return line;
 }
 
 /* moves a line */
@@ -493,10 +514,8 @@ void *pcb_lineop_move_noclip(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_line_t *L
 	if (Layer->meta.real.vis)
 		pcb_line_invalidate_erase(Line);
 	pcb_line_move(Line, ctx->move.dx, ctx->move.dy);
-	if (Layer->meta.real.vis) {
+	if (Layer->meta.real.vis)
 		pcb_line_invalidate_draw(Layer, Line);
-		pcb_draw();
-	}
 	return Line;
 }
 
@@ -505,7 +524,7 @@ void *pcb_lineop_move(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_line_t *Line)
 	pcb_r_delete_entry(Layer->line_tree, (pcb_box_t *) Line);
 	pcb_poly_restore_to_poly(ctx->move.pcb->Data, PCB_TYPE_LINE, Layer, Line);
 	pcb_lineop_move_noclip(ctx, Layer, Line);
-	pcb_r_insert_entry(Layer->line_tree, (pcb_box_t *) Line, 0);
+	pcb_r_insert_entry(Layer->line_tree, (pcb_box_t *) Line);
 	pcb_poly_clear_from_poly(ctx->move.pcb->Data, PCB_TYPE_LINE, Layer, Line);
 	return Line;
 }
@@ -518,7 +537,7 @@ void *pcb_lineop_clip(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_line_t *Line)
 			pcb_poly_restore_to_poly(ctx->clip.pcb->Data, PCB_TYPE_LINE, Layer, Line);
 	}
 	if (ctx->clip.clear) {
-		pcb_r_insert_entry(Layer->line_tree, (pcb_box_t *) Line, 0);
+		pcb_r_insert_entry(Layer->line_tree, (pcb_box_t *) Line);
 		if (ctx->clip.pcb != NULL)
 			pcb_poly_clear_from_poly(ctx->clip.pcb->Data, PCB_TYPE_LINE, Layer, Line);
 	}
@@ -535,13 +554,11 @@ void *pcb_lineop_move_point(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_line_t *Li
 		pcb_r_delete_entry(Layer->line_tree, &Line->BoundingBox);
 		PCB_MOVE(Point->X, Point->Y, ctx->move.dx, ctx->move.dy);
 		pcb_line_bbox(Line);
-		pcb_r_insert_entry(Layer->line_tree, &Line->BoundingBox, 0);
+		pcb_r_insert_entry(Layer->line_tree, &Line->BoundingBox);
 		pcb_poly_clear_from_poly(ctx->move.pcb->Data, PCB_TYPE_LINE, Layer, Line);
-		if (Layer->meta.real.vis) {
+		if (Layer->meta.real.vis)
 			pcb_line_invalidate_draw(Layer, Line);
-			pcb_draw();
-		}
-		return (Line);
+		return Line;
 	}
 	else {												/* must be a rat */
 
@@ -550,12 +567,10 @@ void *pcb_lineop_move_point(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_line_t *Li
 		pcb_r_delete_entry(ctx->move.pcb->Data->rat_tree, &Line->BoundingBox);
 		PCB_MOVE(Point->X, Point->Y, ctx->move.dx, ctx->move.dy);
 		pcb_line_bbox(Line);
-		pcb_r_insert_entry(ctx->move.pcb->Data->rat_tree, &Line->BoundingBox, 0);
-		if (ctx->move.pcb->RatOn) {
+		pcb_r_insert_entry(ctx->move.pcb->Data->rat_tree, &Line->BoundingBox);
+		if (ctx->move.pcb->RatOn)
 			pcb_rat_invalidate_draw((pcb_rat_t *) Line);
-			pcb_draw();
-		}
-		return (Line);
+		return Line;
 	}
 }
 
@@ -589,8 +604,6 @@ void *pcb_lineop_move_point_with_route(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb
 												pcb_gui->control_is_pressed() );
 		pcb_route_apply_to_line(&route,Layer,Line);
 		pcb_route_destroy(&route);
-
-		pcb_draw();
 	}
 	return NULL;
 }
@@ -604,8 +617,8 @@ void *pcb_lineop_move_to_layer_low(pcb_opctx_t *ctx, pcb_layer_t * Source, pcb_l
 	linelist_append(&(Destination->Line), line);
 
 	if (!Destination->line_tree)
-		Destination->line_tree = pcb_r_create_tree(NULL, 0, 0);
-	pcb_r_insert_entry(Destination->line_tree, (pcb_box_t *) line, 0);
+		Destination->line_tree = pcb_r_create_tree();
+	pcb_r_insert_entry(Destination->line_tree, (pcb_box_t *) line);
 
 	PCB_SET_PARENT(line, layer, Destination);
 
@@ -645,12 +658,10 @@ void *pcb_lineop_move_to_layer(pcb_opctx_t *ctx, pcb_layer_t * Layer, pcb_line_t
 		pcb_message(PCB_MSG_WARNING, _("Sorry, the object is locked\n"));
 		return NULL;
 	}
-	if (ctx->move.dst_layer == Layer && Layer->meta.real.vis) {
+	if (ctx->move.dst_layer == Layer && Layer->meta.real.vis)
 		pcb_line_invalidate_draw(Layer, Line);
-		pcb_draw();
-	}
 	if (ctx->move.dst_layer == Layer)
-		return (Line);
+		return Line;
 
 	pcb_undo_add_obj_to_move_to_layer(PCB_TYPE_LINE, Layer, Line, Line);
 	if (Layer->meta.real.vis)
@@ -661,11 +672,10 @@ void *pcb_lineop_move_to_layer(pcb_opctx_t *ctx, pcb_layer_t * Layer, pcb_line_t
 	pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_LINE, ctx->move.dst_layer, newone);
 	if (ctx->move.dst_layer->meta.real.vis)
 		pcb_line_invalidate_draw(ctx->move.dst_layer, newone);
-	pcb_draw();
-	if (!PCB->ViaOn || ctx->move.more_to_come ||
+	if (!conf_core.editor.auto_via || !PCB->ViaOn || ctx->move.more_to_come ||
 			pcb_layer_get_group_(Layer) ==
 			pcb_layer_get_group_(ctx->move.dst_layer) || !(pcb_layer_flags(PCB, pcb_layer_id(PCB->Data, Layer)) & PCB_LYT_COPPER) || !(pcb_layer_flags_(ctx->move.dst_layer) & PCB_LYT_COPPER))
-		return (newone);
+		return newone;
 	/* consider via at Point1 */
 	sb.X1 = newone->Point1.X - newone->Thickness / 2;
 	sb.X2 = newone->Point1.X + newone->Thickness / 2;
@@ -690,8 +700,7 @@ void *pcb_lineop_move_to_layer(pcb_opctx_t *ctx, pcb_layer_t * Layer, pcb_line_t
 		if (setjmp(info.env) == 0)
 			pcb_r_search(Layer->line_tree, &sb, NULL, moveline_callback, &info, NULL);
 	}
-	pcb_draw();
-	return (newone);
+	return newone;
 }
 
 /* destroys a line from a layer */
@@ -754,24 +763,23 @@ void *pcb_lineop_remove_point(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_line_t *
 void *pcb_lineop_remove(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_line_t *Line)
 {
 	/* erase from screen */
-	if (Layer->meta.real.vis) {
+	if (Layer->meta.real.vis)
 		pcb_line_invalidate_erase(Line);
-		if (!ctx->remove.bulk)
-			pcb_draw();
-	}
 	pcb_undo_move_obj_to_remove(PCB_TYPE_LINE, Layer, Line, Line);
 	return NULL;
 }
 
 void *pcb_line_destroy(pcb_layer_t *Layer, pcb_line_t *Line)
 {
+	void *res;
 	pcb_opctx_t ctx;
 
 	ctx.remove.pcb = PCB;
-	ctx.remove.bulk = pcb_false;
 	ctx.remove.destroy_target = NULL;
 
-	return pcb_lineop_remove(&ctx, Layer, Line);
+	res = pcb_lineop_remove(&ctx, Layer, Line);
+	pcb_draw();
+	return res;
 }
 
 /* rotates a line in 90 degree steps */
@@ -806,7 +814,7 @@ void pcb_line_rotate(pcb_layer_t *layer, pcb_line_t *line, pcb_coord_t X, pcb_co
 	pcb_rotate(&line->Point1.X, &line->Point1.Y, X, Y, cosa, sina);
 	pcb_rotate(&line->Point2.X, &line->Point2.Y, X, Y, cosa, sina);
 	pcb_line_bbox(line);
-	pcb_r_insert_entry(layer->line_tree, (pcb_box_t *) line, 0);
+	pcb_r_insert_entry(layer->line_tree, (pcb_box_t *) line);
 }
 
 void pcb_line_mirror(pcb_layer_t *layer, pcb_line_t *line, pcb_coord_t y_offs)
@@ -819,7 +827,7 @@ void pcb_line_mirror(pcb_layer_t *layer, pcb_line_t *line, pcb_coord_t y_offs)
 	line->Point2.Y = PCB_SWAP_Y(line->Point2.Y) + y_offs;
 	pcb_line_bbox(line);
 	if (layer->line_tree != NULL)
-		pcb_r_insert_entry(layer->line_tree, (pcb_box_t *) line, 0);
+		pcb_r_insert_entry(layer->line_tree, (pcb_box_t *) line);
 }
 
 void pcb_line_flip_side(pcb_layer_t *layer, pcb_line_t *line)
@@ -830,7 +838,7 @@ void pcb_line_flip_side(pcb_layer_t *layer, pcb_line_t *line)
 	line->Point2.X = PCB_SWAP_X(line->Point2.X);
 	line->Point2.Y = PCB_SWAP_Y(line->Point2.Y);
 	pcb_line_bbox(line);
-	pcb_r_insert_entry(layer->line_tree, (pcb_box_t *) line, 0);
+	pcb_r_insert_entry(layer->line_tree, (pcb_box_t *) line);
 }
 
 static void rotate_line1(pcb_layer_t *Layer, pcb_line_t *Line)
@@ -851,16 +859,15 @@ static void rotate_line2(pcb_layer_t *Layer, pcb_line_t *Line)
 	pcb_line_bbox(Line);
 	if (Layer) {
 		if (Layer->line_tree != NULL)
-			pcb_r_insert_entry(Layer->line_tree, (pcb_box_t *) Line, 0);
+			pcb_r_insert_entry(Layer->line_tree, (pcb_box_t *) Line);
 		if (!Layer->is_bound)
 			pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_LINE, Layer, Line);
 		pcb_line_invalidate_draw(Layer, Line);
 	}
 	else {
-		pcb_r_insert_entry(PCB->Data->rat_tree, (pcb_box_t *) Line, 0);
+		pcb_r_insert_entry(PCB->Data->rat_tree, (pcb_box_t *) Line);
 		pcb_rat_invalidate_draw((pcb_rat_t *) Line);
 	}
-	pcb_draw();
 }
 
 /* rotates a line's point */
@@ -898,7 +905,7 @@ void *pcb_lineop_insert_point(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_line_t *
 
 	if (((Line->Point1.X == ctx->insert.x) && (Line->Point1.Y == ctx->insert.y)) ||
 			((Line->Point2.X == ctx->insert.x) && (Line->Point2.Y == ctx->insert.y)))
-		return (NULL);
+		return NULL;
 	X = Line->Point2.X;
 	Y = Line->Point2.Y;
 	pcb_undo_add_obj_to_move(PCB_TYPE_LINE_POINT, Layer, Line, &Line->Point2, ctx->insert.x - X, ctx->insert.y - Y);
@@ -908,7 +915,7 @@ void *pcb_lineop_insert_point(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_line_t *
 	Line->Point2.X = ctx->insert.x;
 	Line->Point2.Y = ctx->insert.y;
 	pcb_line_bbox(Line);
-	pcb_r_insert_entry(Layer->line_tree, (pcb_box_t *) Line, 0);
+	pcb_r_insert_entry(Layer->line_tree, (pcb_box_t *) Line);
 	pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_LINE, Layer, Line);
 	pcb_line_invalidate_draw(Layer, Line);
 	/* we must create after playing with Line since creation may
@@ -921,8 +928,7 @@ void *pcb_lineop_insert_point(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_line_t *
 		pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_LINE, Layer, line);
 		/* creation call adds it to the rtree */
 	}
-	pcb_draw();
-	return (line);
+	return line;
 }
 
 void *pcb_lineop_change_flag(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_line_t *Line)
@@ -1031,8 +1037,12 @@ static void pcb_line_draw(pcb_layer_t *layer, pcb_line_t *line, int allow_term_g
 	if (PCB_FLAG_TEST(PCB_FLAG_WARN, line))
 		color = conf_core.appearance.color.warn;
 	else if (PCB_FLAG_TEST(PCB_FLAG_SELECTED | PCB_FLAG_FOUND, line)) {
-		if (PCB_FLAG_TEST(PCB_FLAG_SELECTED, line))
-			color = layer->meta.real.selected_color;
+		if (PCB_FLAG_TEST(PCB_FLAG_SELECTED, line)) {
+			if (layer->is_bound)
+				PCB_OBJ_COLOR_ON_BOUND_LAYER(color, layer, 1);
+			else
+				color = layer->meta.real.selected_color;
+		}
 		else
 			color = conf_core.appearance.color.connected;
 	}
@@ -1040,7 +1050,7 @@ static void pcb_line_draw(pcb_layer_t *layer, pcb_line_t *line, int allow_term_g
 		color = (line->override_color);
 	}
 	else if (layer->is_bound)
-		PCB_OBJ_COLOR_ON_BOUND_LAYER(color, layer);
+		PCB_OBJ_COLOR_ON_BOUND_LAYER(color, layer, 0);
 	else
 		color = layer->meta.real.color;
 
@@ -1058,6 +1068,9 @@ pcb_r_dir_t pcb_line_draw_callback(const pcb_box_t * b, void *cl)
 {
 	pcb_line_t *line = (pcb_line_t *)b;
 
+	if (pcb_hidden_floater((pcb_any_obj_t*)b))
+		return PCB_R_DIR_FOUND_CONTINUE;
+
 	if (!PCB->SubcPartsOn && pcb_lobj_parent_subc(line->parent_type, &line->parent))
 		return PCB_R_DIR_NOT_FOUND;
 
