@@ -1,7 +1,8 @@
 /*
  *                            COPYRIGHT
  *
- *  PCB, interactive printed circuit board design
+ *  pcb-rnd, interactive printed circuit board design
+ *  (this file is based on PCB, interactive printed circuit board design)
  *  Copyright (C) 1994,1995,1996 Thomas Nau
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -18,9 +19,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- *  Contact addresses for paper mail and Email:
- *  Thomas Nau, Schlehenweg 15, 88471 Baustetten, Germany
- *  Thomas.Nau@rz.uni-ulm.de
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  *
  */
 
@@ -38,7 +40,7 @@
 #include "obj_pad.h"
 #include "obj_pad_list.h"
 #include "obj_pad_op.h"
-
+#include "obj_hash.h"
 
 /* TODO: remove this if draw.[ch] pads are moved */
 #include "draw.h"
@@ -99,7 +101,7 @@ pcb_pad_t *pcb_element_pad_new(pcb_element_t *Element, pcb_coord_t X1, pcb_coord
 	PCB_FLAG_CLEAR(PCB_FLAG_WARN, pad);
 	pad->ID = pcb_create_ID_get();
 	pad->Element = Element;
-	return (pad);
+	return pad;
 }
 
 pcb_pad_t *pcb_element_pad_new_rect(pcb_element_t *Element, pcb_coord_t X1, pcb_coord_t Y1, pcb_coord_t X2, pcb_coord_t Y2, pcb_coord_t Clearance, pcb_coord_t Mask, const char *Name, const char *Number, pcb_flag_t Flags)
@@ -211,13 +213,12 @@ void pcb_pad_copper_bbox(pcb_box_t *out, pcb_pad_t *Pad)
 pcb_bool pcb_pad_change_paste(pcb_pad_t *Pad)
 {
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, Pad))
-		return (pcb_false);
+		return pcb_false;
 	pcb_pad_invalidate_erase(Pad);
 	pcb_undo_add_obj_to_flag(Pad);
 	PCB_FLAG_TOGGLE(PCB_FLAG_NOPASTE, Pad);
 	pcb_pad_invalidate_draw(Pad);
-	pcb_draw();
-	return (pcb_true);
+	return pcb_true;
 }
 
 int pcb_pad_eq(const pcb_element_t *e1, const pcb_pad_t *p1, const pcb_element_t *e2, const pcb_pad_t *p2)
@@ -268,7 +269,7 @@ void *pcb_padop_change_size(pcb_opctx_t *ctx, pcb_element_t *Element, pcb_pad_t
 	pcb_coord_t value = (ctx->chgsize.is_absolute) ? ctx->chgsize.value : Pad->Thickness + ctx->chgsize.value;
 
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, Pad))
-		return (NULL);
+		return NULL;
 	if (value <= PCB_MAX_PADSIZE && value >= PCB_MIN_PADSIZE && value != Pad->Thickness) {
 		pcb_undo_add_obj_to_size(PCB_TYPE_PAD, Element, Pad, Pad);
 		pcb_undo_add_obj_to_mask_size(PCB_TYPE_PAD, Element, Pad, Pad);
@@ -281,9 +282,9 @@ void *pcb_padop_change_size(pcb_opctx_t *ctx, pcb_element_t *Element, pcb_pad_t
 		pcb_element_bbox(PCB->Data, Element, pcb_font(PCB, 0, 1));
 		pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_PAD, Element, Pad);
 		pcb_pad_invalidate_draw(Pad);
-		return (Pad);
+		return Pad;
 	}
-	return (NULL);
+	return NULL;
 }
 
 /* changes the clearance size of a pad */
@@ -292,7 +293,7 @@ void *pcb_padop_change_clear_size(pcb_opctx_t *ctx, pcb_element_t *Element, pcb_
 	pcb_coord_t value = (ctx->chgsize.is_absolute) ? ctx->chgsize.value : Pad->Clearance + ctx->chgsize.value;
 
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, Pad))
-		return (NULL);
+		return NULL;
 	if (value < 0)
 		value = 0;
 	else
@@ -328,7 +329,7 @@ void *pcb_padop_change_name(pcb_opctx_t *ctx, pcb_element_t *Element, pcb_pad_t
 	}
 	else
 		Pad->Name = ctx->chgname.new_name;
-	return (old);
+	return old;
 }
 
 /* changes the number of a pad */
@@ -344,14 +345,14 @@ void *pcb_padop_change_num(pcb_opctx_t *ctx, pcb_element_t *Element, pcb_pad_t *
 	}
 	else
 		Pad->Number = ctx->chgname.new_name;
-	return (old);
+	return old;
 }
 
 /* changes the square flag of a pad */
 void *pcb_padop_change_square(pcb_opctx_t *ctx, pcb_element_t *Element, pcb_pad_t *Pad)
 {
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, Pad))
-		return (NULL);
+		return NULL;
 	pcb_pad_invalidate_erase(Pad);
 	pcb_undo_add_obj_to_clear_poly(PCB_TYPE_PAD, Element, Pad, Pad, pcb_false);
 	pcb_poly_restore_to_poly(PCB->Data, PCB_TYPE_PAD, Element, Pad);
@@ -360,7 +361,7 @@ void *pcb_padop_change_square(pcb_opctx_t *ctx, pcb_element_t *Element, pcb_pad_
 	pcb_undo_add_obj_to_clear_poly(PCB_TYPE_PAD, Element, Pad, Pad, pcb_true);
 	pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_PAD, Element, Pad);
 	pcb_pad_invalidate_draw(Pad);
-	return (Pad);
+	return Pad;
 }
 
 /* sets the square flag of a pad */
@@ -368,7 +369,7 @@ void *pcb_padop_set_square(pcb_opctx_t *ctx, pcb_element_t *Element, pcb_pad_t *
 {
 
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, Pad) || PCB_FLAG_TEST(PCB_FLAG_SQUARE, Pad))
-		return (NULL);
+		return NULL;
 
 	return (pcb_padop_change_square(ctx, Element, Pad));
 }
@@ -379,7 +380,7 @@ void *pcb_padop_clear_square(pcb_opctx_t *ctx, pcb_element_t *Element, pcb_pad_t
 {
 
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, Pad) || !PCB_FLAG_TEST(PCB_FLAG_SQUARE, Pad))
-		return (NULL);
+		return NULL;
 
 	return (pcb_padop_change_square(ctx, Element, Pad));
 }
@@ -399,9 +400,9 @@ void *pcb_padop_change_mask_size(pcb_opctx_t *ctx, pcb_element_t *Element, pcb_p
 		Pad->Mask = value;
 		pcb_element_bbox(PCB->Data, Element, pcb_font(PCB, 0, 1));
 		pcb_pad_invalidate_draw(Pad);
-		return (Pad);
+		return Pad;
 	}
-	return (NULL);
+	return NULL;
 }
 
 void *pcb_padop_change_flag(pcb_opctx_t *ctx, pcb_element_t *elem, pcb_pad_t *pad)
