@@ -1,7 +1,8 @@
 /*
  *                            COPYRIGHT
  *
- *  PCB, interactive printed circuit board design
+ *  pcb-rnd, interactive printed circuit board design
+ *  (this file is based on PCB, interactive printed circuit board design)
  *  Copyright (C) 1994,1995,1996 Thomas Nau
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -18,9 +19,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- *  Contact addresses for paper mail and Email:
- *  Thomas Nau, Schlehenweg 15, 88471 Baustetten, Germany
- *  Thomas.Nau@rz.uni-ulm.de
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  *
  */
 
@@ -40,6 +42,7 @@
 #include "obj_pinvia.h"
 #include "obj_pinvia_op.h"
 #include "obj_subc_parent.h"
+#include "obj_hash.h"
 
 /* TODO: consider removing this by moving pin/via functions here: */
 #include "draw.h"
@@ -103,9 +106,9 @@ pcb_pin_t *pcb_via_new(pcb_data_t *Data, pcb_coord_t X, pcb_coord_t Y, pcb_coord
 		PCB_VIA_LOOP(Data);
 		{
 			if (pcb_distance(X, Y, via->X, via->Y) <= via->DrillingHole / 2 + DrillingHole / 2) {
-				pcb_message(PCB_MSG_WARNING, _("%m+Dropping via at %$mD because it's hole would overlap with the via "
+				pcb_message(PCB_MSG_WARNING, _("%m+Dropping via at %$mD because its hole would overlap with the via "
 									"at %$mD\n"), conf_core.editor.grid_unit->allow, X, Y, via->X, via->Y);
-				return (NULL);					/* don't allow via stacking */
+				return NULL;					/* don't allow via stacking */
 			}
 		}
 		PCB_END_LOOP;
@@ -114,7 +117,7 @@ pcb_pin_t *pcb_via_new(pcb_data_t *Data, pcb_coord_t X, pcb_coord_t Y, pcb_coord
 	Via = pcb_via_alloc(Data);
 
 	if (!Via)
-		return (Via);
+		return Via;
 	/* copy values */
 	Via->X = X;
 	Via->Y = Y;
@@ -144,7 +147,7 @@ pcb_pin_t *pcb_via_new(pcb_data_t *Data, pcb_coord_t X, pcb_coord_t Y, pcb_coord
 	}
 
 	pcb_add_via(Data, Via);
-	return (Via);
+	return Via;
 }
 
 static pcb_pin_t *pcb_via_copy_meta(pcb_pin_t *dst, pcb_pin_t *src)
@@ -224,7 +227,7 @@ pcb_pin_t *pcb_element_pin_new(pcb_element_t *Element, pcb_coord_t X, pcb_coord_
 						conf_core.editor.grid_unit->allow, pin->DrillingHole, DrillingHole);
 	}
 
-	return (pin);
+	return pin;
 }
 
 
@@ -233,8 +236,8 @@ void pcb_add_via(pcb_data_t *Data, pcb_pin_t *Via)
 {
 	pcb_pin_bbox(Via);
 	if (!Data->via_tree)
-		Data->via_tree = pcb_r_create_tree(NULL, 0, 0);
-	pcb_r_insert_entry(Data->via_tree, (pcb_box_t *) Via, 0);
+		Data->via_tree = pcb_r_create_tree();
+	pcb_r_insert_entry(Data->via_tree, (pcb_box_t *) Via);
 	PCB_SET_PARENT(Via, data, Data);
 }
 
@@ -295,7 +298,7 @@ void pcb_via_rotate(pcb_data_t *Data, pcb_pin_t *Via, pcb_coord_t X, pcb_coord_t
 	pcb_r_delete_entry(Data->via_tree, (pcb_box_t *) Via);
 	pcb_rotate(&Via->X, &Via->Y, X, Y, cosa, sina);
 	pcb_pin_bbox(Via);
-	pcb_r_insert_entry(Data->via_tree, (pcb_box_t *) Via, 0);
+	pcb_r_insert_entry(Data->via_tree, (pcb_box_t *) Via);
 }
 
 void pcb_via_mirror(pcb_data_t *Data, pcb_pin_t *via, pcb_coord_t y_offs)
@@ -306,7 +309,7 @@ void pcb_via_mirror(pcb_data_t *Data, pcb_pin_t *via, pcb_coord_t y_offs)
 	via->Y = PCB_SWAP_Y(via->Y) + y_offs;
 	pcb_pin_bbox(via);
 	if (Data->via_tree != NULL)
-		pcb_r_insert_entry(Data->via_tree, (pcb_box_t *) via, 0);
+		pcb_r_insert_entry(Data->via_tree, (pcb_box_t *) via);
 }
 
 void pcb_via_flip_side(pcb_data_t *Data, pcb_pin_t *via)
@@ -315,7 +318,7 @@ void pcb_via_flip_side(pcb_data_t *Data, pcb_pin_t *via)
 	via->X = PCB_SWAP_X(via->X);
 	via->Y = PCB_SWAP_Y(via->Y);
 	pcb_pin_bbox(via);
-	pcb_r_insert_entry(Data->via_tree, (pcb_box_t *) via, 0);
+	pcb_r_insert_entry(Data->via_tree, (pcb_box_t *) via);
 }
 
 int pcb_pin_eq(const pcb_element_t *e1, const pcb_pin_t *p1, const pcb_element_t *e2, const pcb_pin_t *p2)
@@ -359,8 +362,8 @@ void *pcb_viaop_add_to_buffer(pcb_opctx_t *ctx, pcb_pin_t *Via)
 	return pcb_via_copy_meta(v, Via);
 }
 
-/* moves a via to paste buffer without allocating memory for the name */
-void *pcb_viaop_move_to_buffer(pcb_opctx_t *ctx, pcb_pin_t * via)
+/* moves a via beteen board and buffer without allocating memory for the name */
+void *pcb_viaop_move_buffer(pcb_opctx_t *ctx, pcb_pin_t * via)
 {
 	pcb_poly_restore_to_poly(ctx->buffer.src, PCB_TYPE_VIA, via, via);
 
@@ -371,8 +374,8 @@ void *pcb_viaop_move_to_buffer(pcb_opctx_t *ctx, pcb_pin_t * via)
 	PCB_FLAG_CLEAR(PCB_FLAG_WARN | PCB_FLAG_FOUND, via);
 
 	if (!ctx->buffer.dst->via_tree)
-		ctx->buffer.dst->via_tree = pcb_r_create_tree(NULL, 0, 0);
-	pcb_r_insert_entry(ctx->buffer.dst->via_tree, (pcb_box_t *) via, 0);
+		ctx->buffer.dst->via_tree = pcb_r_create_tree();
+	pcb_r_insert_entry(ctx->buffer.dst->via_tree, (pcb_box_t *) via);
 	pcb_poly_clear_from_poly(ctx->buffer.dst, PCB_TYPE_VIA, via, via);
 	PCB_SET_PARENT(via, data, ctx->buffer.dst);
 	return via;
@@ -416,7 +419,7 @@ void *pcb_viaop_change_size(pcb_opctx_t *ctx, pcb_pin_t *Via)
 	pcb_coord_t value = ctx->chgsize.is_absolute ? ctx->chgsize.value : Via->Thickness + ctx->chgsize.value;
 
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, Via))
-		return (NULL);
+		return NULL;
 	if (!PCB_FLAG_TEST(PCB_FLAG_HOLE, Via) && value <= PCB_MAX_PINORVIASIZE &&
 			value >= PCB_MIN_PINORVIASIZE && value >= Via->DrillingHole + PCB_MIN_PINORVIACOPPER && value != Via->Thickness) {
 		pcb_undo_add_obj_to_size(PCB_TYPE_VIA, Via, Via, Via);
@@ -429,12 +432,12 @@ void *pcb_viaop_change_size(pcb_opctx_t *ctx, pcb_pin_t *Via)
 		}
 		Via->Thickness = value;
 		pcb_pin_bbox(Via);
-		pcb_r_insert_entry(PCB->Data->via_tree, (pcb_box_t *) Via, 0);
+		pcb_r_insert_entry(PCB->Data->via_tree, (pcb_box_t *) Via);
 		pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_VIA, Via, Via);
 		pcb_via_invalidate_draw(Via);
-		return (Via);
+		return Via;
 	}
-	return (NULL);
+	return NULL;
 }
 
 /* changes the drilling hole of a via */
@@ -443,7 +446,7 @@ void *pcb_viaop_change_2nd_size(pcb_opctx_t *ctx, pcb_pin_t *Via)
 	pcb_coord_t value = (ctx->chgsize.is_absolute) ? ctx->chgsize.value : Via->DrillingHole + ctx->chgsize.value;
 
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, Via))
-		return (NULL);
+		return NULL;
 	if (value <= PCB_MAX_PINORVIASIZE &&
 			value >= PCB_MIN_PINORVIAHOLE && (PCB_FLAG_TEST(PCB_FLAG_HOLE, Via) || value <= Via->Thickness - PCB_MIN_PINORVIACOPPER)
 			&& value != Via->DrillingHole) {
@@ -457,9 +460,9 @@ void *pcb_viaop_change_2nd_size(pcb_opctx_t *ctx, pcb_pin_t *Via)
 		}
 		pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_VIA, Via, Via);
 		pcb_via_invalidate_draw(Via);
-		return (Via);
+		return Via;
 	}
-	return (NULL);
+	return NULL;
 }
 
 /* changes the drilling hole of a pin */
@@ -468,7 +471,7 @@ void *pcb_pinop_change_2nd_size(pcb_opctx_t *ctx, pcb_element_t *Element, pcb_pi
 	pcb_coord_t value = (ctx->chgsize.is_absolute) ? ctx->chgsize.value : Pin->DrillingHole + ctx->chgsize.value;
 
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, Pin))
-		return (NULL);
+		return NULL;
 	if (value <= PCB_MAX_PINORVIASIZE &&
 			value >= PCB_MIN_PINORVIAHOLE && (PCB_FLAG_TEST(PCB_FLAG_HOLE, Pin) || value <= Pin->Thickness - PCB_MIN_PINORVIACOPPER)
 			&& value != Pin->DrillingHole) {
@@ -482,9 +485,9 @@ void *pcb_pinop_change_2nd_size(pcb_opctx_t *ctx, pcb_element_t *Element, pcb_pi
 		}
 		pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_PIN, Element, Pin);
 		pcb_pin_invalidate_draw(Pin);
-		return (Pin);
+		return Pin;
 	}
-	return (NULL);
+	return NULL;
 }
 
 
@@ -494,7 +497,7 @@ void *pcb_viaop_change_clear_size(pcb_opctx_t *ctx, pcb_pin_t *Via)
 	pcb_coord_t value = (ctx->chgsize.is_absolute) ? ctx->chgsize.value : Via->Clearance + ctx->chgsize.value;
 
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, Via))
-		return (NULL);
+		return NULL;
 	if (value < 0)
 		value = 0;
 	value = MIN(PCB_MAX_LINESIZE, value);
@@ -510,11 +513,11 @@ void *pcb_viaop_change_clear_size(pcb_opctx_t *ctx, pcb_pin_t *Via)
 	pcb_r_delete_entry(PCB->Data->via_tree, (pcb_box_t *) Via);
 	Via->Clearance = value;
 	pcb_pin_bbox(Via);
-	pcb_r_insert_entry(PCB->Data->via_tree, (pcb_box_t *) Via, 0);
+	pcb_r_insert_entry(PCB->Data->via_tree, (pcb_box_t *) Via);
 	pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_VIA, Via, Via);
 	pcb_via_invalidate_draw(Via);
 	Via->Element = NULL;
-	return (Via);
+	return Via;
 }
 
 
@@ -524,7 +527,7 @@ void *pcb_pinop_change_size(pcb_opctx_t *ctx, pcb_element_t *Element, pcb_pin_t
 	pcb_coord_t value = (ctx->chgsize.is_absolute) ? ctx->chgsize.value : Pin->Thickness + ctx->chgsize.value;
 
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, Pin))
-		return (NULL);
+		return NULL;
 	if (!PCB_FLAG_TEST(PCB_FLAG_HOLE, Pin) && value <= PCB_MAX_PINORVIASIZE &&
 			value >= PCB_MIN_PINORVIASIZE && value >= Pin->DrillingHole + PCB_MIN_PINORVIACOPPER && value != Pin->Thickness) {
 		pcb_undo_add_obj_to_size(PCB_TYPE_PIN, Element, Pin, Pin);
@@ -538,9 +541,9 @@ void *pcb_pinop_change_size(pcb_opctx_t *ctx, pcb_element_t *Element, pcb_pin_t
 		pcb_element_bbox(PCB->Data, Element, pcb_font(PCB, 0, 1));
 		pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_PIN, Element, Pin);
 		pcb_pin_invalidate_draw(Pin);
-		return (Pin);
+		return Pin;
 	}
-	return (NULL);
+	return NULL;
 }
 
 /* changes the clearance size of a pin */
@@ -549,7 +552,7 @@ void *pcb_pinop_change_clear_size(pcb_opctx_t *ctx, pcb_element_t *Element, pcb_
 	pcb_coord_t value = (ctx->chgsize.is_absolute) ? ctx->chgsize.value : Pin->Clearance + ctx->chgsize.value;
 
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, Pin))
-		return (NULL);
+		return NULL;
 	if (value < 0)
 		value = 0;
 	value = MIN(PCB_MAX_LINESIZE, value);
@@ -568,7 +571,7 @@ void *pcb_pinop_change_clear_size(pcb_opctx_t *ctx, pcb_element_t *Element, pcb_
 	pcb_element_bbox(PCB->Data, Element, pcb_font(PCB, 0, 1));
 	pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_PIN, Element, Pin);
 	pcb_pin_invalidate_draw(Pin);
-	return (Pin);
+	return Pin;
 }
 
 /* changes the name of a via */
@@ -583,7 +586,7 @@ void *pcb_viaop_change_name(pcb_opctx_t *ctx, pcb_pin_t *Via)
 	}
 	else
 		Via->Name = ctx->chgname.new_name;
-	return (old);
+	return old;
 }
 
 /* changes the name of a pin */
@@ -599,7 +602,7 @@ void *pcb_pinop_change_name(pcb_opctx_t *ctx, pcb_element_t *Element, pcb_pin_t
 	}
 	else
 		Pin->Name = ctx->chgname.new_name;
-	return (old);
+	return old;
 }
 
 /* changes the number of a pin */
@@ -615,7 +618,7 @@ void *pcb_pinop_change_num(pcb_opctx_t *ctx, pcb_element_t *Element, pcb_pin_t *
 	}
 	else
 		Pin->Number = ctx->chgname.new_name;
-	return (old);
+	return old;
 }
 
 
@@ -623,7 +626,7 @@ void *pcb_pinop_change_num(pcb_opctx_t *ctx, pcb_element_t *Element, pcb_pin_t *
 void *pcb_viaop_change_square(pcb_opctx_t *ctx, pcb_pin_t *Via)
 {
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, Via))
-		return (NULL);
+		return NULL;
 	pcb_via_invalidate_erase(Via);
 	pcb_undo_add_obj_to_clear_poly(PCB_TYPE_VIA, NULL, Via, Via, pcb_false);
 	pcb_poly_restore_to_poly(PCB->Data, PCB_TYPE_VIA, NULL, Via);
@@ -637,14 +640,14 @@ void *pcb_viaop_change_square(pcb_opctx_t *ctx, pcb_pin_t *Via)
 	pcb_undo_add_obj_to_clear_poly(PCB_TYPE_VIA, NULL, Via, Via, pcb_true);
 	pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_VIA, NULL, Via);
 	pcb_via_invalidate_draw(Via);
-	return (Via);
+	return Via;
 }
 
 /* changes the square flag of a pin */
 void *pcb_pinop_change_square(pcb_opctx_t *ctx, pcb_element_t *Element, pcb_pin_t *Pin)
 {
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, Pin))
-		return (NULL);
+		return NULL;
 	pcb_pin_invalidate_erase(Pin);
 	pcb_undo_add_obj_to_clear_poly(PCB_TYPE_PIN, Element, Pin, Pin, pcb_false);
 	pcb_poly_restore_to_poly(PCB->Data, PCB_TYPE_PIN, Element, Pin);
@@ -658,14 +661,14 @@ void *pcb_pinop_change_square(pcb_opctx_t *ctx, pcb_element_t *Element, pcb_pin_
 	pcb_undo_add_obj_to_clear_poly(PCB_TYPE_PIN, Element, Pin, Pin, pcb_true);
 	pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_PIN, Element, Pin);
 	pcb_pin_invalidate_draw(Pin);
-	return (Pin);
+	return Pin;
 }
 
 /* sets the square flag of a pin */
 void *pcb_pinop_set_square(pcb_opctx_t *ctx, pcb_element_t *Element, pcb_pin_t *Pin)
 {
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, Pin) || PCB_FLAG_TEST(PCB_FLAG_SQUARE, Pin))
-		return (NULL);
+		return NULL;
 
 	return (pcb_pinop_change_square(ctx, Element, Pin));
 }
@@ -674,7 +677,7 @@ void *pcb_pinop_set_square(pcb_opctx_t *ctx, pcb_element_t *Element, pcb_pin_t *
 void *pcb_pinop_clear_square(pcb_opctx_t *ctx, pcb_element_t *Element, pcb_pin_t *Pin)
 {
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, Pin) || !PCB_FLAG_TEST(PCB_FLAG_SQUARE, Pin))
-		return (NULL);
+		return NULL;
 
 	return (pcb_pinop_change_square(ctx, Element, Pin));
 }
@@ -683,7 +686,7 @@ void *pcb_pinop_clear_square(pcb_opctx_t *ctx, pcb_element_t *Element, pcb_pin_t
 void *pcb_viaop_change_octagon(pcb_opctx_t *ctx, pcb_pin_t *Via)
 {
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, Via))
-		return (NULL);
+		return NULL;
 	pcb_via_invalidate_erase(Via);
 	pcb_undo_add_obj_to_clear_poly(PCB_TYPE_VIA, Via, Via, Via, pcb_false);
 	pcb_poly_restore_to_poly(PCB->Data, PCB_TYPE_VIA, Via, Via);
@@ -692,14 +695,14 @@ void *pcb_viaop_change_octagon(pcb_opctx_t *ctx, pcb_pin_t *Via)
 	pcb_undo_add_obj_to_clear_poly(PCB_TYPE_VIA, Via, Via, Via, pcb_true);
 	pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_VIA, Via, Via);
 	pcb_via_invalidate_draw(Via);
-	return (Via);
+	return Via;
 }
 
 /* sets the octagon flag of a via */
 void *pcb_viaop_set_octagon(pcb_opctx_t *ctx, pcb_pin_t *Via)
 {
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, Via) || PCB_FLAG_TEST(PCB_FLAG_OCTAGON, Via))
-		return (NULL);
+		return NULL;
 
 	return (pcb_viaop_change_octagon(ctx, Via));
 }
@@ -708,7 +711,7 @@ void *pcb_viaop_set_octagon(pcb_opctx_t *ctx, pcb_pin_t *Via)
 void *pcb_viaop_clear_octagon(pcb_opctx_t *ctx, pcb_pin_t *Via)
 {
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, Via) || !PCB_FLAG_TEST(PCB_FLAG_OCTAGON, Via))
-		return (NULL);
+		return NULL;
 
 	return (pcb_viaop_change_octagon(ctx, Via));
 }
@@ -717,7 +720,7 @@ void *pcb_viaop_clear_octagon(pcb_opctx_t *ctx, pcb_pin_t *Via)
 void *pcb_pinop_change_octagon(pcb_opctx_t *ctx, pcb_element_t *Element, pcb_pin_t *Pin)
 {
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, Pin))
-		return (NULL);
+		return NULL;
 	pcb_pin_invalidate_erase(Pin);
 	pcb_undo_add_obj_to_clear_poly(PCB_TYPE_PIN, Element, Pin, Pin, pcb_false);
 	pcb_poly_restore_to_poly(PCB->Data, PCB_TYPE_PIN, Element, Pin);
@@ -726,14 +729,14 @@ void *pcb_pinop_change_octagon(pcb_opctx_t *ctx, pcb_element_t *Element, pcb_pin
 	pcb_undo_add_obj_to_clear_poly(PCB_TYPE_PIN, Element, Pin, Pin, pcb_true);
 	pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_PIN, Element, Pin);
 	pcb_pin_invalidate_draw(Pin);
-	return (Pin);
+	return Pin;
 }
 
 /* sets the octagon flag of a pin */
 void *pcb_pinop_set_octagon(pcb_opctx_t *ctx, pcb_element_t *Element, pcb_pin_t *Pin)
 {
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, Pin) || PCB_FLAG_TEST(PCB_FLAG_OCTAGON, Pin))
-		return (NULL);
+		return NULL;
 
 	return (pcb_pinop_change_octagon(ctx, Element, Pin));
 }
@@ -742,7 +745,7 @@ void *pcb_pinop_set_octagon(pcb_opctx_t *ctx, pcb_element_t *Element, pcb_pin_t
 void *pcb_pinop_clear_octagon(pcb_opctx_t *ctx, pcb_element_t *Element, pcb_pin_t *Pin)
 {
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, Pin) || !PCB_FLAG_TEST(PCB_FLAG_OCTAGON, Pin))
-		return (NULL);
+		return NULL;
 
 	return (pcb_pinop_change_octagon(ctx, Element, Pin));
 }
@@ -751,7 +754,7 @@ void *pcb_pinop_clear_octagon(pcb_opctx_t *ctx, pcb_element_t *Element, pcb_pin_
 pcb_bool pcb_pin_change_hole(pcb_pin_t *Via)
 {
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, Via))
-		return (pcb_false);
+		return pcb_false;
 	pcb_via_invalidate_erase(Via);
 	pcb_undo_add_obj_to_flag(Via);
 	pcb_undo_add_obj_to_mask_size(PCB_TYPE_VIA, Via, Via, Via);
@@ -774,11 +777,11 @@ pcb_bool pcb_pin_change_hole(pcb_pin_t *Via)
 	}
 
 	pcb_pin_bbox(Via);
-	pcb_r_insert_entry(PCB->Data->via_tree, (pcb_box_t *) Via, 0);
+	pcb_r_insert_entry(PCB->Data->via_tree, (pcb_box_t *) Via);
 	pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_VIA, Via, Via);
 	pcb_via_invalidate_draw(Via);
 	pcb_draw();
-	return (pcb_true);
+	return pcb_true;
 }
 
 /* changes the mask size of a pin */
@@ -796,9 +799,9 @@ void *pcb_pinop_change_mask_size(pcb_opctx_t *ctx, pcb_element_t *Element, pcb_p
 		Pin->Mask = value;
 		pcb_element_bbox(PCB->Data, Element, pcb_font(PCB, 0, 1));
 		pcb_pin_invalidate_draw(Pin);
-		return (Pin);
+		return Pin;
 	}
-	return (NULL);
+	return NULL;
 }
 
 /* changes the mask size of a via */
@@ -814,11 +817,11 @@ void *pcb_viaop_change_mask_size(pcb_opctx_t *ctx, pcb_pin_t *Via)
 		pcb_r_delete_entry(PCB->Data->via_tree, &Via->BoundingBox);
 		Via->Mask = value;
 		pcb_pin_bbox(Via);
-		pcb_r_insert_entry(PCB->Data->via_tree, &Via->BoundingBox, 0);
+		pcb_r_insert_entry(PCB->Data->via_tree, &Via->BoundingBox);
 		pcb_via_invalidate_draw(Via);
-		return (Via);
+		return Via;
 	}
-	return (NULL);
+	return NULL;
 }
 
 /* copies a via */
@@ -829,10 +832,10 @@ void *pcb_viaop_copy(pcb_opctx_t *ctx, pcb_pin_t *Via)
 	via = pcb_via_new(PCB->Data, Via->X + ctx->copy.DeltaX, Via->Y + ctx->copy.DeltaY,
 										 Via->Thickness, Via->Clearance, Via->Mask, Via->DrillingHole, Via->Name, pcb_flag_mask(Via->Flags, PCB_FLAG_FOUND));
 	if (!via)
-		return (via);
+		return via;
 	pcb_via_invalidate_draw(via);
 	pcb_undo_add_obj_to_create(PCB_TYPE_VIA, via, via, via);
-	return (via);
+	return via;
 }
 
 /* moves a via */
@@ -841,10 +844,8 @@ void *pcb_viaop_move_noclip(pcb_opctx_t *ctx, pcb_pin_t *Via)
 	if (PCB->ViaOn)
 		pcb_via_invalidate_erase(Via);
 	pcb_via_move(Via, ctx->move.dx, ctx->move.dy);
-	if (PCB->ViaOn) {
+	if (PCB->ViaOn)
 		pcb_via_invalidate_draw(Via);
-		pcb_draw();
-	}
 	return Via;
 }
 
@@ -853,7 +854,7 @@ void *pcb_viaop_move(pcb_opctx_t *ctx, pcb_pin_t *Via)
 	pcb_r_delete_entry(PCB->Data->via_tree, (pcb_box_t *) Via);
 	pcb_poly_restore_to_poly(PCB->Data, PCB_TYPE_VIA, Via, Via);
 	pcb_viaop_move_noclip(ctx, Via);
-	pcb_r_insert_entry(PCB->Data->via_tree, (pcb_box_t *) Via, 0);
+	pcb_r_insert_entry(PCB->Data->via_tree, (pcb_box_t *) Via);
 	pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_VIA, Via, Via);
 	return Via;
 }
@@ -865,7 +866,7 @@ void *pcb_viaop_clip(pcb_opctx_t *ctx, pcb_pin_t *Via)
 		pcb_poly_restore_to_poly(PCB->Data, PCB_TYPE_VIA, Via, Via);
 	}
 	if (ctx->clip.clear) {
-		pcb_r_insert_entry(PCB->Data->via_tree, (pcb_box_t *) Via, 0);
+		pcb_r_insert_entry(PCB->Data->via_tree, (pcb_box_t *) Via);
 		pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_VIA, Via, Via);
 	}
 	return Via;
@@ -886,11 +887,8 @@ void *pcb_viaop_destroy(pcb_opctx_t *ctx, pcb_pin_t *Via)
 void *pcb_viaop_remove(pcb_opctx_t *ctx, pcb_pin_t *Via)
 {
 	/* erase from screen and memory */
-	if (PCB->ViaOn) {
+	if (PCB->ViaOn)
 		pcb_via_invalidate_erase(Via);
-		if (!ctx->remove.bulk)
-			pcb_draw();
-	}
 	pcb_undo_move_obj_to_remove(PCB_TYPE_VIA, Via, Via, Via);
 	return NULL;
 }
@@ -908,7 +906,7 @@ void *pcb_viaop_rotate90(pcb_opctx_t *ctx, pcb_pin_t *via)
 	PCB_VIA_ROTATE90(via, ctx->rotate.center_x, ctx->rotate.center_y, ctx->rotate.number);
 	pcb_pin_bbox(via);
 	if (data->via_tree != NULL) {
-		pcb_r_insert_entry(data->via_tree, (pcb_box_t *)via, 0);
+		pcb_r_insert_entry(data->via_tree, (pcb_box_t *)via);
 		pcb_poly_clear_from_poly(ctx->rotate.pcb->Data, PCB_TYPE_VIA, via, via);
 	}
 	pcb_via_invalidate_draw(via);
@@ -930,7 +928,7 @@ void *pcb_viaop_rotate(pcb_opctx_t *ctx, pcb_pin_t *via)
 
 	pcb_pin_bbox(via);
 	if (data->via_tree != NULL) {
-		pcb_r_insert_entry(data->via_tree, (pcb_box_t *)via, 0);
+		pcb_r_insert_entry(data->via_tree, (pcb_box_t *)via);
 		pcb_poly_clear_from_poly(data, PCB_TYPE_VIA, via, via);
 	}
 	pcb_via_invalidate_draw(via);
@@ -1085,12 +1083,18 @@ void pcb_pin_draw(pcb_pin_t *pin, pcb_bool draw_hole)
 
 pcb_r_dir_t pcb_pin_draw_callback(const pcb_box_t * b, void *cl)
 {
+	if (pcb_hidden_floater((pcb_any_obj_t*)b))
+		return PCB_R_DIR_FOUND_CONTINUE;
+
 	pcb_pin_draw((pcb_pin_t *) b, pcb_false);
 	return PCB_R_DIR_FOUND_CONTINUE;
 }
 
 static void draw_pin_name(pcb_pin_t *pin)
 {
+	if (pcb_hidden_floater((pcb_any_obj_t*)pin))
+		return PCB_R_DIR_FOUND_CONTINUE;
+
 	if (!PCB_FLAG_TEST(PCB_FLAG_HOLE, pin) && PCB_FLAG_TEST(PCB_FLAG_TERMNAME, pin))
 		_draw_pv_name(pin);
 }
@@ -1114,6 +1118,9 @@ pcb_r_dir_t pcb_pin_clear_callback(const pcb_box_t * b, void *cl)
 
 static void draw_via(pcb_pin_t *via, pcb_bool draw_hole)
 {
+	if (pcb_hidden_floater((pcb_any_obj_t*)via))
+		return PCB_R_DIR_FOUND_CONTINUE;
+
 	SetPVColor(via, PCB_TYPE_VIA);
 	_draw_pv(via, draw_hole);
 }
@@ -1122,6 +1129,8 @@ pcb_r_dir_t pcb_via_draw_callback(const pcb_box_t * b, void *cl)
 {
 	pcb_pin_t *via = (pcb_pin_t *)b;
 
+	if (pcb_hidden_floater((pcb_any_obj_t*)b))
+		return PCB_R_DIR_FOUND_CONTINUE;
 	if (PCB->SubcPartsOn || !pcb_gobj_parent_subc(via->parent_type, &via->parent))
 		draw_via(via, pcb_false);
 	return PCB_R_DIR_FOUND_CONTINUE;
