@@ -1,7 +1,8 @@
 /*
  *                            COPYRIGHT
  *
- *  PCB, interactive printed circuit board design
+ *  pcb-rnd, interactive printed circuit board design
+ *  (this file is based on PCB, interactive printed circuit board design)
  *  Copyright (C) 1994,1995,1996 Thomas Nau
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -18,9 +19,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- *  Contact addresses for paper mail and Email:
- *  Thomas Nau, Schlehenweg 15, 88471 Baustetten, Germany
- *  Thomas.Nau@rz.uni-ulm.de
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  *
  */
 
@@ -46,6 +48,7 @@
 #include "obj_poly_draw.h"
 
 #include "obj_subc_parent.h"
+#include "obj_hash.h"
 
 /* TODO: get rid of these: */
 #include "draw.h"
@@ -146,7 +149,7 @@ void pcb_poly_rotate(pcb_layer_t *layer, pcb_poly_t *polygon, pcb_coord_t X, pcb
 	}
 	PCB_END_LOOP;
 	pcb_poly_bbox(polygon);
-	pcb_r_insert_entry(layer->polygon_tree, (pcb_box_t *) polygon, 0);
+	pcb_r_insert_entry(layer->polygon_tree, (pcb_box_t *) polygon);
 }
 
 void pcb_poly_mirror(pcb_layer_t *layer, pcb_poly_t *polygon, pcb_coord_t y_offs)
@@ -161,7 +164,7 @@ void pcb_poly_mirror(pcb_layer_t *layer, pcb_poly_t *polygon, pcb_coord_t y_offs
 	PCB_END_LOOP;
 	pcb_poly_bbox(polygon);
 	if (layer->polygon_tree != NULL)
-		pcb_r_insert_entry(layer->polygon_tree, (pcb_box_t *)polygon, 0);
+		pcb_r_insert_entry(layer->polygon_tree, (pcb_box_t *)polygon);
 }
 
 void pcb_poly_flip_side(pcb_layer_t *layer, pcb_poly_t *polygon)
@@ -174,7 +177,7 @@ void pcb_poly_flip_side(pcb_layer_t *layer, pcb_poly_t *polygon)
 	}
 	PCB_END_LOOP;
 	pcb_poly_bbox(polygon);
-	pcb_r_insert_entry(layer->polygon_tree, (pcb_box_t *) polygon, 0);
+	pcb_r_insert_entry(layer->polygon_tree, (pcb_box_t *) polygon);
 	/* hmmm, how to handle clip */
 }
 
@@ -191,17 +194,62 @@ void pcb_poly_bbox(pcb_poly_t *Polygon)
 		PCB_MAKE_MAX(Polygon->BoundingBox.X2, point->X);
 		PCB_MAKE_MAX(Polygon->BoundingBox.Y2, point->Y);
 	}
+	PCB_END_LOOP;
+
+	/* clearance is generally considered to be part of the bbox for all objects */
+	if (PCB_POLY_HAS_CLEARANCE(Polygon)) {
+		pcb_coord_t clr = Polygon->Clearance/2;
+		Polygon->BoundingBox.X1 -= clr;
+		Polygon->BoundingBox.Y1 -= clr;
+		Polygon->BoundingBox.X2 += clr;
+		Polygon->BoundingBox.Y2 += clr;
+	}
+
 	/* boxes don't include the lower right corner */
 	pcb_close_box(&Polygon->BoundingBox);
-	PCB_END_LOOP;
 }
 
+int pcb_poly_eq(const pcb_host_trans_t *tr1, const pcb_poly_t *p1, const pcb_host_trans_t *tr2, const pcb_poly_t *p2)
+{
+	if (p1->PointN != p2->PointN) return 0;
+	if (pcb_field_neq(p1, p2, Clearance)) return 0;
+	if (pcb_neqs(p1->term, p2->term)) return 0;
+
+	if (!PCB_FLAG_TEST(PCB_FLAG_FLOATER, p1) && !PCB_FLAG_TEST(PCB_FLAG_FLOATER, p2)) {
+		pcb_cardinal_t n;
+		for(n = 0; n < p1->PointN; n++) {
+			pcb_coord_t x1, y1, x2, y2;
+			pcb_hash_tr_coords(tr1, &x1, &y1, p1->Points[n].X, p1->Points[n].Y);
+			pcb_hash_tr_coords(tr2, &x2, &y2, p2->Points[n].X, p2->Points[n].Y);
+			if ((x1 != x2) || (y1 != y2)) return 0;
+		}
+	}
+	return 1;
+}
+
+unsigned int pcb_poly_hash(const pcb_host_trans_t *tr, const pcb_poly_t *p)
+{
+	unsigned int crd = 0;
+	pcb_cardinal_t n;
+
+	if (!PCB_FLAG_TEST(PCB_FLAG_FLOATER, p))
+		for(n = 0; n < p->PointN; n++) {
+			pcb_coord_t x, y;
+
+			pcb_hash_tr_coords(tr, &x, &y, p->Points[n].X, p->Points[n].Y);
+			crd ^= pcb_hash_coord(x) ^ pcb_hash_coord(y);
+		}
+
+	return pcb_hash_coord(p->Clearance) ^ pcb_hash_str(p->term) ^ crd;
+}
+
+
 /* creates a new polygon from the old formats rectangle data */
 pcb_poly_t *pcb_poly_new_from_rectangle(pcb_layer_t *Layer, pcb_coord_t X1, pcb_coord_t Y1, pcb_coord_t X2, pcb_coord_t Y2, pcb_coord_t Clearance, pcb_flag_t Flags)
 {
 	pcb_poly_t *polygon = pcb_poly_new(Layer, Clearance, Flags);
 	if (!polygon)
-		return (polygon);
+		return polygon;
 
 	pcb_poly_point_new(polygon, X1, Y1);
 	pcb_poly_point_new(polygon, X2, Y1);
@@ -209,7 +257,7 @@ pcb_poly_t *pcb_poly_new_from_rectangle(pcb_layer_t *Layer, pcb_coord_t X1, pcb_
 	pcb_poly_point_new(polygon, X1, Y2);
 
 	pcb_add_poly_on_layer(Layer, polygon);
-	return (polygon);
+	return polygon;
 }
 
 pcb_poly_t *pcb_poly_new_from_poly(pcb_layer_t *Layer, pcb_poly_t *src, pcb_coord_t offs, pcb_coord_t Clearance, pcb_flag_t Flags)
@@ -221,7 +269,7 @@ pcb_poly_t *pcb_poly_new_from_poly(pcb_layer_t *Layer, pcb_poly_t *src, pcb_coor
 	pcb_poly_t *polygon = pcb_poly_new(Layer, Clearance, Flags);
 
 	if (!polygon)
-		return (polygon);
+		return polygon;
 
 	it.pa = src->Clipped;
 	pcb_poly_island_first(src, &it);
@@ -233,7 +281,7 @@ pcb_poly_t *pcb_poly_new_from_poly(pcb_layer_t *Layer, pcb_poly_t *src, pcb_coor
 	pcb_poly_contours_free(&it.cntr);
 
 	pcb_add_poly_on_layer(Layer, polygon);
-	return (polygon);
+	return polygon;
 }
 
 
@@ -242,8 +290,8 @@ void pcb_add_poly_on_layer(pcb_layer_t *Layer, pcb_poly_t *polygon)
 {
 	pcb_poly_bbox(polygon);
 	if (!Layer->polygon_tree)
-		Layer->polygon_tree = pcb_r_create_tree(NULL, 0, 0);
-	pcb_r_insert_entry(Layer->polygon_tree, (pcb_box_t *) polygon, 0);
+		Layer->polygon_tree = pcb_r_create_tree();
+	pcb_r_insert_entry(Layer->polygon_tree, (pcb_box_t *) polygon);
 	PCB_SET_PARENT(polygon, layer, Layer);
 	pcb_poly_clear_from_poly(Layer->parent, PCB_TYPE_POLY, Layer, polygon);
 }
@@ -260,7 +308,7 @@ pcb_poly_t *pcb_poly_new(pcb_layer_t *Layer, pcb_coord_t Clearance, pcb_flag_t F
 	polygon->Clipped = NULL;
 	polygon->NoHoles = NULL;
 	polygon->NoHolesValid = 0;
-	return (polygon);
+	return polygon;
 }
 
 static pcb_poly_t *pcb_poly_copy_meta(pcb_poly_t *dst, pcb_poly_t *src)
@@ -308,7 +356,7 @@ pcb_point_t *pcb_poly_point_new(pcb_poly_t *Polygon, pcb_coord_t X, pcb_coord_t
 	point->X = X;
 	point->Y = Y;
 	point->ID = pcb_create_ID_get();
-	return (point);
+	return point;
 }
 
 /* creates a new hole in a polygon */
@@ -335,7 +383,7 @@ pcb_poly_t *pcb_poly_copy(pcb_poly_t *Dest, pcb_poly_t *Src, pcb_coord_t dx, pcb
 	pcb_poly_bbox(Dest);
 	Dest->Flags = Src->Flags;
 	PCB_FLAG_CLEAR(PCB_FLAG_FOUND, Dest);
-	return (Dest);
+	return Dest;
 }
 
 static double poly_area(pcb_point_t *points, pcb_cardinal_t n_points)
@@ -391,64 +439,93 @@ void *pcb_polyop_add_to_buffer(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_poly_t
 	 * skeleton polygon object, which won't have correct bounds.
 	 */
 	if (!layer->polygon_tree)
-		layer->polygon_tree = pcb_r_create_tree(NULL, 0, 0);
-	pcb_r_insert_entry(layer->polygon_tree, (pcb_box_t *) polygon, 0);
+		layer->polygon_tree = pcb_r_create_tree();
+	pcb_r_insert_entry(layer->polygon_tree, (pcb_box_t *) polygon);
 
 	PCB_FLAG_CLEAR(PCB_FLAG_FOUND | ctx->buffer.extraflg, polygon);
-	return (polygon);
+	return polygon;
 }
 
 
-/* moves a polygon to buffer. Doesn't allocate memory for the points */
-void *pcb_polyop_move_to_buffer(pcb_opctx_t *ctx, pcb_layer_t * layer, pcb_poly_t * polygon)
+/* moves a polygon between board and buffer. Doesn't allocate memory for the points */
+void *pcb_polyop_move_buffer(pcb_opctx_t *ctx, pcb_layer_t *dstly, pcb_poly_t *polygon)
 {
-	pcb_layer_t *lay = &ctx->buffer.dst->Layer[pcb_layer_id(ctx->buffer.src, layer)];
+	pcb_layer_t *srcly = polygon->parent.layer;
+
+	assert(polygon->parent_type == PCB_PARENT_LAYER);
+	if ((dstly == NULL) || (dstly == srcly)) /* auto layer in dst */
+		dstly = &ctx->buffer.dst->Layer[pcb_layer_id(ctx->buffer.src, srcly)];
 
 	pcb_poly_pprestore(polygon);
 
-	pcb_r_delete_entry(layer->polygon_tree, (pcb_box_t *) polygon);
+	pcb_r_delete_entry(srcly->polygon_tree, (pcb_box_t *)polygon);
 
 	polylist_remove(polygon);
-	polylist_append(&lay->Polygon, polygon);
+	polylist_append(&dstly->Polygon, polygon);
 
 	PCB_FLAG_CLEAR(PCB_FLAG_FOUND, polygon);
 
-	if (!lay->polygon_tree)
-		lay->polygon_tree = pcb_r_create_tree(NULL, 0, 0);
-	pcb_r_insert_entry(lay->polygon_tree, (pcb_box_t *) polygon, 0);
+	if (!dstly->polygon_tree)
+		dstly->polygon_tree = pcb_r_create_tree();
+	pcb_r_insert_entry(dstly->polygon_tree, (pcb_box_t *)polygon);
 
-	PCB_SET_PARENT(polygon, layer, lay);
+	PCB_SET_PARENT(polygon, layer, dstly);
 
 	pcb_poly_ppclear(polygon);
-	return (polygon);
+	return polygon;
 }
 
 /* Handle attempts to change the clearance of a polygon. */
 void *pcb_polyop_change_clear_size(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_poly_t *poly)
 {
-	static int shown_this_message = 0;
-	if (!shown_this_message) {
-		pcb_gui->confirm_dialog(_("To change the clearance of objects in a polygon, "
-													"change the objects, not the polygon.\n"
-													"Hint: To set a minimum clearance for a group of objects, "
-													"select them all then :MinClearGap(Selected,=10,mil)"), "Ok", NULL);
-		shown_this_message = 1;
+	if (PCB_FLAG_TEST(PCB_FLAG_CLEARPOLYPOLY, poly)) {
+		pcb_coord_t value = (ctx->chgsize.is_absolute) ? ctx->chgsize.value : poly->Clearance + ctx->chgsize.value;
+
+		if (PCB_FLAG_TEST(PCB_FLAG_LOCK, poly))
+			return NULL;
+
+		value = MIN(PCB_MAX_LINESIZE, value);
+		if (!ctx->chgsize.is_absolute && ctx->chgsize.value < 0 && value < ctx->chgsize.pcb->Bloat * 2)
+			value = 0;
+		if (ctx->chgsize.value > 0 && value < ctx->chgsize.pcb->Bloat * 2)
+			value = ctx->chgsize.pcb->Bloat * 2 + 2;
+		if (value != poly->Clearance) {
+			pcb_undo_add_obj_to_clear_size(PCB_TYPE_POLY, Layer, poly, poly);
+			pcb_poly_restore_to_poly(ctx->chgsize.pcb->Data, PCB_TYPE_POLY, Layer, poly);
+			pcb_poly_invalidate_erase(poly);
+			pcb_r_delete_entry(Layer->polygon_tree, (pcb_box_t *)poly);
+			poly->Clearance = value;
+			pcb_poly_bbox(poly);
+			pcb_r_insert_entry(Layer->polygon_tree, (pcb_box_t *)poly);
+			pcb_poly_clear_from_poly(ctx->chgsize.pcb->Data, PCB_TYPE_POLY, Layer, poly);
+			pcb_poly_invalidate_draw(Layer, poly);
+			return poly;
+		}
+
+		return poly;
 	}
 
-	return (NULL);
+	/* poly does not clear other polys */
+	pcb_message(PCB_MSG_WARNING,
+		_("To change the clearance of objects in a polygon, change \nthe objects, not the polygon.\n"
+		"Alternatively, set the clearpolypoly flag on the polygon to \nallow it to clear other polygons.\n"
+		"Hint: To set a minimum clearance for a group of objects, \nselect them all then :MinClearGap(Selected,=10,mil)\n"),
+		"Ok", NULL);
+
+	return NULL;
 }
 
 /* changes the CLEARPOLY flag of a polygon */
 void *pcb_polyop_change_clear(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_poly_t *Polygon)
 {
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, Polygon))
-		return (NULL);
+		return NULL;
 	pcb_undo_add_obj_to_clear_poly(PCB_TYPE_POLY, Layer, Polygon, Polygon, pcb_true);
 	pcb_undo_add_obj_to_flag(Polygon);
 	PCB_FLAG_TOGGLE(PCB_FLAG_CLEARPOLY, Polygon);
 	pcb_poly_init_clip(PCB->Data, Layer, Polygon);
 	pcb_poly_invalidate_draw(Layer, Polygon);
-	return (Polygon);
+	return Polygon;
 }
 
 /* inserts a point into a polygon */
@@ -468,7 +545,7 @@ void *pcb_polyop_insert_point(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_poly_t *
 		line.Point1 = Polygon->Points[pcb_poly_contour_prev_point(Polygon, ctx->insert.idx)];
 		line.Point2 = Polygon->Points[ctx->insert.idx];
 		if (pcb_is_point_on_line((float) ctx->insert.x, (float) ctx->insert.y, 0.0, &line))
-			return (NULL);
+			return NULL;
 	}
 	/*
 	 * second, shift the points up to make room for the new point
@@ -489,12 +566,10 @@ void *pcb_polyop_insert_point(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_poly_t *
 	pcb_undo_add_obj_to_insert_point(PCB_TYPE_POLY_POINT, Layer, Polygon, &Polygon->Points[ctx->insert.idx]);
 
 	pcb_poly_bbox(Polygon);
-	pcb_r_insert_entry(Layer->polygon_tree, (pcb_box_t *) Polygon, 0);
+	pcb_r_insert_entry(Layer->polygon_tree, (pcb_box_t *) Polygon);
 	pcb_poly_init_clip(PCB->Data, Layer, Polygon);
-	if (ctx->insert.forcible || !pcb_poly_remove_excess_points(Layer, Polygon)) {
+	if (ctx->insert.forcible || !pcb_poly_remove_excess_points(Layer, Polygon))
 		pcb_poly_invalidate_draw(Layer, Polygon);
-		pcb_draw();
-	}
 
 	pcb_poly_ppclear(Polygon);
 
@@ -505,7 +580,7 @@ void *pcb_polyop_insert_point(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_poly_t *
 void *pcb_polyop_change_join(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_poly_t *poly)
 {
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, poly))
-		return (NULL);
+		return NULL;
 	pcb_poly_invalidate_erase(poly);
 	if (PCB_FLAG_TEST(PCB_FLAG_CLEARPOLYPOLY, poly)) {
 		pcb_undo_add_obj_to_clear_poly(PCB_TYPE_POLY, Layer, poly, poly, pcb_false);
@@ -540,10 +615,8 @@ void *pcb_polyop_move_noclip(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_poly_t *P
 	}
 	pcb_poly_move(Polygon, ctx->move.dx, ctx->move.dy);
 	pcb_poly_init_clip(PCB->Data, Layer, Polygon);
-	if (Layer->meta.real.vis) {
+	if (Layer->meta.real.vis)
 		pcb_poly_invalidate_draw(Layer, Polygon);
-		pcb_draw();
-	}
 	return Polygon;
 }
 
@@ -552,7 +625,7 @@ void *pcb_polyop_move(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_poly_t *Polygon)
 	pcb_r_delete_entry(Layer->polygon_tree, (pcb_box_t *) Polygon);
 	pcb_poly_pprestore(Polygon);
 	pcb_polyop_move_noclip(ctx, Layer, Polygon);
-	pcb_r_insert_entry(Layer->polygon_tree, (pcb_box_t *) Polygon, 0);
+	pcb_r_insert_entry(Layer->polygon_tree, (pcb_box_t *) Polygon);
 	pcb_poly_ppclear(Polygon);
 	return Polygon;
 }
@@ -564,7 +637,7 @@ void *pcb_polyop_clip(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_poly_t *Polygon)
 		pcb_poly_pprestore(Polygon);
 	}
 	if (ctx->clip.clear) {
-		pcb_r_insert_entry(Layer->polygon_tree, (pcb_box_t *) Polygon, 0);
+		pcb_r_insert_entry(Layer->polygon_tree, (pcb_box_t *) Polygon);
 		pcb_poly_ppclear(Polygon);
 	}
 	return Polygon;
@@ -579,15 +652,13 @@ void *pcb_polyop_move_point(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_poly_t *Po
 	pcb_r_delete_entry(Layer->polygon_tree, (pcb_box_t *) Polygon);
 	PCB_MOVE(Point->X, Point->Y, ctx->move.dx, ctx->move.dy);
 	pcb_poly_bbox(Polygon);
-	pcb_r_insert_entry(Layer->polygon_tree, (pcb_box_t *) Polygon, 0);
+	pcb_r_insert_entry(Layer->polygon_tree, (pcb_box_t *) Polygon);
 	pcb_poly_remove_excess_points(Layer, Polygon);
 	pcb_poly_init_clip(PCB->Data, Layer, Polygon);
-	if (Layer->meta.real.vis) {
+	if (Layer->meta.real.vis)
 		pcb_poly_invalidate_draw(Layer, Polygon);
-		pcb_draw();
-	}
 	pcb_poly_ppclear(Polygon);
-	return (Point);
+	return Point;
 }
 
 /* moves a polygon between layers; lowlevel routines */
@@ -600,8 +671,8 @@ void *pcb_polyop_move_to_layer_low(pcb_opctx_t *ctx, pcb_layer_t * Source, pcb_p
 	polylist_append(&Destination->Polygon, polygon);
 
 	if (!Destination->polygon_tree)
-		Destination->polygon_tree = pcb_r_create_tree(NULL, 0, 0);
-	pcb_r_insert_entry(Destination->polygon_tree, (pcb_box_t *) polygon, 0);
+		Destination->polygon_tree = pcb_r_create_tree();
+	pcb_r_insert_entry(Destination->polygon_tree, (pcb_box_t *) polygon);
 
 	PCB_SET_PARENT(polygon, layer, Destination);
 	pcb_poly_ppclear(polygon);
@@ -640,7 +711,7 @@ void *pcb_polyop_move_to_layer(pcb_opctx_t *ctx, pcb_layer_t * Layer, pcb_poly_t
 		return NULL;
 	}
 	if (Layer == ctx->move.dst_layer)
-		return (Polygon);
+		return Polygon;
 	pcb_undo_add_obj_to_move_to_layer(PCB_TYPE_POLY, Layer, Polygon, Polygon);
 	if (Layer->meta.real.vis)
 		pcb_poly_invalidate_erase(Polygon);
@@ -654,11 +725,9 @@ void *pcb_polyop_move_to_layer(pcb_opctx_t *ctx, pcb_layer_t * Layer, pcb_poly_t
 	pcb_r_search(PCB->Data->via_tree, &Polygon->BoundingBox, NULL, mptl_pin_callback, &d, NULL);
 	newone = (struct pcb_poly_s *) pcb_polyop_move_to_layer_low(ctx, Layer, Polygon, ctx->move.dst_layer);
 	pcb_poly_init_clip(PCB->Data, ctx->move.dst_layer, newone);
-	if (ctx->move.dst_layer->meta.real.vis) {
+	if (ctx->move.dst_layer->meta.real.vis)
 		pcb_poly_invalidate_draw(ctx->move.dst_layer, newone);
-		pcb_draw();
-	}
-	return (newone);
+	return newone;
 }
 
 
@@ -704,10 +773,10 @@ void *pcb_polyop_destroy_point(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_poly_t
 			Polygon->HoleIndex[i]--;
 
 	pcb_poly_bbox(Polygon);
-	pcb_r_insert_entry(Layer->polygon_tree, (pcb_box_t *) Polygon, 0);
+	pcb_r_insert_entry(Layer->polygon_tree, (pcb_box_t *) Polygon);
 	pcb_poly_init_clip(PCB->Data, Layer, Polygon);
 	pcb_poly_ppclear(Polygon);
-	return (Polygon);
+	return Polygon;
 }
 
 /* removes a polygon from a layer */
@@ -715,24 +784,23 @@ void *pcb_polyop_remove(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_poly_t *Polygo
 {
 	/* erase from screen */
 	pcb_poly_pprestore(Polygon);
-	if (Layer->meta.real.vis) {
+	if (Layer->meta.real.vis)
 		pcb_poly_invalidate_erase(Polygon);
-		if (!ctx->remove.bulk)
-			pcb_draw();
-	}
 	pcb_undo_move_obj_to_remove(PCB_TYPE_POLY, Layer, Polygon, Polygon);
 	return NULL;
 }
 
 void *pcb_poly_remove(pcb_layer_t *Layer, pcb_poly_t *Polygon)
 {
+	void *res;
 	pcb_opctx_t ctx;
 
 	ctx.remove.pcb = PCB;
-	ctx.remove.bulk = pcb_false;
 	ctx.remove.destroy_target = NULL;
 
-	return pcb_polyop_remove(&ctx, Layer, Polygon);
+	res = pcb_polyop_remove(&ctx, Layer, Polygon);
+	pcb_draw();
+	return res;
 }
 
 /* Removes a contour from a polygon.
@@ -747,11 +815,8 @@ void *pcb_polyop_remove_counter(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_poly_t
 
 	pcb_poly_pprestore(Polygon);
 
-	if (Layer->meta.real.vis) {
+	if (Layer->meta.real.vis)
 		pcb_poly_invalidate_erase(Polygon);
-		if (!ctx->remove.bulk)
-			pcb_draw();
-	}
 
 	/* Copy the polygon to the undo list */
 	pcb_undo_add_obj_to_remove_contour(PCB_TYPE_POLY, Layer, Polygon);
@@ -775,11 +840,8 @@ void *pcb_polyop_remove_counter(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_poly_t
 	pcb_poly_ppclear(Polygon);
 
 	/* redraw polygon if necessary */
-	if (Layer->meta.real.vis) {
+	if (Layer->meta.real.vis)
 		pcb_poly_invalidate_draw(Layer, Polygon);
-		if (!ctx->remove.bulk)
-			pcb_draw();
-	}
 	return NULL;
 }
 
@@ -820,16 +882,13 @@ void *pcb_polyop_remove_point(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_poly_t *
 			Polygon->HoleIndex[i]--;
 
 	pcb_poly_bbox(Polygon);
-	pcb_r_insert_entry(Layer->polygon_tree, (pcb_box_t *) Polygon, 0);
+	pcb_r_insert_entry(Layer->polygon_tree, (pcb_box_t *) Polygon);
 	pcb_poly_remove_excess_points(Layer, Polygon);
 	pcb_poly_init_clip(PCB->Data, Layer, Polygon);
 
 	/* redraw polygon if necessary */
-	if (Layer->meta.real.vis) {
+	if (Layer->meta.real.vis)
 		pcb_poly_invalidate_draw(Layer, Polygon);
-		if (!ctx->remove.bulk)
-			pcb_draw();
-	}
 
 	pcb_poly_ppclear(Polygon);
 	return NULL;
@@ -844,13 +903,13 @@ void *pcb_polyop_copy(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_poly_t *Polygon)
 	pcb_poly_copy(polygon, Polygon, ctx->copy.DeltaX, ctx->copy.DeltaY);
 	pcb_poly_copy_meta(polygon, Polygon);
 	if (!Layer->polygon_tree)
-		Layer->polygon_tree = pcb_r_create_tree(NULL, 0, 0);
-	pcb_r_insert_entry(Layer->polygon_tree, (pcb_box_t *) polygon, 0);
+		Layer->polygon_tree = pcb_r_create_tree();
+	pcb_r_insert_entry(Layer->polygon_tree, (pcb_box_t *) polygon);
 	pcb_poly_init_clip(PCB->Data, Layer, polygon);
 	pcb_poly_invalidate_draw(Layer, polygon);
 	pcb_undo_add_obj_to_create(PCB_TYPE_POLY, Layer, polygon, polygon);
 	pcb_poly_ppclear(polygon);
-	return (polygon);
+	return polygon;
 }
 
 void *pcb_polyop_rotate90(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_poly_t *Polygon)
@@ -860,12 +919,10 @@ void *pcb_polyop_rotate90(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_poly_t *Poly
 		pcb_poly_invalidate_erase(Polygon);
 	pcb_r_delete_entry(Layer->polygon_tree, (pcb_box_t *) Polygon);
 	pcb_poly_rotate90(Polygon, ctx->rotate.center_x, ctx->rotate.center_y, ctx->rotate.number);
-	pcb_r_insert_entry(Layer->polygon_tree, (pcb_box_t *) Polygon, 0);
+	pcb_r_insert_entry(Layer->polygon_tree, (pcb_box_t *) Polygon);
 	pcb_poly_init_clip(PCB->Data, Layer, Polygon);
-	if (Layer->meta.real.vis) {
+	if (Layer->meta.real.vis)
 		pcb_poly_invalidate_draw(Layer, Polygon);
-		pcb_draw();
-	}
 	pcb_poly_ppclear(Polygon);
 	return Polygon;
 }
@@ -877,10 +934,8 @@ void *pcb_polyop_rotate(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_poly_t *Polygo
 		pcb_poly_invalidate_erase(Polygon);
 	pcb_poly_rotate(Layer, Polygon, ctx->rotate.center_x, ctx->rotate.center_y, ctx->rotate.cosa, ctx->rotate.sina);
 	pcb_poly_init_clip(PCB->Data, Layer, Polygon);
-	if (Layer->meta.real.vis) {
+	if (Layer->meta.real.vis)
 		pcb_poly_invalidate_draw(Layer, Polygon);
-		pcb_draw();
-	}
 	pcb_poly_ppclear(Polygon);
 	return Polygon;
 }
@@ -1028,8 +1083,12 @@ static void pcb_poly_draw(pcb_layer_t *layer, pcb_poly_t *polygon, const pcb_box
 
 	if (PCB_FLAG_TEST(PCB_FLAG_WARN, polygon))
 		color = conf_core.appearance.color.warn;
-	else if (PCB_FLAG_TEST(PCB_FLAG_SELECTED, polygon))
-		color = layer->meta.real.selected_color;
+	else if (PCB_FLAG_TEST(PCB_FLAG_SELECTED, polygon)) {
+		if (layer->is_bound)
+			PCB_OBJ_COLOR_ON_BOUND_LAYER(color, layer, 1);
+		else
+			color = layer->meta.real.selected_color;
+	}
 	else if (PCB_FLAG_TEST(PCB_FLAG_FOUND, polygon))
 		color = conf_core.appearance.color.connected;
 	else if (PCB_FLAG_TEST(PCB_FLAG_ONPOINT, polygon)) {
@@ -1041,7 +1100,7 @@ static void pcb_poly_draw(pcb_layer_t *layer, pcb_poly_t *polygon, const pcb_box
 		color = (polygon->override_color);
 	}
 	else if (layer->is_bound)
-		PCB_OBJ_COLOR_ON_BOUND_LAYER(color, layer);
+		PCB_OBJ_COLOR_ON_BOUND_LAYER(color, layer, 0);
 	else
 		color = layer->meta.real.color;
 	pcb_gui->set_color(pcb_draw_out.fgGC, color);
@@ -1054,6 +1113,9 @@ pcb_r_dir_t pcb_poly_draw_callback(const pcb_box_t * b, void *cl)
 	pcb_draw_info_t *i = cl;
 	pcb_poly_t *polygon = (pcb_poly_t *) b;
 
+	if (pcb_hidden_floater((pcb_any_obj_t*)b))
+		return PCB_R_DIR_FOUND_CONTINUE;
+
 	if (!polygon->Clipped)
 		return PCB_R_DIR_NOT_FOUND;
 
