@@ -1,7 +1,8 @@
 /*
  *                            COPYRIGHT
  *
- *  PCB, interactive printed circuit board design
+ *  pcb-rnd, interactive printed circuit board design
+ *  (this file is based on PCB, interactive printed circuit board design)
  *  Copyright (C) 1994,1995,1996 Thomas Nau
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -18,9 +19,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- *  Contact addresses for paper mail and Email:
- *  Thomas Nau, Schlehenweg 15, 88471 Baustetten, Germany
- *  Thomas.Nau@rz.uni-ulm.de
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  *
  */
 
@@ -29,6 +31,7 @@
 #ifndef PCB_OBJ_POLY_H
 #define PCB_OBJ_POLY_H
 
+#include <genlist/gendlist.h>
 #include "obj_common.h"
 #include "polyarea.h"
 
@@ -44,6 +47,7 @@ struct pcb_poly_s  {           /* holds information about a polygon */
 	pcb_cardinal_t *HoleIndex;   /* Index of hole data within the Points array */
 	pcb_cardinal_t HoleIndexN;   /* number of holes in polygon */
 	pcb_cardinal_t HoleIndexMax; /* max number from malloc() */
+	unsigned clip_dirty:1;       /* 1 if polygon should be reclipped after clipping inhibit is over */
 	gdl_elem_t link;             /* a poly is in a list of a layer */
 };
 
@@ -73,6 +77,10 @@ void pcb_poly_flip_side(pcb_layer_t *layer, pcb_poly_t *polygon);
 void pcb_poly_move(pcb_poly_t *Polygon, pcb_coord_t DX, pcb_coord_t DY);
 pcb_poly_t *pcb_poly_copy(pcb_poly_t *Dest, pcb_poly_t *Src, pcb_coord_t dx, pcb_coord_t dy);
 
+/* hash and eq */
+int pcb_poly_eq(const pcb_host_trans_t *tr1, const pcb_poly_t *p1, const pcb_host_trans_t *tr2, const pcb_poly_t *p2);
+unsigned int pcb_poly_hash(const pcb_host_trans_t *tr, const pcb_poly_t *p);
+
 /* Add objects without creating them or making any "sanity modifications" to them */
 void pcb_add_poly_on_layer(pcb_layer_t *Layer, pcb_poly_t *polygon);
 
@@ -198,15 +206,21 @@ PCB_INLINE pcb_polyarea_t *pcb_poly_iterate_polyarea(pcb_polyarea_t *pa, pcb_pol
    it to dst - implemented in polygon.c */
 void pcb_poly_pa_clearance_construct(pcb_polyarea_t **dst, pcb_poly_it_t *it, pcb_coord_t clearance);
 
+
+#define pcb_poly_ppclear_at(poly, layer) \
+do { \
+	if (layer->is_bound) layer = layer->meta.bound.real; \
+	if (PCB_POLY_HAS_CLEARANCE(poly) && (layer != NULL)) \
+		pcb_poly_clear_from_poly(layer->parent, PCB_TYPE_POLY, layer, poly); \
+} while(0)
+
 /* Let the poly clear sorrunding polys in its layer */
 #define pcb_poly_ppclear(poly) \
 do { \
 	if (poly->parent.layer != NULL) { \
 		pcb_layer_t *layer = pcb_layer_get_real(poly->parent.layer); \
 		if ((layer != NULL) && (layer->parent->parent_type == PCB_PARENT_BOARD)) { \
-			if (layer->is_bound) layer = layer->meta.bound.real; \
-			if (PCB_POLY_HAS_CLEARANCE(poly) && (layer != NULL)) \
-				pcb_poly_clear_from_poly(layer->parent, PCB_TYPE_POLY, layer, poly); \
+			pcb_poly_ppclear_at(poly, layer); \
 		} \
 	} \
 } while(0)
