@@ -18,6 +18,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  */
 
 #include "config.h"
@@ -37,13 +41,12 @@
 #include "obj_pstk_inlines.h"
 #include "obj_pstk_op.h"
 #include "obj_subc_parent.h"
+#include "obj_hash.h"
 #include "operation.h"
 #include "search.h"
 #include "undo.h"
 #include "vtpadstack.h"
 
-#define PS_CROSS_SIZE PCB_MM_TO_COORD(1)
-
 #define SQR(o) ((o)*(o))
 
 static const char core_pstk_cookie[] = "padstack";
@@ -104,12 +107,12 @@ void pcb_pstk_add(pcb_data_t *data, pcb_pstk_t *ps)
 {
 	pcb_pstk_bbox(ps);
 	if (!data->padstack_tree)
-		data->padstack_tree = pcb_r_create_tree(NULL, 0, 0);
-	pcb_r_insert_entry(data->padstack_tree, (pcb_box_t *)ps, 0);
+		data->padstack_tree = pcb_r_create_tree();
+	pcb_r_insert_entry(data->padstack_tree, (pcb_box_t *)ps);
 	PCB_SET_PARENT(ps, data, data);
 }
 
-void pcb_pstk_bbox(pcb_pstk_t *ps)
+static void pcb_pstk_bbox_(pcb_box_t *dst, pcb_pstk_t *ps, pcb_bool copper_only)
 {
 	int n, sn;
 	pcb_line_t line;
@@ -117,9 +120,10 @@ void pcb_pstk_bbox(pcb_pstk_t *ps)
 	pcb_pstk_tshape_t *ts = pcb_pstk_get_tshape(ps);
 	assert(proto != NULL);
 
-	ps->BoundingBox.X1 = ps->BoundingBox.X2 = ps->x;
-	ps->BoundingBox.Y1 = ps->BoundingBox.Y2 = ps->y;
+	dst->X1 = dst->X2 = ps->x;
+	dst->Y1 = dst->Y2 = ps->y;
 
+	if (ts != NULL)
 	for(sn = 0; sn < ts->len; sn++) {
 		pcb_pstk_shape_t *shape = ts->shape + sn;
 		switch(shape->shape) {
@@ -136,29 +140,75 @@ void pcb_pstk_bbox(pcb_pstk_t *ps)
 				line.Clearance = 0;
 				line.Flags = pcb_flag_make(shape->data.line.square ? PCB_FLAG_SQUARE : 0);
 				pcb_line_bbox(&line);
-				pcb_box_bump_box(&ps->BoundingBox, &line.BoundingBox);
+				pcb_box_bump_box(dst, &line.BoundingBox);
 				break;
 			case PCB_PSSH_CIRC:
-				pcb_box_bump_point(&ps->BoundingBox, ps->x - shape->data.circ.dia/2, ps->y - shape->data.circ.dia/2);
-				pcb_box_bump_point(&ps->BoundingBox, ps->x + shape->data.circ.dia/2, ps->y + shape->data.circ.dia/2);
+				pcb_box_bump_point(dst, ps->x - shape->data.circ.dia/2, ps->y - shape->data.circ.dia/2);
+				pcb_box_bump_point(dst, ps->x + shape->data.circ.dia/2, ps->y + shape->data.circ.dia/2);
 				break;
 		}
 	}
 
-	if (PCB_NONPOLY_HAS_CLEARANCE(ps)) {
-		ps->BoundingBox.X1 -= ps->Clearance;
-		ps->BoundingBox.Y1 -= ps->Clearance;
-		ps->BoundingBox.X2 += ps->Clearance;
-		ps->BoundingBox.Y2 += ps->Clearance;
+	if (!copper_only && PCB_NONPOLY_HAS_CLEARANCE(ps)) {
+		dst->X1 -= ps->Clearance;
+		dst->Y1 -= ps->Clearance;
+		dst->X2 += ps->Clearance;
+		dst->Y2 += ps->Clearance;
 	}
 
 	if (proto->hdia != 0) {
 		/* corner case: no copper around the hole all 360 deg - let the hole stick out */
-		pcb_box_bump_point(&ps->BoundingBox, ps->x - proto->hdia/2, ps->y - proto->hdia/2);
-		pcb_box_bump_point(&ps->BoundingBox, ps->x + proto->hdia/2, ps->y + proto->hdia/2);
+		pcb_box_bump_point(dst, ps->x - proto->hdia/2, ps->y - proto->hdia/2);
+		pcb_box_bump_point(dst, ps->x + proto->hdia/2, ps->y + proto->hdia/2);
 	}
 
-	pcb_close_box(&ps->BoundingBox);
+	pcb_close_box(dst);
+}
+
+void pcb_pstk_bbox(pcb_pstk_t *ps)
+{
+	pcb_pstk_bbox_(&ps->BoundingBox, ps, pcb_false);
+}
+
+void pcb_pstk_copper_bbox(pcb_box_t *dst, pcb_pstk_t *ps)
+{
+	pcb_pstk_bbox_(dst, ps, pcb_true);
+}
+
+/* hash */
+int pcb_pstk_eq(const pcb_host_trans_t *tr1, const pcb_pstk_t *p1, const pcb_host_trans_t *tr2, const pcb_pstk_t *p2)
+{
+#warning padstack TODO: should compare shape by shape: a 180 deg rotated or mirrored rectangle is still just the same rectangle!
+	if (pcb_field_neq(p1, p2, smirror) || pcb_field_neq(p1, p2, xmirror)) return 0;
+	if (pcb_field_neq(p1, p2, rot)) return 0;
+
+	if (!PCB_FLAG_TEST(PCB_FLAG_FLOATER, p1) && !PCB_FLAG_TEST(PCB_FLAG_FLOATER, p2)) {
+		pcb_coord_t x1, y1, x2, y2;
+		
+		pcb_hash_tr_coords(tr1, &x1, &y1, p1->x, p1->y);
+		pcb_hash_tr_coords(tr2, &x2, &y2, p2->x, p2->y);
+		if ((x1 != x2) || (y1 != y2)) return 0;
+	}
+
+	if (pcb_neqs(p1->term, p2->term)) return 0;
+	return 1;
+}
+
+unsigned int pcb_pstk_hash(const pcb_host_trans_t *tr, const pcb_pstk_t *p)
+{
+	unsigned int crd = 0;
+
+	if (!PCB_FLAG_TEST(PCB_FLAG_FLOATER, p)) {
+		pcb_coord_t x, y;
+		
+		pcb_hash_tr_coords(tr, &x, &y, p->x, p->y);
+		crd = pcb_hash_coord(x) ^ pcb_hash_coord(y);
+	}
+
+	return pcb_hash_angle(tr, p->rot) ^ pcb_hash_coord(p->Clearance) ^
+		pcb_hash_str(p->term) ^ crd ^
+		murmurhash(&p->smirror, sizeof(p->smirror)) ^
+		murmurhash(&p->xmirror, sizeof(p->xmirror));
 }
 
 /*** utils ***/
@@ -339,15 +389,29 @@ pcb_r_dir_t pcb_pstk_draw_callback(const pcb_box_t *b, void *cl)
 	pcb_pstk_draw_t *ctx = cl;
 	pcb_pstk_t *ps = (pcb_pstk_t *)b;
 	pcb_pstk_shape_t *shape;
-	pcb_layergrp_t *grp;
+	pcb_layergrp_t *grp = NULL;
+
+	if (pcb_hidden_floater((pcb_any_obj_t*)b))
+		return PCB_R_DIR_FOUND_CONTINUE;
 
 	if (!PCB->SubcPartsOn && pcb_gobj_parent_subc(ps->parent_type, &ps->parent))
 		return PCB_R_DIR_NOT_FOUND;
 
-	shape = pcb_pstk_shape_gid(ctx->pcb, ps, ctx->gid, ctx->comb, &grp);
+	if (ctx->gid < 0) {
+		if (ctx->shape_mask != 0)
+			shape = pcb_pstk_shape(ps, ctx->shape_mask, ctx->comb);
+		else
+			return PCB_R_DIR_NOT_FOUND;
+	}
+	else
+		shape = pcb_pstk_shape_gid(ctx->pcb, ps, ctx->gid, ctx->comb, &grp);
+
 	if (shape != NULL) {
 		pcb_gui->set_draw_xor(pcb_draw_out.fgGC, 0);
-		set_ps_color(ps, ctx->is_current, grp->type);
+		if (grp == NULL)
+			set_ps_color(ps, ctx->is_current, ctx->shape_mask);
+		else
+			set_ps_color(ps, ctx->is_current, grp->type);
 		if (conf_core.editor.thin_draw || conf_core.editor.wireframe_draw) {
 			pcb_gui->set_line_width(pcb_draw_out.fgGC, 0);
 			pcb_pstk_draw_shape_thin(pcb_draw_out.fgGC, ps, shape);
@@ -367,22 +431,26 @@ pcb_r_dir_t pcb_pstk_draw_mark_callback(const pcb_box_t *b, void *cl)
 
 	/* mark is a cross in the middle, right on the hole;
 	   cross size should extend beyond the hole */
-	mark = PS_CROSS_SIZE/2;
+	mark = conf_core.appearance.padstack.cross_size;
 	proto = pcb_pstk_get_proto(ps);
 	if (proto != NULL)
 		mark += proto->hdia/2;
 
 	/* draw the cross using xor */
-	set_ps_annot_color(pcb_draw_out.fgGC, ps);
-	pcb_gui->set_line_width(pcb_draw_out.fgGC, -3);
-	pcb_gui->set_draw_xor(pcb_draw_out.fgGC, 1);
-	pcb_gui->draw_line(pcb_draw_out.fgGC, ps->x-mark, ps->y, ps->x+mark, ps->y);
-	pcb_gui->draw_line(pcb_draw_out.fgGC, ps->x, ps->y-mark, ps->x, ps->y+mark);
-	pcb_gui->set_draw_xor(pcb_draw_out.fgGC, 0);
+	if (conf_core.appearance.padstack.cross_thick > 0) {
+		set_ps_annot_color(pcb_draw_out.fgGC, ps);
+		pcb_gui->set_line_width(pcb_draw_out.fgGC, -conf_core.appearance.padstack.cross_thick);
+		pcb_gui->set_draw_xor(pcb_draw_out.fgGC, 1);
+		pcb_gui->draw_line(pcb_draw_out.fgGC, ps->x-mark, ps->y, ps->x+mark, ps->y);
+		pcb_gui->draw_line(pcb_draw_out.fgGC, ps->x, ps->y-mark, ps->x, ps->y+mark);
+		pcb_gui->set_draw_xor(pcb_draw_out.fgGC, 0);
+	}
 
 	/* draw the label if enabled, after everything else is drawn */
 	if (ps->term != NULL) {
-		if ((pcb_draw_doing_pinout) || PCB_FLAG_TEST(PCB_FLAG_TERMNAME, ps))
+		if (pcb_draw_doing_pinout)
+			pcb_pstk_draw_label(ps);
+		else if (PCB_FLAG_TEST(PCB_FLAG_TERMNAME, ps))
 			pcb_draw_delay_label_add((pcb_any_obj_t *)ps);
 	}
 	return PCB_R_DIR_FOUND_CONTINUE;
@@ -400,8 +468,12 @@ pcb_r_dir_t pcb_pstk_draw_hole_callback(const pcb_box_t *b, void *cl)
 		return PCB_R_DIR_NOT_FOUND;
 
 	/* no hole in this layer group */
-	if (!pcb_pstk_bb_drills(ctx->pcb, ps, ctx->gid, &proto))
-		return PCB_R_DIR_FOUND_CONTINUE;
+	if (ctx->gid >= 0) {
+		if (!pcb_pstk_bb_drills(ctx->pcb, ps, ctx->gid, &proto))
+			return PCB_R_DIR_FOUND_CONTINUE;
+	}
+	else
+		proto = pcb_pstk_get_proto(ps);
 
 	/* No hole at all */
 	if (proto->hdia <= 0)
@@ -451,7 +523,7 @@ void pcb_pstk_thindraw(pcb_hid_gc_t gc, pcb_pstk_t *ps)
 	}
 	else { /* no pcb means buffer - take the first shape, whichever layer it is for */
 		pcb_pstk_tshape_t *ts = pcb_pstk_get_tshape(ps);
-		if (ts != NULL)
+		if ((ts != NULL)  && (ts->len > 0))
 			shape = ts->shape;
 	}
 
@@ -575,14 +647,31 @@ static int pcb_pstk_near_box_(pcb_pstk_t *ps, pcb_box_t *box, pcb_pstk_shape_t *
 	return 0;
 }
 
+#define	HOLE_IN_BOX(ps, dia, b) \
+	( \
+	PCB_POINT_IN_BOX((ps)->x - dia/2, (ps)->y - dia/2, (b)) && \
+	PCB_POINT_IN_BOX((ps)->x + dia/2, (ps)->y + dia/2, (b)) \
+	)
+
+#define	HOLE_TOUCHES_BOX(ps, dia, b) \
+	PCB_CIRCLE_TOUCHES_BOX((ps)->x, (ps)->y, dia/2, (b))
+
 int pcb_pstk_near_box(pcb_pstk_t *ps, pcb_box_t *box, pcb_layer_t *layer)
 {
 	pcb_pstk_shape_t *shp;
+	pcb_pstk_tshape_t *tshp = pcb_pstk_get_tshape(ps);
+
+	/* special case: no-shape padstack is used for hole */
+	if ((tshp == NULL) || (tshp->len == 0)) {
+		pcb_pstk_proto_t *proto = pcb_pstk_get_proto(ps);
+		if (proto->hdia <= 0)
+			return 0;
+		return (PCB_IS_BOX_NEGATIVE(box) ? HOLE_TOUCHES_BOX(ps, proto->hdia, box) : HOLE_IN_BOX(ps, proto->hdia, box));
+	}
 
 	/* no layer means: "is any shape near?" */
 	if (layer == NULL) {
 		int n;
-		pcb_pstk_tshape_t *tshp = pcb_pstk_get_tshape(ps);
 
 		if (tshp == NULL)
 			return 0;
@@ -604,14 +693,6 @@ static int pcb_is_point_in_pstk_(pcb_pstk_t *ps, pcb_coord_t x, pcb_coord_t y, p
 {
 	pcb_pad_t pad;
 	pcb_vector_t v;
-	pcb_pstk_proto_t *proto = pcb_pstk_get_proto(ps);
-
-	/* cheap check: hole clicked (also necessary for a mounting hole too) */
-	if (proto->hdia > 0) {
-		double dist2 = SQR((double)x - (double)ps->x) + SQR((double)y - (double)ps->y);
-		if (dist2 <= SQR((double)radius + (double)proto->hdia/2.0))
-			return 1;
-	}
 
 	switch(shape->shape) {
 		case PCB_PSSH_CIRC:
@@ -639,6 +720,14 @@ static int pcb_is_point_in_pstk_(pcb_pstk_t *ps, pcb_coord_t x, pcb_coord_t y, p
 int pcb_is_point_in_pstk(pcb_coord_t x, pcb_coord_t y, pcb_coord_t radius, pcb_pstk_t *ps, pcb_layer_t *layer)
 {
 	pcb_pstk_shape_t *shp;
+	pcb_pstk_proto_t *proto = pcb_pstk_get_proto(ps);
+
+	/* cheap check: hole clicked (also necessary for a mounting hole too) */
+	if (proto->hdia > 0) {
+		double dist2 = SQR((double)x - (double)ps->x) + SQR((double)y - (double)ps->y);
+		if (dist2 <= SQR((double)radius + (double)proto->hdia/2.0))
+			return 1;
+	}
 
 	/* no layer means: "is point in any shape?" */
 	if (layer == NULL) {
@@ -773,12 +862,10 @@ void pcb_pstk_mirror(pcb_pstk_t *ps, pcb_coord_t y_offs, int swap_side)
 			pcb_r_delete_entry(ps->parent.data->padstack_tree, (pcb_box_t *)ps);
 
 		ps->y = PCB_SWAP_Y(ps->y) + y_offs;
-		if ((swap_side) && (ps->rot != 0))
-			ps->rot = -ps->rot;
 		pcb_pstk_bbox(ps);
 
 		if (ps->parent.data->padstack_tree != NULL)
-			pcb_r_insert_entry(ps->parent.data->padstack_tree, (pcb_box_t *)ps, 0);
+			pcb_r_insert_entry(ps->parent.data->padstack_tree, (pcb_box_t *)ps);
 		pcb_poly_clear_from_poly(ps->parent.data, PCB_TYPE_PSTK, NULL, ps);
 		pcb_pstk_invalidate_draw(ps);
 	}
@@ -874,7 +961,7 @@ static int undo_change_instance_swap(void *udata)
 
 	pcb_pstk_bbox(ps);
 	if (ps->parent.data->padstack_tree != NULL)
-		pcb_r_insert_entry(ps->parent.data->padstack_tree, (pcb_box_t *)ps, 0);
+		pcb_r_insert_entry(ps->parent.data->padstack_tree, (pcb_box_t *)ps);
 	pcb_poly_clear_from_poly(ps->parent.data, PCB_TYPE_PSTK, NULL, ps);
 	pcb_pstk_invalidate_draw(ps);
 
@@ -956,3 +1043,39 @@ pcb_bool pcb_pstk_is_group_empty(pcb_board_t *pcb, pcb_layergrp_id_t gid)
 
 
 #include "obj_pstk_op.c"
+
+void pcb_pstk_rotate90(pcb_pstk_t *pstk, pcb_coord_t cx, pcb_coord_t cy, int steps)
+{
+	pcb_opctx_t ctx;
+
+	ctx.rotate.center_x = cx;
+	ctx.rotate.center_y = cy;
+	ctx.rotate.number = steps;
+	pcb_pstkop_rotate90(&ctx, pstk);
+}
+
+void pcb_pstk_rotate(pcb_pstk_t *pstk, pcb_coord_t cx, pcb_coord_t cy, double cosa, double sina, double angle)
+{
+	pcb_opctx_t ctx;
+
+	ctx.rotate.center_x = cx;
+	ctx.rotate.center_y = cy;
+	ctx.rotate.angle = angle;
+	ctx.rotate.cosa = cosa;
+	ctx.rotate.sina = sina;
+	pcb_pstkop_rotate(&ctx, pstk);
+}
+
+pcb_coord_t obj_pstk_get_clearance(pcb_board_t *pcb, const pcb_pstk_t *ps, pcb_layer_t *layer)
+{
+	pcb_pstk_shape_t *shp;
+
+	if (!PCB_FLAG_TEST(PCB_FLAG_CLEARLINE, ps))
+		return 0;
+
+	if (ps->Clearance > 0) /* global clarance overrides */
+		return ps->Clearance;
+
+	shp = pcb_pstk_shape_at(pcb, ps, layer);
+	return shp->clearance;
+}
