@@ -18,11 +18,17 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  */
 
 #ifndef PCB_OBJ_PSTK_STRUCT_DECLARED
 #define PCB_OBJ_PSTK_STRUCT_DECLARED
 
+#include <genlist/gendlist.h>
+#include <genvector/vtp0.h>
 #include "obj_common.h"
 
 /* The actual padstack is just a reference to a padstack proto within the same data */
@@ -79,6 +85,12 @@ pcb_pstk_t *pcb_pstk_new(pcb_data_t *data, pcb_cardinal_t proto, pcb_coord_t x,
 pcb_pstk_t *pcb_pstk_new_tr(pcb_data_t *data, pcb_cardinal_t proto, pcb_coord_t x, pcb_coord_t y, pcb_coord_t clearance, pcb_flag_t Flags, double rot, int xmirror, int smirror);
 void pcb_pstk_add(pcb_data_t *data, pcb_pstk_t *ps);
 void pcb_pstk_bbox(pcb_pstk_t *ps);
+void pcb_pstk_copper_bbox(pcb_box_t *dst, pcb_pstk_t *ps);
+
+/* hash and eq */
+int pcb_pstk_eq(const pcb_host_trans_t *tr1, const pcb_pstk_t *p1, const pcb_host_trans_t *tr2, const pcb_pstk_t *p2);
+unsigned int pcb_pstk_hash(const pcb_host_trans_t *tr, const pcb_pstk_t *p);
+
 
 void pcb_pstk_set_thermal(pcb_pstk_t *ps, unsigned long lid, unsigned char shape);
 unsigned char *pcb_pstk_get_thermal(pcb_pstk_t *ps, unsigned long lid, pcb_bool_t alloc);
@@ -97,6 +109,8 @@ pcb_pstk_t *pcb_pstk_copy_meta(pcb_pstk_t *dst, pcb_pstk_t *src);
 /* Copy orientation information (rotatioin and mirror) of an instance */
 pcb_pstk_t *pcb_pstk_copy_orient(pcb_pstk_t *dst, pcb_pstk_t *src);
 
+/* Clearance of the padstack on a given layer */
+pcb_coord_t obj_pstk_get_clearance(pcb_board_t *pcb, const pcb_pstk_t *ps, pcb_layer_t *layer);
 
 /*** proto ***/
 
@@ -106,6 +120,11 @@ pcb_pstk_t *pcb_pstk_copy_orient(pcb_pstk_t *dst, pcb_pstk_t *src);
 pcb_cardinal_t pcb_pstk_conv_selection(pcb_board_t *pcb, int quiet, pcb_coord_t ox, pcb_coord_t oy);
 pcb_cardinal_t pcb_pstk_conv_buffer(int quiet);
 
+/* Low level converter: take an array of (pcb_any_obj_t *) objs and convert
+   them into shapes of the dst proto. Does not remove input objects. */
+int pcb_pstk_proto_conv(pcb_data_t *data, pcb_pstk_proto_t *dst, int quiet, vtp0_t *objs, pcb_coord_t ox, pcb_coord_t oy);
+
+
 /* free fields of a proto (not freeing the proto itself, not removing it from lists */
 void pcb_pstk_proto_free_fields(pcb_pstk_proto_t *dst);
 
@@ -150,6 +169,7 @@ void pcb_pstk_shape_free(pcb_pstk_shape_t *s);
 /* grow (or shrink) a prototype to or by val - change the proto in place */
 void pcb_pstk_proto_grow(pcb_pstk_proto_t *proto, pcb_bool is_absolute, pcb_coord_t val);
 void pcb_pstk_shape_grow(pcb_pstk_shape_t *shp, pcb_bool is_absolute, pcb_coord_t val);
+void pcb_pstk_shape_clr_grow(pcb_pstk_shape_t *shp, pcb_bool is_absolute, pcb_coord_t val);
 
 /* Derive (copy and bloat) the shape at dst_idx from src_idx; set the mask and comb
    for the new shape. If dst_idx is -1, allocate the new shape */
@@ -165,6 +185,10 @@ void pcb_pstk_proto_del_shape_idx(pcb_pstk_proto_t *proto, int idx);
 /* Mirror a padstack (useful for sending to the other side - set swap_side to 1 in that case) */
 void pcb_pstk_mirror(pcb_pstk_t *ps, pcb_coord_t y_offs, int swap_side);
 
+/* Rotate in place (op wrapper) */
+void pcb_pstk_rotate90(pcb_pstk_t *pstk, pcb_coord_t cx, pcb_coord_t cy, int steps);
+void pcb_pstk_rotate(pcb_pstk_t *pstk, pcb_coord_t cx, pcb_coord_t cy, double cosa, double sina, double angle);
+
 /* High level move (op wrapper; no undo) */
 void pcb_pstk_move(pcb_pstk_t *ps, pcb_coord_t dx, pcb_coord_t dy, pcb_bool more_to_come);
 
@@ -174,10 +198,15 @@ void pcb_pstk_move_(pcb_pstk_t *ps, pcb_coord_t dx, pcb_coord_t dy);
 /* Temporary hack until we have a refcounted objects and ID->pcb_any_obj_t hash */
 extern pcb_data_t *pcb_pstk_data_hack;
 
+/* Insert a proto in data and return the proto-id. If the proto is already
+   in data, the fields of the caller's version are free'd, else they are
+   copied into data. In any case, the caller should not free proto. */
+pcb_cardinal_t pcb_pstk_proto_insert_or_free(pcb_data_t *data, pcb_pstk_proto_t *proto, int quiet);
+
 
 /*** hash ***/
-unsigned int pcb_pstk_hash(const pcb_pstk_proto_t *p);
-int pcb_pstk_eq(const pcb_pstk_proto_t *p1, const pcb_pstk_proto_t *p2);
+unsigned int pcb_pstk_proto_hash(const pcb_pstk_proto_t *p);
+int pcb_pstk_proto_eq(const pcb_pstk_proto_t *p1, const pcb_pstk_proto_t *p2);
 
 /*** loops ***/
 #define PCB_PADSTACK_LOOP(top) do {                                \
