@@ -18,6 +18,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  */
 
 #include "polygon.h"
@@ -36,7 +40,8 @@ void *pcb_pstkop_add_to_buffer(pcb_opctx_t *ctx, pcb_pstk_t *ps)
 	return pcb_pstk_copy_meta(p, ps);
 }
 
-void *pcb_pstkop_move_to_buffer(pcb_opctx_t *ctx, pcb_pstk_t *ps)
+/* Move between board and buffer */
+void *pcb_pstkop_move_buffer(pcb_opctx_t *ctx, pcb_pstk_t *ps)
 {
 	pcb_cardinal_t npid;
 	pcb_pstk_proto_t *proto = pcb_pstk_get_proto(ps);
@@ -56,9 +61,9 @@ void *pcb_pstkop_move_to_buffer(pcb_opctx_t *ctx, pcb_pstk_t *ps)
 	PCB_FLAG_CLEAR(PCB_FLAG_WARN | PCB_FLAG_FOUND, ps);
 
 	if (!ctx->buffer.dst->padstack_tree)
-		ctx->buffer.dst->padstack_tree = pcb_r_create_tree(NULL, 0, 0);
+		ctx->buffer.dst->padstack_tree = pcb_r_create_tree();
 
-	pcb_r_insert_entry(ctx->buffer.dst->padstack_tree, (pcb_box_t *)ps, 0);
+	pcb_r_insert_entry(ctx->buffer.dst->padstack_tree, (pcb_box_t *)ps);
 	pcb_poly_clear_from_poly(ctx->buffer.dst, PCB_TYPE_PSTK, NULL, ps);
 
 	PCB_SET_PARENT(ps, data, ctx->buffer.dst);
@@ -91,7 +96,6 @@ void *pcb_pstkop_move_noclip(pcb_opctx_t *ctx, pcb_pstk_t *ps)
 	pcb_pstk_invalidate_erase(ps);
 	pcb_pstk_move_(ps, ctx->move.dx, ctx->move.dy);
 	pcb_pstk_invalidate_draw(ps);
-	pcb_draw();
 	return ps;
 }
 
@@ -103,7 +107,7 @@ void *pcb_pstkop_move(pcb_opctx_t *ctx, pcb_pstk_t *ps)
 	pcb_r_delete_entry(data->padstack_tree, (pcb_box_t *)ps);
 	pcb_poly_restore_to_poly(data, PCB_TYPE_PSTK, NULL, ps);
 	pcb_pstkop_move_noclip(ctx, ps);
-	pcb_r_insert_entry(data->padstack_tree, (pcb_box_t *)ps, 0);
+	pcb_r_insert_entry(data->padstack_tree, (pcb_box_t *)ps);
 	pcb_poly_clear_from_poly(data, PCB_TYPE_PSTK, NULL, ps);
 	return ps;
 }
@@ -120,7 +124,7 @@ void *pcb_pstkop_clip(pcb_opctx_t *ctx, pcb_pstk_t *ps)
 	}
 	if (ctx->clip.clear) {
 		if (data->padstack_tree != NULL)
-			pcb_r_insert_entry(data->padstack_tree, (pcb_box_t *)ps, 0);
+			pcb_r_insert_entry(data->padstack_tree, (pcb_box_t *)ps);
 		pcb_poly_clear_from_poly(data, PCB_TYPE_PSTK, NULL, ps);
 	}
 
@@ -212,7 +216,7 @@ void *pcb_pstkop_rotate(pcb_opctx_t *ctx, pcb_pstk_t *ps)
 			pcb_pstk_move_(ps, nx - ps->x, ny - ps->y);
 
 		pcb_pstk_bbox(ps);
-		pcb_r_insert_entry(ps->parent.data->padstack_tree, (pcb_box_t *)ps, 0);
+		pcb_r_insert_entry(ps->parent.data->padstack_tree, (pcb_box_t *)ps);
 		pcb_poly_clear_from_poly(ps->parent.data, PCB_TYPE_PSTK, NULL, ps);
 		pcb_pstk_invalidate_draw(ps);
 
