@@ -18,6 +18,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  */
 
 #include "config.h"
@@ -35,6 +39,7 @@
 #include "rotate.h"
 #include "undo.h"
 #include "vtpadstack_t.h"
+#include "obj_hash.h"
 
 static const char core_proto_cookie[] = "padstack prototypes";
 
@@ -89,24 +94,38 @@ static void append_circle(pcb_pstk_tshape_t *ts, pcb_layer_type_t lyt, pcb_layer
 	ts->shape[idx].comb = comb;
 }
 
-static int pcb_pstk_proto_conv(pcb_data_t *data, pcb_pstk_proto_t *dst, int quiet, vtp0_t *objs, pcb_coord_t ox, pcb_coord_t oy)
+static void append_tshape(pcb_pstk_tshape_t *ts, pcb_pstk_tshape_t *src, int srci)
 {
-	int ret = -1, n, m, i;
+	int idx = ts->len;
+
+	ts->len++;
+	ts->shape = realloc(ts->shape, ts->len * sizeof(pcb_pstk_shape_t));
+	memcpy(&ts->shape[idx], &src->shape[srci], sizeof(ts->shape[idx]));
+	switch(src->shape[srci].shape) {
+		case PCB_PSSH_LINE:
+		case PCB_PSSH_CIRC:
+			break; /* do nothing, all fields are copied already by the memcpy */
+		case PCB_PSSH_POLY:
+			pcb_pstk_shape_alloc_poly(&ts->shape[idx].data.poly, src->shape[srci].data.poly.len);
+			pcb_pstk_shape_copy_poly(&ts->shape[idx].data.poly, &src->shape[srci].data.poly);
+			break;
+	}
+}
+
+int pcb_pstk_proto_conv(pcb_data_t *data, pcb_pstk_proto_t *dst, int quiet, vtp0_t *objs, pcb_coord_t ox, pcb_coord_t oy)
+{
+	int ret = -1, n, m, i, extra_obj = 0;
 	pcb_any_obj_t **o;
-	pcb_pstk_tshape_t *ts;
+	pcb_pstk_tshape_t *ts, *ts_src;
 	pcb_pin_t *via = NULL;
+	pcb_pstk_t *pstk = NULL;
+	pcb_pstk_proto_t *prt;
 
 	dst->in_use = 1;
 	pcb_vtpadstack_tshape_init(&dst->tr);
 	dst->hdia = 0;
 	dst->htop = dst->hbottom = 0;
 
-	if (vtp0_len(objs) > data->LayerN) {
-		if (!quiet)
-			pcb_message(PCB_MSG_ERROR, "Padstack conversion: too many objects selected\n");
-		goto quit;
-	}
-
 	/* allocate shapes on the canonical tshape (tr[0]) */
 	ts = pcb_vtpadstack_tshape_alloc_append(&dst->tr, 1);
 	ts->rot = 0.0;
@@ -120,9 +139,9 @@ static int pcb_pstk_proto_conv(pcb_data_t *data, pcb_pstk_proto_t *dst, int quie
 				ts->len++;
 				break;
 			case PCB_OBJ_VIA:
-				if (via != NULL) {
+				if ((via != NULL) || (pstk != NULL)) {
 					if (!quiet)
-						pcb_message(PCB_MSG_ERROR, "Padstack conversion: multiple vias\n");
+						pcb_message(PCB_MSG_ERROR, "Padstack conversion: multiple vias/padstacks\n");
 					goto quit;
 				}
 				via = *(pcb_pin_t **)o;
@@ -133,17 +152,46 @@ static int pcb_pstk_proto_conv(pcb_data_t *data, pcb_pstk_proto_t *dst, int quie
 					ox = via->X;
 					oy = via->Y;
 				}
+				extra_obj++;
+				break;
+			case PCB_OBJ_PSTK:
+				if ((via != NULL) || (pstk != NULL)) {
+					if (!quiet)
+						pcb_message(PCB_MSG_ERROR, "Padstack conversion: multiple vias/padstacks\n");
+					goto quit;
+				}
+				pstk = *(pcb_pstk_t **)o;
+				prt = pcb_pstk_get_proto(pstk);
+				if (prt == NULL) {
+					if (!quiet)
+						pcb_message(PCB_MSG_ERROR, "Padstack conversion: invalid input padstacks proto\n");
+					goto quit;
+				}
+				dst->hdia = prt->hdia;
+				dst->hplated = prt->hplated;
+				if ((ox != pstk->x) || (oy != pstk->y)) {
+					pcb_message(PCB_MSG_INFO, "Padstack conversion: adjusting origin to padstack hole\n");
+					ox = pstk->x;
+					oy = pstk->y;
+				}
+				extra_obj++;
 				break;
 			default:;
 				if (!quiet)
-					pcb_message(PCB_MSG_ERROR, "Padstack conversion: invalid object type (%x) selected; must be via, line or polygon\n", (*o)->type);
+					pcb_message(PCB_MSG_ERROR, "Padstack conversion: invalid object type (%x) selected; must be via, padstack, line or polygon\n", (*o)->type);
 				goto quit;
 		}
 	}
 
-	if ((ts->len == 0) && (via == NULL)) {
+	if ((vtp0_len(objs) - extra_obj) > data->LayerN) {
 		if (!quiet)
-			pcb_message(PCB_MSG_ERROR, "Padstack conversion: there are no shapes and there is no via participating in the conversion; can not create empty padstack\n");
+			pcb_message(PCB_MSG_ERROR, "Padstack conversion: too many objects selected\n");
+		goto quit;
+	}
+
+	if ((ts->len == 0) && (via == NULL) && (pstk == NULL)) {
+		if (!quiet)
+			pcb_message(PCB_MSG_ERROR, "Padstack conversion: there are no shapes and there is no via/padstack participating in the conversion; can not create empty padstack\n");
 		goto quit;
 	}
 
@@ -209,6 +257,8 @@ static int pcb_pstk_proto_conv(pcb_data_t *data, pcb_pstk_proto_t *dst, int quie
 		ly = (*o)->parent.layer;
 		ts->shape[n].layer_mask = pcb_layer_flags_(ly);
 		ts->shape[n].comb = ly->comb;
+		if (ts->shape[n].layer_mask & (PCB_LYT_PASTE | PCB_LYT_MASK))
+			ts->shape[n].comb |= PCB_LYC_AUTO;
 
 		for(m = 0; m < n; m++) {
 			if ((ts->shape[n].layer_mask == ts->shape[m].layer_mask) && (ts->shape[n].comb == ts->shape[m].comb)) {
@@ -217,7 +267,7 @@ static int pcb_pstk_proto_conv(pcb_data_t *data, pcb_pstk_proto_t *dst, int quie
 				goto quit;
 			}
 		}
-		if ((ts->shape[n].layer_mask & PCB_LYT_COPPER) && (ts->shape[n].layer_mask & PCB_LYT_INTERN) && (via == NULL)) {
+		if ((ts->shape[n].layer_mask & PCB_LYT_COPPER) && (ts->shape[n].layer_mask & PCB_LYT_INTERN) && (via == NULL) && (pstk == NULL)) {
 			if (!quiet)
 				pcb_message(PCB_MSG_ERROR, "Padstack conversion: can not have internal copper shape if there is no hole\n");
 			goto quit;
@@ -233,15 +283,34 @@ static int pcb_pstk_proto_conv(pcb_data_t *data, pcb_pstk_proto_t *dst, int quie
 		if (pcb_pstk_get_shape_idx(ts, PCB_LYT_COPPER | PCB_LYT_BOTTOM, 0) == -1)
 			append_circle(ts,  PCB_LYT_COPPER | PCB_LYT_BOTTOM, 0, via->Thickness);
 		if (via->Mask > 0) {
-			if (pcb_pstk_get_shape_idx(ts, PCB_LYT_MASK | PCB_LYT_BOTTOM, PCB_LYC_SUB) == -1)
-				append_circle(ts,  PCB_LYT_MASK | PCB_LYT_BOTTOM, PCB_LYC_SUB, via->Mask);
-			if (pcb_pstk_get_shape_idx(ts, PCB_LYT_MASK | PCB_LYT_TOP, PCB_LYC_SUB) == -1)
-				append_circle(ts,  PCB_LYT_MASK | PCB_LYT_TOP, PCB_LYC_SUB, via->Mask);
+			if (pcb_pstk_get_shape_idx(ts, PCB_LYT_MASK | PCB_LYT_BOTTOM, PCB_LYC_SUB | PCB_LYC_AUTO) == -1)
+				append_circle(ts,  PCB_LYT_MASK | PCB_LYT_BOTTOM, PCB_LYC_SUB | PCB_LYC_AUTO, via->Mask);
+			if (pcb_pstk_get_shape_idx(ts, PCB_LYT_MASK | PCB_LYT_TOP, PCB_LYC_SUB | PCB_LYC_AUTO) == -1)
+				append_circle(ts,  PCB_LYT_MASK | PCB_LYT_TOP, PCB_LYC_SUB | PCB_LYC_AUTO, via->Mask);
+		}
+	}
+
+	/* if there was a padstack, use the padstack's shape on layers that are not specified */
+	if (pstk != NULL) {
+		int srci;
+		ts_src = pcb_pstk_get_tshape(pstk);
+		if (ts_src != NULL) {
+#			define MAYBE_COPY(mask, comb) \
+				if ((pcb_pstk_get_shape_idx(ts, mask, comb) == -1) && ((srci = pcb_pstk_get_shape_idx(ts_src, mask, comb)) != -1)) \
+					append_tshape(ts, ts_src, srci);
+			MAYBE_COPY(PCB_LYT_COPPER | PCB_LYT_TOP, 0);
+			MAYBE_COPY(PCB_LYT_COPPER | PCB_LYT_INTERN, 0);
+			MAYBE_COPY(PCB_LYT_COPPER | PCB_LYT_BOTTOM, 0);
+			MAYBE_COPY(PCB_LYT_MASK | PCB_LYT_BOTTOM, PCB_LYC_SUB | PCB_LYC_AUTO);
+			MAYBE_COPY(PCB_LYT_MASK | PCB_LYT_TOP, PCB_LYC_SUB | PCB_LYC_AUTO);
+			MAYBE_COPY(PCB_LYT_PASTE | PCB_LYT_BOTTOM, PCB_LYC_AUTO);
+			MAYBE_COPY(PCB_LYT_PASTE | PCB_LYT_TOP, PCB_LYC_AUTO);
+#			undef MAYBE_COPY
 		}
 	}
 
 	/* all went fine */
-	dst->hash = pcb_pstk_hash(dst);
+	dst->hash = pcb_pstk_proto_hash(dst);
 	ret = 0;
 
 	quit:;
@@ -277,7 +346,7 @@ int pcb_pstk_proto_conv_buffer(pcb_pstk_proto_t *dst, int quiet)
 	oy = (bb.Y1 + bb.Y2) / 2;
 
 	vtp0_init(&objs);
-	pcb_data_list_by_flag(PCB_PASTEBUFFER->Data, &objs, PCB_OBJ_CLASS_REAL, PCB_FLAGS);
+	pcb_data_list_by_flag(PCB_PASTEBUFFER->Data, &objs, PCB_OBJ_CLASS_REAL, 0);
 	ret = pcb_pstk_proto_conv(PCB_PASTEBUFFER->Data, dst, quiet, &objs, ox, oy);
 	vtp0_uninit(&objs);
 
@@ -333,30 +402,36 @@ void pcb_pstk_tshape_copy(pcb_pstk_tshape_t *ts_dst, pcb_pstk_tshape_t *ts_src)
 	}
 }
 
+void pcb_pstk_shape_rot(pcb_pstk_shape_t *sh, double sina, double cosa, double angle)
+{
+	int i;
+
+	switch(sh->shape) {
+		case PCB_PSSH_LINE:
+			pcb_rotate(&sh->data.line.x1, &sh->data.line.y1, 0, 0, cosa, sina);
+			pcb_rotate(&sh->data.line.x2, &sh->data.line.y2, 0, 0, cosa, sina);
+			break;
+		case PCB_PSSH_CIRC:
+			pcb_rotate(&sh->data.circ.x, &sh->data.circ.y, 0, 0, cosa, sina);
+			break;
+		case PCB_PSSH_POLY:
+			if (sh->data.poly.pa != NULL)
+				pcb_polyarea_free(&sh->data.poly.pa);
+			for(i = 0; i < sh->data.poly.len; i++)
+				pcb_rotate(&sh->data.poly.x[i], &sh->data.poly.y[i], 0, 0, cosa, sina);
+			pcb_pstk_shape_update_pa(&sh->data.poly);
+			break;
+	}
+}
+
+
 void pcb_pstk_tshape_rot(pcb_pstk_tshape_t *ts, double angle)
 {
-	int n, i;
+	int n;
 	double cosa = cos(angle / PCB_RAD_TO_DEG), sina = sin(angle / PCB_RAD_TO_DEG);
 
-	for(n = 0; n < ts->len; n++) {
-		pcb_pstk_shape_t *sh = &ts->shape[n];
-		switch(sh->shape) {
-			case PCB_PSSH_LINE:
-				pcb_rotate(&sh->data.line.x1, &sh->data.line.y1, 0, 0, cosa, sina);
-				pcb_rotate(&sh->data.line.x2, &sh->data.line.y2, 0, 0, cosa, sina);
-				break;
-			case PCB_PSSH_CIRC:
-				pcb_rotate(&sh->data.circ.x, &sh->data.circ.y, 0, 0, cosa, sina);
-				break;
-			case PCB_PSSH_POLY:
-				if (sh->data.poly.pa != NULL)
-					pcb_polyarea_free(&sh->data.poly.pa);
-				for(i = 0; i < sh->data.poly.len; i++)
-					pcb_rotate(&sh->data.poly.x[i], &sh->data.poly.y[i], 0, 0, cosa, sina);
-				pcb_pstk_shape_update_pa(&sh->data.poly);
-				break;
-		}
-	}
+	for(n = 0; n < ts->len; n++)
+		pcb_pstk_shape_rot(&ts->shape[n], sina, cosa, angle);
 }
 
 void pcb_pstk_tshape_xmirror(pcb_pstk_tshape_t *ts)
@@ -408,16 +483,18 @@ void pcb_pstk_proto_copy(pcb_pstk_proto_t *dst, const pcb_pstk_proto_t *src)
 	memcpy(dst, src, sizeof(pcb_pstk_proto_t));
 	pcb_vtpadstack_tshape_init(&dst->tr);
 
-	ts_src = &src->tr.array[0];
+	if (src->tr.used > 0) {
+		ts_src = &src->tr.array[0];
 
-	/* allocate shapes on the canonical tshape (tr[0]) */
-	ts_dst = pcb_vtpadstack_tshape_alloc_append(&dst->tr, 1);
-	pcb_pstk_tshape_copy(ts_dst, ts_src);
+		/* allocate shapes on the canonical tshape (tr[0]) */
+		ts_dst = pcb_vtpadstack_tshape_alloc_append(&dst->tr, 1);
+		pcb_pstk_tshape_copy(ts_dst, ts_src);
 
-	/* make sure it's the canonical form */
-	ts_dst->rot = 0.0;
-	ts_dst->xmirror = 0;
-	ts_dst->smirror = 0;
+		/* make sure it's the canonical form */
+		ts_dst->rot = 0.0;
+		ts_dst->xmirror = 0;
+		ts_dst->smirror = 0;
+	}
 
 	dst->in_use = 1;
 }
@@ -436,7 +513,7 @@ static pcb_cardinal_t pcb_pstk_proto_insert_try(pcb_data_t *data, const pcb_pstk
 				first_free = n;
 		}
 		else if (data->ps_protos.array[n].hash == proto->hash) {
-			if (pcb_pstk_eq(&data->ps_protos.array[n], proto))
+			if (pcb_pstk_proto_eq(&data->ps_protos.array[n], proto))
 				return n;
 		}
 	}
@@ -693,6 +770,8 @@ void pcb_pstk_shape_grow(pcb_pstk_shape_t *shp, pcb_bool is_absolute, pcb_coord_
 	pcb_coord_t cx, cy;
 	int n;
 
+#warning padstack TODO: undo
+
 	switch(shp->shape) {
 		case PCB_PSSH_LINE:
 			if (is_absolute)
@@ -734,6 +813,15 @@ void pcb_pstk_shape_grow(pcb_pstk_shape_t *shp, pcb_bool is_absolute, pcb_coord_
 	}
 }
 
+void pcb_pstk_shape_clr_grow(pcb_pstk_shape_t *shp, pcb_bool is_absolute, pcb_coord_t val)
+{
+#warning padstack TODO: undo
+	if (is_absolute)
+		shp->clearance = val;
+	else
+		shp->clearance += val;
+}
+
 void pcb_pstk_proto_grow(pcb_pstk_proto_t *proto, pcb_bool is_absolute, pcb_coord_t val)
 {
 	int n, i;
@@ -825,7 +913,7 @@ static unsigned int pcb_pstk_shape_hash(const pcb_pstk_shape_t *sh)
 	return ret;
 }
 
-unsigned int pcb_pstk_hash(const pcb_pstk_proto_t *p)
+unsigned int pcb_pstk_proto_hash(const pcb_pstk_proto_t *p)
 {
 	pcb_pstk_tshape_t *ts = &p->tr.array[0];
 	unsigned int n, ret = pcb_hash_coord(p->hdia) ^ pcb_hash_coord(p->htop) ^ pcb_hash_coord(p->hbottom) ^ pcb_hash_coord(p->hplated) ^ pcb_hash_coord(ts->len);
@@ -869,7 +957,7 @@ static int pcb_pstk_shape_eq(const pcb_pstk_shape_t *sh1, const pcb_pstk_shape_t
 	return 1;
 }
 
-int pcb_pstk_eq(const pcb_pstk_proto_t *p1, const pcb_pstk_proto_t *p2)
+int pcb_pstk_proto_eq(const pcb_pstk_proto_t *p1, const pcb_pstk_proto_t *p2)
 {
 	pcb_pstk_tshape_t *ts1 = &p1->tr.array[0], *ts2 = &p2->tr.array[0];
 	int n1, n2;
@@ -878,14 +966,18 @@ int pcb_pstk_eq(const pcb_pstk_proto_t *p1, const pcb_pstk_proto_t *p2)
 	if (p1->htop != p2->htop) return 0;
 	if (p1->hbottom != p2->hbottom) return 0;
 	if (p1->hplated != p2->hplated) return 0;
-	if (ts1->len != ts2->len) return 0;
+	if ((ts1 == NULL) && (ts2 != NULL)) return 0;
+	if ((ts2 == NULL) && (ts1 != NULL)) return 0;
 
-	for(n1 = 0; n1 < ts1->len; n1++) {
-		for(n2 = 0; n2 < ts2->len; n2++)
-			if (pcb_pstk_shape_eq(ts1->shape + n1, ts2->shape + n2))
-				goto found;
-		return 0;
-		found:;
+	if (ts1 != NULL) {
+		if (ts1->len != ts2->len) return 0;
+		for(n1 = 0; n1 < ts1->len; n1++) {
+			for(n2 = 0; n2 < ts2->len; n2++)
+				if (pcb_pstk_shape_eq(ts1->shape + n1, ts2->shape + n2))
+					goto found;
+			return 0;
+			found:;
+		}
 	}
 
 	return 1;
