@@ -18,6 +18,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  */
 
 #include "config.h"
@@ -31,6 +35,7 @@
 #include "error.h"
 #include "obj_subc.h"
 #include "obj_subc_op.h"
+#include "obj_subc_parent.h"
 #include "obj_poly_op.h"
 #include "obj_pstk.h"
 #include "obj_pstk_inlines.h"
@@ -40,6 +45,7 @@
 #include "obj_text_draw.h"
 #include "rtree.h"
 #include "draw.h"
+#include "draw_wireframe.h"
 #include "flag.h"
 #include "polygon.h"
 #include "operation.h"
@@ -87,6 +93,13 @@ pcb_subc_t *pcb_subc_alloc(void)
 	return sc;
 }
 
+pcb_subc_t *pcb_subc_new(void)
+{
+	pcb_subc_t *sc = pcb_subc_alloc();
+	sc->ID = pcb_create_ID_get();
+	return sc;
+}
+
 void pcb_subc_free(pcb_subc_t *sc)
 {
 	pcb_term_uninit(&sc->terminals);
@@ -136,21 +149,29 @@ static pcb_line_t *find_aux_line(pcb_layer_t *aux, const char *key)
 	return NULL;
 }
 
+/* Find and save the aux layer in the cache, of it exists */
+static void pcb_subc_cache_find_aux(pcb_subc_t *sc, int alloc)
+{
+	int n;
+	for(n = 0; n < sc->data->LayerN; n++) {
+		if (strcmp(sc->data->Layer[n].name, SUBC_AUX_NAME) == 0) {
+			sc->aux_layer = sc->data->Layer + n;
+			return;
+		}
+	}
+
+	if (sc->aux_layer == NULL)
+		sc->aux_layer = pcb_layer_new_bound(sc->data, PCB_LYT_VIRTUAL | PCB_LYT_NOEXPORT | PCB_LYT_MISC | PCB_LYT_TOP, SUBC_AUX_NAME);
+}
+
 /* Looking up aux objects for determining orientation/origin of a subc
    is somewhat expensive. Instead of doing that every time, we just save
    the pointer of those objects/layers. They don't change once the
    subc is loaded into memory */
 static int pcb_subc_cache_update(pcb_subc_t *sc)
 {
-	if (sc->aux_layer == NULL) {
-		int n;
-		for(n = 0; n < sc->data->LayerN; n++) {
-			if (strcmp(sc->data->Layer[n].name, SUBC_AUX_NAME) == 0) {
-				sc->aux_layer = sc->data->Layer + n;
-				break;
-			}
-		}
-	}
+	if (sc->aux_layer == NULL)
+		pcb_subc_cache_find_aux(sc, 0);
 
 	if (sc->aux_cache[0] != NULL)
 		return 0;
@@ -167,6 +188,25 @@ static int pcb_subc_cache_update(pcb_subc_t *sc)
 	return 0;
 }
 
+pcb_bool pcb_subc_find_aux_point(pcb_subc_t *sc, const char *role, pcb_coord_t *x, pcb_coord_t *y)
+{
+	pcb_line_t *l;
+
+	if (sc->aux_layer == NULL)
+		pcb_subc_cache_update(sc);
+	if (sc->aux_layer == NULL)
+		return pcb_false;
+
+	l = find_aux_line(sc->aux_layer, role);
+	if (l == NULL)
+		return pcb_false;
+
+	*x = l->Point1.X;
+	*y = l->Point1.Y;
+
+	return pcb_true;
+}
+
 int pcb_subc_get_origin(pcb_subc_t *sc, pcb_coord_t *x, pcb_coord_t *y)
 {
 	if ((pcb_subc_cache_update(sc) != 0) || (sc->aux_cache[PCB_SUBCH_ORIGIN] == NULL))
@@ -186,7 +226,7 @@ int pcb_subc_get_rotation(pcb_subc_t *sc, double *rot)
 	return 0;
 }
 
-/* The subc is originally drawn on the top side, and teh aux layer gets
+/* The subc is originally drawn on the top side, and the aux layer gets
    the PCB_LYT_TOP flag. If the subc is ever sent to the other side, the
    aux layer changed side too. This is to be detected here. */
 int pcb_subc_get_side(pcb_subc_t *sc, int *on_bottom)
@@ -206,6 +246,33 @@ int pcb_subc_get_side(pcb_subc_t *sc, int *on_bottom)
 	return 0;
 }
 
+int pcb_subc_get_host_trans(const pcb_subc_t *sc, pcb_host_trans_t *tr)
+{
+	int res = 0;
+	double rr;
+
+	if (pcb_subc_get_origin(sc, &tr->ox, &tr->oy) != 0) {
+		tr->ox = tr->oy = 0;
+		res = -1;
+	}
+
+	if (pcb_subc_get_rotation(sc, &tr->rot) != 0) {
+		tr->rot = 0;
+		res = -1;
+	}
+	
+	if (pcb_subc_get_side(sc, &tr->on_bottom) != 0) {
+		tr->on_bottom = 0;
+		res = -1;
+	}
+
+	rr = tr->rot / PCB_RAD_TO_DEG;
+	tr->cosa = cos(rr);
+	tr->sina = sin(rr);
+
+	return res;
+}
+
 static void pcb_subc_cache_invalidate(pcb_subc_t *sc)
 {
 	sc->aux_cache[0] = NULL;
@@ -287,6 +354,47 @@ static pcb_coord_t read_mask(pcb_any_obj_t *obj)
 	return mask;
 }
 
+static void get_aux_layer(pcb_subc_t *sc, int alloc)
+{
+	if (sc->aux_layer == NULL)
+		pcb_subc_cache_find_aux(sc, 1);
+	pcb_subc_cache_update(sc);
+}
+
+void pcb_subc_create_aux(pcb_subc_t *sc, pcb_coord_t ox, pcb_coord_t oy, double rot, pcb_bool bottom)
+{
+	double unit = PCB_MM_TO_COORD(1);
+	double cs, sn;
+
+	get_aux_layer(sc, 1);
+
+	if (rot == 0.0) {
+		cs = 1;
+		sn = 0;
+	}
+	else {
+		cs = cos(rot/PCB_RAD_TO_DEG);
+		sn = sin(rot/PCB_RAD_TO_DEG);
+	}
+
+	if (bottom) {
+		assert(sc->aux_layer->is_bound);
+		sc->aux_layer->meta.bound.type &= ~PCB_LYT_TOP;
+		sc->aux_layer->meta.bound.type |= PCB_LYT_BOTTOM;
+		cs = -cs;
+	}
+
+	add_aux_line(sc->aux_layer, "subc-role", "origin", ox, oy, ox, oy);
+	add_aux_line(sc->aux_layer, "subc-role", "x", ox, oy, pcb_round((double)ox + cs*unit), pcb_round(oy + sn*unit));
+	add_aux_line(sc->aux_layer, "subc-role", "y", ox, oy, pcb_round((double)ox + sn*unit), pcb_round(oy + cs*unit));
+}
+
+void pcb_subc_create_aux_point(pcb_subc_t *sc, pcb_coord_t x, pcb_coord_t y, const char *role)
+{
+	get_aux_layer(sc, 1);
+	add_aux_line(sc->aux_layer, "subc-role", role, x, y, x, y);
+}
+
 int pcb_subc_convert_from_buffer(pcb_buffer_t *buffer)
 {
 	pcb_subc_t *sc;
@@ -509,15 +617,7 @@ int pcb_subc_convert_from_buffer(pcb_buffer_t *buffer)
 	vtp0_uninit(&mask_pads);
 	vtp0_uninit(&paste_pads);
 
-	/* create aux layer */
-	{
-		pcb_coord_t unit = PCB_MM_TO_COORD(1);
-		pcb_layer_t *aux = pcb_layer_new_bound(sc->data, PCB_LYT_VIRTUAL | PCB_LYT_NOEXPORT | PCB_LYT_MISC | PCB_LYT_TOP, SUBC_AUX_NAME);
-
-		add_aux_line(aux, "subc-role", "origin", buffer->X, buffer->Y, buffer->X, buffer->Y);
-		add_aux_line(aux, "subc-role", "x", buffer->X, buffer->Y, buffer->X+unit, buffer->Y);
-		add_aux_line(aux, "subc-role", "y", buffer->X, buffer->Y, buffer->X, buffer->Y+unit);
-	}
+	pcb_subc_create_aux(sc, buffer->X, buffer->Y, 0.0, pcb_false);
 
 	/* Add refdes */
 	pcb_attribute_put(&sc->Attributes, "refdes", "U0");
@@ -534,7 +634,7 @@ int pcb_subc_convert_from_buffer(pcb_buffer_t *buffer)
 	return 0;
 }
 
-static void pcb_subc_draw_origin(pcb_subc_t *sc, pcb_coord_t DX, pcb_coord_t DY)
+static void pcb_subc_draw_origin(pcb_hid_gc_t GC, pcb_subc_t *sc, pcb_coord_t DX, pcb_coord_t DY)
 {
 	pcb_line_t *origin;
 	pcb_subc_cache_update(sc);
@@ -547,10 +647,10 @@ static void pcb_subc_draw_origin(pcb_subc_t *sc, pcb_coord_t DX, pcb_coord_t DY)
 	DX += (origin->Point1.X + origin->Point2.X) / 2;
 	DY += (origin->Point1.Y + origin->Point2.Y) / 2;
 
-	pcb_gui->draw_line(pcb_crosshair.GC, DX - PCB_EMARK_SIZE, DY, DX, DY - PCB_EMARK_SIZE);
-	pcb_gui->draw_line(pcb_crosshair.GC, DX + PCB_EMARK_SIZE, DY, DX, DY - PCB_EMARK_SIZE);
-	pcb_gui->draw_line(pcb_crosshair.GC, DX - PCB_EMARK_SIZE, DY, DX, DY + PCB_EMARK_SIZE);
-	pcb_gui->draw_line(pcb_crosshair.GC, DX + PCB_EMARK_SIZE, DY, DX, DY + PCB_EMARK_SIZE);
+	pcb_gui->draw_line(GC, DX - PCB_EMARK_SIZE, DY, DX, DY - PCB_EMARK_SIZE);
+	pcb_gui->draw_line(GC, DX + PCB_EMARK_SIZE, DY, DX, DY - PCB_EMARK_SIZE);
+	pcb_gui->draw_line(GC, DX - PCB_EMARK_SIZE, DY, DX, DY + PCB_EMARK_SIZE);
+	pcb_gui->draw_line(GC, DX + PCB_EMARK_SIZE, DY, DX, DY + PCB_EMARK_SIZE);
 }
 
 void XORDrawSubc(pcb_subc_t *sc, pcb_coord_t DX, pcb_coord_t DY, int use_curr_side)
@@ -577,12 +677,32 @@ void XORDrawSubc(pcb_subc_t *sc, pcb_coord_t DX, pcb_coord_t DY, int use_curr_si
 		gdl_iterator_t it;
 
 		linelist_foreach(&ly->Line, &it, line)
-			pcb_gui->draw_line(pcb_crosshair.GC, DX + PCB_CSWAP_X(line->Point1.X, w, mirr), DY + PCB_CSWAP_Y(line->Point1.Y, h, mirr), DX + PCB_CSWAP_X(line->Point2.X, w, mirr), DY + PCB_CSWAP_Y(line->Point2.Y, h, mirr));
+			pcb_draw_wireframe_line(pcb_crosshair.GC, 
+															DX + PCB_CSWAP_X(line->Point1.X, w, mirr), 
+															DY + PCB_CSWAP_Y(line->Point1.Y, h, mirr), 
+															DX + PCB_CSWAP_X(line->Point2.X, w, mirr), 
+															DY + PCB_CSWAP_Y(line->Point2.Y, h, mirr),
+															line->Thickness,
+															0 );
 
 		arclist_foreach(&ly->Arc, &it, arc) {
-			double sa = mirr ? PCB_SWAP_ANGLE(arc->StartAngle) : arc->StartAngle;
-			double da = mirr ? PCB_SWAP_DELTA(arc->Delta) : arc->Delta;
-			pcb_gui->draw_arc(pcb_crosshair.GC, DX + PCB_CSWAP_X(arc->X, w, mirr), DY + PCB_CSWAP_Y(arc->Y, h, mirr), arc->Width, arc->Height, sa, da);
+			if(arc->Width != arc->Height) {
+#warning TODO: The wireframe arc drawing code cannot draw ellipses yet so draw the elliptical arc with a thin line 
+				double sa = mirr ? PCB_SWAP_ANGLE(arc->StartAngle) : arc->StartAngle;
+				double da = mirr ? PCB_SWAP_DELTA(arc->Delta) : arc->Delta;
+				pcb_gui->draw_arc(pcb_crosshair.GC, DX + PCB_CSWAP_X(arc->X, w, mirr), DY + PCB_CSWAP_Y(arc->Y, h, mirr), arc->Width, arc->Height, sa, da);
+			}
+			else {
+				pcb_arc_t temp_arc;
+				temp_arc.StartAngle = mirr ? PCB_SWAP_ANGLE(arc->StartAngle) : arc->StartAngle; 
+				temp_arc.Delta = mirr ? PCB_SWAP_DELTA(arc->Delta) : arc->Delta;
+				temp_arc.X = DX + PCB_CSWAP_X(arc->X, w, mirr);
+				temp_arc.Y = DY + PCB_CSWAP_Y(arc->Y, h, mirr);
+				temp_arc.Width = arc->Width;
+				temp_arc.Height = arc->Height;
+				temp_arc.Thickness = arc->Thickness;
+				pcb_draw_wireframe_arc(pcb_crosshair.GC, &temp_arc);
+			}
 		}
 
 		polylist_foreach(&ly->Polygon, &it, poly)
@@ -643,7 +763,7 @@ void XORDrawSubc(pcb_subc_t *sc, pcb_coord_t DX, pcb_coord_t DY, int use_curr_si
 		}
 	}
 
-	pcb_subc_draw_origin(sc, DX, DY);
+	pcb_subc_draw_origin(pcb_crosshair.GC, sc, DX, DY);
 }
 
 #define MAYBE_KEEP_ID(dst, src) \
@@ -714,7 +834,7 @@ pcb_subc_t *pcb_subc_dup_at(pcb_board_t *pcb, pcb_data_t *dst, pcb_subc_t *src,
 				if (!(dl->meta.bound.type & PCB_LYT_VIRTUAL)) {
 					const char *name = dl->name;
 					if (name == NULL) name = "<anonymous>";
-					pcb_message(PCB_MSG_WARNING, "Couldn't bind a layer %s of subcricuit while placing it\n", name);
+					pcb_message(PCB_MSG_WARNING, "Couldn't bind a layer %s of subcircuit while placing it\n", name);
 				}
 			}
 			else
@@ -763,10 +883,10 @@ pcb_subc_t *pcb_subc_dup_at(pcb_board_t *pcb, pcb_data_t *dst, pcb_subc_t *src,
 	/* bind the via rtree so that vias added in this subc show up on the board */
 	if (pcb != NULL) {
 		if (pcb->Data->via_tree == NULL)
-			pcb->Data->via_tree = pcb_r_create_tree(NULL, 0, 0);
+			pcb->Data->via_tree = pcb_r_create_tree();
 		sc->data->via_tree = pcb->Data->via_tree;
 		if (pcb->Data->padstack_tree == NULL)
-			pcb->Data->padstack_tree = pcb_r_create_tree(NULL, 0, 0);
+			pcb->Data->padstack_tree = pcb_r_create_tree();
 		sc->data->padstack_tree = pcb->Data->padstack_tree;
 	}
 
@@ -816,8 +936,8 @@ pcb_subc_t *pcb_subc_dup_at(pcb_board_t *pcb, pcb_data_t *dst, pcb_subc_t *src,
 
 	if (pcb != NULL) {
 		if (!dst->subc_tree)
-			dst->subc_tree = pcb_r_create_tree(NULL, 0, 0);
-		pcb_r_insert_entry(dst->subc_tree, (pcb_box_t *)sc, 0);
+			dst->subc_tree = pcb_r_create_tree();
+		pcb_r_insert_entry(dst->subc_tree, (pcb_box_t *)sc);
 	}
 
 	return sc;
@@ -852,18 +972,18 @@ void *pcb_subc_op(pcb_data_t *Data, pcb_subc_t *sc, pcb_opfunc_t *opfunc, pcb_op
 {
 	int n;
 
-	sc->BoundingBox.X1 = sc->BoundingBox.Y1 = PCB_MAX_COORD;
-	sc->BoundingBox.X2 = sc->BoundingBox.Y2 = -PCB_MAX_COORD;
-
 	EraseSubc(sc);
-
 	if (pcb_data_get_top(Data) != NULL) {
 		if (Data->subc_tree != NULL)
 			pcb_r_delete_entry(Data->subc_tree, (pcb_box_t *)sc);
 		else
-			Data->subc_tree = pcb_r_create_tree(NULL, 0, 0);
+			Data->subc_tree = pcb_r_create_tree();
 	}
 
+	/* for calculating the new bounding box on the fly */
+	sc->BoundingBox.X1 = sc->BoundingBox.Y1 = PCB_MAX_COORD;
+	sc->BoundingBox.X2 = sc->BoundingBox.Y2 = -PCB_MAX_COORD;
+
 	/* execute on layer locals */
 	for(n = 0; n < sc->data->LayerN; n++) {
 		pcb_layer_t *sl = sc->data->Layer + n;
@@ -916,7 +1036,7 @@ void *pcb_subc_op(pcb_data_t *Data, pcb_subc_t *sc, pcb_opfunc_t *opfunc, pcb_op
 
 	pcb_close_box(&sc->BoundingBox);
 	if (pcb_data_get_top(Data) != NULL)
-		pcb_r_insert_entry(Data->subc_tree, (pcb_box_t *)sc, 0);
+		pcb_r_insert_entry(Data->subc_tree, (pcb_box_t *)sc);
 	DrawSubc(sc);
 	return sc;
 }
@@ -937,6 +1057,7 @@ void *pcb_subcop_copy(pcb_opctx_t *ctx, pcb_subc_t *src)
 		/* move-to-the-other-side is not undoable: it's part of the placement */
 		pcb_undo_inc_serial();
 		last = pcb_undo_serial();
+#warning subc TODO: should not depend on crosshair, because of automatic/scripted placement; test case 1: load subc footprint in buffer, swap side to bottom, place; test case 2: bug_files/cmd_element, execute the cmd while being on the bottom side, without crosshair set subcircuits catapult to negative y
 		pcb_subc_change_side(&sc, 2 * pcb_crosshair.Y - PCB->MaxHeight);
 		pcb_undo_truncate_from(last);
 		
@@ -944,7 +1065,7 @@ void *pcb_subcop_copy(pcb_opctx_t *ctx, pcb_subc_t *src)
 
 	pcb_text_dyn_bbox_update(sc->data);
 
-	return (sc);
+	return sc;
 }
 
 extern pcb_opfunc_t ClipFunctions, MoveFunctions, MoveFunctions_noclip, Rotate90Functions, RotateFunctions, ChgFlagFunctions, ChangeSizeFunctions, ChangeClearSizeFunctions, Change1stSizeFunctions, Change2ndSizeFunctions;
@@ -994,7 +1115,7 @@ void pcb_subc_rotate90(pcb_subc_t *subc, pcb_coord_t cx, pcb_coord_t cy, int ste
 	ctx.rotate.center_x = cx;
 	ctx.rotate.center_y = cy;
 	ctx.rotate.number = steps;
-	pcb_subcop_rotate(&ctx, subc);
+	pcb_subcop_rotate90(&ctx, subc);
 }
 
 void pcb_subc_rotate(pcb_subc_t *subc, pcb_coord_t cx, pcb_coord_t cy, double cosa, double sina, double angle)
@@ -1044,6 +1165,7 @@ static int subc_relocate_layer_objs(pcb_layer_t *dl, pcb_data_t *src_data, pcb_l
 	pcb_arc_t *arc;
 	gdl_iterator_t it;
 	int chg = 0;
+	pcb_data_t *dst_data = dl == NULL ? NULL : dl->parent;
 
 	linelist_foreach(&sl->Line, &it, line) {
 		if (src_has_real_layer) {
@@ -1053,9 +1175,11 @@ static int subc_relocate_layer_objs(pcb_layer_t *dl, pcb_data_t *src_data, pcb_l
 		}
 		PCB_FLAG_CLEAR(PCB_FLAG_WARN | PCB_FLAG_FOUND | PCB_FLAG_SELECTED, line);
 		if ((dl != NULL) && (dl->line_tree != NULL)) {
-			pcb_r_insert_entry(dl->line_tree, (pcb_box_t *)line, 0);
+			pcb_r_insert_entry(dl->line_tree, (pcb_box_t *)line);
 			chg++;
 		}
+		if (dst_is_pcb && (dl != NULL))
+			pcb_poly_clear_from_poly(dst_data, PCB_TYPE_LINE, dl, line);
 	}
 
 	arclist_foreach(&sl->Arc, &it, arc) {
@@ -1066,9 +1190,11 @@ static int subc_relocate_layer_objs(pcb_layer_t *dl, pcb_data_t *src_data, pcb_l
 		}
 		PCB_FLAG_CLEAR(PCB_FLAG_WARN | PCB_FLAG_FOUND | PCB_FLAG_SELECTED, arc);
 		if ((dl != NULL) && (dl->arc_tree != NULL)) {
-			pcb_r_insert_entry(dl->arc_tree, (pcb_box_t *)arc, 0);
+			pcb_r_insert_entry(dl->arc_tree, (pcb_box_t *)arc);
 			chg++;
 		}
+		if (dst_is_pcb && (dl != NULL))
+			pcb_poly_clear_from_poly(dst_data, PCB_TYPE_ARC, dl, arc);
 	}
 
 	textlist_foreach(&sl->Text, &it, text) {
@@ -1079,9 +1205,11 @@ static int subc_relocate_layer_objs(pcb_layer_t *dl, pcb_data_t *src_data, pcb_l
 		}
 		PCB_FLAG_CLEAR(PCB_FLAG_WARN | PCB_FLAG_FOUND | PCB_FLAG_SELECTED, text);
 		if ((dl != NULL) && (dl->text_tree != NULL)) {
-			pcb_r_insert_entry(dl->text_tree, (pcb_box_t *)text, 0);
+			pcb_r_insert_entry(dl->text_tree, (pcb_box_t *)text);
 			chg++;
 		}
+		if (dst_is_pcb && (dl != NULL))
+			pcb_poly_clear_from_poly(dst_data, PCB_TYPE_TEXT, dl, text);
 	}
 
 	polylist_foreach(&sl->Polygon, &it, poly) {
@@ -1092,11 +1220,11 @@ static int subc_relocate_layer_objs(pcb_layer_t *dl, pcb_data_t *src_data, pcb_l
 		}
 		PCB_FLAG_CLEAR(PCB_FLAG_WARN | PCB_FLAG_FOUND | PCB_FLAG_SELECTED, poly);
 		if ((dl != NULL) && (dl->polygon_tree != NULL)) {
-			pcb_r_insert_entry(dl->polygon_tree, (pcb_box_t *)poly, 0);
+			pcb_r_insert_entry(dl->polygon_tree, (pcb_box_t *)poly);
 			chg++;
 		}
-		if (dst_is_pcb)
-			pcb_poly_ppclear(poly);
+		if (dst_is_pcb && (dl != NULL))
+			pcb_poly_ppclear_at(poly, dl);
 	}
 
 	if (!dst_is_pcb) {
@@ -1112,7 +1240,65 @@ static int subc_relocate_layer_objs(pcb_layer_t *dl, pcb_data_t *src_data, pcb_l
 	return chg;
 }
 
-void *pcb_subcop_move_to_buffer(pcb_opctx_t *ctx, pcb_subc_t *sc)
+static int subc_relocate_globals(pcb_data_t *dst, pcb_data_t *new_parent, pcb_subc_t *sc, int dst_is_pcb)
+{
+	pcb_pin_t *via;
+	pcb_pstk_t *ps;
+	gdl_iterator_t it;
+	int chg = 0;
+
+	pinlist_foreach(&sc->data->Via, &it, via) {
+		if (sc->data->via_tree != NULL)
+			pcb_r_delete_entry(sc->data->via_tree, (pcb_box_t *)via);
+		PCB_FLAG_CLEAR(PCB_FLAG_WARN | PCB_FLAG_FOUND | PCB_FLAG_SELECTED, via);
+		if ((dst != NULL) && (dst->via_tree != NULL))
+			pcb_r_insert_entry(dst->via_tree, (pcb_box_t *)via);
+		chg++;
+	}
+
+	padstacklist_foreach(&sc->data->padstack, &it, ps) {
+		const pcb_pstk_proto_t *proto = pcb_pstk_get_proto(ps);
+		assert(proto != NULL); /* the prototype must be accessible at the source else we can't add it in the dest */
+		pcb_poly_restore_to_poly(ps->parent.data, PCB_TYPE_PSTK, NULL, ps);
+		if (sc->data->padstack_tree != NULL)
+			pcb_r_delete_entry(sc->data->padstack_tree, (pcb_box_t *)ps);
+		PCB_FLAG_CLEAR(PCB_FLAG_WARN | PCB_FLAG_FOUND | PCB_FLAG_SELECTED, ps);
+		if ((dst != NULL) && (dst->padstack_tree != NULL))
+			pcb_r_insert_entry(dst->padstack_tree, (pcb_box_t *)ps);
+		if (dst != NULL)
+			ps->proto = pcb_pstk_proto_insert_dup(ps->parent.data, proto, 1);
+		ps->protoi = -1;
+		ps->parent.data = new_parent;
+		if (dst_is_pcb)
+			pcb_poly_clear_from_poly(ps->parent.data, PCB_TYPE_PSTK, NULL, ps);
+		chg++;
+	}
+
+	/* bind globals */
+	if ((dst != NULL) && (dst_is_pcb)) {
+		if (dst->via_tree == NULL)
+			dst->via_tree = pcb_r_create_tree();
+		sc->data->via_tree = dst->via_tree;
+
+		if (dst->padstack_tree == NULL)
+			dst->padstack_tree = pcb_r_create_tree();
+		sc->data->padstack_tree = dst->padstack_tree;
+		chg++;
+	}
+	else {
+		sc->data->via_tree = NULL;
+		sc->data->padstack_tree = NULL;
+	}
+	return chg;
+}
+
+/* change the parent of all global objects to new_parent */
+static void subc_set_parent_globals(pcb_subc_t *sc, pcb_data_t *new_parent)
+{
+	pcb_data_set_parent_globals(sc->data, new_parent);
+}
+
+void *pcb_subcop_move_buffer(pcb_opctx_t *ctx, pcb_subc_t *sc)
 {
 	int n;
 	pcb_board_t *dst_top = pcb_data_get_top(ctx->buffer.dst);
@@ -1129,10 +1315,13 @@ void *pcb_subcop_move_to_buffer(pcb_opctx_t *ctx, pcb_subc_t *sc)
 
 	if (dst_is_pcb) {
 		if (ctx->buffer.dst->subc_tree == NULL)
-			ctx->buffer.dst->subc_tree = pcb_r_create_tree(NULL, 0, 0);
-		pcb_r_insert_entry(ctx->buffer.dst->subc_tree, (pcb_box_t *)sc, 0);
+			ctx->buffer.dst->subc_tree = pcb_r_create_tree();
+		pcb_r_insert_entry(ctx->buffer.dst->subc_tree, (pcb_box_t *)sc);
 	}
 
+	if (dst_is_pcb)
+		PCB_SET_PARENT(sc, data, ctx->buffer.dst);  /* have to set sc parent before relocate globals so that poly clearings will work */
+
 	/* move layer local */
 	for(n = 0; n < sc->data->LayerN; n++) {
 		pcb_layer_t *sl = sc->data->Layer + n;
@@ -1146,11 +1335,12 @@ void *pcb_subcop_move_to_buffer(pcb_opctx_t *ctx, pcb_subc_t *sc)
 			}
 			else {
 				/* need to create the trees so that move and other ops work */
-				if (sl->line_tree == NULL) sl->line_tree = pcb_r_create_tree(NULL, 0, 0);
-				if (sl->arc_tree == NULL) sl->arc_tree = pcb_r_create_tree(NULL, 0, 0);
-				if (sl->text_tree == NULL) sl->text_tree = pcb_r_create_tree(NULL, 0, 0);
-				if (sl->polygon_tree == NULL) sl->polygon_tree = pcb_r_create_tree(NULL, 0, 0);
-				pcb_message(PCB_MSG_ERROR, "Couldn't bind subc layer %s on buffer move\n", sl->name == NULL ? "<anonymous>" : sl->name);
+				if (sl->line_tree == NULL) sl->line_tree = pcb_r_create_tree();
+				if (sl->arc_tree == NULL) sl->arc_tree = pcb_r_create_tree();
+				if (sl->text_tree == NULL) sl->text_tree = pcb_r_create_tree();
+				if (sl->polygon_tree == NULL) sl->polygon_tree = pcb_r_create_tree();
+				if (!(sl->meta.bound.type & PCB_LYT_VIRTUAL))
+					pcb_message(PCB_MSG_ERROR, "Couldn't bind subc layer %s on buffer move\n", sl->name == NULL ? "<anonymous>" : sl->name);
 			}
 		}
 		else
@@ -1159,46 +1349,7 @@ void *pcb_subcop_move_to_buffer(pcb_opctx_t *ctx, pcb_subc_t *sc)
 		subc_relocate_layer_objs(dl, ctx->buffer.src, sl, src_has_real_layer, dst_is_pcb);
 	}
 
-	/* move globals */
-	{
-		pcb_pin_t *via;
-		pcb_pstk_t *ps;
-		gdl_iterator_t it;
-
-		pinlist_foreach(&sc->data->Via, &it, via) {
-			if (sc->data->via_tree != NULL)
-				pcb_r_delete_entry(sc->data->via_tree, (pcb_box_t *)via);
-			PCB_FLAG_CLEAR(PCB_FLAG_WARN | PCB_FLAG_FOUND | PCB_FLAG_SELECTED, via);
-			if (ctx->buffer.dst->via_tree != NULL)
-				pcb_r_insert_entry(ctx->buffer.dst->via_tree, (pcb_box_t *)via, 0);
-		}
-
-		padstacklist_foreach(&sc->data->padstack, &it, ps) {
-			const pcb_pstk_proto_t *proto = pcb_pstk_get_proto(ps);
-			if (sc->data->padstack_tree != NULL)
-				pcb_r_delete_entry(sc->data->padstack_tree, (pcb_box_t *)ps);
-			PCB_FLAG_CLEAR(PCB_FLAG_WARN | PCB_FLAG_FOUND | PCB_FLAG_SELECTED, ps);
-			if (ctx->buffer.dst->padstack_tree != NULL)
-				pcb_r_insert_entry(ctx->buffer.dst->padstack_tree, (pcb_box_t *)ps, 0);
-			ps->proto = pcb_pstk_proto_insert_dup(ctx->buffer.dst, proto, 1);
-			ps->protoi = -1;
-		}
-	}
-
-	/* bind globals */
-	if (dst_is_pcb) {
-		if (ctx->buffer.dst->via_tree == NULL)
-			ctx->buffer.dst->via_tree = pcb_r_create_tree(NULL, 0, 0);
-		sc->data->via_tree = ctx->buffer.dst->via_tree;
-
-		if (ctx->buffer.dst->padstack_tree == NULL)
-			ctx->buffer.dst->padstack_tree = pcb_r_create_tree(NULL, 0, 0);
-		sc->data->padstack_tree = ctx->buffer.dst->padstack_tree;
-	}
-	else {
-		sc->data->via_tree = NULL;
-		sc->data->padstack_tree = NULL;
-	}
+	subc_relocate_globals(ctx->buffer.dst, sc->data, sc, dst_is_pcb);
 
 	PCB_FLAG_CLEAR(PCB_FLAG_WARN | PCB_FLAG_FOUND | PCB_FLAG_SELECTED, sc);
 	PCB_SET_PARENT(sc, data, ctx->buffer.dst);
@@ -1218,19 +1369,31 @@ void *pcb_subcop_add_to_buffer(pcb_opctx_t *ctx, pcb_subc_t *sc)
 pcb_bool pcb_subc_smash_buffer(pcb_buffer_t *buff)
 {
 	pcb_subc_t *subc;
+	int n;
 
 	if (pcb_subclist_length(&buff->Data->subc) != 1)
-		return (pcb_false);
+		return pcb_false;
 
 	subc = pcb_subclist_first(&buff->Data->subc);
 	pcb_subclist_remove(subc);
 
+	/* relocate to NULL to get trees detached */
+	for(n = 0; n < subc->data->LayerN; n++) {
+		pcb_layer_t *sl = subc->data->Layer + n;
+		int src_has_real_layer = (sl->meta.bound.real != NULL);
+		subc_relocate_layer_objs(NULL, subc->data, sl, src_has_real_layer, 0);
+	}
+	subc_relocate_globals(NULL, NULL, subc, 0);
+
+
 	pcb_data_free(buff->Data);
 	buff->Data = subc->data;
 	buff->Data->parent_type = PCB_PARENT_INVALID;
 	buff->Data->parent.data = NULL;
 
-	return (pcb_true);
+	subc_set_parent_globals(subc, buff->Data);
+
+	return pcb_true;
 }
 
 int pcb_subc_rebind(pcb_board_t *pcb, pcb_subc_t *sc)
@@ -1244,7 +1407,7 @@ int pcb_subc_rebind(pcb_board_t *pcb, pcb_subc_t *sc)
 
 	EraseSubc(sc);
 
-	/* relocate bindings */
+/* relocate bindings */
 	for(n = 0; n < sc->data->LayerN; n++) {
 		pcb_layer_t *sl = sc->data->Layer + n;
 		pcb_layer_t *dl = pcb_layer_resolve_binding(dst_top, sl);
@@ -1255,10 +1418,10 @@ int pcb_subc_rebind(pcb_board_t *pcb, pcb_subc_t *sc)
 				continue;
 
 			/* make sure all trees exist on the dest layer - if these are the first objects there we may need to create them */
-			if (dl->line_tree == NULL) dl->line_tree = pcb_r_create_tree(NULL, 0, 0);
-			if (dl->arc_tree == NULL) dl->arc_tree = pcb_r_create_tree(NULL, 0, 0);
-			if (dl->text_tree == NULL) dl->text_tree = pcb_r_create_tree(NULL, 0, 0);
-			if (dl->polygon_tree == NULL) dl->polygon_tree = pcb_r_create_tree(NULL, 0, 0);
+			if (dl->line_tree == NULL) dl->line_tree = pcb_r_create_tree();
+			if (dl->arc_tree == NULL) dl->arc_tree = pcb_r_create_tree();
+			if (dl->text_tree == NULL) dl->text_tree = pcb_r_create_tree();
+			if (dl->polygon_tree == NULL) dl->polygon_tree = pcb_r_create_tree();
 		}
 
 		if (subc_relocate_layer_objs(dl, pcb->Data, sl, src_has_real_layer, 1) > 0)
@@ -1271,6 +1434,17 @@ int pcb_subc_rebind(pcb_board_t *pcb, pcb_subc_t *sc)
 	return chgly;
 }
 
+void pcb_subc_bind_globals(pcb_board_t *pcb, pcb_subc_t *subc)
+{
+	if (pcb->Data->via_tree == NULL)
+		pcb->Data->via_tree = pcb_r_create_tree();
+	subc->data->via_tree = pcb->Data->via_tree;
+	if (pcb->Data->padstack_tree == NULL)
+		pcb->Data->padstack_tree = pcb_r_create_tree();
+	subc->data->padstack_tree = pcb->Data->padstack_tree;
+#warning subc TODO: subc-in-subc: bind subc rtree
+}
+
 void *pcb_subcop_change_size(pcb_opctx_t *ctx, pcb_subc_t *sc)
 {
 	pcb_subc_op(ctx->chgsize.pcb->Data, sc, &ChangeSizeFunctions, ctx);
@@ -1299,15 +1473,20 @@ void *pcb_subcop_change_nonetlist(pcb_opctx_t *ctx, pcb_subc_t *sc)
 {
 #warning subc TODO: add undo
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, sc))
-		return (NULL);
+		return NULL;
 	PCB_FLAG_TOGGLE(PCB_FLAG_NONETLIST, sc);
 	return sc;
 }
 
 void *pcb_subcop_change_name(pcb_opctx_t *ctx, pcb_subc_t *sc)
 {
+	void *old;
+	if (sc->refdes != NULL)
+		old = pcb_strdup(sc->refdes); /* strdup because the pcb_attribute_put() is going to free the original */
+	else
+		old = NULL;
 	pcb_attribute_put(&sc->Attributes, "refdes", ctx->chgname.new_name);
-	return sc;
+	return old;
 }
 
 void *pcb_subcop_destroy(pcb_opctx_t *ctx, pcb_subc_t *sc)
@@ -1324,22 +1503,22 @@ void *pcb_subcop_destroy(pcb_opctx_t *ctx, pcb_subc_t *sc)
 void *pcb_subcop_remove(pcb_opctx_t *ctx, pcb_subc_t *sc)
 {
 	EraseSubc(sc);
-	if (!ctx->remove.bulk)
-		pcb_draw();
 	pcb_undo_move_obj_to_remove(PCB_TYPE_SUBC, sc, sc, sc);
 	return NULL;
 }
 
 void *pcb_subc_remove(pcb_subc_t *sc)
 {
+	void *res;
 	pcb_opctx_t ctx;
 
 	ctx.remove.pcb = PCB;
-	ctx.remove.bulk = pcb_false;
 	ctx.remove.destroy_target = NULL;
 /*	PCB_CLEAR_PARENT(subc);*/
 
-	return pcb_subcop_remove(&ctx, sc);
+	res = pcb_subcop_remove(&ctx, sc);
+	pcb_draw();
+	return res;
 }
 
 
@@ -1426,7 +1605,7 @@ void pcb_subc_mirror(pcb_data_t *data, pcb_subc_t *subc, pcb_coord_t y_offs, pcb
 	pcb_subc_bbox(subc);
 
 	if ((data != NULL) && (data->subc_tree != NULL))
-		pcb_r_insert_entry(data->subc_tree, (pcb_box_t *)subc, 0);
+		pcb_r_insert_entry(data->subc_tree, (pcb_box_t *)subc);
 }
 
 pcb_bool pcb_subc_change_side(pcb_subc_t **subc, pcb_coord_t yoff)
@@ -1438,7 +1617,7 @@ pcb_bool pcb_subc_change_side(pcb_subc_t **subc, pcb_coord_t yoff)
 	pcb_data_t *data, *oldhack;
 
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, *subc))
-		return (pcb_false);
+		return pcb_false;
 
 	assert((*subc)->parent_type = PCB_PARENT_DATA);
 	data = (*subc)->parent.data;
@@ -1452,17 +1631,15 @@ pcb_bool pcb_subc_change_side(pcb_subc_t **subc, pcb_coord_t yoff)
 
 	oldhack = pcb_pstk_data_hack;
 	pcb_pstk_data_hack = ctx.buffer.dst;
-	newsc = pcb_subcop_move_to_buffer(&ctx, *subc);
+	newsc = pcb_subcop_move_buffer(&ctx, *subc);
 
 
 	/* mirror object geometry and stackup */
 	pcb_subc_mirror(NULL, newsc, yoff, pcb_true);
 	for(n = 0; n < newsc->data->LayerN; n++) {
 		pcb_layer_t *ly = newsc->data->Layer + n;
-		if (ly->meta.bound.type & PCB_LYT_TOP)
-			ly->meta.bound.type = (ly->meta.bound.type & ~PCB_LYT_TOP) | PCB_LYT_BOTTOM;
-		else if (ly->meta.bound.type & PCB_LYT_BOTTOM)
-			ly->meta.bound.type = (ly->meta.bound.type & ~PCB_LYT_BOTTOM) | PCB_LYT_TOP;
+		ly->meta.bound.type = pcb_layer_mirror_type(ly->meta.bound.type);
+		ly->meta.bound.stack_offs = -ly->meta.bound.stack_offs;
 	}
 
 	/* place the new subc */
@@ -1491,10 +1668,10 @@ pcb_r_dir_t draw_subc_mark_callback(const pcb_box_t *b, void *cl)
 	pcb_gui->set_line_cap(pcb_draw_out.fgGC, Trace_Cap);
 	pcb_gui->set_line_width(pcb_draw_out.fgGC, 0);
 	pcb_gui->set_draw_xor(pcb_draw_out.fgGC, 1);
-	pcb_subc_draw_origin(subc, 0, 0);
+	pcb_gui->set_color(pcb_draw_out.fgGC, selected ? conf_core.appearance.color.subc_selected : conf_core.appearance.color.subc);
+	pcb_subc_draw_origin(pcb_draw_out.fgGC, subc, 0, 0);
 	pcb_gui->set_draw_xor(pcb_draw_out.fgGC, 0);
 
-	pcb_gui->set_color(pcb_draw_out.fgGC, selected ? conf_core.appearance.color.subc_selected : conf_core.appearance.color.subc);
 	pcb_gui->set_line_width(pcb_draw_out.fgGC, 0);
 	pcb_gui->set_draw_xor(pcb_draw_out.fgGC, 1);
 	pcb_draw_dashed_line(pcb_draw_out.fgGC, bb->X1, bb->Y1, bb->X2, bb->Y1);
@@ -1504,18 +1681,55 @@ pcb_r_dir_t draw_subc_mark_callback(const pcb_box_t *b, void *cl)
 	pcb_gui->set_draw_xor(pcb_draw_out.fgGC, 0);
 
 	if (subc->refdes != NULL) {
-		pcb_coord_t x0, y0;
+		pcb_coord_t x0, y0, dx, dy;
+		pcb_font_t *font = &PCB->fontkit.dflt;
 
-		if (conf_core.editor.view.flip_x)
+		dx = font->MaxWidth/2;
+		dy = font->MaxHeight/2;
+
+		if (conf_core.editor.view.flip_x) {
 			x0 = bb->X2;
+			dx = -dx;
+		}
 		else
 			x0 = bb->X1;
-		if (conf_core.editor.view.flip_y)
+
+		if (conf_core.editor.view.flip_y) {
 			y0 = bb->Y2;
+			dy = -dy;
+		}
 		else
 			y0 = bb->Y1;
 
-		pcb_term_label_draw(x0, y0, 50.0, 0, 0, subc->refdes, subc->intconn);
+		if ((conf_core.editor.subc_id != NULL) && (*conf_core.editor.subc_id != '\0')) {
+			static gds_t s;
+			s.used = 0;
+			if (pcb_append_dyntext(&s, (pcb_any_obj_t *)subc, conf_core.editor.subc_id) == 0) {
+				char *curr, *next;
+				pcb_coord_t x = x0, y = y0;
+
+				for(curr = s.array; curr != NULL; curr = next) {
+					int ctrl = 0;
+					next = strchr(curr, '\\');
+					if (next != NULL) {
+						*next = '\0';
+						next++;
+						ctrl = 1;
+					}
+					pcb_term_label_draw(x, y, 50.0, 0, 0, curr, subc->intconn);
+					if (ctrl) {
+						switch(*next) {
+							case 'n': y += dy; x = x0; break;
+						}
+						next++;
+					}
+				}
+			}
+			else
+				pcb_term_label_draw(x0, y0, 50.0, 0, 0, "<err>", subc->intconn);
+		}
+		else
+			pcb_term_label_draw(x0, y0, 50.0, 0, 0, subc->refdes, subc->intconn);
 	}
 
 	return PCB_R_DIR_FOUND_CONTINUE;
@@ -1525,18 +1739,43 @@ void pcb_subc_draw_preview(const pcb_subc_t *sc, const pcb_box_t *drawn_area)
 {
 	int n;
 	pcb_pstk_draw_t ctx;
+	pcb_rtree_it_t it;
+	pcb_any_obj_t *o;
 
+	/* draw copper only first - order doesn't matter */
 	for(n = 0; n < sc->data->LayerN; n++) {
 		pcb_layer_t *layer = &sc->data->Layer[n];
-		if (layer->meta.bound.type & (PCB_LYT_COPPER | PCB_LYT_SILK | PCB_LYT_OUTLINE))
-			pcb_draw_layer(layer, drawn_area);
+		if (layer->meta.bound.type & PCB_LYT_COPPER)
+			pcb_draw_layer_under(layer, drawn_area, sc->data);
 	}
 
+	/* draw padstacks */
 	ctx.pcb = NULL;
 	ctx.gid = -1;
 	ctx.is_current = 1;
 	ctx.comb = 0;
-	pcb_r_search(sc->data->padstack_tree, drawn_area, NULL, pcb_pstk_draw_callback, &ctx, NULL);
+	ctx.shape_mask = PCB_LYT_COPPER | PCB_LYT_TOP;
+	ctx.holetype = PCB_PHOLE_UNPLATED | PCB_PHOLE_PLATED;
+
+	for(o = pcb_rtree_first(&it, sc->data->padstack_tree, (pcb_rtree_box_t *)drawn_area); o != NULL; o = pcb_rtree_next(&it)) {
+		if (pcb_obj_is_under(o, sc->data)) {
+			pcb_pstk_draw_callback((pcb_box_t *)o, &ctx);
+			if (PCB->hole_on)
+				pcb_pstk_draw_hole_callback((pcb_box_t *)o, &ctx);
+		}
+	}
+
+	/* draw silk and outline, above padstacks */
+	for(n = 0; n < sc->data->LayerN; n++) {
+		pcb_layer_t *layer = &sc->data->Layer[n];
+		if (layer->meta.bound.type & (PCB_LYT_SILK | PCB_LYT_OUTLINE))
+			pcb_draw_layer_under(layer, drawn_area, sc->data);
+	}
+
+	/* padstack mark goes on top */
+	for(o = pcb_rtree_first(&it, sc->data->padstack_tree, (pcb_rtree_box_t *)drawn_area); o != NULL; o = pcb_rtree_next(&it))
+		if (pcb_obj_is_under(o, sc->data))
+			pcb_pstk_draw_mark_callback((pcb_box_t *)o, &ctx);
 }
 
 
@@ -1570,3 +1809,8 @@ pcb_subc_t *pcb_subc_by_id(pcb_data_t *base, long int ID)
 	PCB_END_LOOP;
 	return NULL;
 }
+
+pcb_bool pcb_subc_is_empty(pcb_subc_t *subc)
+{
+	return pcb_data_is_empty(subc->data);
+}
