@@ -18,11 +18,16 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  */
 
 #ifndef PCB_OBJ_SUBC_H
 #define PCB_OBJ_SUBC_H
 
+#include <genlist/gendlist.h>
 #include <libminuid/libminuid.h>
 #include <genht/htsp.h>
 #include "obj_common.h"
@@ -48,6 +53,7 @@ struct pcb_subc_s {
 };
 
 pcb_subc_t *pcb_subc_alloc(void);
+pcb_subc_t *pcb_subc_new(void);
 void pcb_subc_free(pcb_subc_t *sc);
 
 void pcb_add_subc_to_data(pcb_data_t *dt, pcb_subc_t *sc);
@@ -79,9 +85,12 @@ void pcb_subc_draw_preview(const pcb_subc_t *sc, const pcb_box_t *drawn_area);
 
 void XORDrawSubc(pcb_subc_t *sc, pcb_coord_t DX, pcb_coord_t DY, int use_curr_side);
 
-/* Redo the binding after the layer binding recipe changed in sc */
+/* Redo the layer binding after the layer binding recipe changed in sc */
 int pcb_subc_rebind(pcb_board_t *pcb, pcb_subc_t *sc);
 
+/* Do the initial global bindings of subc to pcb (rtree links) */
+void pcb_subc_bind_globals(pcb_board_t *pcb, pcb_subc_t *subc);
+
 #include "rtree.h"
 pcb_r_dir_t draw_subc_mark_callback(const pcb_box_t *b, void *cl);
 void DrawSubc(pcb_subc_t *sc);
@@ -91,6 +100,7 @@ void EraseSubc(pcb_subc_t *sc);
 int pcb_subc_get_origin(pcb_subc_t *sc, pcb_coord_t *x, pcb_coord_t *y);
 int pcb_subc_get_rotation(pcb_subc_t *sc, double *rot);
 int pcb_subc_get_side(pcb_subc_t *sc, int *on_bottom);
+int pcb_subc_get_host_trans(const pcb_subc_t *sc, pcb_host_trans_t *tr);
 
 /* Search for the named subc; name is relative path in hierarchy. Returns
    NULL if not found */
@@ -106,6 +116,24 @@ void *pcb_subc_remove(pcb_subc_t *sc);
    layer that matches brd_layer or brd_layer if not found. */
 pcb_layer_t *pcb_loose_subc_layer(pcb_board_t *pcb, pcb_layer_t *brd_layer);
 
+/* Returns whether there's no object in the subc */
+pcb_bool pcb_subc_is_empty(pcb_subc_t *subc);
+
+/*** subc creation helpers ***/
+
+/* Create the aux layer for a subc, set origin to ox;oy and rotation to rot */
+void pcb_subc_create_aux(pcb_subc_t *sc, pcb_coord_t ox, pcb_coord_t oy, double rot, pcb_bool bottom);
+
+/* Create a new point on the aux layer using a given role string in attribute */
+void pcb_subc_create_aux_point(pcb_subc_t *sc, pcb_coord_t x, pcb_coord_t y, const char *role);
+
+/* Look up an aux point and return pcb_true if found; when found, load x and y
+   with the coords of the point. */
+pcb_bool pcb_subc_find_aux_point(pcb_subc_t *sc, const char *role, pcb_coord_t *x, pcb_coord_t *y);
+
+
+pcb_subc_t *pcb_subc_dup_at(pcb_board_t *pcb, pcb_data_t *dst, pcb_subc_t *src, pcb_coord_t dx, pcb_coord_t dy, pcb_bool keep_ids);
+
 /*** loops ***/
 
 #define PCB_SUBC_LOOP(top) do {                                     \
