@@ -1,7 +1,8 @@
 /*
  *                            COPYRIGHT
  *
- *  PCB, interactive printed circuit board design
+ *  pcb-rnd, interactive printed circuit board design
+ *  (this file is based on PCB, interactive printed circuit board design)
  *  Copyright (C) 1994,1995,1996 Thomas Nau
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -18,9 +19,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- *  Contact addresses for paper mail and Email:
- *  Thomas Nau, Schlehenweg 15, 88471 Baustetten, Germany
- *  Thomas.Nau@rz.uni-ulm.de
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  *
  */
 
@@ -45,6 +47,7 @@
 #include "obj_arc_draw.h"
 
 #include "obj_subc_parent.h"
+#include "obj_hash.h"
 
 /* TODO: remove this if draw.c is moved here: */
 #include "draw.h"
@@ -102,7 +105,7 @@ pcb_text_t *pcb_text_new(pcb_layer_t *Layer, pcb_font_t *PCBFont, pcb_coord_t X,
 
 	pcb_add_text_on_layer(Layer, text, PCBFont);
 
-	return (text);
+	return text;
 }
 
 static pcb_text_t *pcb_text_copy_meta(pcb_text_t *dst, pcb_text_t *src)
@@ -132,8 +135,8 @@ void pcb_add_text_on_layer(pcb_layer_t *Layer, pcb_text_t *text, pcb_font_t *PCB
 	/* calculate size of the bounding box */
 	pcb_text_bbox(PCBFont, text);
 	if (!Layer->text_tree)
-		Layer->text_tree = pcb_r_create_tree(NULL, 0, 0);
-	pcb_r_insert_entry(Layer->text_tree, (pcb_box_t *) text, 0);
+		Layer->text_tree = pcb_r_create_tree();
+	pcb_r_insert_entry(Layer->text_tree, (pcb_box_t *) text);
 }
 
 static int pcb_text_render_str_cb(void *ctx, gds_t *s, const char **input)
@@ -176,10 +179,23 @@ static int pcb_text_render_str_cb(void *ctx, gds_t *s, const char **input)
 /* Render the string of a text, doing substitution if needed - don't allocate if there's no subst */
 static unsigned char *pcb_text_render_str(pcb_text_t *text)
 {
+	unsigned char *res;
+
 	if (!PCB_FLAG_TEST(PCB_FLAG_DYNTEXT, text))
 		return (unsigned char *)text->TextString;
 
-	return (unsigned char *)pcb_strdup_subst(text->TextString, pcb_text_render_str_cb, text, PCB_SUBST_PERCENT | PCB_SUBST_CONF);
+	res = (unsigned char *)pcb_strdup_subst(text->TextString, pcb_text_render_str_cb, text, PCB_SUBST_PERCENT | PCB_SUBST_CONF);
+	if (*res == '\0') {
+		free(res);
+		res = (unsigned char *)pcb_strdup("<?>");
+	}
+
+	return res;
+}
+
+int pcb_append_dyntext(gds_t *dst, pcb_any_obj_t *obj, const char *fmt)
+{
+	return pcb_subst_append(dst, fmt, pcb_text_render_str_cb, obj, PCB_SUBST_PERCENT | PCB_SUBST_CONF, 0);
 }
 
 /* Free rendered if it was allocated */
@@ -325,6 +341,32 @@ void pcb_text_bbox(pcb_font_t *FontPtr, pcb_text_t *Text)
 	pcb_text_free_str(Text, rendered);
 }
 
+int pcb_text_eq(const pcb_host_trans_t *tr1, const pcb_text_t *t1, const pcb_host_trans_t *tr2, const pcb_text_t *t2)
+{
+	if (pcb_neqs(t1->TextString, t2->TextString)) return 0;
+	if (pcb_neqs(t1->term, t2->term)) return 0;
+
+	if (!PCB_FLAG_TEST(PCB_FLAG_FLOATER, t1) && !PCB_FLAG_TEST(PCB_FLAG_FLOATER, t2)) {
+		if (pcb_field_neq(t1, t2, Scale)) return 0;
+		if (pcb_neq_tr_coords(tr1, t1->X, t1->Y, tr2, t2->X, t2->Y)) return 0;
+	}
+
+	return 1;
+}
+
+unsigned int pcb_text_hash(const pcb_host_trans_t *tr, const pcb_text_t *t)
+{
+	unsigned int crd = 0;
+
+	if (!PCB_FLAG_TEST(PCB_FLAG_FLOATER, t)) {
+		pcb_coord_t x, y;
+
+		pcb_hash_tr_coords(tr, &x, &y, t->X, t->Y);
+		crd = pcb_hash_coord(x) ^ pcb_hash_coord(y) ^ pcb_hash_coord(t->Scale);
+	}
+
+	return pcb_hash_str(t->TextString) ^ pcb_hash_str(t->term) ^ crd;
+}
 
 
 /*** ops ***/
@@ -338,25 +380,29 @@ void *pcb_textop_add_to_buffer(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_text_t
 	return t;
 }
 
-/* moves a text to buffer without allocating memory for the name */
-void *pcb_textop_move_to_buffer(pcb_opctx_t *ctx, pcb_layer_t * layer, pcb_text_t * text)
+/* moves a text without allocating memory for the name between board and buffer */
+void *pcb_textop_move_buffer(pcb_opctx_t *ctx, pcb_layer_t *dstly, pcb_text_t *text)
 {
-	pcb_layer_t *lay = &ctx->buffer.dst->Layer[pcb_layer_id(ctx->buffer.src, layer)];
+	pcb_layer_t *srcly = text->parent.layer;
 
-	pcb_r_delete_entry(layer->text_tree, (pcb_box_t *) text);
-	pcb_poly_restore_to_poly(ctx->buffer.src, PCB_TYPE_TEXT, layer, text);
+	assert(text->parent_type == PCB_PARENT_LAYER);
+	if ((dstly == NULL) || (dstly == srcly)) /* auto layer in dst */
+		dstly = &ctx->buffer.dst->Layer[pcb_layer_id(ctx->buffer.src, srcly)];
+
+	pcb_r_delete_entry(srcly->text_tree, (pcb_box_t *) text);
+	pcb_poly_restore_to_poly(ctx->buffer.src, PCB_TYPE_TEXT, srcly, text);
 
 	textlist_remove(text);
-	textlist_append(&lay->Text, text);
+	textlist_append(&dstly->Text, text);
 
-	if (!lay->text_tree)
-		lay->text_tree = pcb_r_create_tree(NULL, 0, 0);
-	pcb_r_insert_entry(lay->text_tree, (pcb_box_t *) text, 0);
-	pcb_poly_clear_from_poly(ctx->buffer.dst, PCB_TYPE_TEXT, lay, text);
+	if (!dstly->text_tree)
+		dstly->text_tree = pcb_r_create_tree();
+	pcb_r_insert_entry(dstly->text_tree, (pcb_box_t *) text);
+	pcb_poly_clear_from_poly(ctx->buffer.dst, PCB_TYPE_TEXT, dstly, text);
 
-	PCB_SET_PARENT(text, layer, lay);
+	PCB_SET_PARENT(text, layer, dstly);
 
-	return (text);
+	return text;
 }
 
 /* changes the scaling factor of a text object */
@@ -366,7 +412,7 @@ void *pcb_textop_change_size(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_text_t *T
 		: Text->Scale + PCB_COORD_TO_MIL(ctx->chgsize.value);
 
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, Text))
-		return (NULL);
+		return NULL;
 	if (value <= PCB_MAX_TEXTSCALE && value >= PCB_MIN_TEXTSCALE && value != Text->Scale) {
 		pcb_undo_add_obj_to_size(PCB_TYPE_TEXT, Layer, Text, Text);
 		pcb_text_invalidate_erase(Layer, Text);
@@ -374,12 +420,12 @@ void *pcb_textop_change_size(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_text_t *T
 		pcb_poly_restore_to_poly(PCB->Data, PCB_TYPE_TEXT, Layer, Text);
 		Text->Scale = value;
 		pcb_text_bbox(pcb_font(PCB, Text->fid, 1), Text);
-		pcb_r_insert_entry(Layer->text_tree, (pcb_box_t *) Text, 0);
+		pcb_r_insert_entry(Layer->text_tree, (pcb_box_t *) Text);
 		pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_TEXT, Layer, Text);
 		pcb_text_invalidate_draw(Layer, Text);
-		return (Text);
+		return Text;
 	}
-	return (NULL);
+	return NULL;
 }
 
 /* sets data of a text object and calculates bounding box; memory must have
@@ -389,7 +435,7 @@ void *pcb_textop_change_name(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_text_t *T
 	char *old = Text->TextString;
 
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, Text))
-		return (NULL);
+		return NULL;
 	pcb_text_invalidate_erase(Layer, Text);
 	pcb_r_delete_entry(Layer->text_tree, (pcb_box_t *)Text);
 	pcb_poly_restore_to_poly(PCB->Data, PCB_TYPE_TEXT, Layer, Text);
@@ -397,17 +443,17 @@ void *pcb_textop_change_name(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_text_t *T
 
 	/* calculate size of the bounding box */
 	pcb_text_bbox(pcb_font(PCB, Text->fid, 1), Text);
-	pcb_r_insert_entry(Layer->text_tree, (pcb_box_t *) Text, 0);
+	pcb_r_insert_entry(Layer->text_tree, (pcb_box_t *) Text);
 	pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_TEXT, Layer, Text);
 	pcb_text_invalidate_draw(Layer, Text);
-	return (old);
+	return old;
 }
 
 /* changes the clearance flag of a text */
 void *pcb_textop_change_join(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_text_t *Text)
 {
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, Text))
-		return (NULL);
+		return NULL;
 	pcb_text_invalidate_erase(Layer, Text);
 	if (PCB_FLAG_TEST(PCB_FLAG_CLEARLINE, Text)) {
 		pcb_undo_add_obj_to_clear_poly(PCB_TYPE_TEXT, Layer, Text, Text, pcb_false);
@@ -420,14 +466,14 @@ void *pcb_textop_change_join(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_text_t *T
 		pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_TEXT, Layer, Text);
 	}
 	pcb_text_invalidate_draw(Layer, Text);
-	return (Text);
+	return Text;
 }
 
 /* sets the clearance flag of a text */
 void *pcb_textop_set_join(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_text_t *Text)
 {
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, Text) || PCB_FLAG_TEST(PCB_FLAG_CLEARLINE, Text))
-		return (NULL);
+		return NULL;
 	return pcb_textop_change_join(ctx, Layer, Text);
 }
 
@@ -435,7 +481,7 @@ void *pcb_textop_set_join(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_text_t *Text
 void *pcb_textop_clear_join(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_text_t *Text)
 {
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, Text) || !PCB_FLAG_TEST(PCB_FLAG_CLEARLINE, Text))
-		return (NULL);
+		return NULL;
 	return pcb_textop_change_join(ctx, Layer, Text);
 }
 
@@ -449,7 +495,7 @@ void *pcb_textop_copy(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_text_t *Text)
 	pcb_text_copy_meta(text, Text);
 	pcb_text_invalidate_draw(Layer, text);
 	pcb_undo_add_obj_to_create(PCB_TYPE_TEXT, Layer, text, text);
-	return (text);
+	return text;
 }
 
 /* moves a text object */
@@ -459,7 +505,6 @@ void *pcb_textop_move_noclip(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_text_t *T
 		pcb_text_invalidate_erase(Layer, Text);
 		pcb_text_move(Text, ctx->move.dx, ctx->move.dy);
 		pcb_text_invalidate_draw(Layer, Text);
-		pcb_draw();
 	}
 	else
 		pcb_text_move(Text, ctx->move.dx, ctx->move.dy);
@@ -471,7 +516,7 @@ void *pcb_textop_move(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_text_t *Text)
 	pcb_r_delete_entry(Layer->text_tree, (pcb_box_t *) Text);
 	pcb_poly_restore_to_poly(PCB->Data, PCB_TYPE_TEXT, Layer, Text);
 	pcb_textop_move_noclip(ctx, Layer, Text);
-	pcb_r_insert_entry(Layer->text_tree, (pcb_box_t *) Text, 0);
+	pcb_r_insert_entry(Layer->text_tree, (pcb_box_t *) Text);
 	pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_TEXT, Layer, Text);
 	return Text;
 }
@@ -483,7 +528,7 @@ void *pcb_textop_clip(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_text_t *Text)
 		pcb_poly_restore_to_poly(PCB->Data, PCB_TYPE_TEXT, Layer, Text);
 	}
 	if (ctx->clip.clear) {
-		pcb_r_insert_entry(Layer->text_tree, (pcb_box_t *) Text, 0);
+		pcb_r_insert_entry(Layer->text_tree, (pcb_box_t *) Text);
 		pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_TEXT, Layer, Text);
 	}
 	return Text;
@@ -507,8 +552,8 @@ void *pcb_textop_move_to_layer_low(pcb_opctx_t *ctx, pcb_layer_t * Source, pcb_t
 	/* re-calculate the bounding box (it could be mirrored now) */
 	pcb_text_bbox(pcb_font(PCB, text->fid, 1), text);
 	if (!Destination->text_tree)
-		Destination->text_tree = pcb_r_create_tree(NULL, 0, 0);
-	pcb_r_insert_entry(Destination->text_tree, (pcb_box_t *) text, 0);
+		Destination->text_tree = pcb_r_create_tree();
+	pcb_r_insert_entry(Destination->text_tree, (pcb_box_t *) text);
 	pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_TEXT, Destination, text);
 
 	PCB_SET_PARENT(text, layer, Destination);
@@ -530,8 +575,6 @@ void *pcb_textop_move_to_layer(pcb_opctx_t *ctx, pcb_layer_t * layer, pcb_text_t
 		text = pcb_textop_move_to_layer_low(ctx, layer, text, ctx->move.dst_layer);
 		if (ctx->move.dst_layer->meta.real.vis)
 			pcb_text_invalidate_draw(ctx->move.dst_layer, text);
-		if (layer->meta.real.vis || ctx->move.dst_layer->meta.real.vis)
-			pcb_draw();
 	}
 	return text;
 }
@@ -554,8 +597,6 @@ void *pcb_textop_remove(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_text_t *Text)
 	if (Layer->meta.real.vis) {
 		pcb_text_invalidate_erase(Layer, Text);
 		pcb_r_delete_entry(Layer->text_tree, (pcb_box_t *)Text);
-		if (!ctx->remove.bulk)
-			pcb_draw();
 	}
 	pcb_undo_move_obj_to_remove(PCB_TYPE_TEXT, Layer, Text, Text);
 	return NULL;
@@ -563,13 +604,15 @@ void *pcb_textop_remove(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_text_t *Text)
 
 void *pcb_text_destroy(pcb_layer_t *Layer, pcb_text_t *Text)
 {
+	void *res;
 	pcb_opctx_t ctx;
 
 	ctx.remove.pcb = PCB;
-	ctx.remove.bulk = pcb_false;
 	ctx.remove.destroy_target = NULL;
 
-	return pcb_textop_remove(&ctx, Layer, Text);
+	res = pcb_textop_remove(&ctx, Layer, Text);
+	pcb_draw();
+	return res;
 }
 
 /* rotates a text in 90 degree steps; only the bounding box is rotated,
@@ -598,11 +641,10 @@ void *pcb_textop_rotate90(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_text_t *Text
 	pcb_poly_restore_to_poly(PCB->Data, PCB_TYPE_TEXT, Layer, Text);
 	pcb_r_delete_entry(Layer->text_tree, (pcb_box_t *) Text);
 	pcb_text_rotate90(Text, ctx->rotate.center_x, ctx->rotate.center_y, ctx->rotate.number);
-	pcb_r_insert_entry(Layer->text_tree, (pcb_box_t *) Text, 0);
+	pcb_r_insert_entry(Layer->text_tree, (pcb_box_t *) Text);
 	pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_TEXT, Layer, Text);
 	pcb_text_invalidate_draw(Layer, Text);
-	pcb_draw();
-	return (Text);
+	return Text;
 }
 
 void *pcb_textop_rotate(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_text_t *Text)
@@ -619,10 +661,10 @@ void *pcb_textop_rotate(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_text_t *Text)
 	pcb_rotate(&Text->X, &Text->Y, ctx->rotate.center_x, ctx->rotate.center_y, ctx->rotate.cosa, ctx->rotate.sina);
 	pcb_text_bbox(NULL, Text);
 
-	pcb_r_insert_entry(Layer->text_tree, (pcb_box_t *) Text, 0);
+	pcb_r_insert_entry(Layer->text_tree, (pcb_box_t *) Text);
 	pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_TEXT, Layer, Text);
 	pcb_text_invalidate_draw(Layer, Text);
-	return (Text);
+	return Text;
 }
 
 void pcb_text_flip_side(pcb_layer_t *layer, pcb_text_t *text, pcb_coord_t y_offs)
@@ -634,7 +676,7 @@ void pcb_text_flip_side(pcb_layer_t *layer, pcb_text_t *text, pcb_coord_t y_offs
 	PCB_FLAG_TOGGLE(PCB_FLAG_ONSOLDER, text);
 	pcb_text_bbox(pcb_font(PCB, text->fid, 1), text);
 	if (layer->text_tree != NULL)
-		pcb_r_insert_entry(layer->text_tree, (pcb_box_t *) text, 0);
+		pcb_r_insert_entry(layer->text_tree, (pcb_box_t *) text);
 }
 
 void pcb_text_set_font(pcb_layer_t *layer, pcb_text_t *text, pcb_font_id_t fid)
@@ -643,7 +685,7 @@ void pcb_text_set_font(pcb_layer_t *layer, pcb_text_t *text, pcb_font_id_t fid)
 	pcb_r_delete_entry(layer->text_tree, (pcb_box_t *) text);
 	text->fid = fid;
 	pcb_text_bbox(pcb_font(PCB, text->fid, 1), text);
-	pcb_r_insert_entry(layer->text_tree, (pcb_box_t *) text, 0);
+	pcb_r_insert_entry(layer->text_tree, (pcb_box_t *) text);
 	pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_TEXT, layer, text);
 }
 
@@ -658,7 +700,7 @@ void pcb_text_update(pcb_layer_t *layer, pcb_text_t *text)
 	pcb_poly_restore_to_poly(data, PCB_TYPE_TEXT, layer, text);
 	pcb_r_delete_entry(layer->text_tree, (pcb_box_t *) text);
 	pcb_text_bbox(pcb_font(pcb, text->fid, 1), text);
-	pcb_r_insert_entry(layer->text_tree, (pcb_box_t *) text, 0);
+	pcb_r_insert_entry(layer->text_tree, (pcb_box_t *) text);
 	pcb_poly_clear_from_poly(data, PCB_TYPE_TEXT, layer, text);
 }
 
@@ -842,7 +884,10 @@ static void DrawTextLowLevel_(pcb_text_t *Text, pcb_coord_t min_line_width, int
 			defaultsymbol.Y1 += Text->Y;
 			defaultsymbol.X2 += Text->X;
 			defaultsymbol.Y2 += Text->Y;
-			pcb_gui->fill_rect(pcb_draw_out.fgGC, defaultsymbol.X1, defaultsymbol.Y1, defaultsymbol.X2, defaultsymbol.Y2);
+			if (xordraw)
+				pcb_gui->draw_rect(pcb_crosshair.GC, xordx+defaultsymbol.X1, xordy+defaultsymbol.Y1, xordx+defaultsymbol.X2, xordy+defaultsymbol.Y2);
+			else
+				pcb_gui->fill_rect(pcb_draw_out.fgGC, defaultsymbol.X1, defaultsymbol.Y1, defaultsymbol.X2, defaultsymbol.Y2);
 
 			/* move on to next cursor position */
 			x += size;
@@ -901,14 +946,21 @@ static void pcb_text_draw(pcb_layer_t *layer, pcb_text_t *text, int allow_term_g
 	int min_silk_line;
 	unsigned int flg = 0;
 
-	if (PCB_FLAG_TEST(PCB_FLAG_SELECTED, text))
-		pcb_gui->set_color(pcb_draw_out.fgGC, layer->meta.real.selected_color);
+	if (PCB_FLAG_TEST(PCB_FLAG_SELECTED, text)) {
+		if (layer->is_bound) {
+			const char *color;
+			PCB_OBJ_COLOR_ON_BOUND_LAYER(color, layer, 1);
+			pcb_gui->set_color(pcb_draw_out.fgGC, color);
+		}
+		else
+			pcb_gui->set_color(pcb_draw_out.fgGC, layer->meta.real.selected_color);
+	}
 	else if (PCB_HAS_COLOROVERRIDE(text)) {
 		pcb_gui->set_color(pcb_draw_out.fgGC, text->override_color);
 	}
 	else if (layer->is_bound) {
 		const char *color;
-		PCB_OBJ_COLOR_ON_BOUND_LAYER(color, layer);
+		PCB_OBJ_COLOR_ON_BOUND_LAYER(color, layer, 0);
 		pcb_gui->set_color(pcb_draw_out.fgGC, color);
 	}
 	else
@@ -930,6 +982,9 @@ pcb_r_dir_t pcb_text_draw_callback(const pcb_box_t * b, void *cl)
 	pcb_layer_t *layer = cl;
 	pcb_text_t *text = (pcb_text_t *) b;
 
+	if (pcb_hidden_floater((pcb_any_obj_t*)b))
+		return PCB_R_DIR_FOUND_CONTINUE;
+
 	if (!PCB->SubcPartsOn && pcb_lobj_parent_subc(text->parent_type, &text->parent))
 		return PCB_R_DIR_FOUND_CONTINUE;
 
@@ -998,7 +1053,7 @@ static int pcb_text_font_chg_data(pcb_data_t *data, pcb_font_id_t fid)
 			need_redraw = 1;
 			pcb_r_delete_entry(data->subc_tree, (pcb_box_t *)subc);
 			pcb_subc_bbox(subc);
-			pcb_r_insert_entry(data->subc_tree, (pcb_box_t *)subc, 0);
+			pcb_r_insert_entry(data->subc_tree, (pcb_box_t *)subc);
 		}
 	} PCB_END_LOOP;
 
