@@ -1,7 +1,8 @@
 /*
  *                            COPYRIGHT
  *
- *  PCB, interactive printed circuit board design
+ *  pcb-rnd, interactive printed circuit board design
+ *  (this file is based on PCB, interactive printed circuit board design)
  *  Copyright (C) 1994,1995,1996 Thomas Nau
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -18,9 +19,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- *  Contact addresses for paper mail and Email:
- *  Thomas Nau, Schlehenweg 15, 88471 Baustetten, Germany
- *  Thomas.Nau@rz.uni-ulm.de
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  *
  */
 
@@ -31,6 +33,7 @@
 
 #include "obj_common.h"
 #include "font.h"
+#include <genvector/gds_char.h>
 
 struct pcb_text_s {
 	PCB_ANYOBJECTFIELDS;
@@ -69,6 +72,13 @@ void pcb_text_update(pcb_layer_t *layer, pcb_text_t *text);
    directly under data - useful e.g. on parent attr change */
 void pcb_text_dyn_bbox_update(pcb_data_t *data);
 
+/* hash and eq */
+int pcb_text_eq(const pcb_host_trans_t *tr1, const pcb_text_t *t1, const pcb_host_trans_t *tr2, const pcb_text_t *t2);
+unsigned int pcb_text_hash(const pcb_host_trans_t *tr, const pcb_text_t *t);
+
+/* Append dyntext fmt rendered from the perspective of obj */
+int pcb_append_dyntext(gds_t *dst, pcb_any_obj_t *obj, const char *fmt);
+
 void pcb_text_init(void);
 void pcb_text_uninit(void);
 
@@ -86,7 +96,7 @@ void pcb_text_uninit(void);
 #define PCB_TEXT_LOOP(layer) do {                                       \
   pcb_text_t *text;                                                   \
   gdl_iterator_t __it__;                                            \
-  linelist_foreach(&(layer)->Text, &__it__, text) {
+  textlist_foreach(&(layer)->Text, &__it__, text) {
 
 #define	PCB_TEXT_ALL_LOOP(top) do {                        \
 	pcb_cardinal_t l;                                   \
