@@ -1,7 +1,8 @@
 /*
  *                            COPYRIGHT
  *
- *  PCB, interactive printed circuit board design
+ *  pcb-rnd, interactive printed circuit board design
+ *  (this file is based on PCB, interactive printed circuit board design)
  *  Copyright (C) 1994,1995,1996 Thomas Nau
  *  Copyright (C) 1997, 1998, 1999, 2000, 2001 Harry Eaton
  *
@@ -19,7 +20,13 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
- *  Contact addresses for paper mail and Email:
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
+ *
+ *
+ *  Old contact info:
  *  Harry Eaton, 6697 Buttonhole Ct, Columbia, MD 21044, USA
  *  haceaton@aplcomm.jhuapl.edu
  *
@@ -206,7 +213,7 @@ static int pcb_act_Attributes(int argc, const char **argv, pcb_coord_t x, pcb_co
 				if ((pcb_search_screen(x, y, PCB_TYPE_SUBC, &ptrtmp, &ptrtmp, &ptrtmp)) != PCB_TYPE_NONE)
 					s = (pcb_subc_t *)ptrtmp;
 				else {
-					pcb_message(PCB_MSG_ERROR, _("No subcricuit found there\n"));
+					pcb_message(PCB_MSG_ERROR, _("No subcircuit found there\n"));
 					return 1;
 				}
 			}
@@ -248,6 +255,8 @@ from.
 
 static void disperse_obj(pcb_board_t *pcb, pcb_any_obj_t *obj, pcb_coord_t ox, pcb_coord_t oy, pcb_coord_t *dx, pcb_coord_t *dy, pcb_coord_t *minx, pcb_coord_t *miny, pcb_coord_t *maxy)
 {
+	pcb_coord_t newx2, newy2;
+
 	/* If we want to disperse selected objects, maybe we need smarter
 	   code here to avoid putting components on top of others which
 	   are not selected.  For now, I'm assuming that this is typically
@@ -279,10 +288,19 @@ static void disperse_obj(pcb_board_t *pcb, pcb_any_obj_t *obj, pcb_coord_t ox, p
 	*dy -= (oy + *dy) % pcb->Grid;
 	*dy += pcb->Grid;
 
+	/* new X2 and Y2 coords with snapping considered */
+	newx2 = obj->BoundingBox.X2 + *dx;
+	newy2 = obj->BoundingBox.Y2 + *dy;
+
 	/* keep track of how tall this row is */
-	*minx += obj->BoundingBox.X2 - obj->BoundingBox.X1 + GAP;
-	if (*maxy < obj->BoundingBox.Y2)
-		*maxy = obj->BoundingBox.Y2;
+	*minx = newx2 + GAP;
+	if (*maxy < newy2) {
+		*maxy = newy2;
+		if (*maxy > PCB->MaxHeight - GAP) {
+			*maxy = GAP;
+			pcb_message(PCB_MSG_WARNING, "The board is too small for hosting all elements,\ndiesperse restarted from the top.\nExpect overlapping elements\n");
+		}
+	}
 }
 
 static int pcb_act_DisperseElements(int argc, const char **argv, pcb_coord_t x, pcb_coord_t y)
@@ -332,7 +350,7 @@ static int pcb_act_DisperseElements(int argc, const char **argv, pcb_coord_t x,
 	{
 		if (!PCB_FLAG_TEST(PCB_FLAG_LOCK, subc) && (all || PCB_FLAG_TEST(PCB_FLAG_SELECTED, subc))) {
 			pcb_coord_t ox, oy;
-			if (pcb_subc_get_origin(subc, &x, &y) != 0) {
+			if (pcb_subc_get_origin(subc, &ox, &oy) != 0) {
 				ox = (subc->BoundingBox.X1 + subc->BoundingBox.X2)/2;
 				oy = (subc->BoundingBox.Y1 + subc->BoundingBox.Y2)/2;
 			}
@@ -658,6 +676,7 @@ static int pcb_act_ElementList(int argc, const char **argv, pcb_coord_t x, pcb_c
 			ny = PCB->MaxHeight - 1;
 
 		/* Place components onto center of board. */
+		pcb_crosshair.Y = ny; /* flipping side depends on the crosshair unfortunately */
 		if (pcb_buffer_copy_to_layout(PCB, nx, ny))
 			pcb_board_set_changed_flag(pcb_true);
 	}
@@ -1341,6 +1360,85 @@ static int pcb_act_CreateText(int argc, const char **argv, pcb_coord_t x, pcb_co
 	return 0;
 }
 
+static const char pcb_acts_subc[] = "subc(hash, [board|selected])\n";
+static const char pcb_acth_subc[] = "Various operations on subc";
+static int pcb_act_subc(int argc, const char **argv, pcb_coord_t x, pcb_coord_t y)
+{
+	if (argc == 0)
+		PCB_ACT_FAIL(subc);
+	switch (pcb_funchash_get(argv[0], NULL)) {
+		case F_Hash:
+			{
+				int selected_only = 0;
+				gdl_iterator_t it;
+				pcb_subc_t *sc;
+
+				if (argc < 1) {
+				
+				}
+				else if (strcmp(argv[1], "selected") == 0)
+					selected_only = 1;
+
+				polylist_foreach(&PCB->Data->subc, &it, sc) {
+					if (selected_only && !PCB_FLAG_TEST(PCB_FLAG_SELECTED, sc))
+						continue;
+					pcb_message(PCB_MSG_INFO, "subc #%ld (%s): %u\n", sc->ID, (sc->refdes == NULL ? "<no refdes>" : sc->refdes), pcb_subc_hash(sc));
+				}
+			}
+			break;
+		case F_Eq:
+			{
+				int selected_only = 0;
+				gdl_iterator_t it;
+				pcb_subc_t *sc;
+				vtp0_t *vt;
+				htip_t hash2scs; /* hash to subcircuit vector */
+				htip_entry_t *e;
+				gds_t str;
+
+				gds_init(&str);
+				htip_init(&hash2scs, longhash, longkeyeq);
+				if (argc < 1) {
+				
+				}
+				else if (strcmp(argv[1], "selected") == 0)
+					selected_only = 1;
+
+				polylist_foreach(&PCB->Data->subc, &it, sc) {
+					unsigned int hash;
+					if (selected_only && !PCB_FLAG_TEST(PCB_FLAG_SELECTED, sc))
+						continue;
+					hash = pcb_subc_hash(sc);
+					vt = htip_get(&hash2scs, hash);
+					if (vt == 0) {
+						vt = calloc(sizeof(vtp0_t), 1);
+						htip_set(&hash2scs, hash, vt);
+					}
+					vtp0_append(vt, sc);
+				}
+
+				/* print the result */
+				for (e = htip_first(&hash2scs); e; e = htip_next(&hash2scs, e)) {
+					int n;
+
+					vt = e->value;
+					str.used = 0;
+					pcb_append_printf(&str, "subc eq %u:", e->key);
+					for(n = 0; n < vt->used; n++) {
+						sc = (pcb_subc_t *)vt->array[n];
+						pcb_append_printf(&str, " #%ld(%s):%d", sc->ID, (sc->refdes == NULL ? "<no refdes>" : sc->refdes), pcb_subc_eq(sc, (pcb_subc_t*)vt->array[0]));
+					}
+					pcb_message(PCB_MSG_INFO, "%s\n", str.array);
+					vtp0_uninit(vt);
+					free(vt);
+				}
+				gds_uninit(&str);
+				htip_uninit(&hash2scs);
+			}
+			break;
+	}
+	return 0;
+}
 
 pcb_hid_action_t object_action_list[] = {
 	{"Attributes", 0, pcb_act_Attributes,
@@ -1379,6 +1477,9 @@ pcb_hid_action_t object_action_list[] = {
 	{"MoveLayer", 0, pcb_act_MoveLayer,
 	 movelayer_help, movelayer_syntax}
 	,
+	{"subc", 0, pcb_act_subc,
+	 pcb_acth_subc, pcb_acts_subc}
+	,
 	{"CreateText", 0, pcb_act_CreateText,
 	 pcb_acth_CreateText, pcb_acts_CreateText}
 };
