@@ -1,7 +1,8 @@
 /*
  *                            COPYRIGHT
  *
- *  PCB, interactive printed circuit board design
+ *  pcb-rnd, interactive printed circuit board design
+ *  (this file is based on PCB, interactive printed circuit board design)
  *  Copyright (C) 1994,1995,1996 Thomas Nau
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -18,9 +19,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- *  Contact addresses for paper mail and Email:
- *  Thomas Nau, Schlehenweg 15, 88471 Baustetten, Germany
- *  Thomas.Nau@rz.uni-ulm.de
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  *
  */
 
@@ -87,7 +89,7 @@ void *pcb_object_operation(pcb_opfunc_t *F, pcb_opctx_t *ctx, int Type, void *Pt
 
 	case PCB_TYPE_SUBC:
 		if (F->subc)
-			return (F->subc(ctx, (pcb_subc_t *) Ptr1));
+			return (F->subc(ctx, (pcb_subc_t *) Ptr2));
 		break;
 
 	case PCB_TYPE_PIN:
@@ -112,10 +114,10 @@ void *pcb_object_operation(pcb_opfunc_t *F, pcb_opctx_t *ctx, int Type, void *Pt
 
 	case PCB_TYPE_RATLINE:
 		if (F->Rat)
-			return (F->Rat(ctx, (pcb_rat_t *) Ptr1));
+			return (F->Rat(ctx, (pcb_rat_t *) Ptr2));
 		break;
 	}
-	return (NULL);
+	return NULL;
 }
 
 /* ----------------------------------------------------------------------
@@ -331,5 +333,5 @@ pcb_bool pcb_selected_operation(pcb_board_t *pcb, pcb_data_t *data, pcb_opfunc_t
 	if (Reset && changed)
 		pcb_undo_inc_serial();
 
-	return (changed);
+	return changed;
 }
