@@ -1,7 +1,8 @@
 /*
  *                            COPYRIGHT
  *
- *  PCB, interactive printed circuit board design
+ *  pcb-rnd, interactive printed circuit board design
+ *  (this file is based on PCB, interactive printed circuit board design)
  *  Copyright (C) 2011 Andrew Poelstra
  *  Copyright (C) 2016 Tibor 'Igor2' Palinkas
  *
@@ -19,7 +20,13 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- *  Contact addresses for paper mail and Email:
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
+ *
+ *
+ *  Old contact info:
  *  Andrew Poelstra, 16966 60A Ave, V3S 8X5 Surrey, BC, Canada
  *  asp11@sfu.ca
  *
@@ -287,7 +294,7 @@ static human_coord_t human_coord[] = {
 
 static inline int try_human_coord(pcb_coord_t coord, const pcb_unit_t *unit, double down_limit, double up_limit, int score_factor, double *value, unsigned int *best, const char **suffix)
 {
-	double v, frac;
+	double v, frac, save;
 	long int digs, zeros;
 	unsigned int score;
 
@@ -296,7 +303,8 @@ static inline int try_human_coord(pcb_coord_t coord, const pcb_unit_t *unit, dou
 		v = PCB_COORD_TO_MM(coord);
 	else
 		v = PCB_COORD_TO_MIL(coord);
-	v = v * unit->scale_factor;
+	save = v = v * unit->scale_factor;
+	if (v < 0) v = -v;
 
 	/* Check if neighbour units are better */
 	if ((down_limit > 0) && (v < down_limit))
@@ -319,7 +327,7 @@ static inline int try_human_coord(pcb_coord_t coord, const pcb_unit_t *unit, dou
 
 	/* update the best score */
 	if (score > *best) {
-		*value = v;
+		*value = save;
 		*best = score;
 		*suffix = unit->suffix;
 	}
