@@ -1,7 +1,8 @@
 /*
  *                            COPYRIGHT
  *
- *  PCB, interactive printed circuit board design
+ *  pcb-rnd, interactive printed circuit board design
+ *  (this file is based on PCB, interactive printed circuit board design)
  *  Copyright (C) 1994,1995,1996,1997,1998,2005,2006 Thomas Nau
  *  Copyright (C) 2015,2016 Tibor 'Igor2' Palinkas
  *
@@ -19,9 +20,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- *  Contact addresses for paper mail and Email:
- *  Thomas Nau, Schlehenweg 15, 88471 Baustetten, Germany
- *  Thomas.Nau@rz.uni-ulm.de
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  *
  */
 
@@ -63,6 +65,8 @@
 #include "compat_lrealpath.h"
 #include "layer_vis.h"
 #include "safe_fs.h"
+#include "plug_footprint.h"
+#include "build_run.h"
 
 /* for opendir */
 #include "compat_inc.h"
@@ -128,43 +132,40 @@ void pcb_set_design_dir(const char *fn)
 	pcb_conf_ro("rc/path/design");
 }
 
-int pcb_parse_pcb(pcb_board_t *Ptr, const char *Filename, const char *fmt, int load_settings, int ignore_missing)
+static int pcb_test_parse_all(FILE *ft, const char *Filename, const char *fmt, pcb_plug_iot_t type, pcb_find_io_t *available, int *accepts, int *accept_total, int maxav, int ignore_missing, int gen_event)
 {
-	int res = -1, len, n;
-	pcb_find_io_t available[PCB_IO_MAX_FORMATS];
-	int accepts[PCB_IO_MAX_FORMATS]; /* test-parse output */
-	FILE *ft;
-	int accept_total = 0;
+	int len, n;
 
-	ft = pcb_fopen(Filename, "r");
 	if (ft == NULL) {
 		if (!ignore_missing)
 			pcb_message(PCB_MSG_ERROR, "Error: can't open %s for reading (format is %s)\n", Filename, fmt);
 		return -1;
 	}
 
-	if (load_settings)
+	if (gen_event)
 		pcb_event(PCB_EVENT_LOAD_PRE, "s", Filename);
 
-	Ptr->Data->loader = NULL;
-
-	len = pcb_find_io(available, sizeof(available)/sizeof(available[0]), PCB_IOT_PCB, 0, fmt);
+	len = pcb_find_io(available, maxav, type, 0, fmt);
 	if (fmt != NULL) {
 		/* explicit format */
 		for(n = 0; n < len; n++) {
-			if (available[n].plug->parse_pcb != NULL) {
+			void (*f)();
+			switch(type) {
+				case PCB_IOT_PCB: f = (void (*)())available[n].plug->parse_pcb; break;
+				default: assert(!"internal error: pcb_test_parse_all: wrong type"); f = NULL;
+			}
+			if (f != NULL) {
 				accepts[n] = 1; /* force-accept - if it can handle the format, and the user explicitly wanted this format, let's try it */
-				accept_total++;
+				(*accept_total)++;
 			}
 		}
 
-		if (accept_total <= 0) {
+		if (*accept_total <= 0) {
 			pcb_message(PCB_MSG_ERROR, "can't find a IO_ plugin to load a PCB using format %s\n", fmt);
-			fclose(ft);
 			return -1;
 		}
 
-		if (accept_total > 1) {
+		if (*accept_total > 1) {
 			pcb_message(PCB_MSG_INFO, "multiple IO_ plugins can handle format %s - I'm going to try them all, but you may want to be more specific next time; formats found:\n", fmt);
 			for(n = 0; n < len; n++)
 				pcb_message(PCB_MSG_INFO, "    %s\n", available[n].plug->description);
@@ -175,9 +176,9 @@ int pcb_parse_pcb(pcb_board_t *Ptr, const char *Filename, const char *fmt, int l
 		if((fgetc(ft) != EOF)) {
 			rewind(ft);
 			for(n = 0; n < len; n++) {
-				if ((available[n].plug->test_parse_pcb == NULL) || (available[n].plug->test_parse_pcb(available[n].plug, Ptr, Filename, ft))) {
+				if ((available[n].plug->test_parse == NULL) || (available[n].plug->test_parse(available[n].plug, type, Filename, ft))) {
 					accepts[n] = 1;
-					accept_total++;
+					(*accept_total)++;
 				}
 				else
 					accepts[n] = 0;
@@ -185,12 +186,30 @@ int pcb_parse_pcb(pcb_board_t *Ptr, const char *Filename, const char *fmt, int l
 			}
 		}
 	}
-	fclose(ft);
 
-	if (accept_total == 0) {
+	if (*accept_total == 0) {
 		pcb_message(PCB_MSG_ERROR, "none of the IO_ plugin recognized the file format of %s - it's either not a valid board file or does not match the format specified\n", Filename);
 		return -1;
 	}
+	return len;
+}
+
+int pcb_parse_pcb(pcb_board_t *Ptr, const char *Filename, const char *fmt, int load_settings, int ignore_missing)
+{
+	int res = -1, len, n;
+	pcb_find_io_t available[PCB_IO_MAX_FORMATS];
+	int accepts[PCB_IO_MAX_FORMATS]; /* test-parse output */
+	int accept_total = 0;
+	FILE *ft;
+
+	ft = pcb_fopen(Filename, "r");
+	len = pcb_test_parse_all(ft, Filename, fmt, PCB_IOT_PCB, available, accepts, &accept_total, sizeof(available)/sizeof(available[0]), ignore_missing, load_settings);
+	if (ft != NULL)
+		fclose(ft);
+	if (len < 0)
+		return -1;
+
+	Ptr->Data->loader = NULL;
 
 	/* try all plugins that said it could handle the file */
 	for(n = 0; n < len; n++) {
@@ -217,14 +236,41 @@ int pcb_parse_pcb(pcb_board_t *Ptr, const char *Filename, const char *fmt, int l
 	return res;
 }
 
-int pcb_parse_element(pcb_data_t *Ptr, const char *name)
+int pcb_parse_element(pcb_data_t *Ptr, const char *Filename, const char *fmt)
 {
-	int res = -1;
+	int res = -1, len, n;
+	pcb_find_io_t available[PCB_IO_MAX_FORMATS];
+	int accepts[PCB_IO_MAX_FORMATS]; /* test-parse output */
+	int accept_total = 0;
+	FILE *f;
+	pcb_fp_fopen_ctx_t fctx;
+
+	f = pcb_fp_fopen(pcb_fp_default_search_path(), Filename, &fctx);
+	len = pcb_test_parse_all(f, Filename, fmt, PCB_IOT_FOOTPRINT, available, accepts, &accept_total, sizeof(available)/sizeof(available[0]), 0, 0);
+	if (f != NULL)
+		pcb_fp_fclose(f, &fctx);
+	if (len < 0)
+		return -1;
 
 	Ptr->loader = NULL;
-	PCB_HOOK_CALL_DO(pcb_plug_io_t, pcb_plug_io_chain, parse_element, res, == 0, (self, Ptr, name), Ptr->loader = self);
 
-	plug_io_err(res, "load element", name);
+	/* try all plugins that said it could handle the file */
+	for(n = 0; n < len; n++) {
+		if ((available[n].plug->parse_element == NULL) || (!accepts[n])) /* can't parse or doesn't want to parse this file */
+			continue;
+		res = available[n].plug->parse_element(available[n].plug, Ptr, Filename);
+		if (res == 0) {
+			if (Ptr->loader == NULL) /* if the loader didn't set this (to some more fine grained, e.g. depending on file format version) */
+				Ptr->loader = available[n].plug;
+			break;
+		}
+	}
+
+	/* remove selected/found flag when loading reusable footprints */
+	if (res == 0)
+		pcb_data_flag_change(Ptr, PCB_OBJ_CLASS_REAL, PCB_CHGFLG_CLEAR, PCB_FLAG_FOUND | PCB_FLAG_SELECTED);
+
+	plug_io_err(res, "load footprint", Filename);
 	return res;
 }
 
@@ -462,7 +508,8 @@ static int real_load_pcb(const char *Filename, const char *fmt, pcb_bool revert,
 		}
 
 		/* have to be called after pcb_board_resize() so vis update is after a board changed update */
-		pcb_layervis_reset_stack(); 
+		pcb_event(PCB_EVENT_LAYERS_CHANGED, NULL);
+		pcb_layervis_reset_stack();
 
 		/* enable default font if necessary */
 		if (!PCB->fontkit.valid) {
@@ -505,7 +552,7 @@ static int real_load_pcb(const char *Filename, const char *fmt, pcb_bool revert,
 		pcb_gui->log("Loading file %s took %f seconds of CPU time\n", new_filename, elapsed);
 #endif
 
-		return (0);
+		return 0;
 	}
 
 	free(new_filename);
@@ -525,7 +572,7 @@ static int real_load_pcb(const char *Filename, const char *fmt, pcb_bool revert,
 	/* release unused memory */
 	pcb_board_remove(newPCB);
 
-	return (1);
+	return 1;
 }
 
 
@@ -643,7 +690,7 @@ FILE *pcb_check_and_open_file(const char *Filename, pcb_bool Confirm, pcb_bool A
 		if ((fp = pcb_fopen(Filename, "w")) == NULL)
 			pcb_open_error_message(Filename);
 	}
-	return (fp);
+	return fp;
 }
 
 /* ---------------------------------------------------------------------------
@@ -688,7 +735,7 @@ int pcb_save_buffer_elements(const char *Filename, const char *fmt)
 	result = pcb_write_pipe(Filename, pcb_false, fmt, pcb_true);
 	if (PCB_SWAP_IDENT)
 		pcb_buffers_flip_side(PCB);
-	return (result);
+	return result;
 }
 
 /* ---------------------------------------------------------------------------
@@ -714,7 +761,11 @@ int pcb_save_pcb(const char *file, const char *fmt)
  */
 int pcb_load_pcb(const char *file, const char *fmt, pcb_bool require_font, int how)
 {
-	return real_load_pcb(file, fmt, pcb_false, require_font, how);
+	int res = real_load_pcb(file, fmt, pcb_false, require_font, how);
+	if (res == 0)
+		pcb_file_loaded_set_at("design", "main", file, PCB->is_footprint ? "footprint" : "board");
+
+	return res;
 }
 
 /* ---------------------------------------------------------------------------
@@ -932,7 +983,7 @@ int pcb_write_pcb_file(const char *Filename, pcb_bool thePcb, const char *fmt, p
 			unlink(fn_tmp);
 		free(fn_tmp);
 	}
-	return (result);
+	return result;
 }
 
 
@@ -1020,6 +1071,20 @@ void pcb_io_list_free(pcb_io_formats_t *out)
 	}
 }
 
+pcb_cardinal_t pcb_io_incompat_save(pcb_data_t *data, pcb_any_obj_t *obj, const char *desc, const char *hint)
+{
+	pcb_message(PCB_MSG_ERROR, "save error: %s\n", desc);
+	if (obj != NULL) {
+		pcb_coord_t x = (obj->BoundingBox.X1 + obj->BoundingBox.X2)/2;
+		pcb_coord_t y = (obj->BoundingBox.Y1 + obj->BoundingBox.Y2)/2;
+		pcb_message(PCB_MSG_ERROR, " near %$mm %$mm\n", x, y);
+	}
+	if (hint != NULL)
+		pcb_message(PCB_MSG_ERROR, " (%s)\n", hint);
+	return 0;
+}
+
+
 void pcb_io_uninit(void)
 {
 	free(last_design_dir);
