@@ -1,7 +1,8 @@
 /*
  *                            COPYRIGHT
  *
- *  PCB, interactive printed circuit board design
+ *  pcb-rnd, interactive printed circuit board design
+ *  (this file is based on PCB, interactive printed circuit board design)
  *  Copyright (C) 1994,1995,1996,2010 Thomas Nau
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -18,9 +19,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
- *  Contact addresses for paper mail and Email:
- *  Thomas Nau, Schlehenweg 15, 88471 Baustetten, Germany
- *  Thomas.Nau@rz.uni-ulm.de
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  *
  */
 
@@ -49,6 +51,7 @@
 #include "compat_nls.h"
 #include "obj_all.h"
 #include "obj_poly_draw.h"
+#include "polygon_selfi.h"
 
 #define ROUND(x) ((long)(((x) >= 0 ? (x) + 0.5  : (x) - 0.5)))
 
@@ -213,15 +216,18 @@ pcb_polyarea_t *pcb_poly_from_contour(pcb_pline_t * contour)
 }
 
 /* Convert a polygon to an unclipped polyarea */
-static pcb_polyarea_t *original_poly(pcb_poly_t * p)
+static pcb_polyarea_t *original_poly(pcb_poly_t *p, pcb_bool *need_full)
 {
 	pcb_pline_t *contour = NULL;
-	pcb_polyarea_t *np = NULL;
+	pcb_polyarea_t *np1 = NULL, *np = NULL;
 	pcb_cardinal_t n;
 	pcb_vector_t v;
 	int hole = 0;
 
-	if ((np = pcb_polyarea_create()) == NULL)
+	*need_full = pcb_false;
+
+	np1 = np = pcb_polyarea_create();
+	if (np == NULL)
 		return NULL;
 
 	/* first make initial polygon contour */
@@ -248,19 +254,49 @@ static pcb_polyarea_t *original_poly(pcb_poly_t * p)
 				pcb_poly_contour_inv(contour);
 			assert(contour->Flags.orient == (hole ? PCB_PLF_INV : PCB_PLF_DIR));
 
-			pcb_polyarea_contour_include(np, contour);
+			/* attempt to auto-correct simple self intersecting cases */
+			if (0 && pcb_pline_is_selfint(contour)) {
+				vtp0_t islands;
+				int n;
+
+				/* self intersecting polygon - attempt to fix it by splitting up into multiple polyareas */
+				vtp0_init(&islands);
+				pcb_pline_split_selfint(contour, &islands);
+				for(n = 0; n < vtp0_len(&islands); n++) {
+					pcb_pline_t *pl = *vtp0_get(&islands, n, 0);
+					if (n > 0) {
+						pcb_polyarea_t *newpa = pcb_polyarea_create();
+						newpa->b = np;
+						newpa->f = np->f;
+						np->f->b = newpa;
+						np->f = newpa;
+						np = newpa;
+						*need_full = pcb_true;
+					}
+					pcb_poly_contour_pre(pl, pcb_true);
+					if (pl->Flags.orient != (hole ? PCB_PLF_INV : PCB_PLF_DIR))
+						pcb_poly_contour_inv(pl);
+					assert(pl->Flags.orient == (hole ? PCB_PLF_INV : PCB_PLF_DIR));
+					pcb_polyarea_contour_include(np, pl);
+				}
+				vtp0_uninit(&islands);
+				free(contour);
+			}
+			else
+				pcb_polyarea_contour_include(np, contour);
 			contour = NULL;
 			assert(pcb_poly_valid(np));
 
 			hole++;
 		}
 	}
-	return biggest(np);
+	return biggest(np1);
 }
 
 pcb_polyarea_t *pcb_poly_from_poly(pcb_poly_t * p)
 {
-	return original_poly(p);
+	pcb_bool tmp;
+	return original_poly(p, &tmp);
 }
 
 pcb_polyarea_t *pcb_poly_from_rect(pcb_coord_t x1, pcb_coord_t x2, pcb_coord_t y1, pcb_coord_t y2)
@@ -1311,10 +1347,11 @@ static int Unsubtract(pcb_polyarea_t * np1, pcb_poly_t * p)
 	pcb_polyarea_t *merged = NULL, *np = np1;
 	pcb_polyarea_t *orig_poly, *clipped_np;
 	int x;
+	pcb_bool need_full;
 	assert(np);
 	assert(p); /* NOTE: p->clipped might be NULL if a poly is "cleared out of existence" and is now coming back */
 
-	orig_poly = original_poly(p);
+	orig_poly = original_poly(p, &need_full);
 
 	x = pcb_polyarea_boolean_free(np, orig_poly, &clipped_np, PCB_PBO_ISECT);
 	if (x != pcb_err_ok) {
@@ -1449,16 +1486,26 @@ static pcb_bool inhibit = pcb_false;
 int pcb_poly_init_clip(pcb_data_t *Data, pcb_layer_t *layer, pcb_poly_t * p)
 {
 	pcb_board_t *pcb;
+	pcb_bool need_full;
 
 	if (inhibit)
 		return 0;
 
+	if (Data->clip_inhibit > 0) {
+		p->clip_dirty = 1;
+		return 0;
+	}
+
 	if (layer->is_bound)
 		layer = layer->meta.bound.real;
 
 	if (p->Clipped)
 		pcb_polyarea_free(&p->Clipped);
-	p->Clipped = original_poly(p);
+	p->Clipped = original_poly(p, &need_full);
+	if (need_full && !PCB_FLAG_TEST(PCB_FLAG_FULLPOLY, p)) {
+		pcb_message(PCB_MSG_WARNING, "Polygon #%ld was self intersecting; it had to be split up and\nthe full poly flag set.\n", (long)p->ID);
+		PCB_FLAG_SET(PCB_FLAG_FULLPOLY, p);
+	}
 	pcb_poly_contours_free(&p->NoHoles);
 
 	if (layer == NULL)
@@ -1493,7 +1540,7 @@ pcb_bool pcb_poly_remove_excess_points(pcb_layer_t *Layer, pcb_poly_t *Polygon)
 	pcb_bool changed = pcb_false;
 
 	if (pcb_undoing())
-		return (pcb_false);
+		return pcb_false;
 
 	for (n = 0; n < Polygon->PointN; n++) {
 		prev = pcb_poly_contour_prev_point(Polygon, n);
@@ -1508,7 +1555,7 @@ pcb_bool pcb_poly_remove_excess_points(pcb_layer_t *Layer, pcb_poly_t *Polygon)
 			changed = pcb_true;
 		}
 	}
-	return (changed);
+	return changed;
 }
 
 /* ---------------------------------------------------------------------------
@@ -1555,7 +1602,7 @@ pcb_cardinal_t pcb_poly_get_lowest_distance_point(pcb_poly_t *Polygon, pcb_coord
 			}
 		}
 	}
-	return (result);
+	return result;
 }
 
 /* ---------------------------------------------------------------------------
@@ -1876,6 +1923,14 @@ static pcb_r_dir_t plow_callback(const pcb_box_t * b, void *cl)
 	return PCB_R_DIR_NOT_FOUND;
 }
 
+/* poly plows while clipping inhibit is on: mark any potentail polygon dirty */
+static pcb_r_dir_t poly_plows_inhibited_cb(pcb_data_t *data, pcb_layer_t *lay, pcb_poly_t *poly, int type, void *ptr1, void *ptr2)
+{
+	poly->clip_dirty = 1;
+	return PCB_R_DIR_FOUND_CONTINUE;
+}
+
+
 int
 pcb_poly_plows(pcb_data_t * Data, int type, void *ptr1, void *ptr2,
 						 pcb_r_dir_t (*call_back) (pcb_data_t *data, pcb_layer_t *lay, pcb_poly_t *poly, int type, void *ptr1, void *ptr2))
@@ -1884,6 +1939,9 @@ pcb_poly_plows(pcb_data_t * Data, int type, void *ptr1, void *ptr2,
 	int r = 0, seen;
 	struct plow_info info;
 
+	if (Data->clip_inhibit > 0)
+		call_back = poly_plows_inhibited_cb;
+
 	info.type = type;
 	info.ptr1 = ptr1;
 	info.ptr2 = ptr2;
@@ -2002,9 +2060,9 @@ pcb_poly_plows(pcb_data_t * Data, int type, void *ptr1, void *ptr2,
 void pcb_poly_restore_to_poly(pcb_data_t * Data, int type, void *ptr1, void *ptr2)
 {
 	pcb_data_t *dt = Data;
-	while(dt->parent_type == PCB_PARENT_SUBC)
+	while((dt != NULL) && (dt->parent_type == PCB_PARENT_SUBC))
 		dt = dt->parent.subc->parent.data;
-	if (dt->parent_type != PCB_PARENT_BOARD) /* clear/restore only on boards */
+	if ((dt == NULL) || (dt->parent_type != PCB_PARENT_BOARD)) /* clear/restore only on boards */
 		return;
 	if (type == PCB_TYPE_POLY)
 		pcb_poly_init_clip(dt, (pcb_layer_t *) ptr1, (pcb_poly_t *) ptr2);
@@ -2014,9 +2072,9 @@ void pcb_poly_restore_to_poly(pcb_data_t * Data, int type, void *ptr1, void *ptr
 void pcb_poly_clear_from_poly(pcb_data_t * Data, int type, void *ptr1, void *ptr2)
 {
 	pcb_data_t *dt = Data;
-	while(dt->parent_type == PCB_PARENT_SUBC)
+	while((dt != NULL) && (dt->parent_type == PCB_PARENT_SUBC))
 		dt = dt->parent.subc->parent.data;
-	if (dt->parent_type != PCB_PARENT_BOARD) /* clear/restore only on boards */
+	if ((dt == NULL) || (dt->parent_type != PCB_PARENT_BOARD)) /* clear/restore only on boards */
 		return;
 	if (type == PCB_TYPE_POLY)
 		pcb_poly_init_clip(dt, (pcb_layer_t *) ptr1, (pcb_poly_t *) ptr2);
@@ -2200,7 +2258,7 @@ pcb_bool pcb_poly_morph(pcb_layer_t *layer, pcb_poly_t *poly)
 			newone->Clipped = p;
 			p = p->f;									/* go to next pline */
 			newone->Clipped->b = newone->Clipped->f = newone->Clipped;	/* unlink from others */
-			pcb_r_insert_entry(layer->polygon_tree, (pcb_box_t *) newone, 0);
+			pcb_r_insert_entry(layer->polygon_tree, (pcb_box_t *) newone);
 			pcb_poly_invalidate_draw(layer, newone);
 		}
 		else {
@@ -2300,8 +2358,8 @@ void pcb_poly_to_polygons_on_layer(pcb_data_t * Destination, pcb_layer_t * Layer
 		pcb_poly_init_clip(Destination, Layer, Polygon);
 		pcb_poly_bbox(Polygon);
 		if (!Layer->polygon_tree)
-			Layer->polygon_tree = pcb_r_create_tree(NULL, 0, 0);
-		pcb_r_insert_entry(Layer->polygon_tree, (pcb_box_t *) Polygon, 0);
+			Layer->polygon_tree = pcb_r_create_tree();
+		pcb_r_insert_entry(Layer->polygon_tree, (pcb_box_t *) Polygon);
 
 		pcb_poly_invalidate_draw(Layer, Polygon);
 		/* add to undo list */
