@@ -1,7 +1,8 @@
 /*
  *                            COPYRIGHT
  *
- *  PCB, interactive printed circuit board design
+ *  pcb-rnd, interactive printed circuit board design
+ *  (this file is based on PCB, interactive printed circuit board design)
  *  Copyright (C) 1994,1995,1996 Thomas Nau
  *
  *  This module, rats.c, was written and is Copyright (C) 1997 by harry eaton
@@ -21,6 +22,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  */
 
 /* rats nest routines */
@@ -49,6 +54,7 @@
 #include "obj_rat_draw.h"
 #include "obj_term.h"
 #include "obj_subc_parent.h"
+#include "obj_pstk_inlines.h"
 
 #define STEP_POINT 100
 
@@ -69,6 +75,22 @@ static void TransferNet(pcb_netlist_t *, pcb_net_t *, pcb_net_t *);
 static pcb_bool badnet = pcb_false;
 static pcb_layergrp_id_t Sgrp = -1, Cgrp = -1; /* layer group holding solder/component side */
 
+static pcb_layergrp_id_t rat_padstack_side(pcb_pstk_t *padstack)
+{
+	pcb_pstk_tshape_t *ts = pcb_pstk_get_tshape(padstack);
+	if (ts != NULL) { /* if there are copper shapes, decide side by where the padstack has copper */
+		int n;
+		for(n = 0; n < ts->len; n++) {
+			if (!(ts->shape[n].layer_mask & PCB_LYT_COPPER)) continue;
+			if (ts->shape[n].layer_mask & PCB_LYT_TOP)
+				return Cgrp;
+			else if (ts->shape[n].layer_mask & PCB_LYT_BOTTOM)
+				return Sgrp;
+		}
+	}
+	return Sgrp; /* ultimate fallback */
+}
+
 /* ---------------------------------------------------------------------------
  * parse a connection description from a string
  * puts the element name in the string and the pin number in
@@ -89,12 +111,12 @@ static pcb_bool ParseConnection(const char *InString, char *ElementName, char *P
 		for (i = 0, j++; InString[j] != '\0'; i++, j++)
 			PinNum[i] = InString[j];
 		PinNum[i] = '\0';
-		return (pcb_false);
+		return pcb_false;
 	}
 	else {
 		ElementName[j] = '\0';
 		pcb_message(PCB_MSG_ERROR, _("Bad net-list format encountered near: \"%s\"\n"), ElementName);
-		return (pcb_true);
+		return pcb_true;
 	}
 }
 
@@ -116,6 +138,9 @@ static pcb_bool pcb_term_find_name_ppt(const char *ElementName, const char *PinN
 	if (obj != NULL) {
 		conn->obj = obj;
 		pcb_obj_center(obj, &conn->X, &conn->Y);
+		if (obj->type == PCB_OBJ_PSTK)
+			conn->group = rat_padstack_side((pcb_pstk_t *)obj);
+#warning subc TODO: heavy terminals: calculate side
 		return pcb_true;
 	}
 
@@ -161,7 +186,7 @@ pcb_bool pcb_rat_seek_pad(pcb_lib_entry_t * entry, pcb_connection_t * conn, pcb_
 	char PinNum[256];
 
 	if (ParseConnection(entry->ListEntry, ElementName, PinNum))
-		return (pcb_false);
+		return pcb_false;
 	for (j = 0; PinNum[j] != '\0'; j++);
 	if (j == 0) {
 		pcb_message(PCB_MSG_ERROR, _("Error! Netlist file is missing pin!\n" "white space after \"%s-\"\n"), ElementName);
@@ -169,16 +194,16 @@ pcb_bool pcb_rat_seek_pad(pcb_lib_entry_t * entry, pcb_connection_t * conn, pcb_
 	}
 	else {
 		if (pcb_term_find_name_ppt(ElementName, PinNum, conn, Same))
-			return (pcb_true);
+			return pcb_true;
 		if (Same)
-			return (pcb_false);
+			return pcb_false;
 		if (PinNum[j - 1] < '0' || PinNum[j - 1] > '9') {
 			pcb_message(PCB_MSG_WARNING, "WARNING! Pin number ending with '%c'"
 							" encountered in netlist file\n" "Probably a bad netlist file format\n", PinNum[j - 1]);
 		}
 	}
 	pcb_message(PCB_MSG_WARNING, _("Can't find %s pin %s called for in netlist.\n"), ElementName, PinNum);
-	return (pcb_false);
+	return pcb_false;
 }
 
 static const char *get_refdes(void *ptr1)
@@ -261,7 +286,7 @@ pcb_netlist_t *pcb_rat_proc_netlist(pcb_lib_t *net_menu)
 	static pcb_netlist_t *Wantlist = NULL;
 
 	if (!net_menu->MenuN)
-		return (NULL);
+		return NULL;
 	pcb_netlist_free(Wantlist);
 	free(Wantlist);
 	badnet = pcb_false;
@@ -326,7 +351,7 @@ pcb_netlist_t *pcb_rat_proc_netlist(pcb_lib_t *net_menu)
 	/* clear all visit marks */
 	clear_drc_flag(0);
 
-	return (Wantlist);
+	return Wantlist;
 }
 
 /*
@@ -501,7 +526,7 @@ static pcb_bool CheckShorts(pcb_lib_menu_t *theNet)
 	}
 	PCB_END_LOOP;
 	vtp0_uninit(&generic);
-	return (warn);
+	return warn;
 }
 
 static void gather_subnet_objs(pcb_data_t *data, pcb_netlist_t *Netl, pcb_net_t *a)
@@ -570,12 +595,13 @@ static void gather_subnet_objs(pcb_data_t *data, pcb_netlist_t *Netl, pcb_net_t
 	PCB_PADSTACK_LOOP(data);
 	{
 		if (PCB_FLAG_TEST(PCB_FLAG_DRC, padstack)) {
+
 			conn = pcb_rat_connection_alloc(a);
 			conn->X = padstack->x;
 			conn->Y = padstack->y;
 			conn->ptr1 = padstack;
 			conn->obj = (pcb_any_obj_t *)padstack;
-			conn->group = Sgrp;
+			conn->group = rat_padstack_side(padstack);
 			if PCB_FLAG_TEST(PCB_FLAG_DRC_INTCONN, padstack)
 				PCB_FLAG_CLEAR(PCB_FLAG_DRC | PCB_FLAG_DRC_INTCONN, padstack);
 		}
@@ -625,7 +651,7 @@ static pcb_bool GatherSubnets(pcb_netlist_t *Netl, pcb_bool NoWarn, pcb_bool And
 			Warned |= CheckShorts(a->Connection[0].menu);
 	}
 	pcb_reset_conns(pcb_false);
-	return (Warned);
+	return Warned;
 }
 
 /* ---------------------------------------------------------------------------
@@ -783,7 +809,7 @@ DrawShortestRats(pcb_netlist_t *Netl,
 		pcb_conn_lookup_uninit();
 		pcb_conn_lookup_init();
 	}
-	return (changed);
+	return changed;
 }
 
 pcb_bool
@@ -803,7 +829,7 @@ pcb_rat_add_all(pcb_bool SelectedOnly,
 	Wantlist = pcb_rat_proc_netlist(&(PCB->NetlistLib[PCB_NETLIST_EDITED]));
 	if (!Wantlist) {
 		pcb_message(PCB_MSG_WARNING, _("Can't add rat lines because no netlist is loaded.\n"));
-		return (pcb_false);
+		return pcb_false;
 	}
 	changed = pcb_false;
 	/* initialize finding engine */
@@ -844,7 +870,7 @@ pcb_rat_add_all(pcb_bool SelectedOnly,
 	pcb_conn_lookup_uninit();
 	pcb_restore_find_flag();
 	if (funcp)
-		return (pcb_true);
+		return pcb_true;
 
 	if (Warned || changed) {
 		pcb_stub_rat_proc_shorts();
@@ -859,7 +885,7 @@ pcb_rat_add_all(pcb_bool SelectedOnly,
 		if (ratlist_length(&PCB->Data->Rat) > 0) {
 			pcb_message(PCB_MSG_INFO, "%d rat line%s remaining\n", ratlist_length(&PCB->Data->Rat), ratlist_length(&PCB->Data->Rat) > 1 ? "s" : "");
 		}
-		return (pcb_true);
+		return pcb_true;
 	}
 	if (!SelectedOnly && !Warned) {
 		if (!ratlist_length(&PCB->Data->Rat) && !badnet)
@@ -868,7 +894,7 @@ pcb_rat_add_all(pcb_bool SelectedOnly,
 			pcb_message(PCB_MSG_WARNING, _("Nothing more to add, but there are\n"
 								"either rat-lines in the layout, disabled nets\n" "in the net-list, or missing components\n"));
 	}
-	return (pcb_false);
+	return pcb_false;
 }
 
 /* XXX: This is copied in large part from AddAllRats above; for
@@ -961,18 +987,18 @@ pcb_rat_t *pcb_rat_add_net(void)
 
 	if (pcb_crosshair.AttachedLine.Point1.X == pcb_crosshair.AttachedLine.Point2.X
 			&& pcb_crosshair.AttachedLine.Point1.Y == pcb_crosshair.AttachedLine.Point2.Y)
-		return (NULL);
+		return NULL;
 
-	found = pcb_search_obj_by_location(PCB_TYPE_PAD | PCB_TYPE_PIN | PCB_TYPE_PSTK, &ptr1, &ptr2, &ptr3,
+	found = pcb_search_obj_by_location(PCB_TYPE_PAD | PCB_TYPE_PIN | PCB_TYPE_PSTK | PCB_TYPE_SUBC_PART, &ptr1, &ptr2, &ptr3,
 																 pcb_crosshair.AttachedLine.Point1.X, pcb_crosshair.AttachedLine.Point1.Y, 5);
 	if (found == PCB_TYPE_NONE) {
 		pcb_message(PCB_MSG_ERROR, _("No pad/pin under rat line\n"));
-		return (NULL);
+		return NULL;
 	}
 	rd = get_refdes(ptr1);
 	if ((rd == NULL) || (*rd == 0) || (*rd == '<')) {
 		pcb_message(PCB_MSG_ERROR, _("You must name the starting element first\n"));
-		return (NULL);
+		return NULL;
 	}
 
 	Sgrp = Cgrp = -1;
@@ -982,16 +1008,16 @@ pcb_rat_t *pcb_rat_add_net(void)
 	/* will work for pins to since the FLAG is common */
 	group1 = (PCB_FLAG_TEST(PCB_FLAG_ONSOLDER, (pcb_pad_t *) ptr2) ? Sgrp : Cgrp);
 	strcpy(name1, pcb_connection_name(ptr2));
-	found = pcb_search_obj_by_location(PCB_TYPE_PAD | PCB_TYPE_PIN, &ptr1, &ptr2, &ptr3,
+	found = pcb_search_obj_by_location(PCB_TYPE_PAD | PCB_TYPE_PIN | PCB_TYPE_PSTK | PCB_TYPE_SUBC_PART, &ptr1, &ptr2, &ptr3,
 																 pcb_crosshair.AttachedLine.Point2.X, pcb_crosshair.AttachedLine.Point2.Y, 5);
 	if (found == PCB_TYPE_NONE) {
 		pcb_message(PCB_MSG_ERROR, _("No pad/pin under rat line\n"));
-		return (NULL);
+		return NULL;
 	}
 	rd = get_refdes(ptr1);
 	if ((rd == NULL) || (*rd == 0) || (*rd == '<')) {
 		pcb_message(PCB_MSG_ERROR, _("You must name the ending element first\n"));
-		return (NULL);
+		return NULL;
 	}
 	group2 = (PCB_FLAG_TEST(PCB_FLAG_ONSOLDER, (pcb_pad_t *) ptr2) ? Sgrp : Cgrp);
 	name2 = pcb_connection_name(ptr2);
@@ -1000,7 +1026,7 @@ pcb_rat_t *pcb_rat_add_net(void)
 	if (menu) {
 		if (pcb_netnode_to_netname(name2)) {
 			pcb_message(PCB_MSG_ERROR, _("Both connections already in netlist - cannot merge nets\n"));
-			return (NULL);
+			return NULL;
 		}
 		entry = pcb_lib_entry_new(menu);
 		entry->ListEntry = pcb_strdup(name2);
@@ -1075,7 +1101,7 @@ char *pcb_connection_name(pcb_any_obj_t *obj)
 	strcpy(name, PCB_UNKNOWN(parent));
 	strcat(name, "-");
 	strcat(name, PCB_UNKNOWN(num));
-	return (name);
+	return name;
 }
 
 /* ---------------------------------------------------------------------------
