@@ -1,7 +1,8 @@
 /*
  *                            COPYRIGHT
  *
- *  PCB, interactive printed circuit board design
+ *  pcb-rnd, interactive printed circuit board design
+ *  (this file is based on PCB, interactive printed circuit board design)
  *  Copyright (C) 1994,1995,1996 Thomas Nau
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -18,9 +19,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- *  Contact addresses for paper mail and Email:
- *  Thomas Nau, Schlehenweg 15, 88471 Baustetten, Germany
- *  Thomas.Nau@rz.uni-ulm.de
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  *
  */
 
@@ -29,6 +31,7 @@
 
 #include "config.h"
 #include "global_typedefs.h"
+#include "compat_misc.h"
 
 /*** Transformation macros ***/
 #define	PCB_COORD_ROTATE90(x,y,x0,y0,n)							\
@@ -48,7 +51,7 @@
 		}											\
 	} while(0)
 
-#define PCB_ROTATE_TYPES (PCB_TYPE_PSTK | PCB_TYPE_ELEMENT | PCB_TYPE_SUBC | PCB_TYPE_TEXT | PCB_TYPE_ELEMENT_NAME | PCB_TYPE_ARC | PCB_TYPE_LINE_POINT | PCB_TYPE_LINE)
+#define PCB_ROTATE_TYPES (PCB_TYPE_PSTK | PCB_TYPE_ELEMENT | PCB_TYPE_SUBC | PCB_TYPE_TEXT | PCB_TYPE_ELEMENT_NAME | PCB_TYPE_ARC | PCB_TYPE_LINE_POINT | PCB_TYPE_LINE | PCB_TYPE_POLY)
 
 /* rotates an object passed;
  * the center of rotation is determined by the current cursor location
@@ -69,11 +72,11 @@ PCB_INLINE void pcb_rotate(pcb_coord_t * x, pcb_coord_t * y, pcb_coord_t cx, pcb
 	pcb_coord_t px = *x - cx;
 	pcb_coord_t py = *y - cy;
 
-	nx = px * cosa + py * sina;
-	ny = py * cosa - px * sina;
+	nx = pcb_round(px * cosa + py * sina + cx);
+	ny = pcb_round(py * cosa - px * sina + cy);
 
-	*x = nx + cx;
-	*y = ny + cy;
+	*x = nx;
+	*y = ny;
 }
 
 #endif
