@@ -1,7 +1,8 @@
 /*
  *                            COPYRIGHT
  *
- *  PCB, interactive printed circuit board design
+ *  pcb-rnd, interactive printed circuit board design
+ *  (this file is based on PCB, interactive printed circuit board design)
  *  Copyright (C) 1994,1995,1996 Thomas Nau
  *  Copyright (C) 1998,1999,2000,2001,2002,2003,2004 harry eaton
  *
@@ -22,7 +23,13 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- *  Contact addresses for paper mail and Email:
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
+ *
+ *
+ *  Old contact info:
  *  harry eaton, 6697 Buttonhole Ct, Columbia, MD 21044 USA
  *  haceaton@aplcomm.jhuapl.edu
  *
@@ -36,6 +43,12 @@
  */
 #include "config.h"
 
+#include "rtree.h"
+
+#if PCB_USE_GENRTREE
+#include "rtree2.c"
+#else
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <assert.h>
@@ -43,7 +56,6 @@
 
 #include "math_helper.h"
 #include "compat_cc.h"
-#include "rtree.h"
 #include "obj_common.h"
 #include "macro.h"
 
@@ -73,7 +85,7 @@ struct rtree_node {
 	struct rtree_node *parent;		/* parent of this node, NULL = root */
 	struct {
 		unsigned is_leaf:1;					/* this is a leaf node */
-		unsigned manage:31;					/* pcb_true==should free 'rect.bptr' if node is destroyed */
+		unsigned unused:31;					/* pcb_true==should free 'rect.bptr' if node is destroyed */
 	} flags;
 	union {
 		struct rtree_node *kids[M_SIZE + 1];	/* when not leaf */
@@ -208,8 +220,7 @@ void pcb_r_dump_tree(struct rtree_node *node, int depth)
 		printf("p=0x%p node X(%d, %d) Y(%d, %d)\n", (void *) node, node->box.X1, node->box.X2, node->box.Y1, node->box.Y2);
 	}
 	else {
-		printf("p=0x%p leaf manage(%02x) X(%d, %d) Y(%d, %d)\n", (void *) node,
-					 node->flags.manage, node->box.X1, node->box.X2, node->box.Y1, node->box.Y2);
+		printf("p=0x%p leaf X(%d, %d) Y(%d, %d)\n", (void *) node, node->box.X1, node->box.X2, node->box.Y1, node->box.Y2);
 		for (j = 0; j < M_SIZE; j++) {
 			if (!node->u.rects[j].bptr)
 				break;
@@ -333,35 +344,38 @@ static void adjust_bounds(struct rtree_node *node)
 	}
 }
 
-/* create an r-tree from an unsorted list of boxes.
- * the r-tree will keep pointers into 
- * it, so don't free the box list until you've called r_destroy_tree.
- * if you set 'manage' to pcb_true, r_destroy_tree will free your boxlist.
- */
-pcb_rtree_t *pcb_r_create_tree(const pcb_box_t * boxlist[], int N, int manage)
+pcb_rtree_t *pcb_r_create_tree(void)
 {
 	pcb_rtree_t *rtree;
 	struct rtree_node *node;
-	int i;
 
-	assert(N >= 0);
-	rtree = (pcb_rtree_t *) calloc(1, sizeof(*rtree));
+	rtree = calloc(1, sizeof(*rtree));
+
 	/* start with a single empty leaf node */
-	node = (struct rtree_node *) calloc(1, sizeof(*node));
+	node = calloc(1, sizeof(*node));
 	node->flags.is_leaf = 1;
 	node->parent = NULL;
 	rtree->root = node;
+	return rtree;
+}
+
+void pcb_r_insert_array(pcb_rtree_t *rtree, const pcb_box_t *boxlist[], int N)
+{
+	int i;
+
+	assert(N >= 0);
+
 	/* simple, just insert all of the boxes! */
 	for (i = 0; i < N; i++) {
 		assert(boxlist[i]);
-		pcb_r_insert_entry(rtree, boxlist[i], manage);
+		pcb_r_insert_entry(rtree, boxlist[i]);
 	}
 #ifdef SLOW_ASSERTS
 	assert(__r_tree_is_good(rtree->root));
 #endif
-	return rtree;
 }
 
+
 static void __r_destroy_tree(struct rtree_node *node)
 {
 	int i, flag = 1;
@@ -370,8 +384,6 @@ static void __r_destroy_tree(struct rtree_node *node)
 		for (i = 0; i < M_SIZE; i++) {
 			if (!node->u.rects[i].bptr)
 				break;
-			if (node->flags.manage & flag)
-				free((void *) node->u.rects[i].bptr);
 			flag = flag << 1;
 		}
 	else
@@ -384,14 +396,39 @@ static void __r_destroy_tree(struct rtree_node *node)
 }
 
 /* free the memory associated with an rtree. */
-void pcb_r_destroy_tree(pcb_rtree_t ** rtree)
+void pcb_r_destroy_tree(pcb_rtree_t **rtree)
 {
-
 	__r_destroy_tree((*rtree)->root);
 	free(*rtree);
 	*rtree = NULL;
 }
 
+static void pcb_r_free_tree_data_(struct rtree_node *node, void (*free)(void *ptr))
+{
+	int i;
+
+	if (node->flags.is_leaf) {
+		for (i = 0; i < M_SIZE; i++) {
+			if (!node->u.rects[i].bptr)
+				break;
+			free((void *) node->u.rects[i].bptr);
+			node->u.rects[i].bptr = NULL;
+		}
+	}
+	else {
+		for (i = 0; i < M_SIZE; i++) {
+			if (!node->u.kids[i])
+				break;
+			pcb_r_free_tree_data_(node->u.kids[i], free);
+		}
+	}
+}
+
+void pcb_r_free_tree_data(pcb_rtree_t *rtree, void (*free)(void *ptr))
+{
+	pcb_r_free_tree_data_(rtree->root, free);
+}
+
 typedef struct {
 	pcb_r_dir_t (*check_it) (const pcb_box_t * region, void *cl);
 	pcb_r_dir_t (*found_it) (const pcb_box_t * box, void *cl);
@@ -576,7 +613,6 @@ struct rtree_node *find_clusters(struct rtree_node *node)
 	pcb_bool belong[M_SIZE + 1];
 	struct centroid center[M_SIZE + 1];
 	int clust_a, clust_b, tries;
-	int a_manage = 0, b_manage = 0;
 	int i, old_ax, old_ay, old_bx, old_by;
 	struct rtree_node *new_node;
 	pcb_box_t *b;
@@ -660,14 +696,10 @@ struct rtree_node *find_clusters(struct rtree_node *node)
 		for (i = 0; i < M_SIZE + 1; i++) {
 			if (belong[i]) {
 				node->u.rects[clust_a++] = node->u.rects[i];
-				if (node->flags.manage & flag)
-					a_manage |= a_flag;
 				a_flag <<= 1;
 			}
 			else {
 				new_node->u.rects[clust_b++] = node->u.rects[i];
-				if (node->flags.manage & flag)
-					b_manage |= b_flag;
 				b_flag <<= 1;
 			}
 			flag <<= 1;
@@ -683,8 +715,6 @@ struct rtree_node *find_clusters(struct rtree_node *node)
 			}
 		}
 	}
-	node->flags.manage = a_manage;
-	new_node->flags.manage = b_manage;
 	assert(clust_a != 0);
 	assert(clust_b != 0);
 	if (node->flags.is_leaf)
@@ -697,7 +727,7 @@ struct rtree_node *find_clusters(struct rtree_node *node)
 	sort_node(node);
 	adjust_bounds(new_node);
 	sort_node(new_node);
-	return (new_node);
+	return new_node;
 }
 
 /* split a node according to clusters
@@ -720,7 +750,7 @@ static void split_node(struct rtree_node *node)
 				if (second->u.kids[i])
 					second->u.kids[i]->parent = second;
 		node->flags.is_leaf = 0;
-		node->flags.manage = 0;
+		node->flags.unused = 0;
 		second->parent = new_node->parent = node;
 		node->u.kids[0] = new_node;
 		node->u.kids[1] = second;
@@ -773,7 +803,7 @@ static inline double penalty(struct rtree_node *node, const pcb_box_t * query)
 	return score;
 }
 
-static void __r_insert_node(struct rtree_node *node, const pcb_box_t * query, int manage, pcb_bool force)
+static void __r_insert_node(struct rtree_node *node, const pcb_box_t * query, pcb_bool force)
 {
 
 #ifdef SLOW_ASSERTS
@@ -786,22 +816,10 @@ static void __r_insert_node(struct rtree_node *node, const pcb_box_t * query, in
 
 	if (node->flags.is_leaf) {
 		register int i;
+		for (i = 0; i < M_SIZE; i++)
+			if (!node->u.rects[i].bptr)
+				break;
 
-		if (PCB_UNLIKELY(manage)) {
-			register int flag = 1;
-
-			for (i = 0; i < M_SIZE; i++) {
-				if (!node->u.rects[i].bptr)
-					break;
-				flag <<= 1;
-			}
-			node->flags.manage |= flag;
-		}
-		else {
-			for (i = 0; i < M_SIZE; i++)
-				if (!node->u.rects[i].bptr)
-					break;
-		}
 		/* the node always has an extra space available */
 		node->u.rects[i].bptr = query;
 		node->u.rects[i].bounds = *query;
@@ -842,7 +860,7 @@ static void __r_insert_node(struct rtree_node *node, const pcb_box_t * query, in
 			if (!node->u.kids[i])
 				break;
 			if (contained(node->u.kids[i], query)) {
-				__r_insert_node(node->u.kids[i], query, manage, pcb_false);
+				__r_insert_node(node->u.kids[i], query, pcb_false);
 				sort_node(node);
 				return;
 			}
@@ -858,8 +876,6 @@ static void __r_insert_node(struct rtree_node *node, const pcb_box_t * query, in
 			new_node->u.rects[0].bptr = query;
 			new_node->u.rects[0].bounds = *query;
 			new_node->box = *query;
-			if (PCB_UNLIKELY(manage))
-				new_node->flags.manage = 1;
 			sort_node(node);
 			return;
 		}
@@ -876,28 +892,28 @@ static void __r_insert_node(struct rtree_node *node, const pcb_box_t * query, in
 				best_node = node->u.kids[i];
 			}
 		}
-		__r_insert_node(best_node, query, manage, pcb_true);
+		__r_insert_node(best_node, query, pcb_true);
 		sort_node(node);
 		return;
 	}
 }
 
-void pcb_r_insert_entry(pcb_rtree_t * rtree, const pcb_box_t * which, int man)
+void pcb_r_insert_entry(pcb_rtree_t * rtree, const pcb_box_t * which)
 {
 	assert(which);
 	assert(which->X1 <= which->X2);
 	assert(which->Y1 <= which->Y2);
 	/* recursively search the tree for the best leaf node */
 	assert(rtree->root);
-	__r_insert_node(rtree->root, which, man,
+	__r_insert_node(rtree->root, which,
 									rtree->root->box.X1 > which->X1
 									|| rtree->root->box.X2 < which->X2 || rtree->root->box.Y1 > which->Y1 || rtree->root->box.Y2 < which->Y2);
 	rtree->size++;
 }
 
-pcb_bool __r_delete(struct rtree_node *node, const pcb_box_t * query)
+pcb_bool __r_delete_free_data(struct rtree_node *node, const pcb_box_t * query, void (*free_data)(void *data))
 {
-	int i, flag, mask, a;
+	int i, mask, a;
 
 	/* the tree might be inconsistent during delete */
 	if (query->X1 < node->box.X1 || query->Y1 < node->box.Y1 || query->X2 > node->box.X2 || query->Y2 > node->box.Y2)
@@ -922,7 +938,7 @@ pcb_bool __r_delete(struct rtree_node *node, const pcb_box_t * query)
 							node->u.rects[i].bptr = NULL;
 						return pcb_true;
 					}
-					return (__r_delete(node->parent, &node->box));
+					return __r_delete_free_data(node->parent, &node->box, free_data);
 				}
 				else
 					/* propagate boundary adjust upward */
@@ -933,7 +949,7 @@ pcb_bool __r_delete(struct rtree_node *node, const pcb_box_t * query)
 				return pcb_true;
 			}
 			if (node->u.kids[i]) {
-				if (__r_delete(node->u.kids[i], query))
+				if (__r_delete_free_data(node->u.kids[i], query, free_data))
 					return pcb_true;
 			}
 			else
@@ -958,14 +974,10 @@ pcb_bool __r_delete(struct rtree_node *node, const pcb_box_t * query)
 	}
 	if (!node->u.rects[i].bptr)
 		return pcb_false;								/* not at this leaf */
-	if (node->flags.manage & a) {
-		free((void *) node->u.rects[i].bptr);
+	if (free_data != NULL) {
+		free_data((void *) node->u.rects[i].bptr);
 		node->u.rects[i].bptr = NULL;
 	}
-	/* squeeze the manage flags together */
-	flag = node->flags.manage & mask;
-	mask = (~mask) << 1;
-	node->flags.manage = flag | ((node->flags.manage & mask) >> 1);
 	/* remove the entry */
 	for (; i < M_SIZE; i++) {
 		node->u.rects[i] = node->u.rects[i + 1];
@@ -974,7 +986,7 @@ pcb_bool __r_delete(struct rtree_node *node, const pcb_box_t * query)
 	}
 	if (!node->u.rects[0].bptr) {
 		if (node->parent)
-			__r_delete(node->parent, &node->box);
+			__r_delete_free_data(node->parent, &node->box, free_data);
 		return pcb_true;
 	}
 	else
@@ -986,13 +998,18 @@ pcb_bool __r_delete(struct rtree_node *node, const pcb_box_t * query)
 	return pcb_true;
 }
 
-pcb_bool pcb_r_delete_entry(pcb_rtree_t * rtree, const pcb_box_t * box)
+pcb_bool __r_delete(struct rtree_node *node, const pcb_box_t * query)
+{
+	return __r_delete_free_data(node, query, NULL);
+}
+
+pcb_bool pcb_r_delete_entry_free_data(pcb_rtree_t * rtree, const pcb_box_t * box, void (*free_data)(void *d))
 {
 	pcb_bool r;
 
 	assert(box);
 	assert(rtree);
-	r = __r_delete(rtree->root, box);
+	r = __r_delete_free_data(rtree->root, box, free_data);
 	if (r)
 		rtree->size--;
 #ifdef SLOW_ASSERTS
@@ -1001,6 +1018,11 @@ pcb_bool pcb_r_delete_entry(pcb_rtree_t * rtree, const pcb_box_t * box)
 	return r;
 }
 
+pcb_bool pcb_r_delete_entry(pcb_rtree_t * rtree, const pcb_box_t * box)
+{
+	return pcb_r_delete_entry_free_data(rtree, box, NULL);
+}
+
 
 static pcb_box_t *pcb_r_next_(pcb_rtree_it_t *it, struct rtree_node *curr)
 {
@@ -1079,3 +1101,4 @@ void pcb_r_end(pcb_rtree_it_t *it)
 	it->alloced = it->used = 0;
 }
 
+#endif
