@@ -1,7 +1,8 @@
 /*
  *                            COPYRIGHT
  *
- *  PCB, interactive printed circuit board design
+ *  pcb-rnd, interactive printed circuit board design
+ *  (this file is based on PCB, interactive printed circuit board design)
  *  Copyright (C) 1994,1995,1996 Thomas Nau
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -18,9 +19,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- *  Contact addresses for paper mail and Email:
- *  Thomas Nau, Schlehenweg 15, 88471 Baustetten, Germany
- *  Thomas.Nau@rz.uni-ulm.de
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  *
  */
 
@@ -41,6 +43,7 @@
 #include "polygon.h"
 #include "search.h"
 #include "obj_subc_parent.h"
+#include "obj_pstk_inlines.h"
 
 static double PosX, PosY;				/* search position for subroutines */
 static pcb_coord_t SearchRadius;
@@ -70,16 +73,19 @@ static pcb_bool SearchSubcByLocation(unsigned long, unsigned long, pcb_subc_t **
 /* Return not-found for subc parts and locked items unless objst says otherwise
    obj is the object to be checked if part of subc; check lock on locked_obj
    Also return not-found if flags are required but the object doesn't have them */
-#define TEST_OBJST(objst, req_flag, locality, obj, locked_obj) \
+#define TEST_OBJST_(objst, req_flag, locality, obj, locked_obj, reject) \
 do { \
 	if ((req_flag != 0) && (!PCB_FLAG_TEST(req_flag, obj))) \
-		return PCB_R_DIR_NOT_FOUND; \
+		reject; \
 	if (!(objst & PCB_TYPE_SUBC_PART) && (pcb_ ## locality ## obj_parent_subc(obj->parent_type, &obj->parent))) \
-		return PCB_R_DIR_NOT_FOUND; \
+		reject; \
 	if (!(objst & PCB_TYPE_LOCKED) && (PCB_FLAG_TEST(objst & PCB_FLAG_LOCK, locked_obj))) \
-		return PCB_R_DIR_NOT_FOUND; \
+		reject; \
 } while(0)
 
+#define TEST_OBJST(objst, req_flag, locality, obj, locked_obj) \
+	TEST_OBJST_(objst, req_flag, locality, obj, locked_obj, return PCB_R_DIR_NOT_FOUND)
+
 /* ---------------------------------------------------------------------------
  * searches a via
  */
@@ -88,6 +94,7 @@ struct ans_info {
 	pcb_bool BackToo;
 	double area;
 	unsigned long objst, req_flag;
+	int on_current;
 };
 
 static pcb_r_dir_t pinorvia_callback(const pcb_box_t * box, void *cl)
@@ -131,16 +138,25 @@ static pcb_r_dir_t padstack_callback(const pcb_box_t *box, void *cl)
 {
 	struct ans_info *i = (struct ans_info *) cl;
 	pcb_pstk_t *ps = (pcb_pstk_t *) box;
+	pcb_data_t *pdata;
 
 	TEST_OBJST(i->objst, i->req_flag, g, ps, ps);
 
 	if (!pcb_is_point_in_pstk(PosX, PosY, SearchRadius, ps, NULL))
 		return PCB_R_DIR_NOT_FOUND;
-	*i->ptr1 = *i->ptr2 = *i->ptr3 = ps;
+	if ((i->on_current) && (pcb_pstk_shape_at(PCB, ps, CURRENT) == NULL))
+		return PCB_R_DIR_NOT_FOUND;
+	*i->ptr2 = *i->ptr3 = ps;
+	assert(ps->parent_type == PCB_PARENT_DATA);
+	pdata = ps->parent.data;
+	if (pdata->parent_type == PCB_PARENT_SUBC)
+		*i->ptr1 = pdata->parent.subc; /* padstack of a subcircuit */
+	else
+		*i->ptr1 = ps; /* padstack on a board (e.g. via) */
 	return PCB_R_DIR_CANCEL; /* found, stop searching */
 }
 
-static pcb_bool SearchPadstackByLocation(unsigned long objst, unsigned long req_flag, pcb_pstk_t **ps, pcb_pstk_t **Dummy1, pcb_pstk_t **Dummy2)
+static pcb_bool SearchPadstackByLocation(unsigned long objst, unsigned long req_flag, pcb_pstk_t **ps, pcb_pstk_t **Dummy1, pcb_pstk_t **Dummy2, int on_current)
 {
 	struct ans_info info;
 
@@ -153,6 +169,7 @@ static pcb_bool SearchPadstackByLocation(unsigned long objst, unsigned long req_
 	info.ptr3 = (void **)Dummy2;
 	info.objst = objst;
 	info.req_flag = req_flag;
+	info.on_current = on_current;
 
 	if (pcb_r_search(PCB->Data->padstack_tree, &SearchBox, NULL, padstack_callback, &info, NULL) != PCB_R_DIR_NOT_FOUND)
 		return pcb_true;
@@ -209,7 +226,7 @@ static pcb_bool SearchPadByLocation(unsigned long objst, unsigned long req_flag,
 
 	/* search only if pin-layer is visible */
 	if (!PCB->PinOn)
-		return (pcb_false);
+		return pcb_false;
 	info.ptr1 = (void **) Element;
 	info.ptr2 = (void **) Pad;
 	info.ptr3 = (void **) Dummy;
@@ -563,8 +580,8 @@ static pcb_bool SearchPointByLocation(unsigned long Type, unsigned long objst, u
 	pcb_r_search(SearchLayer->polygon_tree, &SearchBox, NULL, polypoint_callback, &ctx, NULL);
 
 	if (ctx.found)
-		return (pcb_true);
-	return (pcb_false);
+		return pcb_true;
+	return pcb_false;
 }
 
 static pcb_r_dir_t name_callback(const pcb_box_t * box, void *cl)
@@ -610,7 +627,7 @@ SearchElementNameByLocation(unsigned long objst, unsigned long req_flag, pcb_ele
 		if (pcb_r_search(PCB->Data->name_tree[PCB_ELEMNAME_IDX_VISIBLE()], &SearchBox, NULL, name_callback, &info, NULL))
 			return pcb_true;
 	}
-	return (pcb_false);
+	return pcb_false;
 }
 
 static pcb_r_dir_t element_callback(const pcb_box_t * box, void *cl)
@@ -702,6 +719,54 @@ SearchSubcByLocation(unsigned long objst, unsigned long req_flag, pcb_subc_t **s
 	return pcb_false;
 }
 
+/* find the first floater on any layer */
+static pcb_bool SearchSubcFloaterByLocation(unsigned long objst, unsigned long req_flag, pcb_subc_t **out_subc, pcb_text_t **out_txt, void **dummy, pcb_bool other_side)
+{
+	pcb_rtree_it_t it;
+	void *obj;
+	int n;
+	pcb_layer_type_t my_side_lyt;
+
+	if (other_side)
+		my_side_lyt = conf_core.editor.show_solder_side ? PCB_LYT_TOP : PCB_LYT_BOTTOM;
+	else
+		my_side_lyt = conf_core.editor.show_solder_side ? PCB_LYT_BOTTOM : PCB_LYT_TOP;
+
+	for(n = 0; n < PCB->Data->LayerN; n++)  {
+		pcb_layer_t *ly = &PCB->Data->Layer[n];
+		pcb_layer_type_t lyt;
+
+		if ((ly->text_tree == NULL) || (!ly->meta.real.vis))
+			continue;
+
+		lyt = pcb_layer_flags_(ly);
+		if (!(lyt & my_side_lyt))
+			continue;
+
+		for(obj = pcb_rtree_first(&it, ly->text_tree, (pcb_rtree_box_t *)&SearchBox); obj != NULL; obj = pcb_rtree_next(&it)) {
+			pcb_text_t *txt = (pcb_text_t *)obj;
+			pcb_subc_t *subc;
+
+			if (PCB_FLAG_TEST(PCB_FLAG_FLOATER, txt) == 0)
+				continue;
+
+			subc = pcb_lobj_parent_subc(txt->parent_type, &txt->parent);
+			if (subc == NULL)
+				continue;
+
+			TEST_OBJST_(objst, req_flag, l, txt, subc, continue);
+
+			*out_subc = subc;
+			*out_txt = txt;
+			*dummy = txt;
+			return pcb_true;
+		}
+	}
+
+	return pcb_false;
+}
+
+
 pcb_bool pcb_is_point_in_pin(pcb_coord_t X, pcb_coord_t Y, pcb_coord_t Radius, pcb_pin_t *pin)
 {
 	pcb_coord_t t = PIN_SIZE(pin) / 2;
@@ -724,68 +789,60 @@ pcb_bool pcb_is_point_in_pin(pcb_coord_t X, pcb_coord_t Y, pcb_coord_t Radius, p
 pcb_bool pcb_is_point_on_line_end(pcb_coord_t X, pcb_coord_t Y, pcb_rat_t *Line)
 {
 	if (((X == Line->Point1.X) && (Y == Line->Point1.Y)) || ((X == Line->Point2.X) && (Y == Line->Point2.Y)))
-		return (pcb_true);
-	return (pcb_false);
+		return pcb_true;
+	return pcb_false;
 }
 
 /* ---------------------------------------------------------------------------
- * checks if a line intersects with a PV
+ * Calculate the distance (squared) from a point to a line.
  *
  * let the point be (X,Y) and the line (X1,Y1)(X2,Y2)
- * the length of the line is
  *
- *   L = ((X2-X1)^2 + (Y2-Y1)^2)^0.5
+ * Calculate the position along the line that is closest to X,Y. The value 
+ * ranges from 0 to 1 when the point is within the line boundaries with a 
+ * value of 0 being at point X1,Y1 and a value of 1.0 being at point X2,Y2.
+  *
+ *   t = ((X-X1)*(X2-X1)) + ((Y-Y1)*(Y2-Y1))
+ *       -----------------------------------
+ *             (X2-X1)^2 + (Y2-Y1)^2
  *
  * let Q be the point of perpendicular projection of (X,Y) onto the line
  *
- *   QX = X1 + D1*(X2-X1) / L
- *   QY = Y1 + D1*(Y2-Y1) / L
+ *   QX = X1 + t * (X2-X1)
+ *   QY = Y1 + t * (Y2-Y1)
  *
- * with (from vector geometry)
+ * Calculate the distace (squared) using Pythagorean theorem.
  *
- *        (Y1-Y)(Y1-Y2)+(X1-X)(X1-X2)
- *   D1 = ---------------------------
- *                     L
- *
- *   D1 < 0   Q is on backward extension of the line
- *   D1 > L   Q is on forward extension of the line
- *   else     Q is on the line
- *
- * the signed distance from (X,Y) to Q is
- *
- *        (Y2-Y1)(X-X1)-(X2-X1)(Y-Y1)
- *   D2 = ----------------------------
- *                     L
- *
- * Finally, D1 and D2 are orthogonal, so we can sum them easily
- * by Pythagorean theorem.
+ *   distance^2 = (X-QX)^2 + (Y-QY)^2
  */
 
 double pcb_point_line_dist2(pcb_coord_t X, pcb_coord_t Y, pcb_line_t *Line)
 {
-	double D1, D2, L;
+	const double abx = Line->Point2.X - Line->Point1.X;
+	const double aby = Line->Point2.Y - Line->Point1.Y;
+	const double apx = X - Line->Point1.X;
+	const double apy = Y - Line->Point1.Y;
+	double qx,qy,dx,dy;
 
-	/* Get length of segment */
-	L = pcb_distance2(Line->Point1.X, Line->Point1.Y, Line->Point2.X, Line->Point2.Y);
-	if (L < 0.01)
-		return pcb_distance2(X, Y, Line->Point1.X, Line->Point1.Y);
+	/* Calculate t, the normalised position along the line of the closest point 
+	 * to (X,Y). A value between 0 and 1 indicates that the closest point on the 
+	 * line is within the bounds of the end points. 
+	 */
+	double t = ( (apx * abx) + (apy * aby) ) / ( (abx * abx) + (aby * aby) );
 
-	/* Get distance from (X1, Y1) to Q (on the line) */
-	D1 = ((double) (Y - Line->Point1.Y) * (Line->Point2.Y - Line->Point1.Y)
-				+ (double) (X - Line->Point1.X) * (Line->Point2.X - Line->Point1.X)) / L;
-	/* Translate this into distance to Q from segment */
-	if (D1 < 0)
-		D1 = -D1;
-	else if (D1 > L)
-		D1 -= L;
-	else
-		D1 = 0;
-	/* Get distance from (X, Y) to Q */
-	D2 = ((double) (X - Line->Point1.X) * (Line->Point2.Y - Line->Point1.Y)
-				- (double) (Y - Line->Point1.Y) * (Line->Point2.X - Line->Point1.X)) / L;
+	/* Clamp 't' to the line bounds */
+	if(t < 0.0)		t = 0.0;
+	if(t > 1.0)		t = 1.0;
 
-	/* Total distance is then the Pythagorean sum of these */
-	return D1 * D1 + D2 * D2;
+	/* Calculate the point Q on the line that is closest to (X,Y) */
+	qx = Line->Point1.X + (t * abx);
+	qy = Line->Point1.Y + (t * aby);
+
+	/* Return the distance from Q to (X,Y), squared */
+	dx = X - qx;
+	dy = Y - qy;
+
+	return (dx * dx) + (dy * dy);
 }
 
 pcb_bool pcb_is_point_on_line(pcb_coord_t X, pcb_coord_t Y, pcb_coord_t Radius, pcb_line_t *Line)
@@ -807,6 +864,28 @@ static int is_point_on_line(pcb_coord_t px, pcb_coord_t py, pcb_coord_t lx1, pcb
 	return pcb_is_point_on_line(px, py, 1, &l);
 }
 
+pcb_bool pcb_is_point_on_thinline( pcb_coord_t X, pcb_coord_t Y, pcb_coord_t X1, pcb_coord_t Y1, pcb_coord_t X2,pcb_coord_t Y2 )
+{
+	/* Calculate the cross product of the vector from the first line point to the test point
+	 * and the vector from the first line point to the second line point. If the result is 
+	 * not 0 then the point does not lie on the line.
+	 */
+	const pcb_coord_t lx = X2 - X1;
+	const pcb_coord_t ly = Y2 - Y1;
+	const pcb_coord_t cross = ((X-X1) * ly) - ((Y-Y1) * lx);
+
+	if(cross != 0)
+		return pcb_false;
+
+	/* If the point does lie on the line then we do a simple check to see if the point is
+	 * between the end points. Use the longest side of the line to perform the check.
+	 */
+	if(abs(lx) > abs(ly))
+		return (X >= MIN(X1,X2)) && (X <= MAX(X1,X2)) ? pcb_true : pcb_false;
+
+	return (Y >= MIN(Y1,Y2)) && (Y <= MAX(Y1,Y2)) ? pcb_true : pcb_false;
+}
+
 /* checks if a line crosses a rectangle or is within the rectangle */
 pcb_bool pcb_is_line_in_rectangle(pcb_coord_t X1, pcb_coord_t Y1, pcb_coord_t X2, pcb_coord_t Y2, pcb_line_t *Line)
 {
@@ -815,7 +894,7 @@ pcb_bool pcb_is_line_in_rectangle(pcb_coord_t X1, pcb_coord_t Y1, pcb_coord_t X2
 	/* first, see if point 1 is inside the rectangle */
 	/* in case the whole line is inside the rectangle */
 	if (X1 < Line->Point1.X && X2 > Line->Point1.X && Y1 < Line->Point1.Y && Y2 > Line->Point1.Y)
-		return (pcb_true);
+		return pcb_true;
 	/* construct a set of dummy lines and check each of them */
 	line.Thickness = 0;
 	line.Flags = pcb_no_flags();
@@ -825,30 +904,30 @@ pcb_bool pcb_is_line_in_rectangle(pcb_coord_t X1, pcb_coord_t Y1, pcb_coord_t X2
 	line.Point1.X = X1;
 	line.Point2.X = X2;
 	if (pcb_intersect_line_line(&line, Line))
-		return (pcb_true);
+		return pcb_true;
 
 	/* upper-right to lower-right corner */
 	line.Point1.X = X2;
 	line.Point1.Y = Y1;
 	line.Point2.Y = Y2;
 	if (pcb_intersect_line_line(&line, Line))
-		return (pcb_true);
+		return pcb_true;
 
 	/* lower-right to lower-left corner */
 	line.Point1.Y = Y2;
 	line.Point1.X = X1;
 	line.Point2.X = X2;
 	if (pcb_intersect_line_line(&line, Line))
-		return (pcb_true);
+		return pcb_true;
 
 	/* lower-left to upper-left corner */
 	line.Point2.X = X1;
 	line.Point1.Y = Y1;
 	line.Point2.Y = Y2;
 	if (pcb_intersect_line_line(&line, Line))
-		return (pcb_true);
+		return pcb_true;
 
-	return (pcb_false);
+	return pcb_false;
 }
 
 /*checks if a point (of null radius) is in a slanted rectangle */
@@ -903,27 +982,27 @@ pcb_bool pcb_is_line_in_quadrangle(pcb_point_t p[4], pcb_line_t *Line)
 	line.Point2.X = p[1].X;
 	line.Point2.Y = p[1].Y;
 	if (pcb_intersect_line_line(&line, Line))
-		return (pcb_true);
+		return pcb_true;
 
 	/* upper-right to lower-right corner */
 	line.Point1.X = p[2].X;
 	line.Point1.Y = p[2].Y;
 	if (pcb_intersect_line_line(&line, Line))
-		return (pcb_true);
+		return pcb_true;
 
 	/* lower-right to lower-left corner */
 	line.Point2.X = p[3].X;
 	line.Point2.Y = p[3].Y;
 	if (pcb_intersect_line_line(&line, Line))
-		return (pcb_true);
+		return pcb_true;
 
 	/* lower-left to upper-left corner */
 	line.Point1.X = p[0].X;
 	line.Point1.Y = p[0].Y;
 	if (pcb_intersect_line_line(&line, Line))
-		return (pcb_true);
+		return pcb_true;
 
-	return (pcb_false);
+	return pcb_false;
 }
 
 /* ---------------------------------------------------------------------------
@@ -940,10 +1019,10 @@ pcb_bool pcb_is_arc_in_rectangle(pcb_coord_t X1, pcb_coord_t Y1, pcb_coord_t X2,
 	box.X2 = X2; box.Y2 = Y2;
 	pcb_arc_get_end (Arc, 0, &x, &y);
 	if (PCB_POINT_IN_BOX(x, y, &box))
-		return (pcb_true);
+		return pcb_true;
 	pcb_arc_get_end (Arc, 1, &x, &y);
 	if (PCB_POINT_IN_BOX(x, y, &box))
-		return (pcb_true);
+		return pcb_true;
 
 	/* construct a set of dummy lines and check each of them */
 	line.Thickness = 0;
@@ -954,30 +1033,30 @@ pcb_bool pcb_is_arc_in_rectangle(pcb_coord_t X1, pcb_coord_t Y1, pcb_coord_t X2,
 	line.Point1.X = X1;
 	line.Point2.X = X2;
 	if (pcb_intersect_line_arc(&line, Arc))
-		return (pcb_true);
+		return pcb_true;
 
 	/* upper-right to lower-right corner */
 	line.Point1.X = line.Point2.X = X2;
 	line.Point1.Y = Y1;
 	line.Point2.Y = Y2;
 	if (pcb_intersect_line_arc(&line, Arc))
-		return (pcb_true);
+		return pcb_true;
 
 	/* lower-right to lower-left corner */
 	line.Point1.Y = line.Point2.Y = Y2;
 	line.Point1.X = X1;
 	line.Point2.X = X2;
 	if (pcb_intersect_line_arc(&line, Arc))
-		return (pcb_true);
+		return pcb_true;
 
 	/* lower-left to upper-left corner */
 	line.Point1.X = line.Point2.X = X1;
 	line.Point1.Y = Y1;
 	line.Point2.Y = Y2;
 	if (pcb_intersect_line_arc(&line, Arc))
-		return (pcb_true);
+		return pcb_true;
 
-	return (pcb_false);
+	return pcb_false;
 }
 
 /* ---------------------------------------------------------------------------
@@ -1143,22 +1222,63 @@ pcb_bool pcb_is_point_on_arc(pcb_coord_t X, pcb_coord_t Y, pcb_coord_t Radius, p
 	if (!angle_in_range(ang1, ang2, p_ang) && !angle_in_range(ang1, ang2, p_ang+360)) {
 		pcb_coord_t ArcX, ArcY;
 		ArcX = Arc->X + Arc->Width * cos((Arc->StartAngle + 180) / PCB_RAD_TO_DEG);
-		ArcY = Arc->Y - Arc->Width * sin((Arc->StartAngle + 180) / PCB_RAD_TO_DEG);
+		ArcY = Arc->Y - Arc->Height * sin((Arc->StartAngle + 180) / PCB_RAD_TO_DEG);
 		if (pcb_distance(X, Y, ArcX, ArcY) < Radius + Arc->Thickness / 2)
 			return pcb_true;
 
 		ArcX = Arc->X + Arc->Width * cos((Arc->StartAngle + Arc->Delta + 180) / PCB_RAD_TO_DEG);
-		ArcY = Arc->Y - Arc->Width * sin((Arc->StartAngle + Arc->Delta + 180) / PCB_RAD_TO_DEG);
+		ArcY = Arc->Y - Arc->Height * sin((Arc->StartAngle + Arc->Delta + 180) / PCB_RAD_TO_DEG);
 		if (pcb_distance(X, Y, ArcX, ArcY) < Radius + Arc->Thickness / 2)
 			return pcb_true;
 		return pcb_false;
 	}
 
-	/* If point is inside the arc range, just compare it to the arc */
-	return fabs(pcb_distance(X, Y, Arc->X, Arc->Y) - Arc->Width) < Radius + Arc->Thickness / 2;
+	if ((Arc->Width == Arc->Height) || (fabs(Arc->Width - Arc->Height) < PCB_MM_TO_COORD(0.1))) {
+		/* Simple circular case: if point is inside the arc range, just compare it to the arc */
+		return fabs(pcb_distance(X, Y, Arc->X, Arc->Y) - Arc->Width) < Radius + Arc->Thickness / 2;
+	}
+	else {
+		/* elliptical case: guess where the arc would be in that point, by angle */
+		double ang, dx, dy;
+		pcb_coord_t ax, ay, d;
+
+#warning TODO: elliptical arc: rewrite this as it doesn't work properly on extreme cases
+		dy = (double)(Y - Arc->Y) / (double)Arc->Height;
+		dx = (double)(X - Arc->X) / (double)Arc->Width;
+		ang = -atan2(dy, dx);
+
+		ax = Arc->X + cos(ang) * Arc->Width;
+		ay = Arc->Y - sin(ang) * Arc->Height;
+
+		d = fabs(pcb_distance(X, Y, ax, ay));
+
+		return d < Arc->Thickness / 2;
+	}
 #undef angle_in_range
 }
 
+pcb_line_t *pcb_line_center_cross_point(pcb_layer_t *layer, pcb_coord_t x, pcb_coord_t y, pcb_angle_t *ang, pcb_bool no_subc_part, pcb_bool no_term)
+{
+	pcb_rtree_it_t it;
+	pcb_rtree_box_t pt;
+	pcb_line_t *l;
+
+/* TODO: set pt coords to x and y */
+
+	for(l = pcb_rtree_first(&it, layer->line_tree, &pt); l != NULL; l = pcb_rtree_next(&it)) {
+		/* check if line needs to be ignored and "continue;" if so:
+		   l->term is non-NULL if it's a terminal,
+		   pcb_lobj_parent_subc(pcb_parenttype_t pt, pcb_parent_t *p) returns non-NULL if it's in a subc
+		 */
+		/* if the line shouldn't be ignored, check if x;y is on the centerline and
+		   "continue;" if bad */
+		/* check if angle is required and is correct; "continue;" on mismatch */
+		return l;
+	}
+	return NULL; /* found nothing */
+}
+
+
 /* ---------------------------------------------------------------------------
  * searches for any kind of object or for a set of object types
  * the calling routine passes two pointers to allocated memory for storing
@@ -1214,13 +1334,16 @@ static int pcb_search_obj_by_location_(unsigned long Type, void **Result1, void
 
 	if (Type & PCB_TYPE_RATLINE && PCB->RatOn &&
 			SearchRatLineByLocation(objst, req_flag, (pcb_rat_t **) Result1, (pcb_rat_t **) Result2, (pcb_rat_t **) Result3))
-		return (PCB_TYPE_RATLINE);
+		return PCB_TYPE_RATLINE;
 
 	if (Type & PCB_TYPE_VIA && SearchViaByLocation(objst, req_flag, (pcb_pin_t **) Result1, (pcb_pin_t **) Result2, (pcb_pin_t **) Result3))
-		return (PCB_TYPE_VIA);
+		return PCB_TYPE_VIA;
 
-	if (Type & PCB_TYPE_PSTK && SearchPadstackByLocation(objst, req_flag, (pcb_pstk_t **) Result1, (pcb_pstk_t **) Result2, (pcb_pstk_t **) Result3))
-		return (PCB_TYPE_PSTK);
+	if (Type & PCB_TYPE_PSTK && SearchPadstackByLocation(objst, req_flag, (pcb_pstk_t **) Result1, (pcb_pstk_t **) Result2, (pcb_pstk_t **) Result3, 1))
+		return PCB_TYPE_PSTK;
+
+	if (Type & PCB_TYPE_PSTK && SearchPadstackByLocation(objst, req_flag, (pcb_pstk_t **) Result1, (pcb_pstk_t **) Result2, (pcb_pstk_t **) Result3, 0))
+		return PCB_TYPE_PSTK;
 
 	if (Type & PCB_TYPE_PIN && SearchPinByLocation(objst, req_flag, (pcb_element_t **) pr1, (pcb_pin_t **) pr2, (pcb_pin_t **) pr3))
 		HigherAvail = PCB_TYPE_PIN;
@@ -1236,6 +1359,14 @@ static int pcb_search_obj_by_location_(unsigned long Type, void **Result1, void
 		HigherAvail = PCB_TYPE_ELEMENT_NAME;
 	}
 
+	if (!HigherAvail && (Type & PCB_TYPE_SUBC_FLOATER) && (Type & PCB_TYPE_TEXT) &&
+			SearchSubcFloaterByLocation(objst, req_flag, (pcb_subc_t **)pr1, (pcb_text_t **) pr2, pr3, pcb_false)) {
+		*Result1 = ((pcb_text_t *)r2)->parent.layer;
+		*Result2 = r2;
+		*Result3 = r3;
+		return PCB_TYPE_TEXT;
+	}
+
 	if (!HigherAvail && Type & PCB_TYPE_ELEMENT &&
 			SearchElementByLocation(objst, req_flag, (pcb_element_t **) pr1, (pcb_element_t **) pr2, (pcb_element_t **) pr3, pcb_false)) {
 		pcb_box_t *box = &((pcb_element_t *) r1)->BoundingBox;
@@ -1267,29 +1398,29 @@ static int pcb_search_obj_by_location_(unsigned long Type, void **Result1, void
 			if ((HigherAvail & (PCB_TYPE_PIN | PCB_TYPE_PAD)) == 0 &&
 					Type & PCB_TYPE_POLY_POINT &&
 					SearchPointByLocation(Type, objst, req_flag, (pcb_layer_t **) Result1, (pcb_poly_t **) Result2, (pcb_point_t **) Result3))
-				return (PCB_TYPE_POLY_POINT);
+				return PCB_TYPE_POLY_POINT;
 
 			if ((HigherAvail & (PCB_TYPE_PIN | PCB_TYPE_PAD)) == 0 &&
 					Type & PCB_TYPE_LINE_POINT &&
 					SearchLinePointByLocation(objst, req_flag, (pcb_layer_t **) Result1, (pcb_line_t **) Result2, (pcb_point_t **) Result3))
-				return (PCB_TYPE_LINE_POINT);
+				return PCB_TYPE_LINE_POINT;
 
 			if ((HigherAvail & (PCB_TYPE_PIN | PCB_TYPE_PAD)) == 0 &&
 					Type & PCB_TYPE_ARC_POINT &&
 					SearchArcPointByLocation(objst, req_flag, (pcb_layer_t **) Result1, (pcb_arc_t **) Result2, (int **) Result3))
-				return (PCB_TYPE_ARC_POINT);
+				return PCB_TYPE_ARC_POINT;
 
 			if ((HigherAvail & (PCB_TYPE_PIN | PCB_TYPE_PAD)) == 0 && Type & PCB_TYPE_LINE
 					&& SearchLineByLocation(objst, req_flag, (pcb_layer_t **) Result1, (pcb_line_t **) Result2, (pcb_line_t **) Result3))
-				return (PCB_TYPE_LINE);
+				return PCB_TYPE_LINE;
 
 			if ((HigherAvail & (PCB_TYPE_PIN | PCB_TYPE_PAD)) == 0 && Type & PCB_TYPE_ARC &&
 					SearchArcByLocation(objst, req_flag, (pcb_layer_t **) Result1, (pcb_arc_t **) Result2, (pcb_arc_t **) Result3))
-				return (PCB_TYPE_ARC);
+				return PCB_TYPE_ARC;
 
 			if ((HigherAvail & (PCB_TYPE_PIN | PCB_TYPE_PAD)) == 0 && Type & PCB_TYPE_TEXT
 					&& SearchTextByLocation(objst, req_flag, (pcb_layer_t **) Result1, (pcb_text_t **) Result2, (pcb_text_t **) Result3))
-				return (PCB_TYPE_TEXT);
+				return PCB_TYPE_TEXT;
 
 			if (Type & PCB_TYPE_POLY &&
 					SearchPolygonByLocation(objst, req_flag, (pcb_layer_t **) Result1, (pcb_poly_t **) Result2, (pcb_poly_t **) Result3)) {
@@ -1299,10 +1430,10 @@ static int pcb_search_obj_by_location_(unsigned long Type, void **Result1, void
 					if (HigherBound < area)
 						break;
 					else
-						return (PCB_TYPE_POLY);
+						return PCB_TYPE_POLY;
 				}
 				else
-					return (PCB_TYPE_POLY);
+					return PCB_TYPE_POLY;
 			}
 		}
 	}
@@ -1311,58 +1442,66 @@ static int pcb_search_obj_by_location_(unsigned long Type, void **Result1, void
 		*Result1 = r1;
 		*Result2 = r2;
 		*Result3 = r3;
-		return (PCB_TYPE_PIN);
+		return PCB_TYPE_PIN;
 	}
 
 	if (HigherAvail & PCB_TYPE_PAD) {
 		*Result1 = r1;
 		*Result2 = r2;
 		*Result3 = r3;
-		return (PCB_TYPE_PAD);
+		return PCB_TYPE_PAD;
 	}
 
 	if (HigherAvail & PCB_TYPE_ELEMENT_NAME) {
 		*Result1 = r1;
 		*Result2 = r2;
 		*Result3 = r3;
-		return (PCB_TYPE_ELEMENT_NAME);
+		return PCB_TYPE_ELEMENT_NAME;
 	}
 
 	if (HigherAvail & PCB_TYPE_ELEMENT) {
 		*Result1 = r1;
 		*Result2 = r2;
 		*Result3 = r3;
-		return (PCB_TYPE_ELEMENT);
+		return PCB_TYPE_ELEMENT;
 	}
 
 	if (HigherAvail & PCB_TYPE_SUBC) {
 		*Result1 = r1;
 		*Result2 = r2;
 		*Result3 = r3;
-		return (PCB_TYPE_SUBC);
+		return PCB_TYPE_SUBC;
 	}
 
 	/* search the 'invisible objects' last */
 	if (!PCB->InvisibleObjectsOn)
-		return (PCB_TYPE_NONE);
+		return PCB_TYPE_NONE;
 
 	if (Type & PCB_TYPE_PAD &&
 			SearchPadByLocation(objst, req_flag, (pcb_element_t **) Result1, (pcb_pad_t **) Result2, (pcb_pad_t **) Result3, pcb_true))
-		return (PCB_TYPE_PAD);
+		return PCB_TYPE_PAD;
 
 	if (Type & PCB_TYPE_ELEMENT_NAME &&
 			SearchElementNameByLocation(objst, req_flag, (pcb_element_t **) Result1, (pcb_text_t **) Result2, (pcb_text_t **) Result3, pcb_true))
-		return (PCB_TYPE_ELEMENT_NAME);
+		return PCB_TYPE_ELEMENT_NAME;
+
+	if ((Type & PCB_TYPE_SUBC_FLOATER) && (Type & PCB_TYPE_TEXT) &&
+			SearchSubcFloaterByLocation(objst, req_flag, (pcb_subc_t **)pr1, (pcb_text_t **) pr2, pr3, pcb_true)) {
+		*Result1 = ((pcb_text_t *)r2)->parent.layer;
+		*Result2 = r2;
+		*Result3 = r3;
+		return PCB_TYPE_TEXT;
+	}
 
 	if (Type & PCB_TYPE_ELEMENT &&
 			SearchElementByLocation(objst, req_flag, (pcb_element_t **) Result1, (pcb_element_t **) Result2, (pcb_element_t **) Result3, pcb_true))
-		return (PCB_TYPE_ELEMENT);
+		return PCB_TYPE_ELEMENT;
 
 	if (Type & PCB_TYPE_SUBC && PCB->SubcOn &&
 			SearchSubcByLocation(objst, req_flag, (pcb_subc_t **) Result1, (pcb_subc_t **) Result2, (pcb_subc_t **) Result3, pcb_true))
-		return (PCB_TYPE_SUBC);
+		return PCB_TYPE_SUBC;
 
-	return (PCB_TYPE_NONE);
+	return PCB_TYPE_NONE;
 }
 
 int pcb_search_obj_by_location(unsigned long Type, void **Result1, void **Result2, void **Result3, pcb_coord_t X, pcb_coord_t Y, pcb_coord_t Radius)
@@ -1408,19 +1547,19 @@ static int pcb_search_obj_by_id_(pcb_data_t *Base, void **Result1, void **Result
 			if (line->ID == ID) {
 				*Result1 = (void *) layer;
 				*Result2 = *Result3 = (void *) line;
-				return (PCB_TYPE_LINE);
+				return PCB_TYPE_LINE;
 			}
 			if (line->Point1.ID == ID) {
 				*Result1 = (void *) layer;
 				*Result2 = (void *) line;
 				*Result3 = (void *) &line->Point1;
-				return (PCB_TYPE_LINE_POINT);
+				return PCB_TYPE_LINE_POINT;
 			}
 			if (line->Point2.ID == ID) {
 				*Result1 = (void *) layer;
 				*Result2 = (void *) line;
 				*Result3 = (void *) &line->Point2;
-				return (PCB_TYPE_LINE_POINT);
+				return PCB_TYPE_LINE_POINT;
 			}
 		}
 		PCB_ENDALL_LOOP;
@@ -1431,7 +1570,7 @@ static int pcb_search_obj_by_id_(pcb_data_t *Base, void **Result1, void **Result
 			if (arc->ID == ID) {
 				*Result1 = (void *) layer;
 				*Result2 = *Result3 = (void *) arc;
-				return (PCB_TYPE_ARC);
+				return PCB_TYPE_ARC;
 			}
 		}
 		PCB_ENDALL_LOOP;
@@ -1443,7 +1582,7 @@ static int pcb_search_obj_by_id_(pcb_data_t *Base, void **Result1, void **Result
 			if (text->ID == ID) {
 				*Result1 = (void *) layer;
 				*Result2 = *Result3 = (void *) text;
-				return (PCB_TYPE_TEXT);
+				return PCB_TYPE_TEXT;
 			}
 		}
 		PCB_ENDALL_LOOP;
@@ -1455,7 +1594,7 @@ static int pcb_search_obj_by_id_(pcb_data_t *Base, void **Result1, void **Result
 			if (polygon->ID == ID) {
 				*Result1 = (void *) layer;
 				*Result2 = *Result3 = (void *) polygon;
-				return (PCB_TYPE_POLY);
+				return PCB_TYPE_POLY;
 			}
 			if (type == PCB_TYPE_POLY_POINT)
 				PCB_POLY_POINT_LOOP(polygon);
@@ -1464,7 +1603,7 @@ static int pcb_search_obj_by_id_(pcb_data_t *Base, void **Result1, void **Result
 					*Result1 = (void *) layer;
 					*Result2 = (void *) polygon;
 					*Result3 = (void *) point;
-					return (PCB_TYPE_POLY_POINT);
+					return PCB_TYPE_POLY_POINT;
 				}
 			}
 			PCB_END_LOOP;
@@ -1477,7 +1616,7 @@ static int pcb_search_obj_by_id_(pcb_data_t *Base, void **Result1, void **Result
 		{
 			if (via->ID == ID) {
 				*Result1 = *Result2 = *Result3 = (void *) via;
-				return (PCB_TYPE_VIA);
+				return PCB_TYPE_VIA;
 			}
 		}
 		PCB_END_LOOP;
@@ -1499,19 +1638,19 @@ static int pcb_search_obj_by_id_(pcb_data_t *Base, void **Result1, void **Result
 		{
 			if (line->ID == ID) {
 				*Result1 = *Result2 = *Result3 = (void *) line;
-				return (PCB_TYPE_RATLINE);
+				return PCB_TYPE_RATLINE;
 			}
 			if (line->Point1.ID == ID) {
 				*Result1 = (void *) NULL;
 				*Result2 = (void *) line;
 				*Result3 = (void *) &line->Point1;
-				return (PCB_TYPE_LINE_POINT);
+				return PCB_TYPE_LINE_POINT;
 			}
 			if (line->Point2.ID == ID) {
 				*Result1 = (void *) NULL;
 				*Result2 = (void *) line;
 				*Result3 = (void *) &line->Point2;
-				return (PCB_TYPE_LINE_POINT);
+				return PCB_TYPE_LINE_POINT;
 			}
 		}
 		PCB_END_LOOP;
@@ -1541,7 +1680,7 @@ static int pcb_search_obj_by_id_(pcb_data_t *Base, void **Result1, void **Result
 	{
 		if (element->ID == ID) {
 			*Result1 = *Result2 = *Result3 = (void *) element;
-			return (PCB_TYPE_ELEMENT);
+			return PCB_TYPE_ELEMENT;
 		}
 		if (type == PCB_TYPE_ELEMENT_LINE)
 			PCB_ELEMENT_PCB_LINE_LOOP(element);
@@ -1549,7 +1688,7 @@ static int pcb_search_obj_by_id_(pcb_data_t *Base, void **Result1, void **Result
 			if (line->ID == ID) {
 				*Result1 = (void *) element;
 				*Result2 = *Result3 = (void *) line;
-				return (PCB_TYPE_ELEMENT_LINE);
+				return PCB_TYPE_ELEMENT_LINE;
 			}
 		}
 		PCB_END_LOOP;
@@ -1559,7 +1698,7 @@ static int pcb_search_obj_by_id_(pcb_data_t *Base, void **Result1, void **Result
 			if (arc->ID == ID) {
 				*Result1 = (void *) element;
 				*Result2 = *Result3 = (void *) arc;
-				return (PCB_TYPE_ELEMENT_ARC);
+				return PCB_TYPE_ELEMENT_ARC;
 			}
 		}
 		PCB_END_LOOP;
@@ -1569,7 +1708,7 @@ static int pcb_search_obj_by_id_(pcb_data_t *Base, void **Result1, void **Result
 			if (text->ID == ID) {
 				*Result1 = (void *) element;
 				*Result2 = *Result3 = (void *) text;
-				return (PCB_TYPE_ELEMENT_NAME);
+				return PCB_TYPE_ELEMENT_NAME;
 			}
 		}
 		PCB_END_LOOP;
@@ -1579,7 +1718,7 @@ static int pcb_search_obj_by_id_(pcb_data_t *Base, void **Result1, void **Result
 			if (pin->ID == ID) {
 				*Result1 = (void *) element;
 				*Result2 = *Result3 = (void *) pin;
-				return (PCB_TYPE_PIN);
+				return PCB_TYPE_PIN;
 			}
 		}
 		PCB_END_LOOP;
@@ -1589,14 +1728,14 @@ static int pcb_search_obj_by_id_(pcb_data_t *Base, void **Result1, void **Result
 			if (pad->ID == ID) {
 				*Result1 = (void *) element;
 				*Result2 = *Result3 = (void *) pad;
-				return (PCB_TYPE_PAD);
+				return PCB_TYPE_PAD;
 			}
 		}
 		PCB_END_LOOP;
 	}
 	PCB_END_LOOP;
 
-	return (PCB_TYPE_NONE);
+	return PCB_TYPE_NONE;
 }
 
 int pcb_search_obj_by_id(pcb_data_t *Base, void **Result1, void **Result2, void **Result3, int ID, int type)
@@ -1620,7 +1759,7 @@ pcb_element_t *pcb_search_elem_by_name(pcb_data_t *Base, const char *Name)
 	{
 		if (element->Name[1].TextString && PCB_NSTRCMP(element->Name[1].TextString, Name) == 0) {
 			result = element;
-			return (result);
+			return result;
 		}
 	}
 	PCB_END_LOOP;
@@ -1643,7 +1782,7 @@ int pcb_search_grid_slop(pcb_coord_t X, pcb_coord_t Y, int Type, void **Result1,
 	int ans;
 
 	ans = pcb_search_obj_by_location(Type, Result1, Result2, Result3, X, Y, PCB->Grid / 2);
-	return (ans);
+	return ans;
 }
 
 int pcb_lines_intersect(pcb_coord_t ax1, pcb_coord_t ay1, pcb_coord_t ax2, pcb_coord_t ay2, pcb_coord_t bx1, pcb_coord_t by1, pcb_coord_t bx2, pcb_coord_t by2)
@@ -1709,3 +1848,4 @@ int pcb_lines_intersect(pcb_coord_t ax1, pcb_coord_t ay1, pcb_coord_t ax2, pcb_c
 	check(by1, by2, yi);
 	return 1;
 }
+
