@@ -1,7 +1,8 @@
 /*
  *                            COPYRIGHT
  *
- *  PCB, interactive printed circuit board design
+ *  pcb-rnd, interactive printed circuit board design
+ *  (this file is based on PCB, interactive printed circuit board design)
  *  Copyright (C) 1994,1995,1996 Thomas Nau
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -18,9 +19,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- *  Contact addresses for paper mail and Email:
- *  Thomas Nau, Schlehenweg 15, 88471 Baustetten, Germany
- *  Thomas.Nau@rz.uni-ulm.de
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  *
  */
 
@@ -117,9 +119,9 @@ pcb_bool pcb_select_object(pcb_board_t *pcb)
 
 	pcb_bool changed = pcb_true;
 
-	type = pcb_search_screen(pcb_crosshair.X, pcb_crosshair.Y, PCB_SELECT_TYPES | PCB_LOOSE_SUBC, &ptr1, &ptr2, &ptr3);
+	type = pcb_search_screen(pcb_crosshair.X, pcb_crosshair.Y, PCB_SELECT_TYPES | PCB_LOOSE_SUBC | PCB_TYPE_SUBC_FLOATER, &ptr1, &ptr2, &ptr3);
 	if (type == PCB_TYPE_NONE || PCB_FLAG_TEST(PCB_FLAG_LOCK, (pcb_pin_t *) ptr2))
-		return (pcb_false);
+		return pcb_false;
 	switch (type) {
 	case PCB_TYPE_VIA:
 		pcb_undo_add_obj_to_flag(ptr1);
@@ -214,7 +216,7 @@ pcb_bool pcb_select_object(pcb_board_t *pcb)
 	}
 	pcb_draw();
 	pcb_undo_inc_serial();
-	return (changed);
+	return changed;
 }
 
 static void fix_box_dir(pcb_box_t *Box, int force_pos)
@@ -569,7 +571,7 @@ static pcb_r_dir_t pcb_select_block_cb(const pcb_box_t *box, void *cl)
  * Flag determines if the block is to be selected or unselected
  * returns pcb_true if the state of any object has changed
  */
-pcb_bool pcb_select_block(pcb_board_t *pcb, pcb_box_t *Box, pcb_bool flag)
+pcb_bool pcb_select_block(pcb_board_t *pcb, pcb_box_t *Box, pcb_bool flag, pcb_bool vis_only)
 {
 	select_ctx_t ctx;
 
@@ -580,7 +582,7 @@ pcb_bool pcb_select_block(pcb_board_t *pcb, pcb_box_t *Box, pcb_bool flag)
 
 	fix_box_dir(Box, 1);
 
-	return pcb_data_r_search(pcb->Data, PCB_OBJ_ANY, Box, NULL, pcb_select_block_cb, &ctx, NULL) == PCB_R_DIR_FOUND_CONTINUE;
+	return pcb_data_r_search(pcb->Data, PCB_OBJ_ANY, Box, NULL, pcb_select_block_cb, &ctx, NULL, vis_only) == PCB_R_DIR_FOUND_CONTINUE;
 }
 
 /* ----------------------------------------------------------------------
@@ -679,7 +681,7 @@ pcb_bool pcb_select_connection(pcb_board_t *pcb, pcb_bool Flag)
 		}
 	}
 	PCB_END_LOOP;
-	return (changed);
+	return changed;
 }
 
 /* ---------------------------------------------------------------------------
@@ -718,7 +720,7 @@ pcb_bool pcb_select_object_by_name(pcb_board_t *pcb, int Type, const char *name_
 		if (re_sei_errno(regex) != 0) {
 			pcb_message(PCB_MSG_ERROR, _("regexp error: %s\n"), re_error_str(re_sei_errno(regex)));
 			re_sei_free(regex);
-			return (pcb_false);
+			return pcb_false;
 		}
 	}
 	else {
@@ -883,5 +885,5 @@ pcb_bool pcb_select_object_by_name(pcb_board_t *pcb, int Type, const char *name_
 		free(pat);
 	if (pattern_copy != NULL)
 		free(pattern_copy);
-	return (changed);
+	return changed;
 }
