@@ -1,7 +1,8 @@
 /*
  *                            COPYRIGHT
  *
- *  PCB, interactive printed circuit board design
+ *  pcb-rnd, interactive printed circuit board design
+ *  (this file is based on PCB, interactive printed circuit board design)
  *  Copyright (C) 1994,1995,1996 Thomas Nau
  *  Copyright (C) 1997, 1998, 1999, 2000, 2001 Harry Eaton
  *
@@ -19,7 +20,13 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
- *  Contact addresses for paper mail and Email:
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
+ *
+ *
+ *  Old contact info:
  *  Harry Eaton, 6697 Buttonhole Ct, Columbia, MD 21044, USA
  *  haceaton@aplcomm.jhuapl.edu
  *
@@ -80,9 +87,7 @@ static char *gui_get_pat(pcb_search_method_t * method)
 static const char pcb_acts_Select[] =
 	"Select(Object|ToggleObject)\n"
 	"Select(All|Block|Connection)\n"
-	"Select(ElementByName|ObjectByName|PadByName|PinByName)\n"
-	"Select(ElementByName|ObjectByName|PadByName|PinByName, Name)\n"
-	"Select(TextByName|ViaByName|NetByName)\n" "Select(TextByName|ViaByName|NetByName, Name)\n" "Select(Convert)";
+	"Select(Convert)";
 
 static const char pcb_acth_Select[] = "Toggles or sets the selection.";
 
@@ -90,19 +95,6 @@ static const char pcb_acth_Select[] = "Toggles or sets the selection.";
 
 @table @code
 
-@item ElementByName
-@item ObjectByName
-@item PadByName
-@item PinByName
-@item TextByName
-@item ViaByName
-@item NetByName
-
-These all rely on having a regular expression parser built into
-@code{pcb}.  If the name is not specified then the user is prompted
-for a pattern, and all objects that match the pattern and are of the
-type specified are selected.
-
 @item Object
 @item ToggleObject
 Selects the object under the cursor.
@@ -131,41 +123,6 @@ static int pcb_act_Select(int argc, const char **argv, pcb_coord_t x, pcb_coord_
 		int type;
 
 		switch (pcb_funchash_get(function, NULL)) { /* select objects by their names */
-		case F_ElementByName:
-			type = PCB_TYPE_ELEMENT;
-			goto commonByName;
-		case F_ObjectByName:
-			type = PCB_TYPEMASK_ALL;
-			goto commonByName;
-		case F_PadByName:
-			type = PCB_TYPE_PAD;
-			goto commonByName;
-		case F_PinByName:
-			type = PCB_TYPE_PIN;
-			goto commonByName;
-		case F_TextByName:
-			type = PCB_TYPE_TEXT;
-			goto commonByName;
-		case F_ViaByName:
-			type = PCB_TYPE_VIA;
-			goto commonByName;
-		case F_NetByName:
-			type = PCB_TYPE_NET;
-			goto commonByName;
-
-		commonByName:
-			{
-				const char *pattern = PCB_ACTION_ARG(1);
-				pcb_search_method_t method = PCB_SM_REGEX;
-
-				if (pattern || (pattern = gui_get_pat(&method)) != NULL) {
-					if (pcb_select_object_by_name(PCB, type, pattern, pcb_true, method))
-						pcb_board_set_changed_flag(pcb_true);
-					if (PCB_ACTION_ARG(1) == NULL)
-						free((char*)pattern);
-				}
-				break;
-			}
 
 			/* select a single object */
 		case F_ToggleObject:
@@ -185,7 +142,7 @@ static int pcb_act_Select(int argc, const char **argv, pcb_coord_t x, pcb_coord_
 				box.Y2 = MAX(pcb_crosshair.AttachedBox.Point1.Y, pcb_crosshair.AttachedBox.Point2.Y);
 				pcb_notify_crosshair_change(pcb_false);
 				pcb_tool_notify_block();
-				if (pcb_crosshair.AttachedBox.State == PCB_CH_STATE_THIRD && pcb_select_block(PCB, &box, pcb_true)) {
+				if (pcb_crosshair.AttachedBox.State == PCB_CH_STATE_THIRD && pcb_select_block(PCB, &box, pcb_true, pcb_true)) {
 					pcb_board_set_changed_flag(pcb_true);
 					pcb_crosshair.AttachedBox.State = PCB_CH_STATE_FIRST;
 				}
@@ -193,7 +150,7 @@ static int pcb_act_Select(int argc, const char **argv, pcb_coord_t x, pcb_coord_
 				break;
 			}
 
-			/* select all visible objects */
+			/* select all visible(?) objects */
 		case F_All:
 			{
 				pcb_box_t box;
@@ -202,7 +159,7 @@ static int pcb_act_Select(int argc, const char **argv, pcb_coord_t x, pcb_coord_
 				box.Y1 = -PCB_MAX_COORD;
 				box.X2 = PCB_MAX_COORD;
 				box.Y2 = PCB_MAX_COORD;
-				if (pcb_select_block(PCB, &box, pcb_true))
+				if (pcb_select_block(PCB, &box, pcb_true, pcb_true))
 					pcb_board_set_changed_flag(pcb_true);
 				break;
 			}
@@ -265,10 +222,7 @@ static int pcb_act_Select(int argc, const char **argv, pcb_coord_t x, pcb_coord_
 /* --------------------------------------------------------------------------- */
 
 static const char pcb_acts_Unselect[] =
-	"Unselect(All|Block|Connection)\n"
-	"Unselect(ElementByName|ObjectByName|PadByName|PinByName)\n"
-	"Unselect(ElementByName|ObjectByName|PadByName|PinByName, Name)\n"
-	"Unselect(TextByName|ViaByName)\n" "Unselect(TextByName|ViaByName, Name)\n";
+	"Unselect(All|Block|Connection)\n";
 
 static const char pcb_acth_Unselect[] = "Unselects the object at the pointer location or the specified objects.";
 
@@ -285,19 +239,6 @@ Unselect all objects in a rectangle given by the cursor.
 @item Connection
 Unselect all connections with the ``found'' flag set.
 
-@item ElementByName
-@item ObjectByName
-@item PadByName
-@item PinByName
-@item TextByName
-@item ViaByName
-
-These all rely on having a regular expression parser built into
-@code{pcb}.  If the name is not specified then the user is prompted
-for a pattern, and all objects that match the pattern and are of the
-type specified are unselected.
-
-
 @end table
 
 %end-doc */
@@ -308,42 +249,6 @@ static int pcb_act_Unselect(int argc, const char **argv, pcb_coord_t x, pcb_coor
 	if (function) {
 		int type;
 		switch (pcb_funchash_get(function, NULL)) {
-			/* select objects by their names */
-		case F_ElementByName:
-			type = PCB_TYPE_ELEMENT;
-			goto commonByName;
-		case F_ObjectByName:
-			type = PCB_TYPEMASK_ALL;
-			goto commonByName;
-		case F_PadByName:
-			type = PCB_TYPE_PAD;
-			goto commonByName;
-		case F_PinByName:
-			type = PCB_TYPE_PIN;
-			goto commonByName;
-		case F_TextByName:
-			type = PCB_TYPE_TEXT;
-			goto commonByName;
-		case F_ViaByName:
-			type = PCB_TYPE_VIA;
-			goto commonByName;
-		case F_NetByName:
-			type = PCB_TYPE_NET;
-			goto commonByName;
-
-		commonByName:
-			{
-				const char *pattern = PCB_ACTION_ARG(1);
-				pcb_search_method_t method = PCB_SM_REGEX;
-
-				if (pattern || (pattern = gui_get_pat(&method)) != NULL) {
-					if (pcb_select_object_by_name(PCB, type, pattern, pcb_false, method))
-						pcb_board_set_changed_flag(pcb_true);
-					if (PCB_ACTION_ARG(1) == NULL)
-						free((char*)pattern);
-				}
-				break;
-			}
 
 			/* all objects in block */
 		case F_Block:
@@ -356,7 +261,7 @@ static int pcb_act_Unselect(int argc, const char **argv, pcb_coord_t x, pcb_coor
 				box.Y2 = MAX(pcb_crosshair.AttachedBox.Point1.Y, pcb_crosshair.AttachedBox.Point2.Y);
 				pcb_notify_crosshair_change(pcb_false);
 				pcb_tool_notify_block();
-				if (pcb_crosshair.AttachedBox.State == PCB_CH_STATE_THIRD && pcb_select_block(PCB, &box, pcb_false)) {
+				if (pcb_crosshair.AttachedBox.State == PCB_CH_STATE_THIRD && pcb_select_block(PCB, &box, pcb_false, pcb_true)) {
 					pcb_board_set_changed_flag(pcb_true);
 					pcb_crosshair.AttachedBox.State = PCB_CH_STATE_FIRST;
 				}
@@ -373,7 +278,7 @@ static int pcb_act_Unselect(int argc, const char **argv, pcb_coord_t x, pcb_coor
 				box.Y1 = -PCB_MAX_COORD;
 				box.X2 = PCB_MAX_COORD;
 				box.Y2 = PCB_MAX_COORD;
-				if (pcb_select_block(PCB, &box, pcb_false))
+				if (pcb_select_block(PCB, &box, pcb_false, pcb_false))
 					pcb_board_set_changed_flag(pcb_true);
 				break;
 			}
