@@ -20,7 +20,13 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
- *  Contact addresses for paper mail and Email:
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
+ *
+ *
+ *  Old contact info:
  *  Harry Eaton, 6697 Buttonhole Ct, Columbia, MD 21044, USA
  *  haceaton@aplcomm.jhuapl.edu
  *
@@ -97,7 +103,7 @@ static void click_timer_cb(pcb_hidval_t hv)
 			box.X2 = PCB_MAX_COORD;
 			box.Y2 = PCB_MAX_COORD;
 			/* unselect first if shift key not down */
-			if (!pcb_gui->shift_is_pressed() && pcb_select_block(PCB, &box, pcb_false))
+			if (!pcb_gui->shift_is_pressed() && pcb_select_block(PCB, &box, pcb_false, pcb_false))
 				pcb_board_set_changed_flag(pcb_true);
 			pcb_tool_notify_block();
 			pcb_crosshair.AttachedBox.Point1.X = pcb_tool_note.X;
@@ -122,7 +128,7 @@ void pcb_tool_arrow_notify_mode(void)
 	 * (pcb_tool_note.Moving) or clicked on a MOVE_TYPE
 	 * (pcb_tool_note.Hit)
 	 */
-	for (test = (PCB_SELECT_TYPES | PCB_MOVE_TYPES | PCB_LOOSE_SUBC) & ~PCB_TYPE_RATLINE; test; test &= ~type) {
+	for (test = (PCB_SELECT_TYPES | PCB_MOVE_TYPES | PCB_TYPE_SUBC_FLOATER | PCB_LOOSE_SUBC) & ~PCB_TYPE_RATLINE; test; test &= ~type) {
 		type = pcb_search_screen(pcb_tool_note.X, pcb_tool_note.Y, test, &ptr1, &ptr2, &ptr3);
 		if (!pcb_tool_note.Hit && (type & PCB_MOVE_TYPES) && !PCB_FLAG_TEST(PCB_FLAG_LOCK, (pcb_pin_t *) ptr2)) {
 			pcb_tool_note.Hit = type;
@@ -153,7 +159,7 @@ void pcb_tool_arrow_release_mode(void)
 		pcb_undo_save_serial();
 		/* unselect first if shift key not down */
 		if (!pcb_gui->shift_is_pressed()) {
-			if (pcb_select_block(PCB, &box, pcb_false))
+			if (pcb_select_block(PCB, &box, pcb_false, pcb_false))
 				pcb_board_set_changed_flag(pcb_true);
 			if (pcb_tool_note.Moving) {
 				pcb_tool_note.Moving = 0;
@@ -178,7 +184,7 @@ void pcb_tool_arrow_release_mode(void)
 		box.Y2 = pcb_crosshair.AttachedBox.Point2.Y;
 
 		pcb_undo_restore_serial();
-		if (pcb_select_block(PCB, &box, pcb_true))
+		if (pcb_select_block(PCB, &box, pcb_true, pcb_true))
 			pcb_board_set_changed_flag(pcb_true);
 		else if (pcb_bumped)
 			pcb_undo_inc_serial();
