@@ -20,7 +20,13 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
- *  Contact addresses for paper mail and Email:
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
+ *
+ *
+ *  Old contact info:
  *  Harry Eaton, 6697 Buttonhole Ct, Columbia, MD 21044, USA
  *  haceaton@aplcomm.jhuapl.edu
  *
@@ -48,6 +54,7 @@ void pcb_tool_buffer_notify_mode(void)
 	void *ptr1, *ptr2, *ptr3;
 	pcb_text_t estr[PCB_MAX_ELEMENTNAMES];
 	pcb_element_t *e = 0;
+	pcb_subc_t *orig_subc = NULL;
 
 	if (pcb_gui->shift_is_pressed()) {
 		int type = pcb_search_screen(pcb_tool_note.X, pcb_tool_note.Y, PCB_TYPE_ELEMENT | PCB_TYPE_SUBC, &ptr1, &ptr2,
@@ -63,12 +70,15 @@ void pcb_tool_buffer_notify_mode(void)
 				pcb_element_remove(e);
 			}
 		}
+		if (type == PCB_TYPE_SUBC) {
+			orig_subc = (pcb_subc_t *) ptr1;
+			pcb_subc_remove(orig_subc);
+		}
 	}
 	if (pcb_buffer_copy_to_layout(PCB, pcb_tool_note.X, pcb_tool_note.Y))
 		pcb_board_set_changed_flag(pcb_true);
 	if (e) {
-		int type = pcb_search_screen(pcb_tool_note.X, pcb_tool_note.Y, PCB_TYPE_ELEMENT | PCB_TYPE_SUBC, &ptr1, &ptr2,
-														&ptr3);
+		int type = pcb_search_screen(pcb_tool_note.X, pcb_tool_note.Y, PCB_TYPE_ELEMENT | PCB_TYPE_SUBC, &ptr1, &ptr2, &ptr3);
 		if (type == PCB_TYPE_ELEMENT && ptr1) {
 			int i, save_n;
 			e = (pcb_element_t *) ptr1;
@@ -82,12 +92,23 @@ void pcb_tool_buffer_notify_mode(void)
 				memcpy(&(e->Name[i]), &(estr[i]), sizeof(pcb_text_t));
 				e->Name[i].Element = e;
 				pcb_text_bbox(pcb_font(PCB, 0, 1), &(e->Name[i]));
-				pcb_r_insert_entry(PCB->Data->name_tree[i], (pcb_box_t *) & (e->Name[i]), 0);
+				pcb_r_insert_entry(PCB->Data->name_tree[i], (pcb_box_t *) & (e->Name[i]));
 				if (i == save_n)
 					pcb_elem_name_invalidate_draw(e);
 			}
 		}
 	}
+
+	if (orig_subc != NULL) {
+		int type = pcb_search_screen(pcb_tool_note.X, pcb_tool_note.Y, PCB_TYPE_SUBC, &ptr1, &ptr2, &ptr3);
+		if (type == PCB_TYPE_SUBC && (ptr1 != NULL)) {
+			int n;
+			pcb_attribute_list_t *dst = &(((pcb_subc_t *)ptr1)->Attributes), *src = &orig_subc->Attributes;
+			for (n = 0; n < src->Number; n++)
+				if (strcmp(src->List[n].name, "footprint") != 0)
+					pcb_attribute_put(dst, src->List[n].name, src->List[n].value);
+		}
+	}
 }
 
 void pcb_tool_buffer_release_mode(void)
