@@ -20,7 +20,13 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
- *  Contact addresses for paper mail and Email:
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
+ *
+ *
+ *  Old contact info:
  *  Harry Eaton, 6697 Buttonhole Ct, Columbia, MD 21044, USA
  *  haceaton@aplcomm.jhuapl.edu
  *
@@ -61,15 +67,21 @@ static void notify_line(void)
 		pcb_crosshair_set_local_ref(pcb_crosshair.X, pcb_crosshair.Y, pcb_true);
 	switch (pcb_crosshair.AttachedLine.State) {
 	case PCB_CH_STATE_FIRST:						/* first point */
-		if (PCB->RatDraw && pcb_search_screen(pcb_crosshair.X, pcb_crosshair.Y, PCB_TYPE_PAD | PCB_TYPE_PIN, &ptr1, &ptr1, &ptr1) == PCB_TYPE_NONE) {
+#warning subc TODO: this should work on heavy terminals too!
+		if (PCB->RatDraw && pcb_search_screen(pcb_crosshair.X, pcb_crosshair.Y, PCB_TYPE_PAD | PCB_TYPE_PIN | PCB_TYPE_PSTK | PCB_TYPE_SUBC_PART, &ptr1, &ptr1, &ptr1) == PCB_TYPE_NONE) {
 			pcb_gui->beep();
 			break;
 		}
 		if (conf_core.editor.auto_drc) {
-			type = pcb_search_screen(pcb_crosshair.X, pcb_crosshair.Y, PCB_TYPE_PIN | PCB_TYPE_PAD | PCB_TYPE_VIA, &ptr1, &ptr2, &ptr3);
+			type = pcb_search_screen(pcb_crosshair.X, pcb_crosshair.Y, PCB_TYPE_PIN | PCB_TYPE_PAD | PCB_TYPE_VIA | PCB_TYPE_PSTK | PCB_TYPE_SUBC_PART, &ptr1, &ptr2, &ptr3);
 			pcb_lookup_conn(pcb_crosshair.X, pcb_crosshair.Y, pcb_true, 1, PCB_FLAG_FOUND);
 		}
-		if (type == PCB_TYPE_PIN || type == PCB_TYPE_VIA) {
+		if (type == PCB_TYPE_PSTK) {
+			pcb_pstk_t *pstk = (pcb_pstk_t *)ptr2;
+			pcb_crosshair.AttachedLine.Point1.X = pstk->x;
+			pcb_crosshair.AttachedLine.Point1.Y = pstk->y;
+		}
+		else if (type == PCB_TYPE_PIN || type == PCB_TYPE_VIA) {
 			pcb_crosshair.AttachedLine.Point1.X = pcb_crosshair.AttachedLine.Point2.X = ((pcb_pin_t *) ptr2)->X;
 			pcb_crosshair.AttachedLine.Point1.Y = pcb_crosshair.AttachedLine.Point2.Y = ((pcb_pin_t *) ptr2)->Y;
 		}
@@ -144,7 +156,7 @@ void pcb_tool_line_notify_mode(void)
 		/* place a via if vias are visible, the layer is
 			 in a new group since the last line and there
 			 isn't a pin already here */
-		if (PCB->ViaOn 
+		if (conf_core.editor.auto_via && PCB->ViaOn
 				&& pcb_layer_get_group_(CURRENT) != pcb_layer_get_group_(last_layer)
 				&& pcb_search_obj_by_location(PCB_TYPEMASK_PIN, &ptr1, &ptr2, &ptr3,
 																			pcb_crosshair.AttachedLine.Point1.X,
@@ -325,6 +337,24 @@ void pcb_tool_line_draw_attached(void)
 			pcb_route_draw_drc(&pcb_crosshair.Route,pcb_crosshair.GC);
 		pcb_gui->set_color(pcb_crosshair.GC, conf_core.appearance.color.crosshair);
 	}
+	else {
+		/* Draw a circle (0 length line) to show where the line will be placed */
+    if(CURRENT)
+			pcb_gui->set_color(pcb_crosshair.GC,CURRENT->meta.real.color);
+
+		pcb_draw_wireframe_line(pcb_crosshair.GC,
+														pcb_crosshair.X, pcb_crosshair.Y,
+														pcb_crosshair.X, pcb_crosshair.Y, 
+														conf_core.design.line_thickness,0 );
+
+		if(conf_core.editor.show_drc) {
+			pcb_gui->set_color(pcb_crosshair.GC,conf_core.appearance.color.cross);
+			pcb_draw_wireframe_line(pcb_crosshair.GC,
+															pcb_crosshair.X, pcb_crosshair.Y,
+															pcb_crosshair.X, pcb_crosshair.Y, 
+															conf_core.design.line_thickness + (2 * PCB->Bloat),0 );
+		}
+	}
 }
 
 pcb_bool pcb_tool_line_undo_act(void)
