@@ -20,7 +20,13 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
- *  Contact addresses for paper mail and Email:
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
+ *
+ *
+ *  Old contact info:
  *  Harry Eaton, 6697 Buttonhole Ct, Columbia, MD 21044, USA
  *  haceaton@aplcomm.jhuapl.edu
  *
@@ -64,12 +70,34 @@ void pcb_tool_text_notify_mode(void)
 	}
 }
 
+void pcb_tool_text_draw_attached(void)
+{
+	pcb_text_t text;
+	int flag = PCB_FLAG_CLEARLINE;
+
+	if (pcb_layer_flags(PCB, INDEXOFCURRENT) & PCB_LYT_BOTTOM)
+		flag |= PCB_FLAG_ONSOLDER;
+
+	text.X = pcb_crosshair.X;
+	text.Y = pcb_crosshair.Y;
+	text.Direction = 0;
+	text.Flags = pcb_flag_make(flag);
+	text.Scale = conf_core.design.text_scale;
+	text.TextString = "A";
+	text.fid = conf_core.design.text_font_id;
+	text.ID = 0;
+
+	pcb_text_draw_xor(&text,0,0);
+
+}
+
 pcb_tool_t pcb_tool_text = {
 	"text", NULL, 100,
 	pcb_tool_text_notify_mode,
 	NULL,
 	NULL,
-	NULL,
+	pcb_tool_text_draw_attached,
 	NULL,
 	NULL
 };
+
