@@ -1,7 +1,8 @@
 /*
  *                            COPYRIGHT
  *
- *  PCB, interactive printed circuit board design
+ *  pcb-rnd, interactive printed circuit board design
+ *  (this file is based on PCB, interactive printed circuit board design)
  *  Copyright (C) 1994,1995,1996 Thomas Nau
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -18,9 +19,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- *  Contact addresses for paper mail and Email:
- *  Thomas Nau, Schlehenweg 15, 88471 Baustetten, Germany
- *  Thomas.Nau@rz.uni-ulm.de
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  *
  */
 
@@ -64,6 +66,7 @@
 
 #include "obj_elem_draw.h"
 #include "obj_poly_draw.h"
+#include "obj_subc_parent.h"
 
 #define STEP_REMOVELIST 500
 #define STEP_UNDOLIST   500
@@ -153,9 +156,9 @@ static pcb_bool UndoRotate90(UndoListTypePtr Entry)
 		pcb_obj_rotate90(type, ptr1, ptr2, ptr3,
 								 Entry->Data.Rotate.CenterX, Entry->Data.Rotate.CenterY, (4 - Entry->Data.Rotate.Steps) & 0x03);
 		Entry->Data.Rotate.Steps = (4 - Entry->Data.Rotate.Steps) & 0x03;
-		return (pcb_true);
+		return pcb_true;
 	}
-	return (pcb_false);
+	return pcb_false;
 }
 
 /* ---------------------------------------------------------------------------
@@ -211,9 +214,9 @@ static pcb_bool UndoChangeName(UndoListTypePtr Entry)
 	type = pcb_search_obj_by_id(PCB->Data, &ptr1, &ptr2, &ptr3, Entry->ID, Entry->Kind);
 	if (type != PCB_TYPE_NONE) {
 		Entry->Data.ChangeName.Name = (char *) (pcb_chg_obj_name(type, ptr1, ptr2, ptr3, Entry->Data.ChangeName.Name));
-		return (pcb_true);
+		return pcb_true;
 	}
-	return (pcb_false);
+	return pcb_false;
 }
 
 /* ---------------------------------------------------------------------------
@@ -229,9 +232,9 @@ static pcb_bool UndoChangePinnum(UndoListTypePtr Entry)
 	type = pcb_search_obj_by_id(PCB->Data, &ptr1, &ptr2, &ptr3, Entry->ID, Entry->Kind);
 	if (type != PCB_TYPE_NONE) {
 		Entry->Data.ChangeName.Name = (char *) (pcb_chg_obj_pinnum(type, ptr1, ptr2, ptr3, Entry->Data.ChangeName.Name));
-		return (pcb_true);
+		return pcb_true;
 	}
-	return (pcb_false);
+	return pcb_false;
 }
 
 /* ---------------------------------------------------------------------------
@@ -252,9 +255,9 @@ static pcb_bool UndoChange2ndSize(UndoListTypePtr Entry)
 		((pcb_pin_t *) ptr2)->DrillingHole = Entry->Data.Size;
 		Entry->Data.Size = swap;
 		pcb_draw_obj((pcb_any_obj_t *)ptr2);
-		return (pcb_true);
+		return pcb_true;
 	}
-	return (pcb_false);
+	return pcb_false;
 }
 
 /* ---------------------------------------------------------------------------
@@ -279,13 +282,13 @@ static pcb_bool UndoChangeAngles(UndoListTypePtr Entry)
 		a->StartAngle = Entry->Data.AngleChange.angle[0];
 		a->Delta = Entry->Data.AngleChange.angle[1];
 		pcb_arc_bbox(a);
-		pcb_r_insert_entry(Layer->arc_tree, (pcb_box_t *) a, 0);
+		pcb_r_insert_entry(Layer->arc_tree, (pcb_box_t *) a);
 		Entry->Data.AngleChange.angle[0] = old_sa;
 		Entry->Data.AngleChange.angle[1] = old_da;
 		pcb_draw_obj((pcb_any_obj_t *)a);
-		return (pcb_true);
+		return pcb_true;
 	}
-	return (pcb_false);
+	return pcb_false;
 }
 
 /* ---------------------------------------------------------------------------
@@ -310,13 +313,13 @@ static pcb_bool UndoChangeRadii(UndoListTypePtr Entry)
 		a->Width = Entry->Data.Move.DX;
 		a->Height = Entry->Data.Move.DY;
 		pcb_arc_bbox(a);
-		pcb_r_insert_entry(Layer->arc_tree, (pcb_box_t *) a, 0);
+		pcb_r_insert_entry(Layer->arc_tree, (pcb_box_t *) a);
 		Entry->Data.Move.DX = old_w;
 		Entry->Data.Move.DY = old_h;
 		pcb_draw_obj((pcb_any_obj_t *)a);
-		return (pcb_true);
+		return pcb_true;
 	}
-	return (pcb_false);
+	return pcb_false;
 }
 
 /* ---------------------------------------------------------------------------
@@ -340,9 +343,9 @@ static pcb_bool UndoChangeClearSize(UndoListTypePtr Entry)
 		Entry->Data.Size = swap;
 		if (pcb_undo_and_draw)
 			pcb_draw_obj((pcb_any_obj_t *)ptr2);
-		return (pcb_true);
+		return pcb_true;
 	}
-	return (pcb_false);
+	return pcb_false;
 }
 
 /* ---------------------------------------------------------------------------
@@ -367,9 +370,9 @@ static pcb_bool UndoChangeMaskSize(UndoListTypePtr Entry)
 		Entry->Data.Size = swap;
 		if (pcb_undo_and_draw)
 			pcb_draw_obj((pcb_any_obj_t *)ptr2);
-		return (pcb_true);
+		return pcb_true;
 	}
-	return (pcb_false);
+	return pcb_false;
 }
 
 
@@ -401,9 +404,9 @@ static pcb_bool UndoChangeSize(UndoListTypePtr Entry)
 		pcb_poly_clear_from_poly(PCB->Data, type, ptr1, ptr2);
 		if (pcb_undo_and_draw)
 			pcb_draw_obj((pcb_any_obj_t *)ptr2);
-		return (pcb_true);
+		return pcb_true;
 	}
-	return (pcb_false);
+	return pcb_false;
 }
 
 /* ---------------------------------------------------------------------------
@@ -445,11 +448,11 @@ static pcb_bool UndoFlag(UndoListTypePtr Entry)
 
 		if (pcb_undo_and_draw && must_redraw)
 			pcb_draw_obj((pcb_any_obj_t *)ptr2);
-		return (pcb_true);
+		return pcb_true;
 	}
 	pcb_message(PCB_MSG_ERROR, "hace Internal error: Can't find ID %d type %08x\n", Entry->ID, Entry->Kind);
 	pcb_message(PCB_MSG_ERROR, "for UndoFlag Operation. Previous flags: %s\n", pcb_strflg_f2s(Entry->Data.Flags, 0, NULL));
-	return (pcb_false);
+	return pcb_false;
 }
 
 /* ---------------------------------------------------------------------------
@@ -470,10 +473,10 @@ static pcb_bool UndoMirror(UndoListTypePtr Entry)
 		pcb_element_mirror(PCB->Data, element, Entry->Data.Move.DY);
 		if (pcb_undo_and_draw)
 			pcb_elem_invalidate_draw(element);
-		return (pcb_true);
+		return pcb_true;
 	}
 	pcb_message(PCB_MSG_ERROR, "hace Internal error: UndoMirror on object type %d\n", type);
-	return (pcb_false);
+	return pcb_false;
 }
 
 /* ---------------------------------------------------------------------------
@@ -494,10 +497,10 @@ static pcb_bool UndoOtherSide(UndoListTypePtr Entry)
 		pcb_subc_change_side(&subc, Entry->Data.Move.DY);
 		if (pcb_undo_and_draw)
 			DrawSubc(subc);
-		return (pcb_true);
+		return pcb_true;
 	}
 	pcb_message(PCB_MSG_ERROR, "hace Internal error: UndoOtherside on object type %x\n", type);
-	return (pcb_false);
+	return pcb_false;
 }
 
 /* ---------------------------------------------------------------------------
@@ -519,9 +522,9 @@ static pcb_bool UndoCopyOrCreate(UndoListTypePtr Entry)
 		/* in order to make this re-doable we move it to the pcb_removelist */
 		pcb_move_obj_to_buffer(PCB, pcb_removelist, PCB->Data, type, ptr1, ptr2, ptr3);
 		Entry->Type = PCB_UNDO_REMOVE;
-		return (pcb_true);
+		return pcb_true;
 	}
-	return (pcb_false);
+	return pcb_false;
 }
 
 /* ---------------------------------------------------------------------------
@@ -539,9 +542,9 @@ static pcb_bool UndoMove(UndoListTypePtr Entry)
 		pcb_move_obj(type, ptr1, ptr2, ptr3, -Entry->Data.Move.DX, -Entry->Data.Move.DY);
 		Entry->Data.Move.DX *= -1;
 		Entry->Data.Move.DY *= -1;
-		return (pcb_true);
+		return pcb_true;
 	}
-	return (pcb_false);
+	return pcb_false;
 }
 
 /* ----------------------------------------------------------------------
@@ -552,17 +555,29 @@ static pcb_bool UndoRemove(UndoListTypePtr Entry)
 {
 	void *ptr1, *ptr2, *ptr3;
 	int type;
+	Removed *r = &Entry->Data.Removed;
+	pcb_data_t *data = PCB->Data;
 
 	/* lookup entry by it's ID */
 	type = pcb_search_obj_by_id(pcb_removelist, &ptr1, &ptr2, &ptr3, Entry->ID, Entry->Kind);
 	if (type != PCB_TYPE_NONE) {
-		pcb_move_obj_to_buffer(PCB, PCB->Data, pcb_removelist, type, ptr1, ptr2, ptr3);
+		if (r->p_subc_id > 0) { /* need to use a subc layer - putting back a floater */
+			void *p1, *p2, *p3;
+			if (pcb_search_obj_by_id(PCB->Data, &p1, &p2, &p3, r->p_subc_id, PCB_TYPE_SUBC) != 0) {
+				pcb_subc_t *subc = p2;
+				if (r->p_subc_layer < subc->data->LayerN) {
+					data = subc->data;
+					ptr1 = &data->Layer[r->p_subc_layer];
+				}
+			}
+		}
+		pcb_move_obj_to_buffer(PCB, data, pcb_removelist, type, ptr1, ptr2, ptr3);
 		if (pcb_undo_and_draw)
 			DrawRecoveredObject((pcb_any_obj_t *)ptr2);
 		Entry->Type = PCB_UNDO_CREATE;
-		return (pcb_true);
+		return pcb_true;
 	}
-	return (pcb_false);
+	return pcb_false;
 }
 
 /* ----------------------------------------------------------------------
@@ -581,9 +596,9 @@ static pcb_bool UndoMoveToLayer(UndoListTypePtr Entry)
 		swap = pcb_layer_id(PCB->Data, (pcb_layer_t *) ptr1);
 		pcb_move_obj_to_layer(type, ptr1, ptr2, ptr3, LAYER_PTR(Entry->Data.MoveToLayer.OriginalLayer), pcb_true);
 		Entry->Data.MoveToLayer.OriginalLayer = swap;
-		return (pcb_true);
+		return pcb_true;
 	}
-	return (pcb_false);
+	return pcb_false;
 }
 
 /* ---------------------------------------------------------------------------
@@ -617,11 +632,11 @@ static pcb_bool UndoRemovePoint(UndoListTypePtr Entry)
 			Entry->Type = PCB_UNDO_INSERT_POINT;
 			Entry->ID = Entry->Data.RemovedPoint.ID;
 			Entry->Kind = PCB_TYPE_POLY_POINT;
-			return (pcb_true);
+			return pcb_true;
 		}
 
 	default:
-		return (pcb_false);
+		return pcb_false;
 	}
 }
 
@@ -670,11 +685,11 @@ static pcb_bool UndoInsertPoint(UndoListTypePtr Entry)
 			pcb_destroy_object(PCB->Data, PCB_TYPE_POLY_POINT, layer, polygon, pnt);
 			if (pcb_undo_and_draw && layer->meta.real.vis)
 				pcb_poly_invalidate_draw(layer, polygon);
-			return (pcb_true);
+			return pcb_true;
 		}
 
 	default:
-		return (pcb_false);
+		return pcb_false;
 	}
 }
 
@@ -710,7 +725,7 @@ static pcb_bool UndoSwapCopiedObject(UndoListTypePtr Entry)
 	obj = (pcb_any_obj_t *) pcb_move_obj_to_buffer(PCB, PCB->Data, pcb_removelist, type, ptr1, ptr2, ptr3);
 	if (Entry->Kind == PCB_TYPE_POLY)
 		pcb_poly_init_clip(PCB->Data, (pcb_layer_t *) ptr1b, (pcb_poly_t *) obj);
-	return (pcb_true);
+	return pcb_true;
 }
 
 /* ---------------------------------------------------------------------------
@@ -742,9 +757,21 @@ static pcb_bool UndoLayerMove(UndoListTypePtr Entry)
 	LayerChangeTypePtr l = &Entry->Data.LayerChange;
 	int tmp;
 
-	tmp = l->new_index;
-	l->new_index = l->old_index;
-	l->old_index = tmp;
+	if (l->old_index == -1) { /* was creating new */
+		l->old_index = l->at;
+		l->at = l->new_index;
+		l->new_index = -1;
+	}
+	else if (l->new_index == -1) { /* was deleting existing */
+		l->new_index = l->at;
+		l->at = l->old_index;
+		l->old_index = -1;
+	}
+	else {
+		tmp = l->new_index;
+		l->new_index = l->old_index;
+		l->old_index = tmp;
+	}
 
 	if (pcb_layer_move(PCB, l->old_index, l->new_index, -1))
 		return pcb_false;
@@ -993,13 +1020,43 @@ void pcb_undo_add_obj_to_rotate(int Type, void *Ptr1, void *Ptr2, void *Ptr3, pc
  */
 void pcb_undo_move_obj_to_remove(int Type, void *Ptr1, void *Ptr2, void *Ptr3)
 {
+	UndoListTypePtr Entry;
+	Removed *r;
+	pcb_any_obj_t *o = Ptr3;
+	pcb_subc_t *subc;
+
 	if (Locked)
 		return;
 
 	if (!pcb_removelist)
 		pcb_removelist = pcb_buffer_new(NULL);
 
-	GetUndoSlot(PCB_UNDO_REMOVE, PCB_OBJECT_ID(Ptr3), Type);
+	Entry = GetUndoSlot(PCB_UNDO_REMOVE, PCB_OBJECT_ID(Ptr3), Type);
+	r = &Entry->Data.Removed;
+	r->p_subc_id = 0;
+
+	switch(o->type) {
+		case PCB_OBJ_LINE:
+		case PCB_OBJ_ARC:
+		case PCB_OBJ_TEXT:
+		case PCB_OBJ_POLY:
+			subc = pcb_obj_parent_subc(Ptr3);
+			if (subc != NULL) {
+				r->p_subc_id = subc->ID;
+				r->p_subc_layer = o->parent.layer - subc->data->Layer;
+			}
+			break;
+		case PCB_OBJ_PSTK:
+			subc = pcb_obj_parent_subc(Ptr3);
+			if (subc != NULL)
+				r->p_subc_id = subc->ID;
+			break;
+#warning subc TODO: floater subc in subc should remember its subc parent too
+		default:
+			break;
+	}
+
+
 	pcb_move_obj_to_buffer(PCB, pcb_removelist, PCB->Data, Type, Ptr1, Ptr2, Ptr3);
 }
 
@@ -1102,7 +1159,7 @@ void pcb_undo_add_obj_to_move(int Type, void *Ptr1, void *Ptr2, void *Ptr3, pcb_
 /* ---------------------------------------------------------------------------
  * adds an object to the list of objects with changed names
  */
-void pcb_undo_add_obj_to_change_name(int Type, void *Ptr1, void *Ptr2, void *Ptr3, char *OldName)
+void pcb_undo_add_obj_to_change_name(int Type, void *Ptr1, void *Ptr2, void *Ptr3, const char *OldName)
 {
 	UndoListTypePtr undo;
 
@@ -1293,7 +1350,7 @@ void pcb_undo_add_obj_to_change_radii(int Type, void *Ptr1, void *Ptr2, void *Pt
 /* ---------------------------------------------------------------------------
  * adds a layer change (new, delete, move) to the undo list.
  */
-void pcb_undo_add_layer_move(int old_index, int new_index)
+void pcb_undo_add_layer_move(int old_index, int new_index, int at)
 {
 	UndoListTypePtr undo;
 
@@ -1301,6 +1358,7 @@ void pcb_undo_add_layer_move(int old_index, int new_index)
 		undo = GetUndoSlot(PCB_UNDO_LAYERMOVE, 0, 0);
 		undo->Data.LayerChange.old_index = old_index;
 		undo->Data.LayerChange.new_index = new_index;
+		undo->Data.LayerChange.at = at;
 	}
 }
 
