@@ -1,7 +1,8 @@
 /*
  *                            COPYRIGHT
  *
- *  PCB, interactive printed circuit board design
+ *  pcb-rnd, interactive printed circuit board design
+ *  (this file is based on PCB, interactive printed circuit board design)
  *  Copyright (C) 2011 Andrew Poelstra
  *  Copyright (C) 2016,2017 Tibor 'Igor2' Palinkas
  *
@@ -19,7 +20,13 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- *  Contact addresses for paper mail and Email:
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
+ *
+ *
+ *  Old contact info:
  *  Andrew Poelstra, 16966 60A Ave, V3S 8X5 Surrey, BC, Canada
  *  asp11@sfu.ca
  *
@@ -150,7 +157,7 @@ pcb_increments_t pcb_increments[] = {
 
 #define N_INCREMENTS (sizeof pcb_increments / sizeof pcb_increments[0])
 
-const pcb_unit_t *get_unit_struct(const char *suffix)
+const pcb_unit_t *get_unit_struct_(const char *suffix, int strict)
 {
 	int i;
 	int s_len = 0;
@@ -170,14 +177,29 @@ const pcb_unit_t *get_unit_struct(const char *suffix)
 	}
 
 	/* Do lookup */
-	if (s_len > 0)
+	if (s_len <= 0)
+		return NULL;
+
+	if (strict) {
+		for (i = 0; i < N_UNITS; ++i)
+			if (strcmp(suffix, pcb_units[i].suffix) == 0)
+				return &pcb_units[i];
+	}
+	else {
 		for (i = 0; i < N_UNITS; ++i)
 			if (strncmp(suffix, pcb_units[i].suffix, s_len) == 0 || strncmp(suffix, pcb_units[i].alias[0], s_len) == 0)
 				return &pcb_units[i];
+	}
 
 	return NULL;
 }
 
+const pcb_unit_t *get_unit_struct(const char *suffix)
+{
+	return get_unit_struct_(suffix, 0);
+}
+
+
 const pcb_unit_t *get_unit_struct_by_allow(enum pcb_allow_e allow)
 {
 	int i;
