@@ -1,13 +1,11 @@
-/*!
- * \file autocrop.c
- *
- * \brief Autocrop plug-in for PCB.
+/*
+ * Autocrop plug-in for PCB.
  * Reduce the board dimensions to just enclose the elements.
  *
- * \author Copyright (C) 2007 Ben Jackson <ben@ben.com> based on teardrops.c by
+ * Copyright (C) 2007 Ben Jackson <ben@ben.com> based on teardrops.c by
  * Copyright (C) 2006 DJ Delorie <dj@delorie.com>
  *
- * \copyright Licensed under the terms of the GNU General Public
+ * Licensed under the terms of the GNU General Public
  * License, version 2 or later.
  *
  * Ported to pcb-rnd by Tibor 'Igor2' Palinkas in 2016.
@@ -69,57 +67,38 @@ static const char autocrop_syntax[] =
 
 static void *MyMoveViaLowLevel(pcb_data_t * Data, pcb_pin_t * Via, pcb_coord_t dx, pcb_coord_t dy)
 {
-	if (Data) {
-		pcb_poly_restore_to_poly(Data, PCB_TYPE_VIA, Via, Via);
+	if (Data)
 		pcb_r_delete_entry(Data->via_tree, (pcb_box_t *) Via);
-	}
 	pcb_via_move(Via, dx, dy);
-	if (Data) {
-		pcb_r_insert_entry(Data->via_tree, (pcb_box_t *) Via, 0);
-		pcb_poly_clear_from_poly(Data, PCB_TYPE_VIA, Via, Via);
-	}
+	if (Data)
+		pcb_r_insert_entry(Data->via_tree, (pcb_box_t *) Via);
 	return Via;
 }
 
 static void *MyMoveLineLowLevel(pcb_data_t * Data, pcb_layer_t * Layer, pcb_line_t * Line, pcb_coord_t dx, pcb_coord_t dy)
 {
-	if (Data) {
-		pcb_poly_restore_to_poly(Data, PCB_TYPE_LINE, Layer, Line);
-		pcb_r_delete_entry(Layer->line_tree, (pcb_box_t *) Line);
-	}
 	pcb_line_move(Line, dx, dy);
-	if (Data) {
-		pcb_r_insert_entry(Layer->line_tree, (pcb_box_t *) Line, 0);
-		pcb_poly_clear_from_poly(Data, PCB_TYPE_LINE, Layer, Line);
-	}
 	return Line;
 }
 
 static void *MyMoveArcLowLevel(pcb_data_t * Data, pcb_layer_t * Layer, pcb_arc_t * Arc, pcb_coord_t dx, pcb_coord_t dy)
 {
-	if (Data) {
-		pcb_poly_restore_to_poly(Data, PCB_TYPE_ARC, Layer, Arc);
+	if (Data)
 		pcb_r_delete_entry(Layer->arc_tree, (pcb_box_t *) Arc);
-	}
 	pcb_arc_move(Arc, dx, dy);
-	if (Data) {
-		pcb_r_insert_entry(Layer->arc_tree, (pcb_box_t *) Arc, 0);
-		pcb_poly_clear_from_poly(Data, PCB_TYPE_ARC, Layer, Arc);
-	}
+	if (Data)
+		pcb_r_insert_entry(Layer->arc_tree, (pcb_box_t *) Arc);
 	return Arc;
 }
 
 static void *Mypcb_poly_move(pcb_data_t * Data, pcb_layer_t * Layer, pcb_poly_t * Polygon, pcb_coord_t dx, pcb_coord_t dy)
 {
-	if (Data) {
+	if (Data)
 		pcb_r_delete_entry(Layer->polygon_tree, (pcb_box_t *) Polygon);
-	}
 	/* move.c actually only moves points, note no Data/Layer args */
 	pcb_poly_move(Polygon, dx, dy);
-	if (Data) {
-		pcb_r_insert_entry(Layer->polygon_tree, (pcb_box_t *) Polygon, 0);
-		pcb_poly_init_clip(Data, Layer, Polygon);
-	}
+	if (Data)
+		pcb_r_insert_entry(Layer->polygon_tree, (pcb_box_t *) Polygon);
 	return Polygon;
 }
 
@@ -129,12 +108,11 @@ static void *MyMoveTextLowLevel(pcb_layer_t * Layer, pcb_text_t * Text, pcb_coor
 		pcb_r_delete_entry(Layer->text_tree, (pcb_box_t *) Text);
 	pcb_text_move(Text, dx, dy);
 	if (Layer)
-		pcb_r_insert_entry(Layer->text_tree, (pcb_box_t *) Text, 0);
+		pcb_r_insert_entry(Layer->text_tree, (pcb_box_t *) Text);
 	return Text;
 }
 
-/*!
- * \brief Move everything.
+/* Move everything.
  *
  * Call our own 'MyMove*LowLevel' where they don't exist in move.c.
  * This gets very slow if there are large polygons present, since every
@@ -235,9 +213,9 @@ static int autocrop(int argc, const char **argv, pcb_coord_t x, pcb_coord_t y)
 		return 0;
 	}
 	/* Resize -- XXX cannot be undone */
-	PCB->MaxWidth = box->X2;
-	PCB->MaxHeight = box->Y2;
 	MoveAll(dx, dy);
+	pcb_board_resize(box->X2, box->Y2);
+	pcb_data_clip_all(PCB->Data, pcb_true);
 	pcb_undo_inc_serial();
 	pcb_redraw();
 	pcb_board_set_changed_flag(1);
