@@ -1,7 +1,8 @@
 /*
  *                            COPYRIGHT
  *
- *  PCB, interactive printed circuit board design
+ *  pcb-rnd, interactive printed circuit board design
+ *  (this file is based on PCB, interactive printed circuit board design)
  *  Copyright (C) 1994,1995,1996 Thomas Nau
  *  Copyright (C) 1998,1999,2000,2001 harry eaton
  *
@@ -19,7 +20,13 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- *  Contact addresses for paper mail and Email:
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
+ *
+ *
+ *  Old contact info:
  *  harry eaton, 6697 Buttonhole Ct, Columbia, MD 21044 USA
  *  haceaton@aplcomm.jhuapl.edu
  *
@@ -226,6 +233,14 @@ static vtp0_t collectSelectedElements()
 		}
 	}
 	PCB_END_LOOP;
+	PCB_SUBC_LOOP(PCB->Data);
+	{
+		if (PCB_FLAG_TEST(PCB_FLAG_SELECTED, subc)) {
+			pcb_subc_t **epp = (pcb_subc_t **)vtp0_alloc_append(&list, 1);
+			*epp = subc;
+		}
+	}
+	PCB_END_LOOP;
 	return list;
 }
 
@@ -601,8 +616,10 @@ static double ComputeCost(pcb_netlist_t *Nets, double T0, double T)
 		}
 		PCB_END_LOOP;
 
-		rt_s = pcb_r_create_tree((const pcb_box_t **) seboxes.array, vtp0_len(&seboxes), 1);
-		rt_c = pcb_r_create_tree((const pcb_box_t **) ceboxes.array, vtp0_len(&ceboxes), 1);
+		rt_s = pcb_r_create_tree();
+		pcb_r_insert_array(rt_s, (const pcb_box_t **) seboxes.array, vtp0_len(&seboxes));
+		rt_c = pcb_r_create_tree();
+		pcb_r_insert_array(rt_c, (const pcb_box_t **) ceboxes.array, vtp0_len(&ceboxes));
 		vtp0_uninit(&seboxes);
 		vtp0_uninit(&ceboxes);
 		/* now, for each element, find its neighbor on all four sides */
@@ -666,6 +683,8 @@ static double ComputeCost(pcb_netlist_t *Nets, double T0, double T)
 			PCB_END_LOOP;
 		}
 		/* free k-d tree memory */
+		pcb_r_free_tree_data(rt_s, free);
+		pcb_r_free_tree_data(rt_c, free);
 		pcb_r_destroy_tree(&rt_s);
 		pcb_r_destroy_tree(&rt_c);
 	}
@@ -794,7 +813,7 @@ PerturbationType createPerturbation(vtp0_t *selected, double T)
 	return pt;
 }
 
-void doPerturb(PerturbationType * pt, pcb_bool undo)
+void doPerturb(vtp0_t *selected, PerturbationType *pt, pcb_bool undo)
 {
 	pcb_box_t *bb;
 	pcb_coord_t bbcx, bbcy;
@@ -839,14 +858,20 @@ void doPerturb(PerturbationType * pt, pcb_bool undo)
 			}
 			else {
 				pcb_coord_t y = bb->Y1;
-
 				if (pt->comp->type == PCB_OBJ_ELEMENT) {
 					pcb_element_mirror(PCB->Data, elem, 0);
 					/* mirroring moves the element.  move it back. */
 					pcb_element_move(PCB->Data, elem, 0, y - elem->VBox.Y1);
 				}
 				else {
-					pcb_subc_change_side(&subc, 0);
+					pcb_cardinal_t n;
+					pcb_coord_t y = bb->Y1;
+					pcb_subc_change_side(&subc, (bb->Y1+bb->Y2)/2);
+					/* mirroring moves the subc.  move it back. */
+					pcb_subc_move(subc, 0, y - subc->BoundingBox.Y1, 1);
+					for(n = 0; n < vtp0_len(selected); n++)
+						if (selected->array[n] == pt->comp)
+							selected->array[n] = subc;
 					pt->comp = (pcb_any_obj_t *)subc;
 				}
 			}
@@ -869,16 +894,17 @@ void doPerturb(PerturbationType * pt, pcb_bool undo)
 				pcb_element_move(PCB->Data, (pcb_element_t *)pt->other, x1 - x2, y1 - y2);
 			else
 				pcb_subc_move((pcb_subc_t *)pt->other, x1 - x2, y1 - y2, 1);
-
 			/* then flip both elements if they are on opposite sides */
 			if (on_bottom(pt->comp) != on_bottom(pt->other)) {
 				PerturbationType mypt;
 				mypt.comp = pt->comp;
 				mypt.which = ROTATE;
 				mypt.rotate = 0;				/* flip */
-				doPerturb(&mypt, undo);
+				doPerturb(selected, &mypt, undo);
+				pt->comp = mypt.comp;
 				mypt.comp = pt->other;
-				doPerturb(&mypt, undo);
+				doPerturb(selected, &mypt, undo);
+				pt->other = mypt.comp;
 			}
 			/* done */
 			return;
@@ -896,7 +922,7 @@ pcb_bool AutoPlaceSelected(void)
 	pcb_netlist_t *Nets;
 	vtp0_t Selected;
 	PerturbationType pt;
-	double C0, T0;
+	double C00, C0, T0;
 	pcb_bool changed = pcb_false;
 
 	vtp0_init(&Selected);
@@ -925,12 +951,12 @@ pcb_bool AutoPlaceSelected(void)
 		const double Tx = PCB_MIL_TO_COORD(300), P = 0.95;
 		double Cs = 0.0;
 		int i;
-		C0 = ComputeCost(Nets, Tx, Tx);
+		C00 = C0 = ComputeCost(Nets, Tx, Tx);
 		for (i = 0; i < TRIALS; i++) {
 			pt = createPerturbation(&Selected, PCB_INCH_TO_COORD(1));
-			doPerturb(&pt, pcb_false);
+			doPerturb(&Selected, &pt, pcb_false);
 			Cs += fabs(ComputeCost(Nets, Tx, Tx) - C0);
-			doPerturb(&pt, pcb_true);
+			doPerturb(&Selected, &pt, pcb_true);
 		}
 		T0 = -(Cs / TRIALS) / log(P);
 		printf("Initial T: %f\n", T0);
@@ -947,7 +973,7 @@ pcb_bool AutoPlaceSelected(void)
 		while (1) {
 			double Cprime;
 			pt = createPerturbation(&Selected, T);
-			doPerturb(&pt, pcb_false);
+			doPerturb(&Selected, &pt, pcb_false);
 			Cprime = ComputeCost(Nets, T0, T);
 			if (Cprime < C0) {				/* good move! */
 				C0 = Cprime;
@@ -960,11 +986,16 @@ pcb_bool AutoPlaceSelected(void)
 				steps++;
 			}
 			else
-				doPerturb(&pt, pcb_true);		/* undo last change */
+				doPerturb(&Selected, &pt, pcb_true);		/* undo last change */
 			moves++;
 			/* are we at the end of a stage? */
 			if (good_moves >= good_move_cutoff || moves >= move_cutoff) {
 				printf("END OF STAGE: COST %.0f\t" "GOOD_MOVES %d\tMOVES %d\t" "T: %.1f\n", C0, good_moves, moves, T);
+				pcb_draw();
+				if ((pcb_gui != NULL) && (pcb_gui->progress != NULL)) {
+					if (pcb_gui->progress(C00-T, C00, "Optimizing the placement..."))
+						break;
+				}
 				/* is this the end? */
 				if (T < 5 || good_moves < moves / CostParameter.good_ratio)
 					break;
@@ -978,11 +1009,13 @@ pcb_bool AutoPlaceSelected(void)
 		changed = (steps > 0);
 	}
 done:
+	if ((pcb_gui != NULL) && (pcb_gui->progress != NULL))
+		pcb_gui->progress(0, 0, NULL);
 	if (changed) {
 		pcb_rats_destroy(pcb_false);
 		pcb_rat_add_all(pcb_false, NULL);
 		pcb_redraw();
 	}
 	vtp0_uninit(&Selected);
-	return (changed);
+	return changed;
 }
