@@ -1,7 +1,8 @@
 /*
  *                            COPYRIGHT
  *
- *  PCB, interactive printed circuit board design
+ *  pcb-rnd, interactive printed circuit board design
+ *  (this file is based on PCB, interactive printed circuit board design)
  *  Copyright (C) 1994,1995,1996 Thomas Nau
  *  Copyright (C) 1998,1999,2000,2001 harry eaton
  *
@@ -23,7 +24,13 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- *  Contact addresses for paper mail and Email:
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
+ *
+ *
+ *  Old contact info:
  *  harry eaton, 6697 Buttonhole Ct, Columbia, MD 21044 USA
  *  haceaton@aplcomm.jhuapl.edu
  *
@@ -36,6 +43,7 @@
 
 #include "config.h"
 
+#include <stdlib.h>
 #include <assert.h>
 #include <setjmp.h>
 
@@ -96,9 +104,9 @@ mtspace_t *mtspace_create(void)
 
 	/* create mtspace data structure */
 	mtspace = (mtspace_t *) malloc(sizeof(*mtspace));
-	mtspace->ftree = pcb_r_create_tree(NULL, 0, 0);
-	mtspace->etree = pcb_r_create_tree(NULL, 0, 0);
-	mtspace->otree = pcb_r_create_tree(NULL, 0, 0);
+	mtspace->ftree = pcb_r_create_tree();
+	mtspace->etree = pcb_r_create_tree();
+	mtspace->otree = pcb_r_create_tree();
 	/* done! */
 	return mtspace;
 }
@@ -107,6 +115,9 @@ mtspace_t *mtspace_create(void)
 void mtspace_destroy(mtspace_t ** mtspacep)
 {
 	assert(mtspacep);
+	pcb_r_free_tree_data((*mtspacep)->ftree, free);
+	pcb_r_free_tree_data((*mtspacep)->etree, free);
+	pcb_r_free_tree_data((*mtspacep)->otree, free);
 	pcb_r_destroy_tree(&(*mtspacep)->ftree);
 	pcb_r_destroy_tree(&(*mtspacep)->etree);
 	pcb_r_destroy_tree(&(*mtspacep)->otree);
@@ -130,7 +141,7 @@ static pcb_r_dir_t mts_remove_one(const pcb_box_t * b, void *cl)
 	/* the info box is pre-bloated, so just check equality */
 	if (b->X1 == info->box.X1 && b->X2 == info->box.X2 &&
 			b->Y1 == info->box.Y1 && b->Y2 == info->box.Y2 && box->clearance == info->clearance) {
-		pcb_r_delete_entry(info->tree, b);
+		pcb_r_delete_entry_free_data(info->tree, b, free);
 		longjmp(info->env, 1);
 	}
 	return PCB_R_DIR_NOT_FOUND;
@@ -152,7 +163,7 @@ pcb_rtree_t *which_tree(mtspace_t * mtspace, mtspace_type_t which)
 void mtspace_add(mtspace_t * mtspace, const pcb_box_t * box, mtspace_type_t which, pcb_coord_t clearance)
 {
 	mtspacebox_t *filler = mtspace_create_box(box, clearance);
-	pcb_r_insert_entry(which_tree(mtspace, which), (const pcb_box_t *) filler, 1);
+	pcb_r_insert_entry(which_tree(mtspace, which), (const pcb_box_t *) filler);
 }
 
 /* remove a space-filler from the empty space representation. */
@@ -206,7 +217,17 @@ static pcb_r_dir_t query_one(const pcb_box_t * box, void *cl)
 	struct query_closure *qc = (struct query_closure *) cl;
 	mtspacebox_t *mtsb = (mtspacebox_t *) box;
 	pcb_coord_t shrink;
-	assert(pcb_box_intersect(qc->cbox, &mtsb->box));
+
+#ifndef NDEBUG
+	{
+		/* work around rounding errors: grow both boxes by 2 nm */
+		pcb_box_t b1 = *qc->cbox, b2 = mtsb->box;
+		b1.X1--;b1.Y1--;b1.X2++;b1.Y2++;
+		b2.X1--;b2.Y1--;b2.X2++;b2.Y2++;
+		assert(pcb_box_intersect(&b1, &b2));
+	}
+#endif
+
 	/* we need to satisfy the larger of the two clearances */
 	if (qc->clearance > mtsb->clearance)
 		shrink = mtsb->clearance;
