@@ -21,11 +21,16 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  */
 
 #include "config.h"
 #include "board.h"
 #include "data.h"
+#include "data_it.h"
 #include "flag_str.h"
 #include "layer.h"
 #include "diag_conf.h"
@@ -147,7 +152,7 @@ static int pcb_act_EvalConf(int argc, const char **argv, pcb_coord_t x, pcb_coor
 }
 
 static const char dump_layers_syntax[] =
-	"dumplayers()\n"
+	"dumplayers([all])\n"
 	;
 
 static const char dump_layers_help[] = "Print info about each layer";
@@ -159,6 +164,43 @@ static int pcb_act_DumpLayers(int argc, const char **argv, pcb_coord_t x, pcb_co
 	pcb_layer_id_t arr[128]; /* WARNING: this assumes we won't have more than 128 layers */
 	pcb_layergrp_id_t garr[128]; /* WARNING: this assumes we won't have more than 128 layers */
 
+	if ((argc > 0) && (strcmp(argv[0], "all") == 0)) {
+		printf("Per group:\n");
+		for(g = 0; g < PCB->LayerGroups.len; g++) {
+			pcb_layergrp_t *grp = &PCB->LayerGroups.grp[g];
+			printf(" Group %d: '%s' %x\n", g, grp->name, grp->type);
+			for(n = 0; n < grp->len; n++) {
+				pcb_layer_t *ly = pcb_get_layer(PCB->Data, grp->lid[n]);
+				if (ly != NULL) {
+					printf("  layer %d: '%s'\n", n, ly->name);
+					if (ly->meta.real.grp != g)
+						printf("   ERROR: invalid back-link to group: %ld should be %d\n", ly->meta.real.grp, g);
+				}
+				else
+					printf("  layer %d: <invalid>\n", g);
+			}
+		}
+
+		printf("Per layer:\n");
+		for(n = 0; n < PCB->Data->LayerN; n++) {
+			pcb_layer_t *ly = &PCB->Data->Layer[n];
+			printf(" layer %d: '%s'\n", n, ly->name);
+			if (ly->meta.real.grp >= 0) {
+				pcb_layergrp_t *grp = &PCB->LayerGroups.grp[ly->meta.real.grp];
+				int i, ok = 0;
+				for(i = 0; i < grp->len; i++) {
+					if (grp->lid[i] == n) {
+						ok = 1;
+						break;
+					}
+				}
+				if (!ok)
+					printf("   ERROR: invalid back-link to group: %ld\n", ly->meta.real.grp);
+			}
+		}
+
+		return 0;
+	}
 
 	printf("Max: theoretical=%d current_board=%d\n", PCB_MAX_LAYER+2, pcb_max_layer);
 	used = pcb_layer_list_any(PCB, PCB_LYT_ANYTHING | PCB_LYT_ANYWHERE | PCB_LYT_VIRTUAL, arr, sizeof(arr)/sizeof(arr[0]));
@@ -238,6 +280,62 @@ static int pcb_act_DumpUndo(int argc, const char **argv, pcb_coord_t x, pcb_coor
 }
 #endif
 
+typedef enum { /* bitfield */
+	DD_DRC = 1,
+	DD_COPPER_ONLY = 2
+} dd_flags;
+
+static void dump_data(pcb_data_t *data, dd_flags what, int ind, const char *parent)
+{
+	pcb_any_obj_t *o;
+	pcb_data_it_t it;
+	ind++;
+	for(o = pcb_data_first(&it, data, PCB_OBJ_CLASS_REAL); o != NULL; o = pcb_data_next(&it)) {
+		const char *type = pcb_obj_type_name(o->type);
+		pcb_coord_t cx, cy;
+
+		if ((what & DD_COPPER_ONLY) && (o->type == PCB_OBJ_SUBC))
+			goto skip;
+
+		if ((what & DD_COPPER_ONLY) && (o->parent_type == PCB_PARENT_LAYER)) {
+			pcb_layer_type_t lyt = pcb_layer_flags_(o->parent.layer);
+			if (!(lyt & PCB_LYT_COPPER))
+				goto skip;
+		}
+
+		cx = (o->BoundingBox.X1+o->BoundingBox.X2)/2;
+		cy = (o->BoundingBox.Y1+o->BoundingBox.Y2)/2;
+		printf("%*s %s", ind, "", type);
+		pcb_printf(" #%ld %mm;%mm ", o->ID, cx, cy);
+
+		if (parent != NULL)
+			printf("%s", parent);
+		printf("-");
+		if (o->term != NULL)
+			printf("%s", o->term);
+
+		if (what & DD_DRC)
+			printf(" DRC=%c%c", PCB_FLAG_TEST(PCB_FLAG_FOUND, o) ? 'f':'.', PCB_FLAG_TEST(PCB_FLAG_SELECTED, o) ? 's':'.');
+
+		printf("\n");
+		
+		skip:;
+		if (o->type == PCB_OBJ_SUBC)
+			dump_data(((pcb_subc_t *)o)->data, what, ind, ((pcb_subc_t *)o)->refdes);
+	}
+}
+
+static const char dump_data_syntax[] = "dumpdata()\n";
+static const char dump_data_help[] = "Dump an aspect of the data";
+static int pcb_act_DumpData(int argc, const char **argv, pcb_coord_t x, pcb_coord_t y)
+{
+	dd_flags what = DD_DRC | DD_COPPER_ONLY;
+	printf("DumpData:\n");
+	dump_data(PCB->Data, what, 0, NULL);
+	printf("\n");
+	return 0;
+}
+
 static const char integrity_syntax[] = "integrity()\n";
 static const char integrity_help[] = "perform integrirty check on the current board and generate errors if needed";
 static int pcb_act_integrity(int argc, const char **argv, pcb_coord_t x, pcb_coord_t y)
@@ -331,6 +429,8 @@ pcb_hid_action_t diag_action_list[] = {
 	 dump_layers_help, dump_layers_syntax},
 	{"dumpfonts", 0, pcb_act_DumpFonts,
 	 dump_fonts_help, dump_fonts_syntax},
+	{"dumpdata", 0, pcb_act_DumpData,
+	 dump_data_help, dump_data_syntax},
 #ifndef NDEBUG
 	{"dumpundo", 0, pcb_act_DumpUndo,
 	 dump_undo_help, dump_undo_syntax},
