@@ -18,10 +18,16 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  */
 
 #include "obj_pstk.h"
+#include "obj_pstk_op.h"
 #include "obj_pstk_inlines.h"
+#include "operation.h"
 
 typedef struct pse_proto_layer_s {
 	const char *name;
@@ -56,6 +62,7 @@ typedef struct pse_s {
 	int proto_shape[pse_num_layers];
 	int proto_info[pse_num_layers];
 	int proto_change[pse_num_layers];
+	pcb_coord_t proto_clr[pse_num_layers];
 	int hole_header;
 	int hdia, hplated;
 	int htop_val, htop_text, htop_layer;
@@ -135,14 +142,17 @@ static void pse_ps2dlg(void *hid_ctx, pse_t *pse)
 			switch(shape->shape) {
 				case PCB_PSSH_CIRC:
 					PCB_DAD_SET_VALUE(hid_ctx, pse->proto_shape[n], str_value, "circle");
-					pcb_snprintf(tmp, sizeof(tmp), "dia=$%mm at $%mm;$%mm", shape->data.circ.dia, shape->data.circ.x, shape->data.circ.y);
+					if ((shape->data.circ.x != 0) || (shape->data.circ.y != 0))
+						pcb_snprintf(tmp, sizeof(tmp), "dia=%.06$mm\nat %.06$mm;%.06$mm", shape->data.circ.dia, shape->data.circ.x, shape->data.circ.y);
+					else
+						pcb_snprintf(tmp, sizeof(tmp), "dia=%.06$mm", shape->data.circ.dia);
 					break;
 				case PCB_PSSH_LINE:
 					if (shape->data.line.square)
 						PCB_DAD_SET_VALUE(hid_ctx, pse->proto_shape[n], str_value, "square line");
 					else
 						PCB_DAD_SET_VALUE(hid_ctx, pse->proto_shape[n], str_value, "round line");
-					pcb_snprintf(tmp, sizeof(tmp), "thickness=%mm", shape->data.line.thickness);
+					pcb_snprintf(tmp, sizeof(tmp), "thickness=%.06$mm", shape->data.line.thickness);
 					break;
 				case PCB_PSSH_POLY:
 					PCB_DAD_SET_VALUE(hid_ctx, pse->proto_shape[n], str_value, "polygon");
@@ -153,10 +163,12 @@ static void pse_ps2dlg(void *hid_ctx, pse_t *pse)
 					strcpy(tmp, "<unknown>");
 			}
 			PCB_DAD_SET_VALUE(hid_ctx, pse->proto_info[n], str_value, tmp);
+			PCB_DAD_SET_VALUE(hid_ctx, pse->proto_clr[n], coord_value, shape->clearance);
 		}
 		else {
 			PCB_DAD_SET_VALUE(hid_ctx, pse->proto_shape[n], str_value, "");
 			PCB_DAD_SET_VALUE(hid_ctx, pse->proto_info[n], str_value, "");
+			PCB_DAD_SET_VALUE(hid_ctx, pse->proto_clr[n], coord_value, 0);
 		}
 	}
 
@@ -251,6 +263,52 @@ static void pse_chg_hole(void *hid_ctx, void *caller_data, pcb_hid_attribute_t *
 	pcb_gui->invalidate_all();
 }
 
+static void pse_chg_proto_clr(void *hid_ctx, void *caller_data, pcb_hid_attribute_t *attr)
+{
+	pse_t *pse = caller_data;
+	pcb_pstk_proto_t *proto = pcb_pstk_get_proto(pse->ps);
+	static int lock = 0;
+
+	if (lock != 0)
+		return;
+
+	if (proto != NULL) {
+		int n, idx = -1, sidx;
+		pcb_opctx_t ctx;
+
+		for(n = 0; n < pse_num_layers; n++)
+			if (pse->proto_clr[n] == (attr - pse->attrs))
+				idx = n;
+		if (idx < 0) {
+			pcb_message(PCB_MSG_ERROR, "Can't find shape - clearance unchanged (a)\n");
+			return;
+		}
+
+		sidx = pcb_pstk_get_shape_idx(&proto->tr.array[0], pse_layer[idx].mask, pse_layer[idx].comb);
+		if (sidx < 0) {
+			pcb_message(PCB_MSG_ERROR, "Can't find shape - clearance unchanged (b)\n");
+			return;
+		}
+
+		ctx.clip.clear = 0;
+		ctx.clip.restore = 1;
+		pcb_pstkop_clip(&ctx, pse->ps);
+
+		for(n = 0; n < proto->tr.used; n++)
+			pcb_pstk_shape_clr_grow(&proto->tr.array[n].shape[sidx], pcb_true, pse->attrs[pse->proto_clr[idx]].default_val.coord_value);
+
+		ctx.clip.clear = 1;
+		ctx.clip.restore = 0;
+		pcb_pstkop_clip(&ctx, pse->ps);
+	}
+
+	lock++;
+	pse_ps2dlg(hid_ctx, pse); /* to get calculated text fields updated */
+	lock--;
+
+	pcb_gui->invalidate_all();
+}
+
 static void pse_shape_del(void *hid_ctx, void *caller_data, pcb_hid_attribute_t *attr)
 {
 	pse_t *pse = caller_data;
@@ -473,6 +531,7 @@ static int pcb_act_PadstackEdit(int argc, const char **argv, pcb_coord_t x, pcb_
 			pse.tab_instance = PCB_DAD_CURRENT(dlg);
 			PCB_DAD_BEGIN_VBOX(dlg);
 				PCB_DAD_COMPFLAG(dlg, PCB_HATF_FRAME);
+				PCB_DAD_LABEL(dlg, "Setting that affect only this padstack instance");
 				PCB_DAD_BEGIN_HBOX(dlg);
 					PCB_DAD_LABEL(dlg, "prototype");
 					PCB_DAD_BUTTON(dlg, "#5");
@@ -509,9 +568,10 @@ static int pcb_act_PadstackEdit(int argc, const char **argv, pcb_coord_t x, pcb_
 		PCB_DAD_BEGIN_VBOX(dlg);
 			pse.tab_prototype = PCB_DAD_CURRENT(dlg);
 			PCB_DAD_BEGIN_VBOX(dlg);
+				PCB_DAD_LABEL(dlg, "Setting that affect all padstacks with the same prototype");
 				PCB_DAD_COMPFLAG(dlg, PCB_HATF_FRAME);
 				PCB_DAD_LABEL(dlg, "Pad geometry per layer type:");
-				PCB_DAD_BEGIN_TABLE(dlg, 4);
+				PCB_DAD_BEGIN_TABLE(dlg, 5);
 					PCB_DAD_COMPFLAG(dlg, PCB_HATF_FRAME);
 					for(n = 0; n < pse_num_layers; n++) {
 						PCB_DAD_LABEL(dlg, pse_layer[n].name);
@@ -523,6 +583,12 @@ static int pcb_act_PadstackEdit(int argc, const char **argv, pcb_coord_t x, pcb_
 							pse.proto_change[n] = PCB_DAD_CURRENT(dlg);
 							PCB_DAD_CHANGE_CB(dlg, pse_chg_shape);
 							PCB_DAD_HELP(dlg, "Change the shape on this layer type");
+						PCB_DAD_COORD(dlg, "");
+							pse.proto_clr[n] = PCB_DAD_CURRENT(dlg);
+							PCB_DAD_MINVAL(dlg, 1);
+							PCB_DAD_MAXVAL(dlg, PCB_MM_TO_COORD(1000));
+							PCB_DAD_CHANGE_CB(dlg, pse_chg_proto_clr);
+							PCB_DAD_HELP(dlg, "local, per layer type clearance\n(only when global padstack clearance is 0)");
 					}
 				PCB_DAD_END(dlg);
 			
