@@ -1,10 +1,12 @@
 /*
  *                            COPYRIGHT
  *
- *  PCB, interactive printed circuit board design
+ *  pcb-rnd, interactive printed circuit board design
+ *  (this file is based on PCB, interactive printed circuit board design)
  *
  *  Specctra .dsn export HID
  *  Copyright (C) 2008, 2011 Josh Jordan, Dan McMahill, and Jared Casper
+ *  Copyright (C) 2018 Tibor 'Igor2' Palinkas
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -20,6 +22,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  */
 
 /*
@@ -63,6 +69,7 @@ By Josh Jordan and Dan McMahill, modified from bom.c
 #include "plugins.h"
 #include "obj_line.h"
 #include "obj_pinvia.h"
+#include "obj_pstk_inlines.h"
 
 static const char *dsn_cookie = "dsn exporter";
 
@@ -284,19 +291,21 @@ static void print_placement(FILE * fp)
 		char *ename;
 		pcb_coord_t ox, oy;
 		char *side;
+		int res;
 		pcb_layer_type_t lyt = PCB_LYT_TOP;
 
 		if (subc->aux_layer != NULL)
 			lyt = pcb_layer_flags_(subc->aux_layer);
 		side = (lyt & PCB_LYT_BOTTOM) ? "back" : "front";
 
-		assert(pcb_subc_get_origin(subc, &ox, &oy) == 0);
+		res = pcb_subc_get_origin(subc, &ox, &oy);
+		assert(res == 0);
 		if (subc->refdes != NULL)
 			ename = pcb_strdup(subc->refdes);
 		else
 			ename = pcb_strdup("null");
 		pcb_fprintf(fp, "    (component %d\n", subc->ID);
-		pcb_fprintf(fp, "      (place \"%s\" %.6mm %.6mm %s 0 (PN 0))\n", ename, ox, oy, side);
+		pcb_fprintf(fp, "      (place \"%s\" %.6mm %.6mm %s 0 (PN 0))\n", ename, ox, PCB->MaxHeight - oy, side);
 		pcb_fprintf(fp, "    )\n");
 		g_free(ename);
 	}
@@ -385,15 +394,105 @@ static void print_pad(FILE *fp, GList **pads, pcb_pad_t *pad, pcb_point_t centro
 	add_padstack(pads, padstack);
 }
 
-static void print_term_poly(FILE *fp, gds_t *term_shapes, pcb_poly_t *poly, pcb_coord_t ox, pcb_coord_t oy, int term_side)
+static void print_polyshape(gds_t *term_shapes, pcb_pstk_poly_t *ply, pcb_coord_t ox, pcb_coord_t oy, const char *layer_name, int partsidesign)
+{
+	char tmp[512];
+	int fld;
+	pcb_coord_t x, y;
+	int n;
+
+	pcb_snprintf(tmp, sizeof(tmp), "        (polygon \"%s\" 0", layer_name);
+	gds_append_str(term_shapes, tmp);
+
+	fld = 0;
+	for(n = 0; n < ply->len; n++) {
+		if ((fld % 3) == 0)
+			gds_append_str(term_shapes, "\n       ");
+		pcb_snprintf(tmp, sizeof(tmp), " %.6mm %.6mm", (ply->x[n] - ox), -(ply->y[n] - oy));
+		gds_append_str(term_shapes, tmp);
+		fld++;
+	}
+
+	gds_append_str(term_shapes, "\n        )\n");
+}
+
+static void print_lineshape(gds_t *term_shapes, pcb_pstk_line_t *lin, pcb_coord_t ox, pcb_coord_t oy, const char *layer_name, int partsidesign)
+{
+	char tmp[512];
+	int fld;
+	pcb_coord_t x[4], y[4];
+	int n;
+	pcb_line_t ltmp;
+
+	pcb_snprintf(tmp, sizeof(tmp), "        (polygon \"%s\" 0", layer_name);
+	gds_append_str(term_shapes, tmp);
+
+	memset(&ltmp, 0, sizeof(ltmp));
+	ltmp.Point1.X = lin->x1;
+	ltmp.Point1.Y = lin->y1;
+	ltmp.Point2.X = lin->x2;
+	ltmp.Point2.Y = lin->y2;
+	ltmp.Thickness = lin->thickness;
+	pcb_sqline_to_rect(&ltmp, x, y);
+
+#warning padstack TODO: this ignores round cap
+
+	fld = 0;
+	for(n = 0; n < 4; n++) {
+		if ((fld % 3) == 0)
+			gds_append_str(term_shapes, "\n       ");
+		pcb_snprintf(tmp, sizeof(tmp), " %.6mm %.6mm", (x[n] - ox), -(y[n] - oy));
+		gds_append_str(term_shapes, tmp);
+		fld++;
+	}
+
+	gds_append_str(term_shapes, "\n        )\n");
+}
+
+static void print_circshape(gds_t *term_shapes, pcb_pstk_circ_t *circ, pcb_coord_t ox, pcb_coord_t oy, const char *layer_name, int partsidesign)
+{
+	char tmp[512];
+
+	pcb_snprintf(tmp, sizeof(tmp), "        (circle \"%s\"", layer_name);
+	gds_append_str(term_shapes, tmp);
+
+#warning padstack TODO: this ignores circle center offset
+
+	pcb_snprintf(tmp, sizeof(tmp), " %.6mm)\n", circ->dia);
+	gds_append_str(term_shapes, tmp);
+}
+
+static void print_polyline(gds_t *term_shapes, pcb_poly_it_t *it, pcb_pline_t *pl, pcb_coord_t ox, pcb_coord_t oy, const char *layer_name, int partsidesign)
+{
+	char tmp[512];
+	int fld;
+	pcb_coord_t x, y;
+	int go;
+
+	if (pl != NULL) {
+		pcb_snprintf(tmp, sizeof(tmp), "(polygon \"%s\" 0", layer_name);
+		gds_append_str(term_shapes, tmp);
+
+		fld = 0;
+		for(go = pcb_poly_vect_first(it, &x, &y); go; go = pcb_poly_vect_next(it, &x, &y)) {
+			if ((fld % 3) == 0)
+				gds_append_str(term_shapes, "\n       ");
+			pcb_snprintf(tmp, sizeof(tmp), " %.6mm %.6mm", (x - ox) * partsidesign, y - oy);
+			gds_append_str(term_shapes, tmp);
+			fld++;
+		}
+
+		gds_append_str(term_shapes, "\n        )\n");
+	}
+}
+
+static void print_term_poly(FILE *fp, gds_t *term_shapes, pcb_poly_t *poly, pcb_coord_t ox, pcb_coord_t oy, int term_side, int partsidesign)
 {
 	if (poly->term != NULL) {
 		pcb_layer_t *lay = g_list_nth_data(layerlist, term_side);
 		char *padstack = pcb_strdup_printf("Term_poly_%ld", poly->ID);
-		char tmp[512];
 		pcb_poly_it_t it;
 		pcb_polyarea_t *pa;
-		int fld;
 
 		pcb_fprintf(fp, "      (pin %s \"%s\" %.6mm %.6mm)\n", padstack, poly->term, ox, oy);
 
@@ -401,33 +500,48 @@ static void print_term_poly(FILE *fp, gds_t *term_shapes, pcb_poly_t *poly, pcb_
 		gds_append_str(term_shapes, padstack);
 		gds_append_str(term_shapes, "\n");
 
+		gds_append_str(term_shapes, "      (shape ");
+
 		for(pa = pcb_poly_island_first(poly, &it); pa != NULL; pa = pcb_poly_island_next(&it)) {
-			pcb_coord_t x, y;
 			pcb_pline_t *pl;
-			int go;
 
 			pl = pcb_poly_contour(&it);
-			if (pl != NULL) {
-				pcb_snprintf(tmp, sizeof(tmp), "      (shape (polygon \"%s\" 0", lay->name);
-				gds_append_str(term_shapes, tmp);
 
-				fld = 0;
-				for(go = pcb_poly_vect_first(&it, &x, &y); go; go = pcb_poly_vect_next(&it, &x, &y)) {
-					if ((fld % 3) == 0)
-						gds_append_str(term_shapes, "\n       ");
-					pcb_snprintf(tmp, sizeof(tmp), " %.6mm %.6mm", x - ox, y - oy);
-					gds_append_str(term_shapes, tmp);
-					fld++;
-				}
-
-				gds_append_str(term_shapes, "\n      ))\n");
-				gds_append_str(term_shapes, "      (attach off)\n");
-			}
+			print_polyline(term_shapes, &it, pl, ox, oy, lay->name, partsidesign);
 		}
+
+		gds_append_str(term_shapes, "      )\n");
+		gds_append_str(term_shapes, "      (attach off)\n");
 		gds_append_str(term_shapes, "    )\n");
 	}
 }
 
+void print_pstk_shape(gds_t *term_shapes, pcb_pstk_t *padstack, int lid, pcb_coord_t ox, pcb_coord_t oy, int partsidesign)
+{
+	pcb_pstk_shape_t *shp;
+	pcb_layer_t *lay = g_list_nth_data(layerlist, lid);
+	pcb_layer_type_t lyt = pcb_layer_flags_(lay);
+		pcb_poly_it_t it;
+
+	if (!(lyt & PCB_LYT_COPPER))
+		return;
+
+	shp = pcb_pstk_shape(padstack, lyt, 0);
+	if (shp == NULL)
+		return;
+
+	switch(shp->shape) {
+		case PCB_PSSH_POLY:
+			print_polyshape(term_shapes, &shp->data.poly, ox, oy, lay->name, partsidesign);
+			break;
+		case PCB_PSSH_LINE:
+			print_lineshape(term_shapes, &shp->data.line, ox, oy, lay->name, partsidesign);
+			break;
+		case PCB_PSSH_CIRC:
+			print_circshape(term_shapes, &shp->data.circ, ox, oy, lay->name, partsidesign);
+			break;
+	}
+}
 
 static void print_library(FILE * fp)
 {
@@ -480,11 +594,30 @@ static void print_library(FILE * fp)
 			pcb_layer_type_t lyt = pcb_layer_flags_(layer);
 			if ((lyt & PCB_LYT_COPPER) && ((lyt & PCB_LYT_TOP) || (lyt & PCB_LYT_BOTTOM))) {
 				int termside = (lyt & PCB_LYT_BOTTOM) ? g_list_length(layerlist) - 1 : 0;
-				print_term_poly(fp, &term_shapes, polygon, ox, oy, termside);
+				print_term_poly(fp, &term_shapes, polygon, ox, oy, termside, partsidesign);
 			}
 		}
 		PCB_ENDALL_LOOP;
 
+		PCB_PADSTACK_LOOP(subc->data);
+		{
+			int n;
+			char *pid = pcb_strdup_printf("Pstk_shape_%ld", padstack->ID);
+			pcb_fprintf(fp, "      (pin %s \"%s\" %.6mm %.6mm)\n", pid, padstack->term, -(padstack->x-ox), padstack->y-oy);
+
+			gds_append_str(&term_shapes, "    (padstack ");
+			gds_append_str(&term_shapes, pid);
+			gds_append_str(&term_shapes, "\n");
+
+			gds_append_str(&term_shapes, "      (shape\n");
+			for(n = 0; n < g_list_length(layerlist); n++)
+				print_pstk_shape(&term_shapes, padstack, n, 0, 0, partsidesign);
+			gds_append_str(&term_shapes, "      )\n");
+			gds_append_str(&term_shapes, "      (attach off)\n");
+			gds_append_str(&term_shapes, "    )\n");
+		}
+		PCB_END_LOOP;
+
 		PCB_VIA_LOOP(subc->data);
 		{
 			print_pin(fp, &pads, via, centroid, partsidesign, partside);
@@ -674,7 +807,7 @@ static int PrintSPECCTRA(void)
 
 	fclose(fp);
 
-	return (0);
+	return 0;
 }
 
 
