@@ -1,24 +1,29 @@
- /*
-  *                            COPYRIGHT
-  *
-  *  PCB, interactive printed circuit board design
-  *  Copyright (C) 2017 Tibor 'Igor2' Palinkas
-  *
-  *  This program is free software; you can redistribute it and/or modify
-  *  it under the terms of the GNU General Public License as published by
-  *  the Free Software Foundation; either version 2 of the License, or
-  *  (at your option) any later version.
-  *
-  *  This program is distributed in the hope that it will be useful,
-  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  *  GNU General Public License for more details.
-  *
-  *  You should have received a copy of the GNU General Public License
-  *  along with this program; if not, write to the Free Software
-  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
-  *
-  */
+/*
+ *                            COPYRIGHT
+ *
+ *  pcb-rnd, interactive printed circuit board design
+ *  (this file is based on PCB, interactive printed circuit board design)
+ *  Copyright (C) 2017 Tibor 'Igor2' Palinkas
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ *
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
+ */
 
 /*
  *  Based on the png exporter by Dan McMahill
@@ -56,6 +61,7 @@
 #include "hid_helper.h"
 #include "hid_flags.h"
 
+#include "../src_plugins/lib_compat_help/pstk_compat.h"
 
 static pcb_hid_t fidocadj_hid;
 
@@ -131,7 +137,7 @@ static long int crd(pcb_coord_t c)
 	return pcb_round((double)PCB_COORD_TO_MIL(c) / 5.0);
 }
 
-static int layer_map(unsigned int lflg, int *fidoly_next, int *warned, const char *lyname)
+static int layer_map(unsigned int lflg, int *fidoly_next, const char *lyname)
 {
 	if (lflg & PCB_LYT_COPPER) {
 		if (lflg & PCB_LYT_BOTTOM)
@@ -144,9 +150,9 @@ static int layer_map(unsigned int lflg, int *fidoly_next, int *warned, const cha
 
 	(*fidoly_next)++;
 	if (*fidoly_next > 15) {
-		if (!*warned)
-			pcb_message(PCB_MSG_ERROR, "FidoCadJ can't handle this many layers - layer %s is not exported\n", lyname);
-		*warned = 1;
+		char *msg = pcb_strdup_printf("FidoCadJ can't handle this many layers - layer %s is not exported\n", lyname);
+		pcb_io_incompat_save(NULL, NULL, msg, NULL);
+		free(msg);
 		return -1;
 	}
 	return *fidoly_next;
@@ -154,7 +160,17 @@ static int layer_map(unsigned int lflg, int *fidoly_next, int *warned, const cha
 
 static void write_custom_element(FILE *f, pcb_element_t *e)
 {
-	pcb_message(PCB_MSG_ERROR, "Can't export custom footprint for %s yet\n", e->Name[PCB_ELEMNAME_IDX_REFDES].TextString);
+	char *msg = pcb_strdup_printf("Can't export custom footprint for %s yet\n", e->Name[PCB_ELEMNAME_IDX_REFDES].TextString);
+	pcb_io_incompat_save(e->parent.data, (pcb_any_obj_t *)e, msg, "element omitted - add the footprint type on the footprint list!");
+	free(msg);
+
+}
+
+static void write_custom_subc(FILE *f, pcb_subc_t *sc)
+{
+	char *msg = pcb_strdup_printf("Can't export custom footprint for %s yet\n", sc->refdes);
+	pcb_io_incompat_save(sc->parent.data, (pcb_any_obj_t *)sc, msg, "subcircuit omitted - add the footprint type on the footprint list!");
+	free(msg);
 }
 
 static void fidocadj_do_export(pcb_hid_attr_val_t * options)
@@ -163,7 +179,6 @@ static void fidocadj_do_export(pcb_hid_attr_val_t * options)
 	const char *filename, *libfile;
 	int n, fidoly_next, have_lib;
 	pcb_layer_id_t lid;
-	int layer_warned = 0, hole_warned = 0;
 	htsi_t lib_names; /* hash of names found in the library, if have_lib is 1 */
 
 	if (!options) {
@@ -199,7 +214,7 @@ static void fidocadj_do_export(pcb_hid_attr_val_t * options)
 	for(lid = 0; lid < pcb_max_layer; lid++) {
 		pcb_layer_t *ly = PCB->Data->Layer+lid;
 		unsigned int lflg = pcb_layer_flags(PCB, lid);
-		int fidoly = layer_map(lflg, &fidoly_next, &layer_warned, ly->name);
+		int fidoly = layer_map(lflg, &fidoly_next, ly->name);
 
 		if (fidoly < 0)
 			continue;
@@ -223,9 +238,7 @@ static void fidocadj_do_export(pcb_hid_attr_val_t * options)
 			pcb_pline_t *pl = polygon->Clipped->contours;
 
 			if (polygon->HoleIndexN > 0) {
-				if (!hole_warned)
-					pcb_message(PCB_MSG_ERROR, "FidoCadJ can't handle holes in polygons, ignoring holes for this export - some of the polygons will look different\n");
-				hole_warned = 1;
+				pcb_io_incompat_save(PCB->Data, (pcb_any_obj_t *)polygon, "FidoCadJ can't handle holes in polygons, ignoring holes for this export", "(some of the polygons will look different unless you remove the holes or split up the polygons)");
 			}
 
 			fprintf(f, "PP %ld %ld", crd(pl->head.point[0]), crd(pl->head.point[1]));
@@ -296,6 +309,29 @@ static void fidocadj_do_export(pcb_hid_attr_val_t * options)
 	}
 	PCB_END_LOOP;
 
+	PCB_PADSTACK_LOOP(PCB->Data) {
+		int oshape;
+		pcb_coord_t x, y, drill_dia, pad_dia, clearance, mask;
+		pcb_pstk_compshape_t cshape;
+		pcb_bool plated;
+		if (!pcb_pstk_export_compat_via(padstack, &x, &y, &drill_dia, &pad_dia, &clearance, &mask, &cshape, &plated)) {
+			pcb_io_incompat_save(PCB->Data, (pcb_any_obj_t *)padstack, "can't export non-uniform padstacl", "use a simpler padstack - omiting this one from the export");
+			continue;
+		}
+		switch(cshape) {
+			case PCB_PSTK_COMPAT_OCTAGON:
+				pcb_io_incompat_save(PCB->Data, (pcb_any_obj_t *)padstack, "can't export octagonal shape", "use round or square instead; (fallback to round for now)");
+				/* fall-through */
+			case PCB_PSTK_COMPAT_ROUND:  oshape = 0; break;
+			case PCB_PSTK_COMPAT_SQUARE: oshape = 1; break;  /* rectangle with sharp corners */
+			default:
+				pcb_io_incompat_save(PCB->Data, (pcb_any_obj_t *)padstack, "can't export shaped padstack", "use round or square instead; (fallback to round for now)");
+				oshape = 0;
+		}
+		fprintf(f, "pa %ld %ld %ld %ld %ld %d\n", crd(x), crd(y), crd(pad_dia), crd(pad_dia), crd(drill_dia), oshape);
+	}
+	PCB_END_LOOP;
+
 	PCB_ELEMENT_LOOP(PCB->Data) {
 		const char *fp = element->Name[PCB_ELEMNAME_IDX_DESCRIPTION].TextString;
 		if (have_lib && (htsi_get(&lib_names, fp)))
@@ -305,6 +341,28 @@ static void fidocadj_do_export(pcb_hid_attr_val_t * options)
 	}
 	PCB_END_LOOP;
 
+	PCB_SUBC_LOOP(PCB->Data) {
+		const char *fp = pcb_attribute_get(&subc->Attributes, "footprint");
+		if ((fp != NULL) && have_lib && (htsi_get(&lib_names, fp))) {
+			pcb_coord_t x, y;
+			double rot = 0.0;
+			int on_bottom = 0;
+			if (pcb_subc_get_origin(subc, &x, &y)) {
+				pcb_io_incompat_save(PCB->Data, (pcb_any_obj_t *)subc, "can't figure subcircuit origin", "omitting this part until the proper subc-aux layer objects are added");
+				continue;
+			}
+			pcb_subc_get_rotation(subc, &rot);
+			pcb_subc_get_side(subc, &on_bottom);
+
+#warning TODO: figure how to store rotation
+#warning TODO: figure how to store side
+			fprintf(f, "MC %ld %ld %d 0 %s\n", crd(x), crd(y), 0, fp);
+		}
+		else
+			write_custom_subc(f, subc);
+	}
+	PCB_END_LOOP;
+
 
 	fclose(f);
 
@@ -338,6 +396,7 @@ int pplg_check_ver_export_fidocadj(int ver_needed) { return 0; }
 
 void pplg_uninit_export_fidocadj(void)
 {
+	pcb_hid_remove_attributes_by_cookie(fidocadj_cookie);
 }
 
 int pplg_init_export_fidocadj(void)
