@@ -1,30 +1,35 @@
 /*
+ *                            COPYRIGHT
  *
- * IPC-D-356 Netlist export.
+ *  pcb-rnd, interactive printed circuit board design
+ *  (this file is based on PCB, interactive printed circuit board design)
  *
- * Copyright (C) 2012 Jerome Marchand (Jerome.Marchand@gmail.com)
+ *  IPC-D-356 Netlist export.
  *
- * PCB, interactive printed circuit board design
+ *  Copyright (C) 2012 Jerome Marchand (Jerome.Marchand@gmail.com)
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ *  pcb-rnd, interactive printed circuit board design
+ *  (this file is based on PCB, interactive printed circuit board design)
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
  *
- * Contact addresses for paper mail and Email:
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
- * Thomas Nau, Schlehenweg 15, 88471 Baustetten, Germany
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  *
- * Thomas.Nau@rz.uni-ulm.de
  */
 
 #include "config.h"
@@ -113,8 +118,7 @@ static pcb_hid_attribute_t *IPCD356_get_export_options(int *n)
 	return IPCD356_options;
 }
 
-/*!
- * \brief Writes the IPC-D-356 Header to the file provided.
+/* Writes the IPC-D-356 Header to the file provided.
  *
  * The JOB name is the PCB Name (if set), otherwise the filename
  * (including the path) is used.
@@ -148,23 +152,22 @@ void IPCD356_WriteHeader(FILE * fd)
 }
 
 
-/*!
- * \brief Writes a net to the file provided.
+/* Writes a net to the file provided.
  *
  * The net name is passed through the "net" and should be 14 characters
- * max.\n
+ * max.
  * The function scans through pads, pins and vias  and looks for the
- * \c PCB_FLAG_FOUND.\n
+ * PCB_FLAG_FOUND.
  * Once the object has been added to the net list the \c PCB_FLAG_VISIT is
  * set on that object.
  *
- * \todo 1) The bottom layer is always written as layer #2 (A02).\n
- *          It could output the actual layer number (example: A06 on a
- *          6 layer board).\n
- *          But I could not find an easy way to do this...
+ * TODO 1) The bottom layer is always written as layer #2 (A02).
+ *         It could output the actual layer number (example: A06 on a
+ *         6 layer board).
+ *         But I could not find an easy way to do this...
  *
- * \todo 2) Objects with mutiple connections could have the "M"
- *          (column 32) field written to indicate a Mid Net Point.
+ * TODO 2) Objects with mutiple connections could have the "M"
+ *         (column 32) field written to indicate a Mid Net Point.
  */
 void IPCD356_WriteNet(FILE * fd, char *net)
 {
@@ -376,11 +379,7 @@ void IPCD356_WriteNet(FILE * fd, char *net)
 }
 
 
-/*!
- * \brief The main IPC-D-356 function.
- *
- * Gets the filename for the netlist from the dialog.
- */
+/* The main IPC-D-356 function; gets the filename for the netlist from the dialog */
 int IPCD356_Netlist(void)
 {
 	FILE *fp;
@@ -391,7 +390,7 @@ int IPCD356_Netlist(void)
 
 	if (IPCD356_SanityCheck()) {	/* Check for invalid names + numbers. */
 		pcb_message(PCB_MSG_ERROR, "IPCD356: aborting on the sanity check.\n");
-		return (1);
+		return 1;
 	}
 
 	sprintf(net, "%s.ipc", PCB->Name);
@@ -565,10 +564,10 @@ int IPCD356_SanityCheck()
 	PCB_ELEMENT_LOOP(PCB->Data);
 	if (element->Name[1].TextString == '\0') {
 		pcb_message(PCB_MSG_ERROR, "Error: Found unnamed element. All elements need to be named to create an IPC-D-356 netlist.\n");
-		return (1);
+		return 1;
 	}
 	PCB_END_LOOP;											/* Element. */
-	return (0);
+	return 0;
 }
 
 static void IPCD356_do_export(pcb_hid_attr_val_t * options)
