@@ -1,26 +1,31 @@
- /*
-  *                            COPYRIGHT
-  *
-  *  pcb-rnd, interactive printed circuit board design
-  *  Copyright (C) 2017 Tibor 'Igor2' Palinkas
-  *
-  *  This program is free software; you can redistribute it and/or modify
-  *  it under the terms of the GNU General Public License as published by
-  *  the Free Software Foundation; either version 2 of the License, or
-  *  (at your option) any later version.
-  *
-  *  This program is distributed in the hope that it will be useful,
-  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  *  GNU General Public License for more details.
-  *
-  *  You should have received a copy of the GNU General Public License
-  *  along with this program; if not, write to the Free Software
-  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
-  *
-  */
+/*
+ *                            COPYRIGHT
+ *
+ *  pcb-rnd, interactive printed circuit board design
+ *  Copyright (C) 2017 Tibor 'Igor2' Palinkas
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ *
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
+ */
 
 #include "../lib_polyhelp/topoly.h"
+#include "plug_io.h"
 
 #define TRX_(x) (x)
 #define TRY_(y) (PCB->MaxHeight - (y))
@@ -110,6 +115,22 @@ static void scad_draw_drill(const pcb_pin_t *pin)
 	pcb_fprintf(f, "		cylinder(r=%mm, h=4, center=true, $fn=30);\n", pin->DrillingHole/2);
 }
 
+static void scad_draw_pstk(const pcb_pstk_t *ps)
+{
+	pcb_pstk_proto_t *proto = pcb_pstk_get_proto(ps);
+
+	if (proto == NULL) {
+		pcb_io_incompat_save(ps->parent.data, (pcb_any_obj_t *)ps, "failed to retrieve padstack prototype", "internal pcb-rnd error, please file a bugreport");
+		return;
+	}
+
+#warning padstack TODO: this ignores bbvias
+	if (proto->hdia > 0) {
+		pcb_fprintf(f, "	translate([%mm,%mm,0])\n", TRX_(ps->x), TRY_(ps->y));
+		pcb_fprintf(f, "		cylinder(r=%mm, h=4, center=true, $fn=30);\n", proto->hdia/2);
+	}
+}
+
 static void scad_draw_drills(void)
 {
 	pcb_rtree_it_t it;
@@ -121,6 +142,10 @@ static void scad_draw_drills(void)
 		scad_draw_drill((pcb_pin_t *)obj);
 	pcb_r_end(&it);
 
+	for(obj = pcb_r_first(PCB->Data->padstack_tree, &it); obj != NULL; obj = pcb_r_next(&it))
+		scad_draw_pstk((pcb_pstk_t *)obj);
+	pcb_r_end(&it);
+
 	PCB_PIN_ALL_LOOP(PCB->Data); {
 		scad_draw_drill(pin);
 	} PCB_ENDALL_LOOP;
