@@ -1,7 +1,8 @@
 /*
  *                            COPYRIGHT
  *
- *  PCB, interactive printed circuit board design
+ *  pcb-rnd, interactive printed circuit board design
+ *  (this file is based on PCB, interactive printed circuit board design)
  *  Copyright (C) 2006 DJ Delorie
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -18,7 +19,13 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- *  Contact addresses for paper mail and Email:
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
+ *
+ *
+ *  Old contact info:
  *  DJ Delorie, 334 North Road, Deerfield NH 03037-1110, USA
  *  dj@delorie.com
  *
@@ -76,6 +83,7 @@ static pcb_layer_t *make_layer(pcb_layergrp_id_t grp, const char *lname)
 	assert(grp >= 0);
 	lid = pcb_layer_create(PCB, grp, lname);
 	assert(lid >= 0);
+	PCB->Data->Layer[lid].meta.real.vis = 1;
 	return &PCB->Data->Layer[lid];
 }
 
