@@ -1,7 +1,8 @@
 /*
  *                            COPYRIGHT
  *
- *  PCB, interactive printed circuit board design
+ *  pcb-rnd, interactive printed circuit board design
+ *  (this file is based on PCB, interactive printed circuit board design)
  *  Copyright (C) 1994,1995,1996 Thomas Nau
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -18,9 +19,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- *  Contact addresses for paper mail and Email:
- *  Thomas Nau, Schlehenweg 15, 88471 Baustetten, Germany
- *  Thomas.Nau@rz.uni-ulm.de
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  *
  */
 #include "config.h"
@@ -168,6 +170,7 @@ static int fp_fs_list(pcb_fplibrary_t *pl, const char *subdir, int recurse,
 		 * may exist in a library tree to provide an html browsable
 		 * index of the library.
 		 */
+#warning fp TODO: make this a configurable list
 		l = strlen(subdirentry->d_name);
 		if (!stat(subdirentry->d_name, &buffer)
 				&& subdirentry->d_name[0] != '.'
@@ -365,6 +368,7 @@ static pcb_fptype_t pcb_fp_file_type(const char *fn, void ***tags)
 	int Talloced = 0, Tused = 0;
 	pcb_fptype_t ret = PCB_FP_INVALID;
 
+#warning fp TODO: rather call plug_io if it is not parametric
 	if (tags != NULL)
 		*tags = NULL;
 
@@ -389,6 +393,7 @@ static pcb_fptype_t pcb_fp_file_type(const char *fn, void ***tags)
 				state = ST_COMMENT;
 				break;
 			}
+#warning fp TODO: rather call plug_io if it is not parametric
 			else if ((first_element) && (c == 'l')) {
 				char s[23];
 				/* li:pcb-rnd-subcircuit */
@@ -409,6 +414,16 @@ static pcb_fptype_t pcb_fp_file_type(const char *fn, void ***tags)
 					break;
 				}
 			}
+			else if ((first_element) && (c == '(')) {
+				char s[8];
+				/* module */
+				fgets(s, 7, f);
+				s[6] = '\0';
+				if (strcmp(s, "module") == 0) {
+					state = ST_ELEMENT;
+					break;
+				}
+			}
 			first_element = 0;
 			/* fall-thru for detecting @ */
 		case ST_COMMENT:
@@ -467,14 +482,16 @@ out:;
 }
 
 #define F_IS_PARAMETRIC 0
+#define F_TMPNAME 1
 static FILE *fp_fs_fopen(pcb_plug_fp_t *ctx, const char *path, const char *name, pcb_fp_fopen_ctx_t *fctx)
 {
 	char *basename, *params, *fullname;
-	FILE *f = NULL;
+	FILE *fp, *f = NULL;
 	const char *libshell;
 
 	pcb_conf_cmd_is_safe(rc.library_shell, &libshell, 1);
 
+	fctx->field[F_TMPNAME].p = NULL;
 	fctx->field[F_IS_PARAMETRIC].i = pcb_fp_dupname(name, &basename, &params);
 	if (basename == NULL)
 		return NULL;
@@ -498,7 +515,17 @@ static FILE *fp_fs_fopen(pcb_plug_fp_t *ctx, const char *path, const char *name,
 			cmd = malloc(strlen(libshell) + strlen(fullname) + strlen(params) + 16);
 			sprintf(cmd, "%s%s%s %s", libshell, sep, fullname, params);
 /*fprintf(stderr, " cmd=%s\n",  cmd);*/
-			f = pcb_popen(cmd, "r");
+			fctx->field[F_TMPNAME].p = pcb_tempfile_name_new("pcb-rnd-pfp");
+			f = pcb_fopen((char *)fctx->field[F_TMPNAME].p, "w+");
+			if (f != NULL) {
+				char buff[4096];
+				int len;
+				fp = pcb_popen(cmd, "r");
+				while((len = fread(buff, 1, sizeof(buff), fp)) > 0)
+					fwrite(buff, 1, len, f);
+				pcb_pclose(fp);
+				rewind(f);
+			}
 			free(cmd);
 		}
 		else
@@ -512,10 +539,9 @@ static FILE *fp_fs_fopen(pcb_plug_fp_t *ctx, const char *path, const char *name,
 
 static void fp_fs_fclose(pcb_plug_fp_t *ctx, FILE * f, pcb_fp_fopen_ctx_t *fctx)
 {
-	if (fctx->field[F_IS_PARAMETRIC].i)
-		pcb_pclose(f);
-	else
-		fclose(f);
+	fclose(f);
+	if (fctx->field[F_TMPNAME].p != NULL)
+		pcb_tempfile_unlink((char *)fctx->field[F_TMPNAME].p);
 }
 
 
