@@ -17,13 +17,14 @@
 #include "../src_plugins/lib_gtk_hid/gui.h"
 #include "../src_plugins/lib_gtk_hid/coord_conv.h"
 #include "../src_plugins/lib_gtk_hid/render.h"
+#include "../src_plugins/lib_gtk_hid/preview_helper.h"
 
-#include "opengl.h"
+#include "../src_plugins/lib_hid_gl/opengl.h"
 #include <gtk/gtkgl.h>
-#include "draw_gl.h"
-#include "hidgl.h"
+#include "../src_plugins/lib_hid_gl/draw_gl.h"
+#include "../src_plugins/lib_hid_gl/hidgl.h"
 #include "hid_draw_helpers.h"
-#include "stencil_gl.h"
+#include "../src_plugins/lib_hid_gl/stencil_gl.h"
 
 #include "../src_plugins/lib_gtk_config/hid_gtk_conf.h"
 
@@ -202,6 +203,8 @@ int ghid_gl_set_layer_group(pcb_layergrp_id_t group, pcb_layer_id_t layer, unsig
 		case PCB_LYT_PDRILL:
 		case PCB_LYT_UDRILL:
 			return 1;
+		case PCB_LYT_UI:
+			return 1;
 		case PCB_LYT_RAT:
 			return PCB->RatOn;
 		case PCB_LYT_CSECT:
@@ -374,8 +377,14 @@ static void set_gl_color_for_gc(pcb_hid_gc_t gc)
 	ColorCache *cc;
 	double r, g, b, a;
 
-	if (priv->current_colorname != NULL &&
-			strcmp(priv->current_colorname, gc->colorname) == 0 && priv->current_alpha_mult == gc->alpha_mult)
+	if(gc->colorname == NULL) {
+		fprintf(stderr, "set_gl_color_for_gc:  gc->colorname = NULL, setting to magenta\n");
+		gc->colorname = "magenta";
+	}
+	
+	if (	priv->current_colorname != NULL &&
+				strcmp(priv->current_colorname, gc->colorname) == 0 && 
+				priv->current_alpha_mult == gc->alpha_mult)
 		return;
 
 	free(priv->current_colorname);
@@ -468,6 +477,12 @@ static void set_gl_color_for_gc(pcb_hid_gc_t gc)
 
 void ghid_gl_set_color(pcb_hid_gc_t gc, const char *name)
 {
+	
+	if(name == NULL) {
+		fprintf(stderr, "ghid_gl_set_color():  name = NULL, setting to magenta\n");
+		name = "magenta";
+	}
+
 	gc->colorname = name;
 	set_gl_color_for_gc(gc);
 }
@@ -973,7 +988,7 @@ void ghid_gl_port_drawing_realize_cb(GtkWidget * widget, gpointer data)
 	return;
 }
 
-gboolean ghid_gl_preview_expose(GtkWidget * widget, pcb_gtk_expose_t *ev, pcb_hid_expose_t expcall, const pcb_hid_expose_ctx_t *ctx)
+gboolean ghid_gl_preview_expose(GtkWidget * widget, pcb_gtk_expose_t *ev, pcb_hid_expose_t expcall, pcb_hid_expose_ctx_t *ctx)
 {
 /*	GdkWindow *window = gtk_widget_get_window(widget);*/
 	GdkGLContext *pGlContext = gtk_widget_get_gl_context(widget);
@@ -983,6 +998,8 @@ gboolean ghid_gl_preview_expose(GtkWidget * widget, pcb_gtk_expose_t *ev, pcb_hi
 	pcb_gtk_view_t save_view;
 	int save_width, save_height;
 	double xz, yz, vw, vh;
+	pcb_coord_t ox1 = ctx->view.X1, oy1 = ctx->view.Y1, ox2 = ctx->view.X2, oy2 = ctx->view.Y2;
+	pcb_coord_t nx1, ny1, nx2, ny2;
 
 	vw = ctx->view.X2 - ctx->view.X1;
 	vh = ctx->view.Y2 - ctx->view.Y1;
@@ -1009,6 +1026,8 @@ gboolean ghid_gl_preview_expose(GtkWidget * widget, pcb_gtk_expose_t *ev, pcb_hi
 	gport->view.x0 = (vw - gport->view.width) / 2 + ctx->view.X1;
 	gport->view.y0 = (vh - gport->view.height) / 2 + ctx->view.Y1;
 
+	PCB_GTK_PREVIEW_TUNE_EXTENT(ctx, allocation);
+
 	/* make GL-context "current" */
 	if (!gdk_gl_drawable_gl_begin(pGlDrawable, pGlContext)) {
 		return FALSE;
@@ -1066,6 +1085,8 @@ gboolean ghid_gl_preview_expose(GtkWidget * widget, pcb_gtk_expose_t *ev, pcb_hi
 	gport->render_priv->in_context = pcb_false;
 	gdk_gl_drawable_gl_end(pGlDrawable);
 
+	/* restore the original context and priv */
+	ctx->view.X1 = ox1; ctx->view.X2 = ox2; ctx->view.Y1 = oy1; ctx->view.Y2 = oy2;
 	gport->view = save_view;
 	gport->view.canvas_width = save_width;
 	gport->view.canvas_height = save_height;
