@@ -1,9 +1,10 @@
 /*
  *                            COPYRIGHT
  *
- *  PCB, interactive printed circuit board design
+ *  pcb-rnd, interactive printed circuit board design
+ *  (this file is based on PCB, interactive printed circuit board design)
  *  Copyright (C) 1994,1995,1996 Thomas Nau
- *  pcb-rnd Copyright (C) 2017 Alain Vigne
+ *  pcb-rnd Copyright (C) 2017,2018 Alain Vigne
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,9 +19,10 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
- *  Contact addresses for paper mail and Email:
- *  Thomas Nau, Schlehenweg 15, 88471 Baustetten, Germany
- *  Thomas.Nau@rz.uni-ulm.de
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  *
  */
 
@@ -40,6 +42,7 @@
 
 #include "../src_plugins/lib_gtk_hid/gui.h"
 #include "../src_plugins/lib_gtk_hid/coord_conv.h"
+#include "../src_plugins/lib_gtk_hid/preview_helper.h"
 
 #include "../src_plugins/lib_gtk_config/hid_gtk_conf.h"
 #include "../src_plugins/lib_gtk_config/lib_gtk_config.h"
@@ -561,10 +564,11 @@ static void ghid_cairo_draw_bg_image(void)
 	}
 }
 
-void ghid_cairo_render_burst(pcb_burst_op_t op, const pcb_box_t *screen)
+static void ghid_cairo_render_burst(pcb_burst_op_t op, const pcb_box_t *screen)
 {
 }
 
+/** Drawing modes usually cycle from RESET to (POSITIVE | NEGATIVE) to FLUSH. \p screen is not used in this HID. */
 static void ghid_cairo_set_drawing_mode(pcb_composite_op_t op, pcb_bool direct, const pcb_box_t *screen)
 {
 	render_priv_t *priv = gport->render_priv;
@@ -574,13 +578,6 @@ static void ghid_cairo_set_drawing_mode(pcb_composite_op_t op, pcb_bool direct,
 		return;
 	}
 
-	if (direct) {
-		//priv->out_pixel = priv->base_pixel;
-		//priv->out_clip = NULL;
-		curr_drawing_mode = PCB_HID_COMP_POSITIVE;
-		return;
-	}
-
 	switch(op) {
 		case PCB_HID_COMP_RESET:
 			//ghid_sketch_setup(priv);
@@ -606,11 +603,13 @@ static void ghid_cairo_set_drawing_mode(pcb_composite_op_t op, pcb_bool direct,
 			break;
 
 		case PCB_HID_COMP_FLUSH:
-			priv->cr = priv->cr_target;
-			//cairo_set_operator(priv->cr, CAIRO_OPERATOR_OVER);
-			//cairo_set_source_surface(priv->cr, priv->surf_layer, 0, 0);
-			cairo_mask_surface(priv->cr, priv->surf_layer, 0, 0);
-			cairo_fill(priv->cr);
+			if (direct)
+				end_subcomposite();
+			else {
+				priv->cr = priv->cr_target;
+				cairo_mask_surface(priv->cr, priv->surf_layer, 0, 0);
+				cairo_fill(priv->cr);
+			}
 			//cairo_paint_with_alpha(priv->cr, conf_core.appearance.layer_alpha);
 			//cairo_paint_with_alpha(priv->cr, 1.0);
 
@@ -818,8 +817,13 @@ static int use_gc(pcb_hid_gc_t gc)
 
 	//ghid_cairo_set_color(gc, gc->colorname);
 	gdk_cairo_set_source_rgba(cr, &gc->color);
-	//ghid_cairo_set_line_width(gc, gc->width);
-	cairo_set_line_width(cr, Vz(gc->width));
+
+	/* negative line width means "unit is pixels", so -1 is "1 pixel", -5 is "5 pixels", regardless of the zoom. */
+	if (gc->width <= 0)
+		cairo_set_line_width(cr, 1.0 - gc->width);
+	else
+		cairo_set_line_width(cr, Vz(gc->width));
+
 	//ghid_cairo_set_line_cap(gc, (pcb_cap_style_t) gc->cap);
 	cairo_set_line_cap(cr, gc->cap);
 	cairo_set_line_join(cr, gc->join);
@@ -863,11 +867,16 @@ static void ghid_cairo_draw_line(pcb_hid_gc_t gc, pcb_coord_t x1, pcb_coord_t y1
 	//gdk_draw_line(gport->drawable, priv->u_gc, dx1, dy1, dx2, dy2);
 }
 
+/** Draws an arc from center (cx, cy), cx>0 to the right, cy>0 to the bottom, using
+    xradius on X axis, and yradius on Y axis, sweeping a delta_angle from start_angle.
+
+    Angles (in degres) originate at 9 o'clock and are positive CCW (counter clock wise).
+    delta_angle sweeps the angle from start_angle, CW if negative.
+ */
 static void ghid_cairo_draw_arc(pcb_hid_gc_t gc, pcb_coord_t cx, pcb_coord_t cy,
 																pcb_coord_t xradius, pcb_coord_t yradius, pcb_angle_t start_angle, pcb_angle_t delta_angle)
 {
-	gint vrx2, vry2;
-	double w, h, radius;
+	double w, h, radius, angle1, angle2;
 	render_priv_t *priv = gport->render_priv;
 
 	w = gport->view.canvas_width * gport->view.coord_per_px;
@@ -879,8 +888,6 @@ static void ghid_cairo_draw_arc(pcb_hid_gc_t gc, pcb_coord_t cx, pcb_coord_t cy,
 		return;
 
 	USE_GC(gc);
-	vrx2 = Vz(xradius * 2.0);
-	vry2 = Vz(yradius * 2.0);
 
 	if ((delta_angle > 360.0) || (delta_angle < -360.0)) {
 		start_angle = 0;
@@ -900,10 +907,17 @@ static void ghid_cairo_draw_arc(pcb_hid_gc_t gc, pcb_coord_t cx, pcb_coord_t cy,
 	if (start_angle >= 180)
 		start_angle -= 360;
 
+	/* cairo lib. originates angles at 3 o'clock, positive traveling CW, requires angle1 < angle2 */
+	angle1 = (180.0 - start_angle);
+	angle2 = (delta_angle < 0) ? (angle1 - delta_angle) : angle1;
+	if (delta_angle > 0) {
+		angle1 = (180.0 - start_angle - delta_angle);
+	}
+	angle1 *= (M_PI / 180.0);
+	angle2 *= (M_PI / 180.0);
 	cairo_save(priv->cr);
 	/*FIXME: this will draw an arc of a circle, not an ellipse ! Explore matrix transformation here. */
-	cairo_arc(priv->cr, pcb_round(Vxd(cx)), pcb_round(Vyd(cy)), Vzd(radius),
-						(start_angle) * (M_PI / 180.0), delta_angle * (M_PI / 180.0));
+	cairo_arc(priv->cr, pcb_round(Vxd(cx)), pcb_round(Vyd(cy)), Vzd(radius), angle1, angle2);
 	cairo_stroke(priv->cr);
 	cairo_restore(priv->cr);
 	//gdk_draw_arc(gport->drawable, priv->u_gc, 0,
@@ -996,7 +1010,7 @@ static void ghid_cairo_fill_circle(pcb_hid_gc_t gc, pcb_coord_t cx, pcb_coord_t
 	//gdk_draw_arc(gport->drawable, priv->u_gc, TRUE, Vx(cx) - vr, Vy(cy) - vr, vr * 2, vr * 2, 0, 360 * 64);
 }
 
-/* Intentaional code duplication for performance */
+/** Intentional code duplication from  ghid_cairo_fill_polygon_offs(), for performance */
 static void ghid_cairo_fill_polygon(pcb_hid_gc_t gc, int n_coords, pcb_coord_t * x, pcb_coord_t * y)
 {
 	int i;
@@ -1020,7 +1034,7 @@ static void ghid_cairo_fill_polygon(pcb_hid_gc_t gc, int n_coords, pcb_coord_t *
 	cairo_fill(cr);
 }
 
-/* Intentaional code duplication for performance */
+/** Fills an N-points (x, y) poly, translated to (dx, dy) */
 static void ghid_cairo_fill_polygon_offs(pcb_hid_gc_t gc, int n_coords, pcb_coord_t *x, pcb_coord_t *y, pcb_coord_t dx, pcb_coord_t dy)
 {
 	int i;
@@ -1037,9 +1051,9 @@ static void ghid_cairo_fill_polygon_offs(pcb_hid_gc_t gc, int n_coords, pcb_coor
 		_x = Vx(x[i]);
 		_y = Vy(y[i]);
 		if (i == 0)
-			cairo_move_to(cr, _x + dx, _y + dy);
+			cairo_move_to(cr, _x + Vz(dx), _y + Vz(dy));
 		else
-			cairo_line_to(cr, _x + dx, _y + dy);
+			cairo_line_to(cr, _x + Vz(dx), _y + Vz(dy));
 	}
 	cairo_fill(cr);
 }
@@ -1499,7 +1513,7 @@ static void ghid_cairo_port_drawing_realize_cb(GtkWidget * widget, gpointer data
 }
 
 static gboolean ghid_cairo_preview_expose(GtkWidget * widget, pcb_gtk_expose_t * p,
-																					pcb_hid_expose_t expcall, const pcb_hid_expose_ctx_t * ctx)
+																					pcb_hid_expose_t expcall, pcb_hid_expose_ctx_t *ctx)
 {
 	//GdkWindow *window = gtk_widget_get_window(widget);
 	//GdkDrawable *save_drawable;
@@ -1508,6 +1522,7 @@ static gboolean ghid_cairo_preview_expose(GtkWidget * widget, pcb_gtk_expose_t *
 	int save_width, save_height;
 	double xz, yz, vw, vh;
 	render_priv_t *priv = gport->render_priv;
+	pcb_coord_t ox1 = ctx->view.X1, oy1 = ctx->view.Y1, ox2 = ctx->view.X2, oy2 = ctx->view.Y2;
 
 	vw = ctx->view.X2 - ctx->view.X1;
 	vh = ctx->view.Y2 - ctx->view.Y1;
@@ -1535,6 +1550,8 @@ static gboolean ghid_cairo_preview_expose(GtkWidget * widget, pcb_gtk_expose_t *
 	gport->view.x0 = (vw - gport->view.width) / 2 + ctx->view.X1;
 	gport->view.y0 = (vh - gport->view.height) / 2 + ctx->view.Y1;
 
+	PCB_GTK_PREVIEW_TUNE_EXTENT(ctx, allocation);
+
 	/* Change current pointer to draw in this widget, draws, then come back to pointer to drawing_area. */
 	priv->cr_target = p;
 	priv->cr = p;
@@ -1545,6 +1562,8 @@ static gboolean ghid_cairo_preview_expose(GtkWidget * widget, pcb_gtk_expose_t *
 	priv->cr = priv->cr_drawing_area;
 	//gport->drawable = save_drawable;
 
+	/* restore the original context and priv */
+	ctx->view.X1 = ox1; ctx->view.X2 = ox2; ctx->view.Y1 = oy1; ctx->view.Y2 = oy2;
 	gport->view = save_view;
 	gport->view.canvas_width = save_width;
 	gport->view.canvas_height = save_height;
