@@ -8,6 +8,7 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include <Xm/Notebook.h>
 
 #include "compat_misc.h"
 #include "data.h"
@@ -675,6 +676,7 @@ typedef struct {
 	Widget dialog;
 	pcb_hid_attr_val_t property[PCB_HATP_max];
 	unsigned inhibit_valchg:1;
+	Dimension minw, minh;
 } lesstif_attr_dlg_t;
 
 static void attribute_dialog_readres(lesstif_attr_dlg_t *ctx, int widx)
@@ -759,10 +761,47 @@ static void valchg(Widget w, XtPointer dlg_widget_, XtPointer call_data)
 		ctx->attrs[widx].change_cb(ctx, ctx->caller_data, &ctx->attrs[widx]);
 }
 
+static void pagechg(Widget w, XtPointer client_data, XtPointer call_data)
+{
+	lesstif_attr_dlg_t *ctx;
+	pcb_hid_attribute_t *attr = (pcb_hid_attribute_t *)client_data;
+	XmNotebookCallbackStruct *nptr = (XmNotebookCallbackStruct *)call_data;
+	int widx;
+
+	XtVaGetValues(w, XmNuserData, &ctx, NULL);
+
+	if (ctx == NULL)
+		return;
+
+	if (ctx->inhibit_valchg)
+		return;
+
+	widx = attr - ctx->attrs;
+	if ((widx < 0) || (widx >= ctx->n_attrs))
+		return;
+
+	ctx->results[widx].int_value = nptr->page_number - 1;
+
+	attribute_dialog_readres(ctx, widx);
+	if (ctx->property[PCB_HATP_GLOBAL_CALLBACK].func != NULL)
+		ctx->property[PCB_HATP_GLOBAL_CALLBACK].func(ctx, ctx->caller_data, &ctx->attrs[widx]);
+	if (attr->change_cb != NULL)
+		attr->change_cb(ctx, ctx->caller_data, attr);
+}
+
+/* parent info for tabbed */
+typedef struct {
+	Widget notebook;
+	const char **tablab;
+	Dimension minw;
+	int tabs;
+} attr_dlg_tb_t;
+
 /* returns the index of HATT_END where the loop had to stop */
-static int attribute_dialog_add(lesstif_attr_dlg_t *ctx, Widget parent, int start_from, int add_labels)
+static int attribute_dialog_add(lesstif_attr_dlg_t *ctx, Widget real_parent, attr_dlg_tb_t *tb, int start_from, int add_labels)
 {
 	int len, i, numch, numcol;
+	Widget parent;
 	static XmString empty = 0;
 
 	if (!empty)
@@ -778,6 +817,39 @@ static int attribute_dialog_add(lesstif_attr_dlg_t *ctx, Widget parent, int star
 		if (ctx->attrs[i].help_text == ATTR_UNDOCUMENTED)
 			continue;
 
+		/* if we are in a notebook, a page needs to be created first */
+		if (tb != NULL) {
+			const char *lab;
+			Widget tab;
+			Dimension wi;
+
+			parent = XmCreateRowColumn(tb->notebook, "page", NULL, 0);
+
+			stdarg_n = 0;
+			stdarg(XmNnotebookChildType, XmMAJOR_TAB);
+			if (*tb->tablab != NULL) {
+				lab = *tb->tablab;
+				tb->tablab++;
+			}
+			else
+				lab = "<anon>";
+
+			tab = XmCreatePushButton(tb->notebook, (char *)lab, stdarg_args, stdarg_n);
+			XtManageChild(tab);
+
+			/* update minimum width for tabs */
+			stdarg_n = 0;
+			stdarg(XmNwidth, &wi);
+			XtGetValues(tab, stdarg_args, stdarg_n);
+			if (wi > tb->minw)
+				tb->minw = wi;
+			tb->tabs++;
+
+			XtManageChild(parent);
+		}
+		else
+			parent = real_parent;
+
 		/* Add label */
 		if ((add_labels) && (ctx->attrs[i].type != PCB_HATT_LABEL)) {
 			stdarg_n = 0;
@@ -796,14 +868,14 @@ static int attribute_dialog_add(lesstif_attr_dlg_t *ctx, Widget parent, int star
 			w = pcb_motif_box(parent, XmStrCast(ctx->attrs[i].name), 'h', 0, (ctx->attrs[i].pcb_hatt_flags & PCB_HATF_FRAME), (ctx->attrs[i].pcb_hatt_flags & PCB_HATF_SCROLL));
 			XtManageChild(w);
 			ctx->wl[i] = w;
-			i = attribute_dialog_add(ctx, w, i+1, (ctx->attrs[i].pcb_hatt_flags & PCB_HATF_LABEL));
+			i = attribute_dialog_add(ctx, w, NULL, i+1, (ctx->attrs[i].pcb_hatt_flags & PCB_HATF_LABEL));
 			break;
 
 		case PCB_HATT_BEGIN_VBOX:
 			w = pcb_motif_box(parent, XmStrCast(ctx->attrs[i].name), 'v', 0, (ctx->attrs[i].pcb_hatt_flags & PCB_HATF_FRAME), (ctx->attrs[i].pcb_hatt_flags & PCB_HATF_SCROLL));
 			XtManageChild(w);
 			ctx->wl[i] = w;
-			i = attribute_dialog_add(ctx, w, i+1, (ctx->attrs[i].pcb_hatt_flags & PCB_HATF_LABEL));
+			i = attribute_dialog_add(ctx, w, NULL, i+1, (ctx->attrs[i].pcb_hatt_flags & PCB_HATF_LABEL));
 			break;
 
 		case PCB_HATT_BEGIN_TABLE:
@@ -815,7 +887,7 @@ static int attribute_dialog_add(lesstif_attr_dlg_t *ctx, Widget parent, int star
 
 			ctx->wl[i] = w;
 
-			i = attribute_dialog_add(ctx, w, i+1, (ctx->attrs[i].pcb_hatt_flags & PCB_HATF_LABEL));
+			i = attribute_dialog_add(ctx, w, NULL, i+1, (ctx->attrs[i].pcb_hatt_flags & PCB_HATF_LABEL));
 			while((len % numcol) != 0) {
 				Widget pad;
 
@@ -827,6 +899,43 @@ static int attribute_dialog_add(lesstif_attr_dlg_t *ctx, Widget parent, int star
 			}
 			XtManageChild(w);
 			break;
+
+		case PCB_HATT_BEGIN_TABBED:
+		{
+			Widget scroller;
+			attr_dlg_tb_t tb;
+
+			stdarg_n = 0;
+			stdarg(XmNbackPagePlacement, XmTOP_RIGHT);
+			stdarg(XmNbackPageNumber, 1);
+			stdarg(XmNbackPageSize, 1);
+			stdarg(XmNbindingType, XmNONE);
+			stdarg(XmNorientation, XmVERTICAL);
+
+			stdarg(XmNleftAttachment, XmATTACH_FORM);
+			stdarg(XmNtopAttachment, XmATTACH_FORM);
+			stdarg(XmNrightAttachment, XmATTACH_FORM);
+			stdarg(XmNbottomAttachment, XmATTACH_FORM);
+			stdarg(XmNuserData, ctx);
+			ctx->wl[i] = w = XmCreateNotebook(parent, "notebook", stdarg_args, stdarg_n);
+
+			/* remove the page scroller widget that got automatically created by XmCreateNotebook() */
+			scroller = XtNameToWidget(w, "PageScroller");
+			XtUnmanageChild (scroller);
+
+			XtAddCallback(w, XmNpageChangedCallback, (XtCallbackProc)pagechg, (XtPointer)&ctx->attrs[i]);
+
+			tb.notebook = w;
+			tb.tablab = ctx->attrs[i].enumerations;
+			tb.minw = 0;
+			tb.tabs = 0;
+
+			i = attribute_dialog_add(ctx, w, &tb, i+1, (ctx->attrs[i].pcb_hatt_flags & PCB_HATF_LABEL));
+
+			XtManageChild(w);
+			break;
+		}
+
 		case PCB_HATT_LABEL:
 			stdarg_n = 0;
 			stdarg(XmNalignment, XmALIGNMENT_BEGINNING);
@@ -919,6 +1028,11 @@ static int attribute_dialog_add(lesstif_attr_dlg_t *ctx, Widget parent, int star
 		}
 		if (ctx->wl[i] != NULL)
 			XtManageChild(ctx->wl[i]);
+		if (tb != NULL) {
+			tb->minw = tb->tabs * (tb->minw+10) + 10;
+			if (tb->minw > ctx->minw)
+				ctx->minw = tb->minw;
+		}
 	}
 	return i;
 }
@@ -937,6 +1051,9 @@ static int attribute_dialog_set(lesstif_attr_dlg_t *ctx, int idx, const pcb_hid_
 		case PCB_HATT_BEGIN_TABLE:
 		case PCB_HATT_END:
 			goto err;
+		case PCB_HATT_BEGIN_TABBED:
+			XtVaSetValues(ctx->wl[idx], XmNcurrentPageNumber, val->int_value+1, NULL);
+			break;
 		case PCB_HATT_BUTTON:
 			XtVaSetValues(ctx->wl[idx], XmNlabelString, XmStringCreatePCB(val->str_value), NULL);
 			break;
@@ -996,6 +1113,7 @@ void *lesstif_attr_dlg_new(pcb_hid_attribute_t *attrs, int n_attrs, pcb_hid_attr
 	ctx->results = results;
 	ctx->n_attrs = n_attrs;
 	ctx->caller_data = caller_data;
+	ctx->minw = ctx->minh = 32;
 
 	for (i = 0; i < n_attrs; i++) {
 		if (attrs[i].help_text != ATTR_UNDOCUMENTED)
@@ -1017,14 +1135,27 @@ void *lesstif_attr_dlg_new(pcb_hid_attribute_t *attrs, int n_attrs, pcb_hid_attr
 	stdarg(XmNfractionBase, ctx->n_attrs);
 	XtSetValues(topform, stdarg_args, stdarg_n);
 
+
 	if (!PCB_HATT_IS_COMPOSITE(attrs[0].type)) {
 		stdarg_n = 0;
 		main_tbl = pcb_motif_box(topform, XmStrCast("layout"), 't', pcb_hid_atrdlg_num_children(ctx->attrs, 0, ctx->n_attrs), 0, 0);
 		XtManageChild(main_tbl);
-		attribute_dialog_add(ctx, main_tbl, 0, 1);
+		attribute_dialog_add(ctx, main_tbl, NULL, 0, 1);
 	}
 	else
-		attribute_dialog_add(ctx, topform, 0, (ctx->attrs[0].pcb_hatt_flags & PCB_HATF_LABEL));
+		attribute_dialog_add(ctx, topform, NULL, 0, (ctx->attrs[0].pcb_hatt_flags & PCB_HATF_LABEL));
+
+	/* don't expect screens larger than 800x600 */
+	if (ctx->minw > 750)
+		ctx->minw = 750;
+	if (ctx->minw > 550)
+		ctx->minw = 550;
+
+	/* set top form's minimum width/height to content request */
+	stdarg_n = 0;
+	stdarg(XmNminWidth, ctx->minw);
+	stdarg(XmNminHeight, ctx->minh);
+	XtSetValues(XtParent(ctx->dialog), stdarg_args, stdarg_n);
 
 	if (!modal)
 		XtManageChild(ctx->dialog);
