@@ -64,7 +64,7 @@ static void libnode_select(Widget w, void *v, XmListCallbackStruct * cbs)
 {
 	pcb_fplibrary_t *e = picks.array[last_pick];
 	e = &e->data.dir.children.array[cbs->item_position - 1];
-	if (pcb_element_load_to_buffer(PCB_PASTEBUFFER, e->data.fp.loc_info))
+	if (pcb_element_load_to_buffer(PCB_PASTEBUFFER, e->data.fp.loc_info, NULL))
 		pcb_crosshair_set_mode(PCB_MODE_PASTE_BUFFER);
 }
 
