@@ -53,24 +53,21 @@ Prompts the user for a coordinate, if one is not already selected.
 
 static int GetXY(int argc, const char **argv, pcb_coord_t x, pcb_coord_t y)
 {
+	pcb_gui->get_coords(argv[0], &x, &y);
 	return 0;
 }
 
 /*-----------------------------------------------------------------------------*/
 
-#define LB_SILK	(PCB_MAX_LAYER+0)
-#define LB_RATS	(PCB_MAX_LAYER+1)
+#define LB_RATS	(PCB_MAX_LAYER+0)
 #define LB_NUMPICK (LB_RATS+1)
 /* more */
-#define LB_PINS	(PCB_MAX_LAYER+2)
-#define LB_VIAS	(PCB_MAX_LAYER+3)
-#define LB_BACK	(PCB_MAX_LAYER+4)
-#define LB_MASK	(PCB_MAX_LAYER+5)
-#define LB_PASTE	(PCB_MAX_LAYER+6)
-#define LB_SUBC	(PCB_MAX_LAYER+7)
-#define LB_SUBC_PARTS	(PCB_MAX_LAYER+8)
-
-#define LB_NUM  (PCB_MAX_LAYER+9)
+#define LB_BACK	(PCB_MAX_LAYER+1)
+#define LB_SUBC	(PCB_MAX_LAYER+2)
+#define LB_SUBC_PARTS	(PCB_MAX_LAYER+3)
+#define LB_PSTK_MARKS	(PCB_MAX_LAYER+4)
+#define LB_HOLES	(PCB_MAX_LAYER+5)
+#define LB_NUM  (PCB_MAX_LAYER+6)
 
 typedef struct {
 	Widget w[LB_NUM];
@@ -94,31 +91,32 @@ void LesstifLayersChanged(void *user_data, int argc, pcb_event_arg_t argv[])
 		return;
 	if (PCB && PCB->Data) {
 		pcb_data_t *d = PCB->Data;
-		for (i = 0; i < PCB_MAX_LAYER; i++)
-			fg_colors[i] = lesstif_parse_color(d->Layer[i].meta.real.color);
-		fg_colors[LB_SILK] = lesstif_parse_color(conf_core.appearance.color.element);
+		for (i = 0; i < PCB_MAX_LAYER; i++) {
+			pcb_layer_type_t lyt = pcb_layer_flags(PCB, i);
+#warning layer TODO: hardwired layer colors
+			if (lyt & PCB_LYT_SILK) fg_colors[i] = lesstif_parse_color(conf_core.appearance.color.element);
+			else if (lyt & PCB_LYT_MASK) fg_colors[i] = lesstif_parse_color(conf_core.appearance.color.mask);
+			else if (lyt & PCB_LYT_PASTE) fg_colors[i] = lesstif_parse_color(conf_core.appearance.color.paste);
+			else fg_colors[i] = lesstif_parse_color(d->Layer[i].meta.real.color);
+		}
+
 		fg_colors[LB_RATS] = lesstif_parse_color(conf_core.appearance.color.rat);
-		fg_colors[LB_PINS] = lesstif_parse_color(conf_core.appearance.color.pin);
 		fg_colors[LB_SUBC] = lesstif_parse_color(conf_core.appearance.color.subc);
 		fg_colors[LB_SUBC_PARTS] = lesstif_parse_color(conf_core.appearance.color.subc);
-		fg_colors[LB_VIAS] = lesstif_parse_color(conf_core.appearance.color.via);
+		fg_colors[LB_PSTK_MARKS] = lesstif_parse_color(conf_core.appearance.color.subc);
+		fg_colors[LB_HOLES] = lesstif_parse_color(conf_core.appearance.color.via);
 		fg_colors[LB_BACK] = lesstif_parse_color(conf_core.appearance.color.invisible_objects);
-		fg_colors[LB_MASK] = lesstif_parse_color(conf_core.appearance.color.mask);
-		fg_colors[LB_PASTE] = lesstif_parse_color(conf_core.appearance.color.paste);
 		bg_color = lesstif_parse_color(conf_core.appearance.color.background);
 	}
 	else {
 		for (i = 0; i < PCB_MAX_LAYER; i++)
 			fg_colors[i] = lesstif_parse_color(conf_core.appearance.color.layer[i]);
-		fg_colors[LB_SILK] = lesstif_parse_color(conf_core.appearance.color.element);
 		fg_colors[LB_RATS] = lesstif_parse_color(conf_core.appearance.color.rat);
-		fg_colors[LB_PINS] = lesstif_parse_color(conf_core.appearance.color.pin);
 		fg_colors[LB_SUBC] = lesstif_parse_color(conf_core.appearance.color.subc);
 		fg_colors[LB_SUBC_PARTS] = lesstif_parse_color(conf_core.appearance.color.subc);
-		fg_colors[LB_VIAS] = lesstif_parse_color(conf_core.appearance.color.via);
+		fg_colors[LB_PSTK_MARKS] = lesstif_parse_color(conf_core.appearance.color.subc);
+		fg_colors[LB_HOLES] = lesstif_parse_color(conf_core.appearance.color.via);
 		fg_colors[LB_BACK] = lesstif_parse_color(conf_core.appearance.color.invisible_objects);
-		fg_colors[LB_MASK] = lesstif_parse_color(conf_core.appearance.color.mask);
-		fg_colors[LB_PASTE] = lesstif_parse_color(conf_core.appearance.color.paste);
 		bg_color = lesstif_parse_color(conf_core.appearance.color.background);
 	}
 
@@ -131,33 +129,21 @@ void LesstifLayersChanged(void *user_data, int argc, pcb_event_arg_t argv[])
 		LayerButtons *lb = layer_button_list + l;
 		for (i = 0; i < (lb->is_pick ? LB_NUMPICK : LB_NUM); i++) {
 			switch (i) {
-			case LB_SILK:
-				set = pcb_silk_on(PCB);
-				break;
 			case LB_RATS:
 				set = PCB->RatOn;
 				break;
-			case LB_PINS:
-				set = PCB->PinOn;
-				break;
 			case LB_SUBC:
 				set = PCB->SubcOn;
 				break;
 			case LB_SUBC_PARTS:
 				set = PCB->SubcPartsOn;
 				break;
-			case LB_VIAS:
-				set = PCB->ViaOn;
+			case LB_PSTK_MARKS:
+				set = PCB->padstack_mark_on;
 				break;
 			case LB_BACK:
 				set = PCB->InvisibleObjectsOn;
 				break;
-			case LB_MASK:
-				set = pcb_mask_on(PCB);
-				break;
-			case LB_PASTE:
-				set = pcb_paste_on(PCB);
-				break;
 			default:									/* layers */
 				set = PCB->Data->Layer[i].meta.real.vis;
 				break;
@@ -186,7 +172,7 @@ void LesstifLayersChanged(void *user_data, int argc, pcb_event_arg_t argv[])
 			}
 			XtSetValues(lb->w[i], stdarg_args, stdarg_n);
 
-			if ((i >= pcb_max_layer && i < PCB_MAX_LAYER) || (pcb_layer_flags(PCB, i) & PCB_LYT_SILK))
+			if (i >= pcb_max_layer && i < PCB_MAX_LAYER)
 				XtUnmanageChild(lb->w[i]);
 			else
 				XtManageChild(lb->w[i]);
@@ -197,9 +183,6 @@ void LesstifLayersChanged(void *user_data, int argc, pcb_event_arg_t argv[])
 		case LB_RATS:
 			name = "Rats";
 			break;
-		case LB_SILK:
-			name = "Silk";
-			break;
 		default:
 			name = PCB->Data->Layer[current_layer].name;
 			break;
@@ -242,51 +225,36 @@ static void layer_button_callback(Widget w, int layer, XmPushButtonCallbackStruc
 {
 	int l, set;
 	switch (layer) {
-	case LB_SILK:
-		set = !pcb_silk_on(PCB);
-		PCB->Data->SILKLAYER.meta.real.vis = set;
-		PCB->Data->BACKSILKLAYER.meta.real.vis = set;
-		break;
 	case LB_RATS:
 		set = PCB->RatOn = !PCB->RatOn;
 		break;
-	case LB_PINS:
-		set = PCB->PinOn = !PCB->PinOn;
-		break;
 	case LB_SUBC:
 		set = PCB->SubcOn = !PCB->SubcOn;
 		break;
 	case LB_SUBC_PARTS:
 		set = PCB->SubcPartsOn = !PCB->SubcPartsOn;
 		break;
-	case LB_VIAS:
-		set = PCB->ViaOn = !PCB->ViaOn;
+	case LB_PSTK_MARKS:
+		set = PCB->padstack_mark_on = !PCB->padstack_mark_on;
+		break;
+	case LB_HOLES:
+		set = PCB->hole_on = !PCB->hole_on;
 		break;
 	case LB_BACK:
 		set = PCB->InvisibleObjectsOn = !PCB->InvisibleObjectsOn;
 		break;
-	case LB_MASK:
-#warning layersel TODO
-/*		conf_toggle_editor(show_mask);*/
-		set = pcb_mask_on(PCB);
-		break;
-	case LB_PASTE:
-#warning layersel TODO
-/*		conf_toggle_editor(show_paste);*/
-		set = pcb_paste_on(PCB);
-		break;
 	default:											/* layers */
 		set = PCB->Data->Layer[layer].meta.real.vis = !PCB->Data->Layer[layer].meta.real.vis;
 		break;
 	}
 
 	show_one_layer_button(layer, set);
-	if ((layer < pcb_max_layer) && (!(pcb_layer_flags(PCB, layer) & PCB_LYT_SILK))) {
+	if (layer < pcb_max_layer) {
 		int i;
 		pcb_layergrp_id_t group = pcb_layer_get_group(PCB, layer);
 		for (i = 0; i < PCB->LayerGroups.grp[group].len; i++) {
 			l = PCB->LayerGroups.grp[group].lid[i];
-			if (l != layer && (!(pcb_layer_flags(PCB, l) & PCB_LYT_SILK))) {
+			if (l != layer) {
 				show_one_layer_button(l, set);
 				PCB->Data->Layer[l].meta.real.vis = set;
 			}
@@ -300,7 +268,7 @@ static void layerpick_button_callback(Widget w, int layer, XmPushButtonCallbackS
 	int l, i;
 	const char *name;
 	PCB->RatDraw = (layer == LB_RATS);
-	if ((layer < pcb_max_layer) && (!(pcb_layer_flags(PCB, layer) & PCB_LYT_SILK)))
+	if (layer < pcb_max_layer)
 		pcb_layervis_change_group_vis(layer, 1, 1);
 	for (l = 0; l < num_layer_buttons; l++) {
 		LayerButtons *lb = layer_button_list + l;
@@ -313,9 +281,6 @@ static void layerpick_button_callback(Widget w, int layer, XmPushButtonCallbackS
 	case LB_RATS:
 		name = "Rats";
 		break;
-	case LB_SILK:
-		name = "Silk";
-		break;
 	default:
 		name = PCB->Data->Layer[layer].name;
 		break;
@@ -347,48 +312,30 @@ static void insert_layerview_buttons(Widget menu)
 		Widget btn;
 		namestr[5] = 'A' + i;
 		switch (i) {
-		case LB_SILK:
-			name = "Silk";
-			break;
 		case LB_RATS:
 			name = "Rat Lines";
 			break;
-		case LB_PINS:
-			name = "Pins/Pads";
-			break;
 		case LB_SUBC:
 			name = "Subcircuits";
 			break;
 		case LB_SUBC_PARTS:
 			name = "Subc. parts";
 			break;
-		case LB_VIAS:
-			name = "Vias";
+		case LB_PSTK_MARKS:
+			name = "Pstk. Marks";
+			break;
+		case LB_HOLES:
+			name = "Holes";
 			break;
 		case LB_BACK:
 			name = "Far Side";
 			break;
-		case LB_MASK:
-			name = "Solder Mask";
-			break;
-		case LB_PASTE:
-			name = "Paste";
-			break;
 		}
 		stdarg_n = 0;
 		btn = XmCreateToggleButton(menu, XmStrCast(name), stdarg_args, stdarg_n);
 		XtManageChild(btn);
 		XtAddCallback(btn, XmNvalueChangedCallback, (XtCallbackProc) layer_button_callback, (XtPointer) (size_t) i);
 		lb->w[i] = btn;
-
-		if (i == LB_MASK) {
-#warning layersel TODO
-/*			note_widget_flag(btn, XmNset, "editor/show_mask");*/
-		}
-		else if (i == LB_PASTE) {
-#warning layersel TODO
-/*			note_widget_flag(btn, XmNset, "editor/show_paste");*/
-		}
 	}
 	lb->is_pick = 0;
 	LesstifLayersChanged(0, 0, 0);
@@ -414,10 +361,6 @@ static void insert_layerpick_buttons(Widget menu)
 		Widget btn;
 		namestr[5] = 'A' + i;
 		switch (i) {
-		case LB_SILK:
-			name = "Silk";
-			strcpy(av, "SelectLayer(Silk)");
-			break;
 		case LB_RATS:
 			name = "Rat Lines";
 			strcpy(av, "SelectLayer(Rats)");
@@ -479,6 +422,7 @@ static int del_widget_flag(int idx)
 	wflags[idx].w = NULL;
 	wflags[idx].flagname = NULL;
 	wflags[idx].xres = NULL;
+	return 0;
 }
 
 void lesstif_update_widget_flags()
@@ -508,7 +452,7 @@ void lesstif_update_widget_flags()
 /*-----------------------------------------------------------------------------*/
 
 pcb_hid_action_t lesstif_menu_action_list[] = {
-	{"GetXY", "", GetXY,
+	{"GetXY", 0, GetXY,
 	 getxy_help, getxy_syntax},
 };
 
