@@ -1,7 +1,8 @@
 /*
  *                            COPYRIGHT
  *
- *  PCB, interactive printed circuit board design
+ *  pcb-rnd, interactive printed circuit board design
+ *  (this file is based on PCB, interactive printed circuit board design)
  *  Copyright (C) 1994,1995,1996,1997,1998,2005,2006 Thomas Nau
  *  Copyright (C) 2015,2016 Tibor 'Igor2' Palinkas
  *
@@ -19,6 +20,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  */
 
 #include "config.h"
@@ -58,7 +63,7 @@ static int ReadNetlist(const char *filename)
 	const char *ratcmd;
 
 	if (!filename)
-		return (1);									/* nothing to do */
+		return 1;									/* nothing to do */
 
 	pcb_message(PCB_MSG_INFO, _("Importing PCB netlist %s\n"), filename);
 
@@ -83,7 +88,7 @@ static int ReadNetlist(const char *filename)
 		if (*command == '\0' || (fp = pcb_popen(command, "r")) == NULL) {
 			pcb_popen_error_message(command);
 			free(command);
-			return (1);
+			return 1;
 		}
 		free(command);
 	}
@@ -147,7 +152,7 @@ static int ReadNetlist(const char *filename)
 	if (!lines) {
 		pcb_message(PCB_MSG_ERROR, _("Empty netlist file!\n"));
 		pcb_pclose(fp);
-		return (1);
+		return 1;
 	}
 	if (used_popen)
 		pcb_pclose(fp);
@@ -155,7 +160,7 @@ static int ReadNetlist(const char *filename)
 		fclose(fp);
 	pcb_sort_netlist();
 	pcb_ratspatch_make_edited(PCB);
-	return (0);
+	return 0;
 }
 
 int netlist_support_prio(pcb_plug_import_t *ctx, unsigned int aspects, FILE *fp, const char *filename)
