@@ -1,7 +1,8 @@
 /*
  *                            COPYRIGHT
  *
- *  PCB, interactive printed circuit board design
+ *  pcb-rnd, interactive printed circuit board design
+ *  (this file is based on PCB, interactive printed circuit board design)
  *  Copyright (C) 1994,1995,1996 Thomas Nau
  *  Copyright (C) 1997, 1998, 1999, 2000, 2001 Harry Eaton
  *
@@ -19,7 +20,13 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
- *  Contact addresses for paper mail and Email:
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
+ *
+ *
+ *  Old contact info:
  *  Harry Eaton, 6697 Buttonhole Ct, Columbia, MD 21044, USA
  *  haceaton@aplcomm.jhuapl.edu
  *
@@ -177,9 +184,9 @@ static int pcb_act_Import(int argc, const char **argv, pcb_coord_t x, pcb_coord_
 	const char **sources = NULL;
 	int nsources = 0;
 
-#ifdef DEBUG
-	printf("pcb_act_Import:  ===========  Entering pcb_act_Import  ============\n");
-#endif
+	if (conf_import_sch.plugins.import_sch.verbose)
+		pcb_message(PCB_MSG_DEBUG, "pcb_act_Import:  ===  Entering pcb_act_Import  ===\n");
+
 
 	mode = PCB_ACTION_ARG(0);
 
@@ -335,10 +342,12 @@ static int pcb_act_Import(int argc, const char **argv, pcb_coord_t x, pcb_coord_
 			cmd[8 + i] = pcb_build_fn(sources[i]);
 		cmd[8 + nsources] = NULL;
 
-#ifdef DEBUG
-		printf("pcb_act_Import:  ===========  About to run gnetlist  ============\n");
-		printf("%s %s %s %s %s %s %s %s %s ...\n", cmd[0], cmd[1], cmd[2], cmd[3], cmd[4], cmd[5], cmd[6], cmd[7], cmd[8]);
-#endif
+		if (conf_import_sch.plugins.import_sch.verbose) {
+			pcb_message(PCB_MSG_DEBUG, "pcb_act_Import:  running gnetlist:\n");
+			for (i = 0; i < 8+nsources; i++)
+				pcb_message(PCB_MSG_DEBUG, " %s", cmd[i]);
+			pcb_message(PCB_MSG_DEBUG, "\n");
+		}
 
 		if (pcb_spawnvp(cmd)) {
 			for(i = 0; i < nsources; i++)
@@ -347,9 +356,8 @@ static int pcb_act_Import(int argc, const char **argv, pcb_coord_t x, pcb_coord_
 			return 1;
 		}
 
-#ifdef DEBUG
-		printf("pcb_act_Import:  ===========  About to run pcb_act_ExecuteFile, file = %s  ============\n", tmpfile);
-#endif
+		if (conf_import_sch.plugins.import_sch.verbose)
+			pcb_message(PCB_MSG_DEBUG, "pcb_act_Import:  about to run pcb_act_ExecuteFile, file = %s\n", tmpfile);
 
 		cmd[0] = tmpfile;
 		cmd[1] = NULL;
@@ -445,9 +453,8 @@ static int pcb_act_Import(int argc, const char **argv, pcb_coord_t x, pcb_coord_
 	pcb_rats_destroy(pcb_false);
 	pcb_rat_add_all(pcb_false, NULL);
 
-#ifdef DEBUG
-	printf("pcb_act_Import:  ===========  Leaving pcb_act_Import  ============\n");
-#endif
+	if (conf_import_sch.plugins.import_sch.verbose)
+		pcb_message(PCB_MSG_DEBUG, "pcb_act_Import:  ===  Leaving pcb_act_Import  ===\n");
 
 	return 0;
 }
