@@ -1,7 +1,8 @@
 /*
  *                            COPYRIGHT
  *
- *  PCB, interactive printed circuit board design
+ *  pcb-rnd, interactive printed circuit board design
+ *  (this file is based on PCB, interactive printed circuit board design)
  *
  *  hyperlynx .hyp importer, plugin entry
  *  Copyright (C) 2016 Tibor 'Igor2' Palinkas
@@ -20,6 +21,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  */
 
 #include "config.h"
@@ -117,11 +122,14 @@ pcb_hid_action_t hyp_action_list[] = {
 PCB_REGISTER_ACTIONS(hyp_action_list, hyp_cookie)
 
 /* cheap, partial read of the file to determine if it is worth running the real parser */
-		 int io_hyp_test_parse_pcb(pcb_plug_io_t * ctx, pcb_board_t * Ptr, const char *Filename, FILE * f)
+int io_hyp_test_parse(pcb_plug_io_t *ctx, pcb_plug_iot_t typ, const char *Filename, FILE * f)
 {
 	char line[1024];
 	int found = 0, lineno = 0;
 
+	if (typ != PCB_IOT_PCB)
+		return 0; /* support only boards for now */
+
 	/* look for {VERSION and {BOARD in the first 32 lines, not assuming indentation */
 	while (fgets(line, sizeof(line), f) != NULL) {
 		if ((found == 0) && (strstr(line, "{VERSION=")))
@@ -158,7 +166,7 @@ int pplg_init_io_hyp(void)
 	/* register the IO hook */
 	io_hyp.plugin_data = NULL;
 	io_hyp.fmt_support_prio = io_hyp_fmt;
-	io_hyp.test_parse_pcb = io_hyp_test_parse_pcb;
+	io_hyp.test_parse = io_hyp_test_parse;
 	io_hyp.parse_pcb = io_hyp_read_pcb;
 /*	io_hyp.parse_element = NULL;
 	io_hyp.parse_font = NULL;
