@@ -1,6 +1,7 @@
 /*
  * read hyperlynx files
  * Copyright 2016 Koen De Vleeschauwer.
+ * Copyright (C) 2018 Tibor 'Igor2' Palinkas
  *
  * This file is part of pcb-rnd.
  *
@@ -18,13 +19,6 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* 
- * ~/.indent.pro:
- *   --line-length128 -brs -br -nce --tab-size2 -ut -npsl -npcs -hnl
- * ~/.vimrc:
- *   set tabstop=2
- */
-
 #define MAX_STRING 256
 
 #include "parser.h"
@@ -44,8 +38,14 @@
 #include "plug_io.h"
 #include "compat_misc.h"
 #include "safe_fs.h"
+#include "../src_plugins/lib_compat_help/pstk_help.h"
+#include "../src_plugins/lib_compat_help/subc_help.h"
 #include <string.h>
 
+#define MASK_OFFS  +PCB_MIL_TO_COORD(10)
+#define PASTE_OFFS -PCB_MIL_TO_COORD(10)
+
+
 #undef min
 #undef max
 #define min(x, y) ((x) < (y) ? (x) : (y))
@@ -80,7 +80,7 @@ outline_t *outline_tail;
 void hyp_set_origin();					/* set origin so all coordinates are positive */
 void hyp_perimeter();						/* add board outline to pcb */
 void hyp_draw_polygons();				/* add all hyperlynx polygons to pcb */
-void hyp_create_silkscreen();		/* add silkscreen outlines to all devices on board */
+static void hyp_subcs_fin(void);
 void hyp_resize_board();				/* resize board to fit outline */
 pcb_bool_t hyp_is_bottom_layer(char *);	/* true if bottom layer */
 
@@ -130,6 +130,7 @@ typedef struct device_s {
 	char *name;										/* optional */
 	char *value;									/* optional */
 	char *layer_name;
+	pcb_subc_t *subc;
 	struct device_s *next;
 } device_t;
 
@@ -164,13 +165,7 @@ typedef struct padstack_s {
 padstack_t *padstack_head;
 padstack_t *current_pstk;
 
-	/* pads */
-pcb_element_t *component_side_pads;
-pcb_element_t *solder_side_pads;
-#define PAD_TOP "PAD_TOP"
-#define PAD_BOTTOM "PAD_BOTTOM"
-
-	/* polygons */
+/* polygons */
 
 /* 
  * a hyperlynx polygon is a sequence of line and arc segments.
@@ -299,10 +294,6 @@ void hyp_init(void)
 	unknown_device_number = 0;
 	unknown_pin_number = 0;
 
-	/* clear pads */
-	component_side_pads = NULL;
-	solder_side_pads = NULL;
-
 	/* clear polygon data */
 	polygon_head = NULL;
 	current_vertex = NULL;
@@ -342,15 +333,15 @@ int hyp_parse(pcb_data_t * dest, const char *fname, int debug)
 	/* parse hyperlynx file */
 	hyyin = pcb_fopen(fname, "r");
 	if (hyyin == NULL)
-		return (1);
+		return 1;
 	retval = hyyparse();
 	fclose(hyyin);
 
 	/* set up polygons */
 	hyp_draw_polygons();
 
-	/* create device outlines */
-	hyp_create_silkscreen();
+	/* postprocess/finalize subcircuits */
+	hyp_subcs_fin();
 
 	/* add board outline last */
 	hyp_perimeter();
@@ -360,7 +351,7 @@ int hyp_parse(pcb_data_t * dest, const char *fname, int debug)
 
 	hyp_netlist_end();
 
-	return (retval);
+	return retval;
 }
 
 /* print error message */
@@ -379,7 +370,7 @@ void hyp_error(const char *msg)
 /*
  * find padstack by name 
  */
-
+#warning TODO: use a hash instead
 padstack_t *hyp_pstk_by_name(char *padstack_name)
 {
 	padstack_t *i;
@@ -427,7 +418,7 @@ void hyp_netlist_end()
 /*
  * find hyperlynx device by name 
  */
-
+#warning TODO: convert this into a hash
 device_t *hyp_device_by_name(char *device_name)
 {
 	device_t *i;
@@ -437,48 +428,67 @@ device_t *hyp_device_by_name(char *device_name)
 	return NULL;
 }
 
-/*
- * create pcb element by name
- * parameters: name, (x, y) coordinates of element text.
- */
-
-pcb_element_t *hyp_create_element_by_name(char *element_name, pcb_coord_t x, pcb_coord_t y)
+pcb_subc_t *hyp_create_subc_by_name(char *refdes, pcb_coord_t x, pcb_coord_t y)
 {
-	pcb_element_t *elem;
-	pcb_flag_t flags;
+	pcb_subc_t *subc;
 	pcb_uint8_t text_direction = 0;
 	int text_scale = 100;
+	device_t *dev;
+	int on_bottom;
 
-	flags = pcb_no_flags();
 	/* does the device already exist? */
-	elem = pcb_search_elem_by_name(hyp_dest, element_name);
-	if (elem == NULL) {
-		/* device needs to be created. Search in DEVICE records. */
-		device_t *dev = hyp_device_by_name(element_name);
-		if (dev != NULL) {
-			/* device on component or solder side? */
-			if (hyp_is_bottom_layer(dev->layer_name)) {
-				flags = pcb_flag_add(flags, PCB_FLAG_ONSOLDER);
-				text_direction = 2;
-			}
-			/* create */
-			if (hyp_debug)
-				pcb_message(PCB_MSG_DEBUG, "creating device \"%s\".\n", dev->ref);
-			elem =
-				pcb_element_new(hyp_dest, NULL, pcb_font(PCB, 0, 1), flags, dev->name, dev->ref, dev->value, x, y, text_direction,
-												text_scale, flags, pcb_false);
-		}
-		else {
-			/* no device with this name exists, and no such device has been listed in a DEVICE record. Let's create the device anyhow so we can continue. Assume device is on component side. */
-			pcb_message(PCB_MSG_WARNING, "device \"%s\" not specified in DEVICE record. Assuming device is on component side.\n",
-									element_name);
-			elem =
-				pcb_element_new(hyp_dest, NULL, pcb_font(PCB, 0, 1), flags, element_name, element_name, NULL, x, y, text_direction,
-												text_scale, flags, pcb_false);
-		}
+	subc = pcb_subc_by_refdes(hyp_dest, refdes);
+	if (subc != NULL)
+		return subc;
+
+	/* device needs to be created. Search in DEVICE records. */
+	dev = hyp_device_by_name(refdes);
+	if (dev == NULL) {
+		/* no device with this name exists, and no such device has been listed in a DEVICE record. Let's create the device anyhow so we can continue. Assume device is on component side. */
+		pcb_message(PCB_MSG_WARNING, "device \"%s\" not specified in DEVICE record. Assuming device is on component side.\n", refdes);
+
+		dev = calloc(sizeof(device_t), 1);
+		dev->next = device_head;
+		device_head = dev;
 	}
 
-	return elem;
+	/* device on component or solder side? */
+	if (dev->layer_name != NULL)
+		on_bottom = hyp_is_bottom_layer(dev->layer_name);
+	if (on_bottom)
+		text_direction = 2;
+
+	/* create */
+	if (hyp_debug)
+		pcb_message(PCB_MSG_DEBUG, "creating device \"%s\".\n", dev->ref);
+
+	subc = pcb_subc_alloc();
+	pcb_subc_create_aux(subc, x, y, 0.0, on_bottom);
+	pcb_attribute_put(&subc->Attributes, "refdes", refdes);
+	pcb_subc_add_refdes_text(subc, x, y, text_direction, text_scale, on_bottom);
+	pcb_add_subc_to_data(hyp_dest, subc);
+	pcb_subc_bind_globals(hyp_dest->parent.board, subc);
+
+	dev->subc = subc;
+
+	return subc;
+}
+
+static void hyp_subc_fin(pcb_subc_t *subc)
+{
+	pcb_subc_bbox(subc);
+	if (hyp_dest->subc_tree == NULL)
+		hyp_dest->subc_tree = pcb_r_create_tree();
+	pcb_r_insert_entry(hyp_dest->subc_tree, (pcb_box_t *)subc);
+	pcb_subc_rebind(hyp_dest->parent.board, subc);
+}
+
+static void hyp_subcs_fin(void)
+{
+	device_t *d;
+	for(d = device_head; d != NULL; d = d->next)
+		if (d->subc != NULL)
+			hyp_subc_fin(d->subc);
 }
 
 /*
@@ -778,75 +788,9 @@ void hyp_perimeter()
 	return;
 }
 
-/*
- * silkscreen a simple box around an element
- */
-
-void hyp_element_silkscreen_new(pcb_data_t * Data, pcb_element_t * Element, pcb_font_t * Font)
-{
-	pcb_coord_t x1, x2, y1, y2;
-
-	/* get bounding box of device */
-	if (PCB_FLAG_TEST(PCB_FLAG_ONSOLDER, Element)) {
-		/* put component part no. bottom left. on solder side bottom left is (X2, Y2) */
-		x1 = Element->BoundingBox.X2;
-		x2 = Element->BoundingBox.X1;
-	}
-	else {
-		/* put component part no. bottom left. on component side bottom left is (X1, Y2) */
-		x1 = Element->BoundingBox.X1;
-		x2 = Element->BoundingBox.X2;
-	};
-	y1 = Element->BoundingBox.Y1;
-	y2 = Element->BoundingBox.Y2;
-
-	/* draw rectangle around device on silkscreen */
-	pcb_element_line_new(Element, x1, y1, x1, y2, 1);
-	pcb_element_line_new(Element, x1, y2, x2, y2, 1);
-	pcb_element_line_new(Element, x2, y2, x2, y1, 1);
-	pcb_element_line_new(Element, x2, y1, x1, y1, 1);
-
-	/* put part no. bottom left */
-	PCB_ELEM_TEXT_REFDES(Element).X = x1;
-	PCB_ELEM_TEXT_REFDES(Element).Y = y2;
-
-	/* put description top left */
-	PCB_ELEM_TEXT_DESCRIPTION(Element).X = x1;
-	PCB_ELEM_TEXT_DESCRIPTION(Element).Y = y1;
-
-	/* put value bottom right */
-	PCB_ELEM_TEXT_VALUE(Element).X = x2;
-	PCB_ELEM_TEXT_VALUE(Element).Y = y2;
-
-	/* update */
-	pcb_element_bbox(Data, Element, Font);
-
-	return;
-}
-
-/*
- * create device outlines and clean up device reference text. 
- * call after all devices have been created.
- */
-
-void hyp_create_silkscreen()
-{
-	device_t *i;
-	pcb_element_t *elem;
-
-	for (i = device_head; i != NULL; i = i->next) {
-		elem = pcb_search_elem_by_name(hyp_dest, i->ref);
-		if (elem != NULL)
-			hyp_element_silkscreen_new(hyp_dest, elem, pcb_font(PCB, 0, 1));
-	}
-
-	return;
-}
-
 /* 
  * reset pcb layer stack 
  */
-
 void hyp_reset_layers()
 {
 
@@ -1654,10 +1598,7 @@ pcb_bool exec_plane(parse_param * h)
 	return 0;
 }
 
-/*
- * DEVICE record.
- */
-
+/* DEVICE record. */
 pcb_bool exec_devices(parse_param * h)
 {
 	device_t *new_device;
@@ -1679,7 +1620,7 @@ pcb_bool exec_devices(parse_param * h)
 	}
 
 	/* add device to list  */
-	new_device = malloc(sizeof(device_t));
+	new_device = calloc(sizeof(device_t), 1);
 
 	new_device->ref = pcb_strdup(h->ref);
 
@@ -1840,14 +1781,89 @@ pcb_bool exec_pstk_end(parse_param * h)
 	return 0;
 }
 
-/*
- * draw padstack. Used when drawing vias, pins and pads.
- * ref is an optional string which gives pin reference as device.pin, eg. U1.VCC
- */
-
-void hyp_draw_pstk(padstack_t * padstk, pcb_coord_t x, pcb_coord_t y, char *ref)
+static int hyp_pstk_add_shp(pcb_pstk_shape_t *sh, int *used, int max, padstack_element_t *i, pcb_layer_type_t lyt, pcb_layer_combining_t comb, pcb_coord_t offs)
 {
+	if (*used >= max)
+		return -1;
+	sh[*used].layer_mask = lyt;
+	sh[*used].comb = comb;
+	switch(i->pad_shape) {
+		case 0: /* round */
+		case 2: /* oblong, rounded rectangle - emulate by line */
+			pcb_shape_oval(&sh[*used], i->pad_sx+offs, i->pad_sy+offs);
+			break;
+		case 1: /* rectangle */
+			pcb_shape_rect(&sh[*used], i->pad_sx+offs, i->pad_sy+offs);
+			break;
+		default:
+			return -1;
+	}
+	(*used)++;
+	return 0;
+}
 
+static pcb_pstk_t *hyp_new_pstk(padstack_t *padstk, pcb_data_t *data, pcb_coord_t x, pcb_coord_t y, pcb_bool with_paste, pcb_bool with_mask)
+{
+	pcb_pstk_t *ps;
+	padstack_element_t *i;
+	pcb_pstk_shape_t sh[8];
+	int sh_max = sizeof(sh) / sizeof(sh[0]) - 1; /* leave one entry for terminating */
+	int sh_used = 0, have_top = 0, have_bottom = 0, have_inner = 0;
+	const char *ln_top, *ln_bot;
+
+	memset(sh, 0, sizeof(sh));
+
+	ln_top = pcb_layer_name(PCB->Data, top_layer_id);
+	ln_bot = pcb_layer_name(PCB->Data, bottom_layer_id);
+
+	/* loop over all layers of the padstack and create shapes in sh[] */
+	for (i = padstk->padstack; i != NULL; i = i->next) {
+		int mdef, top_or_bottom;
+		if (i->layer_name == NULL)
+			continue;
+		if (i->pad_type != PAD_TYPE_METAL)
+			continue;
+
+		if (i->pad_angle != 0)
+			pcb_message(PCB_MSG_ERROR, "ignoring pad rotation of padstack at %$mm;%$mm.\n", x, y);
+
+		mdef = strcmp(i->layer_name, "MDEF") == 0;
+		top_or_bottom = 0;
+#warning TODO: check if mask/paste layers can be acquired explicitly as non-metal layers
+		if (mdef || (strcmp(i->layer_name, ln_top) == 0)) {
+			if (!have_top) {
+				hyp_pstk_add_shp(sh, &sh_used, sh_max, i, PCB_LYT_TOP | PCB_LYT_COPPER, 0, 0);
+				if (with_mask) hyp_pstk_add_shp(sh, &sh_used, sh_max, i, PCB_LYT_TOP | PCB_LYT_MASK, PCB_LYC_SUB | PCB_LYC_AUTO, MASK_OFFS);
+				if (with_paste) hyp_pstk_add_shp(sh, &sh_used, sh_max, i, PCB_LYT_TOP | PCB_LYT_PASTE, PCB_LYC_AUTO, PASTE_OFFS);
+			}
+			top_or_bottom = 1;
+			have_top = 1;
+		}
+		if (mdef || (strcmp(i->layer_name, ln_bot) == 0)) {
+			if (!have_bottom) {
+				hyp_pstk_add_shp(sh, &sh_used, sh_max, i, PCB_LYT_BOTTOM | PCB_LYT_COPPER, 0, 0);
+				if (with_mask) hyp_pstk_add_shp(sh, &sh_used, sh_max, i, PCB_LYT_BOTTOM | PCB_LYT_MASK, PCB_LYC_SUB | PCB_LYC_AUTO, MASK_OFFS);
+				if (with_paste) hyp_pstk_add_shp(sh, &sh_used, sh_max, i, PCB_LYT_BOTTOM | PCB_LYT_PASTE, PCB_LYC_AUTO, PASTE_OFFS);
+			}
+			top_or_bottom = 1;
+			have_bottom = 0;
+		}
+		if (!have_inner && (mdef || !top_or_bottom)) {
+			hyp_pstk_add_shp(sh, &sh_used, sh_max, i, PCB_LYT_INTERN | PCB_LYT_COPPER, 0, 0);
+			have_inner = 1;
+		}
+	}
+
+	ps = pcb_pstk_new_from_shape(data, x, y, padstk->drill_size, 1, 0, sh);
+	if (ps == NULL)
+		pcb_message(PCB_MSG_ERROR, "Failed to convert padstack at %$mm;%$mm.\n", x, y);
+	return ps;
+}
+
+/* draw padstack. Used when drawing vias, pins and pads.
+ * ref is an optional string which gives pin reference as device.pin, eg. U1.VCC */
+void hyp_draw_pstk(padstack_t *padstk, pcb_coord_t x, pcb_coord_t y, char *ref)
+{
 /* 
  * We try to map a hyperlynx padstack to its closest pcb-rnd primitive.
  * Choose between pcb_via_new(), pcb_element_pin_new(), pcb_element_pad_new() to draw a padstack.
@@ -1855,24 +1871,13 @@ void hyp_draw_pstk(padstack_t * padstk, pcb_coord_t x, pcb_coord_t y, char *ref)
  * if there is a drill hole and a pin reference, it's a pin. Use pcb_element_pin_new().
  * if there is no drill hole, it's a pad. Use pcb_element_pad_new().
  */
-
-	pcb_element_t *element = NULL;
-	pcb_coord_t x1 = 0;
-	pcb_coord_t y1 = 0;
-	pcb_coord_t x2 = 0;
-	pcb_coord_t y2 = 0;
-	pcb_coord_t thickness = 0;
-	pcb_coord_t clearance = 0;
-	pcb_coord_t mask = 0;
-	pcb_coord_t drillinghole = 0;
+	pcb_subc_t *subc = NULL;
 	char *name = NULL;
 	char *number = NULL;
-	pcb_flag_t flags;
-
 	char *device_name = NULL;
 	char *pin_name = NULL;
-	char *layer_name = NULL;
-	padstack_element_t *i;
+	pcb_pstk_t *pstk;
+	pcb_data_t *data;
 
 	if (padstk == NULL) {
 		if (hyp_debug)
@@ -1880,76 +1885,12 @@ void hyp_draw_pstk(padstack_t * padstk, pcb_coord_t x, pcb_coord_t y, char *ref)
 		return;
 	}
 
-	/* drill size */
-	drillinghole = padstk->drill_size;
-
-	/* pad size and shape */
-	thickness = 0;
-	clearance = 0;
-	mask = 0;
-	flags = pcb_no_flags();
-
-	/* loop over padstack, and choose one entry to implement via. this is suboptimal. 
-	 * order chosen:
-	 * - top layer
-	 * - bottom layer
-	 * - default "MDEF" layer
-	 * - any metal layer 
-	 */
-
-	/* search for top layer */
-	for (i = padstk->padstack; i != NULL; i = i->next) {
-		if (i->layer_name == NULL)
-			continue;
-		if ((strcmp(i->layer_name, pcb_layer_name(PCB->Data, top_layer_id)) == 0) && (i->pad_type != PAD_TYPE_METAL))
-			break;
-	}
-	/* if top layer not found, search for bottom layer */
-	if (i == NULL)
-		for (i = padstk->padstack; i != NULL; i = i->next) {
-			if (i->layer_name == NULL)
-				continue;
-			if ((strcmp(i->layer_name, pcb_layer_name(PCB->Data, bottom_layer_id)) == 0) && (i->pad_type != PAD_TYPE_METAL))
-				break;
-		}
-	/* if bottom layer not found, search for default MDEF layer */
-	if (i == NULL)
-		for (i = padstk->padstack; i != NULL; i = i->next) {
-			if (i->layer_name == NULL)
-				continue;
-			if ((strcmp(i->layer_name, "MDEF") == 0) && (i->pad_type != PAD_TYPE_METAL))
-				break;
-		}
-	/* if default MDEF layer not found, search for any metal layer */
-	if (i == NULL)
-		for (i = padstk->padstack; i != NULL; i = i->next) {
-			if (i->layer_name == NULL)
-				continue;
-			if (i->pad_type == PAD_TYPE_METAL)
-				break;
-		}
-
-	if (i != NULL) {
-		/* layer found */
-		thickness = min(i->pad_sx, i->pad_sy);
-		if (i->pad_shape == 1)
-			flags = pcb_flag_make(PCB_FLAG_SQUARE);	/* rectangular */
-		else if (i->pad_shape == 2)
-			flags = pcb_flag_make(PCB_FLAG_OCTAGON);	/* oblong */
-		else
-			flags = pcb_no_flags();		/* round */
-	}
-
-	mask = thickness;
-
-	if ((drillinghole > 0) && (ref == NULL)) {
-		pcb_via_new(hyp_dest, x, y, thickness, clearance, mask, drillinghole, name, flags);
-		return;
-	}
 
 	/* device and pin name, if any */
 	device_name = NULL;
 	pin_name = NULL;
+	data = hyp_dest;
+
 	if (ref != NULL) {
 		char *dot;
 		/* reference has format 'device_name.pin_name' */
@@ -1972,7 +1913,8 @@ void hyp_draw_pstk(padstack_t * padstk, pcb_coord_t x, pcb_coord_t y, char *ref)
 		}
 
 		/* find device by name */
-		element = hyp_create_element_by_name(device_name, x, y);
+		subc = hyp_create_subc_by_name(device_name, x, y);
+		data = subc->data;
 	}
 
 	/* name and number NULL if no reference given */
@@ -1982,130 +1924,12 @@ void hyp_draw_pstk(padstack_t * padstk, pcb_coord_t x, pcb_coord_t y, char *ref)
 	if (hyp_debug)
 		pcb_message(PCB_MSG_DEBUG, "draw padstack: device_name = \"%s\" pin_name = \"%s\"\n", name, number);
 
-	if ((drillinghole > 0) && (element != NULL)) {
-		/* create */
-		pcb_element_pin_new(element, x, y, thickness, clearance, mask, drillinghole, name, number, flags);
-		/* add pin to current net */
+	pstk = hyp_new_pstk(padstk, data, x, y, (subc != NULL), (subc != NULL));
+	if (pin_name != NULL)
+		pcb_attribute_put(&pstk->Attributes, "term", pin_name);
+
+	if (subc != NULL) /* add pin to current net */
 		hyp_netlist_add(name, number);
-		/* update bounding box */
-		pcb_element_bbox(hyp_dest, element, pcb_font(PCB, 0, 1));
-		return;
-	}
-
-	/* we're now pretty sure it's not a pin or via but a pad. */
-
-	/* layer */
-	layer_name = NULL;
-	x1 = 0;
-	y1 = 0;
-	x2 = 0;
-	y2 = 0;
-
-	/* loop over padstack, find suitable layer, and set pad arguments. */
-	for (i = padstk->padstack; i != NULL; i = i->next) {
-		if (i->layer_name == NULL)
-			continue;
-		if (strcmp(i->layer_name, "ADEF") == 0)
-			continue;									/* skip dummy layer 'ADEF' */
-		if (i->pad_type != PAD_TYPE_METAL)
-			continue;									/* skip antipads and thermal relief */
-
-		x1 = x;
-		y1 = y;
-		x2 = x1;
-		y2 = y1;
-		thickness = 0;
-
-		if (i->pad_sy > i->pad_sx) {
-			pcb_coord_t deltay = (i->pad_sy - i->pad_sx) / 2;
-			thickness = i->pad_sx;
-			x2 = x;
-			y1 = y - deltay;
-			y2 = y + deltay;
-		}
-		else if (i->pad_sx > i->pad_sy) {
-			pcb_coord_t deltax = (i->pad_sx - i->pad_sy) / 2;
-			thickness = i->pad_sy;
-			x1 = x - deltax;
-			x2 = x + deltax;
-			y2 = y;
-		}
-		else {
-			thickness = i->pad_sx;
-			x2 = x;
-			y2 = y;
-		}
-
-		mask = thickness;
-
-		/* rotation */
-		if (i->pad_angle != 0.0) {
-			double angle = i->pad_angle * M_PI / 180.0;
-			pcb_rotate(&x1, &y1, x, y, cos(angle), sin(angle));
-			pcb_rotate(&x2, &y2, x, y, cos(angle), sin(angle));
-		}
-
-		layer_name = pcb_strdup(i->layer_name);
-		if (i->pad_shape == 1)
-			flags = pcb_flag_make(PCB_FLAG_SQUARE);	/* rectangular */
-		else if (i->pad_shape == 2)
-			flags = pcb_flag_make(PCB_FLAG_OCTAGON);	/* oblong */
-		else
-			flags = pcb_no_flags();		/* round */
-	}
-
-	/* set pad "on solder side" flag if needed */
-	if (hyp_is_bottom_layer(layer_name))
-		flags = pcb_flag_add(flags, PCB_FLAG_ONSOLDER);
-
-	/* create dummy element on top or bottom layer for pads without pin reference */
-	if (element == NULL) {
-		if ((layer_name != NULL) && hyp_is_bottom_layer(layer_name)) {
-			if (solder_side_pads == NULL) {
-				solder_side_pads =
-					pcb_element_new(hyp_dest, NULL, pcb_font(PCB, 0, 1), pcb_flag_make(PCB_FLAG_ONSOLDER), "Bottom layer pads",
-													PAD_BOTTOM, NULL, 0, 0, 0, 500, pcb_flag_make(PCB_FLAG_ONSOLDER), pcb_false);
-				PCB_FLAG_TOGGLE(PCB_FLAG_HIDENAME, solder_side_pads);
-			}
-			element = solder_side_pads;
-			name = PAD_BOTTOM;
-		}
-		else {
-			if (component_side_pads == NULL) {
-				component_side_pads =
-					pcb_element_new(hyp_dest, NULL, pcb_font(PCB, 0, 1), pcb_no_flags(), "Top layer pads", PAD_TOP, NULL, 0, 0, 0, 500,
-													pcb_no_flags(), pcb_false);
-				PCB_FLAG_TOGGLE(PCB_FLAG_HIDENAME, component_side_pads);
-			}
-			element = component_side_pads;
-			name = PAD_TOP;
-		}
-	}
-
-	/* check element and pad on same (component/solder) side */
-	if ((element != NULL) && (layer_name != NULL) && (strcmp(layer_name, "MDEF") != 0) && (strcmp(layer_name, "ADEF") != 0)
-			&& (PCB_FLAG_TEST(PCB_FLAG_ONSOLDER, element) != hyp_is_bottom_layer(layer_name)))
-		pcb_message(PCB_MSG_WARNING, "draw padstack: device \"%s\" and pad \"%s\" on different layers. continuing.\n", name, ref);
-
-	/* create pad */
-	if (element != NULL) {
-		/* create pad number */
-		if ((number == NULL) || (strcmp(number, "") == 0)) {
-			number = malloc(MAX_STRING);
-			pcb_sprintf(number, "NONUMBER%0d", unknown_pin_number++);
-		}
-
-		/* create pad */
-		pcb_element_pad_new(element, x1, y1, x2, y2, thickness, clearance, mask, name, number, flags);
-		/* add pad to current net */
-		hyp_netlist_add(name, number);
-		/* update bounding box */
-		pcb_element_bbox(hyp_dest, element, pcb_font(PCB, 0, 1));
-		return;
-	}
-
-	if (hyp_debug)
-		pcb_message(PCB_MSG_DEBUG, "draw padstack: skipped.\n");
 
 	return;
 }
@@ -2897,7 +2721,6 @@ pcb_bool exec_end(parse_param * h)
 {
 	if (hyp_debug)
 		pcb_message(PCB_MSG_DEBUG, "end:\n");
-
 	return 0;
 }
 
