@@ -21,6 +21,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  */
 
 #include "config.h"
@@ -69,9 +73,9 @@ int pplg_init_io_kicad(void)
 	/* register the IO hook */
 	io_kicad.plugin_data = NULL;
 	io_kicad.fmt_support_prio = io_kicad_fmt;
-	io_kicad.test_parse_pcb = io_kicad_test_parse_pcb;
+	io_kicad.test_parse = io_kicad_test_parse;
 	io_kicad.parse_pcb = io_kicad_read_pcb;
-	io_kicad.parse_element = NULL;
+	io_kicad.parse_element = io_kicad_parse_element;
 	io_kicad.parse_font = NULL;
 	io_kicad.write_buffer = io_kicad_write_buffer;
 	io_kicad.write_element = io_kicad_write_element;
