@@ -2,7 +2,7 @@
  *                            COPYRIGHT
  *
  *  pcb-rnd, interactive printed circuit board design
- *  Copyright (C) 2016, 2017 Tibor 'Igor2' Palinkas
+ *  Copyright (C) 2016, 2017, 2018 Tibor 'Igor2' Palinkas
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,6 +18,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  */
 
 #include "config.h"
@@ -40,9 +44,7 @@ int io_lihata_fmt(pcb_plug_io_t *ctx, pcb_plug_iot_t typ, int wr, const char *fm
 	if ((lih) && (typ & PCB_IOT_BUFFER) && (ctx->write_buffer != NULL))
 		return 40;
 
-/*PCB_IOT_FOOTPRINT |*/
-
-	if (!lih || ((typ & (~(PCB_IOT_PCB | PCB_IOT_FONT))) != 0))
+	if (!lih || ((typ & (~(PCB_IOT_PCB | PCB_IOT_FONT | PCB_IOT_FOOTPRINT))) != 0))
 		return 0;
 
 	if (wr)
@@ -63,7 +65,7 @@ int pplg_init_io_lihata(void)
 	/* register the IO hook */
 	plug_io_lihata_v4.plugin_data = NULL;
 	plug_io_lihata_v4.fmt_support_prio = io_lihata_fmt;
-	plug_io_lihata_v4.test_parse_pcb = io_lihata_test_parse_pcb;
+	plug_io_lihata_v4.test_parse = io_lihata_test_parse;
 	plug_io_lihata_v4.parse_pcb = io_lihata_parse_pcb;
 	plug_io_lihata_v4.parse_element = io_lihata_parse_element;
 	plug_io_lihata_v4.parse_font = io_lihata_parse_font;
@@ -82,7 +84,7 @@ int pplg_init_io_lihata(void)
 
 	plug_io_lihata_v3.plugin_data = NULL;
 	plug_io_lihata_v3.fmt_support_prio = io_lihata_fmt;
-	plug_io_lihata_v3.test_parse_pcb = io_lihata_test_parse_pcb;
+	plug_io_lihata_v3.test_parse = io_lihata_test_parse;
 	plug_io_lihata_v3.parse_pcb = io_lihata_parse_pcb;
 	plug_io_lihata_v3.parse_element = io_lihata_parse_element;
 	plug_io_lihata_v3.parse_font = io_lihata_parse_font;
@@ -101,7 +103,7 @@ int pplg_init_io_lihata(void)
 
 	plug_io_lihata_v2.plugin_data = NULL;
 	plug_io_lihata_v2.fmt_support_prio = io_lihata_fmt;
-	plug_io_lihata_v2.test_parse_pcb = io_lihata_test_parse_pcb;
+	plug_io_lihata_v2.test_parse = io_lihata_test_parse;
 	plug_io_lihata_v2.parse_pcb = io_lihata_parse_pcb;
 	plug_io_lihata_v2.parse_element = NULL;
 	plug_io_lihata_v2.parse_font = io_lihata_parse_font;
@@ -120,7 +122,7 @@ int pplg_init_io_lihata(void)
 
 	plug_io_lihata_v1.plugin_data = NULL;
 	plug_io_lihata_v1.fmt_support_prio = io_lihata_fmt;
-	plug_io_lihata_v1.test_parse_pcb = io_lihata_test_parse_pcb;
+	plug_io_lihata_v1.test_parse = io_lihata_test_parse;
 	plug_io_lihata_v1.parse_pcb = io_lihata_parse_pcb;
 	plug_io_lihata_v1.parse_element = NULL;
 	plug_io_lihata_v1.parse_font = io_lihata_parse_font;
