@@ -2,7 +2,7 @@
  *                            COPYRIGHT
  *
  *  pcb-rnd, interactive printed circuit board design
- *  Copyright (C) 2016, 2017 Tibor 'Igor2' Palinkas
+ *  Copyright (C) 2016, 2017, 2018 Tibor 'Igor2' Palinkas
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,6 +18,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  */
 
 /* Load a lihata document in-memory and walk the tree and build pcb native
@@ -53,8 +57,12 @@
 #include "plug_footprint.h"
 #include "vtpadstack.h"
 
+#include "../src_plugins/lib_compat_help/subc_help.h"
+#include "../src_plugins/lib_compat_help/pstk_compat.h"
+#include "../src_plugins/lib_compat_help/elem_rot.h"
+
 #warning cleanup TODO: put these in a gloal load-context-struct
-vtp0_t post_ids, post_thermal;
+vtp0_t post_ids, post_thermal_old, post_thermal_heavy;
 static int rdver;
 
 static pcb_data_t *parse_data(pcb_board_t *pcb, pcb_data_t *dst, lht_node_t *nd, pcb_data_t *subc_parent);
@@ -325,77 +333,6 @@ static int parse_meta(pcb_board_t *pcb, lht_node_t *nd)
 	return 0;
 }
 
-/* pt is a list of lihata node pointers to thermal nodes; each has user
-   data set to the flag. Look up layer info and fill in thermal flags. This
-   needs to be done in a separate pass at the end of parsing because
-   vias may precede layers in the lihata input file. */
-static int post_thermal_assign(pcb_board_t *pcb, vtp0_t *pt)
-{
-	int i;
-
-	for(i = 0; i < vtp0_len(pt); i++) {
-		lht_node_t *n;
-		lht_dom_iterator_t it;
-		io_lihata_flag_holder fh;
-		lht_node_t *thr = pt->array[i];
-		pcb_flag_t *f = thr->user_data;
-
-		memset(&fh, 0, sizeof(fh));
-		fh.Flags = *f;
-		for(n = lht_dom_first(&it, thr); n != NULL; n = lht_dom_next(&it)) {
-			if (n->type == LHT_TEXT) {
-				int layer = pcb_layer_by_name(pcb->Data, n->name);
-				if (layer < 0) {
-					pcb_message(PCB_MSG_ERROR, "#LHT10 Invalid layer name in thermal: '%s'\n", n->name);
-					return -1;
-				}
-				PCB_FLAG_THERM_ASSIGN(layer, io_lihata_resolve_thermal_style(n->data.text.value), &fh);
-			}
-		}
-		*f = fh.Flags;
-	}
-	vtp0_uninit(pt);
-	return 0;
-}
-
-/* NOTE: in case of objects with thermal, f must point to the object's
-   flags because termals will be filled in at the end, in a 2nd pass and
-   we need to store the f pointer. */
-static int parse_flags(pcb_flag_t *f, lht_node_t *fn, int object_type, unsigned char *intconn)
-{
-	io_lihata_flag_holder fh;
-
-	memset(&fh, 0, sizeof(fh));
-
-	if (fn != NULL) {
-		int n;
-		lht_node_t *thr;
-		for (n = 0; n < pcb_object_flagbits_len; n++) {
-			if (pcb_object_flagbits[n].object_types & object_type) {
-				pcb_bool b;
-				if ((parse_bool(&b, lht_dom_hash_get(fn, pcb_object_flagbits[n].name)) == 0) && b)
-					PCB_FLAG_SET(pcb_object_flagbits[n].mask, &fh);
-			}
-		}
-
-		thr = lht_dom_hash_get(fn, "thermal");
-		if (thr != NULL) {
-			thr->user_data = f;
-			vtp0_append(&post_thermal, thr);
-		}
-
-		if (parse_int(&n, lht_dom_hash_get(fn, "shape")) == 0)
-			fh.Flags.q = n;
-
-		if ((intconn != NULL) && (rdver < 3))
-			if (parse_int(&n, lht_dom_hash_get(fn, "intconn")) == 0)
-				*intconn = n;
-	}
-
-	*f = fh.Flags;
-	return 0;
-}
-
 static int parse_thermal(unsigned char *dst, lht_node_t *src)
 {
 	if (src == NULL)
@@ -412,6 +349,125 @@ static int parse_thermal(unsigned char *dst, lht_node_t *src)
 	return 0;
 }
 
+/* pt is a list of lihata node pointers to thermal nodes; each has user
+   data set to the object. Look up layer info and build the thermal. This
+   needs to be done in a separate pass at the end of parsing because
+   vias may precede layers in the lihata input file. */
+static int post_thermal_assign(pcb_board_t *pcb, vtp0_t *old, vtp0_t *heavy)
+{
+	int i;
+	lht_node_t *n;
+	lht_dom_iterator_t it;
+
+	/* pin/via before lihata v4: thermal is part of the object flag*/
+	for(i = 0; i < vtp0_len(old); i++) {
+		lht_node_t *thr = old->array[i];
+		pcb_pstk_t *ps = thr->user_data;
+
+		assert(ps->type == PCB_OBJ_PSTK);
+
+		ps->thermals.used = 0;
+
+		for(n = lht_dom_first(&it, thr); n != NULL; n = lht_dom_next(&it)) {
+			if (n->type == LHT_TEXT) {
+				int layer = pcb_layer_by_name(pcb->Data, n->name) + 1;
+				if (layer > ps->thermals.used)
+					ps->thermals.used = layer;
+			}
+		}
+
+		if (ps->thermals.used > 0) {
+			ps->thermals.shape = calloc(sizeof(ps->thermals.shape[0]), ps->thermals.used);
+			for(n = lht_dom_first(&it, thr); n != NULL; n = lht_dom_next(&it)) {
+				if (n->type == LHT_TEXT) {
+					int layer = pcb_layer_by_name(pcb->Data, n->name);
+					if (layer < 0) {
+						pcb_message(PCB_MSG_ERROR, "#LHT10o Invalid layer name in thermal: '%s'\n", n->name);
+						return -1;
+					}
+					ps->thermals.shape[layer] = io_lihata_resolve_thermal_style_old(n->data.text.value);
+				}
+			}
+		}
+		else
+			ps->thermals.shape = NULL;
+	}
+	vtp0_uninit(old);
+
+	/* from lihata v4 up: thermal is an object property (on heavy terminal layer objects) */
+	for(i = 0; i < vtp0_len(heavy); i++) {
+		lht_node_t *thr = heavy->array[i];
+		pcb_any_obj_t *obj = thr->user_data;
+
+		/* single character thermal: only on the layer the object is on */
+		parse_thermal(&obj->thermal, thr);
+	}
+	vtp0_uninit(heavy);
+
+	return 0;
+}
+
+static int parse_flags(pcb_flag_t *f, lht_node_t *fn, int object_type, unsigned char *intconn, int can_have_thermal)
+{
+	io_lihata_flag_holder fh;
+
+	memset(&fh, 0, sizeof(fh));
+
+	if (fn != NULL) {
+		int n;
+		for (n = 0; n < pcb_object_flagbits_len; n++) {
+			if (pcb_object_flagbits[n].object_types & object_type) {
+				pcb_bool b;
+				if ((parse_bool(&b, lht_dom_hash_get(fn, pcb_object_flagbits[n].name)) == 0) && b)
+					PCB_FLAG_SET(pcb_object_flagbits[n].mask, &fh);
+			}
+		}
+
+		if ((!can_have_thermal) && (lht_dom_hash_get(fn, "thermal") != NULL)) {
+			pcb_message(PCB_MSG_ERROR, "#LHT10o Invalid flag thermal: object type can not have a thermal (ignored)\n");
+		}
+
+		if (parse_int(&n, lht_dom_hash_get(fn, "shape")) == 0)
+			fh.Flags.q = n;
+
+		if ((intconn != NULL) && (rdver < 3))
+			if (parse_int(&n, lht_dom_hash_get(fn, "intconn")) == 0)
+				*intconn = n;
+	}
+
+	*f = fh.Flags;
+	return 0;
+}
+
+static void parse_thermal_old(pcb_any_obj_t *obj, lht_node_t *fn)
+{
+	lht_node_t *thr;
+
+	if (fn == NULL)
+		return;
+
+	thr = lht_dom_hash_get(fn, "thermal");
+	if (thr == NULL)
+		return;
+
+	thr->user_data = obj;
+	vtp0_append(&post_thermal_old, thr);
+}
+
+/* heavy terminal thermal: save for later processing */
+static int parse_thermal_heavy(pcb_any_obj_t *obj, lht_node_t *src)
+{
+	if (src == NULL)
+		return 0;
+
+	if (src->type != LHT_LIST)
+		return -1;
+
+	src->user_data = obj;
+	vtp0_append(&post_thermal_heavy, src);
+
+	return 0;
+}
 
 static int parse_line(pcb_layer_t *ly, pcb_element_t *el, lht_node_t *obj, int no_id, pcb_coord_t dx, pcb_coord_t dy)
 {
@@ -429,12 +485,12 @@ static int parse_line(pcb_layer_t *ly, pcb_element_t *el, lht_node_t *obj, int n
 		line->ID = 0;
 	else
 		parse_id(&line->ID, obj, 5);
-	parse_flags(&line->Flags, lht_dom_hash_get(obj, "flags"), PCB_TYPE_LINE, &intconn);
+	parse_flags(&line->Flags, lht_dom_hash_get(obj, "flags"), PCB_TYPE_LINE, &intconn, 0);
 	pcb_attrib_compat_set_intconn(&line->Attributes, intconn);
 	parse_attributes(&line->Attributes, lht_dom_hash_get(obj, "attributes"));
 
 	if (rdver >= 4)
-		parse_thermal(&line->thermal, lht_dom_hash_get(obj, "thermal"));
+		parse_thermal_heavy((pcb_any_obj_t *)line, lht_dom_hash_get(obj, "thermal"));
 
 	parse_coord(&line->Thickness, lht_dom_hash_get(obj, "thickness"));
 	parse_coord(&line->Clearance, lht_dom_hash_get(obj, "clearance"));
@@ -465,7 +521,7 @@ static int parse_rat(pcb_data_t *dt, lht_node_t *obj)
 	int tmp;
 
 	parse_id(&rat.ID, obj, 4);
-	parse_flags(&rat.Flags, lht_dom_hash_get(obj, "flags"), PCB_TYPE_LINE, NULL);
+	parse_flags(&rat.Flags, lht_dom_hash_get(obj, "flags"), PCB_TYPE_LINE, NULL, 0);
 	parse_attributes(&rat.Attributes, lht_dom_hash_get(obj, "attributes"));
 
 	parse_coord(&rat.Point1.X, lht_dom_hash_get(obj, "x1"));
@@ -478,12 +534,13 @@ static int parse_rat(pcb_data_t *dt, lht_node_t *obj)
 	parse_int(&tmp, lht_dom_hash_get(obj, "lgrp2"));
 	rat.group2 = tmp;
 
-	post_id_req(&rat.Point1);
-	post_id_req(&rat.Point2);
 
 	new_rat = pcb_rat_new(dt, rat.Point1.X, rat.Point1.Y, rat.Point2.X, rat.Point2.Y, rat.group1, rat.group2,
 		conf_core.appearance.rat_thickness, rat.Flags);
 
+	post_id_req(&new_rat->Point1);
+	post_id_req(&new_rat->Point2);
+
 	new_rat->ID = rat.ID;
 
 	return 0;
@@ -502,12 +559,12 @@ static int parse_arc(pcb_layer_t *ly, pcb_element_t *el, lht_node_t *obj, pcb_co
 		return -1;
 
 	parse_id(&arc->ID, obj, 4);
-	parse_flags(&arc->Flags, lht_dom_hash_get(obj, "flags"), PCB_TYPE_ARC, &intconn);
+	parse_flags(&arc->Flags, lht_dom_hash_get(obj, "flags"), PCB_TYPE_ARC, &intconn, 0);
 	pcb_attrib_compat_set_intconn(&arc->Attributes, intconn);
 	parse_attributes(&arc->Attributes, lht_dom_hash_get(obj, "attributes"));
 
 	if (rdver >= 4)
-		parse_thermal(&arc->thermal, lht_dom_hash_get(obj, "thermal"));
+		parse_thermal_heavy((pcb_any_obj_t *)arc, lht_dom_hash_get(obj, "thermal"));
 
 	parse_coord(&arc->Thickness, lht_dom_hash_get(obj, "thickness"));
 	parse_coord(&arc->Clearance, lht_dom_hash_get(obj, "clearance"));
@@ -536,7 +593,7 @@ static int parse_polygon(pcb_layer_t *ly, pcb_element_t *el, lht_node_t *obj)
 	unsigned char intconn = 0;
 
 	parse_id(&poly->ID, obj, 8);
-	parse_flags(&poly->Flags, lht_dom_hash_get(obj, "flags"), PCB_TYPE_POLY, &intconn);
+	parse_flags(&poly->Flags, lht_dom_hash_get(obj, "flags"), PCB_TYPE_POLY, &intconn, 0);
 	pcb_attrib_compat_set_intconn(&poly->Attributes, intconn);
 	parse_attributes(&poly->Attributes, lht_dom_hash_get(obj, "attributes"));
 
@@ -544,7 +601,7 @@ static int parse_polygon(pcb_layer_t *ly, pcb_element_t *el, lht_node_t *obj)
 		parse_coord(&poly->Clearance, lht_dom_hash_get(obj, "clearance"));
 
 	if (rdver >= 4)
-		parse_thermal(&poly->thermal, lht_dom_hash_get(obj, "thermal"));
+		parse_thermal_heavy((pcb_any_obj_t *)poly, lht_dom_hash_get(obj, "thermal"));
 
 	geo = lht_dom_hash_get(obj, "geometry");
 	if ((geo != NULL) && (geo->type == LHT_LIST)) {
@@ -616,7 +673,7 @@ static int parse_pcb_text(pcb_layer_t *ly, pcb_element_t *el, lht_node_t *obj)
 
 	parse_id(&text->ID, obj, 5);
 
-	parse_flags(&text->Flags, lht_dom_hash_get(obj, "flags"), PCB_TYPE_TEXT, &intconn);
+	parse_flags(&text->Flags, lht_dom_hash_get(obj, "flags"), PCB_TYPE_TEXT, &intconn, 0);
 	pcb_attrib_compat_set_intconn(&text->Attributes, intconn);
 	parse_attributes(&text->Attributes, lht_dom_hash_get(obj, "attributes"));
 	parse_int(&text->Scale, lht_dom_hash_get(obj, "scale"));
@@ -768,7 +825,7 @@ static int parse_pstk(pcb_data_t *dt, lht_node_t *obj)
 	ps = pcb_pstk_alloc(dt);
 
 	parse_id(&ps->ID, obj, 13);
-	parse_flags(&ps->Flags, lht_dom_hash_get(obj, "flags"), PCB_TYPE_PSTK, &intconn);
+	parse_flags(&ps->Flags, lht_dom_hash_get(obj, "flags"), PCB_TYPE_PSTK, &intconn, 0);
 	pcb_attrib_compat_set_intconn(&ps->Attributes, intconn);
 	parse_attributes(&ps->Attributes, lht_dom_hash_get(obj, "attributes"));
 
@@ -788,10 +845,22 @@ static int parse_pstk(pcb_data_t *dt, lht_node_t *obj)
 
 	thl = lht_dom_hash_get(obj, "thermal");
 	if ((thl != NULL) && (thl->type == LHT_LIST)) {
-		int n;
-		for(t = thl->data.list.first, n = 0; t != NULL; t = t->next) n++;
-		ps->thermals.used = n;
-		ps->thermals.shape = malloc(sizeof(ps->thermals.shape[0]) * n);
+		int max, n;
+		max = 0;
+		for(t = thl->data.list.first, n = 0; t != NULL; t = t->next,n++)
+		{
+			if (t->type == LHT_LIST) {
+				char *end;
+				int ly = strtol(t->name, &end, 10);
+				if ((*end == '\0') && (ly > max))
+					max = ly;
+			}
+		}
+		if (n > max)
+			max = n;
+
+		ps->thermals.used = max+1;
+		ps->thermals.shape = calloc(sizeof(ps->thermals.shape[0]), ps->thermals.used);
 		for(t = thl->data.list.first, n = 0; t != NULL; t = t->next, n++) {
 			int i;
 			if (t->type == LHT_TEXT) {
@@ -800,88 +869,98 @@ static int parse_pstk(pcb_data_t *dt, lht_node_t *obj)
 			}
 			else if (t->type == LHT_LIST) {
 				unsigned char dst;
-				parse_thermal(&dst, t);
-				ps->thermals.shape[n] = dst;
+				char *end;
+				int ly = strtol(t->name, &end, 10);
+				if ((*end == '\0') && (ly < ps->thermals.used)) {
+					parse_thermal(&dst, t);
+					ps->thermals.shape[ly] = dst;
+				}
 			}
 		}
 	}
 
-
-
 	pcb_pstk_add(dt, ps);
 
 	return 0;
 }
 
-
-/* If el == NULL and dt != NULL it is a via (for now). */
-static int parse_pin(pcb_data_t *dt, pcb_element_t *el, lht_node_t *obj, pcb_coord_t dx, pcb_coord_t dy)
+static int parse_via(pcb_data_t *dt, lht_node_t *obj, pcb_coord_t dx, pcb_coord_t dy, int subc_on_bottom)
 {
-	pcb_pin_t *via;
+	pcb_pstk_t *ps;
 	unsigned char intconn = 0;
+	pcb_coord_t Thickness, Clearance, Mask, DrillingHole, X, Y;
+	char *Name = NULL, *Number = NULL;
+	pcb_flag_t flg;
+	lht_node_t *fln;
 
-	if (dt != NULL)
-		via = pcb_via_alloc(dt);
-	else if (el != NULL)
-		via = pcb_pin_alloc(el);
-	else
+	if (dt == NULL)
 		return -1;
 
-	parse_id(&via->ID, obj, 4);
-	parse_flags(&via->Flags, lht_dom_hash_get(obj, "flags"), PCB_TYPE_VIA, &intconn);
-	pcb_attrib_compat_set_intconn(&via->Attributes, intconn);
-	parse_attributes(&via->Attributes, lht_dom_hash_get(obj, "attributes"));
+	parse_flags(&flg, fln=lht_dom_hash_get(obj, "flags"), PCB_TYPE_VIA, &intconn, 1);
+	parse_coord(&Thickness, lht_dom_hash_get(obj, "thickness"));
+	parse_coord(&Clearance, lht_dom_hash_get(obj, "clearance"));
+	parse_coord(&Mask, lht_dom_hash_get(obj, "mask"));
+	parse_coord(&DrillingHole, lht_dom_hash_get(obj, "hole"));
+	parse_coord(&X, lht_dom_hash_get(obj, "x"));
+	parse_coord(&Y, lht_dom_hash_get(obj, "y"));
+	parse_text(&Name, lht_dom_hash_get(obj, "name"));
+	parse_text(&Number, lht_dom_hash_get(obj, "number"));
 
-	parse_coord(&via->Thickness, lht_dom_hash_get(obj, "thickness"));
-	parse_coord(&via->Clearance, lht_dom_hash_get(obj, "clearance"));
-	parse_coord(&via->Mask, lht_dom_hash_get(obj, "mask"));
-	parse_coord(&via->DrillingHole, lht_dom_hash_get(obj, "hole"));
-	parse_coord(&via->X, lht_dom_hash_get(obj, "x"));
-	parse_coord(&via->Y, lht_dom_hash_get(obj, "y"));
-	parse_text(&via->Name, lht_dom_hash_get(obj, "name"));
-	parse_text(&via->Number, lht_dom_hash_get(obj, "number"));
+	ps = pcb_old_via_new(dt, X+dx, Y+dy, Thickness, Clearance, Mask, DrillingHole, Name, flg);
+	if (ps == NULL) {
+		pcb_message(PCB_MSG_ERROR, "Failed to convert old via to padstack\n");
+		return 0;
+	}
 
-	via->X += dx;
-	via->Y += dy;
+	parse_id(&ps->ID, obj, 4);
+	pcb_attrib_compat_set_intconn(&ps->Attributes, intconn);
+	parse_attributes(&ps->Attributes, lht_dom_hash_get(obj, "attributes"));
 
-	if (dt != NULL)
-		pcb_add_via(dt, via);
-	if (el != NULL)
-		via->Element = el;
+	parse_thermal_old((pcb_any_obj_t *)ps, fln);
+
+	if (Number != NULL)
+		pcb_attribute_put(&ps->Attributes, "term", Number);
+	if (Name != NULL)
+		pcb_attribute_put(&ps->Attributes, "name", Name);
+
+	if (subc_on_bottom)
+		pcb_pstk_mirror(ps, 0, 1);
 
 	return 0;
 }
 
-static int parse_pad(pcb_element_t *el, lht_node_t *obj, pcb_coord_t dx, pcb_coord_t dy)
+static int parse_pad(pcb_subc_t *subc, lht_node_t *obj, pcb_coord_t dx, pcb_coord_t dy, int subc_on_bottom)
 {
-	pcb_pad_t *pad;
+	pcb_pstk_t *p;
 	unsigned char intconn = 0;
+	pcb_flag_t flg;
+	pcb_coord_t X1, Y1, X2, Y2, Thickness, Clearance, Mask;
+	char *Name = NULL, *Number = NULL;
 
-	pad = pcb_pad_alloc(el);
+	parse_flags(&flg, lht_dom_hash_get(obj, "flags"), PCB_TYPE_PAD, &intconn, 0);
 
-	parse_id(&pad->ID, obj, 4);
-	parse_flags(&pad->Flags, lht_dom_hash_get(obj, "flags"), PCB_TYPE_PAD, &intconn);
-	pcb_attrib_compat_set_intconn(&pad->Attributes, intconn);
-	parse_attributes(&pad->Attributes, lht_dom_hash_get(obj, "attributes"));
+	parse_coord(&Thickness, lht_dom_hash_get(obj, "thickness"));
+	parse_coord(&Clearance, lht_dom_hash_get(obj, "clearance"));
+	parse_coord(&Mask, lht_dom_hash_get(obj, "mask"));
+	parse_coord(&X1, lht_dom_hash_get(obj, "x1"));
+	parse_coord(&Y1, lht_dom_hash_get(obj, "y1"));
+	parse_coord(&X2, lht_dom_hash_get(obj, "x2"));
+	parse_coord(&Y2, lht_dom_hash_get(obj, "y2"));
+	parse_text(&Name, lht_dom_hash_get(obj, "name"));
+	parse_text(&Number, lht_dom_hash_get(obj, "number"));
 
-	parse_coord(&pad->Thickness, lht_dom_hash_get(obj, "thickness"));
-	parse_coord(&pad->Clearance, lht_dom_hash_get(obj, "clearance"));
-	parse_coord(&pad->Mask, lht_dom_hash_get(obj, "mask"));
-	parse_coord(&pad->Point1.X, lht_dom_hash_get(obj, "x1"));
-	parse_coord(&pad->Point1.Y, lht_dom_hash_get(obj, "y1"));
-	parse_coord(&pad->Point2.X, lht_dom_hash_get(obj, "x2"));
-	parse_coord(&pad->Point2.Y, lht_dom_hash_get(obj, "y2"));
-	parse_text(&pad->Name, lht_dom_hash_get(obj, "name"));
-	parse_text(&pad->Number, lht_dom_hash_get(obj, "number"));
+	p = pcb_pstk_new_compat_pad(subc->data, X1+dx, Y1+dy, X2+dx, Y2+dy, Thickness, Clearance, Mask, flg.f & PCB_FLAG_SQUARE, flg.f & PCB_FLAG_NOPASTE, (!!(flg.f & PCB_FLAG_ONSOLDER)) != subc_on_bottom);
+	if (Number != NULL)
+		pcb_attribute_put(&p->Attributes, "term", Number);
+	if (Name != NULL)
+		pcb_attribute_put(&p->Attributes, "name", Name);
 
-	pad->Point1.X += dx;
-	pad->Point2.X += dx;
-	pad->Point1.Y += dy;
-	pad->Point2.Y += dy;
+	if (subc_on_bottom)
+		pcb_pstk_mirror(p, 0, 1);
 
-	post_id_req(&pad->Point1);
-	post_id_req(&pad->Point2);
-	pad->Element = el;
+	parse_id(&p->ID, obj, 4);
+	pcb_attrib_compat_set_intconn(&p->Attributes, intconn);
+	parse_attributes(&p->Attributes, lht_dom_hash_get(obj, "attributes"));
 
 	return 0;
 }
@@ -889,47 +968,80 @@ static int parse_pad(pcb_element_t *el, lht_node_t *obj, pcb_coord_t dx, pcb_coo
 
 static int parse_element(pcb_board_t *pcb, pcb_data_t *dt, lht_node_t *obj)
 {
-	pcb_element_t *elem = pcb_element_alloc(dt);
+	pcb_subc_t *subc = pcb_subc_alloc();
+	pcb_layer_t *silk = NULL;
 	lht_node_t *lst, *n;
 	lht_dom_iterator_t it;
-	int onsld;
+	int onsld, tdir = 0, tscale = 100;
+	pcb_coord_t ox = 0, oy = 0, tx, ty;
+	pcb_text_t *txt;
 
-	parse_id(&elem->ID, obj, 8);
-	parse_flags(&elem->Flags, lht_dom_hash_get(obj, "flags"), PCB_TYPE_ELEMENT, NULL);
-	parse_attributes(&elem->Attributes, lht_dom_hash_get(obj, "attributes"));
-	parse_coord(&elem->MarkX, lht_dom_hash_get(obj, "x"));
-	parse_coord(&elem->MarkY, lht_dom_hash_get(obj, "y"));
+	pcb_add_subc_to_data(dt, subc);
 
-	onsld = PCB_FLAG_TEST(PCB_FLAG_ONSOLDER, elem);
+	parse_id(&subc->ID, obj, 8);
+	parse_flags(&subc->Flags, lht_dom_hash_get(obj, "flags"), PCB_TYPE_ELEMENT, NULL, 0);
+	parse_attributes(&subc->Attributes, lht_dom_hash_get(obj, "attributes"));
+	parse_coord(&ox, lht_dom_hash_get(obj, "x"));
+	parse_coord(&oy, lht_dom_hash_get(obj, "y"));
+	tx = ox;
+	ty = oy;
+
+	onsld = PCB_FLAG_TEST(PCB_FLAG_ONSOLDER, subc);
+	subc->Flags.f &= ~PCB_FLAG_ONSOLDER;
+
+	/* bind the via rtree so that vias added in this subc show up on the board */
+	if (pcb != NULL)
+		pcb_subc_bind_globals(pcb, subc);
+
+	/* the only layer objects are put on from and old element is the primary silk layer */
+	{
+		pcb_layer_type_t silk_side = onsld ? PCB_LYT_BOTTOM : PCB_LYT_TOP;
+		const char *name = onsld ? "bottom-silk" : "top-silk";
+		silk = pcb_subc_get_layer(subc, PCB_LYT_SILK | silk_side, /*PCB_LYC_AUTO*/0, pcb_true, name, pcb_false);
+	}
 
 	lst = lht_dom_hash_get(obj, "objects");
 	if (lst->type == LHT_LIST) {
 		for(n = lht_dom_first(&it, lst); n != NULL; n = lht_dom_next(&it)) {
 			if (strncmp(n->name, "line.", 5) == 0)
-				parse_line(NULL, elem, n, 0, elem->MarkX, elem->MarkY);
+				parse_line(silk, NULL, n, 0, ox, oy);
 			if (strncmp(n->name, "arc.", 4) == 0)
-				parse_arc(NULL, elem, n, elem->MarkX, elem->MarkY);
-/*		if (strncmp(n->name, "polygon.", 8) == 0)
-			parse_polygon(ly, elem, n);*/
-			if (strncmp(n->name, "text.", 5) == 0)
-				parse_pcb_text(NULL, elem, n);
+				parse_arc(silk, NULL, n, ox, oy);
+			if (strncmp(n->name, "text.", 5) == 0) {
+				lht_node_t *role   = lht_dom_hash_get(n, "role");
+				lht_node_t *string = lht_dom_hash_get(n, "string");
+				if ((role != NULL) && (role->type == LHT_TEXT) && (string != NULL) && (string->type == LHT_TEXT)) {
+					const char *key = role->data.text.value;
+					const char *val = string->data.text.value;
+					if (strcmp(key, "desc") == 0)   pcb_attribute_put(&subc->Attributes, "footprint", val);
+					if (strcmp(key, "value") == 0)  pcb_attribute_put(&subc->Attributes, "value", val);
+					if (strcmp(key, "name") == 0)   pcb_attribute_put(&subc->Attributes, "refdes", val);
+					parse_coord(&tx, lht_dom_hash_get(n, "x"));
+					parse_coord(&ty, lht_dom_hash_get(n, "y"));
+					parse_coord(&tdir, lht_dom_hash_get(n, "direction"));
+					parse_coord(&tscale, lht_dom_hash_get(n, "scale"));
+				}
+			}
 			if (strncmp(n->name, "pin.", 4) == 0)
-				parse_pin(NULL, elem, n, elem->MarkX, elem->MarkY);
+				parse_via(subc->data, n, ox, oy, onsld);
 			if (strncmp(n->name, "pad.", 4) == 0)
-				parse_pad(elem, n, elem->MarkX, elem->MarkY);
+				parse_pad(subc, n, ox, oy, onsld);
 		}
 	}
 
-	if (onsld) {
-		int n;
-		for(n = 0; n < PCB_MAX_ELEMENTNAMES; n++)
-			PCB_FLAG_SET(PCB_FLAG_ONSOLDER, &elem->Name[n]);
-	}
+#warning subc TODO: TextFlags
+	txt = pcb_subc_add_refdes_text(subc, tx, ty, tdir, tscale, onsld);
+
+	pcb_subc_xy_rot_pnp(subc, ox, oy, onsld);
+
+	pcb_subc_bbox(subc);
+
+	if (dt->subc_tree == NULL)
+		dt->subc_tree = pcb_r_create_tree();
+	pcb_r_insert_entry(dt->subc_tree, (pcb_box_t *)subc);
+
+	pcb_subc_rebind(pcb, subc);
 
-	/* Make sure we use some sort of font */
-	if (pcb == NULL)
-		pcb = PCB;
-	pcb_element_bbox(dt, elem, pcb_font(PCB, 0, 1));
 	return 0;
 }
 
@@ -940,17 +1052,17 @@ static int parse_subc(pcb_board_t *pcb, pcb_data_t *dt, lht_node_t *obj, pcb_sub
 	int n;
 
 	parse_id(&sc->ID, obj, 5);
-	parse_flags(&sc->Flags, lht_dom_hash_get(obj, "flags"), PCB_TYPE_ELEMENT, &intconn);
+	parse_flags(&sc->Flags, lht_dom_hash_get(obj, "flags"), PCB_TYPE_ELEMENT, &intconn, 0);
 	pcb_attrib_compat_set_intconn(&sc->Attributes, intconn);
 	parse_attributes(&sc->Attributes, lht_dom_hash_get(obj, "attributes"));
 	parse_minuid(sc->uid, lht_dom_hash_get(obj, "uid"));
 
 	if (!dt->via_tree)
-		dt->via_tree = pcb_r_create_tree(NULL, 0, 0);
+		dt->via_tree = pcb_r_create_tree();
 	sc->data->via_tree = dt->via_tree;
 
 	if (!dt->padstack_tree)
-		dt->padstack_tree = pcb_r_create_tree(NULL, 0, 0);
+		dt->padstack_tree = pcb_r_create_tree();
 	sc->data->padstack_tree = dt->padstack_tree;
 
 	pcb_add_subc_to_data(dt, sc);
@@ -963,8 +1075,8 @@ static int parse_subc(pcb_board_t *pcb, pcb_data_t *dt, lht_node_t *obj, pcb_sub
 	pcb_data_bbox(&sc->BoundingBox, sc->data, pcb_true);
 
 	if (!dt->subc_tree)
-		dt->subc_tree = pcb_r_create_tree(NULL, 0, 0);
-	pcb_r_insert_entry(dt->subc_tree, (pcb_box_t *)sc, 0);
+		dt->subc_tree = pcb_r_create_tree();
+	pcb_r_insert_entry(dt->subc_tree, (pcb_box_t *)sc);
 
 	if (subc_out != NULL)
 		*subc_out = sc;
@@ -985,7 +1097,7 @@ static int parse_data_objects(pcb_board_t *pcb_for_font, pcb_data_t *dt, lht_nod
 		if (strncmp(n->name, "padstack_ref.", 13) == 0)
 			parse_pstk(dt, n);
 		if (strncmp(n->name, "via.", 4) == 0)
-			parse_pin(dt, NULL, n, 0, 0);
+			parse_via(dt, n, 0, 0, 0);
 		if (strncmp(n->name, "rat.", 4) == 0)
 			parse_rat(dt, n);
 		else if (strncmp(n->name, "element.", 8) == 0)
@@ -1278,6 +1390,8 @@ static int parse_data_pstk_protos(pcb_board_t *pcb, pcb_data_t *dst, lht_node_t
 				pcb_message(PCB_MSG_ERROR, "Invalid padstack proto ID '%s' (syntax)\n", sid);
 				return -1;
 			}
+			if (pid >= dst->ps_protos.used)
+				pcb_vtpadstack_proto_enlarge(&dst->ps_protos, pid);
 			res = parse_data_pstk_proto(pcb, dst->ps_protos.array + pid, pr, subc_parent);
 		}
 		else {
@@ -1614,7 +1728,8 @@ static int parse_board(pcb_board_t *pcb, lht_node_t *nd)
 	}
 
 	vtp0_init(&post_ids);
-	vtp0_init(&post_thermal);
+	vtp0_init(&post_thermal_old);
+	vtp0_init(&post_thermal_heavy);
 
 	memset(&pcb->LayerGroups, 0, sizeof(pcb->LayerGroups));
 
@@ -1636,21 +1751,31 @@ static int parse_board(pcb_board_t *pcb, lht_node_t *nd)
 			return -1;
 	}
 
+	pcb_data_clip_inhibit_inc(pcb->Data);
+
 	sub = lht_dom_hash_get(nd, "data");
-	if ((sub != NULL) && ((parse_data(pcb, pcb->Data, sub, NULL)) == NULL))
+	if ((sub != NULL) && ((parse_data(pcb, pcb->Data, sub, NULL)) == NULL)) {
+		pcb_data_clip_inhibit_dec(pcb->Data, pcb_true);
 		return -1;
+	}
 
 	sub = lht_dom_hash_get(nd, "styles");
-	if ((sub != NULL) && (parse_styles(&pcb->RouteStyle, sub) != 0))
+	if ((sub != NULL) && (parse_styles(&pcb->RouteStyle, sub) != 0)) {
+		pcb_data_clip_inhibit_dec(pcb->Data, pcb_true);
 		return -1;
+	}
 
 	sub = lht_dom_hash_get(nd, "netlists");
-	if ((sub != NULL) && (parse_netlists(pcb, sub) != 0))
+	if ((sub != NULL) && (parse_netlists(pcb, sub) != 0)) {
+		pcb_data_clip_inhibit_dec(pcb->Data, pcb_true);
 		return -1;
+	}
 
 	post_ids_assign(&post_ids);
-	if (post_thermal_assign(pcb, &post_thermal) != 0)
+	if (post_thermal_assign(pcb, &post_thermal_old, &post_thermal_heavy) != 0) {
+		pcb_data_clip_inhibit_dec(pcb->Data, pcb_true);
 		return -1;
+	}
 
 	/* Run poly clipping at the end so we have all IDs and we can
 	   announce the clipping (it's slow, we may need a progress bar) */
@@ -1672,6 +1797,7 @@ static int parse_board(pcb_board_t *pcb, lht_node_t *nd)
 	if (sub != NULL)
 		parse_conf(pcb, sub);
 
+	pcb_data_clip_inhibit_dec(pcb->Data, pcb_true);
 
 	pcb->Data->loader = loader; /* set this manually so the version is remembered */
 	return 0;
@@ -1750,7 +1876,7 @@ void test_parse_ev(lht_parse_t *ctx, lht_event_t ev, lht_node_type_t nt, const c
     - not a valid lihata doc (parser error)
     - lihata, but the wrong root
 */
-int io_lihata_test_parse_pcb(pcb_plug_io_t *plug_ctx, pcb_board_t *Ptr, const char *Filename, FILE *f)
+int io_lihata_test_parse(pcb_plug_io_t *plug_ctx, pcb_plug_iot_t typ, const char *Filename, FILE *f)
 {
 	lht_parse_t ctx;
 	int count;
