@@ -2,7 +2,7 @@
  *                            COPYRIGHT
  *
  *  pcb-rnd, interactive printed circuit board design
- *  Copyright (C) 2016, 2017 Tibor 'Igor2' Palinkas
+ *  Copyright (C) 2016, 2017, 2018 Tibor 'Igor2' Palinkas
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,6 +18,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  */
 
 /* Build an in-memory lihata document that represents the board then save it.
@@ -48,6 +52,8 @@
 #include "safe_fs.h"
 #include "thermal.h"
 
+#include "src_plugins/lib_compat_help/pstk_compat.h"
+
 /*#define CFMT "%[9]"*/
 #define CFMT "%.08$$mH"
 /*#define CFMT "%$$mn"*/
@@ -201,7 +207,7 @@ static lht_node_t *build_flags(pcb_flag_t *f, int object_type, int intconn)
 			if (t != 0) {
 				const char *name;
 				txt = lht_dom_node_alloc(LHT_TEXT, PCB->Data->Layer[layer].name);
-				name = io_lihata_thermal_style(t);
+				name = io_lihata_thermal_style_old(t);
 				if (name != NULL)
 					txt->data.text.value = pcb_strdup(name);
 				else
@@ -357,6 +363,79 @@ static lht_node_t *build_pin(pcb_pin_t *pin, int is_via, pcb_coord_t dx, pcb_coo
 	return obj;
 }
 
+/* attempt to convert a padstack to an old-style via for v1, v2 and v3 */
+static lht_node_t *build_pstk_pinvia(pcb_data_t *data, pcb_pstk_t *ps, pcb_bool is_via, pcb_coord_t dx, pcb_coord_t dy)
+{
+	char buff[128];
+	lht_node_t *obj;
+	pcb_coord_t x, y, drill_dia, pad_dia, clearance, mask;
+	pcb_pstk_compshape_t cshape;
+	pcb_bool plated;
+	pcb_flag_t flg;
+	char *name = pcb_attribute_get(&ps->Attributes, "name");
+
+
+	if (!pcb_pstk_export_compat_via(ps, &x, &y, &drill_dia, &pad_dia, &clearance, &mask, &cshape, &plated)) {
+		pcb_io_incompat_save(data, (pcb_any_obj_t *)ps, "Failed to convert to old-style via", "Old via format is very much restricted; try to use a simpler, uniform shape padstack");
+		return NULL;
+	}
+
+	sprintf(buff, "%s.%ld", is_via ? "via" : "pin", ps->ID);
+	obj = lht_dom_node_alloc(LHT_HASH, buff);
+
+	flg = pcb_pstk_compat_pinvia_flag(ps, cshape);
+
+	lht_dom_hash_put(obj, build_attributes(&ps->Attributes));
+	lht_dom_hash_put(obj, build_flags(&flg, PCB_TYPE_VIA, ps->intconn));
+	lht_dom_hash_put(obj, build_textf("thickness", CFMT, pad_dia));
+	lht_dom_hash_put(obj, build_textf("clearance", CFMT, clearance));
+	lht_dom_hash_put(obj, build_textf("mask", CFMT, mask));
+	lht_dom_hash_put(obj, build_textf("hole", CFMT, drill_dia));
+	lht_dom_hash_put(obj, build_textf("x", CFMT, x + dx));
+	lht_dom_hash_put(obj, build_textf("y", CFMT, y + dy));
+	lht_dom_hash_put(obj, build_text("name", name));
+	lht_dom_hash_put(obj, build_text("number", ps->term));
+	return obj;
+}
+
+/* attempt to convert a padstack to an old-style pad for v1, v2 and v3 */
+static lht_node_t *build_pstk_pad(pcb_data_t *data, pcb_pstk_t *ps, pcb_coord_t dx, pcb_coord_t dy)
+{
+	char buff[128];
+	lht_node_t *obj;
+	pcb_coord_t x1, y1, x2, y2, thickness, clearance, mask;
+	pcb_bool square, nopaste;
+	char *name = pcb_attribute_get(&ps->Attributes, "name");
+	pcb_flag_t flg;
+
+	if (!pcb_pstk_export_compat_pad(ps, &x1, &y1, &x2, &y2, &thickness, &clearance, &mask, &square, &nopaste)) {
+		pcb_io_incompat_save(data, (pcb_any_obj_t *)ps, "Failed to convert to old-style pad", "Old pad format is very much restricted; try to use a simpler, uniform shape padstack, square or round-cap line based");
+		return NULL;
+	}
+
+	sprintf(buff, "pad.%ld", ps->ID);
+	obj = lht_dom_node_alloc(LHT_HASH, buff);
+
+	flg = ps->Flags;
+	if (square)
+		flg.f |= PCB_FLAG_SQUARE;
+	if (nopaste)
+		flg.f |= PCB_FLAG_NOPASTE;
+
+	lht_dom_hash_put(obj, build_attributes(&ps->Attributes));
+	lht_dom_hash_put(obj, build_flags(&flg, PCB_TYPE_PAD, ps->intconn));
+	lht_dom_hash_put(obj, build_textf("thickness", CFMT, thickness));
+	lht_dom_hash_put(obj, build_textf("clearance", CFMT, clearance));
+	lht_dom_hash_put(obj, build_textf("mask", CFMT, mask));
+	lht_dom_hash_put(obj, build_textf("x1", CFMT, x1+dx));
+	lht_dom_hash_put(obj, build_textf("y1", CFMT, y1+dy));
+	lht_dom_hash_put(obj, build_textf("x2", CFMT, x2+dx));
+	lht_dom_hash_put(obj, build_textf("y2", CFMT, y2+dy));
+	lht_dom_hash_put(obj, build_text("name", name));
+	lht_dom_hash_put(obj, build_text("number", ps->term));
+	return obj;
+}
+
 static lht_node_t *build_pad(pcb_pad_t *pad, pcb_coord_t dx, pcb_coord_t dy)
 {
 	char buff[128];
@@ -487,11 +566,101 @@ static lht_node_t *build_element(pcb_element_t *elem)
 	return obj;
 }
 
-static lht_node_t *build_subc_element(pcb_subc_t *sc)
+static lht_node_t *build_subc_element(pcb_subc_t *subc)
 {
-#warning subc TODO: implement conversion for backward compatibility
-	pcb_message(PCB_MSG_ERROR, "Can't save subcircuits in lihata versions lower than v3\n");
-	return NULL;
+	char buff[128];
+	lht_node_t *obj, *lst;
+	pcb_coord_t ox, oy;
+	gdl_iterator_t it;
+	int l, seen_refdes = 0;
+	pcb_pstk_t *ps;
+
+	if (pcb_subc_get_origin(subc, &ox, &oy) != 0) {
+		assert(subc->parent_type == PCB_PARENT_DATA);
+		pcb_io_incompat_save(subc->parent.data, (pcb_any_obj_t *)subc, "Failed to convert subc to old-style element: missing origin", "make sure the subcircuit has the vectors on its subc-aux layer");
+		return NULL;
+	}
+
+	sprintf(buff, "element.%ld", subc->ID);
+	obj = lht_dom_node_alloc(LHT_HASH, buff);
+
+	lht_dom_hash_put(obj, build_attributes(&subc->Attributes));
+	lht_dom_hash_put(obj, build_flags(&subc->Flags, PCB_TYPE_ELEMENT, 0));
+
+	/* build drawing primitives */
+	lst = lht_dom_node_alloc(LHT_LIST, "objects");
+	lht_dom_hash_put(obj, lst);
+
+
+	lht_dom_hash_put(obj, build_textf("x", CFMT, ox));
+	lht_dom_hash_put(obj, build_textf("y", CFMT, oy));
+
+	/* export layer objects: silk lines and arcs */
+	for(l = 0; l < subc->data->LayerN; l++) {
+		pcb_layer_t *ly = &subc->data->Layer[l];
+		pcb_line_t *line;
+		pcb_arc_t *arc;
+		pcb_text_t *text;
+
+		if ((ly->meta.bound.type & PCB_LYT_SILK) && (ly->meta.bound.type & PCB_LYT_TOP)) {
+			linelist_foreach(&ly->Line, &it, line)
+				lht_dom_list_append(lst, build_line(line, -1, -ox, -oy, 0));
+			arclist_foreach(&ly->Arc, &it, arc) 
+				lht_dom_list_append(lst, build_arc(arc, -ox, -oy));
+			textlist_foreach(&ly->Text, &it, text) {
+				if (PCB_FLAG_TEST(PCB_FLAG_DYNTEXT, text)) {
+					if (!seen_refdes) {
+						pcb_text_t tmp;
+						memcpy(&tmp, text, sizeof(tmp));
+						tmp.TextString = pcb_attribute_get(&subc->Attributes, "footprint");
+						lht_dom_list_append(lst, build_pcb_text("desc", &tmp));
+						tmp.TextString = pcb_attribute_get(&subc->Attributes, "refdes");
+						lht_dom_list_append(lst, build_pcb_text("name", &tmp));
+						tmp.TextString = pcb_attribute_get(&subc->Attributes, "value");
+						lht_dom_list_append(lst, build_pcb_text("value", &tmp));
+						seen_refdes = 1;
+					}
+				}
+				else {
+					pcb_io_incompat_save(subc->parent.data, (pcb_any_obj_t *)text, "can't export custom silk text object", "the only text old pcb elements support is the refdes/value/description text");
+				}
+			}
+			if (polylist_length(&ly->Polygon) > 0) {
+				char *desc = pcb_strdup_printf("Polygons on layer %s can not be exported in an element", ly->name);
+				pcb_io_incompat_save(subc->data, NULL, desc, "only lines and arcs are exported");
+				free(desc);
+			}
+			if (textlist_length(&ly->Text) > 1) {
+				char *desc = pcb_strdup_printf("Text on layer %s can not be exported in an element", ly->name);
+				pcb_io_incompat_save(subc->data, NULL, desc, "only lines and arcs are exported");
+				free(desc);
+			}
+			continue;
+		}
+
+		if (!(ly->meta.bound.type & PCB_LYT_VIRTUAL) && (!pcb_layer_is_pure_empty(ly))) {
+			char *desc = pcb_strdup_printf("Objects on layer %s can not be exported in an element", ly->name);
+			pcb_io_incompat_save(subc->data, NULL, desc, "only top silk lines and arcs are exported; heavy terminals are not supported, use padstacks only");
+			free(desc);
+		}
+	}
+
+	for(ps = padstacklist_first(&subc->data->padstack); ps != NULL; ps = padstacklist_next(ps)) {
+		lht_node_t *nps;
+
+		nps = build_pstk_pinvia(subc->data, ps, pcb_false, -ox, -oy);
+		if (nps == NULL)
+			nps = build_pstk_pad(subc->data, ps, -ox, -oy);
+		if (nps != NULL)
+			lht_dom_list_append(lst, nps);
+		else
+			pcb_io_incompat_save(subc->data, (pcb_any_obj_t *)ps, "Padstack can not be exported as pin or pad", "use simpler padstack; for pins, all copper layers must have the same shape and there must be no paste; for pads, use a line or a rectangle; paste and mask must match the copper shape");
+	}
+
+	if (!seen_refdes)
+		pcb_io_incompat_save(subc->parent.data, (pcb_any_obj_t *)subc, "can't export subcircuit without refdes text on silk", "old pcb elements require refdes text on silk");
+
+	return obj;
 }
 
 static lht_node_t *build_subc(pcb_subc_t *sc)
@@ -553,6 +722,7 @@ static lht_node_t *build_pstk_protos(pcb_vtpadstack_proto_t *pp)
 		/* save each shape */
 		lht_dom_hash_put(nproto, nshapelst = lht_dom_node_alloc(LHT_LIST, "shape"));
 		ts = &proto->tr.array[0]; /* save the canonical shape only, the transformation cache is generated runtime */
+		if (ts != NULL)
 		for(sn = 0; sn < ts->len; sn++) {
 			pcb_pstk_shape_t *shape = ts->shape + sn;
 
@@ -829,6 +999,13 @@ static lht_node_t *build_data(pcb_data_t *data)
 		for(ps = padstacklist_first(&data->padstack); ps != NULL; ps = padstacklist_next(ps))
 			lht_dom_list_append(grp, build_pstk(ps));
 	}
+	else {
+		for(ps = padstacklist_first(&data->padstack); ps != NULL; ps = padstacklist_next(ps)) {
+			lht_node_t *p = build_pstk_pinvia(data, ps, pcb_true, 0, 0);
+			if (p != NULL)
+				lht_dom_list_append(grp, p);
+		}
+	}
 
 	for(pi = pinlist_first(&data->Via); pi != NULL; pi = pinlist_next(pi))
 		lht_dom_list_append(grp, build_pin(pi, 1, 0, 0));
