@@ -21,6 +21,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  */
 
 #include "config.h"
@@ -31,6 +35,8 @@
 static pcb_plug_io_t io_pcb[3];
 static io_pcb_ctx_t ctx[3];
 
+pcb_plug_io_t *pcb_preferred_io_pcb, *pcb_nanometer_io_pcb, *pcb_centimil_io_pcb;
+
 int io_pcb_fmt(pcb_plug_io_t *ctx, pcb_plug_iot_t typ, int wr, const char *fmt)
 {
 	if (strcmp(ctx->description, fmt) == 0)
@@ -63,12 +69,12 @@ int pplg_init_io_pcb(void)
 	ctx[0].write_coord_fmt = pcb_printf_slot[8];
 	io_pcb[0].plugin_data = &ctx[0];
 	io_pcb[0].fmt_support_prio = io_pcb_fmt;
-	io_pcb[0].test_parse_pcb = io_pcb_test_parse_pcb;
+	io_pcb[0].test_parse = io_pcb_test_parse;
 	io_pcb[0].parse_pcb = io_pcb_ParsePCB;
 	io_pcb[0].parse_element = io_pcb_ParseElement;
 	io_pcb[0].parse_font = io_pcb_ParseFont;
 	io_pcb[0].write_buffer = io_pcb_WriteBuffer;
-	io_pcb[0].write_element = io_pcb_WriteElementData;
+	io_pcb[0].write_element = io_pcb_WriteSubcData;
 	io_pcb[0].write_pcb = io_pcb_WritePCB;
 	io_pcb[0].default_fmt = "pcb";
 	io_pcb[0].description = "geda/pcb - mainline (centimils)";
@@ -77,12 +83,13 @@ int pplg_init_io_pcb(void)
 	io_pcb[0].fp_extension = ".fp";
 	io_pcb[0].mime_type = "application/x-pcb-layout";
 	PCB_HOOK_REGISTER(pcb_plug_io_t, pcb_plug_io_chain, &(io_pcb[0]));
+	pcb_centimil_io_pcb = &io_pcb[0];
 
 	ctx[1].write_coord_fmt = pcb_printf_slot[9];
 	io_pcb[1].plugin_data = &ctx[1];
 	io_pcb[1].fmt_support_prio = io_pcb_fmt;
 	io_pcb[1].write_buffer = io_pcb_WriteBuffer;
-	io_pcb[1].write_element = io_pcb_WriteElementData;
+	io_pcb[1].write_element = io_pcb_WriteSubcData;
 	io_pcb[1].write_pcb = io_pcb_WritePCB;
 	io_pcb[1].default_fmt = "pcb";
 	io_pcb[1].description = "geda/pcb - readable units";
@@ -91,12 +98,13 @@ int pplg_init_io_pcb(void)
 	io_pcb[1].fp_extension = ".fp";
 	io_pcb[1].mime_type = "application/x-pcb-layout";
 	PCB_HOOK_REGISTER(pcb_plug_io_t, pcb_plug_io_chain, &(io_pcb[1]));
+	pcb_preferred_io_pcb = &io_pcb[1];
 
 	ctx[2].write_coord_fmt = "%$$mn";
 	io_pcb[2].plugin_data = &ctx[2];
 	io_pcb[2].fmt_support_prio = io_pcb_fmt;
 	io_pcb[2].write_buffer = io_pcb_WriteBuffer;
-	io_pcb[2].write_element = io_pcb_WriteElementData;
+	io_pcb[2].write_element = io_pcb_WriteSubcData;
 	io_pcb[2].write_pcb = io_pcb_WritePCB;
 	io_pcb[2].default_fmt = "pcb";
 	io_pcb[2].description = "geda/pcb - nanometer";
@@ -105,6 +113,8 @@ int pplg_init_io_pcb(void)
 	io_pcb[2].fp_extension = ".fp";
 	io_pcb[2].mime_type = "application/x-pcb-layout";
 	PCB_HOOK_REGISTER(pcb_plug_io_t, pcb_plug_io_chain, &(io_pcb[2]));
+	pcb_nanometer_io_pcb = &io_pcb[2];
+
 
 	return 0;
 }
