@@ -18,6 +18,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  */
 
 #include "config.h"
@@ -28,7 +32,9 @@
 #include "obj_pstk_inlines.h"
 #include "compat_misc.h"
 
-#define sqr(o) ((o)*(o))
+#include "plug_io.h"
+
+#define sqr(o) ((double)(o)*(double)(o))
 
 /* emulate old pcb-rnd "pin shape" feature */
 static void octa_shape(pcb_pstk_poly_t *dst, pcb_coord_t x0, pcb_coord_t y0, pcb_coord_t radius, int style)
@@ -115,35 +121,48 @@ pcb_pstk_t *pcb_pstk_new_compat_via(pcb_data_t *data, pcb_coord_t x, pcb_coord_t
 	pcb_pstk_tshape_t tshp;
 	int n;
 
+	/* for plated vias, positive pad is required */
+	if (plated)
+		assert(pad_dia > drill_dia);
+
+	assert(drill_dia > 0);
+	assert(clearance >= 0);
+	assert(mask >= 0);
+
 	memset(&proto, 0, sizeof(proto));
 	memset(&tshp, 0, sizeof(tshp));
 	memset(&copper_master, 0, sizeof(copper_master));
 	memset(&mask_master, 0, sizeof(mask_master));
 
-	tshp.len = 3 + (mask > 0 ? 2 : 0);
 	tshp.shape = shape;
 	proto.tr.alloced = proto.tr.used = 1; /* has the canonical form only */
 	proto.tr.array = &tshp;
 
-/* we need to generate the shape only once as it's the same on all */
-	if (compat_via_shape_gen(&copper_master, cshape, pad_dia) != 0)
-		return NULL;
+	if (plated) {
+		tshp.len = 3 + (mask > 0 ? 2 : 0);
 
-	for(n = 0; n < 3; n++)
-		memcpy(&shape[n], &copper_master, sizeof(copper_master));
-	shape[0].layer_mask = PCB_LYT_COPPER | PCB_LYT_TOP;    shape[0].comb = 0;
-	shape[1].layer_mask = PCB_LYT_COPPER | PCB_LYT_BOTTOM; shape[1].comb = 0;
-	shape[2].layer_mask = PCB_LYT_COPPER | PCB_LYT_INTERN; shape[2].comb = 0;
-
-	if (mask > 0) {
-		if (compat_via_shape_gen(&mask_master, cshape, mask) != 0)
+		/* we need to generate the shape only once as it's the same on all */
+		if (compat_via_shape_gen(&copper_master, cshape, pad_dia) != 0)
 			return NULL;
+
+		for(n = 0; n < 3; n++)
+			memcpy(&shape[n], &copper_master, sizeof(copper_master));
+		shape[0].layer_mask = PCB_LYT_COPPER | PCB_LYT_TOP;    shape[0].comb = 0;
+		shape[1].layer_mask = PCB_LYT_COPPER | PCB_LYT_BOTTOM; shape[1].comb = 0;
+		shape[2].layer_mask = PCB_LYT_COPPER | PCB_LYT_INTERN; shape[2].comb = 0;
+
+		if (mask > 0) {
+			if (compat_via_shape_gen(&mask_master, cshape, mask) != 0)
+				return NULL;
 		
-		memcpy(&shape[3], &mask_master, sizeof(mask_master));
-		memcpy(&shape[4], &mask_master, sizeof(mask_master));
-		shape[3].layer_mask = PCB_LYT_MASK | PCB_LYT_TOP;      shape[3].comb = PCB_LYC_SUB;
-		shape[4].layer_mask = PCB_LYT_MASK | PCB_LYT_BOTTOM;   shape[4].comb = PCB_LYC_SUB;
+			memcpy(&shape[3], &mask_master, sizeof(mask_master));
+			memcpy(&shape[4], &mask_master, sizeof(mask_master));
+			shape[3].layer_mask = PCB_LYT_MASK | PCB_LYT_TOP;      shape[3].comb = PCB_LYC_SUB + PCB_LYC_AUTO;
+			shape[4].layer_mask = PCB_LYT_MASK | PCB_LYT_BOTTOM;   shape[4].comb = PCB_LYC_SUB + PCB_LYC_AUTO;
+		}
 	}
+	else
+		tshp.len = 0;
 
 	proto.hdia = drill_dia;
 	proto.hplated = plated;
@@ -164,11 +183,12 @@ pcb_pstk_t *pcb_pstk_new_compat_via(pcb_data_t *data, pcb_coord_t x, pcb_coord_t
 
 static pcb_pstk_compshape_t get_old_shape_square(pcb_coord_t *dia, const pcb_pstk_shape_t *shp)
 {
-	double len2, l, a;
+	double sq, len2, l, a;
 	int n;
 
+	sq = sqr(shp->data.poly.x[0] - shp->data.poly.x[2]) + sqr(shp->data.poly.y[0] - shp->data.poly.y[2]);
 	/* cross-bars must be of equal length */
-	if (sqr(shp->data.poly.x[0] - shp->data.poly.x[2]) + sqr(shp->data.poly.y[0] - shp->data.poly.y[2]) != sqr(shp->data.poly.x[1] - shp->data.poly.x[3]) + sqr(shp->data.poly.y[1] - shp->data.poly.y[3]))
+	if (sq != sqr(shp->data.poly.x[1] - shp->data.poly.x[3]) + sqr(shp->data.poly.y[1] - shp->data.poly.y[3]))
 		return PCB_PSTK_COMPAT_INVALID;
 
 	/* all sides must be of equal length */
@@ -185,7 +205,8 @@ static pcb_pstk_compshape_t get_old_shape_square(pcb_coord_t *dia, const pcb_pst
 		return PCB_PSTK_COMPAT_INVALID;
 
 	/* found a valid square */
-	*dia = sqrt(len2);
+	*dia = pcb_round(sqrt(sq / 2.0));
+
 	return PCB_PSTK_COMPAT_SQUARE;
 }
 
@@ -232,6 +253,11 @@ static pcb_pstk_compshape_t get_old_shape_octa(pcb_coord_t *dia, const pcb_pstk_
 	if ((minx == 0.0) || (miny == 0.0))
 		return PCB_PSTK_COMPAT_INVALID;
 
+	if (minx < miny)
+		miny = minx;
+	else
+		minx = miny;
+
 	/* normalize the factors */
 	for(n = 0; n < 8; n++) {
 		x[n] = fabs(x[n] / minx);
@@ -252,7 +278,7 @@ static pcb_pstk_compshape_t get_old_shape_octa(pcb_coord_t *dia, const pcb_pstk_
 
 	/* compare to all known shape factors */
 	for(shi = PCB_PSTK_COMPAT_SHAPED; shi <= PCB_PSTK_COMPAT_SHAPED_END; shi++) {
-		pcb_poly_square_pin_factors(shi, xm, ym);
+		pcb_poly_square_pin_factors(shi - PCB_PSTK_COMPAT_SHAPED, xm, ym);
 		found = 1;
 		for(n = 0; n < 8; n++) {
 			if ((xm[n] != x[n]) || (ym[n] != y[n])) {
@@ -261,7 +287,7 @@ static pcb_pstk_compshape_t get_old_shape_octa(pcb_coord_t *dia, const pcb_pstk_
 			}
 		}
 		if (found) {
-			*dia = sqrt(sqr(minx) + sqr(miny)) / sqrt(2);
+			*dia = pcb_round(sqrt((sqr(minx) + sqr(miny)) / 2.0));
 			return shi;
 		}
 	}
@@ -356,7 +382,7 @@ pcb_bool pcb_pstk_export_compat_via(pcb_pstk_t *ps, pcb_coord_t *x, pcb_coord_t
 	*drill_dia = proto->hdia;
 	*pad_dia = old_dia[coppern];
 	*clearance = ps->Clearance;
-	*mask = old_dia[maskn];
+	*mask = maskn >= 0 ? old_dia[maskn] : 0;
 	*cshape = old_shape[0];
 	*plated = proto->hplated;
 
@@ -417,8 +443,9 @@ static void gen_pad(pcb_pstk_shape_t *dst, pcb_coord_t x1, pcb_coord_t y1, pcb_c
 	}
 }
 
-pcb_pstk_t *pcb_pstk_new_compat_pad(pcb_data_t *data, pcb_coord_t x1, pcb_coord_t y1, pcb_coord_t x2, pcb_coord_t y2, pcb_coord_t thickness, pcb_coord_t clearance, pcb_coord_t mask, pcb_bool square, pcb_bool nopaste)
+pcb_pstk_t *pcb_pstk_new_compat_pad(pcb_data_t *data, pcb_coord_t x1, pcb_coord_t y1, pcb_coord_t x2, pcb_coord_t y2, pcb_coord_t thickness, pcb_coord_t clearance, pcb_coord_t mask, pcb_bool square, pcb_bool nopaste, pcb_bool onotherside)
 {
+	pcb_layer_type_t side;
 	pcb_pstk_proto_t proto;
 	pcb_pstk_shape_t shape[3]; /* max number of shapes: 1 copper, 1 mask, 1 paste */
 	pcb_cardinal_t pid;
@@ -426,6 +453,10 @@ pcb_pstk_t *pcb_pstk_new_compat_pad(pcb_data_t *data, pcb_coord_t x1, pcb_coord_
 	int n;
 	pcb_coord_t cx, cy;
 
+	assert(thickness >= 0);
+	assert(clearance >= 0);
+	assert(mask >= 0);
+
 	cx = (x1 + x2) / 2;
 	cy = (y1 + y2) / 2;
 
@@ -443,10 +474,11 @@ pcb_pstk_t *pcb_pstk_new_compat_pad(pcb_data_t *data, pcb_coord_t x1, pcb_coord_
 	if (!nopaste)
 		pcb_pstk_shape_copy(&shape[2], &shape[0]); /* paste is the same */
 
-	shape[0].layer_mask = PCB_LYT_TOP | PCB_LYT_COPPER; shape[0].comb = 0;
-	shape[1].layer_mask = PCB_LYT_TOP | PCB_LYT_MASK;   shape[1].comb = PCB_LYC_AUTO | PCB_LYC_SUB;
+	side = onotherside ? PCB_LYT_BOTTOM : PCB_LYT_TOP;
+	shape[0].layer_mask = side | PCB_LYT_COPPER; shape[0].comb = 0;
+	shape[1].layer_mask = side | PCB_LYT_MASK;   shape[1].comb = PCB_LYC_AUTO | PCB_LYC_SUB;
 	if (!nopaste) {
-		shape[2].layer_mask = PCB_LYT_TOP | PCB_LYT_PASTE;
+		shape[2].layer_mask = side | PCB_LYT_PASTE;
 		shape[2].comb = PCB_LYC_AUTO;
 	}
 
@@ -513,16 +545,28 @@ pcb_bool pcb_pstk_export_compat_pad(pcb_pstk_t *ps, pcb_coord_t *x1, pcb_coord_t
 	/* if the shape is poly, convert to line to make the rest of the code simpler */
 	if (tshp->shape[0].shape == PCB_PSSH_POLY) {
 		for(n = 0; n < tshp->len; n++) {
-			pcb_coord_t w, h;
+			pcb_coord_t w, h, x1, x2, y1, y2;
 
 			if (tshp->shape[0].data.poly.len != 4)
 				return pcb_false;
 
-			w = tshp->shape[0].data.poly.x[0] - tshp->shape[0].data.poly.x[2];
-			h = tshp->shape[0].data.poly.y[0] - tshp->shape[0].data.poly.y[2];
+			x1 = tshp->shape[0].data.poly.x[0];
+			x2 = tshp->shape[0].data.poly.x[2];
+			if (x1 < x2) {
+				x2 = tshp->shape[0].data.poly.x[0];
+				x1 = tshp->shape[0].data.poly.x[2];
+			}
+			y1 = tshp->shape[0].data.poly.y[0];
+			y2 = tshp->shape[0].data.poly.y[2];
+			if (y1 < y2) {
+				y2 = tshp->shape[0].data.poly.y[0];
+				y1 = tshp->shape[0].data.poly.y[2];
+			}
+			w = x1 - x2;
+			h = y1 - y2;
 			lt[n] = (w < h) ? w : h;
-			lx1[n] = tshp->shape[0].data.poly.x[2] + lt[n] / 2;
-			ly1[n] = tshp->shape[0].data.poly.y[2] + lt[n] / 2;
+			lx1[n] = x2 + lt[n] / 2;
+			ly1[n] = y2 + lt[n] / 2;
 			lx2[n] = lx1[n] + (w - lt[n]);
 			ly2[n] = ly1[n] + (h - lt[n]);
 		}
@@ -594,3 +638,89 @@ pcb_bool pcb_pstk_export_compat_pad(pcb_pstk_t *ps, pcb_coord_t *x1, pcb_coord_t
 
 	return pcb_true;
 }
+
+pcb_flag_t pcb_pstk_compat_pinvia_flag(pcb_pstk_t *ps, pcb_pstk_compshape_t cshape)
+{
+	pcb_flag_t flg;
+	int n;
+
+	memset(&flg, 0, sizeof(flg));
+	flg.f = ps->Flags.f & PCB_PSTK_VIA_COMPAT_FLAGS;
+	switch(cshape) {
+		case PCB_PSTK_COMPAT_ROUND:
+			break;
+		case PCB_PSTK_COMPAT_OCTAGON:
+			flg.f |= PCB_FLAG_OCTAGON;
+			break;
+		case PCB_PSTK_COMPAT_SQUARE:
+			flg.f |= PCB_FLAG_SQUARE;
+			flg.q = 1;
+			break;
+		default:
+			if ((cshape >= PCB_PSTK_COMPAT_SHAPED) && (cshape <= PCB_PSTK_COMPAT_SHAPED_END)) {
+				flg.f |= PCB_FLAG_SQUARE;
+				cshape -= PCB_PSTK_COMPAT_SHAPED;
+				if (cshape == 1)
+					cshape = 17;
+				flg.q = cshape;
+			}
+			else
+				pcb_io_incompat_save(ps->parent.data, (pcb_any_obj_t *)ps, "Failed to convert shape to old-style pin/via", "Old pin/via format is very much restricted; try to use a simpler shape (e.g. circle)");
+	}
+
+	for(n = 0; n < sizeof(flg.t) / sizeof(flg.t[0]); n++) {
+		unsigned char *ot = pcb_pstk_get_thermal(ps, n, 0);
+		int nt;
+		if ((ot == NULL) || (*ot == 0) || !((*ot) & PCB_THERMAL_ON))
+			continue;
+		switch(((*ot) & ~PCB_THERMAL_ON)) {
+			case PCB_THERMAL_SHARP | PCB_THERMAL_DIAGONAL: nt = 1; break;
+			case PCB_THERMAL_SHARP: nt = 2; break;
+			case PCB_THERMAL_SOLID: nt = 3; break;
+			case PCB_THERMAL_ROUND | PCB_THERMAL_DIAGONAL: nt = 4; break;
+			case PCB_THERMAL_ROUND: nt = 5; break;
+			default: nt = 0; pcb_io_incompat_save(ps->parent.data, (pcb_any_obj_t *)ps, "Failed to convert thermal to old-style via", "Old via format is very much restricted; try to use a simpler thermal shape");
+		}
+		PCB_FLAG_THERM_ASSIGN_(n, nt, flg);
+	}
+	return flg;
+}
+
+
+pcb_pstk_t *pcb_old_via_new(pcb_data_t *data, pcb_coord_t X, pcb_coord_t Y, pcb_coord_t Thickness, pcb_coord_t Clearance, pcb_coord_t Mask, pcb_coord_t DrillingHole, const char *Name, pcb_flag_t Flags)
+{
+	pcb_pstk_t *p;
+	pcb_pstk_compshape_t shp;
+	int n;
+
+	if (Flags.f & PCB_FLAG_SQUARE) {
+		shp = Flags.q /*+ PCB_PSTK_COMPAT_SHAPED*/;
+		if (shp == 0)
+			shp = PCB_PSTK_COMPAT_SQUARE;
+	}
+	else if (Flags.f & PCB_FLAG_OCTAGON)
+		shp = PCB_PSTK_COMPAT_OCTAGON;
+	else
+		shp = PCB_PSTK_COMPAT_ROUND;
+
+	p = pcb_pstk_new_compat_via(data, X, Y, DrillingHole, Thickness, Clearance/2, Mask, shp, !(Flags.f & PCB_FLAG_HOLE));
+	p->Flags.f |= Flags.f & PCB_PSTK_VIA_COMPAT_FLAGS;
+	for(n = 0; n < sizeof(Flags.t) / sizeof(Flags.t[0]); n++) {
+		int nt = PCB_THERMAL_ON, t = ((Flags.t[n/2] >> (4 * (n % 2))) & 0xf);
+		if (t != 0) {
+			switch(t) {
+				case 1: nt |= PCB_THERMAL_SHARP | PCB_THERMAL_DIAGONAL; break;
+				case 2: nt |= PCB_THERMAL_SHARP; break;
+				case 3: nt |= PCB_THERMAL_SOLID; break;
+				case 4: nt |= PCB_THERMAL_ROUND | PCB_THERMAL_DIAGONAL; break;
+				case 5: nt |= PCB_THERMAL_ROUND; break;
+			}
+			pcb_pstk_set_thermal(p, n, nt);
+		}
+	}
+
+	if (Name != NULL)
+		pcb_attribute_put(&p->Attributes, "name", Name);
+
+	return p;
+}
