@@ -1,7 +1,8 @@
 /*
  *                            COPYRIGHT
  *
- *  PCB, interactive printed circuit board design
+ *  pcb-rnd, interactive printed circuit board design
+ *  (this file is based on PCB, interactive printed circuit board design)
  *  Copyright (C) 1994,1995,1996 Thomas Nau
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -18,9 +19,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- *  Contact addresses for paper mail and Email:
- *  Thomas Nau, Schlehenweg 15, 88471 Baustetten, Germany
- *  Thomas.Nau@rz.uni-ulm.de
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  *
  */
 
@@ -39,19 +41,11 @@
 #include "compat_nls.h"
 
 #include "compat.h"
+#include "bu_box.h"
+#include "bu_check_button.h"
+#include "bu_spin_button.h"
 #include "wt_coord_entry.h"
 
-GtkWidget *ghid_category_vbox(GtkWidget * box, const gchar * category_header, gint header_pad, gint box_pad,
-															gboolean pack_start, gboolean bottom_pad);
-void ghid_spin_button(GtkWidget * box, GtkWidget ** spin_button, gfloat value, gfloat low, gfloat high, gfloat step0,
-											gfloat step1, gint digits, gint width, void (*cb_func) (GtkSpinButton *, pcb_hid_attribute_t *), pcb_hid_attribute_t *data,
-											gboolean right_align, const gchar * string);
-void pcb_gtk_check_button_connected(GtkWidget * box, GtkWidget ** button, gboolean active,
-																		gboolean pack_start, gboolean expand,
-																		gboolean fill, gint pad,
-																		void (*cb_func) (GtkToggleButton *, pcb_hid_attribute_t *), pcb_hid_attribute_t *data,
-																		const gchar * string);
-
 #define PCB_OBJ_PROP "pcb-rnd_context"
 
 typedef struct {
@@ -74,50 +68,70 @@ typedef struct {
 			dst->change_cb(ctx, ctx->caller_data, dst); \
 	} while(0) \
 
-static void set_flag_cb(GtkToggleButton *button, pcb_hid_attribute_t *dst)
+static void set_flag_cb(GtkToggleButton *button, gpointer user_data)
 {
 	attr_dlg_t *ctx = g_object_get_data(G_OBJECT(button), PCB_OBJ_PROP);
+	pcb_hid_attribute_t *dst = user_data;
+
 	dst->changed = 1;
 	if (ctx->inhibit_valchg)
 		return;
+
 	dst->default_val.int_value = gtk_toggle_button_get_active(button);
 	change_cb(ctx, dst);
 }
 
-static void intspinner_changed_cb(GtkSpinButton *spin_button, pcb_hid_attribute_t *dst)
+static void intspinner_changed_cb(GtkSpinButton *spin_button, gpointer user_data)
 {
 	attr_dlg_t *ctx = g_object_get_data(G_OBJECT(spin_button), PCB_OBJ_PROP);
+	pcb_hid_attribute_t *dst = user_data;
 
 	dst->changed = 1;
 	if (ctx->inhibit_valchg)
 		return;
-	dst->default_val.int_value = gtk_spin_button_get_value(GTK_SPIN_BUTTON((GtkWidget *)spin_button));
+	dst->default_val.int_value = gtk_spin_button_get_value(spin_button);
 	change_cb(ctx, dst);
 }
 
 static void coordentry_changed_cb(GtkEntry *entry, pcb_hid_attribute_t *dst)
 {
 	attr_dlg_t *ctx = g_object_get_data(G_OBJECT(entry), PCB_OBJ_PROP);
-	const gchar *s;
-
-	dst->changed = 1;
-	if (ctx->inhibit_valchg)
-		return;
+	const char *s;
+	pcb_coord_t crd;
+	double crdd;
+	const pcb_unit_t *u;
+	pcb_bool succ;
 
 	s = gtk_entry_get_text(entry);
-	dst->default_val.coord_value = pcb_get_value(s, NULL, NULL, NULL);
-	change_cb(ctx, dst);
+	succ = pcb_get_value_unit(s, NULL, 1, &crdd, &u);
+
+	if (!succ)
+		return;
+
+	dst->changed = 1;
+
+	if (ctx->inhibit_valchg)
+		return;
+
+	crd = crdd;
+	pcb_gtk_coord_entry_set_unit(GHID_COORD_ENTRY(entry), u);
+
+	if ((dst->default_val.coord_value != crd) && (crd >= dst->min_val) && (crd <= dst->max_val)) {
+		dst->default_val.coord_value = crd;
+		change_cb(ctx, dst);
+	}
 }
 
-static void dblspinner_changed_cb(GtkSpinButton *spin_button, pcb_hid_attribute_t *dst)
+static void dblspinner_changed_cb(GtkSpinButton *spin_button, gpointer user_data)
 {
 	attr_dlg_t *ctx = g_object_get_data(G_OBJECT(spin_button), PCB_OBJ_PROP);
+	pcb_hid_attribute_t *dst = user_data;
 
 	dst->changed = 1;
 	if (ctx->inhibit_valchg)
 		return;
 
-	dst->default_val.real_value = gtk_spin_button_get_value(GTK_SPIN_BUTTON((GtkWidget *)spin_button));
+	dst->default_val.real_value = gtk_spin_button_get_value(spin_button);
 	change_cb(ctx, dst);
 }
 
@@ -134,7 +148,7 @@ static void entry_changed_cb(GtkEntry *entry, pcb_hid_attribute_t *dst)
 	change_cb(ctx, dst);
 }
 
-static void enum_changed_cb(GtkWidget *combo_box, pcb_hid_attribute_t *dst)
+static void enum_changed_cb(GtkComboBox *combo_box, pcb_hid_attribute_t *dst)
 {
 	attr_dlg_t *ctx = g_object_get_data(G_OBJECT(combo_box), PCB_OBJ_PROP);
 
@@ -142,13 +156,29 @@ static void enum_changed_cb(GtkWidget *combo_box, pcb_hid_attribute_t *dst)
 	if (ctx->inhibit_valchg)
 		return;
 
-	dst->default_val.int_value = gtk_combo_box_get_active(GTK_COMBO_BOX(combo_box));
+	dst->default_val.int_value = gtk_combo_box_get_active(combo_box);
 	change_cb(ctx, dst);
 }
 
-static void button_changed_cb(GtkEntry *entry, pcb_hid_attribute_t *dst)
+static void notebook_changed_cb(GtkNotebook *nb, GtkWidget *page, guint page_num, pcb_hid_attribute_t *dst)
 {
-	attr_dlg_t *ctx = g_object_get_data(G_OBJECT(entry), PCB_OBJ_PROP);
+	attr_dlg_t *ctx = g_object_get_data(G_OBJECT(nb), PCB_OBJ_PROP);
+
+	dst->changed = 1;
+	if (ctx->inhibit_valchg)
+		return;
+
+	/* Gets the index (starting from 0) of the current page in the notebook. If
+	   the notebook has no pages, then -1 will be returned and no call-back occur. */
+	if (gtk_notebook_get_current_page(nb) >= 0) {
+		dst->default_val.int_value = page_num;
+		change_cb(ctx, dst);
+	}
+}
+
+static void button_changed_cb(GtkButton *button, pcb_hid_attribute_t *dst)
+{
+	attr_dlg_t *ctx = g_object_get_data(G_OBJECT(button), PCB_OBJ_PROP);
 
 	dst->changed = 1;
 	if (ctx->inhibit_valchg)
@@ -162,11 +192,12 @@ typedef struct {
 	void *ctx;
 } resp_ctx_t;
 
-static void ghid_attr_dlg_response_cb(GtkDialog *dialog, gint arg1, gpointer user_data)
+static void ghid_attr_dlg_response_cb(GtkDialog *dialog, gint response_id, gpointer user_data)
 {
 	resp_ctx_t *ctx = (resp_ctx_t *)user_data;
+
 	if ((ctx != NULL) && (ctx->cb != NULL)) {
-		switch (arg1) {
+		switch (response_id) {
 		case GTK_RESPONSE_OK:
 			ctx->cb(ctx->ctx, PCB_HID_ATTR_EV_OK);
 			break;
@@ -205,21 +236,26 @@ static GtkWidget *frame_scroll(GtkWidget *parent, pcb_hatt_compflags_t flags)
 }
 
 typedef struct {
-	int cols, rows;
-	int col, row;
-} ghid_attr_tbl_t;
+	enum {
+		TB_TABLE,
+		TB_TABBED
+	} type;
+	union {
+		struct {
+			int cols, rows;
+			int col, row;
+		} table;
+		struct {
+			const char **tablab;
+		} tabbed;
+	} val;
+} ghid_attr_tb_t;
 
-static int ghid_attr_dlg_add(attr_dlg_t *ctx, GtkWidget *real_parent, ghid_attr_tbl_t *tbl_st, int start_from, int add_labels)
+static int ghid_attr_dlg_add(attr_dlg_t *ctx, GtkWidget *real_parent, ghid_attr_tb_t *tb_st, int start_from, int add_labels)
 {
 	int j, i, n;
 	GtkWidget *combo, *widget, *entry, *vbox1, *hbox, *bparent, *parent, *tbl;
 
-	/*
-	 * Iterate over all the export options and build up a dialog box
-	 * that lets us control all of the options.  By doing things this
-	 * way, any changes to the exporter HID's automatically are
-	 * reflected in this dialog box.
-	 */
 	for (j = start_from; j < ctx->n_attrs; j++) {
 		const pcb_unit_t *unit_list;
 		if (ctx->attrs[j].help_text == ATTR_UNDOCUMENTED)
@@ -228,13 +264,28 @@ static int ghid_attr_dlg_add(attr_dlg_t *ctx, GtkWidget *real_parent, ghid_attr_
 			break;
 
 		/* if we are willing a table, allocate parent boxes in row-major */
-		if (tbl_st != NULL) {
-			parent = gtkc_vbox_new(FALSE, 4);
-			gtkc_table_attach1(real_parent, parent, tbl_st->row, tbl_st->col);
-			tbl_st->col++;
-			if (tbl_st->col >= tbl_st->cols) {
-				tbl_st->col = 0;
-				tbl_st->row++;
+		if (tb_st != NULL) {
+			switch(tb_st->type) {
+				case TB_TABLE:
+					parent = gtkc_vbox_new(FALSE, 0);
+					gtkc_table_attach1(real_parent, parent, tb_st->val.table.row, tb_st->val.table.col);
+					tb_st->val.table.col++;
+					if (tb_st->val.table.col >= tb_st->val.table.cols) {
+						tb_st->val.table.col = 0;
+						tb_st->val.table.row++;
+					}
+					break;
+				case TB_TABBED:
+					/* Add a new notebook page with an empty vbox, using tab label present in enumerations. */
+					parent = gtkc_vbox_new(FALSE, 4);
+					if (*tb_st->val.tabbed.tablab) {
+						widget = gtk_label_new(*tb_st->val.tabbed.tablab);
+						tb_st->val.tabbed.tablab++;
+					}
+					else 
+						widget = NULL;
+					gtk_notebook_append_page(GTK_NOTEBOOK(real_parent), parent, widget);
+					break;
 			}
 		}
 		else
@@ -260,19 +311,33 @@ static int ghid_attr_dlg_add(attr_dlg_t *ctx, GtkWidget *real_parent, ghid_attr_
 
 			case PCB_HATT_BEGIN_TABLE:
 				{
-					ghid_attr_tbl_t ts;
+					ghid_attr_tb_t ts;
 					bparent = frame_scroll(parent, ctx->attrs[j].pcb_hatt_flags);
-					ts.cols = ctx->attrs[j].pcb_hatt_table_cols;
-					ts.rows = pcb_hid_atrdlg_num_children(ctx->attrs, j+1, ctx->n_attrs) / ts.cols;
-					ts.col = 0;
-					ts.row = 0;
-					tbl = gtkc_table_static(ts.rows, ts.cols, 1);
+					ts.type = TB_TABLE;
+					ts.val.table.cols = ctx->attrs[j].pcb_hatt_table_cols;
+					ts.val.table.rows = pcb_hid_atrdlg_num_children(ctx->attrs, j+1, ctx->n_attrs) / ts.val.table.cols;
+					ts.val.table.col = 0;
+					ts.val.table.row = 0;
+					tbl = gtkc_table_static(ts.val.table.rows, ts.val.table.cols, 1);
 					gtk_box_pack_start(GTK_BOX(bparent), tbl, FALSE, FALSE, 0);
 					ctx->wl[j] = tbl;
 					j = ghid_attr_dlg_add(ctx, tbl, &ts, j+1, (ctx->attrs[j].pcb_hatt_flags & PCB_HATF_LABEL));
 				}
 				break;
 
+			case PCB_HATT_BEGIN_TABBED:
+				{
+					ghid_attr_tb_t ts;
+					ts.type = TB_TABBED;
+					ts.val.tabbed.tablab = ctx->attrs[j].enumerations;
+					ctx->wl[j] = widget = gtk_notebook_new();
+					gtk_box_pack_start(GTK_BOX(parent), widget, FALSE, FALSE, 0);
+					g_signal_connect(G_OBJECT(widget), "switch-page", G_CALLBACK(notebook_changed_cb), &(ctx->attrs[j]));
+					g_object_set_data(G_OBJECT(widget), PCB_OBJ_PROP, ctx);
+					j = ghid_attr_dlg_add(ctx, widget, &ts, j+1, 0);
+				}
+				break;
+
 			case PCB_HATT_LABEL:
 				ctx->wl[j] = widget = gtk_label_new(ctx->attrs[j].name);
 				g_object_set_data(G_OBJECT(widget), PCB_OBJ_PROP, ctx);
@@ -452,7 +517,7 @@ static int ghid_attr_dlg_add(attr_dlg_t *ctx, GtkWidget *real_parent, ghid_attr_
 				gtk_box_pack_start(GTK_BOX(hbox), ctx->wl[j], FALSE, FALSE, 0);
 
 				gtk_widget_set_tooltip_text(ctx->wl[j], ctx->attrs[j].help_text);
-				g_signal_connect(G_OBJECT(ctx->wl[j]), "pressed", G_CALLBACK(button_changed_cb), &(ctx->attrs[j]));
+				g_signal_connect(G_OBJECT(ctx->wl[j]), "clicked", G_CALLBACK(button_changed_cb), &(ctx->attrs[j]));
 				g_object_set_data(G_OBJECT(ctx->wl[j]), PCB_OBJ_PROP, ctx);
 
 				if (add_labels) {
@@ -484,32 +549,67 @@ static int ghid_attr_dlg_set(attr_dlg_t *ctx, int idx, const pcb_hid_attr_val_t
 			goto error;
 
 		case PCB_HATT_LABEL:
-			gtk_label_set_text(GTK_LABEL(ctx->wl[idx]), val->str_value);
+			{
+				const char *txt = gtk_label_get_text(GTK_LABEL(ctx->wl[idx]));
+				if (strcmp(txt, val->str_value) == 0)
+					goto nochg;
+				gtk_label_set_text(GTK_LABEL(ctx->wl[idx]), val->str_value);
+			}
 			break;
 
 		case PCB_HATT_INTEGER:
-			gtk_spin_button_set_value(GTK_SPIN_BUTTON(ctx->wl[idx]), val->int_value);
+			{
+				double d = gtk_spin_button_get_value(GTK_SPIN_BUTTON(ctx->wl[idx]));
+				if (val->int_value == (int)d)
+					goto nochg;
+				gtk_spin_button_set_value(GTK_SPIN_BUTTON(ctx->wl[idx]), val->int_value);
+			}
 			break;
 
 		case PCB_HATT_COORD:
-			pcb_gtk_coord_entry_set_value(GHID_COORD_ENTRY(ctx->wl[idx]), val->coord_value);
+			{
+				pcb_coord_t crd = pcb_gtk_coord_entry_get_value(GHID_COORD_ENTRY(ctx->wl[idx]));
+				if (crd == val->coord_value)
+					goto nochg;
+				pcb_gtk_coord_entry_set_value(GHID_COORD_ENTRY(ctx->wl[idx]), val->coord_value);
+			}
 			break;
 
 		case PCB_HATT_REAL:
-			gtk_spin_button_set_value(GTK_SPIN_BUTTON(ctx->wl[idx]), val->real_value);
+			{
+				double d = gtk_spin_button_get_value(GTK_SPIN_BUTTON(ctx->wl[idx]));
+				if (val->real_value == d)
+					goto nochg;
+				gtk_spin_button_set_value(GTK_SPIN_BUTTON(ctx->wl[idx]), val->real_value);
+			}
 			break;
 
 		case PCB_HATT_STRING:
 		case PCB_HATT_PATH:
-			gtk_entry_set_text(GTK_ENTRY(ctx->wl[idx]), val->str_value);
+			{
+				const char *s = gtk_entry_get_text(GTK_ENTRY(ctx->wl[idx]));
+				if (strcmp(s, val->str_value) == 0)
+					goto nochg;
+				gtk_entry_set_text(GTK_ENTRY(ctx->wl[idx]), val->str_value);
+			}
 			break;
 
 		case PCB_HATT_BOOL:
-			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ctx->wl[idx]), val->int_value);
+			{
+				int chk = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ctx->wl[idx]));
+				if (chk == val->int_value)
+					goto nochg;
+				gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ctx->wl[idx]), val->int_value);
+			}
 			break;
 
 		case PCB_HATT_ENUM:
-			gtkc_combo_box_set_active(ctx->wl[idx], val->int_value);
+			{
+				int en = gtk_combo_box_get_active(GTK_COMBO_BOX(ctx->wl[idx]));
+				if (en == val->int_value)
+					goto nochg;
+				gtk_combo_box_set_active(GTK_COMBO_BOX(ctx->wl[idx]), val->int_value);
+			}
 			break;
 
 		case PCB_HATT_UNIT:
@@ -517,7 +617,16 @@ static int ghid_attr_dlg_set(attr_dlg_t *ctx, int idx, const pcb_hid_attr_val_t
 			break;
 
 		case PCB_HATT_BUTTON:
-			gtk_button_set_label(GTK_BUTTON(ctx->wl[idx]), val->str_value);
+			{
+				const char *s = gtk_button_get_label(GTK_BUTTON(ctx->wl[idx]));
+				if (strcmp(s, val->str_value) == 0)
+					goto nochg;
+				gtk_button_set_label(GTK_BUTTON(ctx->wl[idx]), val->str_value);
+			}
+			break;
+
+		case PCB_HATT_BEGIN_TABBED:
+			gtk_notebook_set_current_page(GTK_NOTEBOOK(ctx->wl[idx]), val->int_value);
 			break;
 	}
 
@@ -527,6 +636,10 @@ static int ghid_attr_dlg_set(attr_dlg_t *ctx, int idx, const pcb_hid_attr_val_t
 	error:;
 	ctx->inhibit_valchg = save;
 	return -1;
+
+	nochg:;
+	ctx->inhibit_valchg = save;
+	return 1;
 }
 
 void *ghid_attr_dlg_new(GtkWidget *top_window, pcb_hid_attribute_t *attrs, int n_attrs, pcb_hid_attr_val_t *results, const char *title, const char *descr, void *caller_data, pcb_bool modal, void (*button_cb)(void *caller_data, pcb_hid_attr_ev_t ev))
@@ -561,7 +674,7 @@ void *ghid_attr_dlg_new(GtkWidget *top_window, pcb_hid_attribute_t *attrs, int n
 
 	main_vbox = gtkc_vbox_new(FALSE, 6);
 	gtk_container_set_border_width(GTK_CONTAINER(main_vbox), 6);
-	gtk_container_add(GTK_CONTAINER(content_area), main_vbox);
+	gtk_container_add_with_properties(GTK_CONTAINER(content_area), main_vbox, "expand", TRUE, "fill", TRUE, NULL);
 
 	if (!PCB_HATT_IS_COMPOSITE(attrs[0].type)) {
 		vbox = ghid_category_vbox(main_vbox, descr != NULL ? descr : "", 4, 2, TRUE, TRUE);
@@ -654,14 +767,19 @@ int ghid_attr_dlg_widget_hide(void *hid_ctx, int idx, pcb_bool hide)
 int ghid_attr_dlg_set_value(void *hid_ctx, int idx, const pcb_hid_attr_val_t *val)
 {
 	attr_dlg_t *ctx = hid_ctx;
+	int res;
 
 	if ((idx < 0) || (idx >= ctx->n_attrs))
 		return -1;
 
-	if (ghid_attr_dlg_set(ctx, idx, val) == 0) {
+	res = ghid_attr_dlg_set(ctx, idx, val);
+
+	if (res == 0) {
 		ctx->attrs[idx].default_val = *val;
 		return 0;
 	}
+	else if (res == 1)
+		return 0; /* no change */
 
 	return -1;
 }
