@@ -1,7 +1,8 @@
 /*
  *                            COPYRIGHT
  *
- *  PCB, interactive printed circuit board design
+ *  pcb-rnd, interactive printed circuit board design
+ *  (this file is based on PCB, interactive printed circuit board design)
  *  Copyright (C) 1994,1995,1996 Thomas Nau
  *  pcb-rnd, interactive printed circuit board design
  *  Copyright (C) 2017 Alain Vigne
@@ -20,9 +21,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- *  Contact addresses for paper mail and Email:
- *  Thomas Nau, Schlehenweg 15, 88471 Baustetten, Germany
- *  Thomas.Nau@rz.uni-ulm.de
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  *
  */
 
@@ -78,17 +80,10 @@
 #include "dlg_library_param.h"
 
 /** how often should the parametric footprint redrawn while the parameters are
-    changing - in milisec */
-#define PARAM_REFRESH_RATE_MS 100
+    changing - in millisec */
+#define PARAM_REFRESH_RATE_MS 500
 
-/** \todo Open an empty file. Launch the Netlist window ... then : \n <tt>
-(pcb-rnd:14394): Gtk-CRITICAL **: IA__gtk_widget_show_all: assertion 'GTK_IS_WIDGET (widget)' failed \n
-(pcb-rnd:14394): Gtk-CRITICAL **: IA__gtk_tree_view_set_model: assertion 'GTK_IS_TREE_VIEW (tree_view)' failed \n
-Error: can't update netlist window: there is no netlist loaded. \n
-(pcb-rnd:14394): Gtk-CRITICAL **: IA__gtk_window_present_with_time: assertion 'GTK_IS_WINDOW (window)' failed </tt>
-*/
-
-/** \file   wt_library.c
+/** \file   dlg_library.c
     \brief  Implementation of \ref pcb_gtk_library_t widget.
  */
 
@@ -105,8 +100,8 @@ static GtkWidget *library_window;
  */
 #define LIBRARY_FILTER_INTERVAL 200
 
-/** */
-static gint library_window_configure_event_cb(GtkWidget * widget, GdkEventConfigure * ev, gpointer data)
+/** GtkWidget "configure_event" signal emitted when the size, position or stacking of the widget's window has changed. */
+static gint library_window_configure_event_cb(GtkWidget * widget, GdkEvent * ev, gpointer data)
 {
 	wplc_config_event(widget, &hid_gtk_wgeo.library_x, &hid_gtk_wgeo.library_y, &hid_gtk_wgeo.library_width,
 										&hid_gtk_wgeo.library_height);
@@ -121,19 +116,15 @@ enum {
 };
 
 
-/** Processes the response returned by the library dialog.
-
-    This function handles the response \p arg1 of the library \p dialog .
-
-    Parameter \p user_data is a pointer on the relevant toplevel structure.
+/** Processes the \p response_id returned by the library \p dialog.
 
     \param dialog        The library dialog.
-    \param arg1          The response ID.
-    \param user_data     pointer to ??FIXME??
+    \param response_id   The dialog response ID.
+    \param user_data     not used (NULL)
  */
-static void library_window_callback_response(GtkDialog * dialog, gint arg1, gpointer user_data)
+static void library_window_callback_response(GtkDialog * dialog, gint response_id, gpointer user_data)
 {
-	switch (arg1) {
+	switch (response_id) {
 	case GTK_RESPONSE_CLOSE:
 	case GTK_RESPONSE_DELETE_EVENT:
 		gtk_widget_destroy(GTK_WIDGET(library_window));
@@ -279,71 +270,6 @@ static gboolean lib_model_filter_visible_func(GtkTreeModel * model, GtkTreeIter
 	return ret;
 }
 
-/** Handles activation (e.g. double-clicking) of a component row.
-    As a convenience to the user `,` expand `/` contract any node with children.
-
-    \param tree_view     The component treeview.
-    \param path          The GtkTreePath to the activated row.
-    \param column        The GtkTreeViewColumn in which the activation occurre
-    \param user_data     The component selection dialog.
- */
-static void tree_row_activated(GtkTreeView * tree_view, GtkTreePath * path, GtkTreeViewColumn * column, gpointer user_data)
-{
-	GtkTreeModel *model;
-	GtkTreeIter iter;
-
-	model = gtk_tree_view_get_model(tree_view);
-	gtk_tree_model_get_iter(model, &iter, path);
-
-	if (!gtk_tree_model_iter_has_child(model, &iter))
-		return;
-
-	if (gtk_tree_view_row_expanded(tree_view, path))
-		gtk_tree_view_collapse_row(tree_view, path);
-	else
-		gtk_tree_view_expand_row(tree_view, path, FALSE);
-}
-
-/** Handles CTRL-C keypress in the TreeView.
-
-    Keypress activation handler:
-    If CTRL-C is pressed, copy footprint name into the clipboard.
-
-    \param tree_view     The component treeview.
-    \param event         The GdkEventKey with keypress info.
-    \param user_data     Not used.
-    \return              `TRUE` if CTRL-C event was handled, `FALSE` otherwise.
- */
-static gboolean tree_row_key_pressed(GtkTreeView * tree_view, GdkEventKey * event, gpointer user_data)
-{
-	GtkTreeSelection *selection;
-	GtkTreeModel *model;
-	GtkTreeIter iter;
-	GtkClipboard *clipboard;
-	const gchar *compname;
-	guint default_mod_mask = gtk_accelerator_get_default_mod_mask();
-
-	/* Handle both lower- and uppercase `c' */
-	if (((event->state & default_mod_mask) != GDK_CONTROL_MASK)
-			|| ((event->keyval != GDK_KEY_c) && (event->keyval != GDK_KEY_C)))
-		return FALSE;
-
-	selection = gtk_tree_view_get_selection(tree_view);
-	g_return_val_if_fail(selection != NULL, TRUE);
-
-	if (!gtk_tree_selection_get_selected(selection, &model, &iter))
-		return TRUE;
-
-	gtk_tree_model_get(model, &iter, MENU_NAME_COLUMN, &compname, -1);
-
-	clipboard = gtk_clipboard_get(GDK_SELECTION_CLIPBOARD);
-	g_return_val_if_fail(clipboard != NULL, TRUE);
-
-	gtk_clipboard_set_text(clipboard, compname, -1);
-
-	return TRUE;
-}
-
 static void library_window_preview_refresh(pcb_gtk_library_t * library_window, const char *name, pcb_fplibrary_t * entry)
 {
 	GString *pt;
@@ -358,7 +284,7 @@ static void library_window_preview_refresh(pcb_gtk_library_t * library_window, c
 		fullp = entry->data.fp.loc_info;
 	}
 	pcb_crosshair_set_mode(PCB_MODE_ARROW);
-	if (pcb_element_load_to_buffer(PCB_PASTEBUFFER, name == NULL ? fullp : name))
+	if (pcb_element_load_to_buffer(PCB_PASTEBUFFER, name == NULL ? fullp : name, NULL))
 		pcb_crosshair_set_mode(PCB_MODE_PASTE_BUFFER);
 
 	/* update the preview with new symbol data */
@@ -454,7 +380,8 @@ static void library_window_callback_tree_selection_changed(GtkTreeSelection * se
 		lib_param_del_timer(library_window);
 		if (name == NULL) {
 			gtk_entry_set_text(library_window->entry_filter, orig);
-			g_source_remove(library_window->filter_timeout); /* block the above edit from collapsing the tree */
+			if (library_window->filter_timeout != 0)
+				g_source_remove(library_window->filter_timeout); /* block the above edit from collapsing the tree */
 			library_window->filter_timeout = 0;
 			gtk_tree_model_filter_refilter((GtkTreeModelFilter *) model);
 
@@ -578,7 +505,7 @@ static void library_window_callback_filter_button_clicked(GtkButton * button, gp
 
 }
 
-/** Creates the tree model for the "Library" view.
+/** Creates the tree model for the "Library" treeview.
 
     Creates a tree where the branches are the available library
     sources and the leaves are the footprints.
@@ -653,7 +580,163 @@ static void library_window_callback_refresh_library(GtkButton * button, gpointer
 	gtk_tree_view_set_model(library_window->libtreeview, model);
 }
 
-/** Creates the treeview for the "Library" view */
+/** Handles activation (e.g. double-clicking) of a component row.
+    As a convenience to the user GTK provides Shift-Arrow Left, Right to expand or contract any node with children.
+
+    \param tree_view     The component treeview.
+    \param path          The GtkTreePath to the activated row.
+    \param column        The GtkTreeViewColumn in which the activation occurred
+    \param user_data     The library dialog.
+ */
+static void tree_row_activated(GtkTreeView * tree_view, GtkTreePath * path, GtkTreeViewColumn * column, gpointer user_data)
+{
+	GtkTreeModel *model;
+	GtkTreeIter iter;
+
+	model = gtk_tree_view_get_model(tree_view);
+	gtk_tree_model_get_iter(model, &iter, path);
+
+	if (!gtk_tree_model_iter_has_child(model, &iter)) {
+		library_window_callback_tree_selection_changed(gtk_tree_view_get_selection(tree_view), user_data);
+		return;
+	}
+
+	if (gtk_tree_view_row_expanded(tree_view, path))
+		gtk_tree_view_collapse_row(tree_view, path);
+	else
+		gtk_tree_view_expand_row(tree_view, path, FALSE);
+}
+
+/** Handles key pressed in the TreeView. 
+
+    Key pressed activation handler:
+    If CTRL-C is pressed, copy footprint name into the clipboard.
+    If Enter  is pressed, row-activate the treeview.
+
+    \param tree_view     The component treeview.
+    \param event         The GdkEventKey with keypress info.
+    \param user_data     The library dialog.
+    \return              `TRUE` if key event was handled, `FALSE` otherwise.
+ */
+static gboolean treeview_key_press_cb(GtkTreeView * tree_view, GdkEventKey * event, gpointer user_data)
+{
+	GtkTreeSelection *selection;
+	GtkTreeModel *model;
+	GtkTreeIter iter;
+	GtkTreePath *path;
+	pcb_bool key_handled, arrow_key, enter_key, force_activate = pcb_false;
+	GtkClipboard *clipboard;
+	const gchar *compname;
+	guint default_mod_mask = gtk_accelerator_get_default_mod_mask();
+
+	arrow_key = ((event->keyval == GDK_KEY_Up) || (event->keyval == GDK_KEY_KP_Up)
+		|| (event->keyval == GDK_KEY_Down) || (event->keyval == GDK_KEY_KP_Down)
+		|| (event->keyval == GDK_KEY_KP_Page_Down) || (event->keyval == GDK_KEY_KP_Page_Up)
+		|| (event->keyval == GDK_KEY_Page_Down) || (event->keyval == GDK_KEY_Page_Up)
+		|| (event->keyval == GDK_KEY_KP_Home) || (event->keyval == GDK_KEY_KP_End)
+		|| (event->keyval == GDK_KEY_Home) || (event->keyval == GDK_KEY_End));
+	enter_key = (event->keyval == GDK_KEY_Return) || (event->keyval == GDK_KEY_KP_Enter);
+	key_handled = (enter_key || arrow_key);
+
+	/* Handle ctrl+c and ctrl+C: copy current name to clipboard */
+	if (((event->state & default_mod_mask) == GDK_CONTROL_MASK) && ((event->keyval == GDK_KEY_c) || (event->keyval == GDK_KEY_C))) {
+		selection = gtk_tree_view_get_selection(tree_view);
+		g_return_val_if_fail(selection != NULL, TRUE);
+
+		if (!gtk_tree_selection_get_selected(selection, &model, &iter))
+			return TRUE;
+
+		gtk_tree_model_get(model, &iter, MENU_NAME_COLUMN, &compname, -1);
+
+		clipboard = gtk_clipboard_get(GDK_SELECTION_CLIPBOARD);
+		g_return_val_if_fail(clipboard != NULL, TRUE);
+
+		gtk_clipboard_set_text(clipboard, compname, -1);
+
+		return FALSE;
+	}
+
+	if (!key_handled)
+		return FALSE;
+
+	/* If arrows (up or down), let GTK process the selection change. Then activate the new selected row. */
+	if (arrow_key) {
+		GtkWidgetClass *class = GTK_WIDGET_GET_CLASS(tree_view);
+
+		class->key_press_event(GTK_WIDGET(tree_view), event);
+	}
+
+	selection = gtk_tree_view_get_selection(tree_view);
+	g_return_val_if_fail(selection != NULL, TRUE);
+
+	if (!gtk_tree_selection_get_selected(selection, &model, &iter))
+		return TRUE;
+
+	/* arrow key should activate the row only if it's a leaf (real footprint), for
+	   display, but shouldn't open/close levels visited or pop up the parametric
+	   footprint dialog */
+	if (arrow_key) {
+		pcb_fplibrary_t *entry = NULL;
+
+		gtk_tree_model_get(model, &iter, MENU_ENTRY_COLUMN, &entry, -1);
+
+		if ((entry != NULL) && (entry->type == LIB_FOOTPRINT) && (entry->data.fp.type != PCB_FP_PARAMETRIC))
+			force_activate = pcb_true;
+	}
+
+	/* Handle 'Enter' key and arrow keys as "activate" on plain footprints */
+	if (enter_key || force_activate) {
+		path = gtk_tree_model_get_path(model, &iter);
+		if (path != NULL) {
+			tree_row_activated(tree_view, path, NULL, user_data);
+		}
+		gtk_tree_path_free(path);
+	}
+
+	return TRUE;
+}
+
+/** Just handle the double-click to be equivalent to "row-activated" signal */
+static gboolean treeview_button_press_cb(GtkWidget  * widget, GdkEvent * ev, gpointer user_data)
+{
+	GtkTreeView *tv = GTK_TREE_VIEW(widget);
+	GtkTreeModel *model;
+	GtkTreeIter iter;
+	GtkTreePath *path;
+
+	if (ev->button.type == GDK_2BUTTON_PRESS) {
+		model = gtk_tree_view_get_model(tv);
+		gtk_tree_view_get_path_at_pos(tv, ev->button.x, ev->button.y, &path, NULL, NULL, NULL);
+		if (path != NULL) {
+			gtk_tree_model_get_iter(model, &iter, path);
+			tree_row_activated(tv, path, NULL, user_data);
+		}
+	}
+
+	return FALSE;
+}
+
+static gboolean treeview_button_release_cb(GtkWidget  * widget, GdkEvent * ev, gpointer user_data)
+{
+	GtkTreeView *tv = GTK_TREE_VIEW(widget);
+	GtkTreeModel *model;
+	GtkTreeIter iter;
+	GtkTreePath *path;
+
+	model = gtk_tree_view_get_model(tv);
+	gtk_tree_view_get_path_at_pos(tv, ev->button.x, ev->button.y, &path, NULL, NULL, NULL);
+	if (path != NULL) {
+		gtk_tree_model_get_iter(model, &iter, path);
+		/* Do not activate the row if LEFT-click on a "parent category" row. */
+		if (ev->button.button != 1 || !gtk_tree_model_iter_has_child(model, &iter))
+			tree_row_activated(tv, path, NULL, user_data);
+	}
+
+	return FALSE;
+}
+
+
+/** Creates the treeview for the "Library" dialog */
 static GtkWidget *create_lib_treeview(pcb_gtk_library_t * library_window)
 {
 	GtkWidget *libtreeview, *vbox, *scrolled_win, *label, *hbox, *entry, *button;
@@ -664,12 +747,8 @@ static GtkWidget *create_lib_treeview(pcb_gtk_library_t * library_window)
 	/* -- library selection view -- */
 
 	/* vertical box for footprint selection and search entry */
-	/* GTK3 issue there ? */
-	vbox = GTK_WIDGET(g_object_new(GTK_TYPE_VBOX,
-																 /* GtkContainer */
-																 "border-width", 5,
-																 /* GtkBox */
-																 "homogeneous", FALSE, "spacing", 5, NULL));
+	vbox = gtkc_vbox_new(FALSE, 5);
+	gtk_container_set_border_width(GTK_CONTAINER(vbox), 5);
 
 	child_model = create_lib_tree_model(library_window);
 	model = (GtkTreeModel *) g_object_new(GTK_TYPE_TREE_MODEL_FILTER, "child-model", child_model, "virtual-root", NULL, NULL);
@@ -684,14 +763,13 @@ static GtkWidget *create_lib_treeview(pcb_gtk_library_t * library_window)
 																				/* GtkTreeView */
 																				"model", model, "rules-hint", TRUE, "headers-visible", FALSE, NULL));
 
-	g_signal_connect(libtreeview, "row-activated", G_CALLBACK(tree_row_activated), NULL);
+	g_signal_connect(libtreeview, "button-press-event", G_CALLBACK(treeview_button_press_cb), library_window);
+	g_signal_connect(libtreeview, "button-release-event", G_CALLBACK(treeview_button_release_cb), library_window);
 
-	g_signal_connect(libtreeview, "key-press-event", G_CALLBACK(tree_row_key_pressed), NULL);
+	g_signal_connect(libtreeview, "key-press-event", G_CALLBACK(treeview_key_press_cb), library_window);
 
-	/* connect callback to selection */
 	library_window->selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(libtreeview));
 	gtk_tree_selection_set_mode(library_window->selection, GTK_SELECTION_SINGLE);
-	g_signal_connect(library_window->selection, "changed", G_CALLBACK(library_window_callback_tree_selection_changed), library_window);
 
 	/* insert a column to treeview for library/symbol name */
 	renderer = GTK_CELL_RENDERER(g_object_new(GTK_TYPE_CELL_RENDERER_TEXT,
@@ -714,9 +792,7 @@ static GtkWidget *create_lib_treeview(pcb_gtk_library_t * library_window)
 
 
 	/* -- filter area -- */
-	hbox = GTK_WIDGET(g_object_new(GTK_TYPE_HBOX,
-																 /* GtkBox */
-																 "homogeneous", FALSE, "spacing", 3, NULL));
+	hbox = gtkc_hbox_new(FALSE, 3);
 
 	/* create the entry label */
 	label = GTK_WIDGET(g_object_new(GTK_TYPE_LABEL,
@@ -749,7 +825,7 @@ static GtkWidget *create_lib_treeview(pcb_gtk_library_t * library_window)
 																	 "sensitive", TRUE,
 																	 /* GtkButton */
 																	 "relief", GTK_RELIEF_NONE, NULL));
-	gtk_container_add(GTK_CONTAINER(button), gtk_image_new_from_stock(GTK_STOCK_EDIT, GTK_ICON_SIZE_SMALL_TOOLBAR));
+	gtk_container_add(GTK_CONTAINER(button), gtk_image_new_from_icon_name(GTK_STOCK_EDIT, GTK_ICON_SIZE_SMALL_TOOLBAR));
 	g_signal_connect(button, "clicked", G_CALLBACK(library_window_callback_edit_button_clicked), library_window);
 	/* add the clear button to the filter area */
 	gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
@@ -763,7 +839,7 @@ static GtkWidget *create_lib_treeview(pcb_gtk_library_t * library_window)
 																	 "sensitive", FALSE,
 																	 /* GtkButton */
 																	 "relief", GTK_RELIEF_NONE, NULL));
-	gtk_container_add(GTK_CONTAINER(button), gtk_image_new_from_stock(GTK_STOCK_CLEAR, GTK_ICON_SIZE_SMALL_TOOLBAR));
+	gtk_container_add(GTK_CONTAINER(button), gtk_image_new_from_icon_name("edit-clear", GTK_ICON_SIZE_SMALL_TOOLBAR));
 	g_signal_connect(button, "clicked", G_CALLBACK(library_window_callback_filter_button_clicked), library_window);
 	/* add the clear button to the filter area */
 	gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
@@ -777,7 +853,7 @@ static GtkWidget *create_lib_treeview(pcb_gtk_library_t * library_window)
 																	 "sensitive", TRUE,
 																	 /* GtkButton */
 																	 "relief", GTK_RELIEF_NONE, NULL));
-	gtk_container_add(GTK_CONTAINER(button), gtk_image_new_from_stock(GTK_STOCK_REFRESH, GTK_ICON_SIZE_SMALL_TOOLBAR));
+	gtk_container_add(GTK_CONTAINER(button), gtk_image_new_from_icon_name("view-refresh", GTK_ICON_SIZE_SMALL_TOOLBAR));
 	/* add the refresh button to the filter area */
 	gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
 	g_signal_connect(button, "clicked", G_CALLBACK(library_window_callback_refresh_library), library_window);
@@ -802,7 +878,7 @@ static GObject *library_window_constructor(GType type, guint n_construct_propert
 	GtkWidget *hpaned, *notebook;
 	GtkWidget *libview;
 	GtkWidget *preview, *preview_text;
-	GtkWidget *alignment, *frame;
+	GtkWidget *frame;
 
 	/* chain up to constructor of parent class */
 	object = G_OBJECT_CLASS(library_window_parent_class)->constructor(type, n_construct_properties, construct_params);
@@ -820,9 +896,8 @@ static GObject *library_window_constructor(GType type, guint n_construct_propert
 	g_object_set(gtk_dialog_get_content_area(GTK_DIALOG(lib_window)), "homogeneous", FALSE, NULL);
 
 	/* horizontal pane containing selection and preview */
-	hpaned = GTK_WIDGET(g_object_new(GTK_TYPE_HPANED,
-																	 /* GtkContainer */
-																	 "border-width", 5, NULL));
+	hpaned = gtkc_hpaned_new();
+	gtk_container_set_border_width(GTK_CONTAINER(hpaned), 5);
 	lib_window->hpaned = hpaned;
 
 	/* notebook for library views */
@@ -840,12 +915,6 @@ static GObject *library_window_constructor(GType type, guint n_construct_propert
 	frame = GTK_WIDGET(g_object_new(GTK_TYPE_FRAME,
 																	/* GtkFrame */
 																	"label", _("Preview"), NULL));
-	alignment = GTK_WIDGET(g_object_new(GTK_TYPE_ALIGNMENT,
-																			/* GtkAlignment */
-																			"left-padding", 5,
-																			"right-padding", 5,
-																			"top-padding", 5,
-																			"bottom-padding", 5, "xscale", 1.0, "yscale", 1.0, "xalign", 0.5, "yalign", 0.5, NULL));
 
 	preview = pcb_gtk_preview_pinout_new(lwcom, lwcom->init_drawing_widget, lwcom->preview_expose, NULL);
 	gtk_widget_set_size_request(preview, 150, 150);
@@ -857,8 +926,7 @@ static GObject *library_window_constructor(GType type, guint n_construct_propert
 		gtk_box_pack_start(GTK_BOX(vbox), preview, TRUE, TRUE, 0);
 		gtk_box_pack_end(GTK_BOX(vbox), preview_text, FALSE, FALSE, 0);
 
-		gtk_container_add(GTK_CONTAINER(alignment), vbox);
-		gtk_container_add(GTK_CONTAINER(frame), alignment);
+		gtk_container_add(GTK_CONTAINER(frame), vbox);
 	}
 
 	/* set preview of library window */
@@ -942,8 +1010,8 @@ void pcb_gtk_library_create(pcb_gtk_common_t *com)
 	lwcom = com;
 	library_window = (GtkWidget *) g_object_new(GHID_TYPE_LIBRARY_WINDOW, NULL);
 
-	g_signal_connect(library_window, "response", G_CALLBACK(library_window_callback_response), NULL);
-	g_signal_connect(G_OBJECT(library_window), "configure_event", G_CALLBACK(library_window_configure_event_cb), NULL);
+	g_signal_connect(GTK_DIALOG(library_window), "response", G_CALLBACK(library_window_callback_response), NULL);
+	g_signal_connect(library_window, "configure_event", G_CALLBACK(library_window_configure_event_cb), NULL);
 	gtk_window_set_default_size(GTK_WINDOW(library_window), hid_gtk_wgeo.library_width, hid_gtk_wgeo.library_height);
 
 	gtk_window_set_title(GTK_WINDOW(library_window), _("pcb-rnd Library"));
