@@ -3,5 +3,5 @@
 #include "hid.h"
 #include "glue.h"
 
-void ghid_pinout_window_show(pcb_gtk_common_t *com, pcb_element_t *element);
+void ghid_pinout_window_show(pcb_gtk_common_t *com, pcb_subc_t *subc);
 
