@@ -19,6 +19,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  */
 
 /* object property edit dialog */
@@ -118,8 +122,6 @@ static void list_cursor_changed_cb(GtkWidget * tree, pcb_gtk_dlg_propedit_t * dl
 
 	gtk_tree_model_get(tm, &iter, 0, &prop, 1, &comm, -1);
 
-	printf("prop: %s!\n", prop);
-
 	val_combo_reset(dlg);
 
 	val = dlg->propedit_query(dlg->propedit_pe, "v1st", prop, NULL, 0);
@@ -280,6 +282,7 @@ static void do_apply_cb(GtkWidget * tree, pcb_gtk_dlg_propedit_t * dlg)
 			list_cursor_changed_cb(dlg->tree, dlg);
 			if ((*val == '+') || (*val == '-') || (*val == '#'))
 				gtk_entry_set_text(GTK_ENTRY(dlg->entry_val), val);	/* keep relative and forced absolute values intact for a reapply */
+			pcb_gui->invalidate_all();
 		}
 		else
 			pcb_message(PCB_MSG_WARNING, "Failed to change any object - %s is possibly invalid value for %s\n", val, prop);
