@@ -1,7 +1,8 @@
 /*
  *                            COPYRIGHT
  *
- *  PCB, interactive printed circuit board design
+ *  pcb-rnd, interactive printed circuit board design
+ *  (this file is based on PCB, interactive printed circuit board design)
  *  Copyright (C) 1994,1995,1996 Thomas Nau
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -18,6 +19,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  */
 
 /* This file written by Bill Wilson for the PCB Gtk port.
@@ -1617,14 +1622,19 @@ void ghid_config_text_scale_update(void)
 		gtk_spin_button_set_value(GTK_SPIN_BUTTON(config_text_spin_button), (gdouble) conf_core.design.text_scale);
 }
 
-static void config_close_cb(gpointer data)
+static void config_do_close(int apply)
 {
+	if (config_window == NULL) /* may get the event from multiple sources */
+		return;
+
 	/* Config pages may need to check for modified entries, use as default
 	   |  options, etc when the config window is closed.
 	 */
-	config_sizes_apply();
-	config_layers_apply();
-	config_library_apply();
+	if (apply) {
+		config_sizes_apply();
+		config_layers_apply();
+		config_library_apply();
+	}
 
 	config_sizes_vbox = NULL;
 	config_sizes_tab_vbox = NULL;
@@ -1637,14 +1647,14 @@ static void config_close_cb(gpointer data)
 	config_window = NULL;
 }
 
+static void config_close_cb(gpointer data)
+{
+	config_do_close(1);
+}
+
 static void config_destroy_cb(gpointer data)
 {
-	config_sizes_vbox = NULL;
-	config_increments_vbox = NULL;
-	config_groups_vbox = config_groups_table = NULL;
-	config_groups_window = NULL;
-	gtk_widget_destroy(config_window);
-	config_window = NULL;
+	config_do_close(0);
 }
 
 static void config_selection_changed_cb(GtkTreeSelection * selection, gpointer data)
