@@ -18,6 +18,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  */
 
 #include "config.h"
@@ -127,6 +131,14 @@ static void list_epad_cb(void *ctx, pcb_board_t *pcb, pcb_element_t *element, pc
 	pcb_lookup_conn_by_obj(map, (pcb_any_obj_t *)pad, 0, found);
 }
 
+int list_subc_cb(void *ctx, pcb_board_t *pcb, pcb_subc_t *subc, int enter)
+{
+	PCB_PADSTACK_LOOP(subc->data) {
+		list_pstk_cb(ctx, pcb, padstack);
+	} PCB_END_LOOP;
+#warning subc TODO: heavy terminals - test on io_hyp
+	return 0;
+}
 
 int pcb_netmap_init(pcb_netmap_t *map, pcb_board_t *pcb)
 {
@@ -150,7 +162,7 @@ int pcb_netmap_init(pcb_netmap_t *map, pcb_board_t *pcb)
 		NULL, /* etext */
 		list_epin_cb,
 		list_epad_cb,
-		NULL, /* subc */
+		list_subc_cb, /* subc */
 		NULL, /* via */
 		NULL /* pstk */
 	);
