@@ -18,6 +18,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  */
 
 #include "config.h"
@@ -68,18 +72,21 @@ static void cross(FILE *f, pcb_coord_t x, pcb_coord_t y)
 }
 #endif
 
-void pcb_pline_to_lines(pcb_layer_t *dst, const pcb_pline_t *src, pcb_coord_t thickness, pcb_coord_t clearance, pcb_flag_t flags)
+pcb_cardinal_t pcb_pline_to_lines(pcb_layer_t *dst, const pcb_pline_t *src, pcb_coord_t thickness, pcb_coord_t clearance, pcb_flag_t flags)
 {
 	const pcb_vnode_t *v, *n;
 	pcb_pline_t *track = pcb_pline_dup_offset(src, -((thickness/2)+1));
+	pcb_cardinal_t cnt = 0;
 
 	v = &track->head;
 	do {
 		n = v->next;
 		pcb_line_new(dst, v->point[0], v->point[1], n->point[0], n->point[1], thickness, clearance, flags);
+		cnt++;
 	}
 	while((v = v->next) != &track->head);
 	pcb_poly_contour_del(&track);
+	return cnt;
 }
 
 pcb_bool pcb_pline_is_aligned(const pcb_pline_t *src)
@@ -155,7 +162,7 @@ static void add_track_seg(pcb_cpoly_edgetree_t *dst, pcb_coord_t x1, pcb_coord_t
 	e->y2 = y2;
 
 	pcb_box_bump_box(&dst->bbox, b);
-	pcb_r_insert_entry(dst->edge_tree, (pcb_box_t *)e, 0);
+	pcb_r_insert_entry(dst->edge_tree, (pcb_box_t *)e);
 }
 
 static void add_track(pcb_cpoly_edgetree_t *dst, pcb_pline_t *track)
@@ -190,7 +197,7 @@ pcb_cpoly_edgetree_t *pcb_cpoly_edgetree_create(const pcb_poly_t *src, pcb_coord
 
 	res->alloced = alloced;
 	res->used = 0;
-	res->edge_tree = pcb_r_create_tree(NULL, 0, 0);
+	res->edge_tree = pcb_r_create_tree();
 	res->bbox.X1 = res->bbox.Y1 = PCB_MAX_COORD;
 	res->bbox.X2 = res->bbox.Y2 = -PCB_MAX_COORD;
 
@@ -354,8 +361,8 @@ void pcb_cpoly_hatch_lines(pcb_layer_t *dst, const pcb_poly_t *src, pcb_cpoly_ha
 }
 
 
-static const char pcb_acts_PolyHatch[] = "PolyHatch([spacing], [combination of h|v|c])\nPolyHatch(interactive)\n";
-static const char pcb_acth_PolyHatch[] = "hatch the selected polygon(s) with lines of the current style; lines are drawn on the current layer";
+static const char pcb_acts_PolyHatch[] = "PolyHatch([spacing], [hvcp])\nPolyHatch(interactive)\n";
+static const char pcb_acth_PolyHatch[] = "hatch the selected polygon(s) with lines of the current style; lines are drawn on the current layer; flags are h:horizontal, v:vertical, c:contour, p:poly";
 static int pcb_act_PolyHatch(int argc, const char **argv, pcb_coord_t x, pcb_coord_t y)
 {
 	pcb_coord_t period = 0;
