@@ -18,6 +18,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  */
 
 #include "config.h"
@@ -58,7 +62,7 @@ static void sub_layer_all(pcb_board_t *pcb, pcb_tlp_session_t *result, pcb_layer
 			line_tmp.Clearance = result->edge_clearance;
 		}
 		else
-			line_tmp.Clearance = 0;
+			line_tmp.Clearance = 1;
 		pcb_poly_sub_obj(pcb->Data, layer, result->fill, PCB_TYPE_LINE, &line_tmp);
 	}
 	pcb_r_end(&it);
@@ -71,7 +75,7 @@ static void sub_layer_all(pcb_board_t *pcb, pcb_tlp_session_t *result, pcb_layer
 			arc_tmp.Clearance = result->edge_clearance;
 		}
 		else
-			arc_tmp.Clearance = 0;
+			arc_tmp.Clearance = 1;
 		pcb_poly_sub_obj(pcb->Data, layer, result->fill, PCB_TYPE_ARC, &arc_tmp);
 	}
 	pcb_r_end(&it);
@@ -97,35 +101,14 @@ static void sub_group_all(pcb_board_t *pcb, pcb_tlp_session_t *result, pcb_layer
 
 static void sub_global_all(pcb_board_t *pcb, pcb_tlp_session_t *result, pcb_layer_t *layer)
 {
-	pcb_pin_t *pin, pin_tmp;
-	pcb_pad_t *pad, pad_tmp;
+	pcb_pstk_t *ps, ps_tmp;
 	pcb_rtree_it_t it;
 
 #warning TODO: thermals: find out if any of our layers has thermal for the pin and if so, use that layer
-	for(pin = (pcb_pin_t *)pcb_r_first(pcb->Data->via_tree, &it); pin != NULL; pin = (pcb_pin_t *)pcb_r_next(&it)) {
-		memcpy(&pin_tmp, pin, sizeof(pin_tmp));
-		pin_tmp.Clearance = 1;
-		pcb_poly_sub_obj(pcb->Data, layer, result->fill, PCB_TYPE_VIA, &pin_tmp);
-	}
-	pcb_r_end(&it);
-
-	for(pin = (pcb_pin_t *)pcb_r_first(pcb->Data->pin_tree, &it); pin != NULL; pin = (pcb_pin_t *)pcb_r_next(&it)) {
-		memcpy(&pin_tmp, pin, sizeof(pin_tmp));
-		pin_tmp.Clearance = 1;
-		pcb_poly_sub_obj(pcb->Data, layer, result->fill, PCB_TYPE_PIN, &pin_tmp);
-	}
-	pcb_r_end(&it);
-
-	if (result->grp->type & ((PCB_LYT_BOTTOM) | (PCB_LYT_TOP))) {
-		int is_solder = !!(result->grp->type & PCB_LYT_BOTTOM);
-		for(pad = (pcb_pad_t *)pcb_r_first(pcb->Data->pad_tree, &it); pad != NULL; pad = (pcb_pad_t *)pcb_r_next(&it)) {
-			if (PCB_FLAG_TEST(PCB_FLAG_ONSOLDER, pad) == is_solder) {
-				memcpy(&pad_tmp, pad, sizeof(pad_tmp));
-#warning TODO: figure why this can not be 1
-				pad_tmp.Clearance = PCB_MM_TO_COORD(0.01);
-				pcb_poly_sub_obj(pcb->Data, layer, result->fill, PCB_TYPE_PAD, &pad_tmp);
-			}
-		}
+	for(ps = (pcb_pstk_t *)pcb_r_first(pcb->Data->padstack_tree, &it); ps != NULL; ps = (pcb_pstk_t *)pcb_r_next(&it)) {
+		memcpy(&ps_tmp, ps, sizeof(ps_tmp));
+		ps_tmp.Clearance = 1;
+		pcb_poly_sub_obj(pcb->Data, layer, result->fill, PCB_TYPE_PSTK, &ps_tmp);
 	}
 	pcb_r_end(&it);
 }
@@ -197,22 +180,37 @@ static void setup_remove_poly(pcb_board_t *pcb, pcb_tlp_session_t *result, pcb_l
 	sub_global_all(pcb, result, layer);
 }
 
-static void trace_contour(pcb_board_t *pcb, pcb_tlp_session_t *result, int tool_idx, pcb_coord_t extra_offs)
+static pcb_cardinal_t trace_contour(pcb_board_t *pcb, pcb_tlp_session_t *result, int tool_idx, pcb_coord_t extra_offs)
 {
 	pcb_poly_it_t it;
 	pcb_polyarea_t *pa;
 	pcb_coord_t tool_dia = result->tools->dia[tool_idx];
+	pcb_cardinal_t cnt = 0;
 	
 	for(pa = pcb_poly_island_first(result->fill, &it); pa != NULL; pa = pcb_poly_island_next(&it)) {
 		pcb_pline_t *pl = pcb_poly_contour(&it);
 		if (pl != NULL) { /* we have a contour */
 			pcb_pline_to_lines(result->res_path, pl, tool_dia + extra_offs, 0, pcb_no_flags());
-			for(pl = pcb_poly_hole_first(&it); pl != NULL; pl = pcb_poly_hole_next(&it))
+			cnt++;
+			for(pl = pcb_poly_hole_first(&it); pl != NULL; pl = pcb_poly_hole_next(&it)) {
 				pcb_pline_to_lines(result->res_path, pl, tool_dia + extra_offs, 0, pcb_no_flags());
+				cnt++;
+			}
 		}
 	}
+	return cnt;
 }
 
+static void trace_spiral(pcb_board_t *pcb, pcb_tlp_session_t *result, int tool_idx, pcb_coord_t extra_offs)
+{
+	int n;
+	pcb_coord_t tool_dia = result->tools->dia[tool_idx];
+
+	while(trace_contour(pcb, result, tool_idx, extra_offs) > 0)
+		extra_offs += tool_dia;
+}
+
+
 int pcb_tlp_mill_copper_layer(pcb_tlp_session_t *result, pcb_layer_t *layer)
 {
 	pcb_board_t *pcb = pcb_data_get_top(layer->parent);
