@@ -18,6 +18,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  */
 
 #include "config.h"
@@ -37,6 +41,7 @@
 #include "undo.h"
 #include "plugins.h"
 #include "compat_misc.h"
+#include "obj_subc_parent.h"
 
 #include "rats.h"
 #include "pcb-mincut/graph.h"
@@ -46,7 +51,12 @@
 #include "rats_mincut_conf.h"
 conf_mincut_t conf_mincut;
 
-static void debprintf(const char *fmt, ...) {}
+/* define to 1 to enable debug prints */
+#if 0
+#	define debprintf pcb_trace
+#else
+	static void debprintf(const char *fmt, ...) {}
+#endif
 
 typedef struct short_conn_s short_conn_t;
 struct short_conn_s {
@@ -141,44 +151,33 @@ static int proc_short(pcb_any_obj_t *term, int ignore)
 	for (n = short_conns, gids = 2; n != NULL; n = n->next, gids++) {
 		char *s;
 		const char *typ;
-		pcb_element_t *parent;
+		pcb_subc_t *parent;
+		pcb_any_obj_t *o = (pcb_any_obj_t *)n->to;
+
 		n->gid = gids;
 		debprintf(" {%d} found %d %d/%p type %d from %d\n", n->gid, n->to_type, n->to->ID, (void *)n->to, n->type, n->from_id);
 		lut_by_oid[n->to->ID] = n;
 		lut_by_gid[n->gid] = n;
 
-		s = malloc(256);
 		parent = NULL;
 		switch (n->to_type) {
-		case PCB_TYPE_PIN:
-			typ = "pin";
-			parent = ((pcb_pin_t *) (n->to))->Element;
-			break;
-		case PCB_TYPE_VIA:
-			typ = "via";
-			parent = ((pcb_pin_t *) (n->to))->Element;
-			break;
-		case PCB_TYPE_PAD:
-			typ = "pad";
-			parent = ((pcb_pad_t *) (n->to))->Element;
-			break;
-		case PCB_TYPE_LINE:
-			typ = "line";
-			break;
-		default:
-			typ = "other";
-			break;
+			case PCB_TYPE_LINE: typ = "line";       parent = pcb_lobj_parent_subc(o->parent_type, &o->parent); break;
+			case PCB_TYPE_ARC:  typ = "arc";        parent = pcb_lobj_parent_subc(o->parent_type, &o->parent); break;
+			case PCB_TYPE_POLY: typ = "polygon";    parent = pcb_lobj_parent_subc(o->parent_type, &o->parent); break;
+			case PCB_TYPE_TEXT: typ = "text";       parent = pcb_lobj_parent_subc(o->parent_type, &o->parent); break;
+			case PCB_TYPE_PSTK: typ = "padstack";   parent = pcb_gobj_parent_subc(o->parent_type, &o->parent); break;
+			case PCB_TYPE_SUBC: typ = "subcircuit"; parent = pcb_gobj_parent_subc(o->parent_type, &o->parent); break;
+			default: typ = "<unknown>";             parent = NULL; break;
 		}
 		if (parent != NULL) {
-			pcb_text_t *name;
-			name = &parent->Name[1];
-			if ((name->TextString == NULL) || (*name->TextString == '\0'))
-				sprintf(s, "%s #%ld \\nof #%ld", typ, n->to->ID, parent->ID);
+			if (parent->refdes == NULL)
+				s = pcb_strdup_printf("%s #%ld \\nof #%ld", typ, n->to->ID, parent->ID);
 			else
-				sprintf(s, "%s #%ld \\nof %s", typ, n->to->ID, name->TextString);
+				s = pcb_strdup_printf("%s #%ld \\nof %s", typ, n->to->ID, parent->refdes);
 		}
 		else
-			sprintf(s, "%s #%ld", typ, n->to->ID);
+			s = pcb_strdup_printf("%s #%ld", typ, n->to->ID);
+
 		g->node2name[n->gid] = s;
 	}
 	g->node2name[0] = pcb_strdup("S");
@@ -213,13 +212,8 @@ static int proc_short(pcb_any_obj_t *term, int ignore)
 	S = NULL;
 	T = NULL;
 	for (n = short_conns; n != NULL; n = n->next) {
-		void *spare;
+		void *spare = ((pcb_any_obj_t *) n->to)->ratconn;
 
-		spare = NULL;
-		if (n->to_type == PCB_TYPE_PIN)
-			spare = ((pcb_pin_t *) n->to)->ratconn;
-		if (n->to_type == PCB_TYPE_PAD)
-			spare = ((pcb_pad_t *) n->to)->ratconn;
 		if (spare != NULL) {
 			void *net = &(((pcb_lib_menu_t *) spare)->Name[2]);
 			debprintf(" net=%s\n", net);
