@@ -18,6 +18,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  */
 #include "config.h"
 #include "props.h"
@@ -307,6 +311,7 @@ const char *propedit_query(void *pe, const char *cmd, const char *key, const cha
 		typ[1] = '\0';
 		p = pe->value;
 		switch(p->type) {
+			case PCB_PROPT_max:
 			case PCB_PROPT_invalid: return NULL;
 			case PCB_PROPT_STRING: typ[0] = 's'; return typ;
 			case PCB_PROPT_COORD:  typ[0] = 'c'; return typ;
