@@ -18,6 +18,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  */
 
 #include <ctype.h>
@@ -90,11 +94,17 @@ static void map_attr(void *ctx, const pcb_attribute_list_t *list)
 		free(big);
 }
 
+static void map_common(void *ctx, pcb_any_obj_t *obj)
+{
+#warning TODO: flags
+}
+
 static void map_line_cb(void *ctx, pcb_board_t *pcb, pcb_layer_t *layer, pcb_line_t *line)
 {
 	map_chk_skip(ctx, line);
 	map_add_prop(ctx, "p/trace/thickness", pcb_coord_t, line->Thickness);
 	map_add_prop(ctx, "p/trace/clearance", pcb_coord_t, line->Clearance/2);
+	map_common(ctx, (pcb_any_obj_t *)line);
 	map_attr(ctx, &line->Attributes);
 }
 
@@ -107,6 +117,7 @@ static void map_arc_cb(void *ctx, pcb_board_t *pcb, pcb_layer_t *layer, pcb_arc_
 	map_add_prop(ctx, "p/arc/height",      pcb_coord_t, arc->Height);
 	map_add_prop(ctx, "p/arc/angle/start", pcb_angle_t, arc->StartAngle);
 	map_add_prop(ctx, "p/arc/angle/delta", pcb_angle_t, arc->Delta);
+	map_common(ctx, (pcb_any_obj_t *)arc);
 	map_attr(ctx, &arc->Attributes);
 }
 
@@ -116,6 +127,7 @@ static void map_text_cb(void *ctx, pcb_board_t *pcb, pcb_layer_t *layer, pcb_tex
 	map_add_prop(ctx, "p/text/scale", int, text->Scale);
 	map_add_prop(ctx, "p/text/rotation", int, text->Direction);
 	map_add_prop(ctx, "p/text/string", String, text->TextString);
+	map_common(ctx, (pcb_any_obj_t *)text);
 	map_attr(ctx, &text->Attributes);
 }
 
@@ -123,6 +135,8 @@ static void map_poly_cb(void *ctx, pcb_board_t *pcb, pcb_layer_t *layer, pcb_pol
 {
 	map_chk_skip(ctx, poly);
 	map_attr(ctx, &poly->Attributes);
+	map_common(ctx, (pcb_any_obj_t *)poly);
+	map_add_prop(ctx, "p/trace/clearance", pcb_coord_t, poly->Clearance/2);
 }
 
 static void map_eline_cb(void *ctx, pcb_board_t *pcb, pcb_element_t *element, pcb_line_t *line)
@@ -130,6 +144,7 @@ static void map_eline_cb(void *ctx, pcb_board_t *pcb, pcb_element_t *element, pc
 	map_chk_skip(ctx, line);
 	map_line_cb(ctx, pcb, NULL, line);
 	map_attr(ctx, &line->Attributes);
+	map_common(ctx, (pcb_any_obj_t *)line);
 }
 
 static void map_earc_cb(void *ctx, pcb_board_t *pcb, pcb_element_t *element, pcb_arc_t *arc)
@@ -137,6 +152,7 @@ static void map_earc_cb(void *ctx, pcb_board_t *pcb, pcb_element_t *element, pcb
 	map_chk_skip(ctx, arc);
 	map_arc_cb(ctx, pcb, NULL, arc);
 	map_attr(ctx, &arc->Attributes);
+	map_common(ctx, (pcb_any_obj_t *)arc);
 }
 
 static void map_etext_cb(void *ctx, pcb_board_t *pcb, pcb_element_t *element, pcb_text_t *text)
@@ -144,6 +160,7 @@ static void map_etext_cb(void *ctx, pcb_board_t *pcb, pcb_element_t *element, pc
 	map_chk_skip(ctx, text);
 	map_text_cb(ctx, pcb, NULL, text);
 	map_attr(ctx, &text->Attributes);
+	map_common(ctx, (pcb_any_obj_t *)text);
 }
 
 static void map_epin_cb(void *ctx, pcb_board_t *pcb, pcb_element_t *element, pcb_pin_t *pin)
@@ -154,6 +171,7 @@ static void map_epin_cb(void *ctx, pcb_board_t *pcb, pcb_element_t *element, pcb
 	map_add_prop(ctx, "p/pin/mask",      pcb_coord_t, pin->Mask);
 	map_add_prop(ctx, "p/pin/hole",      pcb_coord_t, pin->DrillingHole);
 	map_attr(ctx, &pin->Attributes);
+	map_common(ctx, (pcb_any_obj_t *)pin);
 }
 
 static void map_epad_cb(void *ctx, pcb_board_t *pcb, pcb_element_t *element, pcb_pad_t *pad)
@@ -161,9 +179,21 @@ static void map_epad_cb(void *ctx, pcb_board_t *pcb, pcb_element_t *element, pcb
 	map_chk_skip(ctx, pad);
 	map_add_prop(ctx, "p/pad/mask",      pcb_coord_t, pad->Mask);
 	map_attr(ctx, &pad->Attributes);
+	map_common(ctx, (pcb_any_obj_t *)pad);
 }
 
+static void map_subc_cb_(void *ctx, pcb_board_t *pcb, pcb_subc_t *subc)
+{
+	map_chk_skip(ctx, subc);
+	map_attr(ctx, &subc->Attributes);
+	map_common(ctx, (pcb_any_obj_t *)subc);
+}
 
+static int map_subc_cb(void *ctx, pcb_board_t *pcb, pcb_subc_t *subc, int enter)
+{
+	map_subc_cb_(ctx, pcb, subc);
+	return 0;
+}
 
 static void map_via_cb(void *ctx, pcb_board_t *pcb, pcb_pin_t *via)
 {
@@ -173,6 +203,7 @@ static void map_via_cb(void *ctx, pcb_board_t *pcb, pcb_pin_t *via)
 	map_add_prop(ctx, "p/via/mask",      pcb_coord_t, via->Mask);
 	map_add_prop(ctx, "p/via/hole",      pcb_coord_t, via->DrillingHole);
 	map_attr(ctx, &via->Attributes);
+	map_common(ctx, (pcb_any_obj_t *)via);
 }
 
 static void map_pstk_cb(void *ctx, pcb_board_t *pcb, pcb_pstk_t *ps)
@@ -186,13 +217,14 @@ static void map_pstk_cb(void *ctx, pcb_board_t *pcb, pcb_pstk_t *ps)
 	map_add_prop(ctx, "p/padstack/proto", pcb_coord_t, ps->proto);
 
 	proto = pcb_pstk_get_proto(ps);
-	map_add_prop(ctx, "p/padstack/clearance", pcb_coord_t, ps->Clearance/2);
+	map_add_prop(ctx, "p/padstack/clearance", pcb_coord_t, ps->Clearance);
 	map_add_prop(ctx, "p/padstack/hole", pcb_coord_t, proto->hdia);
 	map_add_prop(ctx, "p/padstack/plated", int, proto->hplated);
 	map_add_prop(ctx, "p/padstack/htop", int, proto->htop);
 	map_add_prop(ctx, "p/padstack/hbottom", int, proto->hbottom);
 
 	map_attr(ctx, &ps->Attributes);
+	map_common(ctx, (pcb_any_obj_t *)ps);
 }
 
 void pcb_propsel_map_core(htsp_t *props)
@@ -204,7 +236,7 @@ void pcb_propsel_map_core(htsp_t *props)
 	pcb_loop_all(PCB, &ctx,
 		NULL, map_line_cb, map_arc_cb, map_text_cb, map_poly_cb,
 		NULL, map_eline_cb, map_earc_cb, map_etext_cb, map_epin_cb, map_epad_cb,
-		NULL,
+		map_subc_cb,
 		map_via_cb, map_pstk_cb
 	);
 }
@@ -238,6 +270,12 @@ static void set_attr(set_ctx_t *st, pcb_attribute_list_t *list)
 
 #define DONE { st->set_cnt++; pcb_undo_restore_serial(); return; }
 
+static int set_common(set_ctx_t *st, pcb_any_obj_t *obj)
+{
+#warning TODO: flags
+	return 0;
+}
+
 static void set_line_cb(void *ctx, pcb_board_t *pcb, pcb_layer_t *layer, pcb_line_t *line)
 {
 	set_ctx_t *st = (set_ctx_t *)ctx;
@@ -250,6 +288,8 @@ static void set_line_cb(void *ctx, pcb_board_t *pcb, pcb_layer_t *layer, pcb_lin
 		return;
 	}
 
+	if (set_common(st, (pcb_any_obj_t *)line)) return;
+
 	if (st->is_trace && st->c_valid && (strcmp(pn, "thickness") == 0) &&
 	    pcb_chg_obj_1st_size(PCB_TYPE_LINE, layer, line, NULL, st->c, st->c_absolute)) DONE;
 
@@ -269,6 +309,8 @@ static void set_arc_cb(void *ctx, pcb_board_t *pcb, pcb_layer_t *layer, pcb_arc_
 		return;
 	}
 
+	if (set_common(st, (pcb_any_obj_t *)arc)) return;
+
 	if (st->is_trace && st->c_valid && (strcmp(pn, "thickness") == 0) &&
 	    pcb_chg_obj_1st_size(PCB_TYPE_ARC, layer, arc, NULL, st->c, st->c_absolute)) DONE;
 
@@ -303,6 +345,8 @@ static void set_text_cb_any(void *ctx, pcb_board_t *pcb, int type, void *layer_o
 		return;
 	}
 
+	if (set_common(st, (pcb_any_obj_t *)text)) return;
+
 	if (st->d_valid && (strcmp(pn, "scale") == 0) &&
 	    pcb_chg_obj_size(type, layer_or_element, text, text, PCB_MIL_TO_COORD(st->d), st->d_absolute)) DONE;
 
@@ -336,9 +380,15 @@ static void set_text_cb(void *ctx, pcb_board_t *pcb, pcb_layer_t *layer, pcb_tex
 static void set_poly_cb(void *ctx, pcb_board_t *pcb, pcb_layer_t *layer, pcb_poly_t *poly)
 {
 	set_ctx_t *st = (set_ctx_t *)ctx;
+	const char *pn = st->name + 8;
 
 	set_chk_skip(st, poly);
 
+	if (set_common(st, (pcb_any_obj_t *)poly)) return;
+
+	if (st->is_trace && st->c_valid && (strcmp(pn, "clearance") == 0) &&
+	    pcb_chg_obj_clear_size(PCB_TYPE_POLY, layer, poly, NULL, st->c*2, st->c_absolute)) DONE;
+
 	if (st->is_attr) {
 		set_attr(st, &poly->Attributes);
 		return;
@@ -355,6 +405,8 @@ static void set_eline_cb(void *ctx, pcb_board_t *pcb, pcb_element_t *element, pc
 		set_attr(st, &line->Attributes);
 		return;
 	}
+
+	if (set_common(st, (pcb_any_obj_t *)line)) return;
 }
 
 static void set_earc_cb(void *ctx, pcb_board_t *pcb, pcb_element_t *element, pcb_arc_t *arc)
@@ -367,6 +419,8 @@ static void set_earc_cb(void *ctx, pcb_board_t *pcb, pcb_element_t *element, pcb
 		set_attr(st, &arc->Attributes);
 		return;
 	}
+
+	if (set_common(st, (pcb_any_obj_t *)arc)) return;
 }
 
 static void set_etext_cb(void *ctx, pcb_board_t *pcb, pcb_element_t *element, pcb_text_t *text)
@@ -386,6 +440,8 @@ static void set_epin_cb(void *ctx, pcb_board_t *pcb, pcb_element_t *element, pcb
 		return;
 	}
 
+	if (set_common(st, (pcb_any_obj_t *)pin)) return;
+
 	if (st->c_valid && (strcmp(pn, "thickness") == 0) &&
 	    pcb_chg_obj_1st_size(PCB_TYPE_PIN, pin->Element, pin, NULL, st->c, st->c_absolute)) DONE;
 
@@ -411,10 +467,33 @@ static void set_epad_cb(void *ctx, pcb_board_t *pcb, pcb_element_t *element, pcb
 		return;
 	}
 
+	if (set_common(st, (pcb_any_obj_t *)pad)) return;
+
 	if (st->c_valid && (strcmp(pn, "mask") == 0) &&
 	    pcb_chg_obj_mask_size(PCB_TYPE_PAD, pad->Element, pad, NULL, st->c, st->c_absolute)) DONE;
 }
 
+static void set_subc_cb_(void *ctx, pcb_board_t *pcb, pcb_subc_t *subc)
+{
+	set_ctx_t *st = (set_ctx_t *)ctx;
+
+	set_chk_skip(st, subc);
+
+	if (set_common(st, (pcb_any_obj_t *)subc)) return;
+
+	if (st->is_attr) {
+		set_attr(st, &subc->Attributes);
+		return;
+	}
+}
+
+static int set_subc_cb(void *ctx, pcb_board_t *pcb, pcb_subc_t *subc, int enter)
+{
+	set_subc_cb_(ctx, pcb, subc);
+	return 0;
+}
+
+
 static void set_via_cb(void *ctx, pcb_board_t *pcb, pcb_pin_t *via)
 {
 	set_ctx_t *st = (set_ctx_t *)ctx;
@@ -427,6 +506,8 @@ static void set_via_cb(void *ctx, pcb_board_t *pcb, pcb_pin_t *via)
 		return;
 	}
 
+	if (set_common(st, (pcb_any_obj_t *)via)) return;
+
 	if (st->c_valid && (strcmp(pn, "thickness") == 0) &&
 	    pcb_chg_obj_1st_size(PCB_TYPE_VIA, via, via, NULL, st->c, st->c_absolute)) DONE;
 
@@ -455,13 +536,22 @@ static void set_pstk_cb(void *ctx, pcb_board_t *pcb, pcb_pstk_t *ps)
 		return;
 	}
 
+	if (set_common(st, (pcb_any_obj_t *)ps)) return;
+
 	ca = i = (st->c != 0);
 	proto = pcb_pstk_get_proto(ps);
 
 	if (st->c_valid && (strcmp(pn, "clearance") == 0) &&
-	    pcb_chg_obj_clear_size(PCB_TYPE_PSTK, ps, ps, NULL, st->c*2, st->c_absolute)) DONE;
-	if (st->d_valid && (strcmp(pn, "rotation") == 0) &&
-	    pcb_obj_rotate(PCB_TYPE_PSTK, ps, ps, NULL, ps->x, ps->y, st->d)) DONE;
+	    pcb_chg_obj_clear_size(PCB_TYPE_PSTK, ps, ps, NULL, st->c, st->c_absolute)) DONE;
+	if (st->d_valid && (strcmp(pn, "rotation") == 0)) {
+		if (st->d_absolute) {
+			if (pcb_obj_rotate(PCB_TYPE_PSTK, ps, ps, NULL, ps->x, ps->y, st->d - ps->rot)) DONE;
+		}
+		else {
+			if (pcb_obj_rotate(PCB_TYPE_PSTK, ps, ps, NULL, ps->x, ps->y, st->d)) DONE;
+		}
+		return;
+	}
 	if (st->c_valid && (strcmp(pn, "xmirror") == 0) &&
 	    (pcb_pstk_change_instance(ps, NULL, NULL, NULL, &i, NULL) == 0)) DONE;
 	if (st->c_valid && (strcmp(pn, "smirror") == 0) &&
@@ -521,14 +611,14 @@ int pcb_propsel_set(const char *prop, const char *value)
 		MAYBE_PROP(ctx.is_trace, "p/line/", set_line_cb),
 		MAYBE_PROP(ctx.is_trace, "p/arc/", set_arc_cb),
 		MAYBE_PROP(0, "p/text/", set_text_cb),
-		MAYBE_ATTR(set_poly_cb),
+		MAYBE_PROP(ctx.is_trace, "p/poly/", set_poly_cb),
 		NULL,
 		MAYBE_ATTR(set_eline_cb),
 		MAYBE_ATTR(set_earc_cb),
 		MAYBE_PROP(0, "p/text/", set_etext_cb),
 		MAYBE_PROP(0, "p/pin/", set_epin_cb),
 		MAYBE_PROP(0, "p/pad/", set_epad_cb),
-		NULL, /* subc */
+		MAYBE_PROP(0, "p/subc/", set_subc_cb),
 		MAYBE_PROP(0, "p/via/", set_via_cb),
 		MAYBE_PROP(0, "p/padstack/", set_pstk_cb)
 	);
@@ -607,6 +697,18 @@ static void del_epad_cb(void *ctx, pcb_board_t *pcb, pcb_element_t *element, pcb
 	del_attr(ctx, &pad->Attributes);
 }
 
+static void del_subc_cb_(void *ctx, pcb_board_t *pcb, pcb_subc_t *subc)
+{
+	map_chk_skip(ctx, subc);
+	del_attr(ctx, &subc->Attributes);
+}
+
+static int del_subc_cb(void *ctx, pcb_board_t *pcb, pcb_subc_t *subc, int enter)
+{
+	del_subc_cb_(ctx, pcb, subc);
+	return 0;
+}
+
 static void del_via_cb(void *ctx, pcb_board_t *pcb, pcb_pin_t *via)
 {
 	map_chk_skip(ctx, via);
@@ -632,7 +734,7 @@ int pcb_propsel_del(const char *key)
 	pcb_loop_all(PCB, &st,
 		NULL, del_line_cb, del_arc_cb, del_text_cb, del_poly_cb,
 		NULL, del_eline_cb, del_earc_cb, del_etext_cb, del_epin_cb, del_epad_cb,
-		NULL, /* subc */
+		del_subc_cb,
 		del_via_cb, del_pstk_cb
 	);
 	return st.del_cnt;
