@@ -1,7 +1,8 @@
 /*
  *                            COPYRIGHT
  *
- *  PCB, interactive printed circuit board design
+ *  pcb-rnd, interactive printed circuit board design
+ *  (this file is based on PCB, interactive printed circuit board design)
  *  Copyright (C) 1994,1995,1996 Thomas Nau
  *  Copyright (C) 2017 Tibor 'Igor2' Palinkas
  *
@@ -22,9 +23,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- *  Contact addresses for paper mail and Email:
- *  Thomas Nau, Schlehenweg 15, 88471 Baustetten, Germany
- *  Thomas.Nau@rz.uni-ulm.de
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  *
  */
 #include "config.h"
@@ -219,7 +221,7 @@ DrillInfoTypePtr GetDrillInfo(pcb_data_t *top)
 	pcb_r_end(&it);
 
 	qsort(AllDrills->Drill, AllDrills->DrillN, sizeof(DrillType), DrillQSort);
-	return (AllDrills);
+	return AllDrills;
 }
 
 #define ROUND(x,n) ((int)(((x)+(n)/2)/(n))*(n))
