@@ -1,8 +1,10 @@
 /*
  *                            COPYRIGHT
  *
- *  PCB, interactive printed circuit board design
+ *  pcb-rnd, interactive printed circuit board design
+ *  (this file is based on PCB, interactive printed circuit board design)
  *  Copyright (C) 1994,1995,1996,1997,1998,1999 Thomas Nau
+ *  Copyright (C) 2018 Tibor 'Igor2' Palinkas
  *
  *  This module, report.c, was written and is Copyright (C) 1997 harry eaton
  *
@@ -20,9 +22,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
- *  Contact addresses for paper mail and Email:
- *  Thomas Nau, Schlehenweg 15, 88471 Baustetten, Germany
- *  Thomas.Nau@rz.uni-ulm.de
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  *
  */
 
@@ -37,6 +40,7 @@
 #include "crosshair.h"
 #include "board.h"
 #include "data.h"
+#include "data_it.h"
 #include "drill.h"
 #include "error.h"
 #include "search.h"
@@ -59,6 +63,7 @@
 #include "obj_all.h"
 #include "obj_pstk.h"
 #include "obj_pstk_inlines.h"
+#include "obj_subc_parent.h"
 
 #include <genregex/regex_sei.h>
 
@@ -130,11 +135,22 @@ This is a shortcut for @code{Report(Object)}.
 	(((obj)->term != NULL) ? (obj)->term : ""), \
 	(((obj)->term != NULL) ? "\n" : "")
 
+static const char *grpname(pcb_layergrp_id_t gid)
+{
+	pcb_layergrp_t *grp = pcb_get_layergrp(PCB, gid);
+	if (grp == NULL)
+		return "<invalid>";
+	if ((grp->name == NULL) || (*grp->name == '\0'))
+		return "<anonymous>";
+	return grp->name;
+}
+
 static int ReportDialog(int argc, const char **argv, pcb_coord_t x, pcb_coord_t y)
 {
 	void *ptr1, *ptr2, *ptr3;
 	int type = REPORT_TYPES;
 	char *report = NULL;
+	pcb_subc_t *subc;
 
 	if ((argv != NULL) && (argv[0] != NULL)) {
 		if (pcb_strncasecmp(argv[0], "Subc", 4) == 0)
@@ -153,7 +169,7 @@ static int ReportDialog(int argc, const char **argv, pcb_coord_t x, pcb_coord_t
 
 #ifndef NDEBUG
 			if (pcb_gui->shift_is_pressed()) {
-				pcb_r_dump_tree(PCB->Data->padstack_tree->root, 0);
+				pcb_r_dump_tree(PCB->Data->padstack_tree, 0);
 				return 0;
 			}
 #endif
@@ -179,7 +195,7 @@ static int ReportDialog(int argc, const char **argv, pcb_coord_t x, pcb_coord_t
 			pcb_pin_t *via;
 #ifndef NDEBUG
 			if (pcb_gui->shift_is_pressed()) {
-				pcb_r_dump_tree(PCB->Data->via_tree->root, 0);
+				pcb_r_dump_tree(PCB->Data->via_tree, 0);
 				return 0;
 			}
 #endif
@@ -217,7 +233,7 @@ static int ReportDialog(int argc, const char **argv, pcb_coord_t x, pcb_coord_t
 			pcb_element_t *element;
 #ifndef NDEBUG
 			if (pcb_gui->shift_is_pressed()) {
-				pcb_r_dump_tree(PCB->Data->pin_tree->root, 0);
+				pcb_r_dump_tree(PCB->Data->pin_tree, 0);
 				return 0;
 			}
 #endif
@@ -265,7 +281,7 @@ static int ReportDialog(int argc, const char **argv, pcb_coord_t x, pcb_coord_t
 #ifndef NDEBUG
 			if (pcb_gui->shift_is_pressed()) {
 				pcb_layer_t *layer = (pcb_layer_t *) ptr1;
-				pcb_r_dump_tree(layer->line_tree->root, 0);
+				pcb_r_dump_tree(layer->line_tree, 0);
 				return 0;
 			}
 #endif
@@ -291,19 +307,19 @@ static int ReportDialog(int argc, const char **argv, pcb_coord_t x, pcb_coord_t
 			pcb_rat_t *line;
 #ifndef NDEBUG
 			if (pcb_gui->shift_is_pressed()) {
-				pcb_r_dump_tree(PCB->Data->rat_tree->root, 0);
+				pcb_r_dump_tree(PCB->Data->rat_tree, 0);
 				return 0;
 			}
 #endif
 			line = (pcb_rat_t *) ptr2;
 			report = pcb_strdup_printf("%m+RAT-LINE ID# %ld;  Flags:%s\n"
 									"FirstPoint(X,Y)  = %$mD; ID = %ld; "
-									"connects to layer group %d.\n"
+									"connects to layer group #%d (%s).\n"
 									"SecondPoint(X,Y) = %$mD; ID = %ld; "
-									"connects to layer group %d.\n",
+									"connects to layer group #%d (%s).\n",
 									USER_UNITMASK, line->ID, pcb_strflg_f2s(line->Flags, PCB_TYPE_LINE, NULL),
-									line->Point1.X, line->Point1.Y,
-									line->Point1.ID, line->group1, line->Point2.X, line->Point2.Y, line->Point2.ID, line->group2);
+									line->Point1.X, line->Point1.Y, line->Point1.ID, line->group1, grpname(line->group1),
+									line->Point2.X, line->Point2.Y, line->Point2.ID, line->group2, grpname(line->group2));
 			break;
 		}
 	case PCB_TYPE_ARC:
@@ -313,7 +329,7 @@ static int ReportDialog(int argc, const char **argv, pcb_coord_t x, pcb_coord_t
 #ifndef NDEBUG
 			if (pcb_gui->shift_is_pressed()) {
 				pcb_layer_t *layer = (pcb_layer_t *) ptr1;
-				pcb_r_dump_tree(layer->arc_tree->root, 0);
+				pcb_r_dump_tree(layer->arc_tree, 0);
 				return 0;
 			}
 #endif
@@ -349,7 +365,7 @@ static int ReportDialog(int argc, const char **argv, pcb_coord_t x, pcb_coord_t
 #ifndef NDEBUG
 			if (pcb_gui->shift_is_pressed()) {
 				pcb_layer_t *layer = (pcb_layer_t *) ptr1;
-				pcb_r_dump_tree(layer->polygon_tree->root, 0);
+				pcb_r_dump_tree(layer->polygon_tree, 0);
 				return 0;
 			}
 #endif
@@ -390,7 +406,7 @@ static int ReportDialog(int argc, const char **argv, pcb_coord_t x, pcb_coord_t
 			pcb_element_t *element;
 #ifndef NDEBUG
 			if (pcb_gui->shift_is_pressed()) {
-				pcb_r_dump_tree(PCB->Data->pad_tree->root, 0);
+				pcb_r_dump_tree(PCB->Data->pad_tree, 0);
 				return 0;
 			}
 #endif
@@ -436,7 +452,7 @@ static int ReportDialog(int argc, const char **argv, pcb_coord_t x, pcb_coord_t
 			pcb_element_t *element;
 #ifndef NDEBUG
 			if (pcb_gui->shift_is_pressed()) {
-				pcb_r_dump_tree(PCB->Data->element_tree->root, 0);
+				pcb_r_dump_tree(PCB->Data->element_tree, 0);
 				return 0;
 			}
 #endif
@@ -469,7 +485,7 @@ static int ReportDialog(int argc, const char **argv, pcb_coord_t x, pcb_coord_t
 			pcb_subc_t *subc;
 #ifndef NDEBUG
 			if (pcb_gui->shift_is_pressed()) {
-				pcb_r_dump_tree(PCB->Data->element_tree->root, 0);
+				pcb_r_dump_tree(PCB->Data->element_tree, 0);
 				return 0;
 			}
 #endif
@@ -489,7 +505,7 @@ static int ReportDialog(int argc, const char **argv, pcb_coord_t x, pcb_coord_t
 #ifndef NDEBUG
 		if (pcb_gui->shift_is_pressed()) {
 			pcb_layer_t *layer = (pcb_layer_t *) ptr1;
-			pcb_r_dump_tree(layer->text_tree->root, 0);
+			pcb_r_dump_tree(layer->text_tree, 0);
 			return 0;
 		}
 #endif
@@ -499,7 +515,7 @@ static int ReportDialog(int argc, const char **argv, pcb_coord_t x, pcb_coord_t
 			pcb_text_t *text;
 #ifndef NDEBUG
 			if (pcb_gui->shift_is_pressed()) {
-				pcb_r_dump_tree(PCB->Data->name_tree[PCB_ELEMNAME_IDX_VISIBLE()]->root, 0);
+				pcb_r_dump_tree(PCB->Data->name_tree[PCB_ELEMNAME_IDX_VISIBLE()], 0);
 				return 0;
 			}
 #endif
@@ -548,11 +564,17 @@ static int ReportDialog(int argc, const char **argv, pcb_coord_t x, pcb_coord_t
 		return 1;
 	}
 
-	if (report != NULL) {
-		/* create dialog box */
-		pcb_gui->report_dialog("Report", report);
-		free(report);
+	/* create dialog box */
+	subc = pcb_obj_parent_subc((pcb_any_obj_t *)ptr2);
+	if (subc != NULL) {
+		int len = strlen(report);
+		report = realloc(report, len + 256);
+		sprintf(report + len, "\nPart of subcircuit #%ld\n", subc->ID);
 	}
+
+	pcb_gui->report_dialog("Report", report);
+	free(report);
+
 	return 0;
 }
 
@@ -560,6 +582,8 @@ static int ReportFoundPins(int argc, const char **argv, pcb_coord_t x, pcb_coord
 {
 	gds_t list;
 	int col = 0;
+	gdl_iterator_t sit;
+	pcb_subc_t *subc;
 
 	gds_init(&list);
 	gds_append_str(&list, "The following pins/pads are FOUND:\n");
@@ -580,6 +604,14 @@ static int ReportFoundPins(int argc, const char **argv, pcb_coord_t x, pcb_coord
 	}
 	PCB_END_LOOP;
 
+	subclist_foreach(&PCB->Data->subc, &sit, subc) {
+		pcb_any_obj_t *o;
+		pcb_data_it_t it;
+		for(o = pcb_data_first(&it, subc->data, PCB_OBJ_CLASS_REAL); o != NULL; o = pcb_data_next(&it))
+			if ((o->term != NULL) && (PCB_FLAG_TEST(PCB_FLAG_FOUND, o)))
+				pcb_append_printf(&list, "%s-%s,%c", subc->refdes, o->term, ((col++ % (conf_report.plugins.report.columns + 1)) == conf_report.plugins.report.columns) ? '\n' : ' ');
+	}
+
 	pcb_gui->report_dialog("Report", list.array);
 	gds_uninit(&list);
 	return 0;
@@ -712,14 +744,12 @@ static int ReportAllNetLengths(int argc, const char **argv, pcb_coord_t x, pcb_c
 	return 0;
 }
 
-static int ReportNetLength(int argc, const char **argv, pcb_coord_t x, pcb_coord_t y)
+static int ReportNetLength_(int argc, const char **argv, pcb_coord_t x, pcb_coord_t y)
 {
 	pcb_coord_t length = 0;
 	char *netname = 0;
 	int found = 0;
 
-	pcb_gui->get_coords("Click on a connection", &x, &y);
-
 	/* Reset all connection flags and save an undo-state to get back
 	 * to the state the board was in when we started this function.
 	 *
@@ -787,10 +817,13 @@ static int ReportNetLength(int argc, const char **argv, pcb_coord_t x, pcb_coord
 	}
 	PCB_END_LOOP;
 
+	goto noelem;
+
 got_net_name:
 	pcb_reset_conns(pcb_false);
 	pcb_undo(pcb_true);
 
+noelem:;
 	{
 		char buf[50];
 		pcb_snprintf(buf, sizeof(buf), "%$m*", conf_core.editor.grid_unit->suffix, length);
@@ -803,6 +836,66 @@ got_net_name:
 	return 0;
 }
 
+static int ReportNetLength(int argc, const char **argv, pcb_coord_t x, pcb_coord_t y, int split)
+{
+
+	if (split) {
+		void *r1, *r2, *r3;
+		pcb_line_t *l;
+		pcb_layer_t *ly;
+		int type;
+		pcb_coord_t ox, oy;
+
+		pcb_gui->get_coords("Click on a copper line", &x, &y);
+
+		type = pcb_search_screen(x, y, PCB_TYPE_LINE, &r1, &r2, &r3);
+		if (type != PCB_TYPE_LINE) {
+			pcb_message(PCB_MSG_ERROR, "can't find a line to split\n");
+			return -1;
+		}
+		l = r2;
+		assert(l->parent_type == PCB_PARENT_LAYER);
+		ly = l->parent.layer;
+		if (!(pcb_layer_flags_(ly) & PCB_LYT_COPPER)) {
+			pcb_message(PCB_MSG_ERROR, "not a copper line, can't split it\n");
+			return -1;
+		}
+
+#define MINDIST PCB_MIL_TO_COORD(40)
+		if ((pcb_distance(l->Point1.X, l->Point1.Y, x, y) < MINDIST) || (pcb_distance(l->Point2.X, l->Point2.Y, x, y) < MINDIST)) {
+			pcb_message(PCB_MSG_ERROR, "Can not split near the endpoint of a line\n");
+			return -1;
+		}
+#undef MINDIST2
+
+		pcb_gui->log("The two arms of the net are:\n");
+		pcb_r_delete_entry(ly->line_tree, (pcb_box_t *)l);
+		ox = l->Point1.X; oy = l->Point1.Y; l->Point1.X = x; l->Point1.Y = y;
+		pcb_r_insert_entry(ly->line_tree, (pcb_box_t *)l);
+		ReportNetLength_(argc, argv, x, y);
+		pcb_r_delete_entry(ly->line_tree, (pcb_box_t *)l);
+		l->Point1.X = ox; l->Point1.Y = oy;
+		pcb_r_insert_entry(ly->line_tree, (pcb_box_t *)l);
+
+		pcb_r_delete_entry(ly->line_tree, (pcb_box_t *)l);
+		ox = l->Point2.X; oy = l->Point2.Y; l->Point2.X = x; l->Point2.Y = y;
+		pcb_r_insert_entry(ly->line_tree, (pcb_box_t *)l);
+		ReportNetLength_(argc, argv, x, y);
+		pcb_r_delete_entry(ly->line_tree, (pcb_box_t *)l);
+		l->Point2.X = ox; l->Point2.Y = oy;
+		pcb_r_insert_entry(ly->line_tree, (pcb_box_t *)l);
+
+		PCB_FLAG_SET(PCB_FLAG_SELECTED, l);
+
+		return 0;
+	}
+	else {
+/*		pcb_gui->get_coords("Click on a connection", &x, &y);*/
+		return ReportNetLength_(argc, argv, x, y);
+	}
+}
+
+
 static int ReportNetLengthByName(const char *tofind, pcb_coord_t x, pcb_coord_t y)
 {
 	char *netname = 0;
@@ -832,7 +925,7 @@ static int ReportNetLengthByName(const char *tofind, pcb_coord_t x, pcb_coord_t
 		if (re_sei_errno(regex) != 0) {
 			pcb_message(PCB_MSG_ERROR, _("regexp error: %s\n"), re_error_str(re_sei_errno(regex)));
 			re_sei_free(regex);
-			return (1);
+			return 1;
 		}
 	}
 
@@ -908,7 +1001,7 @@ static int ReportNetLengthByName(const char *tofind, pcb_coord_t x, pcb_coord_t
  * syntax:
  */
 
-static const char report_syntax[] = "Report(Object|DrillReport|FoundPins|NetLength|AllNetLengths|[,name])";
+static const char report_syntax[] = "Report(Object|DrillReport|FoundPins|NetLength|NetLengthTo|AllNetLengths|[,name])";
 
 static const char report_help[] = "Produce various report.";
 
@@ -958,7 +1051,9 @@ static int Report(int argc, const char **argv, pcb_coord_t x, pcb_coord_t y)
 	else if (pcb_strcasecmp(argv[0], "FoundPins") == 0)
 		return ReportFoundPins(argc - 1, argv + 1, x, y);
 	else if ((pcb_strcasecmp(argv[0], "NetLength") == 0) && (argc == 1))
-		return ReportNetLength(argc - 1, argv + 1, x, y);
+		return ReportNetLength(argc - 1, argv + 1, x, y, 0);
+	else if ((pcb_strcasecmp(argv[0], "NetLengthTo") == 0) && (argc == 1))
+		return ReportNetLength(argc - 1, argv + 1, x, y, 1);
 	else if (pcb_strcasecmp(argv[0], "AllNetLengths") == 0)
 		return ReportAllNetLengths(argc - 1, argv + 1, x, y);
 	else if ((pcb_strcasecmp(argv[0], "NetLength") == 0) && (argc == 2))
