@@ -1,7 +1,9 @@
 /*
  *                            COPYRIGHT
  *
- *  PCB, interactive printed circuit board design
+ *  pcb-rnd, interactive printed circuit board design
+ *  pcb-rnd, interactive printed circuit board design
+ *  (this file is based on PCB, interactive printed circuit board design)
  *  Copyright (C) 1994,1995,1996 Thomas Nau
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -18,9 +20,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- *  Contact addresses for paper mail and Email:
- *  Thomas Nau, Schlehenweg 15, 88471 Baustetten, Germany
- *  Thomas.Nau@rz.uni-ulm.de
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  *
  */
 /* functions used by 'rubberband moves' */
@@ -36,6 +39,7 @@
 
 #include "board.h"
 #include "data.h"
+#include "data_it.h"
 #include "data_list.h"
 #include "error.h"
 #include "event.h"
@@ -48,6 +52,7 @@
 #include "obj_rat_draw.h"
 #include "obj_line_op.h"
 #include "obj_line_draw.h"
+#include "obj_pstk_inlines.h"
 #include "plugins.h"
 #include "conf_core.h"
 #include "layer_grp.h"
@@ -382,6 +387,7 @@ struct rinfo {
 	pcb_layergrp_id_t group;
 	pcb_pin_t *pin;
 	pcb_pad_t *pad;
+	pcb_pstk_t *pstk;
 	pcb_point_t *point;
 	rubber_ctx_t *rbnd;
 	int delta_index;
@@ -418,6 +424,12 @@ static pcb_r_dir_t rat_callback(const pcb_box_t * box, void *cl)
 					rat->Point2.Y == (i->pad->Point1.Y + i->pad->Point2.Y) / 2 && rat->group2 == i->group)
 			pcb_rubber_band_create(rbnd, NULL, (pcb_line_t *) rat, 1,i->delta_index);
 		break;
+	case PCB_TYPE_PSTK:
+		if (rat->Point1.X == i->pstk->x && rat->Point1.Y == i->pstk->y)
+			pcb_rubber_band_create(rbnd, NULL, (pcb_line_t *) rat, 0,i->delta_index);
+		else if (rat->Point2.X == i->pstk->x && rat->Point2.Y == i->pstk->y)
+			pcb_rubber_band_create(rbnd, NULL, (pcb_line_t *) rat, 1,i->delta_index);
+		break;
 	case PCB_TYPE_LINE_POINT:
 		if (rat->group1 == i->group && rat->Point1.X == i->point->X && rat->Point1.Y == i->point->Y)
 			pcb_rubber_band_create(rbnd, NULL, (pcb_line_t *) rat, 0,i->delta_index);
@@ -459,6 +471,17 @@ static void CheckPinForRat(rubber_ctx_t *rbnd, pcb_pin_t *Pin)
 	pcb_r_search(PCB->Data->rat_tree, &Pin->BoundingBox, NULL, rat_callback, &info, NULL);
 }
 
+static void CheckPadstackForRat(rubber_ctx_t *rbnd, pcb_pstk_t *pstk)
+{
+	struct rinfo info;
+
+	info.type = PCB_TYPE_PSTK;
+	info.pstk = pstk;
+	info.rbnd = rbnd;
+	info.delta_index = 0;
+	pcb_r_search(PCB->Data->rat_tree, &pstk->BoundingBox, NULL, rat_callback, &info, NULL);
+}
+
 static void CheckLinePointForRat(rubber_ctx_t *rbnd, pcb_layer_t *Layer, pcb_point_t *Point)
 {
 	struct rinfo info;
@@ -516,32 +539,40 @@ static void CheckPinForRubberbandConnection(rubber_ctx_t *rbnd, pcb_pin_t *Pin)
  */
 static void CheckLinePointForRubberbandConnection(rubber_ctx_t *rbnd, pcb_layer_t *Layer, pcb_line_t *Line, pcb_point_t *LinePoint,int delta_index)
 {
-	pcb_layergrp_id_t group;
-	struct rubber_info info;
-	pcb_coord_t t = Line->Thickness / 2;
+	const pcb_layergrp_id_t group = pcb_layer_get_group_(Layer);
+	pcb_board_t * board = pcb_data_get_top(PCB->Data);
 
-	/* lookup layergroup and check all visible lines in this group */
-	info.radius = -1;
-	info.box.X1 = LinePoint->X - t;
-	info.box.X2 = LinePoint->X + t;
-	info.box.Y1 = LinePoint->Y - t;
-	info.box.Y2 = LinePoint->Y + t;
-	info.line = Line;
-	info.rbnd = rbnd;
-	info.X = LinePoint->X;
-	info.Y = LinePoint->Y;
-	info.delta_index = delta_index;
+	if(board == NULL)
+		board = PCB;
 
-	group = pcb_layer_get_group_(Layer);
-	PCB_COPPER_GROUP_LOOP(PCB->Data, group);
-	{
-		/* check all visible lines of the group member */
-		if (layer->meta.real.vis) {
-			info.layer = layer;
-			pcb_r_search(layer->line_tree, &info.box, NULL, rubber_callback, &info, NULL);
+	if(group >= 0) {
+		pcb_cardinal_t length = board->LayerGroups.grp[group].len;
+		pcb_cardinal_t entry;
+		const pcb_coord_t t = Line->Thickness / 2;
+		const int comb = Layer->comb & PCB_LYC_SUB;
+		struct rubber_info info;
+
+		info.radius = -1;
+		info.box.X1 = LinePoint->X - t;
+		info.box.X2 = LinePoint->X + t;
+		info.box.Y1 = LinePoint->Y - t;
+		info.box.Y2 = LinePoint->Y + t;
+		info.line = Line;
+		info.rbnd = rbnd;
+		info.X = LinePoint->X;
+		info.Y = LinePoint->Y;
+		info.delta_index = delta_index;
+
+		for(entry = 0;entry < length;++entry)	{
+			const pcb_layer_id_t layer_id = board->LayerGroups.grp[group].lid[entry];
+			pcb_layer_t * layer = &PCB->Data->Layer[layer_id];
+
+			if(layer->meta.real.vis && ((layer->comb & PCB_LYC_SUB) == comb))	{
+				info.layer = layer;
+				pcb_r_search(layer->line_tree, &info.box, NULL, rubber_callback, &info, NULL);
+			}
 		}
 	}
-	PCB_END_LOOP;
 }
 
 /* ---------------------------------------------------------------------------
@@ -551,32 +582,40 @@ static void CheckLinePointForRubberbandConnection(rubber_ctx_t *rbnd, pcb_layer_
  */
 static void CheckLinePointForRubberbandArcConnection(rubber_ctx_t *rbnd, pcb_layer_t *Layer, pcb_line_t *Line, pcb_point_t *LinePoint, pcb_bool Exact)
 {
-	pcb_layergrp_id_t group;
-	struct rubber_info info;
-	pcb_coord_t t = Line->Thickness / 2;
+	const pcb_layergrp_id_t group = pcb_layer_get_group_(Layer);
+	pcb_board_t * board = pcb_data_get_top(PCB->Data);
 
-	/* lookup layergroup and check all visible arcs in this group */
-	info.radius = Exact ? -1 : MAX(Line->Thickness / 2, 1);
-	info.box.X1 = LinePoint->X - t;
-	info.box.X2 = LinePoint->X + t;
-	info.box.Y1 = LinePoint->Y - t;
-	info.box.Y2 = LinePoint->Y + t;
-	info.line = Line;
-	info.rbnd = rbnd;
-	info.X = LinePoint->X;
-	info.Y = LinePoint->Y;
-	info.delta_index = 0;
+	if(board == NULL)
+		board = PCB;
 
-	group = pcb_layer_get_group_(Layer);
-	PCB_COPPER_GROUP_LOOP(PCB->Data, group);
-	{
-		/* check all visible lines of the group member */
-		if (layer->meta.real.vis) {
-			info.layer = layer;
-			pcb_r_search(layer->arc_tree, &info.box, NULL, rubber_callback_arc, &info, NULL);
+	if(group >= 0) {
+		pcb_cardinal_t length = board->LayerGroups.grp[group].len;
+		pcb_cardinal_t entry;
+		const pcb_coord_t t = Line->Thickness / 2;
+		const int comb = Layer->comb & PCB_LYC_SUB;
+		struct rubber_info info;
+
+		info.radius = Exact ? -1 : MAX(Line->Thickness / 2, 1);
+		info.box.X1 = LinePoint->X - t;
+		info.box.X2 = LinePoint->X + t;
+		info.box.Y1 = LinePoint->Y - t;
+		info.box.Y2 = LinePoint->Y + t;
+		info.line = Line;
+		info.rbnd = rbnd;
+		info.X = LinePoint->X;
+		info.Y = LinePoint->Y;
+		info.delta_index = 0;
+
+		for(entry = 0;entry < length;++entry)	{
+			const pcb_layer_id_t layer_id = board->LayerGroups.grp[group].lid[entry];
+			pcb_layer_t * layer = &PCB->Data->Layer[layer_id];
+
+			if(layer->meta.real.vis && ((layer->comb & PCB_LYC_SUB) == comb))	{
+				info.layer = layer;
+				pcb_r_search(layer->arc_tree, &info.box, NULL, rubber_callback_arc, &info, NULL);
+			}
 		}
 	}
-	PCB_END_LOOP;
 }
 
 /* ---------------------------------------------------------------------------
@@ -586,36 +625,45 @@ static void CheckLinePointForRubberbandArcConnection(rubber_ctx_t *rbnd, pcb_lay
  */
 static void CheckArcPointForRubberbandConnection(rubber_ctx_t *rbnd, pcb_layer_t *Layer, pcb_arc_t *Arc, int *end_pt, pcb_bool Exact)
 {
-	pcb_layergrp_id_t group = pcb_layer_get_group_(Layer);
-	struct rubber_info info;
-	pcb_coord_t t = Arc->Thickness / 2, ex, ey;
-	int end; /* = end_pt == pcb_arc_start_ptr ? 0 : 1;*/
+	const pcb_layergrp_id_t group = pcb_layer_get_group_(Layer);
+	pcb_board_t * board = pcb_data_get_top(PCB->Data);
 
-	for(end=0;end<=1;++end)
-	{
-		pcb_arc_get_end(Arc, end, &ex, &ey);
+	if(board == NULL)
+		board = PCB;
 
-		/* lookup layergroup and check all visible lines in this group */
-		info.radius = Exact ? -1 : MAX(Arc->Thickness / 2, 1);
-		info.box.X1 = ex - t;
-		info.box.X2 = ex + t;
-		info.box.Y1 = ey - t;
-		info.box.Y2 = ey + t;
-		info.line = NULL; /* used only to make sure the current object is not added - we are adding lines only and the current object is an arc */
-		info.rbnd = rbnd;
-		info.X = ex;
-		info.Y = ey;
-		info.delta_index = end;
+	if(group >= 0) {
+		pcb_cardinal_t length = board->LayerGroups.grp[group].len;
+		pcb_cardinal_t entry;
+		const pcb_coord_t t = Arc->Thickness / 2;
+		const int comb = Layer->comb & PCB_LYC_SUB;
+		struct rubber_info info;
+		int end; /* = end_pt == pcb_arc_start_ptr ? 0 : 1;*/
+	
+		for(end=0;end<=1;++end)	{
+			pcb_coord_t ex, ey;
+			pcb_arc_get_end(Arc, end, &ex, &ey);
 
-		PCB_COPPER_GROUP_LOOP(PCB->Data, group);
-		{
-			/* check all visible lines of the group member */
-			if (layer->meta.real.vis) {
-				info.layer = layer;
-				pcb_r_search(layer->line_tree, &info.box, NULL, rubber_callback, &info, NULL);
+			info.radius = Exact ? -1 : MAX(Arc->Thickness / 2, 1);
+			info.box.X1 = ex - t;
+			info.box.X2 = ex + t;
+			info.box.Y1 = ey - t;
+			info.box.Y2 = ey + t;
+			info.line = NULL; /* used only to make sure the current object is not added - we are adding lines only and the current object is an arc */
+			info.rbnd = rbnd;
+			info.X = ex;
+			info.Y = ey;
+			info.delta_index = end;
+
+			for(entry = 0;entry < length;++entry)	{
+				const pcb_layer_id_t layer_id = board->LayerGroups.grp[group].lid[entry];
+				pcb_layer_t * layer = &PCB->Data->Layer[layer_id];
+
+				if(layer->meta.real.vis && ((layer->comb & PCB_LYC_SUB) == comb))	{
+					info.layer = layer;
+					pcb_r_search(layer->line_tree, &info.box, NULL, rubber_callback, &info, NULL);
+				}
 			}
 		}
-		PCB_END_LOOP;
 	}
 }
 
@@ -626,36 +674,46 @@ static void CheckArcPointForRubberbandConnection(rubber_ctx_t *rbnd, pcb_layer_t
  */
 static void CheckArcForRubberbandConnection(rubber_ctx_t *rbnd, pcb_layer_t *Layer, pcb_arc_t *Arc, pcb_bool Exact)
 {
-	pcb_layergrp_id_t group;
 	struct rubber_info info;
 	int which;
 	pcb_coord_t t = Arc->Thickness / 2, ex, ey;
+	const pcb_layergrp_id_t group = pcb_layer_get_group_(Layer);
+	pcb_board_t * board = pcb_data_get_top(PCB->Data);
 
-	for(which=0; which<=1; ++which) {
-		pcb_arc_get_end(Arc,which,&ex, &ey);
+	if(board == NULL)
+		board = PCB;
 
-		/* lookup layergroup and check all visible lines in this group */
-		info.radius = Exact ? -1 : MAX(Arc->Thickness / 2, 1);
-		info.box.X1 = ex - t;
-		info.box.X2 = ex + t;
-		info.box.Y1 = ey - t;
-		info.box.Y2 = ey + t;
-		info.line = NULL; /* used only to make sure the current object is not added - we are adding lines only and the current object is an arc */
-		info.rbnd = rbnd;
-		info.X = ex;
-		info.Y = ey;
-		info.delta_index = which;
+	if(group >= 0) {
+		pcb_cardinal_t length = board->LayerGroups.grp[group].len;
+		pcb_cardinal_t entry;
+		const int comb = Layer->comb & PCB_LYC_SUB;
 
-		group = pcb_layer_get_group_(Layer);
-		PCB_COPPER_GROUP_LOOP(PCB->Data, group);
-		{
-			/* check all visible lines of the group member */
-			if (layer->meta.real.vis) {
-				info.layer = layer;
-				pcb_r_search(layer->line_tree, &info.box, NULL, rubber_callback, &info, NULL);
+		for(which=0; which<=1; ++which) {
+			pcb_arc_get_end(Arc,which,&ex, &ey);
+
+			/* lookup layergroup and check all visible lines in this group */
+			info.radius = Exact ? -1 : MAX(Arc->Thickness / 2, 1);
+			info.box.X1 = ex - t;
+			info.box.X2 = ex + t;
+			info.box.Y1 = ey - t;
+			info.box.Y2 = ey + t;
+			info.line = NULL; /* used only to make sure the current object is not added - we are adding lines only and the current object is an arc */
+			info.rbnd = rbnd;
+			info.X = ex;
+			info.Y = ey;
+			info.delta_index = which;
+
+			for(entry = 0;entry < length;++entry) {
+				const pcb_layer_id_t layer_id = board->LayerGroups.grp[group].lid[entry];
+				pcb_layer_t * layer = &PCB->Data->Layer[layer_id];
+
+				if(layer->meta.real.vis && ((layer->comb & PCB_LYC_SUB) == comb)) {
+					/* check all visible lines of the group member */
+					info.layer = layer;
+					pcb_r_search(layer->line_tree, &info.box, NULL, rubber_callback, &info, NULL);
+				}
 			}
 		}
-		PCB_END_LOOP;
 	}
 }
 
@@ -666,53 +724,65 @@ static void CheckArcForRubberbandConnection(rubber_ctx_t *rbnd, pcb_layer_t *Lay
  */
 static void CheckEntireArcForRubberbandConnection(rubber_ctx_t *rbnd, pcb_layer_t * Layer, pcb_arc_t * Arc)
 {
+	pcb_board_t * board = pcb_data_get_top(PCB->Data);
+
+	if(board == NULL)
+		board = PCB;
+
 	/* lookup layergroup and check all visible lines in this group */
 	pcb_layergrp_id_t group = pcb_layer_get_group_(Layer);
 
-	PCB_COPPER_GROUP_LOOP(PCB->Data, group);
-	{
-		if (layer->meta.real.vis) {
-			pcb_coord_t thick;
+	if(group >= 0) {
+		pcb_cardinal_t length = board->LayerGroups.grp[group].len;
+		pcb_cardinal_t entry;
+		const int comb = Layer->comb & PCB_LYC_SUB;
 
-			/* the following code just stupidly compares the endpoints of the lines */
-			PCB_LINE_LOOP(layer);
-			{
-				pcb_rubberband_t *have_line = NULL;
-				pcb_bool touches1 = pcb_false;
-				pcb_bool touches2 = pcb_false;
-				int l;
+		for(entry = 0;entry < length;++entry) {
+			const pcb_layer_id_t layer_id = board->LayerGroups.grp[group].lid[entry];
+			pcb_layer_t * layer = &PCB->Data->Layer[layer_id];
 
-				if (PCB_FLAG_TEST(PCB_FLAG_LOCK, line))
-					continue;
+			if(layer->meta.real.vis && ((layer->comb & PCB_LYC_SUB) == comb)) {
+				pcb_coord_t thick;
 
-				/* Check whether the line is already in the rubberband list. */
-				for(l = 0; (l < rbnd->RubberbandN) && (have_line == NULL); l++)
-					if (rbnd->Rubberband[l].Line == line) 
-						have_line = &rbnd->Rubberband[l];
+				/* the following code just stupidly compares the endpoints of the lines */
+				PCB_LINE_LOOP(layer);
+				{
+					pcb_rubberband_t *have_line = NULL;
+					pcb_bool touches1 = pcb_false;
+					pcb_bool touches2 = pcb_false;
+					int l;
 
-				/* Check whether any of the scanned line points touch the passed line */
-				thick = (line->Thickness + 1) / 2;
-				touches1 = pcb_is_point_on_arc(line->Point1.X, line->Point1.Y, thick, Arc);
-				touches2 = pcb_is_point_on_arc(line->Point2.X, line->Point2.Y, thick, Arc);
+					if (PCB_FLAG_TEST(PCB_FLAG_LOCK, line))
+						continue;
 
-				if(touches1) {
-					if(have_line)
-						have_line->delta_index[0] = 0;
-					else 
-						have_line =	pcb_rubber_band_create(rbnd, layer, line, 0,0);
-				}
+					/* Check whether the line is already in the rubberband list. */
+					for(l = 0; (l < rbnd->RubberbandN) && (have_line == NULL); l++)
+						if (rbnd->Rubberband[l].Line == line) 
+							have_line = &rbnd->Rubberband[l];
 
-				if(touches2) {
-					if(have_line)
-						have_line->delta_index[1] = 0;
-					else 
-						have_line =	pcb_rubber_band_create(rbnd, layer, line, 1,0);
+					/* Check whether any of the scanned line points touch the passed line */
+					thick = (line->Thickness + 1) / 2;
+					touches1 = pcb_is_point_on_arc(line->Point1.X, line->Point1.Y, thick, Arc);
+					touches2 = pcb_is_point_on_arc(line->Point2.X, line->Point2.Y, thick, Arc);
+
+					if(touches1) {
+						if(have_line)
+							have_line->delta_index[0] = 0;
+						else 
+							have_line =	pcb_rubber_band_create(rbnd, layer, line, 0,0);
+					}
+
+					if(touches2) {
+						if(have_line)
+							have_line->delta_index[1] = 0;
+						else 
+							have_line =	pcb_rubber_band_create(rbnd, layer, line, 1,0);
+					}
 				}
+				PCB_END_LOOP;
 			}
-			PCB_END_LOOP;
 		}
 	}
-	PCB_END_LOOP;
 }
 
 /* ---------------------------------------------------------------------------
@@ -722,58 +792,68 @@ static void CheckEntireArcForRubberbandConnection(rubber_ctx_t *rbnd, pcb_layer_
  */
 static void CheckPolygonForRubberbandConnection(rubber_ctx_t *rbnd, pcb_layer_t *Layer, pcb_poly_t *Polygon)
 {
-	pcb_layergrp_id_t group;
 	const pcb_bool clearpoly = PCB_FLAG_TEST(PCB_FLAG_CLEARPOLY,Polygon);
+	pcb_layergrp_id_t group = pcb_layer_get_group_(Layer);
+	pcb_board_t * board = pcb_data_get_top(PCB->Data);
 
-	/* lookup layergroup and check all visible lines in this group */
-	group = pcb_layer_get_group_(Layer);
-	PCB_COPPER_GROUP_LOOP(PCB->Data, group);
+	if(board == NULL)
+		board = PCB;
+
+	if(group >= 0)
 	{
-		if (layer->meta.real.vis) {
-			pcb_coord_t thick;
+		pcb_cardinal_t length = board->LayerGroups.grp[group].len;
+		pcb_cardinal_t entry;
+		const int comb = Layer->comb & PCB_LYC_SUB;
 
-			/* the following code just stupidly compares the endpoints
-			 * of the lines
-			 */
-			PCB_LINE_LOOP(layer);
-			{
-				pcb_rubberband_t *have_line = NULL;
-				int touches1 = 0, touches2 = 0;
-				int l;
+		for(entry = 0;entry < length;++entry) {
+			const pcb_layer_id_t layer_id = board->LayerGroups.grp[group].lid[entry];
+			pcb_layer_t * layer = &PCB->Data->Layer[layer_id];
 
-				if (PCB_FLAG_TEST(PCB_FLAG_LOCK, line))
-					continue;
-				if (clearpoly && PCB_OBJ_HAS_CLEARANCE(line))
-					continue;
+			if(layer->meta.real.vis && ((layer->comb & PCB_LYC_SUB) == comb)) {
+				pcb_coord_t thick;
 
-				/* Check whether the line is already in the rubberband list. */
-				for(l = 0; (l < rbnd->RubberbandN) && (have_line == NULL); l++)
-					if (rbnd->Rubberband[l].Line == line) 
-						have_line = &rbnd->Rubberband[l];
+				/* the following code just stupidly compares the endpoints
+				 * of the lines
+				 */
+				PCB_LINE_LOOP(layer);
+				{
+					pcb_rubberband_t *have_line = NULL;
+					int touches1 = 0, touches2 = 0;
+					int l;
 
-				/* Check whether any of the line points touch the polygon */
-				thick = (line->Thickness + 1) / 2;
-				touches1 = (pcb_poly_is_point_in_p(line->Point1.X, line->Point1.Y, thick, Polygon));
-				touches2 = (pcb_poly_is_point_in_p(line->Point2.X, line->Point2.Y, thick, Polygon));
+					if (PCB_FLAG_TEST(PCB_FLAG_LOCK, line))
+						continue;
+					if ((pcb_layer_flags_(layer) & PCB_LYT_COPPER) && clearpoly && PCB_OBJ_HAS_CLEARANCE(line))
+						continue;
 
-				if(touches1) {
-					if(have_line)
-						have_line->delta_index[0] = 0;
-					else 
-						have_line =	pcb_rubber_band_create(rbnd, layer, line, 0,0);
-				}
+					/* Check whether the line is already in the rubberband list. */
+					for(l = 0; (l < rbnd->RubberbandN) && (have_line == NULL); l++)
+						if (rbnd->Rubberband[l].Line == line) 
+							have_line = &rbnd->Rubberband[l];
 
-				if(touches2) {
-					if(have_line)
-						have_line->delta_index[1] = 0;
-					else 
-						have_line =	pcb_rubber_band_create(rbnd, layer, line, 1,0);
+					/* Check whether any of the line points touch the polygon */
+					thick = (line->Thickness + 1) / 2;
+					touches1 = (pcb_poly_is_point_in_p(line->Point1.X, line->Point1.Y, thick, Polygon));
+					touches2 = (pcb_poly_is_point_in_p(line->Point2.X, line->Point2.Y, thick, Polygon));
+
+					if(touches1) {
+						if(have_line)
+							have_line->delta_index[0] = 0;
+						else 
+							have_line =	pcb_rubber_band_create(rbnd, layer, line, 0,0);
+					}
+
+					if(touches2) {
+						if(have_line)
+							have_line->delta_index[1] = 0;
+						else 
+							have_line =	pcb_rubber_band_create(rbnd, layer, line, 1,0);
+					}
 				}
+				PCB_END_LOOP;
 			}
-			PCB_END_LOOP;
 		}
 	}
-	PCB_END_LOOP;
 }
 
 /* ---------------------------------------------------------------------------
@@ -783,53 +863,128 @@ static void CheckPolygonForRubberbandConnection(rubber_ctx_t *rbnd, pcb_layer_t
  */
 static void CheckLineForRubberbandConnection(rubber_ctx_t *rbnd, pcb_layer_t * Layer, pcb_line_t * Line)
 {
-	/* lookup layergroup and check all visible lines in this group */
 	pcb_layergrp_id_t group = pcb_layer_get_group_(Layer);
+	pcb_board_t * board = pcb_data_get_top(PCB->Data);
 
-	PCB_COPPER_GROUP_LOOP(PCB->Data, group);
-	{
-		if (layer->meta.real.vis) {
-			pcb_coord_t thick;
+	if(board == NULL)
+		board = PCB;
 
-			/* the following code just stupidly compares the endpoints of the lines */
-			PCB_LINE_LOOP(layer);
-			{
-				pcb_rubberband_t *have_line = NULL;
-				pcb_bool touches1 = pcb_false;
-				pcb_bool touches2 = pcb_false;
-				int l;
+	if(group >= 0) {
+		pcb_cardinal_t length = board->LayerGroups.grp[group].len;
+		pcb_cardinal_t entry;
+		const int comb = Layer->comb & PCB_LYC_SUB;
 
-				if (PCB_FLAG_TEST(PCB_FLAG_LOCK, line))
-					continue;
+		for(entry = 0;entry < length;++entry) {
+			const pcb_layer_id_t layer_id = board->LayerGroups.grp[group].lid[entry];
+			pcb_layer_t * layer = &PCB->Data->Layer[layer_id];
 
-				/* Check whether the line is already in the rubberband list. */
-				for(l = 0; (l < rbnd->RubberbandN) && (have_line == NULL); l++)
-					if (rbnd->Rubberband[l].Line == line) 
-						have_line = &rbnd->Rubberband[l];
+			if (layer->meta.real.vis && ((layer->comb & PCB_LYC_SUB) == comb)) {
+				pcb_coord_t thick;
 
-				/* Check whether any of the scanned line points touch the passed line */
-				thick = (line->Thickness + 1) / 2;
-				touches1 = pcb_is_point_on_line(line->Point1.X, line->Point1.Y, thick, Line);
-				touches2 = pcb_is_point_on_line(line->Point2.X, line->Point2.Y, thick, Line);
+				/* the following code just stupidly compares the endpoints of the lines */
+				PCB_LINE_LOOP(layer);
+				{
+					pcb_rubberband_t *have_line = NULL;
+					pcb_bool touches1 = pcb_false;
+					pcb_bool touches2 = pcb_false;
+					int l;
 
-				if(touches1) {
-					if(have_line)
-						have_line->delta_index[0] = 0;
-					else 
-						have_line =	pcb_rubber_band_create(rbnd, layer, line, 0,0);
+					if (PCB_FLAG_TEST(PCB_FLAG_LOCK, line))
+						continue;
+
+					/* Check whether the line is already in the rubberband list. */
+					for(l = 0; (l < rbnd->RubberbandN) && (have_line == NULL); l++)
+						if (rbnd->Rubberband[l].Line == line) 
+							have_line = &rbnd->Rubberband[l];
+
+					/* Check whether any of the scanned line points touch the passed line */
+					thick = (line->Thickness + 1) / 2;
+					touches1 = pcb_is_point_on_line(line->Point1.X, line->Point1.Y, thick, Line);
+					touches2 = pcb_is_point_on_line(line->Point2.X, line->Point2.Y, thick, Line);
+
+					if(touches1) {
+						if(have_line)
+							have_line->delta_index[0] = 0;
+						else 
+							have_line =	pcb_rubber_band_create(rbnd, layer, line, 0,0);
+					}
+
+					if(touches2) {
+						if(have_line)
+							have_line->delta_index[1] = 0;
+						else 
+							have_line =	pcb_rubber_band_create(rbnd, layer, line, 1,0);
+					}
 				}
+				PCB_END_LOOP;
+			}
+		}
+	}
+}
 
-				if(touches2) {
-					if(have_line)
-						have_line->delta_index[1] = 0;
-					else 
-						have_line =	pcb_rubber_band_create(rbnd, layer, line, 1,0);
+/* ---------------------------------------------------------------------------
+ * checks all visible lines.  If either of the endpoints of the line lays 
+ * anywhere inside the passed padstack, the scanned line is added to the 
+ * 'rubberband' list.
+ */
+static void CheckPadStackForRubberbandConnection(rubber_ctx_t *rbnd,pcb_pstk_t * pstk)
+{
+	pcb_layergrp_id_t top;	
+	pcb_layergrp_id_t bottom;
+	pcb_bb_type_t bb_type;
+
+	bb_type = pcb_pstk_bbspan(PCB,pstk,&top,&bottom,NULL);
+
+	if(bb_type != PCB_BB_INVALID)
+	{
+		for(;top<=bottom;++top)
+		{
+			PCB_COPPER_GROUP_LOOP(PCB->Data, top);
+			{
+				if (layer->meta.real.vis) {
+					pcb_coord_t thick;
+
+					/* the following code just stupidly compares the endpoints of the lines */
+					PCB_LINE_LOOP(layer);
+					{
+						pcb_rubberband_t *have_line = NULL;
+						pcb_bool touches1 = pcb_false;
+						pcb_bool touches2 = pcb_false;
+						int l;
+
+						if (PCB_FLAG_TEST(PCB_FLAG_LOCK, line))
+							continue;
+
+						/* Check whether the line is already in the rubberband list. */
+						for(l = 0; (l < rbnd->RubberbandN) && (have_line == NULL); l++)
+							if (rbnd->Rubberband[l].Line == line) 
+								have_line = &rbnd->Rubberband[l];
+
+						/* Check whether any of the scanned line points touch the passed padstack */
+						thick = (line->Thickness + 1) / 2;
+						touches1 = pcb_is_point_in_pstk(line->Point1.X, line->Point1.Y, thick, pstk, layer);
+						touches2 = pcb_is_point_in_pstk(line->Point2.X, line->Point2.Y, thick, pstk, layer);
+
+						if(touches1) {
+							if(have_line)
+								have_line->delta_index[0] = 0;
+							else 
+								have_line =	pcb_rubber_band_create(rbnd, layer, line, 0,0);
+						}
+
+						if(touches2) {
+							if(have_line)
+								have_line->delta_index[1] = 0;
+							else 
+								have_line =	pcb_rubber_band_create(rbnd, layer, line, 1,0);
+						}
+					}
+					PCB_END_LOOP;
 				}
 			}
 			PCB_END_LOOP;
 		}
 	}
-	PCB_END_LOOP;
 }
 
 /* ---------------------------------------------------------------------------
@@ -846,34 +1001,31 @@ static void pcb_rubber_band_lookup_lines(rubber_ctx_t *rbnd, int Type, void *Ptr
 	 * is connected
 	 */
 	switch (Type) {
+	case PCB_TYPE_PSTK:
+		CheckPadStackForRubberbandConnection(rbnd,(pcb_pstk_t *)Ptr1);
+		break;
+
 	case PCB_TYPE_SUBC:
 		{
-			vtp0_t objs;
 			pcb_subc_t * subc = (pcb_subc_t *)Ptr1;
-			pcb_any_obj_t ** o;
-			int n;
+			pcb_data_it_t	it;
+			pcb_any_obj_t * p_obj;
+		 
+			p_obj	= pcb_data_first(&it,subc->data,PCB_OBJ_CLASS_REAL);
 
-			vtp0_init(&objs);
-			pcb_data_list_by_flag(subc->data, &objs, PCB_OBJ_CLASS_REAL, PCB_FLAGS);
-
-			for(n = 0, o = (pcb_any_obj_t **)objs.array; n < vtp0_len(&objs); n++,o++) {
-				pcb_layer_t *ly = (*o)->parent.layer;
-
-				assert((*o)->parent_type == PCB_PARENT_LAYER);
-
-				if(((*o)->term == NULL) || !(pcb_layer_flags_(ly) & PCB_LYT_COPPER))
-					continue;
-
-				switch((*o)->type)
+			while(p_obj)
+			{
+				pcb_layer_t * layer = p_obj->parent.layer;
+				switch(p_obj->type)
 				{
-					case PCB_OBJ_LINE :			CheckLineForRubberbandConnection(rbnd, ly, (pcb_line_t *) (*o)); break;
-					case PCB_OBJ_POLY :			CheckPolygonForRubberbandConnection(rbnd, ly, (pcb_poly_t *) (*o)); break;
-					case PCB_OBJ_ARC :			CheckEntireArcForRubberbandConnection(rbnd, ly, (pcb_arc_t *) (*o)); break;
-					default :								/* TODO: Check other object types */ break;
+					case PCB_OBJ_LINE :	if(p_obj->term) CheckLineForRubberbandConnection(rbnd, layer, (pcb_line_t *) p_obj); break;
+					case PCB_OBJ_POLY :	if(p_obj->term) CheckPolygonForRubberbandConnection(rbnd, layer, (pcb_poly_t *) p_obj); break;
+					case PCB_OBJ_ARC :	/*if(p_obj->term)*/ CheckEntireArcForRubberbandConnection(rbnd, layer, (pcb_arc_t *) p_obj); break;
+					case PCB_OBJ_PSTK :	CheckPadStackForRubberbandConnection(rbnd, (pcb_pstk_t *) p_obj); break;
+					default : 					break;
 				}
+				p_obj = pcb_data_next(&it);
 			}
-			
-			vtp0_uninit(&objs);
 			break;
 		}
 
@@ -902,30 +1054,23 @@ static void pcb_rubber_band_lookup_lines(rubber_ctx_t *rbnd, int Type, void *Ptr
 		{
 			pcb_layer_t *layer = (pcb_layer_t *) Ptr1;
 			pcb_line_t *line = (pcb_line_t *) Ptr2;
-			if (pcb_layer_flags_(layer) & PCB_LYT_COPPER) {
-				CheckLinePointForRubberbandConnection(rbnd, layer, line, &line->Point1,0 );
-				CheckLinePointForRubberbandConnection(rbnd, layer, line, &line->Point2,1 );
-			}
+			CheckLinePointForRubberbandConnection(rbnd, layer, line, &line->Point1,0 );
+			CheckLinePointForRubberbandConnection(rbnd, layer, line, &line->Point2,1 );
 			break;
 		}
 
 	case PCB_TYPE_LINE_POINT:
-		if (pcb_layer_flags_((pcb_layer_t *)Ptr1) & PCB_LYT_COPPER)	{
-			CheckLinePointForRubberbandConnection(rbnd, (pcb_layer_t *) Ptr1, (pcb_line_t *) Ptr2, (pcb_point_t *) Ptr3,0);
-
-			if(conf_rbo.plugins.rubberband_orig.enable_rubberband_arcs != 0)
-				CheckLinePointForRubberbandArcConnection(rbnd, (pcb_layer_t *) Ptr1, (pcb_line_t *) Ptr2, (pcb_point_t *) Ptr3, pcb_true);
-		}
+		CheckLinePointForRubberbandConnection(rbnd, (pcb_layer_t *) Ptr1, (pcb_line_t *) Ptr2, (pcb_point_t *) Ptr3,0);
+		if(conf_rbo.plugins.rubberband_orig.enable_rubberband_arcs != 0)
+			CheckLinePointForRubberbandArcConnection(rbnd, (pcb_layer_t *) Ptr1, (pcb_line_t *) Ptr2, (pcb_point_t *) Ptr3, pcb_true);
 		break;
 
 	case PCB_TYPE_ARC_POINT:
-		if (pcb_layer_flags_((pcb_layer_t *)Ptr1) & PCB_LYT_COPPER)
-			CheckArcPointForRubberbandConnection(rbnd, (pcb_layer_t *) Ptr1, (pcb_arc_t *) Ptr2, (int *) Ptr3, pcb_true);
+		CheckArcPointForRubberbandConnection(rbnd, (pcb_layer_t *) Ptr1, (pcb_arc_t *) Ptr2, (int *) Ptr3, pcb_true);
 		break;
 
 	case PCB_TYPE_ARC:
-		if (pcb_layer_flags_((pcb_layer_t *)Ptr1) & PCB_LYT_COPPER)
-			CheckArcForRubberbandConnection(rbnd, (pcb_layer_t *) Ptr1, (pcb_arc_t *) Ptr2, pcb_true);
+		CheckArcForRubberbandConnection(rbnd, (pcb_layer_t *) Ptr1, (pcb_arc_t *) Ptr2, pcb_true);
 		break;
 
 	case PCB_TYPE_VIA:
@@ -933,8 +1078,7 @@ static void pcb_rubber_band_lookup_lines(rubber_ctx_t *rbnd, int Type, void *Ptr
 		break;
 
 	case PCB_TYPE_POLY:
-		if (pcb_layer_flags_((pcb_layer_t *)Ptr1) & PCB_LYT_COPPER)
-			CheckPolygonForRubberbandConnection(rbnd, (pcb_layer_t *) Ptr1, (pcb_poly_t *) Ptr2);
+		CheckPolygonForRubberbandConnection(rbnd, (pcb_layer_t *) Ptr1, (pcb_poly_t *) Ptr2);
 		break;
 	}
 }
@@ -959,6 +1103,18 @@ static void pcb_rubber_band_lookup_rat_lines(rubber_ctx_t *rbnd, int Type, void
 			break;
 		}
 
+	case PCB_TYPE_SUBC:
+		{
+			pcb_subc_t *subc = (pcb_subc_t *) Ptr1;
+
+			PCB_PADSTACK_LOOP(subc->data);
+			{
+				CheckPadstackForRat(rbnd, padstack);
+			}
+			PCB_END_LOOP;
+			break;
+		}
+
 	case PCB_TYPE_LINE:
 		{
 			pcb_layer_t *layer = (pcb_layer_t *) Ptr1;
@@ -1024,7 +1180,7 @@ static pcb_rubberband_t *pcb_rubber_band_create(rubber_ctx_t *rbnd, pcb_layer_t
 	ptr->delta_index[point_number] = delta_index;
 	ptr->delta_index[point_number^1] = -1;
 
-	return (ptr);
+	return ptr;
 }
 
 static pcb_rubberband_arc_t *pcb_rubber_band_create_arc(rubber_ctx_t *rbnd, pcb_layer_t *Layer, pcb_arc_t *Arc, int end)
@@ -1045,7 +1201,7 @@ static pcb_rubberband_arc_t *pcb_rubber_band_create_arc(rubber_ctx_t *rbnd, pcb_
 	
 	ptr->Ends |= end;
 	
-	return (ptr);
+	return ptr;
 }
 
 /*** event handlers ***/
@@ -1293,12 +1449,12 @@ static void rbe_rotate90(void *user_data, int argc, pcb_event_arg_t argv[])
 	
 		pcb_line_bbox(ptr->Line);
 		if (ptr->Layer) {
-			pcb_r_insert_entry(ptr->Layer->line_tree, (pcb_box_t *) ptr->Line, 0);
+			pcb_r_insert_entry(ptr->Layer->line_tree, (pcb_box_t *) ptr->Line);
 			pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_LINE, ptr->Layer, ptr->Line);
 			pcb_line_invalidate_draw(ptr->Layer, ptr->Line);
 		}
 		else {
-			pcb_r_insert_entry(PCB->Data->rat_tree, (pcb_box_t *) ptr->Line, 0);
+			pcb_r_insert_entry(PCB->Data->rat_tree, (pcb_box_t *) ptr->Line);
 			pcb_rat_invalidate_draw((pcb_rat_t *) ptr->Line);
 		}
 
