@@ -1,14 +1,13 @@
-/*!
- * \file teardrops.c
+/*
+ * Teardrops plug-in for PCB.
  *
- * \brief Teardrops plug-in for PCB.
+ * Copyright (C) 2006 DJ Delorie <dj@delorie.com>
  *
- * \author Copyright (C) 2006 DJ Delorie <dj@delorie.com>
- *
- * \copyright Licensed under the terms of the GNU General Public
+ * Licensed under the terms of the GNU General Public
  * License, version 2 or later.
  *
  * Ported to pcb-rnd by Tibor 'Igor2' Palinkas in 2016.
+ * Updated to new data model by Tibor 'Igor2' Palinkas in 2018.
  *
  * Original source: http://www.delorie.com/pcb/teardrops/
 */
@@ -26,9 +25,11 @@
 #include "plugins.h"
 #include "hid_actions.h"
 #include "obj_all.h"
+#include "obj_pstk_inlines.h"
 
 #define MIN_LINE_LENGTH 700
 #define MAX_DISTANCE 700
+
 /* changed MAX_DISTANCE to 0.5 mm below */
  /* 0.5mm */
 /* #define MAX_DISTANCE 500000 */
@@ -36,26 +37,19 @@
  /* 1 mm */
 /* #define MAX_DISTANCE 1000000 */
 
+#define MIN_LINE_LENGTH2 ((double)MIN_LINE_LENGTH*(double)MIN_LINE_LENGTH)
+#define MAX_DISTANCE2 ((double)MAX_DISTANCE*(double)MAX_DISTANCE)
+
 static pcb_pin_t *pin;
 static pcb_pad_t *pad;
+static pcb_pstk_t *pstk;
 static int layer;
-static int px, py;
+static pcb_coord_t px, py;
 static pcb_coord_t thickness;
 static pcb_element_t *element;
 
 static int new_arcs = 0;
 
-int distance_between_points(int x1, int y1, int x2, int y2)
-{
-	/* int a; */
-	/* int b; */
-	int distance;
-	/* a = (x1-x2); */
-	/* b = (y1-y2); */
-	distance = sqrt((pow(x1 - x2, 2)) + (pow(y1 - y2, 2)));
-	return distance;
-}
-
 static pcb_r_dir_t check_line_callback(const pcb_box_t * box, void *cl)
 {
 	pcb_layer_t *lay = &PCB->Data->Layer[layer];
@@ -73,20 +67,20 @@ static pcb_r_dir_t check_line_callback(const pcb_box_t * box, void *cl)
 					PCB_COORD_TO_MM(l->Point1.X), PCB_COORD_TO_MM(l->Point1.Y), PCB_COORD_TO_MM(l->Point2.X), PCB_COORD_TO_MM(l->Point2.Y));
 
 	/* if our line is to short ignore it */
-	if (distance_between_points(l->Point1.X, l->Point1.Y, l->Point2.X, l->Point2.Y) < MIN_LINE_LENGTH) {
+	if (pcb_distance2(l->Point1.X, l->Point1.Y, l->Point2.X, l->Point2.Y) < MIN_LINE_LENGTH2) {
 		fprintf(stderr, "not within max line length\n");
 		return 1;
 	}
 
 	fprintf(stderr, "......Point (%.6f, %.6f): ", PCB_COORD_TO_MM(px), PCB_COORD_TO_MM(py));
 
-	if (distance_between_points(l->Point1.X, l->Point1.Y, px, py) < MAX_DISTANCE) {
+	if (pcb_distance2(l->Point1.X, l->Point1.Y, px, py) < MAX_DISTANCE2) {
 		x1 = l->Point1.X;
 		y1 = l->Point1.Y;
 		x2 = l->Point2.X;
 		y2 = l->Point2.Y;
 	}
-	else if (distance_between_points(l->Point2.X, l->Point2.Y, px, py) < MAX_DISTANCE) {
+	else if (pcb_distance(l->Point2.X, l->Point2.Y, px, py) < MAX_DISTANCE2) {
 		x1 = l->Point2.X;
 		y1 = l->Point2.Y;
 		x2 = l->Point1.X;
@@ -145,8 +139,7 @@ static pcb_r_dir_t check_line_callback(const pcb_box_t * box, void *cl)
 	lx = px + dx * ldist;
 	ly = py + dy * ldist;
 
-	/* We need one up front to determine how many segments it will take
-	   to fill.  */
+	/* We need one up front to determine how many segments it will take to fill.  */
 	ax = lx - dy * adist;
 	ay = ly + dx * adist;
 	vl = sqrt(r * r - t * t);
@@ -192,6 +185,74 @@ static pcb_r_dir_t check_line_callback(const pcb_box_t * box, void *cl)
 	return 1;
 }
 
+static void check_pstk(pcb_pstk_t *ps)
+{
+	pstk = ps;
+
+	for (layer = 0; layer < pcb_max_layer; layer++) {
+		pcb_layer_t *l = &(PCB->Data->Layer[layer]);
+		pcb_pstk_shape_t *shp;
+		pcb_box_t spot;
+		int n;
+		double mindist;
+
+		if (!(pcb_layer_flags(PCB, layer) & PCB_LYT_COPPER))
+			continue;
+
+		shp = pcb_pstk_shape_at(PCB, ps, l);
+		if (shp == NULL)
+			continue;
+
+		switch(shp->shape) {
+			case PCB_PSSH_POLY:
+				/* Simplistic approach on polygons; works only on the simplest cases
+				   How this could be handled better: list the 1 or 2 polygon edges
+				   that cross the incoming line's sides and do the teardrops there;
+				   but there are corner cases lurking: what if the next edges out
+				   from the 1..2 edges are curving back? */
+				px = py = 0;
+				for(n = 0; n < shp->data.poly.len; n++) {
+					px += shp->data.poly.x[n];
+					py += shp->data.poly.y[n];
+				}
+				px /= shp->data.poly.len;
+				py /= shp->data.poly.len;
+
+				mindist = PCB_MM_TO_COORD(8);
+				mindist *= mindist;
+				for(n = 0; n < shp->data.poly.len; n++) {
+					double dist = pcb_distance2(px, py, shp->data.poly.x[n], shp->data.poly.y[n]);
+					if (dist < mindist)
+						mindist = dist;
+				}
+				thickness = sqrt(mindist)*1.4;
+				px += ps->x;
+				py += ps->y;
+				break;
+
+			case PCB_PSSH_LINE:
+				thickness = shp->data.line.thickness;
+				px = ps->x + (shp->data.line.x1 + shp->data.line.x2)/2;
+				py = ps->y + (shp->data.line.y1 + shp->data.line.y2)/2;
+				break;
+
+			case PCB_PSSH_CIRC:
+				thickness = shp->data.circ.dia;
+				px = ps->x + shp->data.circ.x;
+				py = ps->y + shp->data.circ.y;
+				break;
+		}
+
+		spot.X1 = px - 10;
+		spot.Y1 = py - 10;
+		spot.X2 = px + 10;
+		spot.Y2 = py + 10;
+
+		pcb_r_search(l->line_tree, &spot, NULL, check_line_callback, l, NULL);
+	}
+}
+
+
 static void check_pin(pcb_pin_t * _pin)
 {
 	pcb_box_t spot;
@@ -265,17 +326,6 @@ static void check_pad(pcb_pad_t * _pad)
 					PCB_COORD_TO_MM((pad->BoundingBox.Y1 + pad->BoundingBox.Y2) / 2),
 					PCB_EMPTY(PCB_ELEM_NAME_REFDES(element)), PCB_EMPTY(PCB_ELEM_NAME_VALUE(element)), PCB_EMPTY(PCB_ELEM_NAME_DESCRIPTION(element)));
 
-	/* fprintf(stderr, */
-	/*   "Pad %s (%s) at ((%.6f, %.6f), (%.6f, %.6f)) (element %s, %s, %s) \n", */
-	/*           PCB_EMPTY(pad->Number), PCB_EMPTY(pad->Name), */
-	/*           PCB_COORD_TO_MM(pad->BoundingBox.X1), */
-	/*           PCB_COORD_TO_MM(pad->BoundingBox.Y1), */
-	/*           PCB_COORD_TO_MM(pad->BoundingBox.X2), */
-	/*           PCB_COORD_TO_MM(pad->BoundingBox.Y2), */
-	/*           PCB_EMPTY(PCB_ELEM_NAME_REFDES(element)), */
-	/*           PCB_EMPTY(PCB_ELEM_NAME_VALUE(element)), */
-	/*           PCB_EMPTY(PCB_ELEM_NAME_DESCRIPTION(element))); */
-
 	for (layer = 0; layer < pcb_max_layer; layer++) {
 		pcb_layer_t *l = &(PCB->Data->Layer[layer]);
 		if (!(pcb_layer_flags(PCB, layer) & PCB_LYT_COPPER))
@@ -287,6 +337,13 @@ static void check_pad(pcb_pad_t * _pad)
 static int teardrops(int argc, const char **argv, pcb_coord_t x, pcb_coord_t y)
 {
 	new_arcs = 0;
+	pcb_box_t *b;
+	pcb_rtree_it_t it;
+
+	for(b = pcb_r_first(PCB->Data->padstack_tree, &it); b != NULL; b = pcb_r_next(&it))
+		check_pstk((pcb_pstk_t *)b);
+	pcb_r_end(&it);
+
 
 	PCB_VIA_LOOP(PCB->Data);
 	{
