@@ -1,7 +1,8 @@
 /*
  *                            COPYRIGHT
  *
- *  PCB, interactive printed circuit board design
+ *  pcb-rnd, interactive printed circuit board design
+ *  (this file is based on PCB, interactive printed circuit board design)
  *  Copyright (C) 2004, 2007 Dan McMahill
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -18,6 +19,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  */
 
 #include "config.h"
@@ -37,7 +42,6 @@
 #include "draw.h"
 #include "error.h"
 #include "undo.h"
-#include "vendor.h"
 #include "stub_vendor.h"
 #include "plugins.h"
 #include "action_helper.h"
@@ -48,6 +52,7 @@
 #include "compat_misc.h"
 #include "compat_nls.h"
 #include "obj_pinvia.h"
+#include "obj_pstk_inlines.h"
 #include "event.h"
 #include <liblihata/lihata.h>
 #include <liblihata/tree.h>
@@ -59,6 +64,8 @@ static void apply_vendor_map(void);
 static void process_skips(lht_node_t *);
 static pcb_bool rematch(const char *, const char *);
 static void vendor_free_all(void);
+int vendorDrillMap(int in);
+
 
 /* list of vendor drills and a count of them */
 static int *vendor_drills = NULL;
@@ -89,6 +96,9 @@ static int rounding_method = ROUND_UP;
 
 #define FREE(x) if((x) != NULL) { free (x) ; (x) = NULL; }
 
+static pcb_bool vendorIsSubcMappable(pcb_subc_t *subc);
+static pcb_bool vendorIsElementMappable(pcb_element_t *element);
+
 /* ************************************************************ */
 
 static const char apply_vendor_syntax[] = "ApplyVendor()";
@@ -299,9 +309,36 @@ int pcb_act_LoadVendorFrom(int argc, const char **argv, pcb_coord_t x, pcb_coord
 	return 0;
 }
 
+static pcb_cardinal_t apply_vendor_pstk(pcb_data_t *data)
+{
+	gdl_iterator_t it;
+	pcb_pstk_t *pstk;
+	pcb_cardinal_t changed = 0;
+
+	padstacklist_foreach(&data->subc, &it, pstk) {
+		pcb_pstk_proto_t *proto = pcb_pstk_get_proto(pstk);
+		pcb_coord_t target;
+
+		if ((proto == NULL) || (proto->hdia == 0) || PCB_FLAG_TEST(PCB_FLAG_LOCK, pstk)) continue;
+
+		target = vendorDrillMap(proto->hdia);
+		if (proto->hdia != target) {
+			if (pcb_chg_obj_2nd_size(PCB_TYPE_PSTK, pstk, NULL, NULL, target, pcb_true, pcb_false))
+				changed++;
+			else {
+				pcb_message(PCB_MSG_WARNING, _
+					("Via at %ml, %ml not changed.  Possible reasons:\n"
+					 "\t- pad size too small\n"
+					 "\t- new size would be too large or too small\n"), pstk->x, pstk->y);
+			}
+		}
+	}
+	return changed;
+}
+
 static void apply_vendor_map(void)
 {
-	int changed, tot;
+	pcb_cardinal_t changed = 0, tot = 0;
 	pcb_bool state;
 
 	state = conf_vendor.plugins.vendor.enable;
@@ -309,13 +346,11 @@ static void apply_vendor_map(void)
 	/* enable mapping */
 	conf_force_set_bool(conf_vendor.plugins.vendor.enable, 1);
 
-	/* reset our counts */
-	changed = 0;
-	tot = 0;
-
 	/* If we have loaded vendor drills, then apply them to the design */
 	if (n_vendor_drills > 0) {
 
+		changed += apply_vendor_pstk(PCB->Data);
+		
 		/* first all the vias */
 		PCB_VIA_LOOP(PCB->Data);
 		{
@@ -339,6 +374,13 @@ static void apply_vendor_map(void)
 		}
 		PCB_END_LOOP;
 
+		PCB_SUBC_LOOP(PCB->Data);
+		{
+			if (vendorIsSubcMappable(subc))
+				changed += apply_vendor_pstk(subc->data);
+		}
+		PCB_END_LOOP;
+
 #warning subc TODO
 		/* and now the pins */
 		PCB_ELEMENT_LOOP(PCB->Data);
@@ -377,7 +419,7 @@ static void apply_vendor_map(void)
 		}
 		PCB_END_LOOP;
 
-		pcb_message(PCB_MSG_INFO, _("Updated %d drill sizes out of %d total\n"), changed, tot);
+		pcb_message(PCB_MSG_INFO, _("Updated %ld drill sizes out of %ld total\n"), (long)changed, (long)tot);
 
 #warning TODO: this should not happen; modify some local setting?
 #if 0
@@ -570,7 +612,7 @@ static void process_skips(lht_node_t *res)
 	}
 }
 
-pcb_bool vendorIsElementMappable(pcb_element_t *element)
+static pcb_bool vendorIsElementMappable(pcb_element_t *element)
 {
 	int i;
 	int noskip;
@@ -618,6 +660,58 @@ pcb_bool vendorIsElementMappable(pcb_element_t *element)
 		return pcb_false;
 }
 
+static pcb_bool vendorIsSubcMappable(pcb_subc_t *subc)
+{
+	int i;
+	int noskip;
+
+	if (!conf_vendor.plugins.vendor.enable)
+		return pcb_false;
+
+#warning TODO: these 3 loops should be wrapped in a single loop that iterates over attribute keys
+	noskip = 1;
+	for (i = 0; i < n_refdes; i++) {
+		if ((PCB_NSTRCMP(PCB_UNKNOWN(subc->refdes), ignore_refdes[i]) == 0)
+				|| rematch(ignore_refdes[i], PCB_UNKNOWN(subc->refdes))) {
+			pcb_message(PCB_MSG_INFO, _("Vendor mapping skipped because refdes = %s matches %s\n"), PCB_UNKNOWN(subc->refdes), ignore_refdes[i]);
+			noskip = 0;
+		}
+	}
+	if (noskip) {
+		const char *vl = pcb_attribute_get(&subc->Attributes, "value");
+		for (i = 0; i < n_value; i++) {
+			if ((PCB_NSTRCMP(PCB_UNKNOWN(vl), ignore_value[i]) == 0)
+					|| rematch(ignore_value[i], PCB_UNKNOWN(vl))) {
+				pcb_message(PCB_MSG_INFO, _("Vendor mapping skipped because value = %s matches %s\n"), PCB_UNKNOWN(vl), ignore_value[i]);
+				noskip = 0;
+			}
+		}
+	}
+
+	if (noskip) {
+		const char *fp = pcb_attribute_get(&subc->Attributes, "footprint");
+		for (i = 0; i < n_descr; i++) {
+			if ((PCB_NSTRCMP(PCB_UNKNOWN(fp), ignore_descr[i]) == 0)
+					|| rematch(ignore_descr[i], PCB_UNKNOWN(fp))) {
+				pcb_message(PCB_MSG_INFO, _
+								("Vendor mapping skipped because descr = %s matches %s\n"),
+								PCB_UNKNOWN(fp), ignore_descr[i]);
+				noskip = 0;
+			}
+		}
+	}
+
+	if (noskip && PCB_FLAG_TEST(PCB_FLAG_LOCK, subc)) {
+		pcb_message(PCB_MSG_INFO, _("Vendor mapping skipped because element %s is locked\n"), PCB_UNKNOWN(subc->refdes));
+		noskip = 0;
+	}
+
+	if (noskip)
+		return pcb_true;
+	else
+		return pcb_false;
+}
+
 static pcb_bool rematch(const char *re, const char *s)
 {
 	int result;
