@@ -6,16 +6,16 @@
 int main(int argc, char *argv[])
 {
 	const char *fmt = argv[1];
-	Coord crd;
+	pcb_coord_t crd;
 	int n;
 
 	pcb_printf_slot[0] = "%mr";
 
 	for(n = 2; n < argc; n++) {
 		pcb_bool success;
-		double val = GetValueEx(argv[n], NULL, NULL, NULL, "", &success);
+		double val = pcb_get_value_ex(argv[n], NULL, NULL, NULL, "", &success);
 		if (!success) {
-			fprintf(stderr, "Unable to convert '%s' to coord\n", argv[n]);
+			fprintf(stderr, "Unable to convert '%s' to pcb_coord_t\n", argv[n]);
 			return 1;
 		}
 		crd = val;
@@ -26,4 +26,4 @@ int main(int argc, char *argv[])
 	printf("\n");
 
 	return 0;
-}
\ No newline at end of file
+}
