@@ -68,3 +68,12 @@ void pcb_event(pcb_event_id_t ev, const char *fmt, ...)
 
 }
 
+int pcb_file_loaded_set_at(const char *catname, const char *name, const char *path, const char *desc)
+{
+}
+
+int pcb_file_loaded_del_at(const char *catname, const char *name)
+{
+}
+
+
